/* Generated by Cython 3.2.4 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "extra_compile_args": [
            "-O3",
            "-g0"
        ],
        "language": "c",
        "name": "aioesphomeapi.connection",
        "sources": [
            "aioesphomeapi/connection.py"
        ]
    },
    "module_name": "aioesphomeapi.connection"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_4"
#define CYTHON_HEX_VERSION 0x030204F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CInitCode */
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !CYTHON_COMPILING_IN_LIMITED_API && CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) (__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
#else
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) __Pyx_PyList_GetItemRef(o, i)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__aioesphomeapi__connection
#define __PYX_HAVE_API__aioesphomeapi__connection
/* Early includes */
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "aioesphomeapi/connection.py",
  "(tree fragment)",
  "aioesphomeapi/connection.pxd",
  "aioesphomeapi/_frame_helper/base.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto (used by ParseKeywordsImpl) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* IncludeStructmemberH.proto (used by FixUpExtensionType) */
#include <structmember.h>

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper;
struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams;
struct __pyx_obj_13aioesphomeapi_10connection_APIConnection;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response;
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect;

/* "_frame_helper/base.pxd":9
 * cdef bint TYPE_CHECKING
 * 
 * cdef class APIFrameHelper:             # <<<<<<<<<<<<<<
 * 
 *     cdef object _loop
*/
struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper {
  PyObject_HEAD
  struct __pyx_vtabstruct_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *__pyx_vtab;
  PyObject *_loop;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *_connection;
  PyObject *_transport;
  PyObject *_writelines;
  PyObject *ready_future;
  PyObject *_buffer;
  unsigned int _buffer_len;
  unsigned int _pos;
  PyObject *_client_info;
  PyObject *_log_name;
};


/* "aioesphomeapi/connection.pxd":78
 * 
 * 
 * @cython.dataclasses.dataclass             # <<<<<<<<<<<<<<
 * cdef class ConnectionParams:
 * 
*/
struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams {
  PyObject_HEAD
  PyObject *addresses;
  PyObject *port;
  PyObject *password;
  PyObject *client_info;
  PyObject *keepalive;
  PyObject *zeroconf_manager;
  PyObject *noise_psk;
  PyObject *expected_name;
  PyObject *expected_mac;
  PyObject *timezone;
};


/* "aioesphomeapi/connection.pxd":93
 * 
 * 
 * cdef class APIConnection:             # <<<<<<<<<<<<<<
 * 
 *     cdef ConnectionParams _params
*/
struct __pyx_obj_13aioesphomeapi_10connection_APIConnection {
  PyObject_HEAD
  struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *__pyx_vtab;
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *_params;
  PyObject *on_stop;
  PyObject *_socket;
  struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *_frame_helper;
  PyObject *api_version;
  PyObject *connection_state;
  PyObject *_message_handlers;
  PyObject *log_name;
  PyObject *_read_exception_futures;
  PyObject *_ping_timer;
  PyObject *_pong_timer;
  float _keep_alive_interval;
  float _keep_alive_timeout;
  PyObject *_resolve_host_future;
  PyObject *_start_connect_future;
  PyObject *_finish_connect_future;
  PyObject *_fatal_exception;
  int _expected_disconnect;
  PyObject *_loop;
  int _send_pending_ping;
  int is_connected;
  int _handshake_complete;
  int _debug_enabled;
  PyObject *received_name;
  PyObject *connected_address;
  PyObject *_cached_timezone;
  PyObject *_addrs_info;
  int _log_errors;
};


/* "aioesphomeapi/connection.py":68
 * MESSAGE_NUMBER_TO_PROTO: tuple[
 *     tuple[Callable[[], message.Message], Callable[[message.Message, bytes], None]], ...
 * ] = tuple((msg, msg.MergeFromString) for msg in MESSAGE_TYPE_TO_PROTO.values())             # <<<<<<<<<<<<<<
 * 
 * 
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr {
  PyObject_HEAD
  PyObject *__pyx_genexpr_arg_0;
  PyObject *__pyx_v_msg;
  PyObject *__pyx_t_0;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
};


/* "aioesphomeapi/connection.py":386
 *         self._debug_enabled = enable
 * 
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:             # <<<<<<<<<<<<<<
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect {
  PyObject_HEAD
  PyObject *__pyx_8genexpr3__pyx_v_addr;
  PyObject *__pyx_v_addr_infos;
  PyObject *__pyx_v_addrs;
  PyObject *__pyx_v_err;
  PyObject *__pyx_v_genexpr;
  long __pyx_v_interleave;
  PyObject *__pyx_v_last_exception;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_v_sock;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
  PyObject *__pyx_t_3;
  PyObject *__pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *__pyx_t_6;
  PyObject *__pyx_t_7;
  PyObject *__pyx_t_8;
  PyObject *__pyx_t_9;
  PyObject *__pyx_t_10;
};


/* "aioesphomeapi/connection.py":392
 *                 "%s: Connecting to %s",
 *                 self.log_name,
 *                 ", ".join(str(addr.sockaddr) for addr in addrs),             # <<<<<<<<<<<<<<
 *             )
 * 
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr {
  PyObject_HEAD
  PyObject *__pyx_genexpr_arg_0;
  PyObject *__pyx_v_addr;
};


/* "aioesphomeapi/connection.py":481
 *                 return
 * 
 *     async def _connect_init_frame_helper(self) -> None:             # <<<<<<<<<<<<<<
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper {
  PyObject_HEAD
  PyObject *__pyx_v__;
  PyObject *__pyx_v_err;
  PyObject *__pyx_v_fh;
  PyObject *__pyx_v_handshake_handle;
  PyObject *__pyx_v_noise_psk;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
};


/* "aioesphomeapi/connection.py":529
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)
 * 
 *     async def _connect_hello_login(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login {
  PyObject_HEAD
  int __pyx_v_login;
  PyObject *__pyx_v_messages;
  PyObject *__pyx_v_msg_types;
  PyObject *__pyx_v_resp;
  PyObject *__pyx_v_responses;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
};


/* "aioesphomeapi/connection.py":649
 *         )
 * 
 *     async def start_resolve_host(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the host resolution process.
 * 
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host {
  PyObject_HEAD
  PyObject *__pyx_v_ex;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
  PyObject *__pyx_t_3;
  PyObject *__pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *__pyx_t_6;
  PyObject *__pyx_t_7;
  PyObject *__pyx_t_8;
  PyObject *__pyx_t_9;
  PyObject *__pyx_t_10;
};


/* "aioesphomeapi/connection.py":687
 *             self._resolve_host_future = None
 * 
 *     async def start_connection(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the connection process.
 * 
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection {
  PyObject_HEAD
  PyObject *__pyx_v_ex;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
  PyObject *__pyx_t_3;
  PyObject *__pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *__pyx_t_6;
  PyObject *__pyx_t_7;
  PyObject *__pyx_t_8;
  PyObject *__pyx_t_9;
  PyObject *__pyx_t_10;
};


/* "aioesphomeapi/connection.py":768
 *         return new_exc
 * 
 *     async def _do_finish_connect(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process."""
 *         # Cache timezone before registering handlers to ensure it's available
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect {
  PyObject_HEAD
  int __pyx_v_login;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
};


/* "aioesphomeapi/connection.py":783
 *         self._async_schedule_keep_alive(self._loop.time())
 * 
 *     async def finish_connection(self, *, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process.
 * 
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection {
  PyObject_HEAD
  PyObject *__pyx_v_ex;
  int __pyx_v_login;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
  PyObject *__pyx_t_3;
  PyObject *__pyx_t_4;
  PyObject *__pyx_t_5;
  PyObject *__pyx_t_6;
  PyObject *__pyx_t_7;
  PyObject *__pyx_t_8;
  PyObject *__pyx_t_9;
  PyObject *__pyx_t_10;
};


/* "aioesphomeapi/connection.py":917
 *         return self.add_message_callback(on_message, msg_types)
 * 
 *     async def send_messages_await_response_complex(  # pylint: disable=too-many-locals             # <<<<<<<<<<<<<<
 *         self,
 *         messages: tuple[message.Message, ...],
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex {
  PyObject_HEAD
  PyObject *__pyx_v_do_append;
  PyObject *__pyx_v_do_stop;
  PyObject *__pyx_v_err;
  PyObject *__pyx_v_fut;
  PyObject *__pyx_v_messages;
  PyObject *__pyx_v_msg_types;
  PyObject *__pyx_v_on_message;
  PyObject *__pyx_v_response_names;
  PyObject *__pyx_v_responses;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_v_timeout;
  int __pyx_v_timeout_expired;
  PyObject *__pyx_v_timeout_handle;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
};


/* "aioesphomeapi/connection.py":972
 *         return responses
 * 
 *     async def send_message_await_response(             # <<<<<<<<<<<<<<
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response {
  PyObject_HEAD
  PyObject *__pyx_v_response;
  PyObject *__pyx_v_response_type;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_v_send_msg;
  PyObject *__pyx_v_timeout;
};


/* "aioesphomeapi/connection.py":1151
 *         self.send_messages((resp,))
 * 
 *     async def disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Disconnect from the API."""
 *         if self._finish_connect_future is not None:
*/
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect {
  PyObject_HEAD
  PyObject *__pyx_v__;
  PyObject *__pyx_v_pending;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self;
  PyObject *__pyx_t_0;
  PyObject *__pyx_t_1;
  PyObject *__pyx_t_2;
};



/* "_frame_helper/base.pxd":9
 * cdef bint TYPE_CHECKING
 * 
 * cdef class APIFrameHelper:             # <<<<<<<<<<<<<<
 * 
 *     cdef object _loop
*/

struct __pyx_vtabstruct_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper {
  PyObject *(*set_log_name)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_read)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *, int);
  void (*_add_to_buffer)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *, PyObject *);
  void (*_remove_from_buffer)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *);
  void (*write_packets)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *, PyObject *, int, int __pyx_skip_dispatch);
  void (*_write_bytes)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *, PyObject *, int);
  void (*_handle_error_and_close)(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *, PyObject *);
};
static struct __pyx_vtabstruct_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *__pyx_vtabptr_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper;


/* "aioesphomeapi/connection.py":240
 * 
 * 
 * class APIConnection:             # <<<<<<<<<<<<<<
 *     """This class represents _one_ connection to a remote native API device.
 * 
*/

struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection {
  void (*send_message)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  void (*send_messages)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  void (*process_packet)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, unsigned int, PyObject *, int __pyx_skip_dispatch);
  void (*_async_cancel_pong_timer)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
  void (*_async_schedule_keep_alive)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *);
  void (*_cleanup)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
  PyObject *(*set_log_name)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_make_auth_request)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
  void (*_process_hello_resp)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *);
  void (*_set_connection_state)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *);
  void (*report_fatal_error)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  void (*_add_message_callback_without_remove)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, PyObject *);
  PyObject *(*add_message_callback)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  void (*_remove_message_callback)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  void (*_handle_disconnect_request_internal)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  void (*_handle_ping_request_internal)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  void (*_handle_get_time_request_internal)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch);
  void (*_set_fatal_exception_if_unset)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *);
  void (*_register_internal_message_handlers)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
  void (*_increase_recv_buffer_size)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
  void (*_set_start_connect_future)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
  void (*_set_finish_connect_future)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *);
};
static struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *__pyx_vtabptr_13aioesphomeapi_10connection_APIConnection;
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* IterFinish.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto (used by PyObjectCallNoArg) */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* PyObjectCallNoArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectCallOneArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod0.proto (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* RaiseNeedMoreValuesToUnpack.proto (used by UnpackTuple2) */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseTooManyValuesToUnpack.proto (used by UnpackItemEndCheck) */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* UnpackItemEndCheck.proto (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* RaiseNoneIterError.proto (used by UnpackTupleError) */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* UnpackTupleError.proto (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *, Py_ssize_t index);

/* UnpackTuple2.proto (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
    PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple);
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
    PyObject* tuple, PyObject** value1, PyObject** value2, int decref_tuple);
static int __Pyx_unpack_tuple2_generic(
    PyObject* tuple, PyObject** value1, PyObject** value2, int has_known_size, int decref_tuple);

/* dict_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* dict, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_is_dict);
static CYTHON_INLINE int __Pyx_dict_iter_next(PyObject* dict_or_iter, Py_ssize_t orig_length, Py_ssize_t* ppos,
                                              PyObject** pkey, PyObject** pvalue, PyObject** pitem, int is_dict);

/* RaiseUnboundLocalError.proto */
static void __Pyx_RaiseUnboundLocalError(const char *varname);

/* GetException.proto (used by pep479) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* pep479.proto */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen);

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyObjectFormatSimple.proto */
#if CYTHON_COMPILING_IN_PYPY
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#elif CYTHON_USE_TYPE_SLOTS
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        likely(PyLong_CheckExact(s)) ? PyLong_Type.tp_repr(s) :\
        likely(PyFloat_CheckExact(s)) ? PyFloat_Type.tp_repr(s) :\
        PyObject_Format(s, f))
#else
    #define __Pyx_PyObject_FormatSimple(s, f) (\
        likely(PyUnicode_CheckExact(s)) ? (Py_INCREF(s), s) :\
        PyObject_Format(s, f))
#endif

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* PyTimeoutError_Check.proto */
#define __Pyx_PyExc_TimeoutError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TimeoutError)

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* set_iter.proto */
static CYTHON_INLINE PyObject* __Pyx_set_iterator(PyObject* iterable, int is_set,
                                                  Py_ssize_t* p_orig_length, int* p_source_is_set);
static CYTHON_INLINE int __Pyx_set_iter_next(
        PyObject* iter_obj, Py_ssize_t orig_length,
        Py_ssize_t* ppos, PyObject **value,
        int source_is_set);

/* PyObject_Unicode.proto */
#define __Pyx_PyObject_Unicode(obj)\
    (likely(PyUnicode_CheckExact(obj)) ? __Pyx_NewRef(obj) : PyObject_Str(obj))

/* PyObjectDelAttr.proto (used by PyObjectSetAttrStr) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
#define __Pyx_PyObject_DelAttr(o, n) PyObject_SetAttr(o, n, NULL)
#else
#define __Pyx_PyObject_DelAttr(o, n) PyObject_DelAttr(o, n)
#endif

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   __Pyx_PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto (used by FetchCommonType) */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CoroutineBase) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* CallTypeTraverse.proto (used by CoroutineBase) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* IterNextPlain.proto (used by CoroutineBase) */
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator);
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void);
#endif

/* PyObjectCall2Args.proto (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethod1.proto (used by CoroutineBase) */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* ReturnWithStopIteration.proto (used by CoroutineBase) */
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext);

/* CoroutineBase.proto (used by Coroutine) */
struct __pyx_CoroutineObject;
typedef PyObject *(*__pyx_coroutine_body_t)(struct __pyx_CoroutineObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct __pyx_CoroutineObject {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *gi_weakreflist;
#endif
    PyObject *classobj;
    PyObject *yieldfrom;
    __Pyx_pyiter_sendfunc yieldfrom_am_send;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
#if CYTHON_USE_SYS_MONITORING && (CYTHON_PROFILE || CYTHON_TRACE)
    PyMonitoringState __pyx_pymonitoring_state[__Pyx_MonitoringEventTypes_CyGen_count];
    uint64_t __pyx_pymonitoring_version;
#endif
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static __Pyx_PySendResult __Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static __Pyx_PySendResult __Pyx_Coroutine_Close(PyObject *self, PyObject **retval);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen);
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen);
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen);
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure);
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send);
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg);

/* Coroutine.proto (used by CoroutineYieldFrom) */
#define __Pyx_Coroutine_USED
#define __Pyx_Coroutine_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CoroutineType)
#define __Pyx_Coroutine_Check(obj) __Pyx_Coroutine_CheckExact(obj)
#define __Pyx_CoroutineAwait_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CoroutineAwaitType)
#define __Pyx_Coroutine_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_mstate_global->__pyx_CoroutineType, body, code, closure, name, qualname, module_name)
static int __pyx_Coroutine_init(PyObject *module);
static PyObject *__Pyx__Coroutine_await(PyObject *coroutine);
typedef struct {
    PyObject_HEAD
    PyObject *coroutine;
} __pyx_CoroutineAwaitObject;
static __Pyx_PySendResult __Pyx_CoroutineAwait_Close(__pyx_CoroutineAwaitObject *self);

/* Coro_CheckExact.proto (used by GetAwaitIter) */
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_PyCoro_CheckExact(PyObject *o);
#else
#define __Pyx_PyCoro_CheckExact PyCoro_CheckExact
#endif

/* GetAwaitIter.proto (used by CoroutineYieldFrom) */
static CYTHON_INLINE PyObject *__Pyx_Coroutine_GetAwaitableIter(PyObject *o);
static PyObject *__Pyx__Coroutine_GetAwaitableIter(PyObject *o);

/* CoroutineYieldFrom.proto */
static CYTHON_INLINE __Pyx_PySendResult __Pyx_Coroutine_Yield_From(__pyx_CoroutineObject *gen, PyObject *source, PyObject **retval);

/* PyOSError_Check.proto */
#define __Pyx_PyExc_OSError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_OSError)

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* PyAttributeError_Check.proto */
#define __Pyx_PyExc_AttributeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AttributeError)

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_LIMITED_API  ||  PY_VERSION_HEX >= 0x030C0000
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  #if __clang__ || __GNUC__
  __attribute__((no_sanitize("thread")))
  #endif
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyAssertionError_Check.proto */
#define __Pyx_PyExc_AssertionError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AssertionError)

/* PyLongBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static CYTHON_INLINE PyObject* __Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check);
#else
#define __Pyx_PyLong_FloorDivideObjC(op1, op2, intval, inplace, zerodivision_check)\
    (inplace ? PyNumber_InPlaceFloorDivide(op1, op2) : PyNumber_FloorDivide(op1, op2))
#endif

/* RaiseClosureNameError.proto */
static void __Pyx_RaiseClosureNameError(const char *varname);

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* pop_index.proto */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix);
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix);
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (likely(PyList_CheckExact(L) && __Pyx_fits_Py_ssize_t(ix, type, is_signed))) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    __Pyx_fits_Py_ssize_t(ix, type, is_signed) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#else
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)\
    __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
        __Pyx__PyObject_PopIndex(L, py_ix))
#endif

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* UnicodeConcatInPlace.proto */
# if CYTHON_COMPILING_IN_CPYTHON
    #if CYTHON_REFNANNY
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared, __pyx_refnanny)
    #else
        #define __Pyx_PyUnicode_ConcatInPlace(left, right, unsafe_shared) __Pyx_PyUnicode_ConcatInPlaceImpl(&left, right, unsafe_shared)
    #endif
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_OwnStrongReference)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_DefinitelyUnique)
    #define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right) __Pyx_PyUnicode_ConcatInPlace(left, right, __Pyx_ReferenceSharing_SharedReference)
    static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
        #if CYTHON_REFNANNY
        , void* __pyx_refnanny
        #endif
    );
#else
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace __Pyx_PyUnicode_Concat
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace __Pyx_PyUnicode_Concat
#endif
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_DefinitelyUniqueInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_FunctionArgumentInPlace(left, right))
#define __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlaceSafe(left, right)\
    ((unlikely((left) == Py_None) || unlikely((right) == Py_None)) ?\
    PyNumber_InPlaceAdd(left, right) : __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_SharedReferenceInPlace(left, right))

/* RaiseKeywordRequired.proto */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name);

/* DictGetItem.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key);
#define __Pyx_PyObject_Dict_GetItem(obj, name)\
    (likely(PyDict_CheckExact(obj)) ?\
     __Pyx_PyDict_GetItem(obj, name) : PyObject_GetItem(obj, name))
#else
#define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#define __Pyx_PyObject_Dict_GetItem(obj, name)  PyObject_GetItem(obj, name)
#endif

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* pyfrozenset_new.proto (used by py_set_discard_unhashable) */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* py_set_discard.proto */
static CYTHON_INLINE int __Pyx_PySet_Discard(PyObject *set, PyObject *key);

/* PyIndexError_Check.proto */
#define __Pyx_PyExc_IndexError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_IndexError)

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* CheckTypeForFreelists.proto */
#if CYTHON_USE_FREELISTS
#if CYTHON_USE_TYPE_SPECS
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t) == (expected_tp)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  Py_TPFLAGS_IS_ABSTRACT
#else
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t)->tp_basicsize == (expected_size)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)
#endif
#define __PYX_CHECK_TYPE_FOR_FREELISTS(t, expected_tp, expected_size)\
    (__PYX_CHECK_FINAL_TYPE_FOR_FREELISTS((t), (expected_tp), (expected_size)) &\
     (int) (!__Pyx_PyType_HasFeature((t), __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS)))
#endif

/* VoidPtrExport.proto */
static int __Pyx_ExportVoidPtr(PyObject *api_dict, const char *name, void *p, const char *sig);

/* GetApiDict.proto */
static PyObject *__Pyx_ApiExport_GetApiDict(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig);

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_4
#define __PYX_HAVE_RT_ImportType_proto_3_2_4
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_4(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_4(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_4 {
   __Pyx_ImportType_CheckSize_Error_3_2_4 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_4 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_4 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_4(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_4 check_size);
#endif

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr_3_2_4(PyObject *module, const char *name, void **p, const char *sig);

/* HasAttr.proto (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* PyConnectionResetError_Check.proto */
#define __Pyx_PyExc_ConnectionResetError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ConnectionResetError)

/* Py3UpdateBases.proto */
static PyObject* __Pyx_PEP560_update_bases(PyObject *bases);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* DataclassesCallHelper.proto */
static PyObject* __Pyx_DataclassesCallHelper(PyObject *callable, PyObject *kwds);

/* LoadDataclassesModule.proto */
static PyObject *__Pyx_Load_dataclasses_Module(void);

/* GetNameInClass.proto */
#define __Pyx_GetNameInClass(var, nmspace, name)  (var) = __Pyx__GetNameInClass(nmspace, name)
static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name);

/* SetNameInClass.proto */
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? _PyDict_SetItem_KnownHash(ns, name, value, ((PyASCIIObject *) name)->hash) : PyObject_SetItem(ns, name, value))
#elif CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_SetNameInClass(ns, name, value)\
    (likely(PyDict_CheckExact(ns)) ? PyDict_SetItem(ns, name, value) : PyObject_SetItem(ns, name, value))
#else
#define __Pyx_SetNameInClass(ns, name, value)  PyObject_SetItem(ns, name, value)
#endif

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CheckUnpickleChecksum.proto */
static CYTHON_INLINE int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members);

/* GCCDiagnostics.proto */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyLong_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_unsigned_int(unsigned int value);

/* UpdateUnpickledDict.proto */
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index);

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* Generator.proto */
#define __Pyx_Generator_USED
#define __Pyx_Generator_CheckExact(obj) __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_mstate_global->__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(PyObject *module);
static CYTHON_INLINE PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

static PyObject *__pyx_f_13aioesphomeapi_10connection_13APIConnection_set_log_name(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__cleanup(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__increase_recv_buffer_size(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__process_hello_resp(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_resp); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__async_schedule_keep_alive(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_now); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__async_cancel_pong_timer(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_start_connect_future(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_finish_connect_future(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_connection_state(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_state); /* proto*/
static PyObject *__pyx_f_13aioesphomeapi_10connection_13APIConnection__make_auth_request(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_send_message(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msg, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_send_messages(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msgs, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__add_message_callback_without_remove(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types); /* proto*/
static PyObject *__pyx_f_13aioesphomeapi_10connection_13APIConnection_add_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__remove_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_report_fatal_error(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_err, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_fatal_exception_if_unset(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_err); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_process_packet(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, unsigned int __pyx_v_msg_type_proto, PyObject *__pyx_v_data, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__register_internal_message_handlers(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_disconnect_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__msg, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_ping_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__msg, int __pyx_skip_dispatch); /* proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_get_time_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__msg, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from "cython" */

/* Module declarations from "aioesphomeapi._frame_helper.base" */
static int *__pyx_vp_13aioesphomeapi_13_frame_helper_4base_TYPE_CHECKING = 0;
#define __pyx_v_13aioesphomeapi_13_frame_helper_4base_TYPE_CHECKING (*__pyx_vp_13aioesphomeapi_13_frame_helper_4base_TYPE_CHECKING)

/* Module declarations from "aioesphomeapi.connection" */
static PyObject *__pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_OPEN_STATES = 0;
static float __pyx_v_13aioesphomeapi_10connection_KEEP_ALIVE_TIMEOUT_RATIO;
static PyObject *__pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT = 0;
static int __pyx_v_13aioesphomeapi_10connection_TYPE_CHECKING;
static int __pyx_v_13aioesphomeapi_10connection__WIN32;
static PyObject *__pyx_v_13aioesphomeapi_10connection_WRITE_EXCEPTIONS = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_DISCONNECT_RESPONSE_MESSAGES = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_PING_REQUEST_MESSAGES = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_PING_RESPONSE_MESSAGES = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_asyncio_timeout = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CancelledError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_AuthenticationRequest = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_AuthenticationResponse = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_DisconnectRequest = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_PingRequest = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_GetTimeRequest = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_GetTimeResponse = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_HelloRequest = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_HelloResponse = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_APIVersion = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_partial = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_hr = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECT_AND_SETUP_TIMEOUT = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECT_REQUEST_TIMEOUT = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_APIConnectionError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_BadNameAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_HandshakeAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_PingFailedAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_SocketAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_astuple = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_make_hello_request = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection__handle_timeout = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection__handle_complex_message = 0;
static PyObject *__pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO = 0;
static PyObject *__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k;
static PyObject *__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v;
static void __pyx_f_13aioesphomeapi_10connection_handle_timeout(PyObject *, int __pyx_skip_dispatch); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_handle_complex_message(PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_13aioesphomeapi_10connection___pyx_unpickle_ConnectionParams__set_state(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *, PyObject *); /*proto*/
static PyObject *__pyx_f_13aioesphomeapi_10connection___pyx_unpickle_APIConnection__set_state(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "aioesphomeapi.connection"
extern int __pyx_module_is_main_aioesphomeapi__connection;
int __pyx_module_is_main_aioesphomeapi__connection = 0;

/* Implementation of "aioesphomeapi.connection" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin___import__;
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_NotImplemented;
/* #### Code section: string_decls ### */
static const char __pyx_k_addrs_info__cached_timezone__de[] = "_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name";
static const char __pyx_k_addresses_client_info_expected_m[] = "addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager";
/* #### Code section: decls ### */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_10genexpr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_genexpr_arg_0); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams___init__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_addresses, PyObject *__pyx_v_port, PyObject *__pyx_v_password, PyObject *__pyx_v_client_info, PyObject *__pyx_v_keepalive, PyObject *__pyx_v_zeroconf_manager, PyObject *__pyx_v_noise_psk, PyObject *__pyx_v_expected_name, PyObject *__pyx_v_expected_mac, PyObject *__pyx_v_timezone); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_2__repr__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4__eq__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_6__reduce_cython__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8__setstate_cython__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection__make_hello_request(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_client_info); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_2_build_dst_rule_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_rule); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_4_build_parsed_tz_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tz_string); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_6handle_timeout(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_fut); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_8handle_complex_message(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_fut, PyObject *__pyx_v_responses, PyObject *__pyx_v_do_append, PyObject *__pyx_v_do_stop, PyObject *__pyx_v_resp); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection___init__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_params, PyObject *__pyx_v_on_stop, int __pyx_v_debug_enabled, PyObject *__pyx_v_log_name, int __pyx_v_log_errors); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_2set_log_name(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_4set_debug(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_enable); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_genexpr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_genexpr_arg_0); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_6_connect_socket_connect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_addrs); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda2(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda3(PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_9_connect_init_frame_helper(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda4(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_resp); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_12_connect_hello_login(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_login); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_15_handle_login_response(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_login_response); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_async_send_keep_alive(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_async_pong_not_received(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_21start_resolve_host(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_24_set_resolve_host_future(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_26start_connection(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_29_raise_fatal_connection_exception(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_action, PyObject *__pyx_v_ex); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_31_wrap_fatal_connection_exception(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_action, PyObject *__pyx_v_ex); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_33_do_finish_connect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_login); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_36finish_connection(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_login); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_39send_message(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msg); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_41send_messages(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msgs); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_43add_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_45_remove_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_47send_message_callback_response(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_send_msg, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_49send_messages_await_response_complex(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_messages, PyObject *__pyx_v_do_append, PyObject *__pyx_v_do_stop, PyObject *__pyx_v_msg_types, PyObject *__pyx_v_timeout); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_52send_message_await_response(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_send_msg, PyObject *__pyx_v_response_type, PyObject *__pyx_v_timeout); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_55report_fatal_error(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_err); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_57process_packet(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, unsigned int __pyx_v_msg_type_proto, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_59_handle_disconnect_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v__msg); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_61_handle_ping_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v__msg); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_63_handle_get_time_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v__msg); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_65disconnect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_68force_disconnect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_12is_connected___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_12is_connected_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_70__reduce_cython__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_72__setstate_cython__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_13__pyx_unpickle_ConnectionParams(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_13aioesphomeapi_10connection_15__pyx_unpickle_APIConnection(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection_ConnectionParams(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection_APIConnection(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct__genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper;
  PyObject *__pyx_type_13aioesphomeapi_10connection_ConnectionParams;
  PyObject *__pyx_type_13aioesphomeapi_10connection_APIConnection;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response;
  PyObject *__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection_APIConnection;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response;
  PyTypeObject *__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_get;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  __Pyx_CachedCFunction __pyx_umethod_PySet_Type__copy;
  __Pyx_CachedCFunction __pyx_umethod_PyUnicode_Type__title;
  PyObject *__pyx_tuple[3];
  PyObject *__pyx_codeobj_tab[45];
  PyObject *__pyx_string_tab[568];
  PyObject *__pyx_number_tab[18];
/* #### Code section: module_state_contents ### */
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* IterNextPlain.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
PyObject *__Pyx_GetBuiltinNext_LimitedAPI_cache;
#endif

/* Coroutine.module_state_decls */
PyTypeObject *__pyx_CoroutineType;
PyTypeObject *__pyx_CoroutineAwaitType;

/* Coro_CheckExact.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedCoroType;
#endif

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;


#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct__genexpr[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct__genexpr;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response;
#endif

#if CYTHON_USE_FREELISTS
struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect[8];
int __pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect;
#endif
/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* Generator.module_state_decls */
PyTypeObject *__pyx_GeneratorType;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_An_error_that_is_raised_when_a_c __pyx_string_tab[1]
#define __pyx_kp_u_Callable_Any_None __pyx_string_tab[2]
#define __pyx_kp_u_Callable_None __pyx_string_tab[3]
#define __pyx_kp_u_Callable_message_Message_bool_No __pyx_string_tab[4]
#define __pyx_kp_u_Cleaning_up_connection_to_s __pyx_string_tab[5]
#define __pyx_kp_u_Cleaning_up_connection_to_s_erro __pyx_string_tab[6]
#define __pyx_kp_u_Connection_can_only_be_used_once __pyx_string_tab[7]
#define __pyx_kp_u_Connection_closed __pyx_string_tab[8]
#define __pyx_kp_u_Connection_is_closed __pyx_string_tab[9]
#define __pyx_kp_u_Connection_isn_t_established_yet __pyx_string_tab[10]
#define __pyx_kp_u_Connection_must_be_in_HOST_RESOL __pyx_string_tab[11]
#define __pyx_kp_u_Connection_must_be_in_SOCKET_OPE __pyx_string_tab[12]
#define __pyx_kp_u_Error_connecting_to __pyx_string_tab[13]
#define __pyx_kp_u_Error_while __pyx_string_tab[14]
#define __pyx_kp_u_Error_writing_packets __pyx_string_tab[15]
#define __pyx_kp_u_Expected __pyx_string_tab[16]
#define __pyx_kp_u_Handshake_failed __pyx_string_tab[17]
#define __pyx_kp_u_Handshake_timed_out_after __pyx_string_tab[18]
#define __pyx_kp_u_Incompatible_API_version __pyx_string_tab[19]
#define __pyx_kp_u_Invalid_password __pyx_string_tab[20]
#define __pyx_kp_u_Invalid_protobuf_message_type __pyx_string_tab[21]
#define __pyx_kp_u_None __pyx_string_tab[22]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[23]
#define __pyx_kp_u_ParsedTimezoneProto_None __pyx_string_tab[24]
#define __pyx_kp_u_Ping_response_not_received_after __pyx_string_tab[25]
#define __pyx_kp_u_Read_failed __pyx_string_tab[26]
#define __pyx_kp_u_Timed_out_waiting_to_finish_conn __pyx_string_tab[27]
#define __pyx_kp_u_Timeout_waiting_for __pyx_string_tab[28]
#define __pyx_kp_u_Timeout_while_connecting_to __pyx_string_tab[29]
#define __pyx_kp_u__10 __pyx_string_tab[30]
#define __pyx_kp_u__2 __pyx_string_tab[31]
#define __pyx_kp_u__3 __pyx_string_tab[32]
#define __pyx_kp_u__4 __pyx_string_tab[33]
#define __pyx_kp_u__5 __pyx_string_tab[34]
#define __pyx_kp_u__6 __pyx_string_tab[35]
#define __pyx_kp_u__7 __pyx_string_tab[36]
#define __pyx_kp_u__8 __pyx_string_tab[37]
#define __pyx_kp_u__9 __pyx_string_tab[38]
#define __pyx_kp_u_add_note __pyx_string_tab[39]
#define __pyx_kp_u_addresses_2 __pyx_string_tab[40]
#define __pyx_kp_u_after __pyx_string_tab[41]
#define __pyx_kp_u_aioesphomeapi__frame_helper_nois __pyx_string_tab[42]
#define __pyx_kp_u_aioesphomeapi__frame_helper_plai __pyx_string_tab[43]
#define __pyx_kp_u_aioesphomeapi_api_pb2 __pyx_string_tab[44]
#define __pyx_kp_u_aioesphomeapi_connection_py __pyx_string_tab[45]
#define __pyx_kp_u_aioesphomeapi_core __pyx_string_tab[46]
#define __pyx_kp_u_aioesphomeapi_model __pyx_string_tab[47]
#define __pyx_kp_u_aioesphomeapi_posix_tz __pyx_string_tab[48]
#define __pyx_kp_u_aioesphomeapi_timezone __pyx_string_tab[49]
#define __pyx_kp_u_aioesphomeapi_util __pyx_string_tab[50]
#define __pyx_kp_u_aioesphomeapi_zeroconf __pyx_string_tab[51]
#define __pyx_kp_u_asyncio_Future_None __pyx_string_tab[52]
#define __pyx_kp_u_but_server_sent_a_different_nam __pyx_string_tab[53]
#define __pyx_kp_u_client_info_2 __pyx_string_tab[54]
#define __pyx_kp_u_connection_2 __pyx_string_tab[55]
#define __pyx_kp_u_connection_cancelled __pyx_string_tab[56]
#define __pyx_kp_u_data __pyx_string_tab[57]
#define __pyx_kp_u_disable __pyx_string_tab[58]
#define __pyx_kp_u_due_to_fatal_exception __pyx_string_tab[59]
#define __pyx_kp_u_enable __pyx_string_tab[60]
#define __pyx_kp_u_expected_mac_2 __pyx_string_tab[61]
#define __pyx_kp_u_expected_name_2 __pyx_string_tab[62]
#define __pyx_kp_u_gc __pyx_string_tab[63]
#define __pyx_kp_u_getfullargspec __pyx_string_tab[64]
#define __pyx_kp_u_isenabled __pyx_string_tab[65]
#define __pyx_kp_u_keepalive_2 __pyx_string_tab[66]
#define __pyx_kp_u_list_hr_AddrInfo __pyx_string_tab[67]
#define __pyx_kp_u_list_message_Message __pyx_string_tab[68]
#define __pyx_kp_u_list_str __pyx_string_tab[69]
#define __pyx_kp_u_message_Message __pyx_string_tab[70]
#define __pyx_kp_u_noise_psk_2 __pyx_string_tab[71]
#define __pyx_kp_u_password_2 __pyx_string_tab[72]
#define __pyx_kp_u_port_2 __pyx_string_tab[73]
#define __pyx_kp_u_s_Connect_task_didn_t_finish_be __pyx_string_tab[74]
#define __pyx_kp_u_s_Connecting_to_s __pyx_string_tab[75]
#define __pyx_kp_u_s_Connection_error_occurred_s __pyx_string_tab[76]
#define __pyx_kp_u_s_Error_writing_packets_s __pyx_string_tab[77]
#define __pyx_kp_u_s_Failed_to_send_forced_disconn __pyx_string_tab[78]
#define __pyx_kp_u_s_Got_message_of_type_s_len_d_s __pyx_string_tab[79]
#define __pyx_kp_u_s_Incompatible_version_s_Closin __pyx_string_tab[80]
#define __pyx_kp_u_s_Invalid_protobuf_message_type __pyx_string_tab[81]
#define __pyx_kp_u_s_Opened_socket_to_s_s __pyx_string_tab[82]
#define __pyx_kp_u_s_PingResponse_pong_was_not_rec __pyx_string_tab[83]
#define __pyx_kp_u_s_Ping_response_not_received_af __pyx_string_tab[84]
#define __pyx_kp_u_s_Sending_s_s __pyx_string_tab[85]
#define __pyx_kp_u_s_Skipping_unknown_message_type __pyx_string_tab[86]
#define __pyx_kp_u_s_Successfully_connected_s_API __pyx_string_tab[87]
#define __pyx_kp_u_s_TCP_QUICKACK_not_supported __pyx_string_tab[88]
#define __pyx_kp_u_s_Unable_to_increase_the_socket __pyx_string_tab[89]
#define __pyx_kp_u_s_disconnect_request_failed __pyx_string_tab[90]
#define __pyx_kp_u_seconds __pyx_string_tab[91]
#define __pyx_kp_u_str_None __pyx_string_tab[92]
#define __pyx_kp_u_timezone_2 __pyx_string_tab[93]
#define __pyx_kp_u_tree_fragment __pyx_string_tab[94]
#define __pyx_kp_u_tuple_message_Message __pyx_string_tab[95]
#define __pyx_kp_u_tuple_type_Any __pyx_string_tab[96]
#define __pyx_kp_u_zeroconf_manager_2 __pyx_string_tab[97]
#define __pyx_n_u_APIConnection __pyx_string_tab[98]
#define __pyx_n_u_APIConnectionCancelledError __pyx_string_tab[99]
#define __pyx_n_u_APIConnectionError __pyx_string_tab[100]
#define __pyx_n_u_APIConnection___reduce_cython __pyx_string_tab[101]
#define __pyx_n_u_APIConnection___setstate_cython __pyx_string_tab[102]
#define __pyx_n_u_APIConnection__async_pong_not_re __pyx_string_tab[103]
#define __pyx_n_u_APIConnection__async_send_keep_a __pyx_string_tab[104]
#define __pyx_n_u_APIConnection__connect_hello_log __pyx_string_tab[105]
#define __pyx_n_u_APIConnection__connect_init_fram __pyx_string_tab[106]
#define __pyx_n_u_APIConnection__connect_socket_co __pyx_string_tab[107]
#define __pyx_n_u_APIConnection__do_finish_connect __pyx_string_tab[108]
#define __pyx_n_u_APIConnection__handle_disconnect __pyx_string_tab[109]
#define __pyx_n_u_APIConnection__handle_get_time_r __pyx_string_tab[110]
#define __pyx_n_u_APIConnection__handle_login_resp __pyx_string_tab[111]
#define __pyx_n_u_APIConnection__handle_ping_reque __pyx_string_tab[112]
#define __pyx_n_u_APIConnection__raise_fatal_conne __pyx_string_tab[113]
#define __pyx_n_u_APIConnection__remove_message_ca __pyx_string_tab[114]
#define __pyx_n_u_APIConnection__set_resolve_host __pyx_string_tab[115]
#define __pyx_n_u_APIConnection__wrap_fatal_connec __pyx_string_tab[116]
#define __pyx_n_u_APIConnection_add_message_callba __pyx_string_tab[117]
#define __pyx_n_u_APIConnection_disconnect __pyx_string_tab[118]
#define __pyx_n_u_APIConnection_finish_connection __pyx_string_tab[119]
#define __pyx_n_u_APIConnection_force_disconnect __pyx_string_tab[120]
#define __pyx_n_u_APIConnection_process_packet __pyx_string_tab[121]
#define __pyx_n_u_APIConnection_report_fatal_error __pyx_string_tab[122]
#define __pyx_n_u_APIConnection_send_message __pyx_string_tab[123]
#define __pyx_n_u_APIConnection_send_message_await __pyx_string_tab[124]
#define __pyx_n_u_APIConnection_send_message_callb __pyx_string_tab[125]
#define __pyx_n_u_APIConnection_send_messages __pyx_string_tab[126]
#define __pyx_n_u_APIConnection_send_messages_awai __pyx_string_tab[127]
#define __pyx_n_u_APIConnection_set_debug __pyx_string_tab[128]
#define __pyx_n_u_APIConnection_set_log_name __pyx_string_tab[129]
#define __pyx_n_u_APIConnection_start_connection __pyx_string_tab[130]
#define __pyx_n_u_APIConnection_start_resolve_host __pyx_string_tab[131]
#define __pyx_n_u_APINoiseFrameHelper __pyx_string_tab[132]
#define __pyx_n_u_APIPlaintextFrameHelper __pyx_string_tab[133]
#define __pyx_n_u_APIVersion __pyx_string_tab[134]
#define __pyx_n_u_Any __pyx_string_tab[135]
#define __pyx_n_u_AuthenticationRequest __pyx_string_tab[136]
#define __pyx_n_u_AuthenticationResponse __pyx_string_tab[137]
#define __pyx_n_u_BadNameAPIError __pyx_string_tab[138]
#define __pyx_n_u_BaseException __pyx_string_tab[139]
#define __pyx_n_u_CLOSED __pyx_string_tab[140]
#define __pyx_n_u_CONNECTED __pyx_string_tab[141]
#define __pyx_n_u_Callable __pyx_string_tab[142]
#define __pyx_n_u_CancelledError __pyx_string_tab[143]
#define __pyx_n_u_ConnectionInterruptedError __pyx_string_tab[144]
#define __pyx_n_u_ConnectionNotEstablishedAPIError __pyx_string_tab[145]
#define __pyx_n_u_ConnectionParams __pyx_string_tab[146]
#define __pyx_n_u_ConnectionParams___reduce_cython __pyx_string_tab[147]
#define __pyx_n_u_ConnectionParams___setstate_cyth __pyx_string_tab[148]
#define __pyx_n_u_ConnectionState __pyx_string_tab[149]
#define __pyx_n_u_CopyFrom __pyx_string_tab[150]
#define __pyx_n_u_DAY_OF_YEAR __pyx_string_tab[151]
#define __pyx_n_u_DEBUG __pyx_string_tab[152]
#define __pyx_n_u_DISCONNECT_CONNECT_TIMEOUT __pyx_string_tab[153]
#define __pyx_n_u_DISCONNECT_RESPONSE_TIMEOUT __pyx_string_tab[154]
#define __pyx_n_u_DSTRule __pyx_string_tab[155]
#define __pyx_n_u_DSTRuleParsed __pyx_string_tab[156]
#define __pyx_n_u_DSTRuleProto __pyx_string_tab[157]
#define __pyx_n_u_DSTRuleType __pyx_string_tab[158]
#define __pyx_n_u_DST_RULE_TYPE_DAY_OF_YEAR __pyx_string_tab[159]
#define __pyx_n_u_DST_RULE_TYPE_DAY_OF_YEAR_PB __pyx_string_tab[160]
#define __pyx_n_u_DST_RULE_TYPE_JULIAN_NO_LEAP __pyx_string_tab[161]
#define __pyx_n_u_DST_RULE_TYPE_JULIAN_NO_LEAP_PB __pyx_string_tab[162]
#define __pyx_n_u_DST_RULE_TYPE_MAP __pyx_string_tab[163]
#define __pyx_n_u_DST_RULE_TYPE_MONTH_WEEK_DAY __pyx_string_tab[164]
#define __pyx_n_u_DST_RULE_TYPE_MONTH_WEEK_DAY_PB __pyx_string_tab[165]
#define __pyx_n_u_DST_RULE_TYPE_NONE __pyx_string_tab[166]
#define __pyx_n_u_DST_RULE_TYPE_NONE_PB __pyx_string_tab[167]
#define __pyx_n_u_DataclassParams __pyx_string_tab[168]
#define __pyx_n_u_DisconnectRequest __pyx_string_tab[169]
#define __pyx_n_u_DisconnectResponse __pyx_string_tab[170]
#define __pyx_n_u_Enum __pyx_string_tab[171]
#define __pyx_n_u_Exception __pyx_string_tab[172]
#define __pyx_n_u_FIELD __pyx_string_tab[173]
#define __pyx_n_u_GetTimeRequest __pyx_string_tab[174]
#define __pyx_n_u_GetTimeResponse __pyx_string_tab[175]
#define __pyx_n_u_HANDSHAKE_COMPLETE __pyx_string_tab[176]
#define __pyx_n_u_HOST_RESOLVED __pyx_string_tab[177]
#define __pyx_n_u_HandshakeAPIError __pyx_string_tab[178]
#define __pyx_n_u_HelloRequest __pyx_string_tab[179]
#define __pyx_n_u_HelloResponse __pyx_string_tab[180]
#define __pyx_n_u_INITIALIZED __pyx_string_tab[181]
#define __pyx_n_u_IPPROTO_TCP __pyx_string_tab[182]
#define __pyx_n_u_InvalidAuthAPIError __pyx_string_tab[183]
#define __pyx_n_u_JULIAN_NO_LEAP __pyx_string_tab[184]
#define __pyx_n_u_LOGGER __pyx_string_tab[185]
#define __pyx_n_u_MESSAGE_TYPE_TO_PROTO __pyx_string_tab[186]
#define __pyx_n_u_MIN_BUFFER_SIZE __pyx_string_tab[187]
#define __pyx_n_u_MONTH_WEEK_DAY __pyx_string_tab[188]
#define __pyx_n_u_MergeFromString __pyx_string_tab[189]
#define __pyx_n_u_MessageToDict __pyx_string_tab[190]
#define __pyx_n_u_NONE __pyx_string_tab[191]
#define __pyx_n_u_NoReturn __pyx_string_tab[192]
#define __pyx_n_u_None __pyx_string_tab[193]
#define __pyx_n_u_NotImplemented __pyx_string_tab[194]
#define __pyx_n_u_PREFERRED_BUFFER_SIZE __pyx_string_tab[195]
#define __pyx_n_u_ParsedTimezone __pyx_string_tab[196]
#define __pyx_n_u_ParsedTimezoneProto __pyx_string_tab[197]
#define __pyx_n_u_PingFailedAPIError __pyx_string_tab[198]
#define __pyx_n_u_PingRequest __pyx_string_tab[199]
#define __pyx_n_u_PingResponse __pyx_string_tab[200]
#define __pyx_n_u_ProtocolAPIError __pyx_string_tab[201]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[202]
#define __pyx_n_u_ReadFailedAPIError __pyx_string_tab[203]
#define __pyx_n_u_SOCKET_OPENED __pyx_string_tab[204]
#define __pyx_n_u_SOL_SOCKET __pyx_string_tab[205]
#define __pyx_n_u_SO_RCVBUF __pyx_string_tab[206]
#define __pyx_n_u_SerializeToString __pyx_string_tab[207]
#define __pyx_n_u_SocketAPIError __pyx_string_tab[208]
#define __pyx_n_u_SocketClosedAPIError __pyx_string_tab[209]
#define __pyx_n_u_TCP_CONNECT_TIMEOUT __pyx_string_tab[210]
#define __pyx_n_u_TCP_NODELAY __pyx_string_tab[211]
#define __pyx_n_u_TCP_QUICKACK __pyx_string_tab[212]
#define __pyx_n_u_TYPE_CHECKING __pyx_string_tab[213]
#define __pyx_n_u_TimeoutAPIError __pyx_string_tab[214]
#define __pyx_n_u_UnhandledAPIConnectionError __pyx_string_tab[215]
#define __pyx_n_u_WARNING __pyx_string_tab[216]
#define __pyx_n_u_ZeroconfManager __pyx_string_tab[217]
#define __pyx_n_u__11 __pyx_string_tab[218]
#define __pyx_n_u_action __pyx_string_tab[219]
#define __pyx_n_u_add __pyx_string_tab[220]
#define __pyx_n_u_add_message_callback __pyx_string_tab[221]
#define __pyx_n_u_addr __pyx_string_tab[222]
#define __pyx_n_u_addr_infos __pyx_string_tab[223]
#define __pyx_n_u_addresses __pyx_string_tab[224]
#define __pyx_n_u_addrs __pyx_string_tab[225]
#define __pyx_n_u_addrs_info __pyx_string_tab[226]
#define __pyx_n_u_aenter __pyx_string_tab[227]
#define __pyx_n_u_aexit __pyx_string_tab[228]
#define __pyx_n_u_aioesphomeapi_connection __pyx_string_tab[229]
#define __pyx_n_u_aioesphomeapi_host_resolver __pyx_string_tab[230]
#define __pyx_n_u_aiohappyeyeballs __pyx_string_tab[231]
#define __pyx_n_u_api_pb2 __pyx_string_tab[232]
#define __pyx_n_u_api_version __pyx_string_tab[233]
#define __pyx_n_u_api_version_major __pyx_string_tab[234]
#define __pyx_n_u_api_version_minor __pyx_string_tab[235]
#define __pyx_n_u_astuple __pyx_string_tab[236]
#define __pyx_n_u_async_interrupt __pyx_string_tab[237]
#define __pyx_n_u_async_pong_not_received __pyx_string_tab[238]
#define __pyx_n_u_async_resolve_host __pyx_string_tab[239]
#define __pyx_n_u_async_send_keep_alive __pyx_string_tab[240]
#define __pyx_n_u_asyncio __pyx_string_tab[241]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[242]
#define __pyx_n_u_asyncio_timeout __pyx_string_tab[243]
#define __pyx_n_u_await __pyx_string_tab[244]
#define __pyx_n_u_bool __pyx_string_tab[245]
#define __pyx_n_u_build_dst_rule_proto __pyx_string_tab[246]
#define __pyx_n_u_build_parsed_tz_proto __pyx_string_tab[247]
#define __pyx_n_u_bytes __pyx_string_tab[248]
#define __pyx_n_u_cached_make_hello_request __pyx_string_tab[249]
#define __pyx_n_u_cached_timezone __pyx_string_tab[250]
#define __pyx_n_u_call_at __pyx_string_tab[251]
#define __pyx_n_u_cancel __pyx_string_tab[252]
#define __pyx_n_u_cancelling __pyx_string_tab[253]
#define __pyx_n_u_cause __pyx_string_tab[254]
#define __pyx_n_u_cause_2 __pyx_string_tab[255]
#define __pyx_n_u_class __pyx_string_tab[256]
#define __pyx_n_u_class_getitem __pyx_string_tab[257]
#define __pyx_n_u_client_info __pyx_string_tab[258]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[259]
#define __pyx_n_u_close __pyx_string_tab[260]
#define __pyx_n_u_collections_abc __pyx_string_tab[261]
#define __pyx_n_u_compare __pyx_string_tab[262]
#define __pyx_n_u_connect_hello_login __pyx_string_tab[263]
#define __pyx_n_u_connect_hello_login_locals_lamb __pyx_string_tab[264]
#define __pyx_n_u_connect_init_frame_helper __pyx_string_tab[265]
#define __pyx_n_u_connect_init_frame_helper_local __pyx_string_tab[266]
#define __pyx_n_u_connect_socket_connect __pyx_string_tab[267]
#define __pyx_n_u_connect_socket_connect_locals_g __pyx_string_tab[268]
#define __pyx_n_u_connected_address __pyx_string_tab[269]
#define __pyx_n_u_connection __pyx_string_tab[270]
#define __pyx_n_u_connection_state __pyx_string_tab[271]
#define __pyx_n_u_copy __pyx_string_tab[272]
#define __pyx_n_u_core __pyx_string_tab[273]
#define __pyx_n_u_create_connection __pyx_string_tab[274]
#define __pyx_n_u_create_future __pyx_string_tab[275]
#define __pyx_n_u_current_task __pyx_string_tab[276]
#define __pyx_n_u_data_2 __pyx_string_tab[277]
#define __pyx_n_u_dataclass __pyx_string_tab[278]
#define __pyx_n_u_dataclass_fields __pyx_string_tab[279]
#define __pyx_n_u_dataclass_params __pyx_string_tab[280]
#define __pyx_n_u_dataclasses __pyx_string_tab[281]
#define __pyx_n_u_day __pyx_string_tab[282]
#define __pyx_n_u_day_of_week __pyx_string_tab[283]
#define __pyx_n_u_debug __pyx_string_tab[284]
#define __pyx_n_u_debug_enabled __pyx_string_tab[285]
#define __pyx_n_u_debug_enabled_2 __pyx_string_tab[286]
#define __pyx_n_u_default __pyx_string_tab[287]
#define __pyx_n_u_dict __pyx_string_tab[288]
#define __pyx_n_u_dict_2 __pyx_string_tab[289]
#define __pyx_n_u_disconnect __pyx_string_tab[290]
#define __pyx_n_u_do_append __pyx_string_tab[291]
#define __pyx_n_u_do_finish_connect __pyx_string_tab[292]
#define __pyx_n_u_do_stop __pyx_string_tab[293]
#define __pyx_n_u_doc __pyx_string_tab[294]
#define __pyx_n_u_done __pyx_string_tab[295]
#define __pyx_n_u_dst_end __pyx_string_tab[296]
#define __pyx_n_u_dst_offset_seconds __pyx_string_tab[297]
#define __pyx_n_u_dst_start __pyx_string_tab[298]
#define __pyx_n_u_enable __pyx_string_tab[299]
#define __pyx_n_u_enum __pyx_string_tab[300]
#define __pyx_n_u_epoch_seconds __pyx_string_tab[301]
#define __pyx_n_u_eq __pyx_string_tab[302]
#define __pyx_n_u_err __pyx_string_tab[303]
#define __pyx_n_u_err_str __pyx_string_tab[304]
#define __pyx_n_u_error __pyx_string_tab[305]
#define __pyx_n_u_ex __pyx_string_tab[306]
#define __pyx_n_u_exc_info __pyx_string_tab[307]
#define __pyx_n_u_exception __pyx_string_tab[308]
#define __pyx_n_u_expected_disconnect __pyx_string_tab[309]
#define __pyx_n_u_expected_mac __pyx_string_tab[310]
#define __pyx_n_u_expected_name __pyx_string_tab[311]
#define __pyx_n_u_family __pyx_string_tab[312]
#define __pyx_n_u_fatal_exc __pyx_string_tab[313]
#define __pyx_n_u_fatal_exception __pyx_string_tab[314]
#define __pyx_n_u_fh __pyx_string_tab[315]
#define __pyx_n_u_field __pyx_string_tab[316]
#define __pyx_n_u_field_type __pyx_string_tab[317]
#define __pyx_n_u_finish_connect_future __pyx_string_tab[318]
#define __pyx_n_u_finish_connection __pyx_string_tab[319]
#define __pyx_n_u_finishing __pyx_string_tab[320]
#define __pyx_n_u_float __pyx_string_tab[321]
#define __pyx_n_u_float_2 __pyx_string_tab[322]
#define __pyx_n_u_force_disconnect __pyx_string_tab[323]
#define __pyx_n_u_frame_helper __pyx_string_tab[324]
#define __pyx_n_u_frame_helper_noise __pyx_string_tab[325]
#define __pyx_n_u_frame_helper_plain_text __pyx_string_tab[326]
#define __pyx_n_u_frozen __pyx_string_tab[327]
#define __pyx_n_u_func __pyx_string_tab[328]
#define __pyx_n_u_functools __pyx_string_tab[329]
#define __pyx_n_u_fut __pyx_string_tab[330]
#define __pyx_n_u_genexpr __pyx_string_tab[331]
#define __pyx_n_u_get __pyx_string_tab[332]
#define __pyx_n_u_getLogger __pyx_string_tab[333]
#define __pyx_n_u_get_running_loop __pyx_string_tab[334]
#define __pyx_n_u_get_timezone __pyx_string_tab[335]
#define __pyx_n_u_getpeername __pyx_string_tab[336]
#define __pyx_n_u_getstate __pyx_string_tab[337]
#define __pyx_n_u_google_protobuf __pyx_string_tab[338]
#define __pyx_n_u_google_protobuf_json_format __pyx_string_tab[339]
#define __pyx_n_u_handle_complex_message __pyx_string_tab[340]
#define __pyx_n_u_handle_disconnect_request_inter __pyx_string_tab[341]
#define __pyx_n_u_handle_get_time_request_interna __pyx_string_tab[342]
#define __pyx_n_u_handle_login_response __pyx_string_tab[343]
#define __pyx_n_u_handle_ping_request_internal __pyx_string_tab[344]
#define __pyx_n_u_handle_timeout __pyx_string_tab[345]
#define __pyx_n_u_handshake_complete __pyx_string_tab[346]
#define __pyx_n_u_handshake_handle __pyx_string_tab[347]
#define __pyx_n_u_happy_eyeballs_delay __pyx_string_tab[348]
#define __pyx_n_u_hash __pyx_string_tab[349]
#define __pyx_n_u_host_resolver __pyx_string_tab[350]
#define __pyx_n_u_id __pyx_string_tab[351]
#define __pyx_n_u_import __pyx_string_tab[352]
#define __pyx_n_u_info __pyx_string_tab[353]
#define __pyx_n_u_init __pyx_string_tab[354]
#define __pyx_n_u_int __pyx_string_tab[355]
#define __pyx_n_u_int_2 __pyx_string_tab[356]
#define __pyx_n_u_interleave __pyx_string_tab[357]
#define __pyx_n_u_interrupt __pyx_string_tab[358]
#define __pyx_n_u_invalid_password __pyx_string_tab[359]
#define __pyx_n_u_is_connected __pyx_string_tab[360]
#define __pyx_n_u_is_coroutine __pyx_string_tab[361]
#define __pyx_n_u_items __pyx_string_tab[362]
#define __pyx_n_u_keep_alive_interval __pyx_string_tab[363]
#define __pyx_n_u_keep_alive_timeout __pyx_string_tab[364]
#define __pyx_n_u_keepalive __pyx_string_tab[365]
#define __pyx_n_u_klass __pyx_string_tab[366]
#define __pyx_n_u_kw_only __pyx_string_tab[367]
#define __pyx_n_u_lambda __pyx_string_tab[368]
#define __pyx_n_u_last_exception __pyx_string_tab[369]
#define __pyx_n_u_log __pyx_string_tab[370]
#define __pyx_n_u_log_errors __pyx_string_tab[371]
#define __pyx_n_u_log_errors_2 __pyx_string_tab[372]
#define __pyx_n_u_log_name __pyx_string_tab[373]
#define __pyx_n_u_logging __pyx_string_tab[374]
#define __pyx_n_u_login __pyx_string_tab[375]
#define __pyx_n_u_login_response __pyx_string_tab[376]
#define __pyx_n_u_loop __pyx_string_tab[377]
#define __pyx_n_u_loop_2 __pyx_string_tab[378]
#define __pyx_n_u_lru_cache __pyx_string_tab[379]
#define __pyx_n_u_main __pyx_string_tab[380]
#define __pyx_n_u_major __pyx_string_tab[381]
#define __pyx_n_u_make_hello_request __pyx_string_tab[382]
#define __pyx_n_u_match_args __pyx_string_tab[383]
#define __pyx_n_u_maxsize __pyx_string_tab[384]
#define __pyx_n_u_message __pyx_string_tab[385]
#define __pyx_n_u_message_handlers __pyx_string_tab[386]
#define __pyx_n_u_message_types_to_names __pyx_string_tab[387]
#define __pyx_n_u_messages __pyx_string_tab[388]
#define __pyx_n_u_metaclass __pyx_string_tab[389]
#define __pyx_n_u_metadata __pyx_string_tab[390]
#define __pyx_n_u_model __pyx_string_tab[391]
#define __pyx_n_u_module __pyx_string_tab[392]
#define __pyx_n_u_month __pyx_string_tab[393]
#define __pyx_n_u_mro_entries __pyx_string_tab[394]
#define __pyx_n_u_msg __pyx_string_tab[395]
#define __pyx_n_u_msg_2 __pyx_string_tab[396]
#define __pyx_n_u_msg_type_proto __pyx_string_tab[397]
#define __pyx_n_u_msg_types __pyx_string_tab[398]
#define __pyx_n_u_msgs __pyx_string_tab[399]
#define __pyx_n_u_name __pyx_string_tab[400]
#define __pyx_n_u_name_2 __pyx_string_tab[401]
#define __pyx_n_u_new __pyx_string_tab[402]
#define __pyx_n_u_new_exc __pyx_string_tab[403]
#define __pyx_n_u_next __pyx_string_tab[404]
#define __pyx_n_u_noise_psk __pyx_string_tab[405]
#define __pyx_n_u_now __pyx_string_tab[406]
#define __pyx_n_u_object __pyx_string_tab[407]
#define __pyx_n_u_on_message __pyx_string_tab[408]
#define __pyx_n_u_on_stop __pyx_string_tab[409]
#define __pyx_n_u_order __pyx_string_tab[410]
#define __pyx_n_u_params __pyx_string_tab[411]
#define __pyx_n_u_params_2 __pyx_string_tab[412]
#define __pyx_n_u_parse_posix_tz __pyx_string_tab[413]
#define __pyx_n_u_parsed __pyx_string_tab[414]
#define __pyx_n_u_parsed_timezone __pyx_string_tab[415]
#define __pyx_n_u_partial __pyx_string_tab[416]
#define __pyx_n_u_password __pyx_string_tab[417]
#define __pyx_n_u_pending __pyx_string_tab[418]
#define __pyx_n_u_ping_timer __pyx_string_tab[419]
#define __pyx_n_u_platform __pyx_string_tab[420]
#define __pyx_n_u_pong_timer __pyx_string_tab[421]
#define __pyx_n_u_pop __pyx_string_tab[422]
#define __pyx_n_u_pop_addr_infos_interleave __pyx_string_tab[423]
#define __pyx_n_u_port __pyx_string_tab[424]
#define __pyx_n_u_posix_tz __pyx_string_tab[425]
#define __pyx_n_u_prepare __pyx_string_tab[426]
#define __pyx_n_u_process_packet __pyx_string_tab[427]
#define __pyx_n_u_proto __pyx_string_tab[428]
#define __pyx_n_u_pyx_capi __pyx_string_tab[429]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[430]
#define __pyx_n_u_pyx_recursive_repr_guard __pyx_string_tab[431]
#define __pyx_n_u_pyx_result __pyx_string_tab[432]
#define __pyx_n_u_pyx_state __pyx_string_tab[433]
#define __pyx_n_u_pyx_type __pyx_string_tab[434]
#define __pyx_n_u_pyx_unpickle_APIConnection __pyx_string_tab[435]
#define __pyx_n_u_pyx_unpickle_ConnectionParams __pyx_string_tab[436]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[437]
#define __pyx_n_u_qualname __pyx_string_tab[438]
#define __pyx_n_u_raise_fatal_connection_exceptio __pyx_string_tab[439]
#define __pyx_n_u_read_exception_futures __pyx_string_tab[440]
#define __pyx_n_u_received_name __pyx_string_tab[441]
#define __pyx_n_u_reduce __pyx_string_tab[442]
#define __pyx_n_u_reduce_cython __pyx_string_tab[443]
#define __pyx_n_u_reduce_ex __pyx_string_tab[444]
#define __pyx_n_u_remove __pyx_string_tab[445]
#define __pyx_n_u_remove_message_callback __pyx_string_tab[446]
#define __pyx_n_u_report_fatal_error __pyx_string_tab[447]
#define __pyx_n_u_repr __pyx_string_tab[448]
#define __pyx_n_u_resolve_host_future __pyx_string_tab[449]
#define __pyx_n_u_resolving __pyx_string_tab[450]
#define __pyx_n_u_resp __pyx_string_tab[451]
#define __pyx_n_u_response __pyx_string_tab[452]
#define __pyx_n_u_response_names __pyx_string_tab[453]
#define __pyx_n_u_response_type __pyx_string_tab[454]
#define __pyx_n_u_responses __pyx_string_tab[455]
#define __pyx_n_u_return __pyx_string_tab[456]
#define __pyx_n_u_rule __pyx_string_tab[457]
#define __pyx_n_u_running __pyx_string_tab[458]
#define __pyx_n_u_s __pyx_string_tab[459]
#define __pyx_n_u_self __pyx_string_tab[460]
#define __pyx_n_u_send __pyx_string_tab[461]
#define __pyx_n_u_send_message __pyx_string_tab[462]
#define __pyx_n_u_send_message_await_response __pyx_string_tab[463]
#define __pyx_n_u_send_message_callback_response __pyx_string_tab[464]
#define __pyx_n_u_send_messages __pyx_string_tab[465]
#define __pyx_n_u_send_messages_await_response_com __pyx_string_tab[466]
#define __pyx_n_u_send_msg __pyx_string_tab[467]
#define __pyx_n_u_send_pending_ping __pyx_string_tab[468]
#define __pyx_n_u_server_info __pyx_string_tab[469]
#define __pyx_n_u_set_debug __pyx_string_tab[470]
#define __pyx_n_u_set_exception __pyx_string_tab[471]
#define __pyx_n_u_set_log_name __pyx_string_tab[472]
#define __pyx_n_u_set_name __pyx_string_tab[473]
#define __pyx_n_u_set_resolve_host_future __pyx_string_tab[474]
#define __pyx_n_u_set_result __pyx_string_tab[475]
#define __pyx_n_u_setblocking __pyx_string_tab[476]
#define __pyx_n_u_setdefault __pyx_string_tab[477]
#define __pyx_n_u_setsockopt __pyx_string_tab[478]
#define __pyx_n_u_setstate __pyx_string_tab[479]
#define __pyx_n_u_setstate_cython __pyx_string_tab[480]
#define __pyx_n_u_slots __pyx_string_tab[481]
#define __pyx_n_u_slots_2 __pyx_string_tab[482]
#define __pyx_n_u_sock __pyx_string_tab[483]
#define __pyx_n_u_sockaddr __pyx_string_tab[484]
#define __pyx_n_u_socket __pyx_string_tab[485]
#define __pyx_n_u_socket_2 __pyx_string_tab[486]
#define __pyx_n_u_start_connect_future __pyx_string_tab[487]
#define __pyx_n_u_start_connection __pyx_string_tab[488]
#define __pyx_n_u_start_resolve_host __pyx_string_tab[489]
#define __pyx_n_u_starting __pyx_string_tab[490]
#define __pyx_n_u_state __pyx_string_tab[491]
#define __pyx_n_u_std_offset_seconds __pyx_string_tab[492]
#define __pyx_n_u_str __pyx_string_tab[493]
#define __pyx_n_u_sys __pyx_string_tab[494]
#define __pyx_n_u_test __pyx_string_tab[495]
#define __pyx_n_u_throw __pyx_string_tab[496]
#define __pyx_n_u_time __pyx_string_tab[497]
#define __pyx_n_u_time_seconds __pyx_string_tab[498]
#define __pyx_n_u_timeout __pyx_string_tab[499]
#define __pyx_n_u_timeout_expired __pyx_string_tab[500]
#define __pyx_n_u_timeout_handle __pyx_string_tab[501]
#define __pyx_n_u_timezone __pyx_string_tab[502]
#define __pyx_n_u_title __pyx_string_tab[503]
#define __pyx_n_u_type __pyx_string_tab[504]
#define __pyx_n_u_typing __pyx_string_tab[505]
#define __pyx_n_u_tz_string __pyx_string_tab[506]
#define __pyx_n_u_unsafe_hash __pyx_string_tab[507]
#define __pyx_n_u_update __pyx_string_tab[508]
#define __pyx_n_u_use_setstate __pyx_string_tab[509]
#define __pyx_n_u_util __pyx_string_tab[510]
#define __pyx_n_u_value __pyx_string_tab[511]
#define __pyx_n_u_values __pyx_string_tab[512]
#define __pyx_n_u_wait __pyx_string_tab[513]
#define __pyx_n_u_warning __pyx_string_tab[514]
#define __pyx_n_u_weakref_slot __pyx_string_tab[515]
#define __pyx_n_u_week __pyx_string_tab[516]
#define __pyx_n_u_win32 __pyx_string_tab[517]
#define __pyx_n_u_wrap_fatal_connection_exception __pyx_string_tab[518]
#define __pyx_n_u_zeroconf __pyx_string_tab[519]
#define __pyx_n_u_zeroconf_manager __pyx_string_tab[520]
#define __pyx_kp_b_PyObject_float_int_APIConnection __pyx_string_tab[521]
#define __pyx_kp_b_int_TYPE_CHECKING __pyx_string_tab[522]
#define __pyx_kp_b_iso88591_0 __pyx_string_tab[523]
#define __pyx_kp_b_iso88591_1 __pyx_string_tab[524]
#define __pyx_kp_b_iso88591_1F __pyx_string_tab[525]
#define __pyx_kp_b_iso88591_1_1_81_q_q_A_z_Q_4q_6_A_Q_A_Q_1 __pyx_string_tab[526]
#define __pyx_kp_b_iso88591_1_a_ha_XQ_q_HA_T __pyx_string_tab[527]
#define __pyx_kp_b_iso88591_2 __pyx_string_tab[528]
#define __pyx_kp_b_iso88591_2_q_T_D_q_A_d_T_D __pyx_string_tab[529]
#define __pyx_kp_b_iso88591_33I __pyx_string_tab[530]
#define __pyx_kp_b_iso88591_4q_6_A_A_q_a_31D __pyx_string_tab[531]
#define __pyx_kp_b_iso88591_4t1_2_4AT_1_Qd_2Qa_q_1_aq_4q_q __pyx_string_tab[532]
#define __pyx_kp_b_iso88591_55PPQ_1_7q __pyx_string_tab[533]
#define __pyx_kp_b_iso88591_8_iq __pyx_string_tab[534]
#define __pyx_kp_b_iso88591_A_4_D_O7_N_A_z_IQ __pyx_string_tab[535]
#define __pyx_kp_b_iso88591_A_AQ_1 __pyx_string_tab[536]
#define __pyx_kp_b_iso88591_A_A_1_a_wat_6l __pyx_string_tab[537]
#define __pyx_kp_b_iso88591_A_A_L_t_Qa_HAQ __pyx_string_tab[538]
#define __pyx_kp_b_iso88591_A_A_N_1_IQ __pyx_string_tab[539]
#define __pyx_kp_b_iso88591_A_N_1 __pyx_string_tab[540]
#define __pyx_kp_b_iso88591_A_N_A_t_Q __pyx_string_tab[541]
#define __pyx_kp_b_iso88591_A_a_S_U_L_A_4q_WA_N_A __pyx_string_tab[542]
#define __pyx_kp_b_iso88591_A_q_D_9_q_Qd_WM_WA_Ks_A_a __pyx_string_tab[543]
#define __pyx_kp_b_iso88591_A_q_Qd_1_a_Qe_7q_c_vQ_t1_6at1_A __pyx_string_tab[544]
#define __pyx_kp_b_iso88591_L_4_gQ_m1A __pyx_string_tab[545]
#define __pyx_kp_b_iso88591_N_A __pyx_string_tab[546]
#define __pyx_kp_b_iso88591_Q __pyx_string_tab[547]
#define __pyx_kp_b_iso88591_T_T_t_FVVZZffjjvvz_F_F_J_J_Q_Q __pyx_string_tab[548]
#define __pyx_kp_b_iso88591_T_t_6d_K4Offjj_B_B_____o_o_s_s __pyx_string_tab[549]
#define __pyx_kp_b_iso88591_XQ_q_M_336H __pyx_string_tab[550]
#define __pyx_kp_b_iso88591__12 __pyx_string_tab[551]
#define __pyx_kp_b_iso88591__13 __pyx_string_tab[552]
#define __pyx_kp_b_iso88591__14 __pyx_string_tab[553]
#define __pyx_kp_b_iso88591__15 __pyx_string_tab[554]
#define __pyx_kp_b_iso88591_a __pyx_string_tab[555]
#define __pyx_kp_b_iso88591_a_4_A_t_F_d_1_q_t1_4q_T_AQ_z_Q __pyx_string_tab[556]
#define __pyx_kp_b_iso88591_a_t1_q_q_q_a_6_6_q_a_QfA __pyx_string_tab[557]
#define __pyx_kp_b_iso88591_avQ __pyx_string_tab[558]
#define __pyx_kp_b_iso88591_d_Q_4q_aq_t_1_O4vXQ_4D_Q_vQ_q __pyx_string_tab[559]
#define __pyx_kp_b_iso88591_gQ_D_U __pyx_string_tab[560]
#define __pyx_kp_b_iso88591_iq __pyx_string_tab[561]
#define __pyx_kp_b_iso88591_q_0_kQR_7_q0_a_1 __pyx_string_tab[562]
#define __pyx_kp_b_iso88591_q_0_kQR_81A_7_2_3FnTU_1 __pyx_string_tab[563]
#define __pyx_kp_b_iso88591_q_q_HA_T __pyx_string_tab[564]
#define __pyx_kp_b_iso88591_t3e1 __pyx_string_tab[565]
#define __pyx_kp_b_iso88591_t3e1_S_S_WAQ_83e3gQa_1 __pyx_string_tab[566]
#define __pyx_kp_b_void_PyObject_PyObject_PyObject __pyx_string_tab[567]
#define __pyx_float_0_1 __pyx_number_tab[0]
#define __pyx_float_5_0 __pyx_number_tab[1]
#define __pyx_float_10_0 __pyx_number_tab[2]
#define __pyx_float_30_0 __pyx_number_tab[3]
#define __pyx_float_60_0 __pyx_number_tab[4]
#define __pyx_int_0 __pyx_number_tab[5]
#define __pyx_int_1 __pyx_number_tab[6]
#define __pyx_int_2 __pyx_number_tab[7]
#define __pyx_int_3 __pyx_number_tab[8]
#define __pyx_int_4 __pyx_number_tab[9]
#define __pyx_int_5 __pyx_number_tab[10]
#define __pyx_int_8 __pyx_number_tab[11]
#define __pyx_int_14 __pyx_number_tab[12]
#define __pyx_int_16 __pyx_number_tab[13]
#define __pyx_int_131072 __pyx_number_tab[14]
#define __pyx_int_2097152 __pyx_number_tab[15]
#define __pyx_int_180946995 __pyx_number_tab[16]
#define __pyx_int_194108219 __pyx_number_tab[17]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection_ConnectionParams);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection_APIConnection);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection_APIConnection);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response);
  Py_CLEAR(clear_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect);
  Py_CLEAR(clear_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect);
  for (int i=0; i<3; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<45; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<568; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<18; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* Coroutine.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CoroutineType);
Py_CLEAR(clear_module_state->__pyx_CoroutineAwaitType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* Generator.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_GeneratorType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection_ConnectionParams);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection_APIConnection);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection_APIConnection);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response);
  Py_VISIT(traverse_module_state->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect);
  Py_VISIT(traverse_module_state->__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect);
  for (int i=0; i<3; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<45; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<568; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<18; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* Coroutine.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CoroutineType);
Py_VISIT(traverse_module_state->__pyx_CoroutineAwaitType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* Generator.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_GeneratorType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */
static PyObject *__pyx_gb_13aioesphomeapi_10connection_12generator10(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":68
 * MESSAGE_NUMBER_TO_PROTO: tuple[
 *     tuple[Callable[[], message.Message], Callable[[message.Message, bytes], None]], ...
 * ] = tuple((msg, msg.MergeFromString) for msg in MESSAGE_TYPE_TO_PROTO.values())             # <<<<<<<<<<<<<<
 * 
 * 
*/

static PyObject *__pyx_pf_13aioesphomeapi_10connection_10genexpr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_genexpr_arg_0) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct__genexpr(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 68, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_genexpr_arg_0 = __pyx_genexpr_arg_0;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_genexpr_arg_0);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_genexpr_arg_0);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_12generator10, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_genexpr, __pyx_mstate_global->__pyx_n_u_genexpr, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_12generator10(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started generator");
    __PYX_ERR(0, 68, __pyx_L1_error)
  }
  __pyx_t_2 = 0;
  if (unlikely(!__pyx_cur_scope->__pyx_genexpr_arg_0)) { __Pyx_RaiseUnboundLocalError(".0"); __PYX_ERR(0, 68, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_genexpr_arg_0 == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "values");
    __PYX_ERR(0, 68, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_dict_iterator(__pyx_cur_scope->__pyx_genexpr_arg_0, 1, __pyx_mstate_global->__pyx_n_u_values, (&__pyx_t_3), (&__pyx_t_4)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_1);
  __pyx_t_1 = __pyx_t_5;
  __pyx_t_5 = 0;
  while (1) {
    __pyx_t_6 = __Pyx_dict_iter_next(__pyx_t_1, __pyx_t_3, &__pyx_t_2, NULL, &__pyx_t_5, NULL, __pyx_t_4);
    if (unlikely(__pyx_t_6 == 0)) break;
    if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_msg);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_msg, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_msg, __pyx_mstate_global->__pyx_n_u_MergeFromString); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 68, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_msg);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_msg);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_cur_scope->__pyx_v_msg) != (0)) __PYX_ERR(0, 68, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_5) != (0)) __PYX_ERR(0, 68, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    __Pyx_XGIVEREF(__pyx_t_1);
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_2;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_3 = __pyx_t_4;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L6_resume_from_yield:;
    __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
    __pyx_cur_scope->__pyx_t_0 = 0;
    __Pyx_XGOTREF(__pyx_t_1);
    __pyx_t_2 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_2;
    __pyx_t_4 = __pyx_cur_scope->__pyx_t_3;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 68, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __init__(self, addresses, port, password, client_info, keepalive, zeroconf_manager, noise_psk, expected_name, expected_mac, timezone):             # <<<<<<<<<<<<<<
 *     self.addresses = addresses
 *     self.port = port
*/

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_addresses = 0;
  PyObject *__pyx_v_port = 0;
  PyObject *__pyx_v_password = 0;
  PyObject *__pyx_v_client_info = 0;
  PyObject *__pyx_v_keepalive = 0;
  PyObject *__pyx_v_zeroconf_manager = 0;
  PyObject *__pyx_v_noise_psk = 0;
  PyObject *__pyx_v_expected_name = 0;
  PyObject *__pyx_v_expected_mac = 0;
  PyObject *__pyx_v_timezone = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_addresses,&__pyx_mstate_global->__pyx_n_u_port,&__pyx_mstate_global->__pyx_n_u_password,&__pyx_mstate_global->__pyx_n_u_client_info,&__pyx_mstate_global->__pyx_n_u_keepalive,&__pyx_mstate_global->__pyx_n_u_zeroconf_manager,&__pyx_mstate_global->__pyx_n_u_noise_psk,&__pyx_mstate_global->__pyx_n_u_expected_name,&__pyx_mstate_global->__pyx_n_u_expected_mac,&__pyx_mstate_global->__pyx_n_u_timezone,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 1, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 10:
        values[9] = __Pyx_ArgRef_VARARGS(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(1, 1, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 10; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 10, 10, i); __PYX_ERR(1, 1, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 10)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(1, 1, __pyx_L3_error)
      values[9] = __Pyx_ArgRef_VARARGS(__pyx_args, 9);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(1, 1, __pyx_L3_error)
    }
    __pyx_v_addresses = values[0];
    __pyx_v_port = values[1];
    __pyx_v_password = values[2];
    __pyx_v_client_info = values[3];
    __pyx_v_keepalive = values[4];
    __pyx_v_zeroconf_manager = values[5];
    __pyx_v_noise_psk = values[6];
    __pyx_v_expected_name = values[7];
    __pyx_v_expected_mac = values[8];
    __pyx_v_timezone = values[9];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 10, 10, __pyx_nargs); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams___init__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), __pyx_v_addresses, __pyx_v_port, __pyx_v_password, __pyx_v_client_info, __pyx_v_keepalive, __pyx_v_zeroconf_manager, __pyx_v_noise_psk, __pyx_v_expected_name, __pyx_v_expected_mac, __pyx_v_timezone);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams___init__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_addresses, PyObject *__pyx_v_port, PyObject *__pyx_v_password, PyObject *__pyx_v_client_info, PyObject *__pyx_v_keepalive, PyObject *__pyx_v_zeroconf_manager, PyObject *__pyx_v_noise_psk, PyObject *__pyx_v_expected_name, PyObject *__pyx_v_expected_mac, PyObject *__pyx_v_timezone) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "(tree fragment)":2
 * def __init__(self, addresses, port, password, client_info, keepalive, zeroconf_manager, noise_psk, expected_name, expected_mac, timezone):
 *     self.addresses = addresses             # <<<<<<<<<<<<<<
 *     self.port = port
 *     self.password = password
*/
  __pyx_t_1 = __pyx_v_addresses;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->addresses);
  __Pyx_DECREF(__pyx_v_self->addresses);
  __pyx_v_self->addresses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":3
 * def __init__(self, addresses, port, password, client_info, keepalive, zeroconf_manager, noise_psk, expected_name, expected_mac, timezone):
 *     self.addresses = addresses
 *     self.port = port             # <<<<<<<<<<<<<<
 *     self.password = password
 *     self.client_info = client_info
*/
  __Pyx_INCREF(__pyx_v_port);
  __Pyx_GIVEREF(__pyx_v_port);
  __Pyx_GOTREF(__pyx_v_self->port);
  __Pyx_DECREF(__pyx_v_self->port);
  __pyx_v_self->port = __pyx_v_port;

  /* "(tree fragment)":4
 *     self.addresses = addresses
 *     self.port = port
 *     self.password = password             # <<<<<<<<<<<<<<
 *     self.client_info = client_info
 *     self.keepalive = keepalive
*/
  __Pyx_INCREF(__pyx_v_password);
  __Pyx_GIVEREF(__pyx_v_password);
  __Pyx_GOTREF(__pyx_v_self->password);
  __Pyx_DECREF(__pyx_v_self->password);
  __pyx_v_self->password = __pyx_v_password;

  /* "(tree fragment)":5
 *     self.port = port
 *     self.password = password
 *     self.client_info = client_info             # <<<<<<<<<<<<<<
 *     self.keepalive = keepalive
 *     self.zeroconf_manager = zeroconf_manager
*/
  __Pyx_INCREF(__pyx_v_client_info);
  __Pyx_GIVEREF(__pyx_v_client_info);
  __Pyx_GOTREF(__pyx_v_self->client_info);
  __Pyx_DECREF(__pyx_v_self->client_info);
  __pyx_v_self->client_info = __pyx_v_client_info;

  /* "(tree fragment)":6
 *     self.password = password
 *     self.client_info = client_info
 *     self.keepalive = keepalive             # <<<<<<<<<<<<<<
 *     self.zeroconf_manager = zeroconf_manager
 *     self.noise_psk = noise_psk
*/
  __Pyx_INCREF(__pyx_v_keepalive);
  __Pyx_GIVEREF(__pyx_v_keepalive);
  __Pyx_GOTREF(__pyx_v_self->keepalive);
  __Pyx_DECREF(__pyx_v_self->keepalive);
  __pyx_v_self->keepalive = __pyx_v_keepalive;

  /* "(tree fragment)":7
 *     self.client_info = client_info
 *     self.keepalive = keepalive
 *     self.zeroconf_manager = zeroconf_manager             # <<<<<<<<<<<<<<
 *     self.noise_psk = noise_psk
 *     self.expected_name = expected_name
*/
  __Pyx_INCREF(__pyx_v_zeroconf_manager);
  __Pyx_GIVEREF(__pyx_v_zeroconf_manager);
  __Pyx_GOTREF(__pyx_v_self->zeroconf_manager);
  __Pyx_DECREF(__pyx_v_self->zeroconf_manager);
  __pyx_v_self->zeroconf_manager = __pyx_v_zeroconf_manager;

  /* "(tree fragment)":8
 *     self.keepalive = keepalive
 *     self.zeroconf_manager = zeroconf_manager
 *     self.noise_psk = noise_psk             # <<<<<<<<<<<<<<
 *     self.expected_name = expected_name
 *     self.expected_mac = expected_mac
*/
  __Pyx_INCREF(__pyx_v_noise_psk);
  __Pyx_GIVEREF(__pyx_v_noise_psk);
  __Pyx_GOTREF(__pyx_v_self->noise_psk);
  __Pyx_DECREF(__pyx_v_self->noise_psk);
  __pyx_v_self->noise_psk = __pyx_v_noise_psk;

  /* "(tree fragment)":9
 *     self.zeroconf_manager = zeroconf_manager
 *     self.noise_psk = noise_psk
 *     self.expected_name = expected_name             # <<<<<<<<<<<<<<
 *     self.expected_mac = expected_mac
 *     self.timezone = timezone
*/
  __Pyx_INCREF(__pyx_v_expected_name);
  __Pyx_GIVEREF(__pyx_v_expected_name);
  __Pyx_GOTREF(__pyx_v_self->expected_name);
  __Pyx_DECREF(__pyx_v_self->expected_name);
  __pyx_v_self->expected_name = __pyx_v_expected_name;

  /* "(tree fragment)":10
 *     self.noise_psk = noise_psk
 *     self.expected_name = expected_name
 *     self.expected_mac = expected_mac             # <<<<<<<<<<<<<<
 *     self.timezone = timezone
 * __match_args__ = ('addresses', 'port', 'password', 'client_info', 'keepalive', 'zeroconf_manager', 'noise_psk', 'expected_name', 'expected_mac', 'timezone')
*/
  __Pyx_INCREF(__pyx_v_expected_mac);
  __Pyx_GIVEREF(__pyx_v_expected_mac);
  __Pyx_GOTREF(__pyx_v_self->expected_mac);
  __Pyx_DECREF(__pyx_v_self->expected_mac);
  __pyx_v_self->expected_mac = __pyx_v_expected_mac;

  /* "(tree fragment)":11
 *     self.expected_name = expected_name
 *     self.expected_mac = expected_mac
 *     self.timezone = timezone             # <<<<<<<<<<<<<<
 * __match_args__ = ('addresses', 'port', 'password', 'client_info', 'keepalive', 'zeroconf_manager', 'noise_psk', 'expected_name', 'expected_mac', 'timezone')
 * __pyx_recursive_repr_guard = __import__('threading').local()
*/
  __Pyx_INCREF(__pyx_v_timezone);
  __Pyx_GIVEREF(__pyx_v_timezone);
  __Pyx_GOTREF(__pyx_v_self->timezone);
  __Pyx_DECREF(__pyx_v_self->timezone);
  __pyx_v_self->timezone = __pyx_v_timezone;

  /* "(tree fragment)":1
 * def __init__(self, addresses, port, password, client_info, keepalive, zeroconf_manager, noise_psk, expected_name, expected_mac, timezone):             # <<<<<<<<<<<<<<
 *     self.addresses = addresses
 *     self.port = port
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":15
 * __pyx_recursive_repr_guard = __import__('threading').local()
 * __pyx_recursive_repr_guard.running = set()
 * def __repr__(self):             # <<<<<<<<<<<<<<
 *     key = id(self)
 *     guard_set = self.__pyx_recursive_repr_guard.running
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_2__repr__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_2__repr__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_guard_set = NULL;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14[22];
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_t_17;
  char const *__pyx_t_18;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "(tree fragment)":16
 * __pyx_recursive_repr_guard.running = set()
 * def __repr__(self):
 *     key = id(self)             # <<<<<<<<<<<<<<
 *     guard_set = self.__pyx_recursive_repr_guard.running
 *     if key in guard_set: return '...'
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, ((PyObject *)__pyx_v_self)};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 16, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_key = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":17
 * def __repr__(self):
 *     key = id(self)
 *     guard_set = self.__pyx_recursive_repr_guard.running             # <<<<<<<<<<<<<<
 *     if key in guard_set: return '...'
 *     guard_set.add(key)
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_pyx_recursive_repr_guard); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_running); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_guard_set = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":18
 *     key = id(self)
 *     guard_set = self.__pyx_recursive_repr_guard.running
 *     if key in guard_set: return '...'             # <<<<<<<<<<<<<<
 *     guard_set.add(key)
 *     try:
*/
  __pyx_t_4 = (__Pyx_PySequence_ContainsTF(__pyx_v_key, __pyx_v_guard_set, Py_EQ)); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(1, 18, __pyx_L1_error)
  if (__pyx_t_4) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __pyx_r = __pyx_mstate_global->__pyx_kp_u_;
    goto __pyx_L0;
  }

  /* "(tree fragment)":19
 *     guard_set = self.__pyx_recursive_repr_guard.running
 *     if key in guard_set: return '...'
 *     guard_set.add(key)             # <<<<<<<<<<<<<<
 *     try:
 *         name = getattr(type(self), "__qualname__", None) or type(self).__name__
*/
  __pyx_t_1 = __pyx_v_guard_set;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_key};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_add, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 19, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":20
 *     if key in guard_set: return '...'
 *     guard_set.add(key)
 *     try:             # <<<<<<<<<<<<<<
 *         name = getattr(type(self), "__qualname__", None) or type(self).__name__
 *         return f'{name}(addresses={self.addresses!r}, port={self.port!r}, password={self.password!r}, client_info={self.client_info!r}, keepalive={self.keepalive!r}, zeroconf_manager={self.zeroconf_manager!r}, noise_psk={self.noise_psk!r}, expected_name={self.expected_name!r}, expected_mac={self.expected_mac!r}, timezone={self.timezone!r})'
*/
  /*try:*/ {

    /* "(tree fragment)":21
 *     guard_set.add(key)
 *     try:
 *         name = getattr(type(self), "__qualname__", None) or type(self).__name__             # <<<<<<<<<<<<<<
 *         return f'{name}(addresses={self.addresses!r}, port={self.port!r}, password={self.password!r}, client_info={self.client_info!r}, keepalive={self.keepalive!r}, zeroconf_manager={self.zeroconf_manager!r}, noise_psk={self.noise_psk!r}, expected_name={self.expected_name!r}, expected_mac={self.expected_mac!r}, timezone={self.timezone!r})'
 *     finally:
*/
    __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))), __pyx_mstate_global->__pyx_n_u_qualname, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 21, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(1, 21, __pyx_L5_error)
    if (!__pyx_t_4) {
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_2 = __pyx_t_1;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 21, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_L7_bool_binop_done:;
    __pyx_v_name = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "(tree fragment)":22
 *     try:
 *         name = getattr(type(self), "__qualname__", None) or type(self).__name__
 *         return f'{name}(addresses={self.addresses!r}, port={self.port!r}, password={self.password!r}, client_info={self.client_info!r}, keepalive={self.keepalive!r}, zeroconf_manager={self.zeroconf_manager!r}, noise_psk={self.noise_psk!r}, expected_name={self.expected_name!r}, expected_mac={self.expected_mac!r}, timezone={self.timezone!r})'             # <<<<<<<<<<<<<<
 *     finally:
 *         guard_set.remove(key)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_v_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->addresses), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->port), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->password), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->client_info), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->keepalive), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->zeroconf_manager), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_9);
    __pyx_t_10 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->noise_psk), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_10)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->expected_name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_12 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->expected_mac), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_12)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_12);
    __pyx_t_13 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->timezone), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_13);
    __pyx_t_14[0] = __pyx_t_2;
    __pyx_t_14[1] = __pyx_mstate_global->__pyx_kp_u_addresses_2;
    __pyx_t_14[2] = __pyx_t_1;
    __pyx_t_14[3] = __pyx_mstate_global->__pyx_kp_u_port_2;
    __pyx_t_14[4] = __pyx_t_5;
    __pyx_t_14[5] = __pyx_mstate_global->__pyx_kp_u_password_2;
    __pyx_t_14[6] = __pyx_t_6;
    __pyx_t_14[7] = __pyx_mstate_global->__pyx_kp_u_client_info_2;
    __pyx_t_14[8] = __pyx_t_7;
    __pyx_t_14[9] = __pyx_mstate_global->__pyx_kp_u_keepalive_2;
    __pyx_t_14[10] = __pyx_t_8;
    __pyx_t_14[11] = __pyx_mstate_global->__pyx_kp_u_zeroconf_manager_2;
    __pyx_t_14[12] = __pyx_t_9;
    __pyx_t_14[13] = __pyx_mstate_global->__pyx_kp_u_noise_psk_2;
    __pyx_t_14[14] = __pyx_t_10;
    __pyx_t_14[15] = __pyx_mstate_global->__pyx_kp_u_expected_name_2;
    __pyx_t_14[16] = __pyx_t_11;
    __pyx_t_14[17] = __pyx_mstate_global->__pyx_kp_u_expected_mac_2;
    __pyx_t_14[18] = __pyx_t_12;
    __pyx_t_14[19] = __pyx_mstate_global->__pyx_kp_u_timezone_2;
    __pyx_t_14[20] = __pyx_t_13;
    __pyx_t_14[21] = __pyx_mstate_global->__pyx_kp_u__2;
    __pyx_t_15 = __Pyx_PyUnicode_Join(__pyx_t_14, 22, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 11 * 3 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6) + 14 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_7) + 12 * 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 19 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_9) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_10) + 16 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_11) + 15 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_12) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_7) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_9) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_10) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_11) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_12) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13));
    if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 22, __pyx_L5_error)
    __Pyx_GOTREF(__pyx_t_15);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_r = __pyx_t_15;
    __pyx_t_15 = 0;
    goto __pyx_L4_return;
  }

  /* "(tree fragment)":24
 *         return f'{name}(addresses={self.addresses!r}, port={self.port!r}, password={self.password!r}, client_info={self.client_info!r}, keepalive={self.keepalive!r}, zeroconf_manager={self.zeroconf_manager!r}, noise_psk={self.noise_psk!r}, expected_name={self.expected_name!r}, expected_mac={self.expected_mac!r}, timezone={self.timezone!r})'
 *     finally:
 *         guard_set.remove(key)             # <<<<<<<<<<<<<<
 * def __eq__(self, other):
 *     if other.__class__ is not self.__class__: return NotImplemented
*/
  /*finally:*/ {
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_23, &__pyx_t_24);
      if ( unlikely(__Pyx_GetException(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21) < 0)) __Pyx_ErrFetch(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_22);
      __Pyx_XGOTREF(__pyx_t_23);
      __Pyx_XGOTREF(__pyx_t_24);
      __pyx_t_16 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_18 = __pyx_filename;
      {
        __pyx_t_13 = __pyx_v_guard_set;
        __Pyx_INCREF(__pyx_t_13);
        __pyx_t_3 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_v_key};
          __pyx_t_15 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_remove, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 24, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_15);
        }
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      }
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_ErrRestore(__pyx_t_19, __pyx_t_20, __pyx_t_21);
      __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
      __pyx_lineno = __pyx_t_16; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_18;
      goto __pyx_L1_error;
      __pyx_L10_error:;
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_23, __pyx_t_24);
      __Pyx_XDECREF(__pyx_t_19); __pyx_t_19 = 0;
      __Pyx_XDECREF(__pyx_t_20); __pyx_t_20 = 0;
      __Pyx_XDECREF(__pyx_t_21); __pyx_t_21 = 0;
      __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L4_return: {
      __pyx_t_24 = __pyx_r;
      __pyx_r = 0;
      __pyx_t_13 = __pyx_v_guard_set;
      __Pyx_INCREF(__pyx_t_13);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_v_key};
        __pyx_t_15 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_remove, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_15)) __PYX_ERR(1, 24, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_15);
      }
      __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
      __pyx_r = __pyx_t_24;
      __pyx_t_24 = 0;
      goto __pyx_L0;
    }
  }

  /* "(tree fragment)":15
 * __pyx_recursive_repr_guard = __import__('threading').local()
 * __pyx_recursive_repr_guard.running = set()
 * def __repr__(self):             # <<<<<<<<<<<<<<
 *     key = id(self)
 *     guard_set = self.__pyx_recursive_repr_guard.running
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_guard_set);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":25
 *     finally:
 *         guard_set.remove(key)
 * def __eq__(self, other):             # <<<<<<<<<<<<<<
 *     if other.__class__ is not self.__class__: return NotImplemented
 *     cdef ConnectionParams other_cast
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_5__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_5__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4__eq__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4__eq__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_other) {
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_other_cast = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "(tree fragment)":26
 *         guard_set.remove(key)
 * def __eq__(self, other):
 *     if other.__class__ is not self.__class__: return NotImplemented             # <<<<<<<<<<<<<<
 *     cdef ConnectionParams other_cast
 *     other_cast = <ConnectionParams>other
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = (__pyx_t_1 != __pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_builtin_NotImplemented);
    __pyx_r = __pyx_builtin_NotImplemented;
    goto __pyx_L0;
  }

  /* "(tree fragment)":28
 *     if other.__class__ is not self.__class__: return NotImplemented
 *     cdef ConnectionParams other_cast
 *     other_cast = <ConnectionParams>other             # <<<<<<<<<<<<<<
 *     if self.addresses != other_cast.addresses: return False
 *     if self.port != other_cast.port: return False
*/
  __pyx_t_2 = __pyx_v_other;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_other_cast = ((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":29
 *     cdef ConnectionParams other_cast
 *     other_cast = <ConnectionParams>other
 *     if self.addresses != other_cast.addresses: return False             # <<<<<<<<<<<<<<
 *     if self.port != other_cast.port: return False
 *     if self.password != other_cast.password: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->addresses, __pyx_v_other_cast->addresses, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 29, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":30
 *     other_cast = <ConnectionParams>other
 *     if self.addresses != other_cast.addresses: return False
 *     if self.port != other_cast.port: return False             # <<<<<<<<<<<<<<
 *     if self.password != other_cast.password: return False
 *     if self.client_info != other_cast.client_info: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->port, __pyx_v_other_cast->port, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 30, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":31
 *     if self.addresses != other_cast.addresses: return False
 *     if self.port != other_cast.port: return False
 *     if self.password != other_cast.password: return False             # <<<<<<<<<<<<<<
 *     if self.client_info != other_cast.client_info: return False
 *     if self.keepalive != other_cast.keepalive: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->password, __pyx_v_other_cast->password, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 31, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":32
 *     if self.port != other_cast.port: return False
 *     if self.password != other_cast.password: return False
 *     if self.client_info != other_cast.client_info: return False             # <<<<<<<<<<<<<<
 *     if self.keepalive != other_cast.keepalive: return False
 *     if self.zeroconf_manager != other_cast.zeroconf_manager: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->client_info, __pyx_v_other_cast->client_info, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 32, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":33
 *     if self.password != other_cast.password: return False
 *     if self.client_info != other_cast.client_info: return False
 *     if self.keepalive != other_cast.keepalive: return False             # <<<<<<<<<<<<<<
 *     if self.zeroconf_manager != other_cast.zeroconf_manager: return False
 *     if self.noise_psk != other_cast.noise_psk: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->keepalive, __pyx_v_other_cast->keepalive, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 33, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":34
 *     if self.client_info != other_cast.client_info: return False
 *     if self.keepalive != other_cast.keepalive: return False
 *     if self.zeroconf_manager != other_cast.zeroconf_manager: return False             # <<<<<<<<<<<<<<
 *     if self.noise_psk != other_cast.noise_psk: return False
 *     if self.expected_name != other_cast.expected_name: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->zeroconf_manager, __pyx_v_other_cast->zeroconf_manager, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 34, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 34, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":35
 *     if self.keepalive != other_cast.keepalive: return False
 *     if self.zeroconf_manager != other_cast.zeroconf_manager: return False
 *     if self.noise_psk != other_cast.noise_psk: return False             # <<<<<<<<<<<<<<
 *     if self.expected_name != other_cast.expected_name: return False
 *     if self.expected_mac != other_cast.expected_mac: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->noise_psk, __pyx_v_other_cast->noise_psk, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 35, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":36
 *     if self.zeroconf_manager != other_cast.zeroconf_manager: return False
 *     if self.noise_psk != other_cast.noise_psk: return False
 *     if self.expected_name != other_cast.expected_name: return False             # <<<<<<<<<<<<<<
 *     if self.expected_mac != other_cast.expected_mac: return False
 *     if self.timezone != other_cast.timezone: return False
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->expected_name, __pyx_v_other_cast->expected_name, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 36, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 36, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":37
 *     if self.noise_psk != other_cast.noise_psk: return False
 *     if self.expected_name != other_cast.expected_name: return False
 *     if self.expected_mac != other_cast.expected_mac: return False             # <<<<<<<<<<<<<<
 *     if self.timezone != other_cast.timezone: return False
 *     return True
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->expected_mac, __pyx_v_other_cast->expected_mac, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 37, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 37, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":38
 *     if self.expected_name != other_cast.expected_name: return False
 *     if self.expected_mac != other_cast.expected_mac: return False
 *     if self.timezone != other_cast.timezone: return False             # <<<<<<<<<<<<<<
 *     return True
*/
  __pyx_t_2 = PyObject_RichCompare(__pyx_v_self->timezone, __pyx_v_other_cast->timezone, Py_NE); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 38, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 38, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_3) {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;
  }

  /* "(tree fragment)":39
 *     if self.expected_mac != other_cast.expected_mac: return False
 *     if self.timezone != other_cast.timezone: return False
 *     return True             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_True);
  __pyx_r = Py_True;
  goto __pyx_L0;

  /* "(tree fragment)":25
 *     finally:
 *         guard_set.remove(key)
 * def __eq__(self, other):             # <<<<<<<<<<<<<<
 *     if other.__class__ is not self.__class__: return NotImplemented
 *     cdef ConnectionParams other_cast
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_other_cast);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":81
 * cdef class ConnectionParams:
 * 
 *     cdef public list addresses             # <<<<<<<<<<<<<<
 *     cdef public object port
 *     cdef public object password
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->addresses);
  __pyx_r = __pyx_v_self->addresses;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(2, 81, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->addresses);
  __Pyx_DECREF(__pyx_v_self->addresses);
  __pyx_v_self->addresses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.addresses.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9addresses_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->addresses);
  __Pyx_DECREF(__pyx_v_self->addresses);
  __pyx_v_self->addresses = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":82
 * 
 *     cdef public list addresses
 *     cdef public object port             # <<<<<<<<<<<<<<
 *     cdef public object password
 *     cdef public object client_info
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->port);
  __pyx_r = __pyx_v_self->port;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->port);
  __Pyx_DECREF(__pyx_v_self->port);
  __pyx_v_self->port = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_4port_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->port);
  __Pyx_DECREF(__pyx_v_self->port);
  __pyx_v_self->port = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":83
 *     cdef public list addresses
 *     cdef public object port
 *     cdef public object password             # <<<<<<<<<<<<<<
 *     cdef public object client_info
 *     cdef public object keepalive
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->password);
  __pyx_r = __pyx_v_self->password;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->password);
  __Pyx_DECREF(__pyx_v_self->password);
  __pyx_v_self->password = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8password_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->password);
  __Pyx_DECREF(__pyx_v_self->password);
  __pyx_v_self->password = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":84
 *     cdef public object port
 *     cdef public object password
 *     cdef public object client_info             # <<<<<<<<<<<<<<
 *     cdef public object keepalive
 *     cdef public object zeroconf_manager
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->client_info);
  __pyx_r = __pyx_v_self->client_info;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->client_info);
  __Pyx_DECREF(__pyx_v_self->client_info);
  __pyx_v_self->client_info = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_11client_info_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->client_info);
  __Pyx_DECREF(__pyx_v_self->client_info);
  __pyx_v_self->client_info = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":85
 *     cdef public object password
 *     cdef public object client_info
 *     cdef public object keepalive             # <<<<<<<<<<<<<<
 *     cdef public object zeroconf_manager
 *     cdef public object noise_psk
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->keepalive);
  __pyx_r = __pyx_v_self->keepalive;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->keepalive);
  __Pyx_DECREF(__pyx_v_self->keepalive);
  __pyx_v_self->keepalive = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->keepalive);
  __Pyx_DECREF(__pyx_v_self->keepalive);
  __pyx_v_self->keepalive = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":86
 *     cdef public object client_info
 *     cdef public object keepalive
 *     cdef public object zeroconf_manager             # <<<<<<<<<<<<<<
 *     cdef public object noise_psk
 *     cdef public object expected_name
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->zeroconf_manager);
  __pyx_r = __pyx_v_self->zeroconf_manager;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->zeroconf_manager);
  __Pyx_DECREF(__pyx_v_self->zeroconf_manager);
  __pyx_v_self->zeroconf_manager = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->zeroconf_manager);
  __Pyx_DECREF(__pyx_v_self->zeroconf_manager);
  __pyx_v_self->zeroconf_manager = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":87
 *     cdef public object keepalive
 *     cdef public object zeroconf_manager
 *     cdef public object noise_psk             # <<<<<<<<<<<<<<
 *     cdef public object expected_name
 *     cdef public object expected_mac
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->noise_psk);
  __pyx_r = __pyx_v_self->noise_psk;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->noise_psk);
  __Pyx_DECREF(__pyx_v_self->noise_psk);
  __pyx_v_self->noise_psk = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->noise_psk);
  __Pyx_DECREF(__pyx_v_self->noise_psk);
  __pyx_v_self->noise_psk = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":88
 *     cdef public object zeroconf_manager
 *     cdef public object noise_psk
 *     cdef public object expected_name             # <<<<<<<<<<<<<<
 *     cdef public object expected_mac
 *     cdef public object timezone
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->expected_name);
  __pyx_r = __pyx_v_self->expected_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->expected_name);
  __Pyx_DECREF(__pyx_v_self->expected_name);
  __pyx_v_self->expected_name = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->expected_name);
  __Pyx_DECREF(__pyx_v_self->expected_name);
  __pyx_v_self->expected_name = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":89
 *     cdef public object noise_psk
 *     cdef public object expected_name
 *     cdef public object expected_mac             # <<<<<<<<<<<<<<
 *     cdef public object timezone
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->expected_mac);
  __pyx_r = __pyx_v_self->expected_mac;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->expected_mac);
  __Pyx_DECREF(__pyx_v_self->expected_mac);
  __pyx_v_self->expected_mac = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->expected_mac);
  __Pyx_DECREF(__pyx_v_self->expected_mac);
  __pyx_v_self->expected_mac = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":90
 *     cdef public object expected_name
 *     cdef public object expected_mac
 *     cdef public object timezone             # <<<<<<<<<<<<<<
 * 
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone___get__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone___get__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->timezone);
  __pyx_r = __pyx_v_self->timezone;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->timezone);
  __Pyx_DECREF(__pyx_v_self->timezone);
  __pyx_v_self->timezone = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8timezone_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->timezone);
  __Pyx_DECREF(__pyx_v_self->timezone);
  __pyx_v_self->timezone = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_16ConnectionParams_7__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_7__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_7__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_6__reduce_cython__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_6__reduce_cython__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.addresses, self.client_info, self.expected_mac, self.expected_name, self.keepalive, self.noise_psk, self.password, self.port, self.timezone, self.zeroconf_manager)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(10); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->addresses);
  __Pyx_GIVEREF(__pyx_v_self->addresses);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_self->addresses) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->client_info);
  __Pyx_GIVEREF(__pyx_v_self->client_info);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->client_info) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->expected_mac);
  __Pyx_GIVEREF(__pyx_v_self->expected_mac);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_self->expected_mac) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->expected_name);
  __Pyx_GIVEREF(__pyx_v_self->expected_name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 3, __pyx_v_self->expected_name) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->keepalive);
  __Pyx_GIVEREF(__pyx_v_self->keepalive);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 4, __pyx_v_self->keepalive) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->noise_psk);
  __Pyx_GIVEREF(__pyx_v_self->noise_psk);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 5, __pyx_v_self->noise_psk) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->password);
  __Pyx_GIVEREF(__pyx_v_self->password);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 6, __pyx_v_self->password) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->port);
  __Pyx_GIVEREF(__pyx_v_self->port);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 7, __pyx_v_self->port) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->timezone);
  __Pyx_GIVEREF(__pyx_v_self->timezone);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 8, __pyx_v_self->timezone) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->zeroconf_manager);
  __Pyx_GIVEREF(__pyx_v_self->zeroconf_manager);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 9, __pyx_v_self->zeroconf_manager) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.addresses, self.client_info, self.expected_mac, self.expected_name, self.keepalive, self.noise_psk, self.password, self.port, self.timezone, self.zeroconf_manager)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.addresses, self.client_info, self.expected_mac, self.expected_name, self.keepalive, self.noise_psk, self.password, self.port, self.timezone, self.zeroconf_manager)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.addresses is not None or self.client_info is not None or self.expected_mac is not None or self.expected_name is not None or self.keepalive is not None or self.noise_psk is not None or self.password is not None or self.port is not None or self.timezone is not None or self.zeroconf_manager is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.addresses, self.client_info, self.expected_mac, self.expected_name, self.keepalive, self.noise_psk, self.password, self.port, self.timezone, self.zeroconf_manager)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.addresses is not None or self.client_info is not None or self.expected_mac is not None or self.expected_name is not None or self.keepalive is not None or self.noise_psk is not None or self.password is not None or self.port is not None or self.timezone is not None or self.zeroconf_manager is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, None), state
*/
  /*else*/ {
    __pyx_t_3 = (__pyx_v_self->addresses != ((PyObject*)Py_None));
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->client_info != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->expected_mac != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->expected_name != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->keepalive != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->noise_psk != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->password != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->port != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->timezone != Py_None);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_2 = __pyx_t_3;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_3 = (__pyx_v_self->zeroconf_manager != Py_None);
    __pyx_t_2 = __pyx_t_3;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.addresses is not None or self.client_info is not None or self.expected_mac is not None or self.expected_name is not None or self.keepalive is not None or self.noise_psk is not None or self.password is not None or self.port is not None or self.timezone is not None or self.zeroconf_manager is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self.addresses is not None or self.client_info is not None or self.expected_mac is not None or self.expected_name is not None or self.keepalive is not None or self.noise_psk is not None or self.password is not None or self.port is not None or self.timezone is not None or self.zeroconf_manager is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ConnectionParams); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_194108219);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_194108219);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_194108219) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.addresses is not None or self.client_info is not None or self.expected_mac is not None or self.expected_name is not None or self.keepalive is not None or self.noise_psk is not None or self.password is not None or self.port is not None or self.timezone is not None or self.zeroconf_manager is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, None), state
 *     else:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ConnectionParams__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ConnectionParams); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_194108219);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_194108219);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_194108219) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ConnectionParams__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_16ConnectionParams_9__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8__setstate_cython__(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_16ConnectionParams_8__setstate_cython__(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ConnectionParams__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(1, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(1, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_13aioesphomeapi_10connection___pyx_unpickle_ConnectionParams__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ConnectionParams__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aioesphomeapi.connection.ConnectionParams.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":163
 * 
 * 
 * def _make_hello_request(client_info: str) -> HelloRequest:             # <<<<<<<<<<<<<<
 *     """Make a HelloRequest."""
 *     return HelloRequest(
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_1_make_hello_request(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection__make_hello_request, "Make a HelloRequest.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_1_make_hello_request = {"_make_hello_request", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_1_make_hello_request, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection__make_hello_request};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_1_make_hello_request(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_client_info = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_make_hello_request (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_client_info,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 163, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 163, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_make_hello_request", 0) < (0)) __PYX_ERR(0, 163, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_make_hello_request", 1, 1, 1, i); __PYX_ERR(0, 163, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 163, __pyx_L3_error)
    }
    __pyx_v_client_info = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_make_hello_request", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 163, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection._make_hello_request", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_client_info), (&PyUnicode_Type), 0, "client_info", 2))) __PYX_ERR(0, 163, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection__make_hello_request(__pyx_self, __pyx_v_client_info);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection__make_hello_request(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_client_info) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_make_hello_request", 0);

  /* "aioesphomeapi/connection.py":165
 * def _make_hello_request(client_info: str) -> HelloRequest:
 *     """Make a HelloRequest."""
 *     return HelloRequest(             # <<<<<<<<<<<<<<
 *         client_info=client_info, api_version_major=1, api_version_minor=14
 *     )
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_HelloRequest);
  __pyx_t_3 = __pyx_v_13aioesphomeapi_10connection_HelloRequest; 

  /* "aioesphomeapi/connection.py":166
 *     """Make a HelloRequest."""
 *     return HelloRequest(
 *         client_info=client_info, api_version_major=1, api_version_minor=14             # <<<<<<<<<<<<<<
 *     )
 * 
*/
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_client_info, __pyx_v_client_info, __pyx_t_5, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 165, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_api_version_major, __pyx_mstate_global->__pyx_int_1, __pyx_t_5, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 165, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_api_version_minor, __pyx_mstate_global->__pyx_int_14, __pyx_t_5, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 165, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":163
 * 
 * 
 * def _make_hello_request(client_info: str) -> HelloRequest:             # <<<<<<<<<<<<<<
 *     """Make a HelloRequest."""
 *     return HelloRequest(
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("aioesphomeapi.connection._make_hello_request", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":181
 * 
 * 
 * def _build_dst_rule_proto(rule: DSTRuleParsed) -> DSTRuleProto:             # <<<<<<<<<<<<<<
 *     """Build a protobuf DSTRule from a parsed DSTRule."""
 *     return DSTRuleProto(
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_3_build_dst_rule_proto(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_2_build_dst_rule_proto, "Build a protobuf DSTRule from a parsed DSTRule.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_3_build_dst_rule_proto = {"_build_dst_rule_proto", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_3_build_dst_rule_proto, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_2_build_dst_rule_proto};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_3_build_dst_rule_proto(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_rule = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_build_dst_rule_proto (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_rule,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 181, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 181, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_build_dst_rule_proto", 0) < (0)) __PYX_ERR(0, 181, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_build_dst_rule_proto", 1, 1, 1, i); __PYX_ERR(0, 181, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 181, __pyx_L3_error)
    }
    __pyx_v_rule = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_build_dst_rule_proto", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 181, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection._build_dst_rule_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_2_build_dst_rule_proto(__pyx_self, __pyx_v_rule);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_2_build_dst_rule_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_rule) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_build_dst_rule_proto", 0);

  /* "aioesphomeapi/connection.py":183
 * def _build_dst_rule_proto(rule: DSTRuleParsed) -> DSTRuleProto:
 *     """Build a protobuf DSTRule from a parsed DSTRule."""
 *     return DSTRuleProto(             # <<<<<<<<<<<<<<
 *         time_seconds=rule.time_seconds,
 *         day=rule.day,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_DSTRuleProto); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "aioesphomeapi/connection.py":184
 *     """Build a protobuf DSTRule from a parsed DSTRule."""
 *     return DSTRuleProto(
 *         time_seconds=rule.time_seconds,             # <<<<<<<<<<<<<<
 *         day=rule.day,
 *         type=_DST_RULE_TYPE_MAP[rule.type],
*/
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_mstate_global->__pyx_n_u_time_seconds); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 184, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);

  /* "aioesphomeapi/connection.py":185
 *     return DSTRuleProto(
 *         time_seconds=rule.time_seconds,
 *         day=rule.day,             # <<<<<<<<<<<<<<
 *         type=_DST_RULE_TYPE_MAP[rule.type],
 *         month=rule.month,
*/
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_mstate_global->__pyx_n_u_day); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);

  /* "aioesphomeapi/connection.py":186
 *         time_seconds=rule.time_seconds,
 *         day=rule.day,
 *         type=_DST_RULE_TYPE_MAP[rule.type],             # <<<<<<<<<<<<<<
 *         month=rule.month,
 *         week=rule.week,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_MAP); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetItem(__pyx_t_6, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 186, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":187
 *         day=rule.day,
 *         type=_DST_RULE_TYPE_MAP[rule.type],
 *         month=rule.month,             # <<<<<<<<<<<<<<
 *         week=rule.week,
 *         day_of_week=rule.day_of_week,
*/
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_mstate_global->__pyx_n_u_month); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 187, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "aioesphomeapi/connection.py":188
 *         type=_DST_RULE_TYPE_MAP[rule.type],
 *         month=rule.month,
 *         week=rule.week,             # <<<<<<<<<<<<<<
 *         day_of_week=rule.day_of_week,
 *     )
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_mstate_global->__pyx_n_u_week); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 188, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "aioesphomeapi/connection.py":189
 *         month=rule.month,
 *         week=rule.week,
 *         day_of_week=rule.day_of_week,             # <<<<<<<<<<<<<<
 *     )
 * 
*/
  __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_v_rule, __pyx_mstate_global->__pyx_n_u_day_of_week); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_t_10 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_10 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 6 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_11 = __Pyx_MakeVectorcallBuilderKwds(6); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_time_seconds, __pyx_t_4, __pyx_t_11, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_day, __pyx_t_5, __pyx_t_11, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_type, __pyx_t_8, __pyx_t_11, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_month, __pyx_t_7, __pyx_t_11, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_week, __pyx_t_6, __pyx_t_11, __pyx_callargs+1, 4) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_day_of_week, __pyx_t_9, __pyx_t_11, __pyx_callargs+1, 5) < (0)) __PYX_ERR(0, 183, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_10, (1-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_11);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":181
 * 
 * 
 * def _build_dst_rule_proto(rule: DSTRuleParsed) -> DSTRuleProto:             # <<<<<<<<<<<<<<
 *     """Build a protobuf DSTRule from a parsed DSTRule."""
 *     return DSTRuleProto(
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("aioesphomeapi.connection._build_dst_rule_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":193
 * 
 * 
 * @lru_cache(maxsize=8)             # <<<<<<<<<<<<<<
 * def _build_parsed_tz_proto(tz_string: str) -> ParsedTimezoneProto | None:
 *     """Build a protobuf ParsedTimezone from a POSIX TZ string.
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_5_build_parsed_tz_proto(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_4_build_parsed_tz_proto, "Build a protobuf ParsedTimezone from a POSIX TZ string.\n\n    Returns None if the string is empty or unparseable.\n    ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_5_build_parsed_tz_proto = {"_build_parsed_tz_proto", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_5_build_parsed_tz_proto, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_4_build_parsed_tz_proto};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_5_build_parsed_tz_proto(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_tz_string = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_build_parsed_tz_proto (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_tz_string,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 193, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 193, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_build_parsed_tz_proto", 0) < (0)) __PYX_ERR(0, 193, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_build_parsed_tz_proto", 1, 1, 1, i); __PYX_ERR(0, 193, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 193, __pyx_L3_error)
    }
    __pyx_v_tz_string = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_build_parsed_tz_proto", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 193, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection._build_parsed_tz_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_tz_string), (&PyUnicode_Type), 0, "tz_string", 2))) __PYX_ERR(0, 194, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_4_build_parsed_tz_proto(__pyx_self, __pyx_v_tz_string);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_4_build_parsed_tz_proto(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_tz_string) {
  PyObject *__pyx_v_parsed = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_build_parsed_tz_proto", 0);

  /* "aioesphomeapi/connection.py":199
 *     Returns None if the string is empty or unparseable.
 *     """
 *     if not tz_string:             # <<<<<<<<<<<<<<
 *         return None
 *     try:
*/
  {
    Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_v_tz_string);
    if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 199, __pyx_L1_error)
    __pyx_t_1 = (__pyx_temp != 0);
  }

  __pyx_t_2 = (!__pyx_t_1);
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":200
 *     """
 *     if not tz_string:
 *         return None             # <<<<<<<<<<<<<<
 *     try:
 *         parsed = parse_posix_tz(tz_string)
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "aioesphomeapi/connection.py":199
 *     Returns None if the string is empty or unparseable.
 *     """
 *     if not tz_string:             # <<<<<<<<<<<<<<
 *         return None
 *     try:
*/
  }

  /* "aioesphomeapi/connection.py":201
 *     if not tz_string:
 *         return None
 *     try:             # <<<<<<<<<<<<<<
 *         parsed = parse_posix_tz(tz_string)
 *     except ValueError:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "aioesphomeapi/connection.py":202
 *         return None
 *     try:
 *         parsed = parse_posix_tz(tz_string)             # <<<<<<<<<<<<<<
 *     except ValueError:
 *         return None
*/
      __pyx_t_7 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_parse_posix_tz); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 202, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_9 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_7);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_9 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_v_tz_string};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 202, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __pyx_v_parsed = __pyx_t_6;
      __pyx_t_6 = 0;

      /* "aioesphomeapi/connection.py":201
 *     if not tz_string:
 *         return None
 *     try:             # <<<<<<<<<<<<<<
 *         parsed = parse_posix_tz(tz_string)
 *     except ValueError:
*/
    }
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    goto __pyx_L9_try_end;
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "aioesphomeapi/connection.py":203
 *     try:
 *         parsed = parse_posix_tz(tz_string)
 *     except ValueError:             # <<<<<<<<<<<<<<
 *         return None
 *     return ParsedTimezoneProto(
*/
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_ValueError))));
    if (__pyx_t_10) {
      __Pyx_ErrRestore(0,0,0);

      /* "aioesphomeapi/connection.py":204
 *         parsed = parse_posix_tz(tz_string)
 *     except ValueError:
 *         return None             # <<<<<<<<<<<<<<
 *     return ParsedTimezoneProto(
 *         std_offset_seconds=parsed.std_offset_seconds,
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_r = Py_None; __Pyx_INCREF(Py_None);
      goto __pyx_L7_except_return;
    }
    goto __pyx_L6_except_error;

    /* "aioesphomeapi/connection.py":201
 *     if not tz_string:
 *         return None
 *     try:             # <<<<<<<<<<<<<<
 *         parsed = parse_posix_tz(tz_string)
 *     except ValueError:
*/
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L9_try_end:;
  }

  /* "aioesphomeapi/connection.py":205
 *     except ValueError:
 *         return None
 *     return ParsedTimezoneProto(             # <<<<<<<<<<<<<<
 *         std_offset_seconds=parsed.std_offset_seconds,
 *         dst_offset_seconds=parsed.dst_offset_seconds,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ParsedTimezoneProto); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "aioesphomeapi/connection.py":206
 *         return None
 *     return ParsedTimezoneProto(
 *         std_offset_seconds=parsed.std_offset_seconds,             # <<<<<<<<<<<<<<
 *         dst_offset_seconds=parsed.dst_offset_seconds,
 *         dst_start=_build_dst_rule_proto(parsed.dst_start),
*/
  __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_mstate_global->__pyx_n_u_std_offset_seconds); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);

  /* "aioesphomeapi/connection.py":207
 *     return ParsedTimezoneProto(
 *         std_offset_seconds=parsed.std_offset_seconds,
 *         dst_offset_seconds=parsed.dst_offset_seconds,             # <<<<<<<<<<<<<<
 *         dst_start=_build_dst_rule_proto(parsed.dst_start),
 *         dst_end=_build_dst_rule_proto(parsed.dst_end),
*/
  __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_mstate_global->__pyx_n_u_dst_offset_seconds); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 207, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_12);

  /* "aioesphomeapi/connection.py":208
 *         std_offset_seconds=parsed.std_offset_seconds,
 *         dst_offset_seconds=parsed.dst_offset_seconds,
 *         dst_start=_build_dst_rule_proto(parsed.dst_start),             # <<<<<<<<<<<<<<
 *         dst_end=_build_dst_rule_proto(parsed.dst_end),
 *     )
*/
  __pyx_t_14 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_15, __pyx_mstate_global->__pyx_n_u_build_dst_rule_proto); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_15);
  __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_mstate_global->__pyx_n_u_dst_start); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 208, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_16);
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_15))) {
    __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_15);
    assert(__pyx_t_14);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_15);
    __Pyx_INCREF(__pyx_t_14);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_15, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_16};
    __pyx_t_13 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_15, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 208, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
  }

  /* "aioesphomeapi/connection.py":209
 *         dst_offset_seconds=parsed.dst_offset_seconds,
 *         dst_start=_build_dst_rule_proto(parsed.dst_start),
 *         dst_end=_build_dst_rule_proto(parsed.dst_end),             # <<<<<<<<<<<<<<
 *     )
 * 
*/
  __pyx_t_16 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_build_dst_rule_proto); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_17 = __Pyx_PyObject_GetAttrStr(__pyx_v_parsed, __pyx_mstate_global->__pyx_n_u_dst_end); if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 209, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_17);
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_14))) {
    __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
    assert(__pyx_t_16);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
    __Pyx_INCREF(__pyx_t_16);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_16, __pyx_t_17};
    __pyx_t_15 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 209, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_15);
  }
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_8);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_8);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 4 : 0)] = {__pyx_t_8, NULL};
    __pyx_t_14 = __Pyx_MakeVectorcallBuilderKwds(4); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_std_offset_seconds, __pyx_t_11, __pyx_t_14, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 205, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dst_offset_seconds, __pyx_t_12, __pyx_t_14, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 205, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dst_start, __pyx_t_13, __pyx_t_14, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 205, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_dst_end, __pyx_t_15, __pyx_t_14, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 205, __pyx_L1_error)
    __pyx_t_6 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_14);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 205, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
  }
  __pyx_r = __pyx_t_6;
  __pyx_t_6 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":193
 * 
 * 
 * @lru_cache(maxsize=8)             # <<<<<<<<<<<<<<
 * def _build_parsed_tz_proto(tz_string: str) -> ParsedTimezoneProto | None:
 *     """Build a protobuf ParsedTimezone from a POSIX TZ string.
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_17);
  __Pyx_AddTraceback("aioesphomeapi.connection._build_parsed_tz_proto", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parsed);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":213
 * 
 * 
 * def handle_timeout(fut: asyncio.Future[None]) -> None:             # <<<<<<<<<<<<<<
 *     """Handle a timeout."""
 *     if not fut.done():
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_7handle_timeout(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_handle_timeout(PyObject *__pyx_v_fut, CYTHON_UNUSED int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("handle_timeout", 0);

  /* "aioesphomeapi/connection.py":215
 * def handle_timeout(fut: asyncio.Future[None]) -> None:
 *     """Handle a timeout."""
 *     if not fut.done():             # <<<<<<<<<<<<<<
 *         fut.set_exception(TimeoutError)
 * 
*/
  __pyx_t_2 = __pyx_v_fut;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_done, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 215, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 215, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (!__pyx_t_4);
  if (__pyx_t_5) {

    /* "aioesphomeapi/connection.py":216
 *     """Handle a timeout."""
 *     if not fut.done():
 *         fut.set_exception(TimeoutError)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    __pyx_t_2 = __pyx_v_fut;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, ((PyObject *)(((PyTypeObject*)PyExc_TimeoutError)))};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_exception, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":215
 * def handle_timeout(fut: asyncio.Future[None]) -> None:
 *     """Handle a timeout."""
 *     if not fut.done():             # <<<<<<<<<<<<<<
 *         fut.set_exception(TimeoutError)
 * 
*/
  }

  /* "aioesphomeapi/connection.py":213
 * 
 * 
 * def handle_timeout(fut: asyncio.Future[None]) -> None:             # <<<<<<<<<<<<<<
 *     """Handle a timeout."""
 *     if not fut.done():
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aioesphomeapi.connection.handle_timeout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_7handle_timeout(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_6handle_timeout, "Handle a timeout.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_7handle_timeout = {"handle_timeout", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_7handle_timeout, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_6handle_timeout};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_7handle_timeout(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_fut = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("handle_timeout (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_fut,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 213, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 213, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "handle_timeout", 0) < (0)) __PYX_ERR(0, 213, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("handle_timeout", 1, 1, 1, i); __PYX_ERR(0, 213, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 213, __pyx_L3_error)
    }
    __pyx_v_fut = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("handle_timeout", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 213, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.handle_timeout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_6handle_timeout(__pyx_self, __pyx_v_fut);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_6handle_timeout(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_fut) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("handle_timeout", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_handle_timeout(__pyx_v_fut, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.handle_timeout", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":222
 * 
 * 
 * def handle_complex_message(             # <<<<<<<<<<<<<<
 *     fut: asyncio.Future[None],
 *     responses: list[message.Message],
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_9handle_complex_message(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_handle_complex_message(PyObject *__pyx_v_fut, PyObject *__pyx_v_responses, PyObject *__pyx_v_do_append, PyObject *__pyx_v_do_stop, PyObject *__pyx_v_resp, CYTHON_UNUSED int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("handle_complex_message", 0);

  /* "aioesphomeapi/connection.py":230
 * ) -> None:
 *     """Handle a message that is part of a response."""
 *     if not fut.done():             # <<<<<<<<<<<<<<
 *         if do_append is None or do_append(resp):
 *             responses.append(resp)
*/
  __pyx_t_2 = __pyx_v_fut;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_done, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 230, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 230, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = (!__pyx_t_4);
  if (__pyx_t_5) {

    /* "aioesphomeapi/connection.py":231
 *     """Handle a message that is part of a response."""
 *     if not fut.done():
 *         if do_append is None or do_append(resp):             # <<<<<<<<<<<<<<
 *             responses.append(resp)
 *         if do_stop is None or do_stop(resp):
*/
    __pyx_t_4 = (__pyx_v_do_append == Py_None);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = NULL;
    __Pyx_INCREF(__pyx_v_do_append);
    __pyx_t_6 = __pyx_v_do_append; 
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_6))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_6);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_resp};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 231, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_4;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_5) {

      /* "aioesphomeapi/connection.py":232
 *     if not fut.done():
 *         if do_append is None or do_append(resp):
 *             responses.append(resp)             # <<<<<<<<<<<<<<
 *         if do_stop is None or do_stop(resp):
 *             fut.set_result(None)
*/
      __pyx_t_7 = __Pyx_PyList_Append(__pyx_v_responses, __pyx_v_resp); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 232, __pyx_L1_error)

      /* "aioesphomeapi/connection.py":231
 *     """Handle a message that is part of a response."""
 *     if not fut.done():
 *         if do_append is None or do_append(resp):             # <<<<<<<<<<<<<<
 *             responses.append(resp)
 *         if do_stop is None or do_stop(resp):
*/
    }

    /* "aioesphomeapi/connection.py":233
 *         if do_append is None or do_append(resp):
 *             responses.append(resp)
 *         if do_stop is None or do_stop(resp):             # <<<<<<<<<<<<<<
 *             fut.set_result(None)
 * 
*/
    __pyx_t_4 = (__pyx_v_do_stop == Py_None);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_6 = NULL;
    __Pyx_INCREF(__pyx_v_do_stop);
    __pyx_t_2 = __pyx_v_do_stop; 
    __pyx_t_3 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
      assert(__pyx_t_6);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
      __pyx_t_3 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_v_resp};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 233, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = __pyx_t_4;
    __pyx_L8_bool_binop_done:;
    if (__pyx_t_5) {

      /* "aioesphomeapi/connection.py":234
 *             responses.append(resp)
 *         if do_stop is None or do_stop(resp):
 *             fut.set_result(None)             # <<<<<<<<<<<<<<
 * 
 * 
*/
      __pyx_t_2 = __pyx_v_fut;
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_2, Py_None};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_result, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 234, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":233
 *         if do_append is None or do_append(resp):
 *             responses.append(resp)
 *         if do_stop is None or do_stop(resp):             # <<<<<<<<<<<<<<
 *             fut.set_result(None)
 * 
*/
    }

    /* "aioesphomeapi/connection.py":230
 * ) -> None:
 *     """Handle a message that is part of a response."""
 *     if not fut.done():             # <<<<<<<<<<<<<<
 *         if do_append is None or do_append(resp):
 *             responses.append(resp)
*/
  }

  /* "aioesphomeapi/connection.py":222
 * 
 * 
 * def handle_complex_message(             # <<<<<<<<<<<<<<
 *     fut: asyncio.Future[None],
 *     responses: list[message.Message],
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aioesphomeapi.connection.handle_complex_message", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_9handle_complex_message(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_8handle_complex_message, "Handle a message that is part of a response.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_9handle_complex_message = {"handle_complex_message", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_9handle_complex_message, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_8handle_complex_message};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_9handle_complex_message(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_fut = 0;
  PyObject *__pyx_v_responses = 0;
  PyObject *__pyx_v_do_append = 0;
  PyObject *__pyx_v_do_stop = 0;
  PyObject *__pyx_v_resp = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("handle_complex_message (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_fut,&__pyx_mstate_global->__pyx_n_u_responses,&__pyx_mstate_global->__pyx_n_u_do_append,&__pyx_mstate_global->__pyx_n_u_do_stop,&__pyx_mstate_global->__pyx_n_u_resp,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 222, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 222, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 222, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 222, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 222, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 222, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "handle_complex_message", 0) < (0)) __PYX_ERR(0, 222, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 5; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("handle_complex_message", 1, 5, 5, i); __PYX_ERR(0, 222, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 5)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 222, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 222, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 222, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 222, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 222, __pyx_L3_error)
    }
    __pyx_v_fut = values[0];
    __pyx_v_responses = ((PyObject*)values[1]);
    __pyx_v_do_append = values[2];
    __pyx_v_do_stop = values[3];
    __pyx_v_resp = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("handle_complex_message", 1, 5, 5, __pyx_nargs); __PYX_ERR(0, 222, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.handle_complex_message", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_responses), (&PyList_Type), 0, "responses", 2))) __PYX_ERR(0, 224, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_8handle_complex_message(__pyx_self, __pyx_v_fut, __pyx_v_responses, __pyx_v_do_append, __pyx_v_do_stop, __pyx_v_resp);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_8handle_complex_message(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_fut, PyObject *__pyx_v_responses, PyObject *__pyx_v_do_append, PyObject *__pyx_v_do_stop, PyObject *__pyx_v_resp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("handle_complex_message", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_handle_complex_message(__pyx_v_fut, __pyx_v_responses, __pyx_v_do_append, __pyx_v_do_stop, __pyx_v_resp, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.handle_complex_message", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":280
 *     )
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         params: ConnectionParams,
*/

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_params = 0;
  PyObject *__pyx_v_on_stop = 0;
  int __pyx_v_debug_enabled;
  PyObject *__pyx_v_log_name = 0;
  int __pyx_v_log_errors;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_params,&__pyx_mstate_global->__pyx_n_u_on_stop,&__pyx_mstate_global->__pyx_n_u_debug_enabled,&__pyx_mstate_global->__pyx_n_u_log_name,&__pyx_mstate_global->__pyx_n_u_log_errors,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 280, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 280, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, i); __PYX_ERR(0, 280, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 280, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 280, __pyx_L3_error)
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 280, __pyx_L3_error)
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 280, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 280, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_params = ((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)values[0]);
    __pyx_v_on_stop = values[1];
    __pyx_v_debug_enabled = __Pyx_PyObject_IsTrue(values[2]); if (unlikely((__pyx_v_debug_enabled == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 284, __pyx_L3_error)
    __pyx_v_log_name = ((PyObject*)values[3]);
    if (values[4]) {
      __pyx_v_log_errors = __Pyx_PyObject_IsTrue(values[4]); if (unlikely((__pyx_v_log_errors == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 286, __pyx_L3_error)
    } else {

      /* "aioesphomeapi/connection.py":286
 *         debug_enabled: bool,
 *         log_name: str | None,
 *         log_errors: bool = True,             # <<<<<<<<<<<<<<
 *     ) -> None:
 *         self._params = params
*/
      __pyx_v_log_errors = ((int)1);
    }
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 4, 5, __pyx_nargs); __PYX_ERR(0, 280, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_params), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, 0, "params", 0))) __PYX_ERR(0, 282, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_log_name), (&PyUnicode_Type), 1, "log_name", 2))) __PYX_ERR(0, 285, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection___init__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_params, __pyx_v_on_stop, __pyx_v_debug_enabled, __pyx_v_log_name, __pyx_v_log_errors);

  /* "aioesphomeapi/connection.py":280
 *     )
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         params: ConnectionParams,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection___init__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v_params, PyObject *__pyx_v_on_stop, int __pyx_v_debug_enabled, PyObject *__pyx_v_log_name, int __pyx_v_log_errors) {
  PyObject *__pyx_v_keepalive = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  float __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "aioesphomeapi/connection.py":288
 *         log_errors: bool = True,
 *     ) -> None:
 *         self._params = params             # <<<<<<<<<<<<<<
 *         self.on_stop = on_stop
 *         self._socket: socket.socket | None = None
*/
  __Pyx_INCREF((PyObject *)__pyx_v_params);
  __Pyx_GIVEREF((PyObject *)__pyx_v_params);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_params);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_params);
  __pyx_v_self->_params = __pyx_v_params;

  /* "aioesphomeapi/connection.py":289
 *     ) -> None:
 *         self._params = params
 *         self.on_stop = on_stop             # <<<<<<<<<<<<<<
 *         self._socket: socket.socket | None = None
 *         self._frame_helper: None | APINoiseFrameHelper | APIPlaintextFrameHelper = None
*/
  __Pyx_INCREF(__pyx_v_on_stop);
  __Pyx_GIVEREF(__pyx_v_on_stop);
  __Pyx_GOTREF(__pyx_v_self->on_stop);
  __Pyx_DECREF(__pyx_v_self->on_stop);
  __pyx_v_self->on_stop = __pyx_v_on_stop;

  /* "aioesphomeapi/connection.py":290
 *         self._params = params
 *         self.on_stop = on_stop
 *         self._socket: socket.socket | None = None             # <<<<<<<<<<<<<<
 *         self._frame_helper: None | APINoiseFrameHelper | APIPlaintextFrameHelper = None
 *         self.api_version: APIVersion | None = None
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_socket);
  __Pyx_DECREF(__pyx_v_self->_socket);
  __pyx_v_self->_socket = Py_None;

  /* "aioesphomeapi/connection.py":291
 *         self.on_stop = on_stop
 *         self._socket: socket.socket | None = None
 *         self._frame_helper: None | APINoiseFrameHelper | APIPlaintextFrameHelper = None             # <<<<<<<<<<<<<<
 *         self.api_version: APIVersion | None = None
 * 
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_frame_helper);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_frame_helper);
  __pyx_v_self->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)Py_None);

  /* "aioesphomeapi/connection.py":292
 *         self._socket: socket.socket | None = None
 *         self._frame_helper: None | APINoiseFrameHelper | APIPlaintextFrameHelper = None
 *         self.api_version: APIVersion | None = None             # <<<<<<<<<<<<<<
 * 
 *         self.connection_state = CONNECTION_STATE_INITIALIZED
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->api_version);
  __Pyx_DECREF(__pyx_v_self->api_version);
  __pyx_v_self->api_version = Py_None;

  /* "aioesphomeapi/connection.py":294
 *         self.api_version: APIVersion | None = None
 * 
 *         self.connection_state = CONNECTION_STATE_INITIALIZED             # <<<<<<<<<<<<<<
 * 
 *         # Message handlers currently subscribed to incoming messages
*/
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED);
  __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED);
  __Pyx_GOTREF(__pyx_v_self->connection_state);
  __Pyx_DECREF(__pyx_v_self->connection_state);
  __pyx_v_self->connection_state = __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED;

  /* "aioesphomeapi/connection.py":297
 * 
 *         # Message handlers currently subscribed to incoming messages
 *         self._message_handlers: dict[Any, set[Callable[[message.Message], None]]] = {}             # <<<<<<<<<<<<<<
 *         # The friendly name to show for this connection in the logs
 *         self.log_name = log_name or ",".join(params.addresses)
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_message_handlers);
  __Pyx_DECREF(__pyx_v_self->_message_handlers);
  __pyx_v_self->_message_handlers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":299
 *         self._message_handlers: dict[Any, set[Callable[[message.Message], None]]] = {}
 *         # The friendly name to show for this connection in the logs
 *         self.log_name = log_name or ",".join(params.addresses)             # <<<<<<<<<<<<<<
 * 
 *         # futures currently subscribed to exceptions in the read task
*/
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_log_name); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 299, __pyx_L1_error)
  if (!__pyx_t_2) {
  } else {
    __Pyx_INCREF(__pyx_v_log_name);
    __pyx_t_1 = __pyx_v_log_name;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_3 = __pyx_v_params->addresses;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = PyUnicode_Join(__pyx_mstate_global->__pyx_kp_u__3, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_1 = __pyx_t_4;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_L3_bool_binop_done:;
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->log_name);
  __Pyx_DECREF(__pyx_v_self->log_name);
  __pyx_v_self->log_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":302
 * 
 *         # futures currently subscribed to exceptions in the read task
 *         self._read_exception_futures: set[asyncio.Future[None]] = set()             # <<<<<<<<<<<<<<
 * 
 *         self._ping_timer: asyncio.TimerHandle | None = None
*/
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_read_exception_futures);
  __Pyx_DECREF(__pyx_v_self->_read_exception_futures);
  __pyx_v_self->_read_exception_futures = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":304
 *         self._read_exception_futures: set[asyncio.Future[None]] = set()
 * 
 *         self._ping_timer: asyncio.TimerHandle | None = None             # <<<<<<<<<<<<<<
 *         self._pong_timer: asyncio.TimerHandle | None = None
 *         keepalive = params.keepalive
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_ping_timer);
  __Pyx_DECREF(__pyx_v_self->_ping_timer);
  __pyx_v_self->_ping_timer = Py_None;

  /* "aioesphomeapi/connection.py":305
 * 
 *         self._ping_timer: asyncio.TimerHandle | None = None
 *         self._pong_timer: asyncio.TimerHandle | None = None             # <<<<<<<<<<<<<<
 *         keepalive = params.keepalive
 *         self._keep_alive_interval = keepalive
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_pong_timer);
  __Pyx_DECREF(__pyx_v_self->_pong_timer);
  __pyx_v_self->_pong_timer = Py_None;

  /* "aioesphomeapi/connection.py":306
 *         self._ping_timer: asyncio.TimerHandle | None = None
 *         self._pong_timer: asyncio.TimerHandle | None = None
 *         keepalive = params.keepalive             # <<<<<<<<<<<<<<
 *         self._keep_alive_interval = keepalive
 *         self._keep_alive_timeout = keepalive * KEEP_ALIVE_TIMEOUT_RATIO
*/
  __pyx_t_1 = __pyx_v_params->keepalive;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_keepalive = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":307
 *         self._pong_timer: asyncio.TimerHandle | None = None
 *         keepalive = params.keepalive
 *         self._keep_alive_interval = keepalive             # <<<<<<<<<<<<<<
 *         self._keep_alive_timeout = keepalive * KEEP_ALIVE_TIMEOUT_RATIO
 * 
*/
  __pyx_t_5 = __Pyx_PyFloat_AsFloat(__pyx_v_keepalive); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_v_self->_keep_alive_interval = __pyx_t_5;

  /* "aioesphomeapi/connection.py":308
 *         keepalive = params.keepalive
 *         self._keep_alive_interval = keepalive
 *         self._keep_alive_timeout = keepalive * KEEP_ALIVE_TIMEOUT_RATIO             # <<<<<<<<<<<<<<
 * 
 *         self._resolve_host_future: asyncio.Future[None] | None = None
*/
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_13aioesphomeapi_10connection_KEEP_ALIVE_TIMEOUT_RATIO); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = PyNumber_Multiply(__pyx_v_keepalive, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyFloat_AsFloat(__pyx_t_4); if (unlikely((__pyx_t_5 == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_self->_keep_alive_timeout = __pyx_t_5;

  /* "aioesphomeapi/connection.py":310
 *         self._keep_alive_timeout = keepalive * KEEP_ALIVE_TIMEOUT_RATIO
 * 
 *         self._resolve_host_future: asyncio.Future[None] | None = None             # <<<<<<<<<<<<<<
 *         self._start_connect_future: asyncio.Future[None] | None = None
 *         self._finish_connect_future: asyncio.Future[None] | None = None
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_resolve_host_future);
  __Pyx_DECREF(__pyx_v_self->_resolve_host_future);
  __pyx_v_self->_resolve_host_future = Py_None;

  /* "aioesphomeapi/connection.py":311
 * 
 *         self._resolve_host_future: asyncio.Future[None] | None = None
 *         self._start_connect_future: asyncio.Future[None] | None = None             # <<<<<<<<<<<<<<
 *         self._finish_connect_future: asyncio.Future[None] | None = None
 *         self._fatal_exception: Exception | None = None
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_start_connect_future);
  __Pyx_DECREF(__pyx_v_self->_start_connect_future);
  __pyx_v_self->_start_connect_future = Py_None;

  /* "aioesphomeapi/connection.py":312
 *         self._resolve_host_future: asyncio.Future[None] | None = None
 *         self._start_connect_future: asyncio.Future[None] | None = None
 *         self._finish_connect_future: asyncio.Future[None] | None = None             # <<<<<<<<<<<<<<
 *         self._fatal_exception: Exception | None = None
 *         self._expected_disconnect = False
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_finish_connect_future);
  __Pyx_DECREF(__pyx_v_self->_finish_connect_future);
  __pyx_v_self->_finish_connect_future = Py_None;

  /* "aioesphomeapi/connection.py":313
 *         self._start_connect_future: asyncio.Future[None] | None = None
 *         self._finish_connect_future: asyncio.Future[None] | None = None
 *         self._fatal_exception: Exception | None = None             # <<<<<<<<<<<<<<
 *         self._expected_disconnect = False
 *         self._send_pending_ping = False
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_fatal_exception);
  __Pyx_DECREF(__pyx_v_self->_fatal_exception);
  __pyx_v_self->_fatal_exception = ((PyObject*)Py_None);

  /* "aioesphomeapi/connection.py":314
 *         self._finish_connect_future: asyncio.Future[None] | None = None
 *         self._fatal_exception: Exception | None = None
 *         self._expected_disconnect = False             # <<<<<<<<<<<<<<
 *         self._send_pending_ping = False
 *         self._loop = asyncio.get_running_loop()
*/
  __pyx_v_self->_expected_disconnect = 0;

  /* "aioesphomeapi/connection.py":315
 *         self._fatal_exception: Exception | None = None
 *         self._expected_disconnect = False
 *         self._send_pending_ping = False             # <<<<<<<<<<<<<<
 *         self._loop = asyncio.get_running_loop()
 *         self.is_connected = False
*/
  __pyx_v_self->_send_pending_ping = 0;

  /* "aioesphomeapi/connection.py":316
 *         self._expected_disconnect = False
 *         self._send_pending_ping = False
 *         self._loop = asyncio.get_running_loop()             # <<<<<<<<<<<<<<
 *         self.is_connected = False
 *         self._handshake_complete = False
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_asyncio); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get_running_loop); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (1-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 316, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->_loop);
  __Pyx_DECREF(__pyx_v_self->_loop);
  __pyx_v_self->_loop = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "aioesphomeapi/connection.py":317
 *         self._send_pending_ping = False
 *         self._loop = asyncio.get_running_loop()
 *         self.is_connected = False             # <<<<<<<<<<<<<<
 *         self._handshake_complete = False
 *         self._debug_enabled = debug_enabled
*/
  __pyx_v_self->is_connected = 0;

  /* "aioesphomeapi/connection.py":318
 *         self._loop = asyncio.get_running_loop()
 *         self.is_connected = False
 *         self._handshake_complete = False             # <<<<<<<<<<<<<<
 *         self._debug_enabled = debug_enabled
 *         self.received_name: str = ""
*/
  __pyx_v_self->_handshake_complete = 0;

  /* "aioesphomeapi/connection.py":319
 *         self.is_connected = False
 *         self._handshake_complete = False
 *         self._debug_enabled = debug_enabled             # <<<<<<<<<<<<<<
 *         self.received_name: str = ""
 *         self._cached_timezone: str = ""
*/
  __pyx_v_self->_debug_enabled = __pyx_v_debug_enabled;

  /* "aioesphomeapi/connection.py":320
 *         self._handshake_complete = False
 *         self._debug_enabled = debug_enabled
 *         self.received_name: str = ""             # <<<<<<<<<<<<<<
 *         self._cached_timezone: str = ""
 *         self.connected_address: str | None = None
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u__4);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u__4);
  __Pyx_GOTREF(__pyx_v_self->received_name);
  __Pyx_DECREF(__pyx_v_self->received_name);
  __pyx_v_self->received_name = __pyx_mstate_global->__pyx_kp_u__4;

  /* "aioesphomeapi/connection.py":321
 *         self._debug_enabled = debug_enabled
 *         self.received_name: str = ""
 *         self._cached_timezone: str = ""             # <<<<<<<<<<<<<<
 *         self.connected_address: str | None = None
 *         self._addrs_info: list[hr.AddrInfo] = []
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u__4);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u__4);
  __Pyx_GOTREF(__pyx_v_self->_cached_timezone);
  __Pyx_DECREF(__pyx_v_self->_cached_timezone);
  __pyx_v_self->_cached_timezone = __pyx_mstate_global->__pyx_kp_u__4;

  /* "aioesphomeapi/connection.py":322
 *         self.received_name: str = ""
 *         self._cached_timezone: str = ""
 *         self.connected_address: str | None = None             # <<<<<<<<<<<<<<
 *         self._addrs_info: list[hr.AddrInfo] = []
 *         self._log_errors = log_errors
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->connected_address);
  __Pyx_DECREF(__pyx_v_self->connected_address);
  __pyx_v_self->connected_address = ((PyObject*)Py_None);

  /* "aioesphomeapi/connection.py":323
 *         self._cached_timezone: str = ""
 *         self.connected_address: str | None = None
 *         self._addrs_info: list[hr.AddrInfo] = []             # <<<<<<<<<<<<<<
 *         self._log_errors = log_errors
 * 
*/
  __pyx_t_4 = PyList_New(0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 323, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_4);
  __Pyx_GOTREF(__pyx_v_self->_addrs_info);
  __Pyx_DECREF(__pyx_v_self->_addrs_info);
  __pyx_v_self->_addrs_info = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "aioesphomeapi/connection.py":324
 *         self.connected_address: str | None = None
 *         self._addrs_info: list[hr.AddrInfo] = []
 *         self._log_errors = log_errors             # <<<<<<<<<<<<<<
 * 
 *     def set_log_name(self, name: str) -> None:
*/
  __pyx_v_self->_log_errors = __pyx_v_log_errors;

  /* "aioesphomeapi/connection.py":280
 *     )
 * 
 *     def __init__(             # <<<<<<<<<<<<<<
 *         self,
 *         params: ConnectionParams,
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_keepalive);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":326
 *         self._log_errors = log_errors
 * 
 *     def set_log_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """Set the friendly log name for this connection."""
 *         self.log_name = name
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_3set_log_name(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_13aioesphomeapi_10connection_13APIConnection_set_log_name(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_name, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_log_name", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_set_log_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_3set_log_name)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_name};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 326, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":328
 *     def set_log_name(self, name: str) -> None:
 *         """Set the friendly log name for this connection."""
 *         self.log_name = name             # <<<<<<<<<<<<<<
 *         if self._frame_helper is not None:
 *             self._frame_helper.set_log_name(name)
*/
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->log_name);
  __Pyx_DECREF(__pyx_v_self->log_name);
  __pyx_v_self->log_name = __pyx_v_name;

  /* "aioesphomeapi/connection.py":329
 *         """Set the friendly log name for this connection."""
 *         self.log_name = name
 *         if self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             self._frame_helper.set_log_name(name)
 * 
*/
  __pyx_t_6 = (((PyObject *)__pyx_v_self->_frame_helper) != Py_None);
  if (__pyx_t_6) {

    /* "aioesphomeapi/connection.py":330
 *         self.log_name = name
 *         if self._frame_helper is not None:
 *             self._frame_helper.set_log_name(name)             # <<<<<<<<<<<<<<
 * 
 *     def _cleanup(self) -> None:
*/
    __pyx_t_1 = ((struct __pyx_vtabstruct_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)__pyx_v_self->_frame_helper->__pyx_vtab)->set_log_name(__pyx_v_self->_frame_helper, __pyx_v_name, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":329
 *         """Set the friendly log name for this connection."""
 *         self.log_name = name
 *         if self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             self._frame_helper.set_log_name(name)
 * 
*/
  }

  /* "aioesphomeapi/connection.py":326
 *         self._log_errors = log_errors
 * 
 *     def set_log_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """Set the friendly log name for this connection."""
 *         self.log_name = name
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.set_log_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_3set_log_name(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_2set_log_name, "Set the friendly log name for this connection.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_3set_log_name = {"set_log_name", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_3set_log_name, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_2set_log_name};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_3set_log_name(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_log_name (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 326, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 326, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_log_name", 0) < (0)) __PYX_ERR(0, 326, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("set_log_name", 1, 1, 1, i); __PYX_ERR(0, 326, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 326, __pyx_L3_error)
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_log_name", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 326, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.set_log_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyUnicode_Type), 0, "name", 2))) __PYX_ERR(0, 326, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_2set_log_name(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_2set_log_name(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_log_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_13aioesphomeapi_10connection_13APIConnection_set_log_name(__pyx_v_self, __pyx_v_name, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.set_log_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":332
 *             self._frame_helper.set_log_name(name)
 * 
 *     def _cleanup(self) -> None:             # <<<<<<<<<<<<<<
 *         """Clean up all resources that have been allocated.
 * 
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__cleanup(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_v_was_connected;
  PyObject *__pyx_v_fut = NULL;
  PyObject *__pyx_v_err = NULL;
  PyObject *__pyx_v_new_exc = NULL;
  PyObject *__pyx_v_on_stop = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  int __pyx_t_9;
  int __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_cleanup", 0);

  /* "aioesphomeapi/connection.py":337
 *         Safe to call multiple times.
 *         """
 *         if self.connection_state is CONNECTION_STATE_CLOSED:             # <<<<<<<<<<<<<<
 *             return
 *         was_connected = self.is_connected
*/
  __pyx_t_1 = (__pyx_v_self->connection_state == __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":338
 *         """
 *         if self.connection_state is CONNECTION_STATE_CLOSED:
 *             return             # <<<<<<<<<<<<<<
 *         was_connected = self.is_connected
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)
*/
    goto __pyx_L0;

    /* "aioesphomeapi/connection.py":337
 *         Safe to call multiple times.
 *         """
 *         if self.connection_state is CONNECTION_STATE_CLOSED:             # <<<<<<<<<<<<<<
 *             return
 *         was_connected = self.is_connected
*/
  }

  /* "aioesphomeapi/connection.py":339
 *         if self.connection_state is CONNECTION_STATE_CLOSED:
 *             return
 *         was_connected = self.is_connected             # <<<<<<<<<<<<<<
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)
 *         if self._debug_enabled:
*/
  __pyx_t_1 = __pyx_v_self->is_connected;
  __pyx_v_was_connected = __pyx_t_1;

  /* "aioesphomeapi/connection.py":340
 *             return
 *         was_connected = self.is_connected
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)             # <<<<<<<<<<<<<<
 *         if self._debug_enabled:
 *             if self._fatal_exception:
*/
  __pyx_t_2 = __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_set_connection_state(__pyx_v_self, __pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 340, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":341
 *         was_connected = self.is_connected
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             if self._fatal_exception:
 *                 _LOGGER.debug(
*/
  if (__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":342
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)
 *         if self._debug_enabled:
 *             if self._fatal_exception:             # <<<<<<<<<<<<<<
 *                 _LOGGER.debug(
 *                     "Cleaning up connection to %s (error: %s)",
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->_fatal_exception); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 342, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "aioesphomeapi/connection.py":343
 *         if self._debug_enabled:
 *             if self._fatal_exception:
 *                 _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                     "Cleaning up connection to %s (error: %s)",
 *                     self.log_name,
*/
      __pyx_t_3 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 343, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "aioesphomeapi/connection.py":346
 *                     "Cleaning up connection to %s (error: %s)",
 *                     self.log_name,
 *                     self._fatal_exception,             # <<<<<<<<<<<<<<
 *                 )
 *             else:
*/
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Cleaning_up_connection_to_s_erro, __pyx_v_self->log_name, __pyx_v_self->_fatal_exception};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 343, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "aioesphomeapi/connection.py":342
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)
 *         if self._debug_enabled:
 *             if self._fatal_exception:             # <<<<<<<<<<<<<<
 *                 _LOGGER.debug(
 *                     "Cleaning up connection to %s (error: %s)",
*/
      goto __pyx_L5;
    }

    /* "aioesphomeapi/connection.py":349
 *                 )
 *             else:
 *                 _LOGGER.debug("Cleaning up connection to %s", self.log_name)             # <<<<<<<<<<<<<<
 *         for fut in self._read_exception_futures:
 *             if not fut.done():
*/
    /*else*/ {
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 349, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_Cleaning_up_connection_to_s, __pyx_v_self->log_name};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 349, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L5:;

    /* "aioesphomeapi/connection.py":341
 *         was_connected = self.is_connected
 *         self._set_connection_state(CONNECTION_STATE_CLOSED)
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             if self._fatal_exception:
 *                 _LOGGER.debug(
*/
  }

  /* "aioesphomeapi/connection.py":350
 *             else:
 *                 _LOGGER.debug("Cleaning up connection to %s", self.log_name)
 *         for fut in self._read_exception_futures:             # <<<<<<<<<<<<<<
 *             if not fut.done():
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")
*/
  __pyx_t_7 = 0;
  __pyx_t_4 = __Pyx_set_iterator(__pyx_v_self->_read_exception_futures, 1, (&__pyx_t_8), (&__pyx_t_9)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 350, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_2);
  __pyx_t_2 = __pyx_t_4;
  __pyx_t_4 = 0;
  while (1) {
    __pyx_t_10 = __Pyx_set_iter_next(__pyx_t_2, __pyx_t_8, &__pyx_t_7, &__pyx_t_4, __pyx_t_9);
    if (unlikely(__pyx_t_10 == 0)) break;
    if (unlikely(__pyx_t_10 == -1)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_fut, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":351
 *                 _LOGGER.debug("Cleaning up connection to %s", self.log_name)
 *         for fut in self._read_exception_futures:
 *             if not fut.done():             # <<<<<<<<<<<<<<
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")
 *                 new_exc = err
*/
    __pyx_t_5 = __pyx_v_fut;
    __Pyx_INCREF(__pyx_t_5);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
      __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_done, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 351, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_11 = (!__pyx_t_1);
    if (__pyx_t_11) {

      /* "aioesphomeapi/connection.py":352
 *         for fut in self._read_exception_futures:
 *             if not fut.done():
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")             # <<<<<<<<<<<<<<
 *                 new_exc = err
 *                 if not isinstance(err, APIConnectionError):
*/
      __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v_self->_fatal_exception); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(0, 352, __pyx_L1_error)
      if (!__pyx_t_11) {
      } else {
        __Pyx_INCREF(__pyx_v_self->_fatal_exception);
        __pyx_t_4 = __pyx_v_self->_fatal_exception;
        goto __pyx_L9_bool_binop_done;
      }
      __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_APIConnectionError);
      __pyx_t_12 = __pyx_v_13aioesphomeapi_10connection_APIConnectionError; 
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_12))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_12);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Connection_closed};
        __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 352, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_4 = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_L9_bool_binop_done:;
      __Pyx_XDECREF_SET(__pyx_v_err, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "aioesphomeapi/connection.py":353
 *             if not fut.done():
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")
 *                 new_exc = err             # <<<<<<<<<<<<<<
 *                 if not isinstance(err, APIConnectionError):
 *                     new_exc = ReadFailedAPIError(str(err) or "Read failed")
*/
      __Pyx_INCREF(__pyx_v_err);
      __Pyx_XDECREF_SET(__pyx_v_new_exc, __pyx_v_err);

      /* "aioesphomeapi/connection.py":354
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")
 *                 new_exc = err
 *                 if not isinstance(err, APIConnectionError):             # <<<<<<<<<<<<<<
 *                     new_exc = ReadFailedAPIError(str(err) or "Read failed")
 *                     new_exc.__cause__ = err
*/
      __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_APIConnectionError;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_11 = PyObject_IsInstance(__pyx_v_err, __pyx_t_4); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 354, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_1 = (!__pyx_t_11);
      if (__pyx_t_1) {

        /* "aioesphomeapi/connection.py":355
 *                 new_exc = err
 *                 if not isinstance(err, APIConnectionError):
 *                     new_exc = ReadFailedAPIError(str(err) or "Read failed")             # <<<<<<<<<<<<<<
 *                     new_exc.__cause__ = err
 *                 fut.set_exception(new_exc)
*/
        __pyx_t_5 = NULL;
        __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError);
        __pyx_t_12 = __pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError; 
        __pyx_t_13 = __Pyx_PyObject_Unicode(__pyx_v_err); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 355, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_13); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 355, __pyx_L1_error)
        if (!__pyx_t_1) {
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        } else {
          __Pyx_INCREF(__pyx_t_13);
          __pyx_t_3 = __pyx_t_13;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          goto __pyx_L12_bool_binop_done;
        }
        __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_Read_failed);
        __pyx_t_3 = __pyx_mstate_global->__pyx_kp_u_Read_failed;
        __pyx_L12_bool_binop_done:;
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_12);
          assert(__pyx_t_5);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_3};
          __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_DECREF_SET(__pyx_v_new_exc, __pyx_t_4);
        __pyx_t_4 = 0;

        /* "aioesphomeapi/connection.py":356
 *                 if not isinstance(err, APIConnectionError):
 *                     new_exc = ReadFailedAPIError(str(err) or "Read failed")
 *                     new_exc.__cause__ = err             # <<<<<<<<<<<<<<
 *                 fut.set_exception(new_exc)
 *         self._read_exception_futures.clear()
*/
        if (__Pyx_PyObject_SetAttrStr(__pyx_v_new_exc, __pyx_mstate_global->__pyx_n_u_cause, __pyx_v_err) < (0)) __PYX_ERR(0, 356, __pyx_L1_error)

        /* "aioesphomeapi/connection.py":354
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")
 *                 new_exc = err
 *                 if not isinstance(err, APIConnectionError):             # <<<<<<<<<<<<<<
 *                     new_exc = ReadFailedAPIError(str(err) or "Read failed")
 *                     new_exc.__cause__ = err
*/
      }

      /* "aioesphomeapi/connection.py":357
 *                     new_exc = ReadFailedAPIError(str(err) or "Read failed")
 *                     new_exc.__cause__ = err
 *                 fut.set_exception(new_exc)             # <<<<<<<<<<<<<<
 *         self._read_exception_futures.clear()
 * 
*/
      __pyx_t_12 = __pyx_v_fut;
      __Pyx_INCREF(__pyx_t_12);
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_v_new_exc};
        __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_exception, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 357, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "aioesphomeapi/connection.py":351
 *                 _LOGGER.debug("Cleaning up connection to %s", self.log_name)
 *         for fut in self._read_exception_futures:
 *             if not fut.done():             # <<<<<<<<<<<<<<
 *                 err = self._fatal_exception or APIConnectionError("Connection closed")
 *                 new_exc = err
*/
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":358
 *                     new_exc.__cause__ = err
 *                 fut.set_exception(new_exc)
 *         self._read_exception_futures.clear()             # <<<<<<<<<<<<<<
 * 
 *         self._set_resolve_host_future()
*/
  if (unlikely(__pyx_v_self->_read_exception_futures == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "clear");
    __PYX_ERR(0, 358, __pyx_L1_error)
  }
  __pyx_t_14 = PySet_Clear(__pyx_v_self->_read_exception_futures); if (unlikely(__pyx_t_14 == ((int)-1))) __PYX_ERR(0, 358, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":360
 *         self._read_exception_futures.clear()
 * 
 *         self._set_resolve_host_future()             # <<<<<<<<<<<<<<
 *         self._set_start_connect_future()
 *         self._set_finish_connect_future()
*/
  __pyx_t_4 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_resolve_host_future, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":361
 * 
 *         self._set_resolve_host_future()
 *         self._set_start_connect_future()             # <<<<<<<<<<<<<<
 *         self._set_finish_connect_future()
 * 
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_set_start_connect_future(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 361, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":362
 *         self._set_resolve_host_future()
 *         self._set_start_connect_future()
 *         self._set_finish_connect_future()             # <<<<<<<<<<<<<<
 * 
 *         if self._frame_helper is not None:
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_set_finish_connect_future(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 362, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":364
 *         self._set_finish_connect_future()
 * 
 *         if self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             self._frame_helper.close()
 *             self._frame_helper = None
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_self->_frame_helper) != Py_None);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":365
 * 
 *         if self._frame_helper is not None:
 *             self._frame_helper.close()             # <<<<<<<<<<<<<<
 *             self._frame_helper = None
 * 
*/
    __pyx_t_4 = ((PyObject *)__pyx_v_self->_frame_helper);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_close, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 365, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":366
 *         if self._frame_helper is not None:
 *             self._frame_helper.close()
 *             self._frame_helper = None             # <<<<<<<<<<<<<<
 * 
 *         if self._socket is not None:
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF((PyObject *)__pyx_v_self->_frame_helper);
    __Pyx_DECREF((PyObject *)__pyx_v_self->_frame_helper);
    __pyx_v_self->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)Py_None);

    /* "aioesphomeapi/connection.py":364
 *         self._set_finish_connect_future()
 * 
 *         if self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             self._frame_helper.close()
 *             self._frame_helper = None
*/
  }

  /* "aioesphomeapi/connection.py":368
 *             self._frame_helper = None
 * 
 *         if self._socket is not None:             # <<<<<<<<<<<<<<
 *             self._socket.close()
 *             self._socket = None
*/
  __pyx_t_1 = (__pyx_v_self->_socket != Py_None);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":369
 * 
 *         if self._socket is not None:
 *             self._socket.close()             # <<<<<<<<<<<<<<
 *             self._socket = None
 * 
*/
    __pyx_t_4 = __pyx_v_self->_socket;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_close, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":370
 *         if self._socket is not None:
 *             self._socket.close()
 *             self._socket = None             # <<<<<<<<<<<<<<
 * 
 *         self._async_cancel_pong_timer()
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_socket);
    __Pyx_DECREF(__pyx_v_self->_socket);
    __pyx_v_self->_socket = Py_None;

    /* "aioesphomeapi/connection.py":368
 *             self._frame_helper = None
 * 
 *         if self._socket is not None:             # <<<<<<<<<<<<<<
 *             self._socket.close()
 *             self._socket = None
*/
  }

  /* "aioesphomeapi/connection.py":372
 *             self._socket = None
 * 
 *         self._async_cancel_pong_timer()             # <<<<<<<<<<<<<<
 * 
 *         if self._ping_timer is not None:
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_async_cancel_pong_timer(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 372, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":374
 *         self._async_cancel_pong_timer()
 * 
 *         if self._ping_timer is not None:             # <<<<<<<<<<<<<<
 *             self._ping_timer.cancel()
 *             self._ping_timer = None
*/
  __pyx_t_1 = (__pyx_v_self->_ping_timer != Py_None);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":375
 * 
 *         if self._ping_timer is not None:
 *             self._ping_timer.cancel()             # <<<<<<<<<<<<<<
 *             self._ping_timer = None
 * 
*/
    __pyx_t_4 = __pyx_v_self->_ping_timer;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_6 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancel, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":376
 *         if self._ping_timer is not None:
 *             self._ping_timer.cancel()
 *             self._ping_timer = None             # <<<<<<<<<<<<<<
 * 
 *         if (on_stop := self.on_stop) is not None and was_connected:
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_ping_timer);
    __Pyx_DECREF(__pyx_v_self->_ping_timer);
    __pyx_v_self->_ping_timer = Py_None;

    /* "aioesphomeapi/connection.py":374
 *         self._async_cancel_pong_timer()
 * 
 *         if self._ping_timer is not None:             # <<<<<<<<<<<<<<
 *             self._ping_timer.cancel()
 *             self._ping_timer = None
*/
  }

  /* "aioesphomeapi/connection.py":378
 *             self._ping_timer = None
 * 
 *         if (on_stop := self.on_stop) is not None and was_connected:             # <<<<<<<<<<<<<<
 *             self.on_stop = None
 *             on_stop(self._expected_disconnect)
*/
  __pyx_t_2 = __pyx_v_self->on_stop;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_on_stop = __pyx_t_2;
  __pyx_t_11 = (__pyx_t_2 != Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_11) {
  } else {
    __pyx_t_1 = __pyx_t_11;
    goto __pyx_L18_bool_binop_done;
  }
  __pyx_t_1 = __pyx_v_was_connected;
  __pyx_L18_bool_binop_done:;
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":379
 * 
 *         if (on_stop := self.on_stop) is not None and was_connected:
 *             self.on_stop = None             # <<<<<<<<<<<<<<
 *             on_stop(self._expected_disconnect)
 * 
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->on_stop);
    __Pyx_DECREF(__pyx_v_self->on_stop);
    __pyx_v_self->on_stop = Py_None;

    /* "aioesphomeapi/connection.py":380
 *         if (on_stop := self.on_stop) is not None and was_connected:
 *             self.on_stop = None
 *             on_stop(self._expected_disconnect)             # <<<<<<<<<<<<<<
 * 
 *     def set_debug(self, enable: bool) -> None:
*/
    __pyx_t_4 = NULL;
    __Pyx_INCREF(__pyx_v_on_stop);
    __pyx_t_12 = __pyx_v_on_stop; 
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->_expected_disconnect); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 380, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(__pyx_t_12))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_12);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 380, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":378
 *             self._ping_timer = None
 * 
 *         if (on_stop := self.on_stop) is not None and was_connected:             # <<<<<<<<<<<<<<
 *             self.on_stop = None
 *             on_stop(self._expected_disconnect)
*/
  }

  /* "aioesphomeapi/connection.py":332
 *             self._frame_helper.set_log_name(name)
 * 
 *     def _cleanup(self) -> None:             # <<<<<<<<<<<<<<
 *         """Clean up all resources that have been allocated.
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._cleanup", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_fut);
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_XDECREF(__pyx_v_new_exc);
  __Pyx_XDECREF(__pyx_v_on_stop);
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":382
 *             on_stop(self._expected_disconnect)
 * 
 *     def set_debug(self, enable: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Enable or disable debug logging."""
 *         self._debug_enabled = enable
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_5set_debug(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_4set_debug, "Enable or disable debug logging.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_5set_debug = {"set_debug", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_5set_debug, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_4set_debug};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_5set_debug(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_enable;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_debug (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_enable,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 382, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 382, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "set_debug", 0) < (0)) __PYX_ERR(0, 382, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("set_debug", 1, 1, 1, i); __PYX_ERR(0, 382, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 382, __pyx_L3_error)
    }
    __pyx_v_enable = __Pyx_PyObject_IsTrue(values[0]); if (unlikely((__pyx_v_enable == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_debug", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 382, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.set_debug", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_4set_debug(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_enable);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_4set_debug(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_enable) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_debug", 0);

  /* "aioesphomeapi/connection.py":384
 *     def set_debug(self, enable: bool) -> None:
 *         """Enable or disable debug logging."""
 *         self._debug_enabled = enable             # <<<<<<<<<<<<<<
 * 
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:
*/
  __pyx_v_self->_debug_enabled = __pyx_v_enable;

  /* "aioesphomeapi/connection.py":382
 *             on_stop(self._expected_disconnect)
 * 
 *     def set_debug(self, enable: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Enable or disable debug logging."""
 *         self._debug_enabled = enable
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_8generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":386
 *         self._debug_enabled = enable
 * 
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:             # <<<<<<<<<<<<<<
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_connect_socket_connect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_6_connect_socket_connect, "Step 2 in connect process: connect the socket.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_7_connect_socket_connect = {"_connect_socket_connect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_connect_socket_connect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_6_connect_socket_connect};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_connect_socket_connect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_addrs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_connect_socket_connect (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_addrs,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 386, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 386, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_connect_socket_connect", 0) < (0)) __PYX_ERR(0, 386, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_connect_socket_connect", 1, 1, 1, i); __PYX_ERR(0, 386, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 386, __pyx_L3_error)
    }
    __pyx_v_addrs = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_connect_socket_connect", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 386, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_addrs), (&PyList_Type), 0, "addrs", 2))) __PYX_ERR(0, 386, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_6_connect_socket_connect(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_addrs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_2generator11(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":392
 *                 "%s: Connecting to %s",
 *                 self.log_name,
 *                 ", ".join(str(addr.sockaddr) for addr in addrs),             # <<<<<<<<<<<<<<
 *             )
 * 
*/

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_genexpr(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_genexpr_arg_0) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 392, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_genexpr_arg_0 = __pyx_genexpr_arg_0;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_genexpr_arg_0);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_genexpr_arg_0);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_2generator11, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_genexpr, __pyx_mstate_global->__pyx_n_u_connect_socket_connect_locals_g, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_socket_connect.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_2generator11(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 392, __pyx_L1_error)
  __pyx_r = PyList_New(0); if (unlikely(!__pyx_r)) __PYX_ERR(0, 392, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_r);
  if (unlikely(!__pyx_cur_scope->__pyx_genexpr_arg_0)) { __Pyx_RaiseUnboundLocalError(".0"); __PYX_ERR(0, 392, __pyx_L1_error) }
  __pyx_t_1 = __pyx_cur_scope->__pyx_genexpr_arg_0; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 392, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    __pyx_t_3 = __Pyx_PyList_GetItemRefFast(__pyx_t_1, __pyx_t_2, __Pyx_ReferenceSharing_OwnStrongReference);
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_addr);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_addr, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_v_addr, __pyx_mstate_global->__pyx_n_u_sockaddr); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_Unicode(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(__Pyx_ListComp_Append(__pyx_r, (PyObject*)__pyx_t_4))) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":386
 *         self._debug_enabled = enable
 * 
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:             # <<<<<<<<<<<<<<
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:
*/

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_6_connect_socket_connect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_addrs) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *__pyx_cur_scope;
  PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_2generator11 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_connect_socket_connect", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 386, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_addrs = __pyx_v_addrs;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_addrs);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_addrs);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_8generator, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_connect_socket_connect, __pyx_mstate_global->__pyx_n_u_APIConnection__connect_socket_co, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 386, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XDECREF(__pyx_gb_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_2generator11);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_8generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  Py_ssize_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  __Pyx_PySendResult __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_t_22;
  int __pyx_t_23;
  int __pyx_t_24;
  char const *__pyx_t_25;
  PyObject *__pyx_t_26 = NULL;
  PyObject *__pyx_t_27[4];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_connect_socket_connect", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L19_resume_from_await;
    case 2: goto __pyx_L33_resume_from_await;
    case 3: goto __pyx_L37_resume_from_await;
    case 4: goto __pyx_L38_resume_from_await;
    case 5: goto __pyx_L39_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 386, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":388
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Connecting to %s",
*/
  if (__pyx_cur_scope->__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":389
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:
 *             _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                 "%s: Connecting to %s",
 *                 self.log_name,
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 389, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":392
 *                 "%s: Connecting to %s",
 *                 self.log_name,
 *                 ", ".join(str(addr.sockaddr) for addr in addrs),             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_3 = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_23_connect_socket_connect_genexpr(NULL, __pyx_cur_scope->__pyx_v_addrs); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_Generator_GetInlinedResult(__pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyUnicode_Join(__pyx_mstate_global->__pyx_kp_u__5, __pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 392, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_s_Connecting_to_s, __pyx_cur_scope->__pyx_v_self->log_name, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":388
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Connecting to %s",
*/
  }

  /* "aioesphomeapi/connection.py":395
 *             )
 * 
 *         addr_infos: list[aiohappyeyeballs.AddrInfoType] = [             # <<<<<<<<<<<<<<
 *             (
 *                 addr.family,
*/
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 395, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "aioesphomeapi/connection.py":403
 *                 astuple(addr.sockaddr),
 *             )
 *             for addr in addrs             # <<<<<<<<<<<<<<
 *         ]
 *         last_exception: Exception | None = None
*/
    __pyx_t_4 = __pyx_cur_scope->__pyx_v_addrs; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_7 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_t_4);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 403, __pyx_L1_error)
        #endif
        if (__pyx_t_7 >= __pyx_temp) break;
      }
      __pyx_t_3 = __Pyx_PyList_GetItemRefFast(__pyx_t_4, __pyx_t_7, __Pyx_ReferenceSharing_OwnStrongReference);
      ++__pyx_t_7;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 403, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_cur_scope->__pyx_8genexpr3__pyx_v_addr);
      __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_8genexpr3__pyx_v_addr, __pyx_t_3);
      __Pyx_GIVEREF(__pyx_t_3);
      __pyx_t_3 = 0;

      /* "aioesphomeapi/connection.py":397
 *         addr_infos: list[aiohappyeyeballs.AddrInfoType] = [
 *             (
 *                 addr.family,             # <<<<<<<<<<<<<<
 *                 addr.type,
 *                 addr.proto,
*/
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_8genexpr3__pyx_v_addr, __pyx_mstate_global->__pyx_n_u_family); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);

      /* "aioesphomeapi/connection.py":398
 *             (
 *                 addr.family,
 *                 addr.type,             # <<<<<<<<<<<<<<
 *                 addr.proto,
 *                 "",
*/
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_8genexpr3__pyx_v_addr, __pyx_mstate_global->__pyx_n_u_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 398, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "aioesphomeapi/connection.py":399
 *                 addr.family,
 *                 addr.type,
 *                 addr.proto,             # <<<<<<<<<<<<<<
 *                 "",
 *                 astuple(addr.sockaddr),
*/
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_8genexpr3__pyx_v_addr, __pyx_mstate_global->__pyx_n_u_proto); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 399, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "aioesphomeapi/connection.py":401
 *                 addr.proto,
 *                 "",
 *                 astuple(addr.sockaddr),             # <<<<<<<<<<<<<<
 *             )
 *             for addr in addrs
*/
      __pyx_t_9 = NULL;
      __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_astuple);
      __pyx_t_10 = __pyx_v_13aioesphomeapi_10connection_astuple; 
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_cur_scope->__pyx_8genexpr3__pyx_v_addr, __pyx_mstate_global->__pyx_n_u_sockaddr); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 401, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_9);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_t_11};
        __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 401, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
      }

      /* "aioesphomeapi/connection.py":397
 *         addr_infos: list[aiohappyeyeballs.AddrInfoType] = [
 *             (
 *                 addr.family,             # <<<<<<<<<<<<<<
 *                 addr.type,
 *                 addr.proto,
*/
      __pyx_t_10 = PyTuple_New(5); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 397, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_GIVEREF(__pyx_t_3);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_3) != (0)) __PYX_ERR(0, 397, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_2);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 397, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_5);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 2, __pyx_t_5) != (0)) __PYX_ERR(0, 397, __pyx_L1_error);
      __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u__4);
      __Pyx_GIVEREF(__pyx_mstate_global->__pyx_kp_u__4);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 3, __pyx_mstate_global->__pyx_kp_u__4) != (0)) __PYX_ERR(0, 397, __pyx_L1_error);
      __Pyx_GIVEREF(__pyx_t_8);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 4, __pyx_t_8) != (0)) __PYX_ERR(0, 397, __pyx_L1_error);
      __pyx_t_3 = 0;
      __pyx_t_2 = 0;
      __pyx_t_5 = 0;
      __pyx_t_8 = 0;
      if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_10))) __PYX_ERR(0, 395, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

      /* "aioesphomeapi/connection.py":403
 *                 astuple(addr.sockaddr),
 *             )
 *             for addr in addrs             # <<<<<<<<<<<<<<
 *         ]
 *         last_exception: Exception | None = None
*/
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } /* exit inner scope */
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_addr_infos = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":405
 *             for addr in addrs
 *         ]
 *         last_exception: Exception | None = None             # <<<<<<<<<<<<<<
 *         sock: socket.socket | None = None
 *         interleave = 1
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __pyx_cur_scope->__pyx_v_last_exception = ((PyObject*)Py_None);

  /* "aioesphomeapi/connection.py":406
 *         ]
 *         last_exception: Exception | None = None
 *         sock: socket.socket | None = None             # <<<<<<<<<<<<<<
 *         interleave = 1
 *         while addr_infos:
*/
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __pyx_cur_scope->__pyx_v_sock = Py_None;

  /* "aioesphomeapi/connection.py":407
 *         last_exception: Exception | None = None
 *         sock: socket.socket | None = None
 *         interleave = 1             # <<<<<<<<<<<<<<
 *         while addr_infos:
 *             try:
*/
  __pyx_cur_scope->__pyx_v_interleave = 1;

  /* "aioesphomeapi/connection.py":408
 *         sock: socket.socket | None = None
 *         interleave = 1
 *         while addr_infos:             # <<<<<<<<<<<<<<
 *             try:
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):
*/
  while (1) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyList_GET_SIZE(__pyx_cur_scope->__pyx_v_addr_infos);
      if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 408, __pyx_L1_error)
      __pyx_t_12 = (__pyx_temp != 0);
    }

    if (!__pyx_t_12) break;

    /* "aioesphomeapi/connection.py":409
 *         interleave = 1
 *         while addr_infos:
 *             try:             # <<<<<<<<<<<<<<
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):
 *                     # Devices are likely on the local network so we
*/
    {
      __Pyx_ExceptionSave(&__pyx_t_13, &__pyx_t_14, &__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_15);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":410
 *         while addr_infos:
 *             try:
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):             # <<<<<<<<<<<<<<
 *                     # Devices are likely on the local network so we
 *                     # only use a 100ms happy eyeballs delay
*/
        /*with:*/ {
          __pyx_t_4 = NULL;
          __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_asyncio_timeout);
          __pyx_t_10 = __pyx_v_13aioesphomeapi_10connection_asyncio_timeout; 
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_TCP_CONNECT_TIMEOUT); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 410, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_8);
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_10))) {
            __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_10);
            assert(__pyx_t_4);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_8};
            __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
            if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 410, __pyx_L10_error)
            __Pyx_GOTREF(__pyx_t_1);
          }
          __pyx_t_16 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_aexit); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 410, __pyx_L10_error)
          __Pyx_GOTREF(__pyx_t_16);
          __pyx_t_8 = NULL;
          __pyx_t_4 = __Pyx_PyObject_LookupSpecial(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_aenter); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 410, __pyx_L18_error)
          __Pyx_GOTREF(__pyx_t_4);
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_4))) {
            __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
            assert(__pyx_t_8);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
            __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 410, __pyx_L18_error)
            __Pyx_GOTREF(__pyx_t_10);
          }
          __pyx_t_17 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_10, &__pyx_r);
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (likely(__pyx_t_17 == PYGEN_NEXT)) {
            __Pyx_GOTREF(__pyx_r);
            __Pyx_XGIVEREF(__pyx_t_1);
            __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
            __Pyx_XGIVEREF(__pyx_t_13);
            __pyx_cur_scope->__pyx_t_1 = __pyx_t_13;
            __Pyx_XGIVEREF(__pyx_t_14);
            __pyx_cur_scope->__pyx_t_2 = __pyx_t_14;
            __Pyx_XGIVEREF(__pyx_t_15);
            __pyx_cur_scope->__pyx_t_3 = __pyx_t_15;
            __Pyx_XGIVEREF(__pyx_t_16);
            __pyx_cur_scope->__pyx_t_4 = __pyx_t_16;
            __Pyx_XGIVEREF(__pyx_r);
            __Pyx_RefNannyFinishContext();
            __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
            /* return from generator, awaiting value */
            __pyx_generator->resume_label = 1;
            return __pyx_r;
            __pyx_L19_resume_from_await:;
            __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
            __pyx_cur_scope->__pyx_t_0 = 0;
            __Pyx_XGOTREF(__pyx_t_1);
            __pyx_t_13 = __pyx_cur_scope->__pyx_t_1;
            __pyx_cur_scope->__pyx_t_1 = 0;
            __Pyx_XGOTREF(__pyx_t_13);
            __pyx_t_14 = __pyx_cur_scope->__pyx_t_2;
            __pyx_cur_scope->__pyx_t_2 = 0;
            __Pyx_XGOTREF(__pyx_t_14);
            __pyx_t_15 = __pyx_cur_scope->__pyx_t_3;
            __pyx_cur_scope->__pyx_t_3 = 0;
            __Pyx_XGOTREF(__pyx_t_15);
            __pyx_t_16 = __pyx_cur_scope->__pyx_t_4;
            __pyx_cur_scope->__pyx_t_4 = 0;
            __Pyx_XGOTREF(__pyx_t_16);
            if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 410, __pyx_L18_error)
            __pyx_t_10 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_10);
          } else if (likely(__pyx_t_17 == PYGEN_RETURN)) {
            __Pyx_GOTREF(__pyx_r);
            __pyx_t_10 = __pyx_r; __pyx_r = NULL;
          } else {
            __Pyx_XGOTREF(__pyx_r);
            __PYX_ERR(0, 410, __pyx_L18_error)
          }
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          /*try:*/ {
            {
              __Pyx_ExceptionSave(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
              __Pyx_XGOTREF(__pyx_t_18);
              __Pyx_XGOTREF(__pyx_t_19);
              __Pyx_XGOTREF(__pyx_t_20);
              /*try:*/ {

                /* "aioesphomeapi/connection.py":413
 *                     # Devices are likely on the local network so we
 *                     # only use a 100ms happy eyeballs delay
 *                     sock = await aiohappyeyeballs.start_connection(             # <<<<<<<<<<<<<<
 *                         addr_infos,
 *                         happy_eyeballs_delay=0.1,
*/
                __pyx_t_10 = NULL;
                __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_aiohappyeyeballs); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 413, __pyx_L25_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_start_connection); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 413, __pyx_L25_error)
                __Pyx_GOTREF(__pyx_t_8);
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

                /* "aioesphomeapi/connection.py":416
 *                         addr_infos,
 *                         happy_eyeballs_delay=0.1,
 *                         interleave=interleave,             # <<<<<<<<<<<<<<
 *                         loop=self._loop,
 *                     )
*/
                __pyx_t_4 = __Pyx_PyLong_From_long(__pyx_cur_scope->__pyx_v_interleave); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 416, __pyx_L25_error)
                __Pyx_GOTREF(__pyx_t_4);

                /* "aioesphomeapi/connection.py":417
 *                         happy_eyeballs_delay=0.1,
 *                         interleave=interleave,
 *                         loop=self._loop,             # <<<<<<<<<<<<<<
 *                     )
 *                     break
*/
                __pyx_t_6 = 1;
                #if CYTHON_UNPACK_METHODS
                if (unlikely(PyMethod_Check(__pyx_t_8))) {
                  __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
                  assert(__pyx_t_10);
                  PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
                  __Pyx_INCREF(__pyx_t_10);
                  __Pyx_INCREF(__pyx__function);
                  __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
                  __pyx_t_6 = 0;
                }
                #endif
                {
                  PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_addr_infos};
                  __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 413, __pyx_L25_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_happy_eyeballs_delay, __pyx_mstate_global->__pyx_float_0_1, __pyx_t_5, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 413, __pyx_L25_error)
                  if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_interleave, __pyx_t_4, __pyx_t_5, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 413, __pyx_L25_error)
                  if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_loop, __pyx_cur_scope->__pyx_v_self->_loop, __pyx_t_5, __pyx_callargs+2, 2) < (0)) __PYX_ERR(0, 413, __pyx_L25_error)
                  __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
                  __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
                  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
                  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 413, __pyx_L25_error)
                  __Pyx_GOTREF(__pyx_t_1);
                }
                __pyx_t_17 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_1, &__pyx_r);
                __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
                if (likely(__pyx_t_17 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_t_16);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_16;
                  __Pyx_XGIVEREF(__pyx_t_18);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_18;
                  __Pyx_XGIVEREF(__pyx_t_19);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_19;
                  __Pyx_XGIVEREF(__pyx_t_20);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_20;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 2;
                  return __pyx_r;
                  __pyx_L33_resume_from_await:;
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  __pyx_t_16 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_16);
                  __pyx_t_18 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_18);
                  __pyx_t_19 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_19);
                  __pyx_t_20 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_20);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 413, __pyx_L25_error)
                  __pyx_t_1 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_1);
                } else if (likely(__pyx_t_17 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_1 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 413, __pyx_L25_error)
                }
                __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_sock);
                __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_sock, __pyx_t_1);
                __Pyx_GIVEREF(__pyx_t_1);
                __pyx_t_1 = 0;

                /* "aioesphomeapi/connection.py":419
 *                         loop=self._loop,
 *                     )
 *                     break             # <<<<<<<<<<<<<<
 *             except (OSError, TimeoutError) as err:
 *                 last_exception = err
*/
                goto __pyx_L30_try_break;

                /* "aioesphomeapi/connection.py":410
 *         while addr_infos:
 *             try:
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):             # <<<<<<<<<<<<<<
 *                     # Devices are likely on the local network so we
 *                     # only use a 100ms happy eyeballs delay
*/
              }
              __pyx_L25_error:;
              __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
              __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
              __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
              __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              /*except:*/ {
                __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_8, &__pyx_t_5) < 0) __PYX_ERR(0, 410, __pyx_L27_except_error)
                __Pyx_XGOTREF(__pyx_t_1);
                __Pyx_XGOTREF(__pyx_t_8);
                __Pyx_XGOTREF(__pyx_t_5);
                __pyx_t_4 = PyTuple_Pack(3, __pyx_t_1, __pyx_t_8, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 410, __pyx_L27_except_error)
                __Pyx_GOTREF(__pyx_t_4);
                __pyx_t_21 = __Pyx_PyObject_Call(__pyx_t_16, __pyx_t_4, NULL);
                __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
                if (unlikely(!__pyx_t_21)) __PYX_ERR(0, 410, __pyx_L27_except_error)
                __Pyx_GOTREF(__pyx_t_21);
                __pyx_t_17 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_21, &__pyx_r);
                __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                if (likely(__pyx_t_17 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_1);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_1;
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_8);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_8;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_t_16);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_16;
                  __Pyx_XGIVEREF(__pyx_t_18);
                  __pyx_cur_scope->__pyx_t_7 = __pyx_t_18;
                  __Pyx_XGIVEREF(__pyx_t_19);
                  __pyx_cur_scope->__pyx_t_8 = __pyx_t_19;
                  __Pyx_XGIVEREF(__pyx_t_20);
                  __pyx_cur_scope->__pyx_t_9 = __pyx_t_20;
                  __Pyx_XGIVEREF(__pyx_t_21);
                  __pyx_cur_scope->__pyx_t_10 = __pyx_t_21;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_SwapException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 3;
                  return __pyx_r;
                  __pyx_L37_resume_from_await:;
                  __pyx_t_1 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_1);
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_8 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_8);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  __pyx_t_16 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_16);
                  __pyx_t_18 = __pyx_cur_scope->__pyx_t_7;
                  __pyx_cur_scope->__pyx_t_7 = 0;
                  __Pyx_XGOTREF(__pyx_t_18);
                  __pyx_t_19 = __pyx_cur_scope->__pyx_t_8;
                  __pyx_cur_scope->__pyx_t_8 = 0;
                  __Pyx_XGOTREF(__pyx_t_19);
                  __pyx_t_20 = __pyx_cur_scope->__pyx_t_9;
                  __pyx_cur_scope->__pyx_t_9 = 0;
                  __Pyx_XGOTREF(__pyx_t_20);
                  __pyx_t_21 = __pyx_cur_scope->__pyx_t_10;
                  __pyx_cur_scope->__pyx_t_10 = 0;
                  __Pyx_XGOTREF(__pyx_t_21);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 410, __pyx_L27_except_error)
                  __pyx_t_4 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_4);
                } else if (likely(__pyx_t_17 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_4 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 410, __pyx_L27_except_error)
                }
                __pyx_t_21 = __pyx_t_4;
                __pyx_t_4 = 0;
                __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_t_21);
                __Pyx_DECREF(__pyx_t_21); __pyx_t_21 = 0;
                if (__pyx_t_12 < (0)) __PYX_ERR(0, 410, __pyx_L27_except_error)
                __pyx_t_22 = (!__pyx_t_12);
                if (unlikely(__pyx_t_22)) {
                  __Pyx_GIVEREF(__pyx_t_1);
                  __Pyx_GIVEREF(__pyx_t_8);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __Pyx_ErrRestoreWithState(__pyx_t_1, __pyx_t_8, __pyx_t_5);
                  __pyx_t_1 = 0;  __pyx_t_8 = 0;  __pyx_t_5 = 0; 
                  __PYX_ERR(0, 410, __pyx_L27_except_error)
                }
                __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
                goto __pyx_L26_exception_handled;
              }
              __pyx_L27_except_error:;
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
              goto __pyx_L10_error;
              __pyx_L30_try_break:;
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
              goto __pyx_L21_break;
              __pyx_L26_exception_handled:;
              __Pyx_XGIVEREF(__pyx_t_18);
              __Pyx_XGIVEREF(__pyx_t_19);
              __Pyx_XGIVEREF(__pyx_t_20);
              __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_19, __pyx_t_20);
            }
          }
          /*finally:*/ {
            /*normal exit:*/{
              if (__pyx_t_16) {
                __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_16, __pyx_mstate_global->__pyx_tuple[0], NULL);
                __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 410, __pyx_L10_error)
                __Pyx_GOTREF(__pyx_t_20);
                __pyx_t_17 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_20, &__pyx_r);
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                if (likely(__pyx_t_17 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_t_16);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_16;
                  __Pyx_XGIVEREF(__pyx_t_20);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_20;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 4;
                  return __pyx_r;
                  __pyx_L38_resume_from_await:;
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  __pyx_t_16 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_16);
                  __pyx_t_20 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_20);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 410, __pyx_L10_error)
                  __pyx_t_5 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_5);
                } else if (likely(__pyx_t_17 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_5 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 410, __pyx_L10_error)
                }
                __pyx_t_20 = __pyx_t_5;
                __pyx_t_5 = 0;
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
              }
              goto __pyx_L24;
            }
            __pyx_L21_break: {
              if (__pyx_t_16) {
                __pyx_t_20 = __Pyx_PyObject_Call(__pyx_t_16, __pyx_mstate_global->__pyx_tuple[0], NULL);
                __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
                if (unlikely(!__pyx_t_20)) __PYX_ERR(0, 410, __pyx_L10_error)
                __Pyx_GOTREF(__pyx_t_20);
                __pyx_t_17 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_20, &__pyx_r);
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
                if (likely(__pyx_t_17 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_t_16);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_16;
                  __Pyx_XGIVEREF(__pyx_t_20);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_20;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 5;
                  return __pyx_r;
                  __pyx_L39_resume_from_await:;
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  __pyx_t_16 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_16);
                  __pyx_t_20 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_20);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 410, __pyx_L10_error)
                  __pyx_t_5 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_5);
                } else if (likely(__pyx_t_17 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_5 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 410, __pyx_L10_error)
                }
                __pyx_t_20 = __pyx_t_5;
                __pyx_t_5 = 0;
                __Pyx_DECREF(__pyx_t_20); __pyx_t_20 = 0;
              }
              goto __pyx_L15_try_break;
            }
            __pyx_L24:;
          }
          goto __pyx_L40;
          __pyx_L18_error:;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          goto __pyx_L10_error;
          __pyx_L40:;
        }

        /* "aioesphomeapi/connection.py":409
 *         interleave = 1
 *         while addr_infos:
 *             try:             # <<<<<<<<<<<<<<
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):
 *                     # Devices are likely on the local network so we
*/
      }
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      goto __pyx_L17_try_end;
      __pyx_L10_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "aioesphomeapi/connection.py":420
 *                     )
 *                     break
 *             except (OSError, TimeoutError) as err:             # <<<<<<<<<<<<<<
 *                 last_exception = err
 *                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)
*/
      __pyx_t_23 = __Pyx_PyErr_ExceptionMatches2(((PyObject *)(((PyTypeObject*)PyExc_OSError))), ((PyObject *)(((PyTypeObject*)PyExc_TimeoutError))));
      if (__pyx_t_23) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_5, &__pyx_t_8, &__pyx_t_1) < 0) __PYX_ERR(0, 420, __pyx_L12_except_error)
        __Pyx_XGOTREF(__pyx_t_5);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_8);
        __pyx_cur_scope->__pyx_v_err = __pyx_t_8;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":421
 *                     break
 *             except (OSError, TimeoutError) as err:
 *                 last_exception = err             # <<<<<<<<<<<<<<
 *                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)
 * 
*/
          __pyx_t_4 = __pyx_cur_scope->__pyx_v_err;
          __Pyx_INCREF(__pyx_t_4);
          if (!(likely(__Pyx_PyException_Check(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_4))) __PYX_ERR(0, 421, __pyx_L46_error)
          __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_last_exception);
          __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_last_exception, ((PyObject*)__pyx_t_4));
          __Pyx_GIVEREF(__pyx_t_4);
          __pyx_t_4 = 0;

          /* "aioesphomeapi/connection.py":422
 *             except (OSError, TimeoutError) as err:
 *                 last_exception = err
 *                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)             # <<<<<<<<<<<<<<
 * 
 *         if sock is None:
*/
          __pyx_t_10 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aiohappyeyeballs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L46_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_pop_addr_infos_interleave); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 422, __pyx_L46_error)
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = __Pyx_PyLong_From_long(__pyx_cur_scope->__pyx_v_interleave); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L46_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_6 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_10);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_10);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_6 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_10, __pyx_cur_scope->__pyx_v_addr_infos, __pyx_t_2};
            __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 422, __pyx_L46_error)
            __Pyx_GOTREF(__pyx_t_4);
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        }

        /* "aioesphomeapi/connection.py":420
 *                     )
 *                     break
 *             except (OSError, TimeoutError) as err:             # <<<<<<<<<<<<<<
 *                 last_exception = err
 *                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_err);
            __Pyx_DECREF(__pyx_cur_scope->__pyx_v_err); __pyx_cur_scope->__pyx_v_err = 0;
            goto __pyx_L47;
          }
          __pyx_L46_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_16 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_21 = 0; __pyx_t_26 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_18, &__pyx_t_21, &__pyx_t_26);
            if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_20, &__pyx_t_19) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_20, &__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_21);
            __Pyx_XGOTREF(__pyx_t_26);
            __pyx_t_23 = __pyx_lineno; __pyx_t_24 = __pyx_clineno; __pyx_t_25 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_err);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_err); __pyx_cur_scope->__pyx_v_err = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_26);
            __Pyx_ExceptionReset(__pyx_t_18, __pyx_t_21, __pyx_t_26);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_ErrRestore(__pyx_t_16, __pyx_t_20, __pyx_t_19);
            __pyx_t_16 = 0; __pyx_t_20 = 0; __pyx_t_19 = 0; __pyx_t_18 = 0; __pyx_t_21 = 0; __pyx_t_26 = 0;
            __pyx_lineno = __pyx_t_23; __pyx_clineno = __pyx_t_24; __pyx_filename = __pyx_t_25;
            goto __pyx_L12_except_error;
          }
          __pyx_L47:;
        }
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L11_exception_handled;
      }
      goto __pyx_L12_except_error;

      /* "aioesphomeapi/connection.py":409
 *         interleave = 1
 *         while addr_infos:
 *             try:             # <<<<<<<<<<<<<<
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):
 *                     # Devices are likely on the local network so we
*/
      __pyx_L12_except_error:;
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      goto __pyx_L1_error;
      __pyx_L15_try_break:;
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      goto __pyx_L9_break;
      __pyx_L11_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_ExceptionReset(__pyx_t_13, __pyx_t_14, __pyx_t_15);
      __pyx_L17_try_end:;
    }
  }
  __pyx_L9_break:;

  /* "aioesphomeapi/connection.py":424
 *                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)
 * 
 *         if sock is None:             # <<<<<<<<<<<<<<
 *             if isinstance(last_exception, TimeoutError):
 *                 raise TimeoutAPIError(
*/
  __pyx_t_22 = (__pyx_cur_scope->__pyx_v_sock == Py_None);
  if (__pyx_t_22) {

    /* "aioesphomeapi/connection.py":425
 * 
 *         if sock is None:
 *             if isinstance(last_exception, TimeoutError):             # <<<<<<<<<<<<<<
 *                 raise TimeoutAPIError(
 *                     f"Timeout while connecting to {addrs}"
*/
    __pyx_t_22 = __Pyx_PyExc_TimeoutError_Check(__pyx_cur_scope->__pyx_v_last_exception); 
    if (unlikely(__pyx_t_22)) {

      /* "aioesphomeapi/connection.py":426
 *         if sock is None:
 *             if isinstance(last_exception, TimeoutError):
 *                 raise TimeoutAPIError(             # <<<<<<<<<<<<<<
 *                     f"Timeout while connecting to {addrs}"
 *                 ) from last_exception
*/
      __pyx_t_8 = NULL;
      __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError);
      __pyx_t_5 = __pyx_v_13aioesphomeapi_10connection_TimeoutAPIError; 

      /* "aioesphomeapi/connection.py":427
 *             if isinstance(last_exception, TimeoutError):
 *                 raise TimeoutAPIError(
 *                     f"Timeout while connecting to {addrs}"             # <<<<<<<<<<<<<<
 *                 ) from last_exception
 *             raise SocketAPIError(
*/
      __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_addrs, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Timeout_while_connecting_to, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 427, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_8);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_3};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 426, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }

      /* "aioesphomeapi/connection.py":428
 *                 raise TimeoutAPIError(
 *                     f"Timeout while connecting to {addrs}"
 *                 ) from last_exception             # <<<<<<<<<<<<<<
 *             raise SocketAPIError(
 *                 f"Error connecting to {addrs}: {last_exception}"
*/
      __Pyx_Raise(__pyx_t_1, 0, 0, __pyx_cur_scope->__pyx_v_last_exception);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 426, __pyx_L1_error)

      /* "aioesphomeapi/connection.py":425
 * 
 *         if sock is None:
 *             if isinstance(last_exception, TimeoutError):             # <<<<<<<<<<<<<<
 *                 raise TimeoutAPIError(
 *                     f"Timeout while connecting to {addrs}"
*/
    }

    /* "aioesphomeapi/connection.py":429
 *                     f"Timeout while connecting to {addrs}"
 *                 ) from last_exception
 *             raise SocketAPIError(             # <<<<<<<<<<<<<<
 *                 f"Error connecting to {addrs}: {last_exception}"
 *             ) from last_exception
*/
    __pyx_t_5 = NULL;
    __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_SocketAPIError);
    __pyx_t_3 = __pyx_v_13aioesphomeapi_10connection_SocketAPIError; 

    /* "aioesphomeapi/connection.py":430
 *                 ) from last_exception
 *             raise SocketAPIError(
 *                 f"Error connecting to {addrs}: {last_exception}"             # <<<<<<<<<<<<<<
 *             ) from last_exception
 * 
*/
    __pyx_t_8 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_addrs, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_4 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_last_exception, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_27[0] = __pyx_mstate_global->__pyx_kp_u_Error_connecting_to;
    __pyx_t_27[1] = __pyx_t_8;
    __pyx_t_27[2] = __pyx_mstate_global->__pyx_kp_u__6;
    __pyx_t_27[3] = __pyx_t_4;
    __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_27, 4, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_8) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_8) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_4));
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 430, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_2};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 429, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }

    /* "aioesphomeapi/connection.py":431
 *             raise SocketAPIError(
 *                 f"Error connecting to {addrs}: {last_exception}"
 *             ) from last_exception             # <<<<<<<<<<<<<<
 * 
 *         self._socket = sock
*/
    __Pyx_Raise(__pyx_t_1, 0, 0, __pyx_cur_scope->__pyx_v_last_exception);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 429, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":424
 *                 aiohappyeyeballs.pop_addr_infos_interleave(addr_infos, interleave)
 * 
 *         if sock is None:             # <<<<<<<<<<<<<<
 *             if isinstance(last_exception, TimeoutError):
 *                 raise TimeoutAPIError(
*/
  }

  /* "aioesphomeapi/connection.py":433
 *             ) from last_exception
 * 
 *         self._socket = sock             # <<<<<<<<<<<<<<
 *         sock.setblocking(False)
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
*/
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_sock);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_sock);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_socket);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_socket);
  __pyx_cur_scope->__pyx_v_self->_socket = __pyx_cur_scope->__pyx_v_sock;

  /* "aioesphomeapi/connection.py":434
 * 
 *         self._socket = sock
 *         sock.setblocking(False)             # <<<<<<<<<<<<<<
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 *         try:
*/
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_sock;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, Py_False};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_setblocking, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 434, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":435
 *         self._socket = sock
 *         sock.setblocking(False)
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)             # <<<<<<<<<<<<<<
 *         try:
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]
*/
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_sock;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_socket); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_IPPROTO_TCP); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_socket); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_TCP_NODELAY); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_t_5, __pyx_t_4, __pyx_mstate_global->__pyx_int_1};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_setsockopt, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":436
 *         sock.setblocking(False)
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 *         try:             # <<<<<<<<<<<<<<
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]
 *         except (AttributeError, OSError):  # On FreeBSD this may throw OSError
*/
  {
    __Pyx_ExceptionSave(&__pyx_t_15, &__pyx_t_14, &__pyx_t_13);
    __Pyx_XGOTREF(__pyx_t_15);
    __Pyx_XGOTREF(__pyx_t_14);
    __Pyx_XGOTREF(__pyx_t_13);
    /*try:*/ {

      /* "aioesphomeapi/connection.py":437
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 *         try:
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]             # <<<<<<<<<<<<<<
 *         except (AttributeError, OSError):  # On FreeBSD this may throw OSError
 *             _LOGGER.debug(
*/
      __pyx_t_4 = __pyx_cur_scope->__pyx_v_sock;
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_socket); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 437, __pyx_L54_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_IPPROTO_TCP); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 437, __pyx_L54_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_socket); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 437, __pyx_L54_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_TCP_QUICKACK); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L54_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_6 = 0;
      {
        PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_t_3, __pyx_t_2, __pyx_mstate_global->__pyx_int_1};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_setsockopt, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L54_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":436
 *         sock.setblocking(False)
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 *         try:             # <<<<<<<<<<<<<<
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]
 *         except (AttributeError, OSError):  # On FreeBSD this may throw OSError
*/
    }
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    goto __pyx_L59_try_end;
    __pyx_L54_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "aioesphomeapi/connection.py":438
 *         try:
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]
 *         except (AttributeError, OSError):  # On FreeBSD this may throw OSError             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: TCP_QUICKACK not supported",
*/
    __pyx_t_24 = __Pyx_PyErr_ExceptionMatches2(((PyObject *)(((PyTypeObject*)PyExc_AttributeError))), ((PyObject *)(((PyTypeObject*)PyExc_OSError))));
    if (__pyx_t_24) {
      __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_2, &__pyx_t_3) < 0) __PYX_ERR(0, 438, __pyx_L56_except_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_3);

      /* "aioesphomeapi/connection.py":439
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]
 *         except (AttributeError, OSError):  # On FreeBSD this may throw OSError
 *             _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                 "%s: TCP_QUICKACK not supported",
 *                 self.log_name,
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 439, __pyx_L56_except_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 439, __pyx_L56_except_error)
      __Pyx_GOTREF(__pyx_t_10);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "aioesphomeapi/connection.py":441
 *             _LOGGER.debug(
 *                 "%s: TCP_QUICKACK not supported",
 *                 self.log_name,             # <<<<<<<<<<<<<<
 *             )
 *         self._increase_recv_buffer_size()
*/
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_10))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_10);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_s_TCP_QUICKACK_not_supported, __pyx_cur_scope->__pyx_v_self->log_name};
        __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 439, __pyx_L56_except_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L55_exception_handled;
    }
    goto __pyx_L56_except_error;

    /* "aioesphomeapi/connection.py":436
 *         sock.setblocking(False)
 *         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 *         try:             # <<<<<<<<<<<<<<
 *             sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_QUICKACK, 1)  # type: ignore[attr-defined, unused-ignore]
 *         except (AttributeError, OSError):  # On FreeBSD this may throw OSError
*/
    __pyx_L56_except_error:;
    __Pyx_XGIVEREF(__pyx_t_15);
    __Pyx_XGIVEREF(__pyx_t_14);
    __Pyx_XGIVEREF(__pyx_t_13);
    __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_14, __pyx_t_13);
    goto __pyx_L1_error;
    __pyx_L55_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_15);
    __Pyx_XGIVEREF(__pyx_t_14);
    __Pyx_XGIVEREF(__pyx_t_13);
    __Pyx_ExceptionReset(__pyx_t_15, __pyx_t_14, __pyx_t_13);
    __pyx_L59_try_end:;
  }

  /* "aioesphomeapi/connection.py":443
 *                 self.log_name,
 *             )
 *         self._increase_recv_buffer_size()             # <<<<<<<<<<<<<<
 *         self.connected_address = sock.getpeername()[0]
 * 
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_increase_recv_buffer_size(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 443, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":444
 *             )
 *         self._increase_recv_buffer_size()
 *         self.connected_address = sock.getpeername()[0]             # <<<<<<<<<<<<<<
 * 
 *         if self._debug_enabled:
*/
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_sock;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_6 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_getpeername, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 444, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __Pyx_GetItemInt(__pyx_t_3, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(PyUnicode_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_2))) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->connected_address);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->connected_address);
  __pyx_cur_scope->__pyx_v_self->connected_address = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":446
 *         self.connected_address = sock.getpeername()[0]
 * 
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Opened socket to %s:%s",
*/
  if (__pyx_cur_scope->__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":447
 * 
 *         if self._debug_enabled:
 *             _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                 "%s: Opened socket to %s:%s",
 *                 self.log_name,
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 447, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 447, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":451
 *                 self.log_name,
 *                 self.connected_address,
 *                 self._params.port,             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[5] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_s_Opened_socket_to_s_s, __pyx_cur_scope->__pyx_v_self->log_name, __pyx_cur_scope->__pyx_v_self->connected_address, __pyx_cur_scope->__pyx_v_self->_params->port};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (5-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 447, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":446
 *         self.connected_address = sock.getpeername()[0]
 * 
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Opened socket to %s:%s",
*/
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":386
 *         self._debug_enabled = enable
 * 
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:             # <<<<<<<<<<<<<<
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("_connect_socket_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":454
 *             )
 * 
 *     def _increase_recv_buffer_size(self) -> None:             # <<<<<<<<<<<<<<
 *         """Increase the recv buffer size."""
 *         if TYPE_CHECKING:
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__increase_recv_buffer_size(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_v_new_buffer_size = NULL;
  PyObject *__pyx_v_err = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  size_t __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_increase_recv_buffer_size", 0);

  /* "aioesphomeapi/connection.py":456
 *     def _increase_recv_buffer_size(self) -> None:
 *         """Increase the recv buffer size."""
 *         if TYPE_CHECKING:             # <<<<<<<<<<<<<<
 *             assert self._socket is not None
 *         new_buffer_size = PREFERRED_BUFFER_SIZE
*/
  if (__pyx_v_13aioesphomeapi_10connection_TYPE_CHECKING) {

    /* "aioesphomeapi/connection.py":457
 *         """Increase the recv buffer size."""
 *         if TYPE_CHECKING:
 *             assert self._socket is not None             # <<<<<<<<<<<<<<
 *         new_buffer_size = PREFERRED_BUFFER_SIZE
 *         while True:
*/
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = (__pyx_v_self->_socket != Py_None);
      if (unlikely(!__pyx_t_1)) {
        __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
        __PYX_ERR(0, 457, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(0, 457, __pyx_L1_error)
    #endif

    /* "aioesphomeapi/connection.py":456
 *     def _increase_recv_buffer_size(self) -> None:
 *         """Increase the recv buffer size."""
 *         if TYPE_CHECKING:             # <<<<<<<<<<<<<<
 *             assert self._socket is not None
 *         new_buffer_size = PREFERRED_BUFFER_SIZE
*/
  }

  /* "aioesphomeapi/connection.py":458
 *         if TYPE_CHECKING:
 *             assert self._socket is not None
 *         new_buffer_size = PREFERRED_BUFFER_SIZE             # <<<<<<<<<<<<<<
 *         while True:
 *             # Try to reduce the pressure on ESPHome device as it measures
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_PREFERRED_BUFFER_SIZE); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_new_buffer_size = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":459
 *             assert self._socket is not None
 *         new_buffer_size = PREFERRED_BUFFER_SIZE
 *         while True:             # <<<<<<<<<<<<<<
 *             # Try to reduce the pressure on ESPHome device as it measures
 *             # ram in bytes and we measure ram in megabytes.
*/
  while (1) {

    /* "aioesphomeapi/connection.py":462
 *             # Try to reduce the pressure on ESPHome device as it measures
 *             # ram in bytes and we measure ram in megabytes.
 *             try:             # <<<<<<<<<<<<<<
 *                 self._socket.setsockopt(
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":463
 *             # ram in bytes and we measure ram in megabytes.
 *             try:
 *                 self._socket.setsockopt(             # <<<<<<<<<<<<<<
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size
 *                 )
*/
        __pyx_t_6 = __pyx_v_self->_socket;
        __Pyx_INCREF(__pyx_t_6);

        /* "aioesphomeapi/connection.py":464
 *             try:
 *                 self._socket.setsockopt(
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size             # <<<<<<<<<<<<<<
 *                 )
 *             except OSError as err:  # noqa: PERF203
*/
        __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_socket); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 464, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_SOL_SOCKET); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 464, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_socket); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 464, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_7);
        __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_SO_RCVBUF); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 464, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_9);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_10 = 0;
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_6, __pyx_t_8, __pyx_t_9, __pyx_v_new_buffer_size};
          __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_setsockopt, __pyx_callargs+__pyx_t_10, (4-__pyx_t_10) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 463, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "aioesphomeapi/connection.py":462
 *             # Try to reduce the pressure on ESPHome device as it measures
 *             # ram in bytes and we measure ram in megabytes.
 *             try:             # <<<<<<<<<<<<<<
 *                 self._socket.setsockopt(
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size
*/
      }

      /* "aioesphomeapi/connection.py":479
 *                 new_buffer_size //= 2
 *             else:
 *                 return             # <<<<<<<<<<<<<<
 * 
 *     async def _connect_init_frame_helper(self) -> None:
*/
      /*else:*/ {
        goto __pyx_L9_except_return;
      }
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "aioesphomeapi/connection.py":466
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size
 *                 )
 *             except OSError as err:  # noqa: PERF203             # <<<<<<<<<<<<<<
 *                 if new_buffer_size <= MIN_BUFFER_SIZE:
 *                     _LOGGER.warning(
*/
      __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_OSError))));
      if (__pyx_t_11) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._increase_recv_buffer_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_9, &__pyx_t_8) < 0) __PYX_ERR(0, 466, __pyx_L8_except_error)
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __pyx_v_err = __pyx_t_9;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":467
 *                 )
 *             except OSError as err:  # noqa: PERF203
 *                 if new_buffer_size <= MIN_BUFFER_SIZE:             # <<<<<<<<<<<<<<
 *                     _LOGGER.warning(
 *                         "%s: Unable to increase the socket receive buffer size to %s; "
*/
          __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_MIN_BUFFER_SIZE); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 467, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_7 = PyObject_RichCompare(__pyx_v_new_buffer_size, __pyx_t_6, Py_LE); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 467, __pyx_L19_error)
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 467, __pyx_L19_error)
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
          if (__pyx_t_1) {

            /* "aioesphomeapi/connection.py":468
 *             except OSError as err:  # noqa: PERF203
 *                 if new_buffer_size <= MIN_BUFFER_SIZE:
 *                     _LOGGER.warning(             # <<<<<<<<<<<<<<
 *                         "%s: Unable to increase the socket receive buffer size to %s; "
 *                         "The connection may unstable if the ESPHome device sends "
*/
            __pyx_t_6 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 468, __pyx_L19_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_warning); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 468, __pyx_L19_error)
            __Pyx_GOTREF(__pyx_t_13);
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

            /* "aioesphomeapi/connection.py":474
 *                         self.log_name,
 *                         new_buffer_size,
 *                         err,             # <<<<<<<<<<<<<<
 *                     )
 *                     return
*/
            __pyx_t_10 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_13))) {
              __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_13);
              assert(__pyx_t_6);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
              __Pyx_INCREF(__pyx_t_6);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
              __pyx_t_10 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[5] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_s_Unable_to_increase_the_socket, __pyx_v_self->log_name, __pyx_v_new_buffer_size, __pyx_v_err};
              __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_10, (5-__pyx_t_10) | (__pyx_t_10*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 468, __pyx_L19_error)
              __Pyx_GOTREF(__pyx_t_7);
            }
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

            /* "aioesphomeapi/connection.py":476
 *                         err,
 *                     )
 *                     return             # <<<<<<<<<<<<<<
 *                 new_buffer_size //= 2
 *             else:
*/
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            goto __pyx_L18_return;

            /* "aioesphomeapi/connection.py":467
 *                 )
 *             except OSError as err:  # noqa: PERF203
 *                 if new_buffer_size <= MIN_BUFFER_SIZE:             # <<<<<<<<<<<<<<
 *                     _LOGGER.warning(
 *                         "%s: Unable to increase the socket receive buffer size to %s; "
*/
          }

          /* "aioesphomeapi/connection.py":477
 *                     )
 *                     return
 *                 new_buffer_size //= 2             # <<<<<<<<<<<<<<
 *             else:
 *                 return
*/
          __pyx_t_7 = __Pyx_PyLong_FloorDivideObjC(__pyx_v_new_buffer_size, __pyx_mstate_global->__pyx_int_2, 2, 1, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 477, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_DECREF_SET(__pyx_v_new_buffer_size, __pyx_t_7);
          __pyx_t_7 = 0;
        }

        /* "aioesphomeapi/connection.py":466
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size
 *                 )
 *             except OSError as err:  # noqa: PERF203             # <<<<<<<<<<<<<<
 *                 if new_buffer_size <= MIN_BUFFER_SIZE:
 *                     _LOGGER.warning(
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_DECREF(__pyx_v_err); __pyx_v_err = 0;
            goto __pyx_L20;
          }
          __pyx_L19_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_declare
            __Pyx_PyThreadState_assign
            __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
            if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_16);
            __Pyx_XGOTREF(__pyx_t_17);
            __Pyx_XGOTREF(__pyx_t_18);
            __Pyx_XGOTREF(__pyx_t_19);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_21);
            __pyx_t_11 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
            {
              __Pyx_DECREF(__pyx_v_err); __pyx_v_err = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_19);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_16);
            __Pyx_XGIVEREF(__pyx_t_17);
            __Pyx_XGIVEREF(__pyx_t_18);
            __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
            __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
            __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
            goto __pyx_L8_except_error;
          }
          __pyx_L18_return: {
            __Pyx_DECREF(__pyx_v_err); __pyx_v_err = 0;
            goto __pyx_L9_except_return;
          }
          __pyx_L20:;
        }
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L7_exception_handled;
      }
      goto __pyx_L8_except_error;

      /* "aioesphomeapi/connection.py":462
 *             # Try to reduce the pressure on ESPHome device as it measures
 *             # ram in bytes and we measure ram in megabytes.
 *             try:             # <<<<<<<<<<<<<<
 *                 self._socket.setsockopt(
 *                     socket.SOL_SOCKET, socket.SO_RCVBUF, new_buffer_size
*/
      __pyx_L8_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L9_except_return:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L0;
      __pyx_L7_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    }
  }

  /* "aioesphomeapi/connection.py":454
 *             )
 * 
 *     def _increase_recv_buffer_size(self) -> None:             # <<<<<<<<<<<<<<
 *         """Increase the recv buffer size."""
 *         if TYPE_CHECKING:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._increase_recv_buffer_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_new_buffer_size);
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_RefNannyFinishContext();
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_11generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":481
 *                 return
 * 
 *     async def _connect_init_frame_helper(self) -> None:             # <<<<<<<<<<<<<<
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_10_connect_init_frame_helper(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_9_connect_init_frame_helper, "Step 3 in connect process: initialize the frame helper and init read loop.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_10_connect_init_frame_helper = {"_connect_init_frame_helper", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_10_connect_init_frame_helper, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_9_connect_init_frame_helper};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_10_connect_init_frame_helper(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_connect_init_frame_helper (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_connect_init_frame_helper", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_connect_init_frame_helper", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_9_connect_init_frame_helper(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":489
 *         if (noise_psk := self._params.noise_psk) is None:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APIPlaintextFrameHelper(             # <<<<<<<<<<<<<<
 *                     connection=self,
 *                     client_info=self._params.client_info,
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_lambda2(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_lambda2 = {"lambda2", (PyCFunction)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_lambda2, METH_NOARGS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_lambda2(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda2 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_lambda_funcdef_lambda2(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda2(PyObject *__pyx_self) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_cur_scope;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda2", 0);
  __pyx_outer_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_APIPlaintextFrameHelper); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 489, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "aioesphomeapi/connection.py":490
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APIPlaintextFrameHelper(
 *                     connection=self,             # <<<<<<<<<<<<<<
 *                     client_info=self._params.client_info,
 *                     log_name=self.log_name,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 490, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":491
 *                 lambda: APIPlaintextFrameHelper(
 *                     connection=self,
 *                     client_info=self._params.client_info,             # <<<<<<<<<<<<<<
 *                     log_name=self.log_name,
 *                 ),
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 491, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":492
 *                     connection=self,
 *                     client_info=self._params.client_info,
 *                     log_name=self.log_name,             # <<<<<<<<<<<<<<
 *                 ),
 *                 sock=self._socket,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 492, __pyx_L1_error) }
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_connection, ((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_t_5, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 489, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_client_info, __pyx_cur_scope->__pyx_v_self->_params->client_info, __pyx_t_5, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 489, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_log_name, __pyx_cur_scope->__pyx_v_self->log_name, __pyx_t_5, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 489, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":489
 *         if (noise_psk := self._params.noise_psk) is None:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APIPlaintextFrameHelper(             # <<<<<<<<<<<<<<
 *                     connection=self,
 *                     client_info=self._params.client_info,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_init_frame_helper.lambda2", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":498
 *         else:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APINoiseFrameHelper(             # <<<<<<<<<<<<<<
 *                     noise_psk=noise_psk,
 *                     expected_name=self._params.expected_name,
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_1lambda3(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_1lambda3 = {"lambda3", (PyCFunction)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_1lambda3, METH_NOARGS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_1lambda3(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda3 (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_lambda_funcdef_lambda3(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda3(PyObject *__pyx_self) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_cur_scope;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda3", 0);
  __pyx_outer_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_APINoiseFrameHelper); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 498, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "aioesphomeapi/connection.py":499
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APINoiseFrameHelper(
 *                     noise_psk=noise_psk,             # <<<<<<<<<<<<<<
 *                     expected_name=self._params.expected_name,
 *                     expected_mac=self._params.expected_mac,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_noise_psk)) { __Pyx_RaiseClosureNameError("noise_psk"); __PYX_ERR(0, 499, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":500
 *                 lambda: APINoiseFrameHelper(
 *                     noise_psk=noise_psk,
 *                     expected_name=self._params.expected_name,             # <<<<<<<<<<<<<<
 *                     expected_mac=self._params.expected_mac,
 *                     connection=self,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 500, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":501
 *                     noise_psk=noise_psk,
 *                     expected_name=self._params.expected_name,
 *                     expected_mac=self._params.expected_mac,             # <<<<<<<<<<<<<<
 *                     connection=self,
 *                     client_info=self._params.client_info,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 501, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":502
 *                     expected_name=self._params.expected_name,
 *                     expected_mac=self._params.expected_mac,
 *                     connection=self,             # <<<<<<<<<<<<<<
 *                     client_info=self._params.client_info,
 *                     log_name=self.log_name,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 502, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":503
 *                     expected_mac=self._params.expected_mac,
 *                     connection=self,
 *                     client_info=self._params.client_info,             # <<<<<<<<<<<<<<
 *                     log_name=self.log_name,
 *                 ),
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 503, __pyx_L1_error) }

  /* "aioesphomeapi/connection.py":504
 *                     connection=self,
 *                     client_info=self._params.client_info,
 *                     log_name=self.log_name,             # <<<<<<<<<<<<<<
 *                 ),
 *                 sock=self._socket,
*/
  if (unlikely(!__pyx_cur_scope->__pyx_v_self)) { __Pyx_RaiseClosureNameError("self"); __PYX_ERR(0, 504, __pyx_L1_error) }
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 6 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 498, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_noise_psk, __pyx_cur_scope->__pyx_v_noise_psk, __pyx_t_5, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 498, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_expected_name, __pyx_cur_scope->__pyx_v_self->_params->expected_name, __pyx_t_5, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 498, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_expected_mac, __pyx_cur_scope->__pyx_v_self->_params->expected_mac, __pyx_t_5, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 498, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_connection, ((PyObject *)__pyx_cur_scope->__pyx_v_self), __pyx_t_5, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 498, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_client_info, __pyx_cur_scope->__pyx_v_self->_params->client_info, __pyx_t_5, __pyx_callargs+1, 4) < (0)) __PYX_ERR(0, 498, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_log_name, __pyx_cur_scope->__pyx_v_self->log_name, __pyx_t_5, __pyx_callargs+1, 5) < (0)) __PYX_ERR(0, 498, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 498, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":498
 *         else:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APINoiseFrameHelper(             # <<<<<<<<<<<<<<
 *                     noise_psk=noise_psk,
 *                     expected_name=self._params.expected_name,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_init_frame_helper.lambda3", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":481
 *                 return
 * 
 *     async def _connect_init_frame_helper(self) -> None:             # <<<<<<<<<<<<<<
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
*/

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_9_connect_init_frame_helper(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_connect_init_frame_helper", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 481, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_11generator1, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_connect_init_frame_helper, __pyx_mstate_global->__pyx_n_u_APIConnection__connect_init_fram, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 481, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_init_frame_helper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_11generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_PySendResult __pyx_t_7;
  PyObject *(*__pyx_t_8)(PyObject *);
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16[3];
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  char const *__pyx_t_26;
  char const *__pyx_t_27;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_connect_init_frame_helper", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L6_resume_from_await;
    case 2: goto __pyx_L9_resume_from_await;
    case 3: goto __pyx_L21_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 481, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":484
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
 *         if TYPE_CHECKING:             # <<<<<<<<<<<<<<
 *             assert self._socket is not None
 * 
*/
  if (__pyx_v_13aioesphomeapi_10connection_TYPE_CHECKING) {

    /* "aioesphomeapi/connection.py":485
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
 *         if TYPE_CHECKING:
 *             assert self._socket is not None             # <<<<<<<<<<<<<<
 * 
 *         if (noise_psk := self._params.noise_psk) is None:
*/
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = (__pyx_cur_scope->__pyx_v_self->_socket != Py_None);
      if (unlikely(!__pyx_t_1)) {
        __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
        __PYX_ERR(0, 485, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(0, 485, __pyx_L1_error)
    #endif

    /* "aioesphomeapi/connection.py":484
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
 *         if TYPE_CHECKING:             # <<<<<<<<<<<<<<
 *             assert self._socket is not None
 * 
*/
  }

  /* "aioesphomeapi/connection.py":487
 *             assert self._socket is not None
 * 
 *         if (noise_psk := self._params.noise_psk) is None:             # <<<<<<<<<<<<<<
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APIPlaintextFrameHelper(
*/
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_self->_params->noise_psk;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_noise_psk = __pyx_t_2;
  __pyx_t_1 = (__pyx_t_2 == Py_None);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":488
 * 
 *         if (noise_psk := self._params.noise_psk) is None:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]             # <<<<<<<<<<<<<<
 *                 lambda: APIPlaintextFrameHelper(
 *                     connection=self,
*/
    __pyx_t_3 = __pyx_cur_scope->__pyx_v_self->_loop;
    __Pyx_INCREF(__pyx_t_3);

    /* "aioesphomeapi/connection.py":489
 *         if (noise_psk := self._params.noise_psk) is None:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APIPlaintextFrameHelper(             # <<<<<<<<<<<<<<
 *                     connection=self,
 *                     client_info=self._params.client_info,
*/
    __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_lambda2, 0, __pyx_mstate_global->__pyx_n_u_connect_init_frame_helper_local, ((PyObject*)__pyx_cur_scope), __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "aioesphomeapi/connection.py":494
 *                     log_name=self.log_name,
 *                 ),
 *                 sock=self._socket,             # <<<<<<<<<<<<<<
 *             )
 *         else:
*/
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_sock, __pyx_cur_scope->__pyx_v_self->_socket, __pyx_t_6, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 488, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_create_connection, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_7 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (likely(__pyx_t_7 == PYGEN_NEXT)) {
      __Pyx_GOTREF(__pyx_r);
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, awaiting value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L6_resume_from_await:;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 488, __pyx_L1_error)
      __pyx_t_2 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_2);
    } else if (likely(__pyx_t_7 == PYGEN_RETURN)) {
      __Pyx_GOTREF(__pyx_r);
      __pyx_t_2 = __pyx_r; __pyx_r = NULL;
    } else {
      __Pyx_XGOTREF(__pyx_r);
      __PYX_ERR(0, 488, __pyx_L1_error)
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 488, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_6);
        __pyx_t_4 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_4);
      } else {
        __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 488, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_6);
        __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_4);
      }
      #else
      __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_3 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 488, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3);
      index = 0; __pyx_t_6 = __pyx_t_8(__pyx_t_3); if (unlikely(!__pyx_t_6)) goto __pyx_L7_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      index = 1; __pyx_t_4 = __pyx_t_8(__pyx_t_3); if (unlikely(!__pyx_t_4)) goto __pyx_L7_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_4);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_3), 2) < (0)) __PYX_ERR(0, 488, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      goto __pyx_L8_unpacking_done;
      __pyx_L7_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 488, __pyx_L1_error)
      __pyx_L8_unpacking_done:;
    }

    /* "aioesphomeapi/connection.py":488
 * 
 *         if (noise_psk := self._params.noise_psk) is None:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]             # <<<<<<<<<<<<<<
 *                 lambda: APIPlaintextFrameHelper(
 *                     connection=self,
*/
    __Pyx_GIVEREF(__pyx_t_6);
    __pyx_cur_scope->__pyx_v__ = __pyx_t_6;
    __pyx_t_6 = 0;
    __Pyx_GIVEREF(__pyx_t_4);
    __pyx_cur_scope->__pyx_v_fh = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":487
 *             assert self._socket is not None
 * 
 *         if (noise_psk := self._params.noise_psk) is None:             # <<<<<<<<<<<<<<
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APIPlaintextFrameHelper(
*/
    goto __pyx_L5;
  }

  /* "aioesphomeapi/connection.py":497
 *             )
 *         else:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]             # <<<<<<<<<<<<<<
 *                 lambda: APINoiseFrameHelper(
 *                     noise_psk=noise_psk,
*/
  /*else*/ {
    __pyx_t_4 = __pyx_cur_scope->__pyx_v_self->_loop;
    __Pyx_INCREF(__pyx_t_4);

    /* "aioesphomeapi/connection.py":498
 *         else:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]
 *                 lambda: APINoiseFrameHelper(             # <<<<<<<<<<<<<<
 *                     noise_psk=noise_psk,
 *                     expected_name=self._params.expected_name,
*/
    __pyx_t_6 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_26_connect_init_frame_helper_1lambda3, 0, __pyx_mstate_global->__pyx_n_u_connect_init_frame_helper_local, ((PyObject*)__pyx_cur_scope), __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 498, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);

    /* "aioesphomeapi/connection.py":506
 *                     log_name=self.log_name,
 *                 ),
 *                 sock=self._socket,             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, __pyx_t_6};
      __pyx_t_3 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_sock, __pyx_cur_scope->__pyx_v_self->_socket, __pyx_t_3, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 497, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_create_connection, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_3);
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_7 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (likely(__pyx_t_7 == PYGEN_NEXT)) {
      __Pyx_GOTREF(__pyx_r);
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, awaiting value */
      __pyx_generator->resume_label = 2;
      return __pyx_r;
      __pyx_L9_resume_from_await:;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 497, __pyx_L1_error)
      __pyx_t_2 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_2);
    } else if (likely(__pyx_t_7 == PYGEN_RETURN)) {
      __Pyx_GOTREF(__pyx_r);
      __pyx_t_2 = __pyx_r; __pyx_r = NULL;
    } else {
      __Pyx_XGOTREF(__pyx_r);
      __PYX_ERR(0, 497, __pyx_L1_error)
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 497, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_3);
        __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_6);
      } else {
        __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 497, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_3);
        __pyx_t_6 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 497, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_6);
      }
      #else
      __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_6 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_4 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 497, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_8 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_4);
      index = 0; __pyx_t_3 = __pyx_t_8(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_3);
      index = 1; __pyx_t_6 = __pyx_t_8(__pyx_t_4); if (unlikely(!__pyx_t_6)) goto __pyx_L10_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_8(__pyx_t_4), 2) < (0)) __PYX_ERR(0, 497, __pyx_L1_error)
      __pyx_t_8 = NULL;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      goto __pyx_L11_unpacking_done;
      __pyx_L10_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_8 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 497, __pyx_L1_error)
      __pyx_L11_unpacking_done:;
    }

    /* "aioesphomeapi/connection.py":497
 *             )
 *         else:
 *             _, fh = await self._loop.create_connection(  # type: ignore[type-var]             # <<<<<<<<<<<<<<
 *                 lambda: APINoiseFrameHelper(
 *                     noise_psk=noise_psk,
*/
    __Pyx_GIVEREF(__pyx_t_3);
    __pyx_cur_scope->__pyx_v__ = __pyx_t_3;
    __pyx_t_3 = 0;
    __Pyx_GIVEREF(__pyx_t_6);
    __pyx_cur_scope->__pyx_v_fh = __pyx_t_6;
    __pyx_t_6 = 0;
  }
  __pyx_L5:;

  /* "aioesphomeapi/connection.py":511
 *         # Set the frame helper right away to ensure
 *         # the socket gets closed if we fail to handshake
 *         self._frame_helper = fh             # <<<<<<<<<<<<<<
 *         handshake_handle = self._loop.call_at(
 *             self._loop.time() + HANDSHAKE_TIMEOUT,
*/
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_fh;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper))))) __PYX_ERR(0, 511, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF((PyObject *)__pyx_cur_scope->__pyx_v_self->_frame_helper);
  __Pyx_DECREF((PyObject *)__pyx_cur_scope->__pyx_v_self->_frame_helper);
  __pyx_cur_scope->__pyx_v_self->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":512
 *         # the socket gets closed if we fail to handshake
 *         self._frame_helper = fh
 *         handshake_handle = self._loop.call_at(             # <<<<<<<<<<<<<<
 *             self._loop.time() + HANDSHAKE_TIMEOUT,
 *             _handle_timeout,
*/
  __pyx_t_6 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_6);

  /* "aioesphomeapi/connection.py":513
 *         self._frame_helper = fh
 *         handshake_handle = self._loop.call_at(
 *             self._loop.time() + HANDSHAKE_TIMEOUT,             # <<<<<<<<<<<<<<
 *             _handle_timeout,
 *             self._frame_helper.ready_future,
*/
  __pyx_t_4 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_time, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 513, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_4 = PyNumber_Add(__pyx_t_3, __pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 513, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":515
 *             self._loop.time() + HANDSHAKE_TIMEOUT,
 *             _handle_timeout,
 *             self._frame_helper.ready_future,             # <<<<<<<<<<<<<<
 *         )
 *         try:
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_6, __pyx_t_4, __pyx_v_13aioesphomeapi_10connection__handle_timeout, __pyx_cur_scope->__pyx_v_self->_frame_helper->ready_future};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_call_at, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 512, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_handshake_handle = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":517
 *             self._frame_helper.ready_future,
 *         )
 *         try:             # <<<<<<<<<<<<<<
 *             await self._frame_helper.ready_future
 *         except TimeoutError as err:
*/
  /*try:*/ {
    {
      __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_11);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":518
 *         )
 *         try:
 *             await self._frame_helper.ready_future             # <<<<<<<<<<<<<<
 *         except TimeoutError as err:
 *             raise TimeoutAPIError(
*/
        __pyx_t_7 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_cur_scope->__pyx_v_self->_frame_helper->ready_future, &__pyx_r);
        if (likely(__pyx_t_7 == PYGEN_NEXT)) {
          __Pyx_GOTREF(__pyx_r);
          __Pyx_XGIVEREF(__pyx_t_9);
          __pyx_cur_scope->__pyx_t_0 = __pyx_t_9;
          __Pyx_XGIVEREF(__pyx_t_10);
          __pyx_cur_scope->__pyx_t_1 = __pyx_t_10;
          __Pyx_XGIVEREF(__pyx_t_11);
          __pyx_cur_scope->__pyx_t_2 = __pyx_t_11;
          __Pyx_XGIVEREF(__pyx_r);
          __Pyx_RefNannyFinishContext();
          __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
          /* return from generator, awaiting value */
          __pyx_generator->resume_label = 3;
          return __pyx_r;
          __pyx_L21_resume_from_await:;
          __pyx_t_9 = __pyx_cur_scope->__pyx_t_0;
          __pyx_cur_scope->__pyx_t_0 = 0;
          __Pyx_XGOTREF(__pyx_t_9);
          __pyx_t_10 = __pyx_cur_scope->__pyx_t_1;
          __pyx_cur_scope->__pyx_t_1 = 0;
          __Pyx_XGOTREF(__pyx_t_10);
          __pyx_t_11 = __pyx_cur_scope->__pyx_t_2;
          __pyx_cur_scope->__pyx_t_2 = 0;
          __Pyx_XGOTREF(__pyx_t_11);
          if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 518, __pyx_L15_error)
        } else if (likely(__pyx_t_7 == PYGEN_RETURN)) {
          __Pyx_GOTREF(__pyx_r);
          __Pyx_DECREF(__pyx_r); __pyx_r = 0;
        } else {
          __Pyx_XGOTREF(__pyx_r);
          __PYX_ERR(0, 518, __pyx_L15_error)
        }

        /* "aioesphomeapi/connection.py":517
 *             self._frame_helper.ready_future,
 *         )
 *         try:             # <<<<<<<<<<<<<<
 *             await self._frame_helper.ready_future
 *         except TimeoutError as err:
*/
      }
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      goto __pyx_L20_try_end;
      __pyx_L15_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "aioesphomeapi/connection.py":519
 *         try:
 *             await self._frame_helper.ready_future
 *         except TimeoutError as err:             # <<<<<<<<<<<<<<
 *             raise TimeoutAPIError(
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
*/
      __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_TimeoutError))));
      if (__pyx_t_12) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_init_frame_helper", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_6) < 0) __PYX_ERR(0, 519, __pyx_L17_except_error)
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        __pyx_cur_scope->__pyx_v_err = __pyx_t_4;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":520
 *             await self._frame_helper.ready_future
 *         except TimeoutError as err:
 *             raise TimeoutAPIError(             # <<<<<<<<<<<<<<
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
 *             ) from err
*/
          __pyx_t_13 = NULL;
          __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError);
          __pyx_t_14 = __pyx_v_13aioesphomeapi_10connection_TimeoutAPIError; 

          /* "aioesphomeapi/connection.py":521
 *         except TimeoutError as err:
 *             raise TimeoutAPIError(
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"             # <<<<<<<<<<<<<<
 *             ) from err
 *         except OSError as err:
*/
          __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 521, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_16[0] = __pyx_mstate_global->__pyx_kp_u_Handshake_timed_out_after;
          __pyx_t_16[1] = __pyx_t_15;
          __pyx_t_16[2] = __pyx_mstate_global->__pyx_n_u_s;
          __pyx_t_17 = __Pyx_PyUnicode_Join(__pyx_t_16, 3, 26 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 521, __pyx_L27_error)
          __Pyx_GOTREF(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_5 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_14))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_14);
            assert(__pyx_t_13);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
            __pyx_t_5 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_17};
            __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 520, __pyx_L27_error)
            __Pyx_GOTREF(__pyx_t_3);
          }

          /* "aioesphomeapi/connection.py":522
 *             raise TimeoutAPIError(
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
 *             ) from err             # <<<<<<<<<<<<<<
 *         except OSError as err:
 *             raise HandshakeAPIError(f"Handshake failed: {err}") from err
*/
          __Pyx_Raise(__pyx_t_3, 0, 0, __pyx_cur_scope->__pyx_v_err);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __PYX_ERR(0, 520, __pyx_L27_error)
        }

        /* "aioesphomeapi/connection.py":519
 *         try:
 *             await self._frame_helper.ready_future
 *         except TimeoutError as err:             # <<<<<<<<<<<<<<
 *             raise TimeoutAPIError(
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
*/
        /*finally:*/ {
          __pyx_L27_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
            if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_21);
            __Pyx_XGOTREF(__pyx_t_22);
            __Pyx_XGOTREF(__pyx_t_23);
            __Pyx_XGOTREF(__pyx_t_24);
            __Pyx_XGOTREF(__pyx_t_25);
            __pyx_t_12 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_err);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_err); __pyx_cur_scope->__pyx_v_err = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_23);
            __Pyx_XGIVEREF(__pyx_t_24);
            __Pyx_XGIVEREF(__pyx_t_25);
            __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_22);
            __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
            __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
            __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
            goto __pyx_L17_except_error;
          }
        }
      }

      /* "aioesphomeapi/connection.py":523
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
 *             ) from err
 *         except OSError as err:             # <<<<<<<<<<<<<<
 *             raise HandshakeAPIError(f"Handshake failed: {err}") from err
 *         finally:
*/
      __pyx_t_18 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_OSError))));
      if (__pyx_t_18) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_init_frame_helper", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_4, &__pyx_t_2) < 0) __PYX_ERR(0, 523, __pyx_L17_except_error)
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        __pyx_cur_scope->__pyx_v_err = __pyx_t_4;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":524
 *             ) from err
 *         except OSError as err:
 *             raise HandshakeAPIError(f"Handshake failed: {err}") from err             # <<<<<<<<<<<<<<
 *         finally:
 *             handshake_handle.cancel()
*/
          __pyx_t_14 = NULL;
          __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_HandshakeAPIError);
          __pyx_t_17 = __pyx_v_13aioesphomeapi_10connection_HandshakeAPIError; 
          __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_err, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 524, __pyx_L38_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_15 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_Handshake_failed, __pyx_t_13); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 524, __pyx_L38_error)
          __Pyx_GOTREF(__pyx_t_15);
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __pyx_t_5 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_17))) {
            __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_17);
            assert(__pyx_t_14);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_17);
            __Pyx_INCREF(__pyx_t_14);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_17, __pyx__function);
            __pyx_t_5 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_14, __pyx_t_15};
            __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_17, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 524, __pyx_L38_error)
            __Pyx_GOTREF(__pyx_t_3);
          }
          __Pyx_Raise(__pyx_t_3, 0, 0, __pyx_cur_scope->__pyx_v_err);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __PYX_ERR(0, 524, __pyx_L38_error)
        }

        /* "aioesphomeapi/connection.py":523
 *                 f"Handshake timed out after {HANDSHAKE_TIMEOUT}s"
 *             ) from err
 *         except OSError as err:             # <<<<<<<<<<<<<<
 *             raise HandshakeAPIError(f"Handshake failed: {err}") from err
 *         finally:
*/
        /*finally:*/ {
          __pyx_L38_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_22, &__pyx_t_21, &__pyx_t_20);
            if ( unlikely(__Pyx_GetException(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23) < 0)) __Pyx_ErrFetch(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23);
            __Pyx_XGOTREF(__pyx_t_25);
            __Pyx_XGOTREF(__pyx_t_24);
            __Pyx_XGOTREF(__pyx_t_23);
            __Pyx_XGOTREF(__pyx_t_22);
            __Pyx_XGOTREF(__pyx_t_21);
            __Pyx_XGOTREF(__pyx_t_20);
            __pyx_t_18 = __pyx_lineno; __pyx_t_12 = __pyx_clineno; __pyx_t_26 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_err);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_err); __pyx_cur_scope->__pyx_v_err = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_22);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_ExceptionReset(__pyx_t_22, __pyx_t_21, __pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_25);
            __Pyx_XGIVEREF(__pyx_t_24);
            __Pyx_XGIVEREF(__pyx_t_23);
            __Pyx_ErrRestore(__pyx_t_25, __pyx_t_24, __pyx_t_23);
            __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0; __pyx_t_22 = 0; __pyx_t_21 = 0; __pyx_t_20 = 0;
            __pyx_lineno = __pyx_t_18; __pyx_clineno = __pyx_t_12; __pyx_filename = __pyx_t_26;
            goto __pyx_L17_except_error;
          }
        }
      }
      goto __pyx_L17_except_error;

      /* "aioesphomeapi/connection.py":517
 *             self._frame_helper.ready_future,
 *         )
 *         try:             # <<<<<<<<<<<<<<
 *             await self._frame_helper.ready_future
 *         except TimeoutError as err:
*/
      __pyx_L17_except_error:;
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
      goto __pyx_L13_error;
      __pyx_L20_try_end:;
    }
  }

  /* "aioesphomeapi/connection.py":526
 *             raise HandshakeAPIError(f"Handshake failed: {err}") from err
 *         finally:
 *             handshake_handle.cancel()             # <<<<<<<<<<<<<<
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_4 = __pyx_cur_scope->__pyx_v_handshake_handle;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancel, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L14;
    }
    __pyx_L13_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_assign
      __pyx_t_11 = 0; __pyx_t_10 = 0; __pyx_t_9 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
      if ( unlikely(__Pyx_GetException(&__pyx_t_11, &__pyx_t_10, &__pyx_t_9) < 0)) __Pyx_ErrFetch(&__pyx_t_11, &__pyx_t_10, &__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_11);
      __Pyx_XGOTREF(__pyx_t_10);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_22);
      __pyx_t_12 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_27 = __pyx_filename;
      {
        __pyx_t_4 = __pyx_cur_scope->__pyx_v_handshake_handle;
        __Pyx_INCREF(__pyx_t_4);
        __pyx_t_5 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancel, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 526, __pyx_L45_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_21, __pyx_t_22);
      __Pyx_XGIVEREF(__pyx_t_11);
      __Pyx_XGIVEREF(__pyx_t_10);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_ErrRestore(__pyx_t_11, __pyx_t_10, __pyx_t_9);
      __pyx_t_11 = 0; __pyx_t_10 = 0; __pyx_t_9 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
      __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_27;
      goto __pyx_L1_error;
      __pyx_L45_error:;
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_ExceptionReset(__pyx_t_20, __pyx_t_21, __pyx_t_22);
      __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L14:;
  }

  /* "aioesphomeapi/connection.py":527
 *         finally:
 *             handshake_handle.cancel()
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)             # <<<<<<<<<<<<<<
 * 
 *     async def _connect_hello_login(self, login: bool) -> None:
*/
  __pyx_t_2 = __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_connection_state(__pyx_cur_scope->__pyx_v_self, __pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 527, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":481
 *                 return
 * 
 *     async def _connect_init_frame_helper(self) -> None:             # <<<<<<<<<<<<<<
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_17);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("_connect_init_frame_helper", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_14generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":529
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)
 * 
 *     async def _connect_hello_login(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_connect_hello_login(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_12_connect_hello_login, "Step 4 in connect process: send hello and login and get api version.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_13_connect_hello_login = {"_connect_hello_login", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_connect_hello_login, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_12_connect_hello_login};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_connect_hello_login(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_login;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_connect_hello_login (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_login,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 529, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 529, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_connect_hello_login", 0) < (0)) __PYX_ERR(0, 529, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_connect_hello_login", 1, 1, 1, i); __PYX_ERR(0, 529, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 529, __pyx_L3_error)
    }
    __pyx_v_login = __Pyx_PyObject_IsTrue(values[0]); if (unlikely((__pyx_v_login == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 529, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_connect_hello_login", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 529, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_hello_login", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_12_connect_hello_login(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_login);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":542
 *             tuple(messages),
 *             None,
 *             lambda resp: (             # <<<<<<<<<<<<<<
 *                 type(resp)  # pylint: disable=unidiomatic-typecheck
 *                 is HelloResponse
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_connect_hello_login_lambda4(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_20_connect_hello_login_lambda4 = {"lambda4", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_connect_hello_login_lambda4, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_connect_hello_login_lambda4(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_resp = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda4 (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_resp,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 542, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 542, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "lambda4", 0) < (0)) __PYX_ERR(0, 542, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("lambda4", 1, 1, 1, i); __PYX_ERR(0, 542, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 542, __pyx_L3_error)
    }
    __pyx_v_resp = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lambda4", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 542, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_hello_login.lambda4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_lambda_funcdef_lambda4(__pyx_self, __pyx_v_resp);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda4(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_resp) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda4", 0);

  /* "aioesphomeapi/connection.py":544
 *             lambda resp: (
 *                 type(resp)  # pylint: disable=unidiomatic-typecheck
 *                 is HelloResponse             # <<<<<<<<<<<<<<
 *             ),  # Only wait for HelloResponse
 *             tuple(msg_types),
*/
  __Pyx_XDECREF(__pyx_r);

  /* "aioesphomeapi/connection.py":543
 *             None,
 *             lambda resp: (
 *                 type(resp)  # pylint: disable=unidiomatic-typecheck             # <<<<<<<<<<<<<<
 *                 is HelloResponse
 *             ),  # Only wait for HelloResponse
*/
  __pyx_t_1 = (((PyObject *)Py_TYPE(__pyx_v_resp)) == __pyx_v_13aioesphomeapi_10connection_HelloResponse);

  /* "aioesphomeapi/connection.py":544
 *             lambda resp: (
 *                 type(resp)  # pylint: disable=unidiomatic-typecheck
 *                 is HelloResponse             # <<<<<<<<<<<<<<
 *             ),  # Only wait for HelloResponse
 *             tuple(msg_types),
*/
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 544, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":542
 *             tuple(messages),
 *             None,
 *             lambda resp: (             # <<<<<<<<<<<<<<
 *                 type(resp)  # pylint: disable=unidiomatic-typecheck
 *                 is HelloResponse
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_hello_login.lambda4", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":529
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)
 * 
 *     async def _connect_hello_login(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]
*/

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_12_connect_hello_login(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_login) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_connect_hello_login", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 529, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_login = __pyx_v_login;
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_14generator2, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_connect_hello_login, __pyx_mstate_global->__pyx_n_u_APIConnection__connect_hello_log, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 529, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._connect_hello_login", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_14generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_PySendResult __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_connect_hello_login", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L5_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 529, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":531
 *     async def _connect_hello_login(self, login: bool) -> None:
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]             # <<<<<<<<<<<<<<
 *         msg_types = [HelloResponse]
 *         if login:
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_make_hello_request);
  __pyx_t_3 = __pyx_v_13aioesphomeapi_10connection_make_hello_request; 
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_cur_scope->__pyx_v_self->_params->client_info};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 531, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 531, __pyx_L1_error);
  __pyx_t_1 = 0;
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_cur_scope->__pyx_v_messages = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":532
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]
 *         msg_types = [HelloResponse]             # <<<<<<<<<<<<<<
 *         if login:
 *             messages.append(self._make_auth_request())
*/
  __pyx_t_3 = PyList_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 532, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_HelloResponse);
  __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_HelloResponse);
  if (__Pyx_PyList_SET_ITEM(__pyx_t_3, 0, __pyx_v_13aioesphomeapi_10connection_HelloResponse) != (0)) __PYX_ERR(0, 532, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_cur_scope->__pyx_v_msg_types = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":533
 *         messages = [make_hello_request(self._params.client_info)]
 *         msg_types = [HelloResponse]
 *         if login:             # <<<<<<<<<<<<<<
 *             messages.append(self._make_auth_request())
 *             # Never wait for AuthenticationResponse - the device will either
*/
  if (__pyx_cur_scope->__pyx_v_login) {

    /* "aioesphomeapi/connection.py":534
 *         msg_types = [HelloResponse]
 *         if login:
 *             messages.append(self._make_auth_request())             # <<<<<<<<<<<<<<
 *             # Never wait for AuthenticationResponse - the device will either
 *             # send it immediately if authentication fails, or not send it at all
*/
    __pyx_t_3 = ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_make_auth_request(__pyx_cur_scope->__pyx_v_self); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_messages, __pyx_t_3); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 534, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":533
 *         messages = [make_hello_request(self._params.client_info)]
 *         msg_types = [HelloResponse]
 *         if login:             # <<<<<<<<<<<<<<
 *             messages.append(self._make_auth_request())
 *             # Never wait for AuthenticationResponse - the device will either
*/
  }

  /* "aioesphomeapi/connection.py":539
 *             # if it doesn't support password authentication
 * 
 *         responses = await self.send_messages_await_response_complex(             # <<<<<<<<<<<<<<
 *             tuple(messages),
 *             None,
*/
  __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_INCREF(__pyx_t_1);

  /* "aioesphomeapi/connection.py":540
 * 
 *         responses = await self.send_messages_await_response_complex(
 *             tuple(messages),             # <<<<<<<<<<<<<<
 *             None,
 *             lambda resp: (
*/
  __pyx_t_2 = PyList_AsTuple(__pyx_cur_scope->__pyx_v_messages); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 540, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "aioesphomeapi/connection.py":542
 *             tuple(messages),
 *             None,
 *             lambda resp: (             # <<<<<<<<<<<<<<
 *                 type(resp)  # pylint: disable=unidiomatic-typecheck
 *                 is HelloResponse
*/
  __pyx_t_6 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_20_connect_hello_login_lambda4, 0, __pyx_mstate_global->__pyx_n_u_connect_hello_login_locals_lamb, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 542, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);

  /* "aioesphomeapi/connection.py":546
 *                 is HelloResponse
 *             ),  # Only wait for HelloResponse
 *             tuple(msg_types),             # <<<<<<<<<<<<<<
 *             CONNECT_REQUEST_TIMEOUT,
 *         )
*/
  __pyx_t_7 = PyList_AsTuple(__pyx_cur_scope->__pyx_v_msg_types); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 546, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "aioesphomeapi/connection.py":547
 *             ),  # Only wait for HelloResponse
 *             tuple(msg_types),
 *             CONNECT_REQUEST_TIMEOUT,             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[6] = {__pyx_t_1, __pyx_t_2, Py_None, __pyx_t_6, __pyx_t_7, __pyx_v_13aioesphomeapi_10connection_CONNECT_REQUEST_TIMEOUT};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_send_messages_await_response_com, __pyx_callargs+__pyx_t_4, (6-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_8 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_3, &__pyx_r);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (likely(__pyx_t_8 == PYGEN_NEXT)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, awaiting value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L5_resume_from_await:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 539, __pyx_L1_error)
    __pyx_t_3 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_3);
  } else if (likely(__pyx_t_8 == PYGEN_RETURN)) {
    __Pyx_GOTREF(__pyx_r);
    __pyx_t_3 = __pyx_r; __pyx_r = NULL;
  } else {
    __Pyx_XGOTREF(__pyx_r);
    __PYX_ERR(0, 539, __pyx_L1_error)
  }
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_cur_scope->__pyx_v_responses = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":550
 *         )
 * 
 *         resp = responses.pop(0)             # <<<<<<<<<<<<<<
 *         self._process_hello_resp(resp)
 * 
*/
  __pyx_t_3 = __Pyx_PyObject_PopIndex(__pyx_cur_scope->__pyx_v_responses, __pyx_mstate_global->__pyx_int_0, 0, 1, Py_ssize_t, PyLong_FromSsize_t); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 550, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __pyx_cur_scope->__pyx_v_resp = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":551
 * 
 *         resp = responses.pop(0)
 *         self._process_hello_resp(resp)             # <<<<<<<<<<<<<<
 * 
 *     def _handle_login_response(self, login_response: AuthenticationResponse) -> None:
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_process_hello_resp(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_resp); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 551, __pyx_L1_error)
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":529
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)
 * 
 *     async def _connect_hello_login(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("_connect_hello_login", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":553
 *         self._process_hello_resp(resp)
 * 
 *     def _handle_login_response(self, login_response: AuthenticationResponse) -> None:             # <<<<<<<<<<<<<<
 *         """Handle a AuthenticationResponse."""
 *         if login_response.invalid_password:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_handle_login_response(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_15_handle_login_response, "Handle a AuthenticationResponse.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_16_handle_login_response = {"_handle_login_response", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_handle_login_response, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_15_handle_login_response};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_handle_login_response(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_login_response = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_handle_login_response (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_login_response,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 553, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 553, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_handle_login_response", 0) < (0)) __PYX_ERR(0, 553, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_handle_login_response", 1, 1, 1, i); __PYX_ERR(0, 553, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 553, __pyx_L3_error)
    }
    __pyx_v_login_response = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_handle_login_response", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 553, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_login_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_15_handle_login_response(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_login_response);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_15_handle_login_response(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_login_response) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_login_response", 0);

  /* "aioesphomeapi/connection.py":555
 *     def _handle_login_response(self, login_response: AuthenticationResponse) -> None:
 *         """Handle a AuthenticationResponse."""
 *         if login_response.invalid_password:             # <<<<<<<<<<<<<<
 *             self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_login_response, __pyx_mstate_global->__pyx_n_u_invalid_password); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":556
 *         """Handle a AuthenticationResponse."""
 *         if login_response.invalid_password:
 *             self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))             # <<<<<<<<<<<<<<
 * 
 *     def _process_hello_resp(self, resp: HelloResponse) -> None:
*/
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError);
    __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError; 
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Invalid_password};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    if (!(likely(__Pyx_PyException_Check(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_1))) __PYX_ERR(0, 556, __pyx_L1_error)
    ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->report_fatal_error(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 556, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":555
 *     def _handle_login_response(self, login_response: AuthenticationResponse) -> None:
 *         """Handle a AuthenticationResponse."""
 *         if login_response.invalid_password:             # <<<<<<<<<<<<<<
 *             self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))
 * 
*/
  }

  /* "aioesphomeapi/connection.py":553
 *         self._process_hello_resp(resp)
 * 
 *     def _handle_login_response(self, login_response: AuthenticationResponse) -> None:             # <<<<<<<<<<<<<<
 *         """Handle a AuthenticationResponse."""
 *         if login_response.invalid_password:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_login_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":558
 *             self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))
 * 
 *     def _process_hello_resp(self, resp: HelloResponse) -> None:             # <<<<<<<<<<<<<<
 *         """Process a HelloResponse."""
 *         if self._debug_enabled:
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__process_hello_resp(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_resp) {
  PyObject *__pyx_v_api_version = NULL;
  PyObject *__pyx_v_expected_name = NULL;
  PyObject *__pyx_v_received_name = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9[3];
  int __pyx_t_10;
  PyObject *__pyx_t_11[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_process_hello_resp", 0);

  /* "aioesphomeapi/connection.py":560
 *     def _process_hello_resp(self, resp: HelloResponse) -> None:
 *         """Process a HelloResponse."""
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Successfully connected ('%s' API=%s.%s)",
*/
  if (__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":561
 *         """Process a HelloResponse."""
 *         if self._debug_enabled:
 *             _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                 "%s: Successfully connected ('%s' API=%s.%s)",
 *                 self.log_name,
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 561, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":564
 *                 "%s: Successfully connected ('%s' API=%s.%s)",
 *                 self.log_name,
 *                 resp.server_info,             # <<<<<<<<<<<<<<
 *                 resp.api_version_major,
 *                 resp.api_version_minor,
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_server_info); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 564, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "aioesphomeapi/connection.py":565
 *                 self.log_name,
 *                 resp.server_info,
 *                 resp.api_version_major,             # <<<<<<<<<<<<<<
 *                 resp.api_version_minor,
 *             )
*/
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_api_version_major); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 565, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);

    /* "aioesphomeapi/connection.py":566
 *                 resp.server_info,
 *                 resp.api_version_major,
 *                 resp.api_version_minor,             # <<<<<<<<<<<<<<
 *             )
 *         api_version = APIVersion(resp.api_version_major, resp.api_version_minor)
*/
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_api_version_minor); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 566, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[6] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_s_Successfully_connected_s_API, __pyx_v_self->log_name, __pyx_t_3, __pyx_t_5, __pyx_t_6};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (6-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 561, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":560
 *     def _process_hello_resp(self, resp: HelloResponse) -> None:
 *         """Process a HelloResponse."""
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Successfully connected ('%s' API=%s.%s)",
*/
  }

  /* "aioesphomeapi/connection.py":568
 *                 resp.api_version_minor,
 *             )
 *         api_version = APIVersion(resp.api_version_major, resp.api_version_minor)             # <<<<<<<<<<<<<<
 *         if api_version.major > 2:
 *             _LOGGER.error(
*/
  __pyx_t_4 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_APIVersion);
  __pyx_t_6 = __pyx_v_13aioesphomeapi_10connection_APIVersion; 
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_api_version_major); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_api_version_minor); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 568, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_7 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_6))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_6);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
    __pyx_t_7 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_t_5, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 568, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_api_version = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":569
 *             )
 *         api_version = APIVersion(resp.api_version_major, resp.api_version_minor)
 *         if api_version.major > 2:             # <<<<<<<<<<<<<<
 *             _LOGGER.error(
 *                 "%s: Incompatible version %s! Closing connection",
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_api_version, __pyx_mstate_global->__pyx_n_u_major); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_int_2, Py_GT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 569, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(__pyx_t_8)) {

    /* "aioesphomeapi/connection.py":570
 *         api_version = APIVersion(resp.api_version_major, resp.api_version_minor)
 *         if api_version.major > 2:
 *             _LOGGER.error(             # <<<<<<<<<<<<<<
 *                 "%s: Incompatible version %s! Closing connection",
 *                 self.log_name,
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_error); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 570, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":573
 *                 "%s: Incompatible version %s! Closing connection",
 *                 self.log_name,
 *                 api_version.major,             # <<<<<<<<<<<<<<
 *             )
 *             raise APIConnectionError(f"Incompatible API version ({api_version}).")
*/
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_api_version, __pyx_mstate_global->__pyx_n_u_major); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 573, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_s_Incompatible_version_s_Closin, __pyx_v_self->log_name, __pyx_t_3};
      __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (4-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 570, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

    /* "aioesphomeapi/connection.py":575
 *                 api_version.major,
 *             )
 *             raise APIConnectionError(f"Incompatible API version ({api_version}).")             # <<<<<<<<<<<<<<
 * 
 *         self.api_version = api_version
*/
    __pyx_t_5 = NULL;
    __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_APIConnectionError);
    __pyx_t_3 = __pyx_v_13aioesphomeapi_10connection_APIConnectionError; 
    __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_api_version, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9[0] = __pyx_mstate_global->__pyx_kp_u_Incompatible_API_version;
    __pyx_t_9[1] = __pyx_t_1;
    __pyx_t_9[2] = __pyx_mstate_global->__pyx_kp_u__7;
    __pyx_t_4 = __Pyx_PyUnicode_Join(__pyx_t_9, 3, 26 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 2, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 575, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_5);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
      __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 575, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
    }
    __Pyx_Raise(__pyx_t_6, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __PYX_ERR(0, 575, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":569
 *             )
 *         api_version = APIVersion(resp.api_version_major, resp.api_version_minor)
 *         if api_version.major > 2:             # <<<<<<<<<<<<<<
 *             _LOGGER.error(
 *                 "%s: Incompatible version %s! Closing connection",
*/
  }

  /* "aioesphomeapi/connection.py":577
 *             raise APIConnectionError(f"Incompatible API version ({api_version}).")
 * 
 *         self.api_version = api_version             # <<<<<<<<<<<<<<
 *         expected_name = self._params.expected_name
 *         if received_name := resp.name:
*/
  __Pyx_INCREF(__pyx_v_api_version);
  __Pyx_GIVEREF(__pyx_v_api_version);
  __Pyx_GOTREF(__pyx_v_self->api_version);
  __Pyx_DECREF(__pyx_v_self->api_version);
  __pyx_v_self->api_version = __pyx_v_api_version;

  /* "aioesphomeapi/connection.py":578
 * 
 *         self.api_version = api_version
 *         expected_name = self._params.expected_name             # <<<<<<<<<<<<<<
 *         if received_name := resp.name:
 *             if expected_name is not None and received_name != expected_name:
*/
  __pyx_t_6 = __pyx_v_self->_params->expected_name;
  __Pyx_INCREF(__pyx_t_6);
  __pyx_v_expected_name = __pyx_t_6;
  __pyx_t_6 = 0;

  /* "aioesphomeapi/connection.py":579
 *         self.api_version = api_version
 *         expected_name = self._params.expected_name
 *         if received_name := resp.name:             # <<<<<<<<<<<<<<
 *             if expected_name is not None and received_name != expected_name:
 *                 raise BadNameAPIError(
*/
  __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_name_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_INCREF(__pyx_t_6);
  __pyx_v_received_name = __pyx_t_6;
  __pyx_t_8 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_8 < 0))) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (__pyx_t_8) {

    /* "aioesphomeapi/connection.py":580
 *         expected_name = self._params.expected_name
 *         if received_name := resp.name:
 *             if expected_name is not None and received_name != expected_name:             # <<<<<<<<<<<<<<
 *                 raise BadNameAPIError(
 *                     f"Expected '{expected_name}' but server sent "
*/
    __pyx_t_10 = (__pyx_v_expected_name != Py_None);
    if (__pyx_t_10) {
    } else {
      __pyx_t_8 = __pyx_t_10;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_6 = PyObject_RichCompare(__pyx_v_received_name, __pyx_v_expected_name, Py_NE); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 580, __pyx_L1_error)
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely((__pyx_t_10 < 0))) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_8 = __pyx_t_10;
    __pyx_L7_bool_binop_done:;
    if (unlikely(__pyx_t_8)) {

      /* "aioesphomeapi/connection.py":581
 *         if received_name := resp.name:
 *             if expected_name is not None and received_name != expected_name:
 *                 raise BadNameAPIError(             # <<<<<<<<<<<<<<
 *                     f"Expected '{expected_name}' but server sent "
 *                     f"a different name: '{received_name}'",
*/
      __pyx_t_3 = NULL;
      __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_BadNameAPIError);
      __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_BadNameAPIError; 

      /* "aioesphomeapi/connection.py":582
 *             if expected_name is not None and received_name != expected_name:
 *                 raise BadNameAPIError(
 *                     f"Expected '{expected_name}' but server sent "             # <<<<<<<<<<<<<<
 *                     f"a different name: '{received_name}'",
 *                     received_name,
*/
      __pyx_t_5 = __Pyx_PyObject_FormatSimple(__pyx_v_expected_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 582, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "aioesphomeapi/connection.py":583
 *                 raise BadNameAPIError(
 *                     f"Expected '{expected_name}' but server sent "
 *                     f"a different name: '{received_name}'",             # <<<<<<<<<<<<<<
 *                     received_name,
 *                 )
*/
      __pyx_t_1 = __Pyx_PyObject_FormatSimple(__pyx_v_received_name, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_11[0] = __pyx_mstate_global->__pyx_kp_u_Expected;
      __pyx_t_11[1] = __pyx_t_5;
      __pyx_t_11[2] = __pyx_mstate_global->__pyx_kp_u_but_server_sent_a_different_nam;
      __pyx_t_11[3] = __pyx_t_1;
      __pyx_t_11[4] = __pyx_mstate_global->__pyx_kp_u__8;

      /* "aioesphomeapi/connection.py":582
 *             if expected_name is not None and received_name != expected_name:
 *                 raise BadNameAPIError(
 *                     f"Expected '{expected_name}' but server sent "             # <<<<<<<<<<<<<<
 *                     f"a different name: '{received_name}'",
 *                     received_name,
*/
      __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_11, 5, 10 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_5) + 37 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_5) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1));
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":584
 *                     f"Expected '{expected_name}' but server sent "
 *                     f"a different name: '{received_name}'",
 *                     received_name,             # <<<<<<<<<<<<<<
 *                 )
 * 
*/
      __pyx_t_7 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_3);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_7 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_2, __pyx_v_received_name};
        __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 581, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
      }
      __Pyx_Raise(__pyx_t_6, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __PYX_ERR(0, 581, __pyx_L1_error)

      /* "aioesphomeapi/connection.py":580
 *         expected_name = self._params.expected_name
 *         if received_name := resp.name:
 *             if expected_name is not None and received_name != expected_name:             # <<<<<<<<<<<<<<
 *                 raise BadNameAPIError(
 *                     f"Expected '{expected_name}' but server sent "
*/
    }

    /* "aioesphomeapi/connection.py":587
 *                 )
 * 
 *             self.received_name = received_name             # <<<<<<<<<<<<<<
 *             self.set_log_name(self.received_name)
 * 
*/
    __pyx_t_6 = __pyx_v_received_name;
    __Pyx_INCREF(__pyx_t_6);
    if (!(likely(PyUnicode_CheckExact(__pyx_t_6))||((__pyx_t_6) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_6))) __PYX_ERR(0, 587, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_6);
    __Pyx_GOTREF(__pyx_v_self->received_name);
    __Pyx_DECREF(__pyx_v_self->received_name);
    __pyx_v_self->received_name = ((PyObject*)__pyx_t_6);
    __pyx_t_6 = 0;

    /* "aioesphomeapi/connection.py":588
 * 
 *             self.received_name = received_name
 *             self.set_log_name(self.received_name)             # <<<<<<<<<<<<<<
 * 
 *     def _async_schedule_keep_alive(self, now: _float) -> None:
*/
    __pyx_t_6 = __pyx_v_self->received_name;
    __Pyx_INCREF(__pyx_t_6);
    __pyx_t_4 = ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->set_log_name(__pyx_v_self, ((PyObject*)__pyx_t_6), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":579
 *         self.api_version = api_version
 *         expected_name = self._params.expected_name
 *         if received_name := resp.name:             # <<<<<<<<<<<<<<
 *             if expected_name is not None and received_name != expected_name:
 *                 raise BadNameAPIError(
*/
  }

  /* "aioesphomeapi/connection.py":558
 *             self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))
 * 
 *     def _process_hello_resp(self, resp: HelloResponse) -> None:             # <<<<<<<<<<<<<<
 *         """Process a HelloResponse."""
 *         if self._debug_enabled:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._process_hello_resp", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_api_version);
  __Pyx_XDECREF(__pyx_v_expected_name);
  __Pyx_XDECREF(__pyx_v_received_name);
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":590
 *             self.set_log_name(self.received_name)
 * 
 *     def _async_schedule_keep_alive(self, now: _float) -> None:             # <<<<<<<<<<<<<<
 *         """Start the keep alive task."""
 *         self._send_pending_ping = True
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__async_schedule_keep_alive(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_now) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_async_schedule_keep_alive", 0);

  /* "aioesphomeapi/connection.py":592
 *     def _async_schedule_keep_alive(self, now: _float) -> None:
 *         """Start the keep alive task."""
 *         self._send_pending_ping = True             # <<<<<<<<<<<<<<
 *         self._ping_timer = self._loop.call_at(
 *             now + self._keep_alive_interval, self._async_send_keep_alive
*/
  __pyx_v_self->_send_pending_ping = 1;

  /* "aioesphomeapi/connection.py":593
 *         """Start the keep alive task."""
 *         self._send_pending_ping = True
 *         self._ping_timer = self._loop.call_at(             # <<<<<<<<<<<<<<
 *             now + self._keep_alive_interval, self._async_send_keep_alive
 *         )
*/
  __pyx_t_2 = __pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_2);

  /* "aioesphomeapi/connection.py":594
 *         self._send_pending_ping = True
 *         self._ping_timer = self._loop.call_at(
 *             now + self._keep_alive_interval, self._async_send_keep_alive             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_interval); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyNumber_Add(__pyx_v_now, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_async_send_keep_alive); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 594, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_4, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_call_at, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }

  /* "aioesphomeapi/connection.py":593
 *         """Start the keep alive task."""
 *         self._send_pending_ping = True
 *         self._ping_timer = self._loop.call_at(             # <<<<<<<<<<<<<<
 *             now + self._keep_alive_interval, self._async_send_keep_alive
 *         )
*/
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_ping_timer);
  __Pyx_DECREF(__pyx_v_self->_ping_timer);
  __pyx_v_self->_ping_timer = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":590
 *             self.set_log_name(self.received_name)
 * 
 *     def _async_schedule_keep_alive(self, now: _float) -> None:             # <<<<<<<<<<<<<<
 *         """Start the keep alive task."""
 *         self._send_pending_ping = True
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._async_schedule_keep_alive", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":597
 *         )
 * 
 *     def _async_send_keep_alive(self) -> None:             # <<<<<<<<<<<<<<
 *         """Send a keep alive message."""
 *         now = self._loop.time()
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_18_async_send_keep_alive(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_17_async_send_keep_alive, "Send a keep alive message.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_18_async_send_keep_alive = {"_async_send_keep_alive", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_18_async_send_keep_alive, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_17_async_send_keep_alive};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_18_async_send_keep_alive(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_async_send_keep_alive (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_async_send_keep_alive", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_async_send_keep_alive", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_async_send_keep_alive(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_async_send_keep_alive(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_v_now = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_async_send_keep_alive", 0);

  /* "aioesphomeapi/connection.py":599
 *     def _async_send_keep_alive(self) -> None:
 *         """Send a keep alive message."""
 *         now = self._loop.time()             # <<<<<<<<<<<<<<
 * 
 *         if self._send_pending_ping:
*/
  __pyx_t_2 = __pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_time, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 599, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_now = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":601
 *         now = self._loop.time()
 * 
 *         if self._send_pending_ping:             # <<<<<<<<<<<<<<
 *             self.send_messages(PING_REQUEST_MESSAGES)
 *             if self._pong_timer is None:
*/
  if (__pyx_v_self->_send_pending_ping) {

    /* "aioesphomeapi/connection.py":602
 * 
 *         if self._send_pending_ping:
 *             self.send_messages(PING_REQUEST_MESSAGES)             # <<<<<<<<<<<<<<
 *             if self._pong_timer is None:
 *                 # Do not reset the timer if it's already set
*/
    __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_PING_REQUEST_MESSAGES;
    __Pyx_INCREF(__pyx_t_1);
    ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 602, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":603
 *         if self._send_pending_ping:
 *             self.send_messages(PING_REQUEST_MESSAGES)
 *             if self._pong_timer is None:             # <<<<<<<<<<<<<<
 *                 # Do not reset the timer if it's already set
 *                 # since the only thing we want to reset the timer
*/
    __pyx_t_4 = (__pyx_v_self->_pong_timer == Py_None);
    if (__pyx_t_4) {

      /* "aioesphomeapi/connection.py":607
 *                 # since the only thing we want to reset the timer
 *                 # is if we receive a pong.
 *                 self._pong_timer = self._loop.call_at(             # <<<<<<<<<<<<<<
 *                     now + self._keep_alive_timeout, self._async_pong_not_received
 *                 )
*/
      __pyx_t_2 = __pyx_v_self->_loop;
      __Pyx_INCREF(__pyx_t_2);

      /* "aioesphomeapi/connection.py":608
 *                 # is if we receive a pong.
 *                 self._pong_timer = self._loop.call_at(
 *                     now + self._keep_alive_timeout, self._async_pong_not_received             # <<<<<<<<<<<<<<
 *                 )
 *             elif self._debug_enabled:
*/
      __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_timeout); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 608, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = PyNumber_Add(__pyx_v_now, __pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 608, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_async_pong_not_received); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 608, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_t_6, __pyx_t_5};
        __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_call_at, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 607, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }

      /* "aioesphomeapi/connection.py":607
 *                 # since the only thing we want to reset the timer
 *                 # is if we receive a pong.
 *                 self._pong_timer = self._loop.call_at(             # <<<<<<<<<<<<<<
 *                     now + self._keep_alive_timeout, self._async_pong_not_received
 *                 )
*/
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->_pong_timer);
      __Pyx_DECREF(__pyx_v_self->_pong_timer);
      __pyx_v_self->_pong_timer = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":603
 *         if self._send_pending_ping:
 *             self.send_messages(PING_REQUEST_MESSAGES)
 *             if self._pong_timer is None:             # <<<<<<<<<<<<<<
 *                 # Do not reset the timer if it's already set
 *                 # since the only thing we want to reset the timer
*/
      goto __pyx_L4;
    }

    /* "aioesphomeapi/connection.py":610
 *                     now + self._keep_alive_timeout, self._async_pong_not_received
 *                 )
 *             elif self._debug_enabled:             # <<<<<<<<<<<<<<
 *                 #
 *                 # We haven't reached the ping response (pong) timeout yet
*/
    if (__pyx_v_self->_debug_enabled) {

      /* "aioesphomeapi/connection.py":619
 *                 # us to force a TCP RST aka connection reset by peer.
 *                 #
 *                 _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                     "%s: PingResponse (pong) was not received "
 *                     "since last keep alive after %s seconds; "
*/
      __pyx_t_5 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 619, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "aioesphomeapi/connection.py":624
 *                     "rescheduling keep alive",
 *                     self.log_name,
 *                     self._keep_alive_interval,             # <<<<<<<<<<<<<<
 *                 )
 * 
*/
      __pyx_t_6 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_interval); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 624, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
        assert(__pyx_t_5);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_2, __pyx__function);
        __pyx_t_3 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[4] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_s_PingResponse_pong_was_not_rec, __pyx_v_self->log_name, __pyx_t_6};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 619, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":610
 *                     now + self._keep_alive_timeout, self._async_pong_not_received
 *                 )
 *             elif self._debug_enabled:             # <<<<<<<<<<<<<<
 *                 #
 *                 # We haven't reached the ping response (pong) timeout yet
*/
    }
    __pyx_L4:;

    /* "aioesphomeapi/connection.py":601
 *         now = self._loop.time()
 * 
 *         if self._send_pending_ping:             # <<<<<<<<<<<<<<
 *             self.send_messages(PING_REQUEST_MESSAGES)
 *             if self._pong_timer is None:
*/
  }

  /* "aioesphomeapi/connection.py":627
 *                 )
 * 
 *         self._async_schedule_keep_alive(now)             # <<<<<<<<<<<<<<
 * 
 *     def _async_cancel_pong_timer(self) -> None:
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_async_schedule_keep_alive(__pyx_v_self, __pyx_v_now); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 627, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":597
 *         )
 * 
 *     def _async_send_keep_alive(self) -> None:             # <<<<<<<<<<<<<<
 *         """Send a keep alive message."""
 *         now = self._loop.time()
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._async_send_keep_alive", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_now);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":629
 *         self._async_schedule_keep_alive(now)
 * 
 *     def _async_cancel_pong_timer(self) -> None:             # <<<<<<<<<<<<<<
 *         """Cancel the pong timer."""
 *         if self._pong_timer is not None:
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__async_cancel_pong_timer(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_async_cancel_pong_timer", 0);

  /* "aioesphomeapi/connection.py":631
 *     def _async_cancel_pong_timer(self) -> None:
 *         """Cancel the pong timer."""
 *         if self._pong_timer is not None:             # <<<<<<<<<<<<<<
 *             self._pong_timer.cancel()
 *             self._pong_timer = None
*/
  __pyx_t_1 = (__pyx_v_self->_pong_timer != Py_None);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":632
 *         """Cancel the pong timer."""
 *         if self._pong_timer is not None:
 *             self._pong_timer.cancel()             # <<<<<<<<<<<<<<
 *             self._pong_timer = None
 * 
*/
    __pyx_t_3 = __pyx_v_self->_pong_timer;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancel, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 632, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":633
 *         if self._pong_timer is not None:
 *             self._pong_timer.cancel()
 *             self._pong_timer = None             # <<<<<<<<<<<<<<
 * 
 *     def _async_pong_not_received(self) -> None:
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_pong_timer);
    __Pyx_DECREF(__pyx_v_self->_pong_timer);
    __pyx_v_self->_pong_timer = Py_None;

    /* "aioesphomeapi/connection.py":631
 *     def _async_cancel_pong_timer(self) -> None:
 *         """Cancel the pong timer."""
 *         if self._pong_timer is not None:             # <<<<<<<<<<<<<<
 *             self._pong_timer.cancel()
 *             self._pong_timer = None
*/
  }

  /* "aioesphomeapi/connection.py":629
 *         self._async_schedule_keep_alive(now)
 * 
 *     def _async_cancel_pong_timer(self) -> None:             # <<<<<<<<<<<<<<
 *         """Cancel the pong timer."""
 *         if self._pong_timer is not None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._async_cancel_pong_timer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":635
 *             self._pong_timer = None
 * 
 *     def _async_pong_not_received(self) -> None:             # <<<<<<<<<<<<<<
 *         """Ping not received."""
 *         if self._debug_enabled:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_async_pong_not_received(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_19_async_pong_not_received, "Ping not received.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_20_async_pong_not_received = {"_async_pong_not_received", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_async_pong_not_received, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_19_async_pong_not_received};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_async_pong_not_received(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_async_pong_not_received (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_async_pong_not_received", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_async_pong_not_received", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_async_pong_not_received(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_async_pong_not_received(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7[3];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_async_pong_not_received", 0);

  /* "aioesphomeapi/connection.py":637
 *     def _async_pong_not_received(self) -> None:
 *         """Ping not received."""
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Ping response not received after %s seconds",
*/
  if (__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":638
 *         """Ping not received."""
 *         if self._debug_enabled:
 *             _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                 "%s: Ping response not received after %s seconds",
 *                 self.log_name,
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 638, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 638, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":641
 *                 "%s: Ping response not received after %s seconds",
 *                 self.log_name,
 *                 self._keep_alive_timeout,             # <<<<<<<<<<<<<<
 *             )
 *         self.report_fatal_error(
*/
    __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_timeout); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 641, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_mstate_global->__pyx_kp_u_s_Ping_response_not_received_af, __pyx_v_self->log_name, __pyx_t_3};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 638, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":637
 *     def _async_pong_not_received(self) -> None:
 *         """Ping not received."""
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Ping response not received after %s seconds",
*/
  }

  /* "aioesphomeapi/connection.py":644
 *             )
 *         self.report_fatal_error(
 *             PingFailedAPIError(             # <<<<<<<<<<<<<<
 *                 f"Ping response not received after {self._keep_alive_timeout} seconds"
 *             )
*/
  __pyx_t_4 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_PingFailedAPIError);
  __pyx_t_3 = __pyx_v_13aioesphomeapi_10connection_PingFailedAPIError; 

  /* "aioesphomeapi/connection.py":645
 *         self.report_fatal_error(
 *             PingFailedAPIError(
 *                 f"Ping response not received after {self._keep_alive_timeout} seconds"             # <<<<<<<<<<<<<<
 *             )
 *         )
*/
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_timeout); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_t_2, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7[0] = __pyx_mstate_global->__pyx_kp_u_Ping_response_not_received_after;
  __pyx_t_7[1] = __pyx_t_6;
  __pyx_t_7[2] = __pyx_mstate_global->__pyx_kp_u_seconds;
  __pyx_t_2 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, 33 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6) + 8, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6));
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_2};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 644, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }

  /* "aioesphomeapi/connection.py":644
 *             )
 *         self.report_fatal_error(
 *             PingFailedAPIError(             # <<<<<<<<<<<<<<
 *                 f"Ping response not received after {self._keep_alive_timeout} seconds"
 *             )
*/
  if (!(likely(__Pyx_PyException_Check(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_1))) __PYX_ERR(0, 644, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":643
 *                 self._keep_alive_timeout,
 *             )
 *         self.report_fatal_error(             # <<<<<<<<<<<<<<
 *             PingFailedAPIError(
 *                 f"Ping response not received after {self._keep_alive_timeout} seconds"
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->report_fatal_error(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":635
 *             self._pong_timer = None
 * 
 *     def _async_pong_not_received(self) -> None:             # <<<<<<<<<<<<<<
 *         """Ping not received."""
 *         if self._debug_enabled:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._async_pong_not_received", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_23generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":649
 *         )
 * 
 *     async def start_resolve_host(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the host resolution process.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_22start_resolve_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_21start_resolve_host, "Start the host resolution process.\n\n        This part of the process resolves the hostnames to IP addresses\n        and prepares the connection for the next step.\n        ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_22start_resolve_host = {"start_resolve_host", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_22start_resolve_host, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_21start_resolve_host};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_22start_resolve_host(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_resolve_host (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("start_resolve_host", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("start_resolve_host", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_21start_resolve_host(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_21start_resolve_host(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("start_resolve_host", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 649, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_23generator3, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_start_resolve_host, __pyx_mstate_global->__pyx_n_u_APIConnection_start_resolve_host, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 649, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.start_resolve_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_23generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_PySendResult __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  char const *__pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  char const *__pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_resolve_host", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L15_resume_from_await;
    case 2: goto __pyx_L25_resume_from_await;
    case 3: goto __pyx_L29_resume_from_await;
    case 4: goto __pyx_L30_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 649, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":655
 *         and prepares the connection for the next step.
 *         """
 *         if self.connection_state is not CONNECTION_STATE_INITIALIZED:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "Connection can only be used once, connection is not in init state"
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_self->connection_state != __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED);
  if (unlikely(__pyx_t_1)) {

    /* "aioesphomeapi/connection.py":656
 *         """
 *         if self.connection_state is not CONNECTION_STATE_INITIALIZED:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 "Connection can only be used once, connection is not in init state"
 *             )
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Connection_can_only_be_used_once};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 656, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 656, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":655
 *         and prepares the connection for the next step.
 *         """
 *         if self.connection_state is not CONNECTION_STATE_INITIALIZED:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "Connection can only be used once, connection is not in init state"
*/
  }

  /* "aioesphomeapi/connection.py":660
 *             )
 * 
 *         self._resolve_host_future = self._loop.create_future()             # <<<<<<<<<<<<<<
 *         try:
 *             async with interrupt(
*/
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_create_future, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 660, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_resolve_host_future);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_resolve_host_future);
  __pyx_cur_scope->__pyx_v_self->_resolve_host_future = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":661
 * 
 *         self._resolve_host_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._resolve_host_future, ConnectionInterruptedError, None
*/
  /*try:*/ {
    {
      __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":662
 *         self._resolve_host_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._resolve_host_future, ConnectionInterruptedError, None
 *             ):
*/
        /*with:*/ {
          __pyx_t_3 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_interrupt); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 662, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_8);

          /* "aioesphomeapi/connection.py":663
 *         try:
 *             async with interrupt(
 *                 self._resolve_host_future, ConnectionInterruptedError, None             # <<<<<<<<<<<<<<
 *             ):
 *                 self._addrs_info = await hr.async_resolve_host(
*/
          __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 663, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_8);
            assert(__pyx_t_3);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_self->_resolve_host_future, __pyx_t_9, Py_None};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (4-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 662, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __pyx_t_10 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aexit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 662, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_10);

          /* "aioesphomeapi/connection.py":662
 *         self._resolve_host_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._resolve_host_future, ConnectionInterruptedError, None
 *             ):
*/
          __pyx_t_9 = NULL;
          __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aenter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 662, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_9);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_9, NULL};
            __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 662, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_8);
          }
          __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_8, &__pyx_r);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (likely(__pyx_t_11 == PYGEN_NEXT)) {
            __Pyx_GOTREF(__pyx_r);
            __Pyx_XGIVEREF(__pyx_t_2);
            __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
            __Pyx_XGIVEREF(__pyx_t_5);
            __pyx_cur_scope->__pyx_t_1 = __pyx_t_5;
            __Pyx_XGIVEREF(__pyx_t_6);
            __pyx_cur_scope->__pyx_t_2 = __pyx_t_6;
            __Pyx_XGIVEREF(__pyx_t_7);
            __pyx_cur_scope->__pyx_t_3 = __pyx_t_7;
            __Pyx_XGIVEREF(__pyx_t_10);
            __pyx_cur_scope->__pyx_t_4 = __pyx_t_10;
            __Pyx_XGIVEREF(__pyx_r);
            __Pyx_RefNannyFinishContext();
            __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
            /* return from generator, awaiting value */
            __pyx_generator->resume_label = 1;
            return __pyx_r;
            __pyx_L15_resume_from_await:;
            __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
            __pyx_cur_scope->__pyx_t_0 = 0;
            __Pyx_XGOTREF(__pyx_t_2);
            __pyx_t_5 = __pyx_cur_scope->__pyx_t_1;
            __pyx_cur_scope->__pyx_t_1 = 0;
            __Pyx_XGOTREF(__pyx_t_5);
            __pyx_t_6 = __pyx_cur_scope->__pyx_t_2;
            __pyx_cur_scope->__pyx_t_2 = 0;
            __Pyx_XGOTREF(__pyx_t_6);
            __pyx_t_7 = __pyx_cur_scope->__pyx_t_3;
            __pyx_cur_scope->__pyx_t_3 = 0;
            __Pyx_XGOTREF(__pyx_t_7);
            __pyx_t_10 = __pyx_cur_scope->__pyx_t_4;
            __pyx_cur_scope->__pyx_t_4 = 0;
            __Pyx_XGOTREF(__pyx_t_10);
            if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 662, __pyx_L14_error)
            __pyx_t_8 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_8);
          } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
            __Pyx_GOTREF(__pyx_r);
            __pyx_t_8 = __pyx_r; __pyx_r = NULL;
          } else {
            __Pyx_XGOTREF(__pyx_r);
            __PYX_ERR(0, 662, __pyx_L14_error)
          }
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          /*try:*/ {
            {
              __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_14);
              /*try:*/ {

                /* "aioesphomeapi/connection.py":665
 *                 self._resolve_host_future, ConnectionInterruptedError, None
 *             ):
 *                 self._addrs_info = await hr.async_resolve_host(             # <<<<<<<<<<<<<<
 *                     self._params.addresses,
 *                     self._params.port,
*/
                __pyx_t_8 = __pyx_v_13aioesphomeapi_10connection_hr;
                __Pyx_INCREF(__pyx_t_8);

                /* "aioesphomeapi/connection.py":668
 *                     self._params.addresses,
 *                     self._params.port,
 *                     self._params.zeroconf_manager,             # <<<<<<<<<<<<<<
 *                 )
 *         except (Exception, CancelledError) as ex:
*/
                __pyx_t_4 = 0;
                {
                  PyObject *__pyx_callargs[4] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_self->_params->addresses, __pyx_cur_scope->__pyx_v_self->_params->port, __pyx_cur_scope->__pyx_v_self->_params->zeroconf_manager};
                  __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_async_resolve_host, __pyx_callargs+__pyx_t_4, (4-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 665, __pyx_L19_error)
                  __Pyx_GOTREF(__pyx_t_2);
                }
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_12);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_12;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 2;
                  return __pyx_r;
                  __pyx_L25_resume_from_await:;
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_12 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_12);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 665, __pyx_L19_error)
                  __pyx_t_2 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_2);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_2 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 665, __pyx_L19_error)
                }

                /* "aioesphomeapi/connection.py":665
 *                 self._resolve_host_future, ConnectionInterruptedError, None
 *             ):
 *                 self._addrs_info = await hr.async_resolve_host(             # <<<<<<<<<<<<<<
 *                     self._params.addresses,
 *                     self._params.port,
*/
                if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_2))) __PYX_ERR(0, 665, __pyx_L19_error)
                __Pyx_GIVEREF(__pyx_t_2);
                __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_addrs_info);
                __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_addrs_info);
                __pyx_cur_scope->__pyx_v_self->_addrs_info = ((PyObject*)__pyx_t_2);
                __pyx_t_2 = 0;

                /* "aioesphomeapi/connection.py":662
 *         self._resolve_host_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._resolve_host_future, ConnectionInterruptedError, None
 *             ):
*/
              }
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              goto __pyx_L24_try_end;
              __pyx_L19_error:;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              /*except:*/ {
                __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.start_resolve_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_8, &__pyx_t_3) < 0) __PYX_ERR(0, 662, __pyx_L21_except_error)
                __Pyx_XGOTREF(__pyx_t_2);
                __Pyx_XGOTREF(__pyx_t_8);
                __Pyx_XGOTREF(__pyx_t_3);
                __pyx_t_9 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_8, __pyx_t_3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 662, __pyx_L21_except_error)
                __Pyx_GOTREF(__pyx_t_9);
                __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_9, NULL);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 662, __pyx_L21_except_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_15, &__pyx_r);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_2);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
                  __Pyx_XGIVEREF(__pyx_t_3);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_8);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_12);
                  __pyx_cur_scope->__pyx_t_7 = __pyx_t_12;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_8 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_9 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_10 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_SwapException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 3;
                  return __pyx_r;
                  __pyx_L29_resume_from_await:;
                  __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_2);
                  __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_3);
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_8);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_12 = __pyx_cur_scope->__pyx_t_7;
                  __pyx_cur_scope->__pyx_t_7 = 0;
                  __Pyx_XGOTREF(__pyx_t_12);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_8;
                  __pyx_cur_scope->__pyx_t_8 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_9;
                  __pyx_cur_scope->__pyx_t_9 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_10;
                  __pyx_cur_scope->__pyx_t_10 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 662, __pyx_L21_except_error)
                  __pyx_t_9 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_9);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_9 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 662, __pyx_L21_except_error)
                }
                __pyx_t_15 = __pyx_t_9;
                __pyx_t_9 = 0;
                __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_15);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (__pyx_t_1 < (0)) __PYX_ERR(0, 662, __pyx_L21_except_error)
                __pyx_t_16 = (!__pyx_t_1);
                if (unlikely(__pyx_t_16)) {
                  __Pyx_GIVEREF(__pyx_t_2);
                  __Pyx_GIVEREF(__pyx_t_8);
                  __Pyx_XGIVEREF(__pyx_t_3);
                  __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_8, __pyx_t_3);
                  __pyx_t_2 = 0;  __pyx_t_8 = 0;  __pyx_t_3 = 0; 
                  __PYX_ERR(0, 662, __pyx_L21_except_error)
                }
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                goto __pyx_L20_exception_handled;
              }
              __pyx_L21_except_error:;
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              goto __pyx_L8_error;
              __pyx_L20_exception_handled:;
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              __pyx_L24_try_end:;
            }
          }
          /*finally:*/ {
            /*normal exit:*/{
              if (__pyx_t_10) {
                __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[0], NULL);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 662, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_14, &__pyx_r);
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 4;
                  return __pyx_r;
                  __pyx_L30_resume_from_await:;
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 662, __pyx_L8_error)
                  __pyx_t_3 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_3);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_3 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 662, __pyx_L8_error)
                }
                __pyx_t_14 = __pyx_t_3;
                __pyx_t_3 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              }
              goto __pyx_L18;
            }
            __pyx_L18:;
          }
          goto __pyx_L31;
          __pyx_L14_error:;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          goto __pyx_L8_error;
          __pyx_L31:;
        }

        /* "aioesphomeapi/connection.py":661
 * 
 *         self._resolve_host_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._resolve_host_future, ConnectionInterruptedError, None
*/
      }
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L13_try_end;
      __pyx_L8_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "aioesphomeapi/connection.py":670
 *                     self._params.zeroconf_manager,
 *                 )
 *         except (Exception, CancelledError) as ex:             # <<<<<<<<<<<<<<
 *             # Clean up the connection; re-raise the original CancelledError
 *             # if the task is actually being cancelled so TaskGroup /
*/
      __pyx_t_17 = __Pyx_PyErr_ExceptionMatches2(((PyObject *)(((PyTypeObject*)PyExc_Exception))), __pyx_v_13aioesphomeapi_10connection_CancelledError);
      if (__pyx_t_17) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.start_resolve_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_8, &__pyx_t_2) < 0) __PYX_ERR(0, 670, __pyx_L10_except_error)
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_8);
        __pyx_cur_scope->__pyx_v_ex = __pyx_t_8;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":674
 *             # if the task is actually being cancelled so TaskGroup /
 *             # asyncio.timeout semantics are preserved.
 *             self._raise_fatal_connection_exception("resolving", ex)             # <<<<<<<<<<<<<<
 *         finally:
 *             self._set_resolve_host_future()
*/
          __pyx_t_18 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
          __Pyx_INCREF(__pyx_t_18);
          __pyx_t_4 = 0;
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_18, __pyx_mstate_global->__pyx_n_u_resolving, __pyx_cur_scope->__pyx_v_ex};
            __pyx_t_9 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_raise_fatal_connection_exceptio, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 674, __pyx_L37_error)
            __Pyx_GOTREF(__pyx_t_9);
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }

        /* "aioesphomeapi/connection.py":670
 *                     self._params.zeroconf_manager,
 *                 )
 *         except (Exception, CancelledError) as ex:             # <<<<<<<<<<<<<<
 *             # Clean up the connection; re-raise the original CancelledError
 *             # if the task is actually being cancelled so TaskGroup /
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
            __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
            goto __pyx_L38;
          }
          __pyx_L37_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_10 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_15 = 0; __pyx_t_21 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_12, &__pyx_t_15, &__pyx_t_21);
            if ( unlikely(__Pyx_GetException(&__pyx_t_10, &__pyx_t_14, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_10, &__pyx_t_14, &__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_14);
            __Pyx_XGOTREF(__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_21);
            __pyx_t_17 = __pyx_lineno; __pyx_t_19 = __pyx_clineno; __pyx_t_20 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_15, __pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_14);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ErrRestore(__pyx_t_10, __pyx_t_14, __pyx_t_13);
            __pyx_t_10 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_15 = 0; __pyx_t_21 = 0;
            __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_19; __pyx_filename = __pyx_t_20;
            goto __pyx_L10_except_error;
          }
          __pyx_L38:;
        }
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L9_exception_handled;
      }
      goto __pyx_L10_except_error;

      /* "aioesphomeapi/connection.py":661
 * 
 *         self._resolve_host_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._resolve_host_future, ConnectionInterruptedError, None
*/
      __pyx_L10_except_error:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      goto __pyx_L6_error;
      __pyx_L9_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      __pyx_L13_try_end:;
    }
  }

  /* "aioesphomeapi/connection.py":676
 *             self._raise_fatal_connection_exception("resolving", ex)
 *         finally:
 *             self._set_resolve_host_future()             # <<<<<<<<<<<<<<
 *         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_8 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
      __Pyx_INCREF(__pyx_t_8);
      __pyx_t_4 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_resolve_host_future, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_assign
      __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_15, &__pyx_t_12);
      if ( unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_12);
      __pyx_t_19 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_22 = __pyx_filename;
      {
        __pyx_t_8 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_4 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_resolve_host_future, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 676, __pyx_L44_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      }
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_15, __pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_6, __pyx_t_5);
      __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      __pyx_lineno = __pyx_t_19; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_22;
      goto __pyx_L1_error;
      __pyx_L44_error:;
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_15, __pyx_t_12);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "aioesphomeapi/connection.py":677
 *         finally:
 *             self._set_resolve_host_future()
 *         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)             # <<<<<<<<<<<<<<
 * 
 *     def _set_resolve_host_future(self) -> None:
*/
  __pyx_t_2 = __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_connection_state(__pyx_cur_scope->__pyx_v_self, __pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 677, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":649
 *         )
 * 
 *     async def start_resolve_host(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the host resolution process.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_18);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("start_resolve_host", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":679
 *         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)
 * 
 *     def _set_resolve_host_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._resolve_host_future is not None
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_25_set_resolve_host_future(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_25_set_resolve_host_future = {"_set_resolve_host_future", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_25_set_resolve_host_future, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_25_set_resolve_host_future(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_set_resolve_host_future (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("_set_resolve_host_future", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("_set_resolve_host_future", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_24_set_resolve_host_future(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_24_set_resolve_host_future(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_resolve_host_future", 0);

  /* "aioesphomeapi/connection.py":681
 *     def _set_resolve_host_future(self) -> None:
 *         if (
 *             self._resolve_host_future is not None             # <<<<<<<<<<<<<<
 *             and not self._resolve_host_future.done()
 *         ):
*/
  __pyx_t_2 = (__pyx_v_self->_resolve_host_future != Py_None);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "aioesphomeapi/connection.py":682
 *         if (
 *             self._resolve_host_future is not None
 *             and not self._resolve_host_future.done()             # <<<<<<<<<<<<<<
 *         ):
 *             self._resolve_host_future.set_result(None)
*/
  __pyx_t_4 = __pyx_v_self->_resolve_host_future;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_done, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 682, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 682, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = (!__pyx_t_2);
  __pyx_t_1 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;

  /* "aioesphomeapi/connection.py":680
 * 
 *     def _set_resolve_host_future(self) -> None:
 *         if (             # <<<<<<<<<<<<<<
 *             self._resolve_host_future is not None
 *             and not self._resolve_host_future.done()
*/
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":684
 *             and not self._resolve_host_future.done()
 *         ):
 *             self._resolve_host_future.set_result(None)             # <<<<<<<<<<<<<<
 *             self._resolve_host_future = None
 * 
*/
    __pyx_t_4 = __pyx_v_self->_resolve_host_future;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, Py_None};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_result, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 684, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":685
 *         ):
 *             self._resolve_host_future.set_result(None)
 *             self._resolve_host_future = None             # <<<<<<<<<<<<<<
 * 
 *     async def start_connection(self) -> None:
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_resolve_host_future);
    __Pyx_DECREF(__pyx_v_self->_resolve_host_future);
    __pyx_v_self->_resolve_host_future = Py_None;

    /* "aioesphomeapi/connection.py":680
 * 
 *     def _set_resolve_host_future(self) -> None:
 *         if (             # <<<<<<<<<<<<<<
 *             self._resolve_host_future is not None
 *             and not self._resolve_host_future.done()
*/
  }

  /* "aioesphomeapi/connection.py":679
 *         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)
 * 
 *     def _set_resolve_host_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._resolve_host_future is not None
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._set_resolve_host_future", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_28generator4(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":687
 *             self._resolve_host_future = None
 * 
 *     async def start_connection(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the connection process.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_27start_connection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_26start_connection, "Start the connection process.\n\n        This part of the process establishes the socket connection but\n        does not initialize the frame helper or send the hello message.\n        ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_27start_connection = {"start_connection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_27start_connection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_26start_connection};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_27start_connection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_connection (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("start_connection", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("start_connection", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_26start_connection(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_26start_connection(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("start_connection", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 687, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_28generator4, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_start_connection, __pyx_mstate_global->__pyx_n_u_APIConnection_start_connection, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 687, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.start_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_28generator4(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_PySendResult __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  char const *__pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  char const *__pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("start_connection", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L15_resume_from_await;
    case 2: goto __pyx_L25_resume_from_await;
    case 3: goto __pyx_L29_resume_from_await;
    case 4: goto __pyx_L30_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 687, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":693
 *         does not initialize the frame helper or send the hello message.
 *         """
 *         if self.connection_state is not CONNECTION_STATE_HOST_RESOLVED:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "Connection must be in HOST_RESOLVED state to start connection"
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_self->connection_state != __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED);
  if (unlikely(__pyx_t_1)) {

    /* "aioesphomeapi/connection.py":694
 *         """
 *         if self.connection_state is not CONNECTION_STATE_HOST_RESOLVED:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 "Connection must be in HOST_RESOLVED state to start connection"
 *             )
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Connection_must_be_in_HOST_RESOL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 694, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 694, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":693
 *         does not initialize the frame helper or send the hello message.
 *         """
 *         if self.connection_state is not CONNECTION_STATE_HOST_RESOLVED:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "Connection must be in HOST_RESOLVED state to start connection"
*/
  }

  /* "aioesphomeapi/connection.py":698
 *             )
 * 
 *         self._start_connect_future = self._loop.create_future()             # <<<<<<<<<<<<<<
 *         try:
 *             async with interrupt(
*/
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_create_future, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 698, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_start_connect_future);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_start_connect_future);
  __pyx_cur_scope->__pyx_v_self->_start_connect_future = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":699
 * 
 *         self._start_connect_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._start_connect_future, ConnectionInterruptedError, None
*/
  /*try:*/ {
    {
      __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":700
 *         self._start_connect_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._start_connect_future, ConnectionInterruptedError, None
 *             ):
*/
        /*with:*/ {
          __pyx_t_3 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_interrupt); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 700, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_8);

          /* "aioesphomeapi/connection.py":701
 *         try:
 *             async with interrupt(
 *                 self._start_connect_future, ConnectionInterruptedError, None             # <<<<<<<<<<<<<<
 *             ):
 *                 await self._connect_socket_connect(self._addrs_info)
*/
          __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 701, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_8);
            assert(__pyx_t_3);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_self->_start_connect_future, __pyx_t_9, Py_None};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (4-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 700, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __pyx_t_10 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aexit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 700, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_10);

          /* "aioesphomeapi/connection.py":700
 *         self._start_connect_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._start_connect_future, ConnectionInterruptedError, None
 *             ):
*/
          __pyx_t_9 = NULL;
          __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aenter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 700, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_9);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_9, NULL};
            __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 700, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_8);
          }
          __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_8, &__pyx_r);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (likely(__pyx_t_11 == PYGEN_NEXT)) {
            __Pyx_GOTREF(__pyx_r);
            __Pyx_XGIVEREF(__pyx_t_2);
            __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
            __Pyx_XGIVEREF(__pyx_t_5);
            __pyx_cur_scope->__pyx_t_1 = __pyx_t_5;
            __Pyx_XGIVEREF(__pyx_t_6);
            __pyx_cur_scope->__pyx_t_2 = __pyx_t_6;
            __Pyx_XGIVEREF(__pyx_t_7);
            __pyx_cur_scope->__pyx_t_3 = __pyx_t_7;
            __Pyx_XGIVEREF(__pyx_t_10);
            __pyx_cur_scope->__pyx_t_4 = __pyx_t_10;
            __Pyx_XGIVEREF(__pyx_r);
            __Pyx_RefNannyFinishContext();
            __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
            /* return from generator, awaiting value */
            __pyx_generator->resume_label = 1;
            return __pyx_r;
            __pyx_L15_resume_from_await:;
            __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
            __pyx_cur_scope->__pyx_t_0 = 0;
            __Pyx_XGOTREF(__pyx_t_2);
            __pyx_t_5 = __pyx_cur_scope->__pyx_t_1;
            __pyx_cur_scope->__pyx_t_1 = 0;
            __Pyx_XGOTREF(__pyx_t_5);
            __pyx_t_6 = __pyx_cur_scope->__pyx_t_2;
            __pyx_cur_scope->__pyx_t_2 = 0;
            __Pyx_XGOTREF(__pyx_t_6);
            __pyx_t_7 = __pyx_cur_scope->__pyx_t_3;
            __pyx_cur_scope->__pyx_t_3 = 0;
            __Pyx_XGOTREF(__pyx_t_7);
            __pyx_t_10 = __pyx_cur_scope->__pyx_t_4;
            __pyx_cur_scope->__pyx_t_4 = 0;
            __Pyx_XGOTREF(__pyx_t_10);
            if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 700, __pyx_L14_error)
            __pyx_t_8 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_8);
          } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
            __Pyx_GOTREF(__pyx_r);
            __pyx_t_8 = __pyx_r; __pyx_r = NULL;
          } else {
            __Pyx_XGOTREF(__pyx_r);
            __PYX_ERR(0, 700, __pyx_L14_error)
          }
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          /*try:*/ {
            {
              __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_14);
              /*try:*/ {

                /* "aioesphomeapi/connection.py":703
 *                 self._start_connect_future, ConnectionInterruptedError, None
 *             ):
 *                 await self._connect_socket_connect(self._addrs_info)             # <<<<<<<<<<<<<<
 *         except (Exception, CancelledError) as ex:
 *             # Clean up the connection; re-raise the original CancelledError
*/
                __pyx_t_8 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
                __Pyx_INCREF(__pyx_t_8);
                __pyx_t_4 = 0;
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_cur_scope->__pyx_v_self->_addrs_info};
                  __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_connect_socket_connect, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 703, __pyx_L19_error)
                  __Pyx_GOTREF(__pyx_t_2);
                }
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_12);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_12;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 2;
                  return __pyx_r;
                  __pyx_L25_resume_from_await:;
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_12 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_12);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 703, __pyx_L19_error)
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_DECREF(__pyx_r); __pyx_r = 0;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 703, __pyx_L19_error)
                }

                /* "aioesphomeapi/connection.py":700
 *         self._start_connect_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._start_connect_future, ConnectionInterruptedError, None
 *             ):
*/
              }
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              goto __pyx_L24_try_end;
              __pyx_L19_error:;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              /*except:*/ {
                __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.start_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_8, &__pyx_t_3) < 0) __PYX_ERR(0, 700, __pyx_L21_except_error)
                __Pyx_XGOTREF(__pyx_t_2);
                __Pyx_XGOTREF(__pyx_t_8);
                __Pyx_XGOTREF(__pyx_t_3);
                __pyx_t_9 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_8, __pyx_t_3); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 700, __pyx_L21_except_error)
                __Pyx_GOTREF(__pyx_t_9);
                __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_9, NULL);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 700, __pyx_L21_except_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_15, &__pyx_r);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_2);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
                  __Pyx_XGIVEREF(__pyx_t_3);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_8);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_12);
                  __pyx_cur_scope->__pyx_t_7 = __pyx_t_12;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_8 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_9 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_10 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_SwapException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 3;
                  return __pyx_r;
                  __pyx_L29_resume_from_await:;
                  __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_2);
                  __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_3);
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_8);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_12 = __pyx_cur_scope->__pyx_t_7;
                  __pyx_cur_scope->__pyx_t_7 = 0;
                  __Pyx_XGOTREF(__pyx_t_12);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_8;
                  __pyx_cur_scope->__pyx_t_8 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_9;
                  __pyx_cur_scope->__pyx_t_9 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_10;
                  __pyx_cur_scope->__pyx_t_10 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 700, __pyx_L21_except_error)
                  __pyx_t_9 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_9);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_9 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 700, __pyx_L21_except_error)
                }
                __pyx_t_15 = __pyx_t_9;
                __pyx_t_9 = 0;
                __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_15);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (__pyx_t_1 < (0)) __PYX_ERR(0, 700, __pyx_L21_except_error)
                __pyx_t_16 = (!__pyx_t_1);
                if (unlikely(__pyx_t_16)) {
                  __Pyx_GIVEREF(__pyx_t_2);
                  __Pyx_GIVEREF(__pyx_t_8);
                  __Pyx_XGIVEREF(__pyx_t_3);
                  __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_8, __pyx_t_3);
                  __pyx_t_2 = 0;  __pyx_t_8 = 0;  __pyx_t_3 = 0; 
                  __PYX_ERR(0, 700, __pyx_L21_except_error)
                }
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                goto __pyx_L20_exception_handled;
              }
              __pyx_L21_except_error:;
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              goto __pyx_L8_error;
              __pyx_L20_exception_handled:;
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              __pyx_L24_try_end:;
            }
          }
          /*finally:*/ {
            /*normal exit:*/{
              if (__pyx_t_10) {
                __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[0], NULL);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 700, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_14, &__pyx_r);
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 4;
                  return __pyx_r;
                  __pyx_L30_resume_from_await:;
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 700, __pyx_L8_error)
                  __pyx_t_3 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_3);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_3 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 700, __pyx_L8_error)
                }
                __pyx_t_14 = __pyx_t_3;
                __pyx_t_3 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              }
              goto __pyx_L18;
            }
            __pyx_L18:;
          }
          goto __pyx_L31;
          __pyx_L14_error:;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          goto __pyx_L8_error;
          __pyx_L31:;
        }

        /* "aioesphomeapi/connection.py":699
 * 
 *         self._start_connect_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._start_connect_future, ConnectionInterruptedError, None
*/
      }
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L13_try_end;
      __pyx_L8_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "aioesphomeapi/connection.py":704
 *             ):
 *                 await self._connect_socket_connect(self._addrs_info)
 *         except (Exception, CancelledError) as ex:             # <<<<<<<<<<<<<<
 *             # Clean up the connection; re-raise the original CancelledError
 *             # if the task is actually being cancelled so TaskGroup /
*/
      __pyx_t_17 = __Pyx_PyErr_ExceptionMatches2(((PyObject *)(((PyTypeObject*)PyExc_Exception))), __pyx_v_13aioesphomeapi_10connection_CancelledError);
      if (__pyx_t_17) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.start_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_8, &__pyx_t_2) < 0) __PYX_ERR(0, 704, __pyx_L10_except_error)
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_GIVEREF(__pyx_t_8);
        __pyx_cur_scope->__pyx_v_ex = __pyx_t_8;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":708
 *             # if the task is actually being cancelled so TaskGroup /
 *             # asyncio.timeout semantics are preserved.
 *             self._raise_fatal_connection_exception("starting", ex)             # <<<<<<<<<<<<<<
 *         finally:
 *             self._set_start_connect_future()
*/
          __pyx_t_18 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
          __Pyx_INCREF(__pyx_t_18);
          __pyx_t_4 = 0;
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_18, __pyx_mstate_global->__pyx_n_u_starting, __pyx_cur_scope->__pyx_v_ex};
            __pyx_t_9 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_raise_fatal_connection_exceptio, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 708, __pyx_L37_error)
            __Pyx_GOTREF(__pyx_t_9);
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }

        /* "aioesphomeapi/connection.py":704
 *             ):
 *                 await self._connect_socket_connect(self._addrs_info)
 *         except (Exception, CancelledError) as ex:             # <<<<<<<<<<<<<<
 *             # Clean up the connection; re-raise the original CancelledError
 *             # if the task is actually being cancelled so TaskGroup /
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
            __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
            goto __pyx_L38;
          }
          __pyx_L37_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_10 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_15 = 0; __pyx_t_21 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_12, &__pyx_t_15, &__pyx_t_21);
            if ( unlikely(__Pyx_GetException(&__pyx_t_10, &__pyx_t_14, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_10, &__pyx_t_14, &__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_14);
            __Pyx_XGOTREF(__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_21);
            __pyx_t_17 = __pyx_lineno; __pyx_t_19 = __pyx_clineno; __pyx_t_20 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_15, __pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_14);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ErrRestore(__pyx_t_10, __pyx_t_14, __pyx_t_13);
            __pyx_t_10 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_15 = 0; __pyx_t_21 = 0;
            __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_19; __pyx_filename = __pyx_t_20;
            goto __pyx_L10_except_error;
          }
          __pyx_L38:;
        }
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L9_exception_handled;
      }
      goto __pyx_L10_except_error;

      /* "aioesphomeapi/connection.py":699
 * 
 *         self._start_connect_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._start_connect_future, ConnectionInterruptedError, None
*/
      __pyx_L10_except_error:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      goto __pyx_L6_error;
      __pyx_L9_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      __pyx_L13_try_end:;
    }
  }

  /* "aioesphomeapi/connection.py":710
 *             self._raise_fatal_connection_exception("starting", ex)
 *         finally:
 *             self._set_start_connect_future()             # <<<<<<<<<<<<<<
 *         self._set_connection_state(CONNECTION_STATE_SOCKET_OPENED)
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_start_connect_future(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 710, __pyx_L1_error)
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_assign
      __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_15, &__pyx_t_12);
      if ( unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_12);
      __pyx_t_19 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_22 = __pyx_filename;
      {
        ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_start_connect_future(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 710, __pyx_L44_error)
      }
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_15, __pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_6, __pyx_t_5);
      __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      __pyx_lineno = __pyx_t_19; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_22;
      goto __pyx_L1_error;
      __pyx_L44_error:;
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_15, __pyx_t_12);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "aioesphomeapi/connection.py":711
 *         finally:
 *             self._set_start_connect_future()
 *         self._set_connection_state(CONNECTION_STATE_SOCKET_OPENED)             # <<<<<<<<<<<<<<
 * 
 *     def _set_start_connect_future(self) -> None:
*/
  __pyx_t_2 = __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_connection_state(__pyx_cur_scope->__pyx_v_self, __pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":687
 *             self._resolve_host_future = None
 * 
 *     async def start_connection(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the connection process.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_18);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("start_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":713
 *         self._set_connection_state(CONNECTION_STATE_SOCKET_OPENED)
 * 
 *     def _set_start_connect_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._start_connect_future is not None
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_start_connect_future(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_start_connect_future", 0);

  /* "aioesphomeapi/connection.py":715
 *     def _set_start_connect_future(self) -> None:
 *         if (
 *             self._start_connect_future is not None             # <<<<<<<<<<<<<<
 *             and not self._start_connect_future.done()
 *         ):
*/
  __pyx_t_2 = (__pyx_v_self->_start_connect_future != Py_None);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "aioesphomeapi/connection.py":716
 *         if (
 *             self._start_connect_future is not None
 *             and not self._start_connect_future.done()             # <<<<<<<<<<<<<<
 *         ):
 *             self._start_connect_future.set_result(None)
*/
  __pyx_t_4 = __pyx_v_self->_start_connect_future;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_done, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 716, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 716, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = (!__pyx_t_2);
  __pyx_t_1 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;

  /* "aioesphomeapi/connection.py":714
 * 
 *     def _set_start_connect_future(self) -> None:
 *         if (             # <<<<<<<<<<<<<<
 *             self._start_connect_future is not None
 *             and not self._start_connect_future.done()
*/
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":718
 *             and not self._start_connect_future.done()
 *         ):
 *             self._start_connect_future.set_result(None)             # <<<<<<<<<<<<<<
 *             self._start_connect_future = None
 * 
*/
    __pyx_t_4 = __pyx_v_self->_start_connect_future;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, Py_None};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_result, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 718, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":719
 *         ):
 *             self._start_connect_future.set_result(None)
 *             self._start_connect_future = None             # <<<<<<<<<<<<<<
 * 
 *     def _raise_fatal_connection_exception(
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_start_connect_future);
    __Pyx_DECREF(__pyx_v_self->_start_connect_future);
    __pyx_v_self->_start_connect_future = Py_None;

    /* "aioesphomeapi/connection.py":714
 * 
 *     def _set_start_connect_future(self) -> None:
 *         if (             # <<<<<<<<<<<<<<
 *             self._start_connect_future is not None
 *             and not self._start_connect_future.done()
*/
  }

  /* "aioesphomeapi/connection.py":713
 *         self._set_connection_state(CONNECTION_STATE_SOCKET_OPENED)
 * 
 *     def _set_start_connect_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._start_connect_future is not None
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._set_start_connect_future", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":721
 *             self._start_connect_future = None
 * 
 *     def _raise_fatal_connection_exception(             # <<<<<<<<<<<<<<
 *         self, action: str, ex: BaseException
 *     ) -> NoReturn:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_30_raise_fatal_connection_exception(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_29_raise_fatal_connection_exception, "Report a fatal connection exception and raise the appropriate error.\n\n        If the awaiting task is actually being cancelled by a parent (for\n        example, a ``TaskGroup`` or ``asyncio.timeout``), the original\n        ``CancelledError`` is re-raised so the task's cancellation state is\n        preserved. Otherwise the exception is wrapped as an\n        ``APIConnectionError`` subclass as before.\n        ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_30_raise_fatal_connection_exception = {"_raise_fatal_connection_exception", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_30_raise_fatal_connection_exception, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_29_raise_fatal_connection_exception};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_30_raise_fatal_connection_exception(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_action = 0;
  PyObject *__pyx_v_ex = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_raise_fatal_connection_exception (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_action,&__pyx_mstate_global->__pyx_n_u_ex,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 721, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 721, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 721, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_raise_fatal_connection_exception", 0) < (0)) __PYX_ERR(0, 721, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_raise_fatal_connection_exception", 1, 2, 2, i); __PYX_ERR(0, 721, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 721, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 721, __pyx_L3_error)
    }
    __pyx_v_action = ((PyObject*)values[0]);
    __pyx_v_ex = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_raise_fatal_connection_exception", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 721, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._raise_fatal_connection_exception", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_action), (&PyUnicode_Type), 0, "action", 2))) __PYX_ERR(0, 722, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_ex), (((PyTypeObject*)PyExc_BaseException)), 0, "ex", 0))) __PYX_ERR(0, 722, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_29_raise_fatal_connection_exception(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_action, __pyx_v_ex);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_29_raise_fatal_connection_exception(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_action, PyObject *__pyx_v_ex) {
  PyObject *__pyx_v_fatal_exc = NULL;
  PyObject *__pyx_v_current_task = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_raise_fatal_connection_exception", 0);

  /* "aioesphomeapi/connection.py":732
 *         ``APIConnectionError`` subclass as before.
 *         """
 *         fatal_exc = self._wrap_fatal_connection_exception(action, ex)             # <<<<<<<<<<<<<<
 *         self.report_fatal_error(fatal_exc)
 *         if isinstance(ex, CancelledError) and (
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_2, __pyx_v_action, __pyx_v_ex};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_wrap_fatal_connection_exception, __pyx_callargs+__pyx_t_3, (3-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 732, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_fatal_exc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":733
 *         """
 *         fatal_exc = self._wrap_fatal_connection_exception(action, ex)
 *         self.report_fatal_error(fatal_exc)             # <<<<<<<<<<<<<<
 *         if isinstance(ex, CancelledError) and (
 *             (current_task := asyncio.current_task()) is not None
*/
  __pyx_t_1 = __pyx_v_fatal_exc;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(__Pyx_PyException_Check(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_1))) __PYX_ERR(0, 733, __pyx_L1_error)
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->report_fatal_error(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 733, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":734
 *         fatal_exc = self._wrap_fatal_connection_exception(action, ex)
 *         self.report_fatal_error(fatal_exc)
 *         if isinstance(ex, CancelledError) and (             # <<<<<<<<<<<<<<
 *             (current_task := asyncio.current_task()) is not None
 *             and current_task.cancelling() > 0
*/
  __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_CancelledError;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = PyObject_IsInstance(__pyx_v_ex, __pyx_t_1); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 734, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }

  /* "aioesphomeapi/connection.py":735
 *         self.report_fatal_error(fatal_exc)
 *         if isinstance(ex, CancelledError) and (
 *             (current_task := asyncio.current_task()) is not None             # <<<<<<<<<<<<<<
 *             and current_task.cancelling() > 0
 *         ):
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_asyncio); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_current_task); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 735, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 735, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_current_task = __pyx_t_1;
  __pyx_t_5 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {
  } else {
    __pyx_t_4 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }

  /* "aioesphomeapi/connection.py":736
 *         if isinstance(ex, CancelledError) and (
 *             (current_task := asyncio.current_task()) is not None
 *             and current_task.cancelling() > 0             # <<<<<<<<<<<<<<
 *         ):
 *             raise ex
*/
  __pyx_t_7 = __pyx_v_current_task;
  __Pyx_INCREF(__pyx_t_7);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancelling, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 736, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_7 = PyObject_RichCompare(__pyx_t_1, __pyx_mstate_global->__pyx_int_0, Py_GT); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_7); if (unlikely((__pyx_t_5 < 0))) __PYX_ERR(0, 736, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_4 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;

  /* "aioesphomeapi/connection.py":734
 *         fatal_exc = self._wrap_fatal_connection_exception(action, ex)
 *         self.report_fatal_error(fatal_exc)
 *         if isinstance(ex, CancelledError) and (             # <<<<<<<<<<<<<<
 *             (current_task := asyncio.current_task()) is not None
 *             and current_task.cancelling() > 0
*/
  if (unlikely(__pyx_t_4)) {

    /* "aioesphomeapi/connection.py":738
 *             and current_task.cancelling() > 0
 *         ):
 *             raise ex             # <<<<<<<<<<<<<<
 *         raise fatal_exc
 * 
*/
    __Pyx_Raise(__pyx_v_ex, 0, 0, 0);
    __PYX_ERR(0, 738, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":734
 *         fatal_exc = self._wrap_fatal_connection_exception(action, ex)
 *         self.report_fatal_error(fatal_exc)
 *         if isinstance(ex, CancelledError) and (             # <<<<<<<<<<<<<<
 *             (current_task := asyncio.current_task()) is not None
 *             and current_task.cancelling() > 0
*/
  }

  /* "aioesphomeapi/connection.py":739
 *         ):
 *             raise ex
 *         raise fatal_exc             # <<<<<<<<<<<<<<
 * 
 *     def _wrap_fatal_connection_exception(
*/
  __Pyx_Raise(__pyx_v_fatal_exc, 0, 0, 0);
  __PYX_ERR(0, 739, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":721
 *             self._start_connect_future = None
 * 
 *     def _raise_fatal_connection_exception(             # <<<<<<<<<<<<<<
 *         self, action: str, ex: BaseException
 *     ) -> NoReturn:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._raise_fatal_connection_exception", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XDECREF(__pyx_v_fatal_exc);
  __Pyx_XDECREF(__pyx_v_current_task);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":741
 *         raise fatal_exc
 * 
 *     def _wrap_fatal_connection_exception(             # <<<<<<<<<<<<<<
 *         self, action: str, ex: BaseException
 *     ) -> APIConnectionError:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_32_wrap_fatal_connection_exception(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_31_wrap_fatal_connection_exception, "Ensure a fatal exception is wrapped as as an APIConnectionError.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_32_wrap_fatal_connection_exception = {"_wrap_fatal_connection_exception", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_32_wrap_fatal_connection_exception, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_31_wrap_fatal_connection_exception};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_32_wrap_fatal_connection_exception(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_action = 0;
  PyObject *__pyx_v_ex = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_wrap_fatal_connection_exception (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_action,&__pyx_mstate_global->__pyx_n_u_ex,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 741, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 741, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 741, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_wrap_fatal_connection_exception", 0) < (0)) __PYX_ERR(0, 741, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_wrap_fatal_connection_exception", 1, 2, 2, i); __PYX_ERR(0, 741, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 741, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 741, __pyx_L3_error)
    }
    __pyx_v_action = ((PyObject*)values[0]);
    __pyx_v_ex = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_wrap_fatal_connection_exception", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 741, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._wrap_fatal_connection_exception", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_action), (&PyUnicode_Type), 0, "action", 2))) __PYX_ERR(0, 742, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_ex), (((PyTypeObject*)PyExc_BaseException)), 0, "ex", 0))) __PYX_ERR(0, 742, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_31_wrap_fatal_connection_exception(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_action, __pyx_v_ex);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_31_wrap_fatal_connection_exception(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_action, PyObject *__pyx_v_ex) {
  PyObject *__pyx_v_cause = 0;
  PyObject *__pyx_v_err_str = NULL;
  PyObject *__pyx_v_klass = NULL;
  PyObject *__pyx_v_new_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7[4];
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_wrap_fatal_connection_exception", 0);

  /* "aioesphomeapi/connection.py":745
 *     ) -> APIConnectionError:
 *         """Ensure a fatal exception is wrapped as as an APIConnectionError."""
 *         if isinstance(ex, APIConnectionError):             # <<<<<<<<<<<<<<
 *             return ex
 *         cause: BaseException | None = None
*/
  __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_APIConnectionError;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_ex, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 745, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":746
 *         """Ensure a fatal exception is wrapped as as an APIConnectionError."""
 *         if isinstance(ex, APIConnectionError):
 *             return ex             # <<<<<<<<<<<<<<
 *         cause: BaseException | None = None
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_ex);
    __pyx_r = __pyx_v_ex;
    goto __pyx_L0;

    /* "aioesphomeapi/connection.py":745
 *     ) -> APIConnectionError:
 *         """Ensure a fatal exception is wrapped as as an APIConnectionError."""
 *         if isinstance(ex, APIConnectionError):             # <<<<<<<<<<<<<<
 *             return ex
 *         cause: BaseException | None = None
*/
  }

  /* "aioesphomeapi/connection.py":747
 *         if isinstance(ex, APIConnectionError):
 *             return ex
 *         cause: BaseException | None = None             # <<<<<<<<<<<<<<
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):
 *             err_str = f"{action.title()} connection cancelled"
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_cause = ((PyObject*)Py_None);

  /* "aioesphomeapi/connection.py":748
 *             return ex
 *         cause: BaseException | None = None
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):             # <<<<<<<<<<<<<<
 *             err_str = f"{action.title()} connection cancelled"
 *             if self._fatal_exception:
*/
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 748, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_CancelledError);
  __pyx_t_3 = __pyx_v_13aioesphomeapi_10connection_CancelledError;
  __pyx_t_4 = PyObject_IsInstance(__pyx_v_ex, __pyx_t_1); 
  if (!__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_4 = PyObject_IsInstance(__pyx_v_ex, __pyx_t_3); 
  __pyx_t_2 = __pyx_t_4;
  __pyx_L5_bool_binop_done:;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":749
 *         cause: BaseException | None = None
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):
 *             err_str = f"{action.title()} connection cancelled"             # <<<<<<<<<<<<<<
 *             if self._fatal_exception:
 *                 err_str += f" due to fatal exception: {self._fatal_exception}"
*/
    __pyx_t_1 = __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyUnicode_Type__title, __pyx_v_action); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_connection_cancelled); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_err_str = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":750
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):
 *             err_str = f"{action.title()} connection cancelled"
 *             if self._fatal_exception:             # <<<<<<<<<<<<<<
 *                 err_str += f" due to fatal exception: {self._fatal_exception}"
 *                 cause = self._fatal_exception
*/
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_self->_fatal_exception); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 750, __pyx_L1_error)
    if (__pyx_t_2) {

      /* "aioesphomeapi/connection.py":751
 *             err_str = f"{action.title()} connection cancelled"
 *             if self._fatal_exception:
 *                 err_str += f" due to fatal exception: {self._fatal_exception}"             # <<<<<<<<<<<<<<
 *                 cause = self._fatal_exception
 *         else:
*/
      __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_self->_fatal_exception, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 751, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __Pyx_PyUnicode_Concat(__pyx_mstate_global->__pyx_kp_u_due_to_fatal_exception, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 751, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyUnicode_Concat__Pyx_ReferenceSharing_OwnStrongReferenceInPlace(__pyx_v_err_str, __pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 751, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF_SET(__pyx_v_err_str, __pyx_t_3);
      __pyx_t_3 = 0;

      /* "aioesphomeapi/connection.py":752
 *             if self._fatal_exception:
 *                 err_str += f" due to fatal exception: {self._fatal_exception}"
 *                 cause = self._fatal_exception             # <<<<<<<<<<<<<<
 *         else:
 *             err_str = str(ex) or type(ex).__name__
*/
      __pyx_t_3 = __pyx_v_self->_fatal_exception;
      __Pyx_INCREF(__pyx_t_3);
      if (!(likely(__Pyx_PyBaseException_Check(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("BaseException", __pyx_t_3))) __PYX_ERR(0, 752, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_cause, ((PyObject*)__pyx_t_3));
      __pyx_t_3 = 0;

      /* "aioesphomeapi/connection.py":750
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):
 *             err_str = f"{action.title()} connection cancelled"
 *             if self._fatal_exception:             # <<<<<<<<<<<<<<
 *                 err_str += f" due to fatal exception: {self._fatal_exception}"
 *                 cause = self._fatal_exception
*/
    }

    /* "aioesphomeapi/connection.py":748
 *             return ex
 *         cause: BaseException | None = None
 *         if isinstance(ex, (ConnectionInterruptedError, CancelledError)):             # <<<<<<<<<<<<<<
 *             err_str = f"{action.title()} connection cancelled"
 *             if self._fatal_exception:
*/
    goto __pyx_L4;
  }

  /* "aioesphomeapi/connection.py":754
 *                 cause = self._fatal_exception
 *         else:
 *             err_str = str(ex) or type(ex).__name__             # <<<<<<<<<<<<<<
 *             cause = ex
 *         if isinstance(self._fatal_exception, APIConnectionError):
*/
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Unicode(__pyx_v_ex); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 754, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 754, __pyx_L1_error)
    if (!__pyx_t_2) {
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_3 = __pyx_t_1;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      goto __pyx_L8_bool_binop_done;
    }
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_ex)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 754, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = __pyx_t_1;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_L8_bool_binop_done:;
    __pyx_v_err_str = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":755
 *         else:
 *             err_str = str(ex) or type(ex).__name__
 *             cause = ex             # <<<<<<<<<<<<<<
 *         if isinstance(self._fatal_exception, APIConnectionError):
 *             klass = type(self._fatal_exception)
*/
    __Pyx_INCREF(__pyx_v_ex);
    __Pyx_DECREF_SET(__pyx_v_cause, __pyx_v_ex);
  }
  __pyx_L4:;

  /* "aioesphomeapi/connection.py":756
 *             err_str = str(ex) or type(ex).__name__
 *             cause = ex
 *         if isinstance(self._fatal_exception, APIConnectionError):             # <<<<<<<<<<<<<<
 *             klass = type(self._fatal_exception)
 *         elif isinstance(ex, CancelledError):
*/
  __pyx_t_3 = __pyx_v_self->_fatal_exception;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_APIConnectionError;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_t_3, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 756, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":757
 *             cause = ex
 *         if isinstance(self._fatal_exception, APIConnectionError):
 *             klass = type(self._fatal_exception)             # <<<<<<<<<<<<<<
 *         elif isinstance(ex, CancelledError):
 *             klass = APIConnectionCancelledError
*/
    __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_self->_fatal_exception)));
    __pyx_v_klass = ((PyObject *)Py_TYPE(__pyx_v_self->_fatal_exception));

    /* "aioesphomeapi/connection.py":756
 *             err_str = str(ex) or type(ex).__name__
 *             cause = ex
 *         if isinstance(self._fatal_exception, APIConnectionError):             # <<<<<<<<<<<<<<
 *             klass = type(self._fatal_exception)
 *         elif isinstance(ex, CancelledError):
*/
    goto __pyx_L10;
  }

  /* "aioesphomeapi/connection.py":758
 *         if isinstance(self._fatal_exception, APIConnectionError):
 *             klass = type(self._fatal_exception)
 *         elif isinstance(ex, CancelledError):             # <<<<<<<<<<<<<<
 *             klass = APIConnectionCancelledError
 *         elif isinstance(ex, OSError):
*/
  __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_CancelledError;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_ex, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 758, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":759
 *             klass = type(self._fatal_exception)
 *         elif isinstance(ex, CancelledError):
 *             klass = APIConnectionCancelledError             # <<<<<<<<<<<<<<
 *         elif isinstance(ex, OSError):
 *             klass = SocketAPIError
*/
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_APIConnectionCancelledError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 759, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_klass = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":758
 *         if isinstance(self._fatal_exception, APIConnectionError):
 *             klass = type(self._fatal_exception)
 *         elif isinstance(ex, CancelledError):             # <<<<<<<<<<<<<<
 *             klass = APIConnectionCancelledError
 *         elif isinstance(ex, OSError):
*/
    goto __pyx_L10;
  }

  /* "aioesphomeapi/connection.py":760
 *         elif isinstance(ex, CancelledError):
 *             klass = APIConnectionCancelledError
 *         elif isinstance(ex, OSError):             # <<<<<<<<<<<<<<
 *             klass = SocketAPIError
 *         else:
*/
  __pyx_t_2 = __Pyx_PyExc_OSError_Check(__pyx_v_ex); 
  if (__pyx_t_2) {

    /* "aioesphomeapi/connection.py":761
 *             klass = APIConnectionCancelledError
 *         elif isinstance(ex, OSError):
 *             klass = SocketAPIError             # <<<<<<<<<<<<<<
 *         else:
 *             klass = UnhandledAPIConnectionError
*/
    __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_SocketAPIError);
    __pyx_v_klass = __pyx_v_13aioesphomeapi_10connection_SocketAPIError;

    /* "aioesphomeapi/connection.py":760
 *         elif isinstance(ex, CancelledError):
 *             klass = APIConnectionCancelledError
 *         elif isinstance(ex, OSError):             # <<<<<<<<<<<<<<
 *             klass = SocketAPIError
 *         else:
*/
    goto __pyx_L10;
  }

  /* "aioesphomeapi/connection.py":763
 *             klass = SocketAPIError
 *         else:
 *             klass = UnhandledAPIConnectionError             # <<<<<<<<<<<<<<
 *         new_exc = klass(f"Error while {action} connection: {err_str}")
 *         new_exc.__cause__ = cause or ex
*/
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_UnhandledAPIConnectionError); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 763, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_v_klass = __pyx_t_1;
    __pyx_t_1 = 0;
  }
  __pyx_L10:;

  /* "aioesphomeapi/connection.py":764
 *         else:
 *             klass = UnhandledAPIConnectionError
 *         new_exc = klass(f"Error while {action} connection: {err_str}")             # <<<<<<<<<<<<<<
 *         new_exc.__cause__ = cause or ex
 *         return new_exc
*/
  __pyx_t_3 = NULL;
  __Pyx_INCREF(__pyx_v_klass);
  __pyx_t_5 = __pyx_v_klass; 
  __pyx_t_6 = __Pyx_PyObject_FormatSimple(__pyx_v_err_str, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7[0] = __pyx_mstate_global->__pyx_kp_u_Error_while;
  __pyx_t_7[1] = __pyx_v_action;
  __pyx_t_7[2] = __pyx_mstate_global->__pyx_kp_u_connection_2;
  __pyx_t_7[3] = __pyx_t_6;
  __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_7, 4, 12 + __Pyx_PyUnicode_GET_LENGTH(__pyx_v_action) + 13 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_6), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_v_action) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_6));
  if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 764, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_8};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 764, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_new_exc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":765
 *             klass = UnhandledAPIConnectionError
 *         new_exc = klass(f"Error while {action} connection: {err_str}")
 *         new_exc.__cause__ = cause or ex             # <<<<<<<<<<<<<<
 *         return new_exc
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_cause); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 765, __pyx_L1_error)
  if (!__pyx_t_2) {
  } else {
    __Pyx_INCREF(__pyx_v_cause);
    __pyx_t_1 = __pyx_v_cause;
    goto __pyx_L11_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_v_ex);
  __pyx_t_1 = __pyx_v_ex;
  __pyx_L11_bool_binop_done:;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_new_exc, __pyx_mstate_global->__pyx_n_u_cause, __pyx_t_1) < (0)) __PYX_ERR(0, 765, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":766
 *         new_exc = klass(f"Error while {action} connection: {err_str}")
 *         new_exc.__cause__ = cause or ex
 *         return new_exc             # <<<<<<<<<<<<<<
 * 
 *     async def _do_finish_connect(self, login: bool) -> None:
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_new_exc);
  __pyx_r = __pyx_v_new_exc;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":741
 *         raise fatal_exc
 * 
 *     def _wrap_fatal_connection_exception(             # <<<<<<<<<<<<<<
 *         self, action: str, ex: BaseException
 *     ) -> APIConnectionError:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._wrap_fatal_connection_exception", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_cause);
  __Pyx_XDECREF(__pyx_v_err_str);
  __Pyx_XDECREF(__pyx_v_klass);
  __Pyx_XDECREF(__pyx_v_new_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_35generator5(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":768
 *         return new_exc
 * 
 *     async def _do_finish_connect(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process."""
 *         # Cache timezone before registering handlers to ensure it's available
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_34_do_finish_connect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_33_do_finish_connect, "Finish the connection process.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_34_do_finish_connect = {"_do_finish_connect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_34_do_finish_connect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_33_do_finish_connect};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_34_do_finish_connect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_login;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_do_finish_connect (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_login,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 768, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 768, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_do_finish_connect", 0) < (0)) __PYX_ERR(0, 768, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_do_finish_connect", 1, 1, 1, i); __PYX_ERR(0, 768, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 768, __pyx_L3_error)
    }
    __pyx_v_login = __Pyx_PyObject_IsTrue(values[0]); if (unlikely((__pyx_v_login == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 768, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_do_finish_connect", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 768, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._do_finish_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_33_do_finish_connect(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_login);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_33_do_finish_connect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_login) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_do_finish_connect", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 768, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_login = __pyx_v_login;
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_35generator5, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_do_finish_connect, __pyx_mstate_global->__pyx_n_u_APIConnection__do_finish_connect, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 768, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._do_finish_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_35generator5(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  __Pyx_PySendResult __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_do_finish_connect", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L4_resume_from_await;
    case 2: goto __pyx_L5_resume_from_await;
    case 3: goto __pyx_L6_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 768, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":774
 *         # Use provided timezone from params (converted from IANA to POSIX if needed),
 *         # or fall back to local timezone detection
 *         self._cached_timezone = await get_timezone(self._params.timezone)             # <<<<<<<<<<<<<<
 *         # Register internal handlers before
 *         # connecting the helper so we can ensure
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_get_timezone); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_cur_scope->__pyx_v_self->_params->timezone};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 774, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_1, &__pyx_r);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_5 == PYGEN_NEXT)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, awaiting value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L4_resume_from_await:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 774, __pyx_L1_error)
    __pyx_t_1 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_1);
  } else if (likely(__pyx_t_5 == PYGEN_RETURN)) {
    __Pyx_GOTREF(__pyx_r);
    __pyx_t_1 = __pyx_r; __pyx_r = NULL;
  } else {
    __Pyx_XGOTREF(__pyx_r);
    __PYX_ERR(0, 774, __pyx_L1_error)
  }
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(0, 774, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_cached_timezone);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_cached_timezone);
  __pyx_cur_scope->__pyx_v_self->_cached_timezone = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":778
 *         # connecting the helper so we can ensure
 *         # we handle any messages that are received immediately
 *         self._register_internal_message_handlers()             # <<<<<<<<<<<<<<
 *         await self._connect_init_frame_helper()
 *         await self._connect_hello_login(login)
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_register_internal_message_handlers(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 778, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":779
 *         # we handle any messages that are received immediately
 *         self._register_internal_message_handlers()
 *         await self._connect_init_frame_helper()             # <<<<<<<<<<<<<<
 *         await self._connect_hello_login(login)
 *         self._async_schedule_keep_alive(self._loop.time())
*/
  __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_connect_init_frame_helper, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 779, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_1, &__pyx_r);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_5 == PYGEN_NEXT)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, awaiting value */
    __pyx_generator->resume_label = 2;
    return __pyx_r;
    __pyx_L5_resume_from_await:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 779, __pyx_L1_error)
  } else if (likely(__pyx_t_5 == PYGEN_RETURN)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_DECREF(__pyx_r); __pyx_r = 0;
  } else {
    __Pyx_XGOTREF(__pyx_r);
    __PYX_ERR(0, 779, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":780
 *         self._register_internal_message_handlers()
 *         await self._connect_init_frame_helper()
 *         await self._connect_hello_login(login)             # <<<<<<<<<<<<<<
 *         self._async_schedule_keep_alive(self._loop.time())
 * 
*/
  __pyx_t_3 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_cur_scope->__pyx_v_login); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 780, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_2};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_connect_hello_login, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 780, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_5 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_1, &__pyx_r);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_5 == PYGEN_NEXT)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, awaiting value */
    __pyx_generator->resume_label = 3;
    return __pyx_r;
    __pyx_L6_resume_from_await:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 780, __pyx_L1_error)
  } else if (likely(__pyx_t_5 == PYGEN_RETURN)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_DECREF(__pyx_r); __pyx_r = 0;
  } else {
    __Pyx_XGOTREF(__pyx_r);
    __PYX_ERR(0, 780, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":781
 *         await self._connect_init_frame_helper()
 *         await self._connect_hello_login(login)
 *         self._async_schedule_keep_alive(self._loop.time())             # <<<<<<<<<<<<<<
 * 
 *     async def finish_connection(self, *, login: bool) -> None:
*/
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_time, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 781, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_async_schedule_keep_alive(__pyx_cur_scope->__pyx_v_self, __pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 781, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":768
 *         return new_exc
 * 
 *     async def _do_finish_connect(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process."""
 *         # Cache timezone before registering handlers to ensure it's available
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("_do_finish_connect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_38generator6(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":783
 *         self._async_schedule_keep_alive(self._loop.time())
 * 
 *     async def finish_connection(self, *, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_37finish_connection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_36finish_connection, "Finish the connection process.\n\n        This part of the process initializes the frame helper and sends the hello message\n        than starts the keep alive process.\n        ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_37finish_connection = {"finish_connection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_37finish_connection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_36finish_connection};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_37finish_connection(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  int __pyx_v_login;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("finish_connection (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_login,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 783, __pyx_L3_error)
    if (likely(__pyx_kwds_len > 0)) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "finish_connection", 0) < (0)) __PYX_ERR(0, 783, __pyx_L3_error)
      for (Py_ssize_t i = 0; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseKeywordRequired("finish_connection", *(__pyx_pyargnames[i - 0])); __PYX_ERR(0, 783, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("finish_connection", __pyx_mstate_global->__pyx_n_u_login); __PYX_ERR(0, 783, __pyx_L3_error)
    }
    __pyx_v_login = __Pyx_PyObject_IsTrue(values[0]); if (unlikely((__pyx_v_login == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 783, __pyx_L3_error)
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("finish_connection", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 783, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.finish_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_36finish_connection(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_login);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_36finish_connection(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, int __pyx_v_login) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("finish_connection", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 783, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_login = __pyx_v_login;
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_38generator6, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_finish_connection, __pyx_mstate_global->__pyx_n_u_APIConnection_finish_connection, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 783, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.finish_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_38generator6(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  __Pyx_PySendResult __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_t_16;
  int __pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  char const *__pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  char const *__pyx_t_22;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("finish_connection", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L15_resume_from_await;
    case 2: goto __pyx_L25_resume_from_await;
    case 3: goto __pyx_L29_resume_from_await;
    case 4: goto __pyx_L30_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 783, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":789
 *         than starts the keep alive process.
 *         """
 *         if self.connection_state is not CONNECTION_STATE_SOCKET_OPENED:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "Connection must be in SOCKET_OPENED state to finish connection"
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_self->connection_state != __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED);
  if (unlikely(__pyx_t_1)) {

    /* "aioesphomeapi/connection.py":790
 *         """
 *         if self.connection_state is not CONNECTION_STATE_SOCKET_OPENED:
 *             raise RuntimeError(             # <<<<<<<<<<<<<<
 *                 "Connection must be in SOCKET_OPENED state to finish connection"
 *             )
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Connection_must_be_in_SOCKET_OPE};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 790, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 790, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":789
 *         than starts the keep alive process.
 *         """
 *         if self.connection_state is not CONNECTION_STATE_SOCKET_OPENED:             # <<<<<<<<<<<<<<
 *             raise RuntimeError(
 *                 "Connection must be in SOCKET_OPENED state to finish connection"
*/
  }

  /* "aioesphomeapi/connection.py":793
 *                 "Connection must be in SOCKET_OPENED state to finish connection"
 *             )
 *         self._finish_connect_future = self._loop.create_future()             # <<<<<<<<<<<<<<
 *         try:
 *             async with interrupt(
*/
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_create_future, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 793, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_self->_finish_connect_future);
  __Pyx_DECREF(__pyx_cur_scope->__pyx_v_self->_finish_connect_future);
  __pyx_cur_scope->__pyx_v_self->_finish_connect_future = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":794
 *             )
 *         self._finish_connect_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._finish_connect_future, ConnectionInterruptedError, None
*/
  /*try:*/ {
    {
      __Pyx_ExceptionSave(&__pyx_t_5, &__pyx_t_6, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_7);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":795
 *         self._finish_connect_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._finish_connect_future, ConnectionInterruptedError, None
 *             ):
*/
        /*with:*/ {
          __pyx_t_3 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_interrupt); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 795, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_8);

          /* "aioesphomeapi/connection.py":796
 *         try:
 *             async with interrupt(
 *                 self._finish_connect_future, ConnectionInterruptedError, None             # <<<<<<<<<<<<<<
 *             ):
 *                 await self._do_finish_connect(login)
*/
          __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 796, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_9);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_8);
            assert(__pyx_t_3);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_cur_scope->__pyx_v_self->_finish_connect_future, __pyx_t_9, Py_None};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_4, (4-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 795, __pyx_L8_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __pyx_t_10 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aexit); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 795, __pyx_L8_error)
          __Pyx_GOTREF(__pyx_t_10);

          /* "aioesphomeapi/connection.py":795
 *         self._finish_connect_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._finish_connect_future, ConnectionInterruptedError, None
 *             ):
*/
          __pyx_t_9 = NULL;
          __pyx_t_3 = __Pyx_PyObject_LookupSpecial(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_aenter); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 795, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_4 = 1;
          #if CYTHON_UNPACK_METHODS
          if (likely(PyMethod_Check(__pyx_t_3))) {
            __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_3);
            assert(__pyx_t_9);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
            __pyx_t_4 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_9, NULL};
            __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 795, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_8);
          }
          __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_8, &__pyx_r);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (likely(__pyx_t_11 == PYGEN_NEXT)) {
            __Pyx_GOTREF(__pyx_r);
            __Pyx_XGIVEREF(__pyx_t_2);
            __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
            __Pyx_XGIVEREF(__pyx_t_5);
            __pyx_cur_scope->__pyx_t_1 = __pyx_t_5;
            __Pyx_XGIVEREF(__pyx_t_6);
            __pyx_cur_scope->__pyx_t_2 = __pyx_t_6;
            __Pyx_XGIVEREF(__pyx_t_7);
            __pyx_cur_scope->__pyx_t_3 = __pyx_t_7;
            __Pyx_XGIVEREF(__pyx_t_10);
            __pyx_cur_scope->__pyx_t_4 = __pyx_t_10;
            __Pyx_XGIVEREF(__pyx_r);
            __Pyx_RefNannyFinishContext();
            __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
            /* return from generator, awaiting value */
            __pyx_generator->resume_label = 1;
            return __pyx_r;
            __pyx_L15_resume_from_await:;
            __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
            __pyx_cur_scope->__pyx_t_0 = 0;
            __Pyx_XGOTREF(__pyx_t_2);
            __pyx_t_5 = __pyx_cur_scope->__pyx_t_1;
            __pyx_cur_scope->__pyx_t_1 = 0;
            __Pyx_XGOTREF(__pyx_t_5);
            __pyx_t_6 = __pyx_cur_scope->__pyx_t_2;
            __pyx_cur_scope->__pyx_t_2 = 0;
            __Pyx_XGOTREF(__pyx_t_6);
            __pyx_t_7 = __pyx_cur_scope->__pyx_t_3;
            __pyx_cur_scope->__pyx_t_3 = 0;
            __Pyx_XGOTREF(__pyx_t_7);
            __pyx_t_10 = __pyx_cur_scope->__pyx_t_4;
            __pyx_cur_scope->__pyx_t_4 = 0;
            __Pyx_XGOTREF(__pyx_t_10);
            if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 795, __pyx_L14_error)
            __pyx_t_8 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_8);
          } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
            __Pyx_GOTREF(__pyx_r);
            __pyx_t_8 = __pyx_r; __pyx_r = NULL;
          } else {
            __Pyx_XGOTREF(__pyx_r);
            __PYX_ERR(0, 795, __pyx_L14_error)
          }
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          /*try:*/ {
            {
              __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
              __Pyx_XGOTREF(__pyx_t_12);
              __Pyx_XGOTREF(__pyx_t_13);
              __Pyx_XGOTREF(__pyx_t_14);
              /*try:*/ {

                /* "aioesphomeapi/connection.py":798
 *                 self._finish_connect_future, ConnectionInterruptedError, None
 *             ):
 *                 await self._do_finish_connect(login)             # <<<<<<<<<<<<<<
 *         except (Exception, CancelledError) as ex:
 *             # Clean up the connection; re-raise the original CancelledError
*/
                __pyx_t_8 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
                __Pyx_INCREF(__pyx_t_8);
                __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_cur_scope->__pyx_v_login); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 798, __pyx_L19_error)
                __Pyx_GOTREF(__pyx_t_3);
                __pyx_t_4 = 0;
                {
                  PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_3};
                  __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_do_finish_connect, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                  __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
                  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 798, __pyx_L19_error)
                  __Pyx_GOTREF(__pyx_t_2);
                }
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_12);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_12;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 2;
                  return __pyx_r;
                  __pyx_L25_resume_from_await:;
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_12 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_12);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 798, __pyx_L19_error)
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_DECREF(__pyx_r); __pyx_r = 0;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 798, __pyx_L19_error)
                }

                /* "aioesphomeapi/connection.py":795
 *         self._finish_connect_future = self._loop.create_future()
 *         try:
 *             async with interrupt(             # <<<<<<<<<<<<<<
 *                 self._finish_connect_future, ConnectionInterruptedError, None
 *             ):
*/
              }
              __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
              __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
              goto __pyx_L24_try_end;
              __pyx_L19_error:;
              __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              /*except:*/ {
                __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.finish_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
                if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_3, &__pyx_t_8) < 0) __PYX_ERR(0, 795, __pyx_L21_except_error)
                __Pyx_XGOTREF(__pyx_t_2);
                __Pyx_XGOTREF(__pyx_t_3);
                __Pyx_XGOTREF(__pyx_t_8);
                __pyx_t_9 = PyTuple_Pack(3, __pyx_t_2, __pyx_t_3, __pyx_t_8); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 795, __pyx_L21_except_error)
                __Pyx_GOTREF(__pyx_t_9);
                __pyx_t_15 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_t_9, NULL);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
                if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 795, __pyx_L21_except_error)
                __Pyx_GOTREF(__pyx_t_15);
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_15, &__pyx_r);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_2);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_2;
                  __Pyx_XGIVEREF(__pyx_t_3);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_3;
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_8);
                  __pyx_cur_scope->__pyx_t_5 = __pyx_t_8;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_6 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_12);
                  __pyx_cur_scope->__pyx_t_7 = __pyx_t_12;
                  __Pyx_XGIVEREF(__pyx_t_13);
                  __pyx_cur_scope->__pyx_t_8 = __pyx_t_13;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_9 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_t_15);
                  __pyx_cur_scope->__pyx_t_10 = __pyx_t_15;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_SwapException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 3;
                  return __pyx_r;
                  __pyx_L29_resume_from_await:;
                  __pyx_t_2 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_2);
                  __pyx_t_3 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_3);
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_8 = __pyx_cur_scope->__pyx_t_5;
                  __pyx_cur_scope->__pyx_t_5 = 0;
                  __Pyx_XGOTREF(__pyx_t_8);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_6;
                  __pyx_cur_scope->__pyx_t_6 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_12 = __pyx_cur_scope->__pyx_t_7;
                  __pyx_cur_scope->__pyx_t_7 = 0;
                  __Pyx_XGOTREF(__pyx_t_12);
                  __pyx_t_13 = __pyx_cur_scope->__pyx_t_8;
                  __pyx_cur_scope->__pyx_t_8 = 0;
                  __Pyx_XGOTREF(__pyx_t_13);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_9;
                  __pyx_cur_scope->__pyx_t_9 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  __pyx_t_15 = __pyx_cur_scope->__pyx_t_10;
                  __pyx_cur_scope->__pyx_t_10 = 0;
                  __Pyx_XGOTREF(__pyx_t_15);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 795, __pyx_L21_except_error)
                  __pyx_t_9 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_9);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_9 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 795, __pyx_L21_except_error)
                }
                __pyx_t_15 = __pyx_t_9;
                __pyx_t_9 = 0;
                __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_15);
                __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
                if (__pyx_t_1 < (0)) __PYX_ERR(0, 795, __pyx_L21_except_error)
                __pyx_t_16 = (!__pyx_t_1);
                if (unlikely(__pyx_t_16)) {
                  __Pyx_GIVEREF(__pyx_t_2);
                  __Pyx_GIVEREF(__pyx_t_3);
                  __Pyx_XGIVEREF(__pyx_t_8);
                  __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_3, __pyx_t_8);
                  __pyx_t_2 = 0;  __pyx_t_3 = 0;  __pyx_t_8 = 0; 
                  __PYX_ERR(0, 795, __pyx_L21_except_error)
                }
                __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
                __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
                __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
                goto __pyx_L20_exception_handled;
              }
              __pyx_L21_except_error:;
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              goto __pyx_L8_error;
              __pyx_L20_exception_handled:;
              __Pyx_XGIVEREF(__pyx_t_12);
              __Pyx_XGIVEREF(__pyx_t_13);
              __Pyx_XGIVEREF(__pyx_t_14);
              __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
              __pyx_L24_try_end:;
            }
          }
          /*finally:*/ {
            /*normal exit:*/{
              if (__pyx_t_10) {
                __pyx_t_14 = __Pyx_PyObject_Call(__pyx_t_10, __pyx_mstate_global->__pyx_tuple[0], NULL);
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 795, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_14);
                __pyx_t_11 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_14, &__pyx_r);
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
                if (likely(__pyx_t_11 == PYGEN_NEXT)) {
                  __Pyx_GOTREF(__pyx_r);
                  __Pyx_XGIVEREF(__pyx_t_5);
                  __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
                  __Pyx_XGIVEREF(__pyx_t_6);
                  __pyx_cur_scope->__pyx_t_1 = __pyx_t_6;
                  __Pyx_XGIVEREF(__pyx_t_7);
                  __pyx_cur_scope->__pyx_t_2 = __pyx_t_7;
                  __Pyx_XGIVEREF(__pyx_t_10);
                  __pyx_cur_scope->__pyx_t_3 = __pyx_t_10;
                  __Pyx_XGIVEREF(__pyx_t_14);
                  __pyx_cur_scope->__pyx_t_4 = __pyx_t_14;
                  __Pyx_XGIVEREF(__pyx_r);
                  __Pyx_RefNannyFinishContext();
                  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
                  /* return from generator, awaiting value */
                  __pyx_generator->resume_label = 4;
                  return __pyx_r;
                  __pyx_L30_resume_from_await:;
                  __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
                  __pyx_cur_scope->__pyx_t_0 = 0;
                  __Pyx_XGOTREF(__pyx_t_5);
                  __pyx_t_6 = __pyx_cur_scope->__pyx_t_1;
                  __pyx_cur_scope->__pyx_t_1 = 0;
                  __Pyx_XGOTREF(__pyx_t_6);
                  __pyx_t_7 = __pyx_cur_scope->__pyx_t_2;
                  __pyx_cur_scope->__pyx_t_2 = 0;
                  __Pyx_XGOTREF(__pyx_t_7);
                  __pyx_t_10 = __pyx_cur_scope->__pyx_t_3;
                  __pyx_cur_scope->__pyx_t_3 = 0;
                  __Pyx_XGOTREF(__pyx_t_10);
                  __pyx_t_14 = __pyx_cur_scope->__pyx_t_4;
                  __pyx_cur_scope->__pyx_t_4 = 0;
                  __Pyx_XGOTREF(__pyx_t_14);
                  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 795, __pyx_L8_error)
                  __pyx_t_8 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_8);
                } else if (likely(__pyx_t_11 == PYGEN_RETURN)) {
                  __Pyx_GOTREF(__pyx_r);
                  __pyx_t_8 = __pyx_r; __pyx_r = NULL;
                } else {
                  __Pyx_XGOTREF(__pyx_r);
                  __PYX_ERR(0, 795, __pyx_L8_error)
                }
                __pyx_t_14 = __pyx_t_8;
                __pyx_t_8 = 0;
                __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              }
              goto __pyx_L18;
            }
            __pyx_L18:;
          }
          goto __pyx_L31;
          __pyx_L14_error:;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          goto __pyx_L8_error;
          __pyx_L31:;
        }

        /* "aioesphomeapi/connection.py":794
 *             )
 *         self._finish_connect_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._finish_connect_future, ConnectionInterruptedError, None
*/
      }
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      goto __pyx_L13_try_end;
      __pyx_L8_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

      /* "aioesphomeapi/connection.py":799
 *             ):
 *                 await self._do_finish_connect(login)
 *         except (Exception, CancelledError) as ex:             # <<<<<<<<<<<<<<
 *             # Clean up the connection; re-raise the original CancelledError
 *             # if the task is actually being cancelled so TaskGroup /
*/
      __pyx_t_17 = __Pyx_PyErr_ExceptionMatches2(((PyObject *)(((PyTypeObject*)PyExc_Exception))), __pyx_v_13aioesphomeapi_10connection_CancelledError);
      if (__pyx_t_17) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.finish_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 799, __pyx_L10_except_error)
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_3);
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_3);
        __pyx_cur_scope->__pyx_v_ex = __pyx_t_3;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":803
 *             # if the task is actually being cancelled so TaskGroup /
 *             # asyncio.timeout semantics are preserved.
 *             self._raise_fatal_connection_exception("finishing", ex)             # <<<<<<<<<<<<<<
 *         finally:
 *             self._set_finish_connect_future()
*/
          __pyx_t_18 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
          __Pyx_INCREF(__pyx_t_18);
          __pyx_t_4 = 0;
          {
            PyObject *__pyx_callargs[3] = {__pyx_t_18, __pyx_mstate_global->__pyx_n_u_finishing, __pyx_cur_scope->__pyx_v_ex};
            __pyx_t_9 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_raise_fatal_connection_exceptio, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 803, __pyx_L37_error)
            __Pyx_GOTREF(__pyx_t_9);
          }
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        }

        /* "aioesphomeapi/connection.py":799
 *             ):
 *                 await self._do_finish_connect(login)
 *         except (Exception, CancelledError) as ex:             # <<<<<<<<<<<<<<
 *             # Clean up the connection; re-raise the original CancelledError
 *             # if the task is actually being cancelled so TaskGroup /
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
            __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
            goto __pyx_L38;
          }
          __pyx_L37_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_10 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_15 = 0; __pyx_t_21 = 0;
            __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
            __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_12, &__pyx_t_15, &__pyx_t_21);
            if ( unlikely(__Pyx_GetException(&__pyx_t_10, &__pyx_t_14, &__pyx_t_13) < 0)) __Pyx_ErrFetch(&__pyx_t_10, &__pyx_t_14, &__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_14);
            __Pyx_XGOTREF(__pyx_t_13);
            __Pyx_XGOTREF(__pyx_t_12);
            __Pyx_XGOTREF(__pyx_t_15);
            __Pyx_XGOTREF(__pyx_t_21);
            __pyx_t_17 = __pyx_lineno; __pyx_t_19 = __pyx_clineno; __pyx_t_20 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_ex);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_ex); __pyx_cur_scope->__pyx_v_ex = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_12);
            __Pyx_XGIVEREF(__pyx_t_15);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_15, __pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_10);
            __Pyx_XGIVEREF(__pyx_t_14);
            __Pyx_XGIVEREF(__pyx_t_13);
            __Pyx_ErrRestore(__pyx_t_10, __pyx_t_14, __pyx_t_13);
            __pyx_t_10 = 0; __pyx_t_14 = 0; __pyx_t_13 = 0; __pyx_t_12 = 0; __pyx_t_15 = 0; __pyx_t_21 = 0;
            __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_19; __pyx_filename = __pyx_t_20;
            goto __pyx_L10_except_error;
          }
          __pyx_L38:;
        }
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        goto __pyx_L9_exception_handled;
      }
      goto __pyx_L10_except_error;

      /* "aioesphomeapi/connection.py":794
 *             )
 *         self._finish_connect_future = self._loop.create_future()
 *         try:             # <<<<<<<<<<<<<<
 *             async with interrupt(
 *                 self._finish_connect_future, ConnectionInterruptedError, None
*/
      __pyx_L10_except_error:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      goto __pyx_L6_error;
      __pyx_L9_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ExceptionReset(__pyx_t_5, __pyx_t_6, __pyx_t_7);
      __pyx_L13_try_end:;
    }
  }

  /* "aioesphomeapi/connection.py":805
 *             self._raise_fatal_connection_exception("finishing", ex)
 *         finally:
 *             self._set_finish_connect_future()             # <<<<<<<<<<<<<<
 *         self._set_connection_state(CONNECTION_STATE_CONNECTED)
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_finish_connect_future(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 805, __pyx_L1_error)
      goto __pyx_L7;
    }
    __pyx_L6_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_assign
      __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_15, &__pyx_t_12);
      if ( unlikely(__Pyx_GetException(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5) < 0)) __Pyx_ErrFetch(&__pyx_t_7, &__pyx_t_6, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_6);
      __Pyx_XGOTREF(__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_15);
      __Pyx_XGOTREF(__pyx_t_12);
      __pyx_t_19 = __pyx_lineno; __pyx_t_17 = __pyx_clineno; __pyx_t_22 = __pyx_filename;
      {
        ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_finish_connect_future(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 805, __pyx_L44_error)
      }
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_15, __pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_6);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ErrRestore(__pyx_t_7, __pyx_t_6, __pyx_t_5);
      __pyx_t_7 = 0; __pyx_t_6 = 0; __pyx_t_5 = 0; __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      __pyx_lineno = __pyx_t_19; __pyx_clineno = __pyx_t_17; __pyx_filename = __pyx_t_22;
      goto __pyx_L1_error;
      __pyx_L44_error:;
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_15);
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_15, __pyx_t_12);
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_21 = 0; __pyx_t_15 = 0; __pyx_t_12 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L7:;
  }

  /* "aioesphomeapi/connection.py":806
 *         finally:
 *             self._set_finish_connect_future()
 *         self._set_connection_state(CONNECTION_STATE_CONNECTED)             # <<<<<<<<<<<<<<
 * 
 *     def _set_finish_connect_future(self) -> None:
*/
  __pyx_t_2 = __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_connection_state(__pyx_cur_scope->__pyx_v_self, __pyx_t_2); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 806, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":783
 *         self._async_schedule_keep_alive(self._loop.time())
 * 
 *     async def finish_connection(self, *, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process.
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_18);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("finish_connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":808
 *         self._set_connection_state(CONNECTION_STATE_CONNECTED)
 * 
 *     def _set_finish_connect_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._finish_connect_future is not None
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_finish_connect_future(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_set_finish_connect_future", 0);

  /* "aioesphomeapi/connection.py":810
 *     def _set_finish_connect_future(self) -> None:
 *         if (
 *             self._finish_connect_future is not None             # <<<<<<<<<<<<<<
 *             and not self._finish_connect_future.done()
 *         ):
*/
  __pyx_t_2 = (__pyx_v_self->_finish_connect_future != Py_None);
  if (__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }

  /* "aioesphomeapi/connection.py":811
 *         if (
 *             self._finish_connect_future is not None
 *             and not self._finish_connect_future.done()             # <<<<<<<<<<<<<<
 *         ):
 *             self._finish_connect_future.set_result(None)
*/
  __pyx_t_4 = __pyx_v_self->_finish_connect_future;
  __Pyx_INCREF(__pyx_t_4);
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_done, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 811, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 < 0))) __PYX_ERR(0, 811, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_6 = (!__pyx_t_2);
  __pyx_t_1 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;

  /* "aioesphomeapi/connection.py":809
 * 
 *     def _set_finish_connect_future(self) -> None:
 *         if (             # <<<<<<<<<<<<<<
 *             self._finish_connect_future is not None
 *             and not self._finish_connect_future.done()
*/
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":813
 *             and not self._finish_connect_future.done()
 *         ):
 *             self._finish_connect_future.set_result(None)             # <<<<<<<<<<<<<<
 *             self._finish_connect_future = None
 * 
*/
    __pyx_t_4 = __pyx_v_self->_finish_connect_future;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, Py_None};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_set_result, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 813, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":814
 *         ):
 *             self._finish_connect_future.set_result(None)
 *             self._finish_connect_future = None             # <<<<<<<<<<<<<<
 * 
 *     def _set_connection_state(self, state: ConnectionState) -> None:
*/
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_finish_connect_future);
    __Pyx_DECREF(__pyx_v_self->_finish_connect_future);
    __pyx_v_self->_finish_connect_future = Py_None;

    /* "aioesphomeapi/connection.py":809
 * 
 *     def _set_finish_connect_future(self) -> None:
 *         if (             # <<<<<<<<<<<<<<
 *             self._finish_connect_future is not None
 *             and not self._finish_connect_future.done()
*/
  }

  /* "aioesphomeapi/connection.py":808
 *         self._set_connection_state(CONNECTION_STATE_CONNECTED)
 * 
 *     def _set_finish_connect_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._finish_connect_future is not None
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._set_finish_connect_future", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":816
 *             self._finish_connect_future = None
 * 
 *     def _set_connection_state(self, state: ConnectionState) -> None:             # <<<<<<<<<<<<<<
 *         """Set the connection state and log the change."""
 *         self.connection_state = state
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_connection_state(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_state) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("_set_connection_state", 0);

  /* "aioesphomeapi/connection.py":818
 *     def _set_connection_state(self, state: ConnectionState) -> None:
 *         """Set the connection state and log the change."""
 *         self.connection_state = state             # <<<<<<<<<<<<<<
 *         self.is_connected = state is CONNECTION_STATE_CONNECTED
 *         self._handshake_complete = (
*/
  __Pyx_INCREF(__pyx_v_state);
  __Pyx_GIVEREF(__pyx_v_state);
  __Pyx_GOTREF(__pyx_v_self->connection_state);
  __Pyx_DECREF(__pyx_v_self->connection_state);
  __pyx_v_self->connection_state = __pyx_v_state;

  /* "aioesphomeapi/connection.py":819
 *         """Set the connection state and log the change."""
 *         self.connection_state = state
 *         self.is_connected = state is CONNECTION_STATE_CONNECTED             # <<<<<<<<<<<<<<
 *         self._handshake_complete = (
 *             state is CONNECTION_STATE_HANDSHAKE_COMPLETE
*/
  __pyx_t_1 = (__pyx_v_state == __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED);
  __pyx_v_self->is_connected = __pyx_t_1;

  /* "aioesphomeapi/connection.py":821
 *         self.is_connected = state is CONNECTION_STATE_CONNECTED
 *         self._handshake_complete = (
 *             state is CONNECTION_STATE_HANDSHAKE_COMPLETE             # <<<<<<<<<<<<<<
 *             or state is CONNECTION_STATE_CONNECTED
 *         )
*/
  __pyx_t_2 = (__pyx_v_state == __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L3_bool_binop_done;
  }

  /* "aioesphomeapi/connection.py":822
 *         self._handshake_complete = (
 *             state is CONNECTION_STATE_HANDSHAKE_COMPLETE
 *             or state is CONNECTION_STATE_CONNECTED             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_2 = (__pyx_v_state == __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;

  /* "aioesphomeapi/connection.py":820
 *         self.connection_state = state
 *         self.is_connected = state is CONNECTION_STATE_CONNECTED
 *         self._handshake_complete = (             # <<<<<<<<<<<<<<
 *             state is CONNECTION_STATE_HANDSHAKE_COMPLETE
 *             or state is CONNECTION_STATE_CONNECTED
*/
  __pyx_v_self->_handshake_complete = __pyx_t_1;

  /* "aioesphomeapi/connection.py":816
 *             self._finish_connect_future = None
 * 
 *     def _set_connection_state(self, state: ConnectionState) -> None:             # <<<<<<<<<<<<<<
 *         """Set the connection state and log the change."""
 *         self.connection_state = state
*/

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":825
 *         )
 * 
 *     def _make_auth_request(self) -> AuthenticationRequest:             # <<<<<<<<<<<<<<
 *         """Make a AuthenticationRequest."""
 *         if self._params.password is not None:
*/

static PyObject *__pyx_f_13aioesphomeapi_10connection_13APIConnection__make_auth_request(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_make_auth_request", 0);

  /* "aioesphomeapi/connection.py":827
 *     def _make_auth_request(self) -> AuthenticationRequest:
 *         """Make a AuthenticationRequest."""
 *         if self._params.password is not None:             # <<<<<<<<<<<<<<
 *             return AuthenticationRequest(password=self._params.password)
 *         return NO_PASSWORD_AUTH_REQUEST
*/
  __pyx_t_1 = (__pyx_v_self->_params->password != Py_None);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":828
 *         """Make a AuthenticationRequest."""
 *         if self._params.password is not None:
 *             return AuthenticationRequest(password=self._params.password)             # <<<<<<<<<<<<<<
 *         return NO_PASSWORD_AUTH_REQUEST
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = NULL;
    __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_AuthenticationRequest);
    __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_AuthenticationRequest; 
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, NULL};
      __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 828, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_password, __pyx_v_self->_params->password, __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 828, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 828, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "aioesphomeapi/connection.py":827
 *     def _make_auth_request(self) -> AuthenticationRequest:
 *         """Make a AuthenticationRequest."""
 *         if self._params.password is not None:             # <<<<<<<<<<<<<<
 *             return AuthenticationRequest(password=self._params.password)
 *         return NO_PASSWORD_AUTH_REQUEST
*/
  }

  /* "aioesphomeapi/connection.py":829
 *         if self._params.password is not None:
 *             return AuthenticationRequest(password=self._params.password)
 *         return NO_PASSWORD_AUTH_REQUEST             # <<<<<<<<<<<<<<
 * 
 *     def send_message(self, msg: message.Message) -> None:
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST);
  __pyx_r = __pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":825
 *         )
 * 
 *     def _make_auth_request(self) -> AuthenticationRequest:             # <<<<<<<<<<<<<<
 *         """Make a AuthenticationRequest."""
 *         if self._params.password is not None:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._make_auth_request", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":831
 *         return NO_PASSWORD_AUTH_REQUEST
 * 
 *     def send_message(self, msg: message.Message) -> None:             # <<<<<<<<<<<<<<
 *         """Send a message to the remote."""
 *         self.send_messages((msg,))
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_40send_message(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_send_message(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msg, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_message", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_send_message); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_40send_message)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_msg};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 831, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":833
 *     def send_message(self, msg: message.Message) -> None:
 *         """Send a message to the remote."""
 *         self.send_messages((msg,))             # <<<<<<<<<<<<<<
 * 
 *     def send_messages(self, msgs: tuple[message.Message, ...]) -> None:
*/
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_msg);
  __Pyx_GIVEREF(__pyx_v_msg);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_msg) != (0)) __PYX_ERR(0, 833, __pyx_L1_error);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 833, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":831
 *         return NO_PASSWORD_AUTH_REQUEST
 * 
 *     def send_message(self, msg: message.Message) -> None:             # <<<<<<<<<<<<<<
 *         """Send a message to the remote."""
 *         self.send_messages((msg,))
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_40send_message(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_39send_message, "Send a message to the remote.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_40send_message = {"send_message", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_40send_message, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_39send_message};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_40send_message(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_msg = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_message (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_msg,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 831, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 831, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "send_message", 0) < (0)) __PYX_ERR(0, 831, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("send_message", 1, 1, 1, i); __PYX_ERR(0, 831, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 831, __pyx_L3_error)
    }
    __pyx_v_msg = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send_message", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 831, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_39send_message(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_msg);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_39send_message(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msg) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_message", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection_send_message(__pyx_v_self, __pyx_v_msg, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 831, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":835
 *         self.send_messages((msg,))
 * 
 *     def send_messages(self, msgs: tuple[message.Message, ...]) -> None:             # <<<<<<<<<<<<<<
 *         """Send a protobuf message to the remote."""
 *         if not self._handshake_complete:
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_42send_messages(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_send_messages(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msgs, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_msg_type = 0;
  PyObject *__pyx_v_packets = 0;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_v_err = NULL;
  PyObject *__pyx_v_wrapped_err = NULL;
  PyObject *__pyx_8genexpr4__pyx_v_msg = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7[3];
  PyObject *__pyx_t_8 = NULL;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_t_17;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_messages", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_send_messages); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_42send_messages)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_msgs};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 835, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":837
 *     def send_messages(self, msgs: tuple[message.Message, ...]) -> None:
 *         """Send a protobuf message to the remote."""
 *         if not self._handshake_complete:             # <<<<<<<<<<<<<<
 *             raise ConnectionNotEstablishedAPIError(
 *                 f"Connection isn't established yet ({self.connection_state})"
*/
  __pyx_t_6 = (!__pyx_v_self->_handshake_complete);
  if (unlikely(__pyx_t_6)) {

    /* "aioesphomeapi/connection.py":838
 *         """Send a protobuf message to the remote."""
 *         if not self._handshake_complete:
 *             raise ConnectionNotEstablishedAPIError(             # <<<<<<<<<<<<<<
 *                 f"Connection isn't established yet ({self.connection_state})"
 *             )
*/
    __pyx_t_2 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_ConnectionNotEstablishedAPIError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 838, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);

    /* "aioesphomeapi/connection.py":839
 *         if not self._handshake_complete:
 *             raise ConnectionNotEstablishedAPIError(
 *                 f"Connection isn't established yet ({self.connection_state})"             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_3 = __Pyx_PyObject_FormatSimple(__pyx_v_self->connection_state, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 839, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7[0] = __pyx_mstate_global->__pyx_kp_u_Connection_isn_t_established_yet;
    __pyx_t_7[1] = __pyx_t_3;
    __pyx_t_7[2] = __pyx_mstate_global->__pyx_kp_u__2;
    __pyx_t_8 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, 34 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3));
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 839, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_2);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 838, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 838, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":837
 *     def send_messages(self, msgs: tuple[message.Message, ...]) -> None:
 *         """Send a protobuf message to the remote."""
 *         if not self._handshake_complete:             # <<<<<<<<<<<<<<
 *             raise ConnectionNotEstablishedAPIError(
 *                 f"Connection isn't established yet ({self.connection_state})"
*/
  }

  /* "aioesphomeapi/connection.py":842
 *             )
 * 
 *         packets: list[tuple[int, bytes]] = [             # <<<<<<<<<<<<<<
 *             (msg_type[0], msg_type[1](msg))
 *             for msg in msgs
*/
  { /* enter inner scope */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 842, __pyx_L6_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "aioesphomeapi/connection.py":844
 *         packets: list[tuple[int, bytes]] = [
 *             (msg_type[0], msg_type[1](msg))
 *             for msg in msgs             # <<<<<<<<<<<<<<
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])
 *         ]
*/
    __pyx_t_4 = __pyx_v_msgs; __Pyx_INCREF(__pyx_t_4);
    __pyx_t_9 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_4);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 844, __pyx_L6_error)
        #endif
        if (__pyx_t_9 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_8 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_4, __pyx_t_9));
      #else
      __pyx_t_8 = __Pyx_PySequence_ITEM(__pyx_t_4, __pyx_t_9);
      #endif
      ++__pyx_t_9;
      if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 844, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_XDECREF_SET(__pyx_8genexpr4__pyx_v_msg, __pyx_t_8);
      __pyx_t_8 = 0;

      /* "aioesphomeapi/connection.py":845
 *             (msg_type[0], msg_type[1](msg))
 *             for msg in msgs
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])             # <<<<<<<<<<<<<<
 *         ]
 *         if self._debug_enabled:
*/
      if (unlikely(__pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 845, __pyx_L6_error)
      }
      __pyx_t_8 = __Pyx_PyDict_GetItem(__pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE, ((PyObject *)Py_TYPE(__pyx_8genexpr4__pyx_v_msg))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 845, __pyx_L6_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (!(likely(PyTuple_CheckExact(__pyx_t_8))||((__pyx_t_8) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_8))) __PYX_ERR(0, 845, __pyx_L6_error)
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_XDECREF_SET(__pyx_v_msg_type, ((PyObject*)__pyx_t_8));
      if (__pyx_t_8 == Py_None) __pyx_t_6 = 0;
      else
      {
        Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_8);
        if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 845, __pyx_L6_error)
        __pyx_t_6 = (__pyx_temp != 0);
      }

      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (__pyx_t_6) {

        /* "aioesphomeapi/connection.py":843
 * 
 *         packets: list[tuple[int, bytes]] = [
 *             (msg_type[0], msg_type[1](msg))             # <<<<<<<<<<<<<<
 *             for msg in msgs
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])
*/
        if (unlikely(__pyx_v_msg_type == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 843, __pyx_L6_error)
        }
        __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v_msg_type, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 843, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_3 = NULL;
        if (unlikely(__pyx_v_msg_type == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 843, __pyx_L6_error)
        }
        __pyx_t_10 = __Pyx_GetItemInt_Tuple(__pyx_v_msg_type, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 843, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_10);
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_10))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_10);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_8genexpr4__pyx_v_msg};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 843, __pyx_L6_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_10 = PyTuple_New(2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 843, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_GIVEREF(__pyx_t_8);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 0, __pyx_t_8) != (0)) __PYX_ERR(0, 843, __pyx_L6_error);
        __Pyx_GIVEREF(__pyx_t_2);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_10, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 843, __pyx_L6_error);
        __pyx_t_8 = 0;
        __pyx_t_2 = 0;
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)__pyx_t_10))) __PYX_ERR(0, 842, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "aioesphomeapi/connection.py":845
 *             (msg_type[0], msg_type[1](msg))
 *             for msg in msgs
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])             # <<<<<<<<<<<<<<
 *         ]
 *         if self._debug_enabled:
*/
      }

      /* "aioesphomeapi/connection.py":844
 *         packets: list[tuple[int, bytes]] = [
 *             (msg_type[0], msg_type[1](msg))
 *             for msg in msgs             # <<<<<<<<<<<<<<
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])
 *         ]
*/
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_msg); __pyx_8genexpr4__pyx_v_msg = 0;
    goto __pyx_L11_exit_scope;
    __pyx_L6_error:;
    __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_msg); __pyx_8genexpr4__pyx_v_msg = 0;
    goto __pyx_L1_error;
    __pyx_L11_exit_scope:;
  } /* exit inner scope */
  __pyx_v_packets = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":847
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])
 *         ]
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             for msg in msgs:
 *                 _LOGGER.debug(
*/
  if (__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":848
 *         ]
 *         if self._debug_enabled:
 *             for msg in msgs:             # <<<<<<<<<<<<<<
 *                 _LOGGER.debug(
 *                     "%s: Sending %s: %s",
*/
    __pyx_t_1 = __pyx_v_msgs; __Pyx_INCREF(__pyx_t_1);
    __pyx_t_9 = 0;
    for (;;) {
      {
        Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 848, __pyx_L1_error)
        #endif
        if (__pyx_t_9 >= __pyx_temp) break;
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_9));
      #else
      __pyx_t_4 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_9);
      #endif
      ++__pyx_t_9;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 848, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_msg, __pyx_t_4);
      __pyx_t_4 = 0;

      /* "aioesphomeapi/connection.py":849
 *         if self._debug_enabled:
 *             for msg in msgs:
 *                 _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                     "%s: Sending %s: %s",
 *                     self.log_name,
*/
      __pyx_t_10 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 849, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 849, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "aioesphomeapi/connection.py":852
 *                     "%s: Sending %s: %s",
 *                     self.log_name,
 *                     type(msg).__name__,             # <<<<<<<<<<<<<<
 *                     # calling __str__ on the message may crash on
 *                     # Windows systems due to a bug in the protobuf library
*/
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_msg)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 852, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "aioesphomeapi/connection.py":856
 *                     # Windows systems due to a bug in the protobuf library
 *                     # so we call MessageToDict instead
 *                     MessageToDict(msg) if _WIN32 else msg,             # <<<<<<<<<<<<<<
 *                 )
 * 
*/
      if (__pyx_v_13aioesphomeapi_10connection__WIN32) {
        __pyx_t_12 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_MessageToDict); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 856, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_13))) {
          __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);
          assert(__pyx_t_12);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_12);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_v_msg};
          __pyx_t_11 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 856, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_11);
        }
        __pyx_t_3 = __pyx_t_11;
        __pyx_t_11 = 0;
      } else {
        __Pyx_INCREF(__pyx_v_msg);
        __pyx_t_3 = __pyx_v_msg;
      }
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_8);
        assert(__pyx_t_10);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[5] = {__pyx_t_10, __pyx_mstate_global->__pyx_kp_u_s_Sending_s_s, __pyx_v_self->log_name, __pyx_t_2, __pyx_t_3};
        __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (5-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 849, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "aioesphomeapi/connection.py":848
 *         ]
 *         if self._debug_enabled:
 *             for msg in msgs:             # <<<<<<<<<<<<<<
 *                 _LOGGER.debug(
 *                     "%s: Sending %s: %s",
*/
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "aioesphomeapi/connection.py":847
 *             if (msg_type := PROTO_TO_MESSAGE_TYPE[type(msg)])
 *         ]
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             for msg in msgs:
 *                 _LOGGER.debug(
*/
  }

  /* "aioesphomeapi/connection.py":860
 * 
 *         # Check frame_helper is not None to prevent segfault in Cython code
 *         if self._frame_helper is None:             # <<<<<<<<<<<<<<
 *             raise self._fatal_exception or ConnectionNotEstablishedAPIError(
 *                 f"Connection is closed ({self.connection_state})"
*/
  __pyx_t_6 = (((PyObject *)__pyx_v_self->_frame_helper) == Py_None);
  if (unlikely(__pyx_t_6)) {

    /* "aioesphomeapi/connection.py":861
 *         # Check frame_helper is not None to prevent segfault in Cython code
 *         if self._frame_helper is None:
 *             raise self._fatal_exception or ConnectionNotEstablishedAPIError(             # <<<<<<<<<<<<<<
 *                 f"Connection is closed ({self.connection_state})"
 *             )
*/
    __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_self->_fatal_exception); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 861, __pyx_L1_error)
    if (!__pyx_t_6) {
    } else {
      __Pyx_INCREF(__pyx_v_self->_fatal_exception);
      __pyx_t_1 = __pyx_v_self->_fatal_exception;
      goto __pyx_L17_bool_binop_done;
    }
    __pyx_t_8 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_ConnectionNotEstablishedAPIError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 861, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "aioesphomeapi/connection.py":862
 *         if self._frame_helper is None:
 *             raise self._fatal_exception or ConnectionNotEstablishedAPIError(
 *                 f"Connection is closed ({self.connection_state})"             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    __pyx_t_2 = __Pyx_PyObject_FormatSimple(__pyx_v_self->connection_state, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_7[0] = __pyx_mstate_global->__pyx_kp_u_Connection_is_closed;
    __pyx_t_7[1] = __pyx_t_2;
    __pyx_t_7[2] = __pyx_mstate_global->__pyx_kp_u__2;
    __pyx_t_10 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, 22 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
    if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 862, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_3);
      assert(__pyx_t_8);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_8);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
      __pyx_t_5 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_10};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 861, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_L17_bool_binop_done:;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 861, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":860
 * 
 *         # Check frame_helper is not None to prevent segfault in Cython code
 *         if self._frame_helper is None:             # <<<<<<<<<<<<<<
 *             raise self._fatal_exception or ConnectionNotEstablishedAPIError(
 *                 f"Connection is closed ({self.connection_state})"
*/
  }

  /* "aioesphomeapi/connection.py":865
 *             )
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             self._frame_helper.write_packets(packets, self._debug_enabled)
 *         except WRITE_EXCEPTIONS as err:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
    __Pyx_XGOTREF(__pyx_t_14);
    __Pyx_XGOTREF(__pyx_t_15);
    __Pyx_XGOTREF(__pyx_t_16);
    /*try:*/ {

      /* "aioesphomeapi/connection.py":866
 * 
 *         try:
 *             self._frame_helper.write_packets(packets, self._debug_enabled)             # <<<<<<<<<<<<<<
 *         except WRITE_EXCEPTIONS as err:
 *             # If writing packet fails, we don't know what state the frames
*/
      ((struct __pyx_vtabstruct_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)__pyx_v_self->_frame_helper->__pyx_vtab)->write_packets(__pyx_v_self->_frame_helper, __pyx_v_packets, __pyx_v_self->_debug_enabled, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 866, __pyx_L19_error)

      /* "aioesphomeapi/connection.py":865
 *             )
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             self._frame_helper.write_packets(packets, self._debug_enabled)
 *         except WRITE_EXCEPTIONS as err:
*/
    }
    __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
    __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
    goto __pyx_L24_try_end;
    __pyx_L19_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "aioesphomeapi/connection.py":867
 *         try:
 *             self._frame_helper.write_packets(packets, self._debug_enabled)
 *         except WRITE_EXCEPTIONS as err:             # <<<<<<<<<<<<<<
 *             # If writing packet fails, we don't know what state the frames
 *             # are in anymore and we have to close the connection
*/
    __pyx_t_17 = __Pyx_PyErr_ExceptionMatches(__pyx_v_13aioesphomeapi_10connection_WRITE_EXCEPTIONS);
    if (__pyx_t_17) {
      __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_4, &__pyx_t_3) < 0) __PYX_ERR(0, 867, __pyx_L21_except_error)
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_v_err = __pyx_t_4;
      /*try:*/ {

        /* "aioesphomeapi/connection.py":870
 *             # If writing packet fails, we don't know what state the frames
 *             # are in anymore and we have to close the connection
 *             _LOGGER.info("%s: Error writing packets: %s", self.log_name, err)             # <<<<<<<<<<<<<<
 *             wrapped_err = SocketClosedAPIError(
 *                 f"{self.log_name}: Error writing packets: {err}"
*/
        __pyx_t_8 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 870, __pyx_L30_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_info); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 870, __pyx_L30_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_11))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_11);
          assert(__pyx_t_8);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_11, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[4] = {__pyx_t_8, __pyx_mstate_global->__pyx_kp_u_s_Error_writing_packets_s, __pyx_v_self->log_name, __pyx_v_err};
          __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_11, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 870, __pyx_L30_error)
          __Pyx_GOTREF(__pyx_t_10);
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "aioesphomeapi/connection.py":871
 *             # are in anymore and we have to close the connection
 *             _LOGGER.info("%s: Error writing packets: %s", self.log_name, err)
 *             wrapped_err = SocketClosedAPIError(             # <<<<<<<<<<<<<<
 *                 f"{self.log_name}: Error writing packets: {err}"
 *             )
*/
        __pyx_t_11 = NULL;
        __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError);
        __pyx_t_8 = __pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError; 

        /* "aioesphomeapi/connection.py":872
 *             _LOGGER.info("%s: Error writing packets: %s", self.log_name, err)
 *             wrapped_err = SocketClosedAPIError(
 *                 f"{self.log_name}: Error writing packets: {err}"             # <<<<<<<<<<<<<<
 *             )
 *             wrapped_err.__cause__ = err
*/
        __pyx_t_2 = __Pyx_PyUnicode_Unicode(__pyx_v_self->log_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 872, __pyx_L30_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_13 = __Pyx_PyObject_FormatSimple(__pyx_v_err, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 872, __pyx_L30_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_7[0] = __pyx_t_2;
        __pyx_t_7[1] = __pyx_mstate_global->__pyx_kp_u_Error_writing_packets;
        __pyx_t_7[2] = __pyx_t_13;
        __pyx_t_12 = __Pyx_PyUnicode_Join(__pyx_t_7, 3, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 25 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_13), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_13));
        if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 872, __pyx_L30_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_8);
          assert(__pyx_t_11);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_11, __pyx_t_12};
          __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 871, __pyx_L30_error)
          __Pyx_GOTREF(__pyx_t_10);
        }
        __pyx_v_wrapped_err = __pyx_t_10;
        __pyx_t_10 = 0;

        /* "aioesphomeapi/connection.py":874
 *                 f"{self.log_name}: Error writing packets: {err}"
 *             )
 *             wrapped_err.__cause__ = err             # <<<<<<<<<<<<<<
 *             self.report_fatal_error(wrapped_err)
 *             raise wrapped_err from err
*/
        if (__Pyx_PyObject_SetAttrStr(__pyx_v_wrapped_err, __pyx_mstate_global->__pyx_n_u_cause, __pyx_v_err) < (0)) __PYX_ERR(0, 874, __pyx_L30_error)

        /* "aioesphomeapi/connection.py":875
 *             )
 *             wrapped_err.__cause__ = err
 *             self.report_fatal_error(wrapped_err)             # <<<<<<<<<<<<<<
 *             raise wrapped_err from err
 * 
*/
        __pyx_t_10 = __pyx_v_wrapped_err;
        __Pyx_INCREF(__pyx_t_10);
        if (!(likely(__Pyx_PyException_Check(__pyx_t_10))||((__pyx_t_10) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_10))) __PYX_ERR(0, 875, __pyx_L30_error)
        ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->report_fatal_error(__pyx_v_self, ((PyObject*)__pyx_t_10), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 875, __pyx_L30_error)
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;

        /* "aioesphomeapi/connection.py":876
 *             wrapped_err.__cause__ = err
 *             self.report_fatal_error(wrapped_err)
 *             raise wrapped_err from err             # <<<<<<<<<<<<<<
 * 
 *     def _add_message_callback_without_remove(
*/
        __Pyx_Raise(__pyx_v_wrapped_err, 0, 0, __pyx_v_err);
        __PYX_ERR(0, 876, __pyx_L30_error)
      }

      /* "aioesphomeapi/connection.py":867
 *         try:
 *             self._frame_helper.write_packets(packets, self._debug_enabled)
 *         except WRITE_EXCEPTIONS as err:             # <<<<<<<<<<<<<<
 *             # If writing packet fails, we don't know what state the frames
 *             # are in anymore and we have to close the connection
*/
      /*finally:*/ {
        __pyx_L30_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
          if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __Pyx_XGOTREF(__pyx_t_25);
          __pyx_t_17 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_err); __pyx_v_err = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_25);
          __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
          __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
          __pyx_lineno = __pyx_t_17; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
          goto __pyx_L21_except_error;
        }
      }
    }
    goto __pyx_L21_except_error;

    /* "aioesphomeapi/connection.py":865
 *             )
 * 
 *         try:             # <<<<<<<<<<<<<<
 *             self._frame_helper.write_packets(packets, self._debug_enabled)
 *         except WRITE_EXCEPTIONS as err:
*/
    __pyx_L21_except_error:;
    __Pyx_XGIVEREF(__pyx_t_14);
    __Pyx_XGIVEREF(__pyx_t_15);
    __Pyx_XGIVEREF(__pyx_t_16);
    __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
    goto __pyx_L1_error;
    __pyx_L24_try_end:;
  }

  /* "aioesphomeapi/connection.py":835
 *         self.send_messages((msg,))
 * 
 *     def send_messages(self, msgs: tuple[message.Message, ...]) -> None:             # <<<<<<<<<<<<<<
 *         """Send a protobuf message to the remote."""
 *         if not self._handshake_complete:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_msg_type);
  __Pyx_XDECREF(__pyx_v_packets);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF(__pyx_v_err);
  __Pyx_XDECREF(__pyx_v_wrapped_err);
  __Pyx_XDECREF(__pyx_8genexpr4__pyx_v_msg);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_42send_messages(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_41send_messages, "Send a protobuf message to the remote.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_42send_messages = {"send_messages", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_42send_messages, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_41send_messages};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_42send_messages(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_msgs = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_messages (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_msgs,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 835, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 835, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "send_messages", 0) < (0)) __PYX_ERR(0, 835, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("send_messages", 1, 1, 1, i); __PYX_ERR(0, 835, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 835, __pyx_L3_error)
    }
    __pyx_v_msgs = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send_messages", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 835, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_msgs), (&PyTuple_Type), 0, "msgs", 2))) __PYX_ERR(0, 835, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_41send_messages(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_msgs);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_41send_messages(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_msgs) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_messages", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection_send_messages(__pyx_v_self, __pyx_v_msgs, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 835, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":878
 *             raise wrapped_err from err
 * 
 *     def _add_message_callback_without_remove(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> None:
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__add_message_callback_without_remove(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types) {
  PyObject *__pyx_v_handlers = 0;
  PyObject *__pyx_v_msg_type = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_add_message_callback_without_remove", 0);

  /* "aioesphomeapi/connection.py":882
 *     ) -> None:
 *         """Add a message callback without returning a remove callable."""
 *         for msg_type in msg_types:             # <<<<<<<<<<<<<<
 *             if (handlers := self._message_handlers.get(msg_type)) is None:
 *                 self._message_handlers[msg_type] = {on_message}
*/
  __pyx_t_1 = __pyx_v_msg_types; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 882, __pyx_L1_error)
      #endif
      if (__pyx_t_2 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_3 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_2));
    #else
    __pyx_t_3 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_2);
    #endif
    ++__pyx_t_2;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 882, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_msg_type, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":883
 *         """Add a message callback without returning a remove callable."""
 *         for msg_type in msg_types:
 *             if (handlers := self._message_handlers.get(msg_type)) is None:             # <<<<<<<<<<<<<<
 *                 self._message_handlers[msg_type] = {on_message}
 *             else:
*/
    if (unlikely(__pyx_v_self->_message_handlers == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
      __PYX_ERR(0, 883, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_message_handlers, __pyx_v_msg_type, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PySet_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None) || __Pyx_RaiseUnexpectedTypeError("set", __pyx_t_3))) __PYX_ERR(0, 883, __pyx_L1_error)
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_XDECREF_SET(__pyx_v_handlers, ((PyObject*)__pyx_t_3));
    __pyx_t_4 = (__pyx_t_3 == ((PyObject*)Py_None));
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_4) {

      /* "aioesphomeapi/connection.py":884
 *         for msg_type in msg_types:
 *             if (handlers := self._message_handlers.get(msg_type)) is None:
 *                 self._message_handlers[msg_type] = {on_message}             # <<<<<<<<<<<<<<
 *             else:
 *                 handlers.add(on_message)
*/
      __pyx_t_3 = PySet_New(0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 884, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      if (PySet_Add(__pyx_t_3, __pyx_v_on_message) < (0)) __PYX_ERR(0, 884, __pyx_L1_error)
      if (unlikely(__pyx_v_self->_message_handlers == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 884, __pyx_L1_error)
      }
      if (unlikely((PyDict_SetItem(__pyx_v_self->_message_handlers, __pyx_v_msg_type, __pyx_t_3) < 0))) __PYX_ERR(0, 884, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "aioesphomeapi/connection.py":883
 *         """Add a message callback without returning a remove callable."""
 *         for msg_type in msg_types:
 *             if (handlers := self._message_handlers.get(msg_type)) is None:             # <<<<<<<<<<<<<<
 *                 self._message_handlers[msg_type] = {on_message}
 *             else:
*/
      goto __pyx_L5;
    }

    /* "aioesphomeapi/connection.py":886
 *                 self._message_handlers[msg_type] = {on_message}
 *             else:
 *                 handlers.add(on_message)             # <<<<<<<<<<<<<<
 * 
 *     def add_message_callback(
*/
    /*else*/ {
      if (unlikely(__pyx_v_handlers == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
        __PYX_ERR(0, 886, __pyx_L1_error)
      }
      __pyx_t_5 = PySet_Add(__pyx_v_handlers, __pyx_v_on_message); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 886, __pyx_L1_error)
    }
    __pyx_L5:;

    /* "aioesphomeapi/connection.py":882
 *     ) -> None:
 *         """Add a message callback without returning a remove callable."""
 *         for msg_type in msg_types:             # <<<<<<<<<<<<<<
 *             if (handlers := self._message_handlers.get(msg_type)) is None:
 *                 self._message_handlers[msg_type] = {on_message}
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":878
 *             raise wrapped_err from err
 * 
 *     def _add_message_callback_without_remove(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._add_message_callback_without_remove", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_handlers);
  __Pyx_XDECREF(__pyx_v_msg_type);
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":888
 *                 handlers.add(on_message)
 * 
 *     def add_message_callback(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> Callable[[], None]:
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_44add_message_callback(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_13aioesphomeapi_10connection_13APIConnection_add_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_message_callback", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_add_message_callback); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 888, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_44add_message_callback)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_on_message, __pyx_v_msg_types};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 888, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":892
 *     ) -> Callable[[], None]:
 *         """Add a message callback."""
 *         self._add_message_callback_without_remove(on_message, msg_types)             # <<<<<<<<<<<<<<
 *         return partial(self._remove_message_callback, on_message, msg_types)
 * 
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_add_message_callback_without_remove(__pyx_v_self, __pyx_v_on_message, __pyx_v_msg_types); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 892, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":893
 *         """Add a message callback."""
 *         self._add_message_callback_without_remove(on_message, msg_types)
 *         return partial(self._remove_message_callback, on_message, msg_types)             # <<<<<<<<<<<<<<
 * 
 *     def _remove_message_callback(
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_partial);
  __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_partial; 
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_remove_message_callback); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 893, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_2, __pyx_t_3, __pyx_v_on_message, __pyx_v_msg_types};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (4-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 893, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":888
 *                 handlers.add(on_message)
 * 
 *     def add_message_callback(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> Callable[[], None]:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.add_message_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_44add_message_callback(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_43add_message_callback, "Add a message callback.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_44add_message_callback = {"add_message_callback", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_44add_message_callback, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_43add_message_callback};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_44add_message_callback(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_on_message = 0;
  PyObject *__pyx_v_msg_types = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_message_callback (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_on_message,&__pyx_mstate_global->__pyx_n_u_msg_types,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 888, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 888, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 888, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "add_message_callback", 0) < (0)) __PYX_ERR(0, 888, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("add_message_callback", 1, 2, 2, i); __PYX_ERR(0, 888, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 888, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 888, __pyx_L3_error)
    }
    __pyx_v_on_message = values[0];
    __pyx_v_msg_types = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_message_callback", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 888, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.add_message_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_msg_types), (&PyTuple_Type), 0, "msg_types", 2))) __PYX_ERR(0, 889, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_43add_message_callback(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_on_message, __pyx_v_msg_types);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_43add_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_message_callback", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_13aioesphomeapi_10connection_13APIConnection_add_message_callback(__pyx_v_self, __pyx_v_on_message, __pyx_v_msg_types, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 888, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.add_message_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":895
 *         return partial(self._remove_message_callback, on_message, msg_types)
 * 
 *     def _remove_message_callback(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> None:
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__remove_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_handlers = 0;
  PyObject *__pyx_v_msg_type = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_remove_message_callback", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_remove_message_callback); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_v_on_message, __pyx_v_msg_types};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 895, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":899
 *     ) -> None:
 *         """Remove a message callback."""
 *         for msg_type in msg_types:             # <<<<<<<<<<<<<<
 *             handlers = self._message_handlers[msg_type]
 *             handlers.discard(on_message)
*/
  __pyx_t_1 = __pyx_v_msg_types; __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = 0;
  for (;;) {
    {
      Py_ssize_t __pyx_temp = __Pyx_PyTuple_GET_SIZE(__pyx_t_1);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely((__pyx_temp < 0))) __PYX_ERR(0, 899, __pyx_L1_error)
      #endif
      if (__pyx_t_6 >= __pyx_temp) break;
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    __pyx_t_2 = __Pyx_NewRef(PyTuple_GET_ITEM(__pyx_t_1, __pyx_t_6));
    #else
    __pyx_t_2 = __Pyx_PySequence_ITEM(__pyx_t_1, __pyx_t_6);
    #endif
    ++__pyx_t_6;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 899, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_msg_type, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":900
 *         """Remove a message callback."""
 *         for msg_type in msg_types:
 *             handlers = self._message_handlers[msg_type]             # <<<<<<<<<<<<<<
 *             handlers.discard(on_message)
 * 
*/
    if (unlikely(__pyx_v_self->_message_handlers == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 900, __pyx_L1_error)
    }
    __pyx_t_2 = __Pyx_PyDict_GetItem(__pyx_v_self->_message_handlers, __pyx_v_msg_type); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 900, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (!(likely(PySet_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("set", __pyx_t_2))) __PYX_ERR(0, 900, __pyx_L1_error)
    __Pyx_XDECREF_SET(__pyx_v_handlers, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":901
 *         for msg_type in msg_types:
 *             handlers = self._message_handlers[msg_type]
 *             handlers.discard(on_message)             # <<<<<<<<<<<<<<
 * 
 *     def send_message_callback_response(
*/
    if (unlikely(__pyx_v_handlers == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "discard");
      __PYX_ERR(0, 901, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_PySet_Discard(__pyx_v_handlers, __pyx_v_on_message); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 901, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":899
 *     ) -> None:
 *         """Remove a message callback."""
 *         for msg_type in msg_types:             # <<<<<<<<<<<<<<
 *             handlers = self._message_handlers[msg_type]
 *             handlers.discard(on_message)
*/
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":895
 *         return partial(self._remove_message_callback, on_message, msg_types)
 * 
 *     def _remove_message_callback(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._remove_message_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_handlers);
  __Pyx_XDECREF(__pyx_v_msg_type);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_45_remove_message_callback, "Remove a message callback.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback = {"_remove_message_callback", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_45_remove_message_callback};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_on_message = 0;
  PyObject *__pyx_v_msg_types = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_remove_message_callback (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_on_message,&__pyx_mstate_global->__pyx_n_u_msg_types,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 895, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 895, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 895, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_remove_message_callback", 0) < (0)) __PYX_ERR(0, 895, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_remove_message_callback", 1, 2, 2, i); __PYX_ERR(0, 895, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 895, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 895, __pyx_L3_error)
    }
    __pyx_v_on_message = values[0];
    __pyx_v_msg_types = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_remove_message_callback", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 895, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._remove_message_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_msg_types), (&PyTuple_Type), 0, "msg_types", 2))) __PYX_ERR(0, 896, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_45_remove_message_callback(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_on_message, __pyx_v_msg_types);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_45_remove_message_callback(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_remove_message_callback", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection__remove_message_callback(__pyx_v_self, __pyx_v_on_message, __pyx_v_msg_types, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 895, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._remove_message_callback", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":903
 *             handlers.discard(on_message)
 * 
 *     def send_message_callback_response(             # <<<<<<<<<<<<<<
 *         self,
 *         send_msg: message.Message,
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_48send_message_callback_response(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_47send_message_callback_response, "Send a message to the remote and register the given message handler.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_48send_message_callback_response = {"send_message_callback_response", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_48send_message_callback_response, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_47send_message_callback_response};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_48send_message_callback_response(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_send_msg = 0;
  PyObject *__pyx_v_on_message = 0;
  PyObject *__pyx_v_msg_types = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_message_callback_response (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_send_msg,&__pyx_mstate_global->__pyx_n_u_on_message,&__pyx_mstate_global->__pyx_n_u_msg_types,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 903, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 903, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 903, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 903, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "send_message_callback_response", 0) < (0)) __PYX_ERR(0, 903, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("send_message_callback_response", 1, 3, 3, i); __PYX_ERR(0, 903, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 903, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 903, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 903, __pyx_L3_error)
    }
    __pyx_v_send_msg = values[0];
    __pyx_v_on_message = values[1];
    __pyx_v_msg_types = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send_message_callback_response", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 903, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message_callback_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_msg_types), (&PyTuple_Type), 0, "msg_types", 2))) __PYX_ERR(0, 907, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_47send_message_callback_response(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_send_msg, __pyx_v_on_message, __pyx_v_msg_types);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_47send_message_callback_response(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_send_msg, PyObject *__pyx_v_on_message, PyObject *__pyx_v_msg_types) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_message_callback_response", 0);

  /* "aioesphomeapi/connection.py":910
 *     ) -> Callable[[], None]:
 *         """Send a message to the remote and register the given message handler."""
 *         self.send_messages((send_msg,))             # <<<<<<<<<<<<<<
 *         # Since we do not return control to the event loop (no awaits)
 *         # between sending the message and registering the handler
*/
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_send_msg);
  __Pyx_GIVEREF(__pyx_v_send_msg);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_send_msg) != (0)) __PYX_ERR(0, 910, __pyx_L1_error);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 910, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":915
 *         # we can be sure that we will not miss any messages even though
 *         # we register the handler after sending the message
 *         return self.add_message_callback(on_message, msg_types)             # <<<<<<<<<<<<<<
 * 
 *     async def send_messages_await_response_complex(  # pylint: disable=too-many-locals
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->add_message_callback(__pyx_v_self, __pyx_v_on_message, __pyx_v_msg_types, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 915, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "aioesphomeapi/connection.py":903
 *             handlers.discard(on_message)
 * 
 *     def send_message_callback_response(             # <<<<<<<<<<<<<<
 *         self,
 *         send_msg: message.Message,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message_callback_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_51generator7(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":917
 *         return self.add_message_callback(on_message, msg_types)
 * 
 *     async def send_messages_await_response_complex(  # pylint: disable=too-many-locals             # <<<<<<<<<<<<<<
 *         self,
 *         messages: tuple[message.Message, ...],
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_50send_messages_await_response_complex(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_49send_messages_await_response_complex, "Send a message to the remote and build up a list response.\n\n        :param send_msg: The message (request) to send.\n        :param do_append: Predicate to check if a received message is part of the response.\n        :param do_stop: Predicate to check if a received message is the stop response.\n        :param timeout: The maximum amount of time to wait for the stop response.\n\n        :raises TimeoutAPIError: if a timeout occurred\n        ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_50send_messages_await_response_complex = {"send_messages_await_response_complex", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_50send_messages_await_response_complex, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_49send_messages_await_response_complex};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_50send_messages_await_response_complex(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_messages = 0;
  PyObject *__pyx_v_do_append = 0;
  PyObject *__pyx_v_do_stop = 0;
  PyObject *__pyx_v_msg_types = 0;
  PyObject *__pyx_v_timeout = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[5] = {0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_messages_await_response_complex (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_messages,&__pyx_mstate_global->__pyx_n_u_do_append,&__pyx_mstate_global->__pyx_n_u_do_stop,&__pyx_mstate_global->__pyx_n_u_msg_types,&__pyx_mstate_global->__pyx_n_u_timeout,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 917, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  5:
        values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 917, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 917, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 917, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 917, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 917, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "send_messages_await_response_complex", 0) < (0)) __PYX_ERR(0, 917, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 5; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("send_messages_await_response_complex", 1, 5, 5, i); __PYX_ERR(0, 917, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 5)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 917, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 917, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 917, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_FASTCALL(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 917, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_FASTCALL(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 917, __pyx_L3_error)
    }
    __pyx_v_messages = ((PyObject*)values[0]);
    __pyx_v_do_append = values[1];
    __pyx_v_do_stop = values[2];
    __pyx_v_msg_types = ((PyObject*)values[3]);
    __pyx_v_timeout = values[4];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send_messages_await_response_complex", 1, 5, 5, __pyx_nargs); __PYX_ERR(0, 917, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages_await_response_complex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_messages), (&PyTuple_Type), 0, "messages", 2))) __PYX_ERR(0, 919, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_msg_types), (&PyTuple_Type), 0, "msg_types", 2))) __PYX_ERR(0, 922, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_49send_messages_await_response_complex(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_messages, __pyx_v_do_append, __pyx_v_do_stop, __pyx_v_msg_types, __pyx_v_timeout);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_49send_messages_await_response_complex(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_messages, PyObject *__pyx_v_do_append, PyObject *__pyx_v_do_stop, PyObject *__pyx_v_msg_types, PyObject *__pyx_v_timeout) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_messages_await_response_complex", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 917, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_messages = __pyx_v_messages;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_messages);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_messages);
  __pyx_cur_scope->__pyx_v_do_append = __pyx_v_do_append;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_do_append);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_do_append);
  __pyx_cur_scope->__pyx_v_do_stop = __pyx_v_do_stop;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_do_stop);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_do_stop);
  __pyx_cur_scope->__pyx_v_msg_types = __pyx_v_msg_types;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_msg_types);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_msg_types);
  __pyx_cur_scope->__pyx_v_timeout = __pyx_v_timeout;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_timeout);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_timeout);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_51generator7, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_send_messages_await_response_com, __pyx_mstate_global->__pyx_n_u_APIConnection_send_messages_awai, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 917, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages_await_response_complex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_51generator7(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_PySendResult __pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16[5];
  PyObject *__pyx_t_17 = NULL;
  int __pyx_t_18;
  char const *__pyx_t_19;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  int __pyx_t_26;
  char const *__pyx_t_27;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_messages_await_response_complex", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L13_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 917, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":937
 *         # This is safe because we are not awaiting between
 *         # sending the message and registering the handler
 *         self.send_messages(messages)             # <<<<<<<<<<<<<<
 *         # Unsafe to await between sending the message and registering the handler
 *         fut: asyncio.Future[None] = self._loop.create_future()
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->send_messages(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_messages, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 937, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":939
 *         self.send_messages(messages)
 *         # Unsafe to await between sending the message and registering the handler
 *         fut: asyncio.Future[None] = self._loop.create_future()             # <<<<<<<<<<<<<<
 *         responses: list[message.Message] = []
 *         on_message = partial(
*/
  __pyx_t_2 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_create_future, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 939, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_fut = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":940
 *         # Unsafe to await between sending the message and registering the handler
 *         fut: asyncio.Future[None] = self._loop.create_future()
 *         responses: list[message.Message] = []             # <<<<<<<<<<<<<<
 *         on_message = partial(
 *             _handle_complex_message, fut, responses, do_append, do_stop
*/
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 940, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_responses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":941
 *         fut: asyncio.Future[None] = self._loop.create_future()
 *         responses: list[message.Message] = []
 *         on_message = partial(             # <<<<<<<<<<<<<<
 *             _handle_complex_message, fut, responses, do_append, do_stop
 *         )
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_partial);
  __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_partial; 

  /* "aioesphomeapi/connection.py":942
 *         responses: list[message.Message] = []
 *         on_message = partial(
 *             _handle_complex_message, fut, responses, do_append, do_stop             # <<<<<<<<<<<<<<
 *         )
 *         self._add_message_callback_without_remove(on_message, msg_types)
*/
  __pyx_t_3 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_3 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[6] = {__pyx_t_2, __pyx_v_13aioesphomeapi_10connection__handle_complex_message, __pyx_cur_scope->__pyx_v_fut, __pyx_cur_scope->__pyx_v_responses, __pyx_cur_scope->__pyx_v_do_append, __pyx_cur_scope->__pyx_v_do_stop};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_3, (6-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 941, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_on_message = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":944
 *             _handle_complex_message, fut, responses, do_append, do_stop
 *         )
 *         self._add_message_callback_without_remove(on_message, msg_types)             # <<<<<<<<<<<<<<
 * 
 *         self._read_exception_futures.add(fut)
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_add_message_callback_without_remove(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_on_message, __pyx_cur_scope->__pyx_v_msg_types); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 944, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":946
 *         self._add_message_callback_without_remove(on_message, msg_types)
 * 
 *         self._read_exception_futures.add(fut)             # <<<<<<<<<<<<<<
 *         # Now safe to await since we have registered the handler
 * 
*/
  if (unlikely(__pyx_cur_scope->__pyx_v_self->_read_exception_futures == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "add");
    __PYX_ERR(0, 946, __pyx_L1_error)
  }
  __pyx_t_5 = PySet_Add(__pyx_cur_scope->__pyx_v_self->_read_exception_futures, __pyx_cur_scope->__pyx_v_fut); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 946, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":952
 *         # the message could fail to be removed if the
 *         # the await is cancelled
 *         timeout_handle = self._loop.call_at(             # <<<<<<<<<<<<<<
 *             self._loop.time() + timeout, _handle_timeout, fut
 *         )
*/
  __pyx_t_4 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_4);

  /* "aioesphomeapi/connection.py":953
 *         # the await is cancelled
 *         timeout_handle = self._loop.call_at(
 *             self._loop.time() + timeout, _handle_timeout, fut             # <<<<<<<<<<<<<<
 *         )
 *         timeout_expired = False
*/
  __pyx_t_6 = __pyx_cur_scope->__pyx_v_self->_loop;
  __Pyx_INCREF(__pyx_t_6);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_time, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 953, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_6 = PyNumber_Add(__pyx_t_2, __pyx_cur_scope->__pyx_v_timeout); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 953, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[4] = {__pyx_t_4, __pyx_t_6, __pyx_v_13aioesphomeapi_10connection__handle_timeout, __pyx_cur_scope->__pyx_v_fut};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_call_at, __pyx_callargs+__pyx_t_3, (4-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 952, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_timeout_handle = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":955
 *             self._loop.time() + timeout, _handle_timeout, fut
 *         )
 *         timeout_expired = False             # <<<<<<<<<<<<<<
 *         try:
 *             await fut
*/
  __pyx_cur_scope->__pyx_v_timeout_expired = 0;

  /* "aioesphomeapi/connection.py":956
 *         )
 *         timeout_expired = False
 *         try:             # <<<<<<<<<<<<<<
 *             await fut
 *         except TimeoutError as err:
*/
  /*try:*/ {
    {
      __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_9);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":957
 *         timeout_expired = False
 *         try:
 *             await fut             # <<<<<<<<<<<<<<
 *         except TimeoutError as err:
 *             timeout_expired = True
*/
        __pyx_t_10 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_cur_scope->__pyx_v_fut, &__pyx_r);
        if (likely(__pyx_t_10 == PYGEN_NEXT)) {
          __Pyx_GOTREF(__pyx_r);
          __Pyx_XGIVEREF(__pyx_t_7);
          __pyx_cur_scope->__pyx_t_0 = __pyx_t_7;
          __Pyx_XGIVEREF(__pyx_t_8);
          __pyx_cur_scope->__pyx_t_1 = __pyx_t_8;
          __Pyx_XGIVEREF(__pyx_t_9);
          __pyx_cur_scope->__pyx_t_2 = __pyx_t_9;
          __Pyx_XGIVEREF(__pyx_r);
          __Pyx_RefNannyFinishContext();
          __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
          /* return from generator, awaiting value */
          __pyx_generator->resume_label = 1;
          return __pyx_r;
          __pyx_L13_resume_from_await:;
          __pyx_t_7 = __pyx_cur_scope->__pyx_t_0;
          __pyx_cur_scope->__pyx_t_0 = 0;
          __Pyx_XGOTREF(__pyx_t_7);
          __pyx_t_8 = __pyx_cur_scope->__pyx_t_1;
          __pyx_cur_scope->__pyx_t_1 = 0;
          __Pyx_XGOTREF(__pyx_t_8);
          __pyx_t_9 = __pyx_cur_scope->__pyx_t_2;
          __pyx_cur_scope->__pyx_t_2 = 0;
          __Pyx_XGOTREF(__pyx_t_9);
          if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 957, __pyx_L7_error)
        } else if (likely(__pyx_t_10 == PYGEN_RETURN)) {
          __Pyx_GOTREF(__pyx_r);
          __Pyx_DECREF(__pyx_r); __pyx_r = 0;
        } else {
          __Pyx_XGOTREF(__pyx_r);
          __PYX_ERR(0, 957, __pyx_L7_error)
        }

        /* "aioesphomeapi/connection.py":956
 *         )
 *         timeout_expired = False
 *         try:             # <<<<<<<<<<<<<<
 *             await fut
 *         except TimeoutError as err:
*/
      }
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      goto __pyx_L12_try_end;
      __pyx_L7_error:;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "aioesphomeapi/connection.py":958
 *         try:
 *             await fut
 *         except TimeoutError as err:             # <<<<<<<<<<<<<<
 *             timeout_expired = True
 *             response_names = message_types_to_names(msg_types)
*/
      __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_TimeoutError))));
      if (__pyx_t_11) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_messages_await_response_complex", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_6, &__pyx_t_4) < 0) __PYX_ERR(0, 958, __pyx_L9_except_error)
        __Pyx_XGOTREF(__pyx_t_1);
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_6);
        __pyx_cur_scope->__pyx_v_err = __pyx_t_6;
        /*try:*/ {

          /* "aioesphomeapi/connection.py":959
 *             await fut
 *         except TimeoutError as err:
 *             timeout_expired = True             # <<<<<<<<<<<<<<
 *             response_names = message_types_to_names(msg_types)
 *             raise TimeoutAPIError(
*/
          __pyx_cur_scope->__pyx_v_timeout_expired = 1;

          /* "aioesphomeapi/connection.py":960
 *         except TimeoutError as err:
 *             timeout_expired = True
 *             response_names = message_types_to_names(msg_types)             # <<<<<<<<<<<<<<
 *             raise TimeoutAPIError(
 *                 f"Timeout waiting for {response_names} after {timeout}s"
*/
          __pyx_t_12 = NULL;
          __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_message_types_to_names); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 960, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_13);
          __pyx_t_3 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_13))) {
            __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_13);
            assert(__pyx_t_12);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
            __Pyx_INCREF(__pyx_t_12);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
            __pyx_t_3 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_12, __pyx_cur_scope->__pyx_v_msg_types};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 960, __pyx_L19_error)
            __Pyx_GOTREF(__pyx_t_2);
          }
          __Pyx_GIVEREF(__pyx_t_2);
          __pyx_cur_scope->__pyx_v_response_names = __pyx_t_2;
          __pyx_t_2 = 0;

          /* "aioesphomeapi/connection.py":961
 *             timeout_expired = True
 *             response_names = message_types_to_names(msg_types)
 *             raise TimeoutAPIError(             # <<<<<<<<<<<<<<
 *                 f"Timeout waiting for {response_names} after {timeout}s"
 *             ) from err
*/
          __pyx_t_13 = NULL;
          __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError);
          __pyx_t_12 = __pyx_v_13aioesphomeapi_10connection_TimeoutAPIError; 

          /* "aioesphomeapi/connection.py":962
 *             response_names = message_types_to_names(msg_types)
 *             raise TimeoutAPIError(
 *                 f"Timeout waiting for {response_names} after {timeout}s"             # <<<<<<<<<<<<<<
 *             ) from err
 *         finally:
*/
          __pyx_t_14 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_response_names, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 962, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_14);
          __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_cur_scope->__pyx_v_timeout, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 962, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_15);
          __pyx_t_16[0] = __pyx_mstate_global->__pyx_kp_u_Timeout_waiting_for;
          __pyx_t_16[1] = __pyx_t_14;
          __pyx_t_16[2] = __pyx_mstate_global->__pyx_kp_u_after;
          __pyx_t_16[3] = __pyx_t_15;
          __pyx_t_16[4] = __pyx_mstate_global->__pyx_n_u_s;
          __pyx_t_17 = __Pyx_PyUnicode_Join(__pyx_t_16, 5, 20 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + 7 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15));
          if (unlikely(!__pyx_t_17)) __PYX_ERR(0, 962, __pyx_L19_error)
          __Pyx_GOTREF(__pyx_t_17);
          __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
          __pyx_t_3 = 1;
          #if CYTHON_UNPACK_METHODS
          if (unlikely(PyMethod_Check(__pyx_t_12))) {
            __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
            assert(__pyx_t_13);
            PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(__pyx__function);
            __Pyx_DECREF_SET(__pyx_t_12, __pyx__function);
            __pyx_t_3 = 0;
          }
          #endif
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_t_17};
            __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_12, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_DECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 961, __pyx_L19_error)
            __Pyx_GOTREF(__pyx_t_2);
          }

          /* "aioesphomeapi/connection.py":963
 *             raise TimeoutAPIError(
 *                 f"Timeout waiting for {response_names} after {timeout}s"
 *             ) from err             # <<<<<<<<<<<<<<
 *         finally:
 *             if not timeout_expired:
*/
          __Pyx_Raise(__pyx_t_2, 0, 0, __pyx_cur_scope->__pyx_v_err);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __PYX_ERR(0, 961, __pyx_L19_error)
        }

        /* "aioesphomeapi/connection.py":958
 *         try:
 *             await fut
 *         except TimeoutError as err:             # <<<<<<<<<<<<<<
 *             timeout_expired = True
 *             response_names = message_types_to_names(msg_types)
*/
        /*finally:*/ {
          __pyx_L19_error:;
          /*exception exit:*/{
            __Pyx_PyThreadState_assign
            __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
            __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
            __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
            __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
            __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
            __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
            __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
             __Pyx_ExceptionSwap(&__pyx_t_23, &__pyx_t_24, &__pyx_t_25);
            if ( unlikely(__Pyx_GetException(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22) < 0)) __Pyx_ErrFetch(&__pyx_t_20, &__pyx_t_21, &__pyx_t_22);
            __Pyx_XGOTREF(__pyx_t_20);
            __Pyx_XGOTREF(__pyx_t_21);
            __Pyx_XGOTREF(__pyx_t_22);
            __Pyx_XGOTREF(__pyx_t_23);
            __Pyx_XGOTREF(__pyx_t_24);
            __Pyx_XGOTREF(__pyx_t_25);
            __pyx_t_11 = __pyx_lineno; __pyx_t_18 = __pyx_clineno; __pyx_t_19 = __pyx_filename;
            {
              __Pyx_GOTREF(__pyx_cur_scope->__pyx_v_err);
              __Pyx_DECREF(__pyx_cur_scope->__pyx_v_err); __pyx_cur_scope->__pyx_v_err = 0;
            }
            __Pyx_XGIVEREF(__pyx_t_23);
            __Pyx_XGIVEREF(__pyx_t_24);
            __Pyx_XGIVEREF(__pyx_t_25);
            __Pyx_ExceptionReset(__pyx_t_23, __pyx_t_24, __pyx_t_25);
            __Pyx_XGIVEREF(__pyx_t_20);
            __Pyx_XGIVEREF(__pyx_t_21);
            __Pyx_XGIVEREF(__pyx_t_22);
            __Pyx_ErrRestore(__pyx_t_20, __pyx_t_21, __pyx_t_22);
            __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0;
            __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_18; __pyx_filename = __pyx_t_19;
            goto __pyx_L9_except_error;
          }
        }
      }
      goto __pyx_L9_except_error;

      /* "aioesphomeapi/connection.py":956
 *         )
 *         timeout_expired = False
 *         try:             # <<<<<<<<<<<<<<
 *             await fut
 *         except TimeoutError as err:
*/
      __pyx_L9_except_error:;
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
      goto __pyx_L5_error;
      __pyx_L12_try_end:;
    }
  }

  /* "aioesphomeapi/connection.py":965
 *             ) from err
 *         finally:
 *             if not timeout_expired:             # <<<<<<<<<<<<<<
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)
*/
  /*finally:*/ {
    /*normal exit:*/{
      __pyx_t_26 = (!__pyx_cur_scope->__pyx_v_timeout_expired);
      if (__pyx_t_26) {

        /* "aioesphomeapi/connection.py":966
 *         finally:
 *             if not timeout_expired:
 *                 timeout_handle.cancel()             # <<<<<<<<<<<<<<
 *             self._remove_message_callback(on_message, msg_types)
 *             self._read_exception_futures.discard(fut)
*/
        __pyx_t_6 = __pyx_cur_scope->__pyx_v_timeout_handle;
        __Pyx_INCREF(__pyx_t_6);
        __pyx_t_3 = 0;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
          __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancel, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 966, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "aioesphomeapi/connection.py":965
 *             ) from err
 *         finally:
 *             if not timeout_expired:             # <<<<<<<<<<<<<<
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)
*/
      }

      /* "aioesphomeapi/connection.py":967
 *             if not timeout_expired:
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)             # <<<<<<<<<<<<<<
 *             self._read_exception_futures.discard(fut)
 * 
*/
      ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_remove_message_callback(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_on_message, __pyx_cur_scope->__pyx_v_msg_types, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 967, __pyx_L1_error)

      /* "aioesphomeapi/connection.py":968
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)
 *             self._read_exception_futures.discard(fut)             # <<<<<<<<<<<<<<
 * 
 *         return responses
*/
      if (unlikely(__pyx_cur_scope->__pyx_v_self->_read_exception_futures == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "discard");
        __PYX_ERR(0, 968, __pyx_L1_error)
      }
      __pyx_t_5 = __Pyx_PySet_Discard(__pyx_cur_scope->__pyx_v_self->_read_exception_futures, __pyx_cur_scope->__pyx_v_fut); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 968, __pyx_L1_error)
      goto __pyx_L6;
    }
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_assign
      __pyx_t_9 = 0; __pyx_t_8 = 0; __pyx_t_7 = 0; __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0;
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
      __Pyx_XDECREF(__pyx_t_17); __pyx_t_17 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_25, &__pyx_t_24, &__pyx_t_23);
      if ( unlikely(__Pyx_GetException(&__pyx_t_9, &__pyx_t_8, &__pyx_t_7) < 0)) __Pyx_ErrFetch(&__pyx_t_9, &__pyx_t_8, &__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_9);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_XGOTREF(__pyx_t_7);
      __Pyx_XGOTREF(__pyx_t_25);
      __Pyx_XGOTREF(__pyx_t_24);
      __Pyx_XGOTREF(__pyx_t_23);
      __pyx_t_18 = __pyx_lineno; __pyx_t_11 = __pyx_clineno; __pyx_t_27 = __pyx_filename;
      {

        /* "aioesphomeapi/connection.py":965
 *             ) from err
 *         finally:
 *             if not timeout_expired:             # <<<<<<<<<<<<<<
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)
*/
        __pyx_t_26 = (!__pyx_cur_scope->__pyx_v_timeout_expired);
        if (__pyx_t_26) {

          /* "aioesphomeapi/connection.py":966
 *         finally:
 *             if not timeout_expired:
 *                 timeout_handle.cancel()             # <<<<<<<<<<<<<<
 *             self._remove_message_callback(on_message, msg_types)
 *             self._read_exception_futures.discard(fut)
*/
          __pyx_t_6 = __pyx_cur_scope->__pyx_v_timeout_handle;
          __Pyx_INCREF(__pyx_t_6);
          __pyx_t_3 = 0;
          {
            PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
            __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_cancel, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
            __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
            if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 966, __pyx_L27_error)
            __Pyx_GOTREF(__pyx_t_4);
          }
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

          /* "aioesphomeapi/connection.py":965
 *             ) from err
 *         finally:
 *             if not timeout_expired:             # <<<<<<<<<<<<<<
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)
*/
        }

        /* "aioesphomeapi/connection.py":967
 *             if not timeout_expired:
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)             # <<<<<<<<<<<<<<
 *             self._read_exception_futures.discard(fut)
 * 
*/
        ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_remove_message_callback(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_on_message, __pyx_cur_scope->__pyx_v_msg_types, 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 967, __pyx_L27_error)

        /* "aioesphomeapi/connection.py":968
 *                 timeout_handle.cancel()
 *             self._remove_message_callback(on_message, msg_types)
 *             self._read_exception_futures.discard(fut)             # <<<<<<<<<<<<<<
 * 
 *         return responses
*/
        if (unlikely(__pyx_cur_scope->__pyx_v_self->_read_exception_futures == Py_None)) {
          PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "discard");
          __PYX_ERR(0, 968, __pyx_L27_error)
        }
        __pyx_t_5 = __Pyx_PySet_Discard(__pyx_cur_scope->__pyx_v_self->_read_exception_futures, __pyx_cur_scope->__pyx_v_fut); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 968, __pyx_L27_error)
      }
      __Pyx_XGIVEREF(__pyx_t_25);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_ExceptionReset(__pyx_t_25, __pyx_t_24, __pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_9);
      __Pyx_XGIVEREF(__pyx_t_8);
      __Pyx_XGIVEREF(__pyx_t_7);
      __Pyx_ErrRestore(__pyx_t_9, __pyx_t_8, __pyx_t_7);
      __pyx_t_9 = 0; __pyx_t_8 = 0; __pyx_t_7 = 0; __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0;
      __pyx_lineno = __pyx_t_18; __pyx_clineno = __pyx_t_11; __pyx_filename = __pyx_t_27;
      goto __pyx_L1_error;
      __pyx_L27_error:;
      __Pyx_XGIVEREF(__pyx_t_25);
      __Pyx_XGIVEREF(__pyx_t_24);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_ExceptionReset(__pyx_t_25, __pyx_t_24, __pyx_t_23);
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_25 = 0; __pyx_t_24 = 0; __pyx_t_23 = 0;
      goto __pyx_L1_error;
    }
    __pyx_L6:;
  }

  /* "aioesphomeapi/connection.py":970
 *             self._read_exception_futures.discard(fut)
 * 
 *         return responses             # <<<<<<<<<<<<<<
 * 
 *     async def send_message_await_response(
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_responses);
  __pyx_r = __pyx_cur_scope->__pyx_v_responses;
  goto __pyx_L0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":917
 *         return self.add_message_callback(on_message, msg_types)
 * 
 *     async def send_messages_await_response_complex(  # pylint: disable=too-many-locals             # <<<<<<<<<<<<<<
 *         self,
 *         messages: tuple[message.Message, ...],
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_17);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("send_messages_await_response_complex", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_54generator8(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":972
 *         return responses
 * 
 *     async def send_message_await_response(             # <<<<<<<<<<<<<<
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_53send_message_await_response(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_53send_message_await_response = {"send_message_await_response", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_53send_message_await_response, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_53send_message_await_response(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_send_msg = 0;
  PyObject *__pyx_v_response_type = 0;
  PyObject *__pyx_v_timeout = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_message_await_response (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_send_msg,&__pyx_mstate_global->__pyx_n_u_response_type,&__pyx_mstate_global->__pyx_n_u_timeout,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 972, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 972, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 972, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 972, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "send_message_await_response", 0) < (0)) __PYX_ERR(0, 972, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_float_10_0));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("send_message_await_response", 0, 2, 3, i); __PYX_ERR(0, 972, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 972, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 972, __pyx_L3_error)
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 972, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_float_10_0));
    }
    __pyx_v_send_msg = values[0];
    __pyx_v_response_type = values[1];
    __pyx_v_timeout = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("send_message_await_response", 0, 2, 3, __pyx_nargs); __PYX_ERR(0, 972, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message_await_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_52send_message_await_response(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_send_msg, __pyx_v_response_type, __pyx_v_timeout);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_52send_message_await_response(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_send_msg, PyObject *__pyx_v_response_type, PyObject *__pyx_v_timeout) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("send_message_await_response", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 972, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __pyx_cur_scope->__pyx_v_send_msg = __pyx_v_send_msg;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_send_msg);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_send_msg);
  __pyx_cur_scope->__pyx_v_response_type = __pyx_v_response_type;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_response_type);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_response_type);
  __pyx_cur_scope->__pyx_v_timeout = __pyx_v_timeout;
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_timeout);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_timeout);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_54generator8, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_send_message_await_response, __pyx_mstate_global->__pyx_n_u_APIConnection_send_message_await, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 972, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.send_message_await_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_54generator8(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  __Pyx_PySendResult __pyx_t_6;
  PyObject *(*__pyx_t_7)(PyObject *);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("send_message_await_response", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L4_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 972, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":975
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
 *         [response] = await self.send_messages_await_response_complex(             # <<<<<<<<<<<<<<
 *             (send_msg,),
 *             None,  # we will only get responses of `response_type`
*/
  __pyx_t_2 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);

  /* "aioesphomeapi/connection.py":976
 *     ) -> Any:
 *         [response] = await self.send_messages_await_response_complex(
 *             (send_msg,),             # <<<<<<<<<<<<<<
 *             None,  # we will only get responses of `response_type`
 *             None,  # we will only get responses of `response_type`
*/
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 976, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_send_msg);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_send_msg);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_cur_scope->__pyx_v_send_msg) != (0)) __PYX_ERR(0, 976, __pyx_L1_error);

  /* "aioesphomeapi/connection.py":979
 *             None,  # we will only get responses of `response_type`
 *             None,  # we will only get responses of `response_type`
 *             (response_type,),             # <<<<<<<<<<<<<<
 *             timeout,
 *         )
*/
  __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 979, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_response_type);
  __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_response_type);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_cur_scope->__pyx_v_response_type) != (0)) __PYX_ERR(0, 979, __pyx_L1_error);

  /* "aioesphomeapi/connection.py":980
 *             None,  # we will only get responses of `response_type`
 *             (response_type,),
 *             timeout,             # <<<<<<<<<<<<<<
 *         )
 *         return response
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[6] = {__pyx_t_2, __pyx_t_3, Py_None, Py_None, __pyx_t_4, __pyx_cur_scope->__pyx_v_timeout};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_send_messages_await_response_com, __pyx_callargs+__pyx_t_5, (6-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 975, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_6 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_1, &__pyx_r);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_6 == PYGEN_NEXT)) {
    __Pyx_GOTREF(__pyx_r);
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, awaiting value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L4_resume_from_await:;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 975, __pyx_L1_error)
    __pyx_t_1 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_1);
  } else if (likely(__pyx_t_6 == PYGEN_RETURN)) {
    __Pyx_GOTREF(__pyx_r);
    __pyx_t_1 = __pyx_r; __pyx_r = NULL;
  } else {
    __Pyx_XGOTREF(__pyx_r);
    __PYX_ERR(0, 975, __pyx_L1_error)
  }
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
    if (unlikely(size != 1)) {
      if (size > 1) __Pyx_RaiseTooManyValuesError(1);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 975, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 0);
      __Pyx_INCREF(__pyx_t_4);
    } else {
      __pyx_t_4 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 975, __pyx_L1_error)
      __Pyx_XGOTREF(__pyx_t_4);
    }
    #else
    __pyx_t_4 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 975, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 975, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_7 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_3);
    index = 0; __pyx_t_4 = __pyx_t_7(__pyx_t_3); if (unlikely(!__pyx_t_4)) goto __pyx_L5_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_3), 1) < (0)) __PYX_ERR(0, 975, __pyx_L1_error)
    __pyx_t_7 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L6_unpacking_done;
    __pyx_L5_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_7 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 975, __pyx_L1_error)
    __pyx_L6_unpacking_done:;
  }

  /* "aioesphomeapi/connection.py":975
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
 *         [response] = await self.send_messages_await_response_complex(             # <<<<<<<<<<<<<<
 *             (send_msg,),
 *             None,  # we will only get responses of `response_type`
*/
  __Pyx_GIVEREF(__pyx_t_4);
  __pyx_cur_scope->__pyx_v_response = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "aioesphomeapi/connection.py":982
 *             timeout,
 *         )
 *         return response             # <<<<<<<<<<<<<<
 * 
 *     def report_fatal_error(self, err: Exception) -> None:
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_response);
  __pyx_r = __pyx_cur_scope->__pyx_v_response;
  goto __pyx_L0;
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":972
 *         return responses
 * 
 *     async def send_message_await_response(             # <<<<<<<<<<<<<<
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("send_message_await_response", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":984
 *         return response
 * 
 *     def report_fatal_error(self, err: Exception) -> None:             # <<<<<<<<<<<<<<
 *         """Report a fatal error that occurred during an operation.
 * 
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_report_fatal_error(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_err, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_log_level = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("report_fatal_error", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_report_fatal_error); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 984, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v_err};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 984, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":993
 *         This method does not log the error, the call site should do so.
 *         """
 *         if self._fatal_exception is None:             # <<<<<<<<<<<<<<
 *             if self._expected_disconnect is False and self._log_errors:
 *                 # Only log the first error
*/
  __pyx_t_6 = (__pyx_v_self->_fatal_exception == ((PyObject*)Py_None));
  if (__pyx_t_6) {

    /* "aioesphomeapi/connection.py":994
 *         """
 *         if self._fatal_exception is None:
 *             if self._expected_disconnect is False and self._log_errors:             # <<<<<<<<<<<<<<
 *                 # Only log the first error
 *                 log_level = (
*/
    __pyx_t_7 = (__pyx_v_self->_expected_disconnect == 0);
    if (__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_6 = __pyx_v_self->_log_errors;
    __pyx_L5_bool_binop_done:;
    if (__pyx_t_6) {

      /* "aioesphomeapi/connection.py":998
 *                 log_level = (
 *                     logging.DEBUG
 *                     if isinstance(err, APIConnectionCancelledError)             # <<<<<<<<<<<<<<
 *                     else logging.WARNING
 *                 )
*/
      __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_APIConnectionCancelledError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 998, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = PyObject_IsInstance(__pyx_v_err, __pyx_t_2); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 998, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (__pyx_t_6) {

        /* "aioesphomeapi/connection.py":997
 *                 # Only log the first error
 *                 log_level = (
 *                     logging.DEBUG             # <<<<<<<<<<<<<<
 *                     if isinstance(err, APIConnectionCancelledError)
 *                     else logging.WARNING
*/
        __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_logging); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 997, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_DEBUG); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 997, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_1 = __pyx_t_4;
        __pyx_t_4 = 0;
      } else {

        /* "aioesphomeapi/connection.py":999
 *                     logging.DEBUG
 *                     if isinstance(err, APIConnectionCancelledError)
 *                     else logging.WARNING             # <<<<<<<<<<<<<<
 *                 )
 *                 _LOGGER.log(
*/
        __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_logging); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 999, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_WARNING); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 999, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_1 = __pyx_t_2;
        __pyx_t_2 = 0;
      }
      __pyx_v_log_level = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":1001
 *                     else logging.WARNING
 *                 )
 *                 _LOGGER.log(             # <<<<<<<<<<<<<<
 *                     log_level,
 *                     "%s: Connection error occurred: %s",
*/
      __pyx_t_2 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1001, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_log); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1001, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "aioesphomeapi/connection.py":1005
 *                     "%s: Connection error occurred: %s",
 *                     self.log_name,
 *                     err or type(err),             # <<<<<<<<<<<<<<
 *                     exc_info=not str(err)
 *                     or isinstance(
*/
      __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_err); if (unlikely((__pyx_t_6 < 0))) __PYX_ERR(0, 1005, __pyx_L1_error)
      if (!__pyx_t_6) {
      } else {
        __Pyx_INCREF(__pyx_v_err);
        __pyx_t_4 = __pyx_v_err;
        goto __pyx_L7_bool_binop_done;
      }
      __Pyx_INCREF(((PyObject *)Py_TYPE(__pyx_v_err)));
      __pyx_t_4 = ((PyObject *)Py_TYPE(__pyx_v_err));
      __pyx_L7_bool_binop_done:;

      /* "aioesphomeapi/connection.py":1006
 *                     self.log_name,
 *                     err or type(err),
 *                     exc_info=not str(err)             # <<<<<<<<<<<<<<
 *                     or isinstance(
 *                         err, UnhandledAPIConnectionError
*/
      __pyx_t_9 = __Pyx_PyObject_Unicode(__pyx_v_err); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1006, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      {
        Py_ssize_t __pyx_temp = __Pyx_PyUnicode_IS_TRUE(__pyx_t_9);
        if (unlikely(((!CYTHON_ASSUME_SAFE_SIZE) && __pyx_temp < 0))) __PYX_ERR(0, 1006, __pyx_L1_error)
        __pyx_t_6 = (__pyx_temp != 0);
      }

      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_7 = (!__pyx_t_6);
      if (!__pyx_t_7) {
      } else {
        __pyx_t_9 = __Pyx_PyBool_FromLong(__pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1006, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_8 = __pyx_t_9;
        __pyx_t_9 = 0;
        goto __pyx_L9_bool_binop_done;
      }

      /* "aioesphomeapi/connection.py":1008
 *                     exc_info=not str(err)
 *                     or isinstance(
 *                         err, UnhandledAPIConnectionError             # <<<<<<<<<<<<<<
 *                     ),  # Log the full stack on empty error string
 *                 )
*/
      __Pyx_GetModuleGlobalName(__pyx_t_9, __pyx_mstate_global->__pyx_n_u_UnhandledAPIConnectionError); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1008, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);

      /* "aioesphomeapi/connection.py":1007
 *                     err or type(err),
 *                     exc_info=not str(err)
 *                     or isinstance(             # <<<<<<<<<<<<<<
 *                         err, UnhandledAPIConnectionError
 *                     ),  # Log the full stack on empty error string
*/
      __pyx_t_7 = PyObject_IsInstance(__pyx_v_err, __pyx_t_9); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1007, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_t_9 = __Pyx_PyBool_FromLong(__pyx_t_7); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1007, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_9);
      __pyx_t_8 = __pyx_t_9;
      __pyx_t_9 = 0;
      __pyx_L9_bool_binop_done:;
      __pyx_t_5 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
        assert(__pyx_t_2);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
        __pyx_t_5 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[5 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_v_log_level, __pyx_mstate_global->__pyx_kp_u_s_Connection_error_occurred_s, __pyx_v_self->log_name, __pyx_t_4};
        __pyx_t_9 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 1001, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_exc_info, __pyx_t_8, __pyx_t_9, __pyx_callargs+5, 0) < (0)) __PYX_ERR(0, 1001, __pyx_L1_error)
        __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (5-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_9);
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1001, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":994
 *         """
 *         if self._fatal_exception is None:
 *             if self._expected_disconnect is False and self._log_errors:             # <<<<<<<<<<<<<<
 *                 # Only log the first error
 *                 log_level = (
*/
    }

    /* "aioesphomeapi/connection.py":1015
 *             # error will be lost (ie RequiresEncryptionAPIError) and than
 *             # SocketClosedAPIError will be raised instead
 *             self._set_fatal_exception_if_unset(err)             # <<<<<<<<<<<<<<
 * 
 *         self._cleanup()
*/
    ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_set_fatal_exception_if_unset(__pyx_v_self, __pyx_v_err); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1015, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":993
 *         This method does not log the error, the call site should do so.
 *         """
 *         if self._fatal_exception is None:             # <<<<<<<<<<<<<<
 *             if self._expected_disconnect is False and self._log_errors:
 *                 # Only log the first error
*/
  }

  /* "aioesphomeapi/connection.py":1017
 *             self._set_fatal_exception_if_unset(err)
 * 
 *         self._cleanup()             # <<<<<<<<<<<<<<
 * 
 *     def _set_fatal_exception_if_unset(self, err: Exception) -> None:
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_cleanup(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1017, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":984
 *         return response
 * 
 *     def report_fatal_error(self, err: Exception) -> None:             # <<<<<<<<<<<<<<
 *         """Report a fatal error that occurred during an operation.
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.report_fatal_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_log_level);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_55report_fatal_error, "Report a fatal error that occurred during an operation.\n\n        This should only be called for errors that mean the connection\n        can no longer be used.\n\n        The connection will be closed, all exception handlers notified.\n        This method does not log the error, the call site should do so.\n        ");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error = {"report_fatal_error", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_55report_fatal_error};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_err = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("report_fatal_error (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_err,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 984, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 984, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "report_fatal_error", 0) < (0)) __PYX_ERR(0, 984, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("report_fatal_error", 1, 1, 1, i); __PYX_ERR(0, 984, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 984, __pyx_L3_error)
    }
    __pyx_v_err = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("report_fatal_error", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 984, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.report_fatal_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_err), (((PyTypeObject*)PyExc_Exception)), 0, "err", 0))) __PYX_ERR(0, 984, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_55report_fatal_error(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_err);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_55report_fatal_error(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_err) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("report_fatal_error", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection_report_fatal_error(__pyx_v_self, __pyx_v_err, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 984, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.report_fatal_error", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":1019
 *         self._cleanup()
 * 
 *     def _set_fatal_exception_if_unset(self, err: Exception) -> None:             # <<<<<<<<<<<<<<
 *         """Set the fatal exception if it hasn't been set yet."""
 *         if self._fatal_exception is None:
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__set_fatal_exception_if_unset(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_err) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("_set_fatal_exception_if_unset", 0);

  /* "aioesphomeapi/connection.py":1021
 *     def _set_fatal_exception_if_unset(self, err: Exception) -> None:
 *         """Set the fatal exception if it hasn't been set yet."""
 *         if self._fatal_exception is None:             # <<<<<<<<<<<<<<
 *             self._fatal_exception = err
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_fatal_exception == ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":1022
 *         """Set the fatal exception if it hasn't been set yet."""
 *         if self._fatal_exception is None:
 *             self._fatal_exception = err             # <<<<<<<<<<<<<<
 * 
 *     def process_packet(self, msg_type_proto: _int, data: _bytes) -> None:
*/
    __Pyx_INCREF(__pyx_v_err);
    __Pyx_GIVEREF(__pyx_v_err);
    __Pyx_GOTREF(__pyx_v_self->_fatal_exception);
    __Pyx_DECREF(__pyx_v_self->_fatal_exception);
    __pyx_v_self->_fatal_exception = __pyx_v_err;

    /* "aioesphomeapi/connection.py":1021
 *     def _set_fatal_exception_if_unset(self, err: Exception) -> None:
 *         """Set the fatal exception if it hasn't been set yet."""
 *         if self._fatal_exception is None:             # <<<<<<<<<<<<<<
 *             self._fatal_exception = err
 * 
*/
  }

  /* "aioesphomeapi/connection.py":1019
 *         self._cleanup()
 * 
 *     def _set_fatal_exception_if_unset(self, err: Exception) -> None:             # <<<<<<<<<<<<<<
 *         """Set the fatal exception if it hasn't been set yet."""
 *         if self._fatal_exception is None:
*/

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":1024
 *             self._fatal_exception = err
 * 
 *     def process_packet(self, msg_type_proto: _int, data: _bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Process an incoming packet."""
 *         # This method is HOT and extremely performance critical
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_58process_packet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection_process_packet(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, unsigned int __pyx_v_msg_type_proto, PyObject *__pyx_v_data, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_handlers = 0;
  PyObject *__pyx_v_handlers_copy = 0;
  PyObject *__pyx_v_klass_merge = 0;
  PyObject *__pyx_v_klass = NULL;
  PyObject *__pyx_v_merge = NULL;
  PyObject *__pyx_v_msg = NULL;
  PyObject *__pyx_v_e = NULL;
  PyObject *__pyx_v_handler = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  long __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17[6];
  PyObject *__pyx_t_18 = NULL;
  int __pyx_t_19;
  char const *__pyx_t_20;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  PyObject *__pyx_t_24 = NULL;
  PyObject *__pyx_t_25 = NULL;
  PyObject *__pyx_t_26 = NULL;
  Py_ssize_t __pyx_t_27;
  Py_ssize_t __pyx_t_28;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_packet", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_process_packet); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1024, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_58process_packet)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = __Pyx_PyLong_From_unsigned_int(__pyx_v_msg_type_proto); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1024, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_3, __pyx_t_5, __pyx_v_data};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1024, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":1029
 *         # since its called for every incoming packet. Take
 *         # extra care when modifying this method.
 *         try:             # <<<<<<<<<<<<<<
 *             # MESSAGE_NUMBER_TO_PROTO is 0-indexed
 *             # but the message type is 1-indexed
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_9);
    /*try:*/ {

      /* "aioesphomeapi/connection.py":1032
 *             # MESSAGE_NUMBER_TO_PROTO is 0-indexed
 *             # but the message type is 1-indexed
 *             klass_merge = MESSAGE_NUMBER_TO_PROTO[msg_type_proto - 1]             # <<<<<<<<<<<<<<
 *             klass, merge = klass_merge
 *             msg = klass()
*/
      if (unlikely(__pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 1032, __pyx_L3_error)
      }
      __pyx_t_10 = (__pyx_v_msg_type_proto - 1);
      __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO, __pyx_t_10, long, 1, __Pyx_PyLong_From_long, 0, 1, 1, 1, __Pyx_ReferenceSharing_SharedReference); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1032, __pyx_L3_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(0, 1032, __pyx_L3_error)
      __pyx_v_klass_merge = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "aioesphomeapi/connection.py":1033
 *             # but the message type is 1-indexed
 *             klass_merge = MESSAGE_NUMBER_TO_PROTO[msg_type_proto - 1]
 *             klass, merge = klass_merge             # <<<<<<<<<<<<<<
 *             msg = klass()
 *             merge(msg, data)
*/
      if (likely(__pyx_v_klass_merge != Py_None)) {
        PyObject* sequence = __pyx_v_klass_merge;
        Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 1033, __pyx_L3_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_2);
        #else
        __pyx_t_1 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1033, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1033, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
        #endif
      } else {
        __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 1033, __pyx_L3_error)
      }
      __pyx_v_klass = __pyx_t_1;
      __pyx_t_1 = 0;
      __pyx_v_merge = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "aioesphomeapi/connection.py":1034
 *             klass_merge = MESSAGE_NUMBER_TO_PROTO[msg_type_proto - 1]
 *             klass, merge = klass_merge
 *             msg = klass()             # <<<<<<<<<<<<<<
 *             merge(msg, data)
 *         except Exception as e:
*/
      __pyx_t_1 = NULL;
      __Pyx_INCREF(__pyx_v_klass);
      __pyx_t_4 = __pyx_v_klass; 
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
        assert(__pyx_t_1);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_1, NULL};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1034, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __pyx_v_msg = __pyx_t_2;
      __pyx_t_2 = 0;

      /* "aioesphomeapi/connection.py":1035
 *             klass, merge = klass_merge
 *             msg = klass()
 *             merge(msg, data)             # <<<<<<<<<<<<<<
 *         except Exception as e:
 *             # IndexError will be very rare so we check for it
*/
      __pyx_t_4 = NULL;
      __Pyx_INCREF(__pyx_v_merge);
      __pyx_t_1 = __pyx_v_merge; 
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_1))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_1, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[3] = {__pyx_t_4, __pyx_v_msg, __pyx_v_data};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_1, __pyx_callargs+__pyx_t_6, (3-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1035, __pyx_L3_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "aioesphomeapi/connection.py":1029
 *         # since its called for every incoming packet. Take
 *         # extra care when modifying this method.
 *         try:             # <<<<<<<<<<<<<<
 *             # MESSAGE_NUMBER_TO_PROTO is 0-indexed
 *             # but the message type is 1-indexed
*/
    }
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    goto __pyx_L8_try_end;
    __pyx_L3_error:;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "aioesphomeapi/connection.py":1036
 *             msg = klass()
 *             merge(msg, data)
 *         except Exception as e:             # <<<<<<<<<<<<<<
 *             # IndexError will be very rare so we check for it
 *             # after the broad exception catch to avoid having
*/
    __pyx_t_11 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_Exception))));
    if (__pyx_t_11) {
      __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.process_packet", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_1, &__pyx_t_4) < 0) __PYX_ERR(0, 1036, __pyx_L5_except_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_1);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_v_e = __pyx_t_1;
      /*try:*/ {

        /* "aioesphomeapi/connection.py":1040
 *             # after the broad exception catch to avoid having
 *             # to check the exception type twice for the common case
 *             if isinstance(e, IndexError):             # <<<<<<<<<<<<<<
 *                 if self._debug_enabled:
 *                     _LOGGER.debug(
*/
        __pyx_t_12 = __Pyx_PyExc_IndexError_Check(__pyx_v_e); 
        if (__pyx_t_12) {

          /* "aioesphomeapi/connection.py":1041
 *             # to check the exception type twice for the common case
 *             if isinstance(e, IndexError):
 *                 if self._debug_enabled:             # <<<<<<<<<<<<<<
 *                     _LOGGER.debug(
 *                         "%s: Skipping unknown message type %s",
*/
          if (__pyx_v_self->_debug_enabled) {

            /* "aioesphomeapi/connection.py":1042
 *             if isinstance(e, IndexError):
 *                 if self._debug_enabled:
 *                     _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                         "%s: Skipping unknown message type %s",
 *                         self.log_name,
*/
            __pyx_t_3 = NULL;
            __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1042, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1042, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_14);
            __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

            /* "aioesphomeapi/connection.py":1045
 *                         "%s: Skipping unknown message type %s",
 *                         self.log_name,
 *                         msg_type_proto,             # <<<<<<<<<<<<<<
 *                     )
 *                 return
*/
            __pyx_t_13 = __Pyx_PyLong_From_unsigned_int(__pyx_v_msg_type_proto); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1045, __pyx_L14_error)
            __Pyx_GOTREF(__pyx_t_13);
            __pyx_t_6 = 1;
            #if CYTHON_UNPACK_METHODS
            if (unlikely(PyMethod_Check(__pyx_t_14))) {
              __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_14);
              assert(__pyx_t_3);
              PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
              __Pyx_INCREF(__pyx_t_3);
              __Pyx_INCREF(__pyx__function);
              __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
              __pyx_t_6 = 0;
            }
            #endif
            {
              PyObject *__pyx_callargs[4] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_s_Skipping_unknown_message_type, __pyx_v_self->log_name, __pyx_t_13};
              __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_6, (4-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
              __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
              __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
              __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
              if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1042, __pyx_L14_error)
              __Pyx_GOTREF(__pyx_t_5);
            }
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

            /* "aioesphomeapi/connection.py":1041
 *             # to check the exception type twice for the common case
 *             if isinstance(e, IndexError):
 *                 if self._debug_enabled:             # <<<<<<<<<<<<<<
 *                     _LOGGER.debug(
 *                         "%s: Skipping unknown message type %s",
*/
          }

          /* "aioesphomeapi/connection.py":1047
 *                         msg_type_proto,
 *                     )
 *                 return             # <<<<<<<<<<<<<<
 *             _LOGGER.exception(
 *                 "%s: Invalid protobuf message: type=%s data=%s",
*/
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          goto __pyx_L13_return;

          /* "aioesphomeapi/connection.py":1040
 *             # after the broad exception catch to avoid having
 *             # to check the exception type twice for the common case
 *             if isinstance(e, IndexError):             # <<<<<<<<<<<<<<
 *                 if self._debug_enabled:
 *                     _LOGGER.debug(
*/
        }

        /* "aioesphomeapi/connection.py":1048
 *                     )
 *                 return
 *             _LOGGER.exception(             # <<<<<<<<<<<<<<
 *                 "%s: Invalid protobuf message: type=%s data=%s",
 *                 self.log_name,
*/
        __pyx_t_14 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1048, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_exception); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1048, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

        /* "aioesphomeapi/connection.py":1051
 *                 "%s: Invalid protobuf message: type=%s data=%s",
 *                 self.log_name,
 *                 klass.__name__,             # <<<<<<<<<<<<<<
 *                 data,
 *             )
*/
        if (unlikely(!__pyx_v_klass)) { __Pyx_RaiseUnboundLocalError("klass"); __PYX_ERR(0, 1051, __pyx_L14_error) }
        __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_v_klass, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1051, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_13);

        /* "aioesphomeapi/connection.py":1052
 *                 self.log_name,
 *                 klass.__name__,
 *                 data,             # <<<<<<<<<<<<<<
 *             )
 *             self.report_fatal_error(
*/
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_14 = PyMethod_GET_SELF(__pyx_t_3);
          assert(__pyx_t_14);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_14);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[5] = {__pyx_t_14, __pyx_mstate_global->__pyx_kp_u_s_Invalid_protobuf_message_type, __pyx_v_self->log_name, __pyx_t_13, __pyx_v_data};
          __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (5-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1048, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "aioesphomeapi/connection.py":1055
 *             )
 *             self.report_fatal_error(
 *                 ProtocolAPIError(             # <<<<<<<<<<<<<<
 *                     f"Invalid protobuf message: type={klass.__name__} data={data!r}: {e}"
 *                 )
*/
        __pyx_t_3 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_ProtocolAPIError); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1055, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_13);

        /* "aioesphomeapi/connection.py":1056
 *             self.report_fatal_error(
 *                 ProtocolAPIError(
 *                     f"Invalid protobuf message: type={klass.__name__} data={data!r}: {e}"             # <<<<<<<<<<<<<<
 *                 )
 *             )
*/
        if (unlikely(!__pyx_v_klass)) { __Pyx_RaiseUnboundLocalError("klass"); __PYX_ERR(0, 1056, __pyx_L14_error) }
        __pyx_t_14 = __Pyx_PyObject_GetAttrStr(__pyx_v_klass, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1056, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_15 = __Pyx_PyObject_FormatSimple(__pyx_t_14, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_15)) __PYX_ERR(0, 1056, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_15);
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __pyx_t_14 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_data), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1056, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_14);
        __pyx_t_16 = __Pyx_PyObject_FormatSimple(__pyx_v_e, __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1056, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_16);
        __pyx_t_17[0] = __pyx_mstate_global->__pyx_kp_u_Invalid_protobuf_message_type;
        __pyx_t_17[1] = __pyx_t_15;
        __pyx_t_17[2] = __pyx_mstate_global->__pyx_kp_u_data;
        __pyx_t_17[3] = __pyx_t_14;
        __pyx_t_17[4] = __pyx_mstate_global->__pyx_kp_u__6;
        __pyx_t_17[5] = __pyx_t_16;
        __pyx_t_18 = __Pyx_PyUnicode_Join(__pyx_t_17, 6, 31 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_15) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_14) + 2 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_16), 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_15) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_14) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_16));
        if (unlikely(!__pyx_t_18)) __PYX_ERR(0, 1056, __pyx_L14_error)
        __Pyx_GOTREF(__pyx_t_18);
        __Pyx_DECREF(__pyx_t_15); __pyx_t_15 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
        __pyx_t_6 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_13))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_13);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
          __pyx_t_6 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_18};
          __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1055, __pyx_L14_error)
          __Pyx_GOTREF(__pyx_t_5);
        }

        /* "aioesphomeapi/connection.py":1055
 *             )
 *             self.report_fatal_error(
 *                 ProtocolAPIError(             # <<<<<<<<<<<<<<
 *                     f"Invalid protobuf message: type={klass.__name__} data={data!r}: {e}"
 *                 )
*/
        if (!(likely(__Pyx_PyException_Check(__pyx_t_5))||((__pyx_t_5) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_5))) __PYX_ERR(0, 1055, __pyx_L14_error)

        /* "aioesphomeapi/connection.py":1054
 *                 data,
 *             )
 *             self.report_fatal_error(             # <<<<<<<<<<<<<<
 *                 ProtocolAPIError(
 *                     f"Invalid protobuf message: type={klass.__name__} data={data!r}: {e}"
*/
        ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->report_fatal_error(__pyx_v_self, ((PyObject*)__pyx_t_5), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1054, __pyx_L14_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "aioesphomeapi/connection.py":1059
 *                 )
 *             )
 *             raise             # <<<<<<<<<<<<<<
 * 
 *         if self._debug_enabled:
*/
        __Pyx_GIVEREF(__pyx_t_2);
        __Pyx_GIVEREF(__pyx_t_1);
        __Pyx_XGIVEREF(__pyx_t_4);
        __Pyx_ErrRestoreWithState(__pyx_t_2, __pyx_t_1, __pyx_t_4);
        __pyx_t_2 = 0;  __pyx_t_1 = 0;  __pyx_t_4 = 0; 
        __PYX_ERR(0, 1059, __pyx_L14_error)
      }

      /* "aioesphomeapi/connection.py":1036
 *             msg = klass()
 *             merge(msg, data)
 *         except Exception as e:             # <<<<<<<<<<<<<<
 *             # IndexError will be very rare so we check for it
 *             # after the broad exception catch to avoid having
*/
      /*finally:*/ {
        __pyx_L14_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0; __pyx_t_26 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
          __Pyx_XDECREF(__pyx_t_15); __pyx_t_15 = 0;
          __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
          __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_24, &__pyx_t_25, &__pyx_t_26);
          if ( unlikely(__Pyx_GetException(&__pyx_t_21, &__pyx_t_22, &__pyx_t_23) < 0)) __Pyx_ErrFetch(&__pyx_t_21, &__pyx_t_22, &__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_21);
          __Pyx_XGOTREF(__pyx_t_22);
          __Pyx_XGOTREF(__pyx_t_23);
          __Pyx_XGOTREF(__pyx_t_24);
          __Pyx_XGOTREF(__pyx_t_25);
          __Pyx_XGOTREF(__pyx_t_26);
          __pyx_t_11 = __pyx_lineno; __pyx_t_19 = __pyx_clineno; __pyx_t_20 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_24);
          __Pyx_XGIVEREF(__pyx_t_25);
          __Pyx_XGIVEREF(__pyx_t_26);
          __Pyx_ExceptionReset(__pyx_t_24, __pyx_t_25, __pyx_t_26);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_22);
          __Pyx_XGIVEREF(__pyx_t_23);
          __Pyx_ErrRestore(__pyx_t_21, __pyx_t_22, __pyx_t_23);
          __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0; __pyx_t_24 = 0; __pyx_t_25 = 0; __pyx_t_26 = 0;
          __pyx_lineno = __pyx_t_11; __pyx_clineno = __pyx_t_19; __pyx_filename = __pyx_t_20;
          goto __pyx_L5_except_error;
        }
        __pyx_L13_return: {
          __Pyx_DECREF(__pyx_v_e); __pyx_v_e = 0;
          goto __pyx_L6_except_return;
        }
      }
    }
    goto __pyx_L5_except_error;

    /* "aioesphomeapi/connection.py":1029
 *         # since its called for every incoming packet. Take
 *         # extra care when modifying this method.
 *         try:             # <<<<<<<<<<<<<<
 *             # MESSAGE_NUMBER_TO_PROTO is 0-indexed
 *             # but the message type is 1-indexed
*/
    __pyx_L5_except_error:;
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L1_error;
    __pyx_L6_except_return:;
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
    goto __pyx_L0;
    __pyx_L8_try_end:;
  }

  /* "aioesphomeapi/connection.py":1061
 *             raise
 * 
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Got message of type %s (len=%d): %s",
*/
  if (__pyx_v_self->_debug_enabled) {

    /* "aioesphomeapi/connection.py":1062
 * 
 *         if self._debug_enabled:
 *             _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                 "%s: Got message of type %s (len=%d): %s",
 *                 self.log_name,
*/
    __pyx_t_1 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1062, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "aioesphomeapi/connection.py":1065
 *                 "%s: Got message of type %s (len=%d): %s",
 *                 self.log_name,
 *                 type(msg).__name__,             # <<<<<<<<<<<<<<
 *                 len(data),
 *                 # calling __str__ on the message may crash on
*/
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)Py_TYPE(__pyx_v_msg)), __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1065, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "aioesphomeapi/connection.py":1066
 *                 self.log_name,
 *                 type(msg).__name__,
 *                 len(data),             # <<<<<<<<<<<<<<
 *                 # calling __str__ on the message may crash on
 *                 # Windows systems due to a bug in the protobuf library
*/
    __pyx_t_27 = PyObject_Length(__pyx_v_data); if (unlikely(__pyx_t_27 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1066, __pyx_L1_error)
    __pyx_t_13 = PyLong_FromSsize_t(__pyx_t_27); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1066, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);

    /* "aioesphomeapi/connection.py":1070
 *                 # Windows systems due to a bug in the protobuf library
 *                 # so we call MessageToDict instead
 *                 MessageToDict(msg) if _WIN32 else msg,             # <<<<<<<<<<<<<<
 *             )
 * 
*/
    if (__pyx_v_13aioesphomeapi_10connection__WIN32) {
      __pyx_t_16 = NULL;
      __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_MessageToDict); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 1070, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_14);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_14))) {
        __pyx_t_16 = PyMethod_GET_SELF(__pyx_t_14);
        assert(__pyx_t_16);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_14);
        __Pyx_INCREF(__pyx_t_16);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_14, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_16, __pyx_v_msg};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_16); __pyx_t_16 = 0;
        __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1070, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __pyx_t_18 = __pyx_t_3;
      __pyx_t_3 = 0;
    } else {
      __Pyx_INCREF(__pyx_v_msg);
      __pyx_t_18 = __pyx_v_msg;
    }
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_1);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[6] = {__pyx_t_1, __pyx_mstate_global->__pyx_kp_u_s_Got_message_of_type_s_len_d_s, __pyx_v_self->log_name, __pyx_t_2, __pyx_t_13, __pyx_t_18};
      __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (6-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1062, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":1061
 *             raise
 * 
 *         if self._debug_enabled:             # <<<<<<<<<<<<<<
 *             _LOGGER.debug(
 *                 "%s: Got message of type %s (len=%d): %s",
*/
  }

  /* "aioesphomeapi/connection.py":1073
 *             )
 * 
 *         if self._pong_timer is not None:             # <<<<<<<<<<<<<<
 *             # Any valid message from the remote cancels the pong timer
 *             # as we know the connection is still alive
*/
  __pyx_t_12 = (__pyx_v_self->_pong_timer != Py_None);
  if (__pyx_t_12) {

    /* "aioesphomeapi/connection.py":1076
 *             # Any valid message from the remote cancels the pong timer
 *             # as we know the connection is still alive
 *             self._async_cancel_pong_timer()             # <<<<<<<<<<<<<<
 * 
 *         if self._send_pending_ping:
*/
    ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_async_cancel_pong_timer(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1076, __pyx_L1_error)

    /* "aioesphomeapi/connection.py":1073
 *             )
 * 
 *         if self._pong_timer is not None:             # <<<<<<<<<<<<<<
 *             # Any valid message from the remote cancels the pong timer
 *             # as we know the connection is still alive
*/
  }

  /* "aioesphomeapi/connection.py":1078
 *             self._async_cancel_pong_timer()
 * 
 *         if self._send_pending_ping:             # <<<<<<<<<<<<<<
 *             # Any valid message from the remove cancels the pending ping
 *             # since we know the connection is still alive
*/
  if (__pyx_v_self->_send_pending_ping) {

    /* "aioesphomeapi/connection.py":1081
 *             # Any valid message from the remove cancels the pending ping
 *             # since we know the connection is still alive
 *             self._send_pending_ping = False             # <<<<<<<<<<<<<<
 * 
 *         if (handlers := self._message_handlers.get(type(msg))) is None:
*/
    __pyx_v_self->_send_pending_ping = 0;

    /* "aioesphomeapi/connection.py":1078
 *             self._async_cancel_pong_timer()
 * 
 *         if self._send_pending_ping:             # <<<<<<<<<<<<<<
 *             # Any valid message from the remove cancels the pending ping
 *             # since we know the connection is still alive
*/
  }

  /* "aioesphomeapi/connection.py":1083
 *             self._send_pending_ping = False
 * 
 *         if (handlers := self._message_handlers.get(type(msg))) is None:             # <<<<<<<<<<<<<<
 *             return
 * 
*/
  if (unlikely(__pyx_v_self->_message_handlers == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 1083, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyDict_GetItemDefault(__pyx_v_self->_message_handlers, ((PyObject *)Py_TYPE(__pyx_v_msg)), Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1083, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (!(likely(PySet_CheckExact(__pyx_t_4))||((__pyx_t_4) == Py_None) || __Pyx_RaiseUnexpectedTypeError("set", __pyx_t_4))) __PYX_ERR(0, 1083, __pyx_L1_error)
  __Pyx_INCREF(__pyx_t_4);
  __pyx_v_handlers = ((PyObject*)__pyx_t_4);
  __pyx_t_12 = (__pyx_t_4 == ((PyObject*)Py_None));
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__pyx_t_12) {

    /* "aioesphomeapi/connection.py":1084
 * 
 *         if (handlers := self._message_handlers.get(type(msg))) is None:
 *             return             # <<<<<<<<<<<<<<
 * 
 *         if len(handlers) > 1:
*/
    goto __pyx_L0;

    /* "aioesphomeapi/connection.py":1083
 *             self._send_pending_ping = False
 * 
 *         if (handlers := self._message_handlers.get(type(msg))) is None:             # <<<<<<<<<<<<<<
 *             return
 * 
*/
  }

  /* "aioesphomeapi/connection.py":1086
 *             return
 * 
 *         if len(handlers) > 1:             # <<<<<<<<<<<<<<
 *             # Handlers are allowed to remove themselves
 *             # so we need to copy the set to avoid a
*/
  if (unlikely(__pyx_v_handlers == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 1086, __pyx_L1_error)
  }
  __pyx_t_27 = __Pyx_PySet_GET_SIZE(__pyx_v_handlers); if (unlikely(__pyx_t_27 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1086, __pyx_L1_error)
  __pyx_t_12 = (__pyx_t_27 > 1);
  if (__pyx_t_12) {

    /* "aioesphomeapi/connection.py":1093
 *             # than one handler registered for the message
 *             # type.
 *             handlers_copy = handlers.copy()             # <<<<<<<<<<<<<<
 *             for handler in handlers_copy:
 *                 handler(msg)
*/
    __pyx_t_4 = __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PySet_Type__copy, __pyx_v_handlers); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1093, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_handlers_copy = ((PyObject*)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":1094
 *             # type.
 *             handlers_copy = handlers.copy()
 *             for handler in handlers_copy:             # <<<<<<<<<<<<<<
 *                 handler(msg)
 *             return
*/
    __pyx_t_27 = 0;
    __pyx_t_5 = __Pyx_set_iterator(__pyx_v_handlers_copy, 1, (&__pyx_t_28), (&__pyx_t_19)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1094, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4);
    __pyx_t_4 = __pyx_t_5;
    __pyx_t_5 = 0;
    while (1) {
      __pyx_t_11 = __Pyx_set_iter_next(__pyx_t_4, __pyx_t_28, &__pyx_t_27, &__pyx_t_5, __pyx_t_19);
      if (unlikely(__pyx_t_11 == 0)) break;
      if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 1094, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_handler, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "aioesphomeapi/connection.py":1095
 *             handlers_copy = handlers.copy()
 *             for handler in handlers_copy:
 *                 handler(msg)             # <<<<<<<<<<<<<<
 *             return
 * 
*/
      __pyx_t_18 = NULL;
      __Pyx_INCREF(__pyx_v_handler);
      __pyx_t_13 = __pyx_v_handler; 
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_13))) {
        __pyx_t_18 = PyMethod_GET_SELF(__pyx_t_13);
        assert(__pyx_t_18);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
        __Pyx_INCREF(__pyx_t_18);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_18, __pyx_v_msg};
        __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_18); __pyx_t_18 = 0;
        __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1095, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":1096
 *             for handler in handlers_copy:
 *                 handler(msg)
 *             return             # <<<<<<<<<<<<<<
 * 
 *         # Most common case, only one handler:
*/
    goto __pyx_L0;

    /* "aioesphomeapi/connection.py":1086
 *             return
 * 
 *         if len(handlers) > 1:             # <<<<<<<<<<<<<<
 *             # Handlers are allowed to remove themselves
 *             # so we need to copy the set to avoid a
*/
  }

  /* "aioesphomeapi/connection.py":1103
 *         # only one handler because Cython will
 *         # poorly optimize next(iter(handlers))
 *         for handler in handlers:             # <<<<<<<<<<<<<<
 *             handler(msg)
 *             break
*/
  __pyx_t_28 = 0;
  __pyx_t_5 = __Pyx_set_iterator(__pyx_v_handlers, 1, (&__pyx_t_27), (&__pyx_t_19)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_4);
  __pyx_t_4 = __pyx_t_5;
  __pyx_t_5 = 0;
  while (1) {
    __pyx_t_11 = __Pyx_set_iter_next(__pyx_t_4, __pyx_t_27, &__pyx_t_28, &__pyx_t_5, __pyx_t_19);
    if (unlikely(__pyx_t_11 == 0)) break;
    if (unlikely(__pyx_t_11 == -1)) __PYX_ERR(0, 1103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_v_handler = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "aioesphomeapi/connection.py":1104
 *         # poorly optimize next(iter(handlers))
 *         for handler in handlers:
 *             handler(msg)             # <<<<<<<<<<<<<<
 *             break
 * 
*/
    __pyx_t_13 = NULL;
    __Pyx_INCREF(__pyx_v_handler);
    __pyx_t_18 = __pyx_v_handler; 
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_18))) {
      __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_18);
      assert(__pyx_t_13);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_18);
      __Pyx_INCREF(__pyx_t_13);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_18, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_13, __pyx_v_msg};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_18, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_DECREF(__pyx_t_18); __pyx_t_18 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1104, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "aioesphomeapi/connection.py":1105
 *         for handler in handlers:
 *             handler(msg)
 *             break             # <<<<<<<<<<<<<<
 * 
 *     def _register_internal_message_handlers(self) -> None:
*/
    goto __pyx_L30_break;
  }
  __pyx_L30_break:;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "aioesphomeapi/connection.py":1024
 *             self._fatal_exception = err
 * 
 *     def process_packet(self, msg_type_proto: _int, data: _bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Process an incoming packet."""
 *         # This method is HOT and extremely performance critical
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_XDECREF(__pyx_t_16);
  __Pyx_XDECREF(__pyx_t_18);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.process_packet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_handlers);
  __Pyx_XDECREF(__pyx_v_handlers_copy);
  __Pyx_XDECREF(__pyx_v_klass_merge);
  __Pyx_XDECREF(__pyx_v_klass);
  __Pyx_XDECREF(__pyx_v_merge);
  __Pyx_XDECREF(__pyx_v_msg);
  __Pyx_XDECREF(__pyx_v_e);
  __Pyx_XDECREF(__pyx_v_handler);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_58process_packet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_57process_packet, "Process an incoming packet.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_58process_packet = {"process_packet", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_58process_packet, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_57process_packet};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_58process_packet(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  unsigned int __pyx_v_msg_type_proto;
  PyObject *__pyx_v_data = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("process_packet (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_msg_type_proto,&__pyx_mstate_global->__pyx_n_u_data_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1024, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1024, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1024, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "process_packet", 0) < (0)) __PYX_ERR(0, 1024, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("process_packet", 1, 2, 2, i); __PYX_ERR(0, 1024, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1024, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1024, __pyx_L3_error)
    }
    __pyx_v_msg_type_proto = __Pyx_PyLong_As_unsigned_int(values[0]); if (unlikely((__pyx_v_msg_type_proto == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(0, 1024, __pyx_L3_error)
    __pyx_v_data = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("process_packet", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 1024, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.process_packet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_57process_packet(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v_msg_type_proto, __pyx_v_data);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_57process_packet(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, unsigned int __pyx_v_msg_type_proto, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("process_packet", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection_process_packet(__pyx_v_self, __pyx_v_msg_type_proto, __pyx_v_data, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1024, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.process_packet", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":1107
 *             break
 * 
 *     def _register_internal_message_handlers(self) -> None:             # <<<<<<<<<<<<<<
 *         """Register internal message handlers."""
 *         self._add_message_callback_without_remove(
*/

static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__register_internal_message_handlers(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_register_internal_message_handlers", 0);

  /* "aioesphomeapi/connection.py":1110
 *         """Register internal message handlers."""
 *         self._add_message_callback_without_remove(
 *             self._handle_disconnect_request_internal, (DisconnectRequest,)             # <<<<<<<<<<<<<<
 *         )
 *         self._add_message_callback_without_remove(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_disconnect_request_inter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_DisconnectRequest);
  __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_DisconnectRequest);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_13aioesphomeapi_10connection_DisconnectRequest) != (0)) __PYX_ERR(0, 1110, __pyx_L1_error);

  /* "aioesphomeapi/connection.py":1109
 *     def _register_internal_message_handlers(self) -> None:
 *         """Register internal message handlers."""
 *         self._add_message_callback_without_remove(             # <<<<<<<<<<<<<<
 *             self._handle_disconnect_request_internal, (DisconnectRequest,)
 *         )
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_add_message_callback_without_remove(__pyx_v_self, __pyx_t_1, ((PyObject*)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":1113
 *         )
 *         self._add_message_callback_without_remove(
 *             self._handle_ping_request_internal, (PingRequest,)             # <<<<<<<<<<<<<<
 *         )
 *         self._add_message_callback_without_remove(
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_ping_request_internal); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_PingRequest);
  __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_PingRequest);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_13aioesphomeapi_10connection_PingRequest) != (0)) __PYX_ERR(0, 1113, __pyx_L1_error);

  /* "aioesphomeapi/connection.py":1112
 *             self._handle_disconnect_request_internal, (DisconnectRequest,)
 *         )
 *         self._add_message_callback_without_remove(             # <<<<<<<<<<<<<<
 *             self._handle_ping_request_internal, (PingRequest,)
 *         )
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_add_message_callback_without_remove(__pyx_v_self, __pyx_t_2, ((PyObject*)__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1112, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":1116
 *         )
 *         self._add_message_callback_without_remove(
 *             self._handle_get_time_request_internal, (GetTimeRequest,)             # <<<<<<<<<<<<<<
 *         )
 *         self._add_message_callback_without_remove(
*/
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_get_time_request_interna); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_GetTimeRequest);
  __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_GetTimeRequest);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_13aioesphomeapi_10connection_GetTimeRequest) != (0)) __PYX_ERR(0, 1116, __pyx_L1_error);

  /* "aioesphomeapi/connection.py":1115
 *             self._handle_ping_request_internal, (PingRequest,)
 *         )
 *         self._add_message_callback_without_remove(             # <<<<<<<<<<<<<<
 *             self._handle_get_time_request_internal, (GetTimeRequest,)
 *         )
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_add_message_callback_without_remove(__pyx_v_self, __pyx_t_1, ((PyObject*)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":1119
 *         )
 *         self._add_message_callback_without_remove(
 *             self._handle_login_response, (AuthenticationResponse,)             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_login_response); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_AuthenticationResponse);
  __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_AuthenticationResponse);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_13aioesphomeapi_10connection_AuthenticationResponse) != (0)) __PYX_ERR(0, 1119, __pyx_L1_error);

  /* "aioesphomeapi/connection.py":1118
 *             self._handle_get_time_request_internal, (GetTimeRequest,)
 *         )
 *         self._add_message_callback_without_remove(             # <<<<<<<<<<<<<<
 *             self._handle_login_response, (AuthenticationResponse,)
 *         )
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_add_message_callback_without_remove(__pyx_v_self, __pyx_t_2, ((PyObject*)__pyx_t_1)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1118, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":1107
 *             break
 * 
 *     def _register_internal_message_handlers(self) -> None:             # <<<<<<<<<<<<<<
 *         """Register internal message handlers."""
 *         self._add_message_callback_without_remove(
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._register_internal_message_handlers", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "aioesphomeapi/connection.py":1122
 *         )
 * 
 *     def _handle_disconnect_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: DisconnectRequest
 *     ) -> None:
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_disconnect_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__msg, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_disconnect_request_internal", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_disconnect_request_inter); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v__msg};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1122, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":1129
 *         # the response if for some reason sending the response
 *         # fails we will still mark the disconnect as expected
 *         self._expected_disconnect = True             # <<<<<<<<<<<<<<
 *         self.send_messages(DISCONNECT_RESPONSE_MESSAGES)
 *         self._cleanup()
*/
  __pyx_v_self->_expected_disconnect = 1;

  /* "aioesphomeapi/connection.py":1130
 *         # fails we will still mark the disconnect as expected
 *         self._expected_disconnect = True
 *         self.send_messages(DISCONNECT_RESPONSE_MESSAGES)             # <<<<<<<<<<<<<<
 *         self._cleanup()
 * 
*/
  __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_DISCONNECT_RESPONSE_MESSAGES;
  __Pyx_INCREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1130, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":1131
 *         self._expected_disconnect = True
 *         self.send_messages(DISCONNECT_RESPONSE_MESSAGES)
 *         self._cleanup()             # <<<<<<<<<<<<<<
 * 
 *     def _handle_ping_request_internal(  # pylint: disable=unused-argument
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_cleanup(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1131, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":1122
 *         )
 * 
 *     def _handle_disconnect_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: DisconnectRequest
 *     ) -> None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_disconnect_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_59_handle_disconnect_request_internal, "Handle a DisconnectRequest.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal = {"_handle_disconnect_request_internal", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_59_handle_disconnect_request_internal};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v__msg = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_handle_disconnect_request_internal (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_msg_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1122, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1122, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_handle_disconnect_request_internal", 0) < (0)) __PYX_ERR(0, 1122, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_handle_disconnect_request_internal", 1, 1, 1, i); __PYX_ERR(0, 1122, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1122, __pyx_L3_error)
    }
    __pyx_v__msg = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_handle_disconnect_request_internal", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1122, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_disconnect_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_59_handle_disconnect_request_internal(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v__msg);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_59_handle_disconnect_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v__msg) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_disconnect_request_internal", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_disconnect_request_internal(__pyx_v_self, __pyx_v__msg, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1122, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_disconnect_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":1133
 *         self._cleanup()
 * 
 *     def _handle_ping_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: PingRequest
 *     ) -> None:
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_ping_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__msg, int __pyx_skip_dispatch) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_ping_request_internal", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_ping_request_internal); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1133, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v__msg};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1133, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":1137
 *     ) -> None:
 *         """Handle a PingRequest."""
 *         self.send_messages(PING_RESPONSE_MESSAGES)             # <<<<<<<<<<<<<<
 * 
 *     def _handle_get_time_request_internal(  # pylint: disable=unused-argument
*/
  __pyx_t_1 = __pyx_v_13aioesphomeapi_10connection_PING_RESPONSE_MESSAGES;
  __Pyx_INCREF(__pyx_t_1);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_1), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1137, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":1133
 *         self._cleanup()
 * 
 *     def _handle_ping_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: PingRequest
 *     ) -> None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_ping_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_61_handle_ping_request_internal, "Handle a PingRequest.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal = {"_handle_ping_request_internal", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_61_handle_ping_request_internal};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v__msg = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_handle_ping_request_internal (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_msg_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1133, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1133, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_handle_ping_request_internal", 0) < (0)) __PYX_ERR(0, 1133, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_handle_ping_request_internal", 1, 1, 1, i); __PYX_ERR(0, 1133, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1133, __pyx_L3_error)
    }
    __pyx_v__msg = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_handle_ping_request_internal", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1133, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_ping_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_61_handle_ping_request_internal(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v__msg);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_61_handle_ping_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v__msg) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_ping_request_internal", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_ping_request_internal(__pyx_v_self, __pyx_v__msg, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1133, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_ping_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":1139
 *         self.send_messages(PING_RESPONSE_MESSAGES)
 * 
 *     def _handle_get_time_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: GetTimeRequest
 *     ) -> None:
*/

static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static void __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_get_time_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v__msg, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_resp = NULL;
  PyObject *__pyx_v_parsed_tz = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_get_time_request_internal", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_handle_get_time_request_interna); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1139, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal)) {
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v__msg};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1139, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "aioesphomeapi/connection.py":1143
 *     ) -> None:
 *         """Handle a GetTimeRequest."""
 *         resp = GetTimeResponse()             # <<<<<<<<<<<<<<
 *         resp.epoch_seconds = int(time.time())
 *         resp.timezone = self._cached_timezone
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_GetTimeResponse);
  __pyx_t_4 = __pyx_v_13aioesphomeapi_10connection_GetTimeResponse; 
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_2);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1143, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_resp = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "aioesphomeapi/connection.py":1144
 *         """Handle a GetTimeRequest."""
 *         resp = GetTimeResponse()
 *         resp.epoch_seconds = int(time.time())             # <<<<<<<<<<<<<<
 *         resp.timezone = self._cached_timezone
 *         parsed_tz = _build_parsed_tz_proto(self._cached_timezone)
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_time); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_time); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_4);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_4);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_t_3 = __Pyx_PyNumber_Int(__pyx_t_1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_epoch_seconds, __pyx_t_3) < (0)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":1145
 *         resp = GetTimeResponse()
 *         resp.epoch_seconds = int(time.time())
 *         resp.timezone = self._cached_timezone             # <<<<<<<<<<<<<<
 *         parsed_tz = _build_parsed_tz_proto(self._cached_timezone)
 *         if parsed_tz is not None:
*/
  __pyx_t_3 = __pyx_v_self->_cached_timezone;
  __Pyx_INCREF(__pyx_t_3);
  if (__Pyx_PyObject_SetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_timezone, __pyx_t_3) < (0)) __PYX_ERR(0, 1145, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":1146
 *         resp.epoch_seconds = int(time.time())
 *         resp.timezone = self._cached_timezone
 *         parsed_tz = _build_parsed_tz_proto(self._cached_timezone)             # <<<<<<<<<<<<<<
 *         if parsed_tz is not None:
 *             resp.parsed_timezone.CopyFrom(parsed_tz)
*/
  __pyx_t_1 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_build_parsed_tz_proto); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
    assert(__pyx_t_1);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_1, __pyx_v_self->_cached_timezone};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_v_parsed_tz = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":1147
 *         resp.timezone = self._cached_timezone
 *         parsed_tz = _build_parsed_tz_proto(self._cached_timezone)
 *         if parsed_tz is not None:             # <<<<<<<<<<<<<<
 *             resp.parsed_timezone.CopyFrom(parsed_tz)
 *         self.send_messages((resp,))
*/
  __pyx_t_6 = (__pyx_v_parsed_tz != Py_None);
  if (__pyx_t_6) {

    /* "aioesphomeapi/connection.py":1148
 *         parsed_tz = _build_parsed_tz_proto(self._cached_timezone)
 *         if parsed_tz is not None:
 *             resp.parsed_timezone.CopyFrom(parsed_tz)             # <<<<<<<<<<<<<<
 *         self.send_messages((resp,))
 * 
*/
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_resp, __pyx_mstate_global->__pyx_n_u_parsed_timezone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1148, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_parsed_tz};
      __pyx_t_3 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_CopyFrom, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1148, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "aioesphomeapi/connection.py":1147
 *         resp.timezone = self._cached_timezone
 *         parsed_tz = _build_parsed_tz_proto(self._cached_timezone)
 *         if parsed_tz is not None:             # <<<<<<<<<<<<<<
 *             resp.parsed_timezone.CopyFrom(parsed_tz)
 *         self.send_messages((resp,))
*/
  }

  /* "aioesphomeapi/connection.py":1149
 *         if parsed_tz is not None:
 *             resp.parsed_timezone.CopyFrom(parsed_tz)
 *         self.send_messages((resp,))             # <<<<<<<<<<<<<<
 * 
 *     async def disconnect(self) -> None:
*/
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_resp);
  __Pyx_GIVEREF(__pyx_v_resp);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_resp) != (0)) __PYX_ERR(0, 1149, __pyx_L1_error);
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_3), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "aioesphomeapi/connection.py":1139
 *         self.send_messages(PING_RESPONSE_MESSAGES)
 * 
 *     def _handle_get_time_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: GetTimeRequest
 *     ) -> None:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_get_time_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_resp);
  __Pyx_XDECREF(__pyx_v_parsed_tz);
  __Pyx_RefNannyFinishContext();
}

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_63_handle_get_time_request_internal, "Handle a GetTimeRequest.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal = {"_handle_get_time_request_internal", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_63_handle_get_time_request_internal};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v__msg = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_handle_get_time_request_internal (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_msg_2,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1139, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1139, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "_handle_get_time_request_internal", 0) < (0)) __PYX_ERR(0, 1139, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("_handle_get_time_request_internal", 1, 1, 1, i); __PYX_ERR(0, 1139, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1139, __pyx_L3_error)
    }
    __pyx_v__msg = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_handle_get_time_request_internal", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1139, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_get_time_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_63_handle_get_time_request_internal(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v__msg);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_63_handle_get_time_request_internal(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v__msg) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_handle_get_time_request_internal", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_get_time_request_internal(__pyx_v_self, __pyx_v__msg, 1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1139, __pyx_L1_error)
  __pyx_t_1 = __Pyx_void_to_None(NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handle_get_time_request_internal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_67generator9(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "aioesphomeapi/connection.py":1151
 *         self.send_messages((resp,))
 * 
 *     async def disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Disconnect from the API."""
 *         if self._finish_connect_future is not None:
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_66disconnect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_65disconnect, "Disconnect from the API.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_66disconnect = {"disconnect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_66disconnect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_65disconnect};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_66disconnect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("disconnect", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("disconnect", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_65disconnect(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_65disconnect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("disconnect", 0);
  __pyx_cur_scope = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 1151, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Coroutine_New((__pyx_coroutine_body_t) __pyx_gb_13aioesphomeapi_10connection_13APIConnection_67generator9, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14]), (PyObject *) __pyx_cur_scope, __pyx_mstate_global->__pyx_n_u_disconnect, __pyx_mstate_global->__pyx_n_u_APIConnection_disconnect, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection); if (unlikely(!gen)) __PYX_ERR(0, 1151, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_13aioesphomeapi_10connection_13APIConnection_67generator9(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *__pyx_cur_scope = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_PySendResult __pyx_t_9;
  PyObject *(*__pyx_t_10)(PyObject *);
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("disconnect", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L5_resume_from_await;
    case 2: goto __pyx_L19_resume_from_await;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(__pyx_sent_value != Py_None)) {
    if (unlikely(__pyx_sent_value)) PyErr_SetString(PyExc_TypeError, "can't send non-None value to a just-started coroutine");
    __PYX_ERR(0, 1151, __pyx_L1_error)
  }

  /* "aioesphomeapi/connection.py":1153
 *     async def disconnect(self) -> None:
 *         """Disconnect from the API."""
 *         if self._finish_connect_future is not None:             # <<<<<<<<<<<<<<
 *             # Try to wait for the handshake to finish so we can send
 *             # a disconnect request. If it doesn't finish in time
*/
  __pyx_t_1 = (__pyx_cur_scope->__pyx_v_self->_finish_connect_future != Py_None);
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":1157
 *             # a disconnect request. If it doesn't finish in time
 *             # we will just close the socket.
 *             _, pending = await asyncio.wait(             # <<<<<<<<<<<<<<
 *                 [self._finish_connect_future], timeout=DISCONNECT_CONNECT_TIMEOUT
 *             )
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_asyncio); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_wait); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "aioesphomeapi/connection.py":1158
 *             # we will just close the socket.
 *             _, pending = await asyncio.wait(
 *                 [self._finish_connect_future], timeout=DISCONNECT_CONNECT_TIMEOUT             # <<<<<<<<<<<<<<
 *             )
 *             if pending:
*/
    __pyx_t_4 = PyList_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->_finish_connect_future);
    __Pyx_GIVEREF(__pyx_cur_scope->__pyx_v_self->_finish_connect_future);
    if (__Pyx_PyList_SET_ITEM(__pyx_t_4, 0, __pyx_cur_scope->__pyx_v_self->_finish_connect_future) != (0)) __PYX_ERR(0, 1158, __pyx_L1_error);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_DISCONNECT_CONNECT_TIMEOUT); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1158, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_7 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_3, __pyx_t_4};
      __pyx_t_8 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_timeout, __pyx_t_6, __pyx_t_8, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __pyx_t_2 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_8);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __pyx_t_9 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (likely(__pyx_t_9 == PYGEN_NEXT)) {
      __Pyx_GOTREF(__pyx_r);
      __Pyx_XGIVEREF(__pyx_r);
      __Pyx_RefNannyFinishContext();
      __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
      /* return from generator, awaiting value */
      __pyx_generator->resume_label = 1;
      return __pyx_r;
      __pyx_L5_resume_from_await:;
      if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __pyx_t_2 = __pyx_sent_value; __Pyx_INCREF(__pyx_t_2);
    } else if (likely(__pyx_t_9 == PYGEN_RETURN)) {
      __Pyx_GOTREF(__pyx_r);
      __pyx_t_2 = __pyx_r; __pyx_r = NULL;
    } else {
      __Pyx_XGOTREF(__pyx_r);
      __PYX_ERR(0, 1157, __pyx_L1_error)
    }
    if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 2)) {
        if (size > 2) __Pyx_RaiseTooManyValuesError(2);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 1157, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      if (likely(PyTuple_CheckExact(sequence))) {
        __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0);
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_8 = PyTuple_GET_ITEM(sequence, 1);
        __Pyx_INCREF(__pyx_t_8);
      } else {
        __pyx_t_5 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1157, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_5);
        __pyx_t_8 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1157, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_t_8);
      }
      #else
      __pyx_t_5 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_8 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      Py_ssize_t index = -1;
      __pyx_t_6 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_10 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_6);
      index = 0; __pyx_t_5 = __pyx_t_10(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_5);
      index = 1; __pyx_t_8 = __pyx_t_10(__pyx_t_6); if (unlikely(!__pyx_t_8)) goto __pyx_L6_unpacking_failed;
      __Pyx_GOTREF(__pyx_t_8);
      if (__Pyx_IternextUnpackEndCheck(__pyx_t_10(__pyx_t_6), 2) < (0)) __PYX_ERR(0, 1157, __pyx_L1_error)
      __pyx_t_10 = NULL;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      goto __pyx_L7_unpacking_done;
      __pyx_L6_unpacking_failed:;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = NULL;
      if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
      __PYX_ERR(0, 1157, __pyx_L1_error)
      __pyx_L7_unpacking_done:;
    }

    /* "aioesphomeapi/connection.py":1157
 *             # a disconnect request. If it doesn't finish in time
 *             # we will just close the socket.
 *             _, pending = await asyncio.wait(             # <<<<<<<<<<<<<<
 *                 [self._finish_connect_future], timeout=DISCONNECT_CONNECT_TIMEOUT
 *             )
*/
    __Pyx_GIVEREF(__pyx_t_5);
    __pyx_cur_scope->__pyx_v__ = __pyx_t_5;
    __pyx_t_5 = 0;
    __Pyx_GIVEREF(__pyx_t_8);
    __pyx_cur_scope->__pyx_v_pending = __pyx_t_8;
    __pyx_t_8 = 0;

    /* "aioesphomeapi/connection.py":1160
 *                 [self._finish_connect_future], timeout=DISCONNECT_CONNECT_TIMEOUT
 *             )
 *             if pending:             # <<<<<<<<<<<<<<
 *                 self._set_fatal_exception_if_unset(
 *                     TimeoutAPIError(
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_cur_scope->__pyx_v_pending); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 1160, __pyx_L1_error)
    if (__pyx_t_1) {

      /* "aioesphomeapi/connection.py":1162
 *             if pending:
 *                 self._set_fatal_exception_if_unset(
 *                     TimeoutAPIError(             # <<<<<<<<<<<<<<
 *                         "Timed out waiting to finish connect before disconnecting"
 *                     )
*/
      __pyx_t_8 = NULL;
      __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError);
      __pyx_t_5 = __pyx_v_13aioesphomeapi_10connection_TimeoutAPIError; 
      __pyx_t_7 = 1;
      #if CYTHON_UNPACK_METHODS
      if (unlikely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_8);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_7 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_mstate_global->__pyx_kp_u_Timed_out_waiting_to_finish_conn};
        __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_7, (2-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1162, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      if (!(likely(__Pyx_PyException_Check(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_2))) __PYX_ERR(0, 1162, __pyx_L1_error)

      /* "aioesphomeapi/connection.py":1161
 *             )
 *             if pending:
 *                 self._set_fatal_exception_if_unset(             # <<<<<<<<<<<<<<
 *                     TimeoutAPIError(
 *                         "Timed out waiting to finish connect before disconnecting"
*/
      ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_set_fatal_exception_if_unset(__pyx_cur_scope->__pyx_v_self, ((PyObject*)__pyx_t_2)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1161, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "aioesphomeapi/connection.py":1166
 *                     )
 *                 )
 *                 if self._debug_enabled:             # <<<<<<<<<<<<<<
 *                     _LOGGER.debug(
 *                         "%s: Connect task didn't finish before disconnect",
*/
      if (__pyx_cur_scope->__pyx_v_self->_debug_enabled) {

        /* "aioesphomeapi/connection.py":1167
 *                 )
 *                 if self._debug_enabled:
 *                     _LOGGER.debug(             # <<<<<<<<<<<<<<
 *                         "%s: Connect task didn't finish before disconnect",
 *                         self.log_name,
*/
        __pyx_t_5 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1167, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_debug); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1167, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

        /* "aioesphomeapi/connection.py":1169
 *                     _LOGGER.debug(
 *                         "%s: Connect task didn't finish before disconnect",
 *                         self.log_name,             # <<<<<<<<<<<<<<
 *                     )
 * 
*/
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_6))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_6);
          assert(__pyx_t_5);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_6, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_5, __pyx_mstate_global->__pyx_kp_u_s_Connect_task_didn_t_finish_be, __pyx_cur_scope->__pyx_v_self->log_name};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_6, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1167, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

        /* "aioesphomeapi/connection.py":1166
 *                     )
 *                 )
 *                 if self._debug_enabled:             # <<<<<<<<<<<<<<
 *                     _LOGGER.debug(
 *                         "%s: Connect task didn't finish before disconnect",
*/
      }

      /* "aioesphomeapi/connection.py":1160
 *                 [self._finish_connect_future], timeout=DISCONNECT_CONNECT_TIMEOUT
 *             )
 *             if pending:             # <<<<<<<<<<<<<<
 *                 self._set_fatal_exception_if_unset(
 *                     TimeoutAPIError(
*/
    }

    /* "aioesphomeapi/connection.py":1153
 *     async def disconnect(self) -> None:
 *         """Disconnect from the API."""
 *         if self._finish_connect_future is not None:             # <<<<<<<<<<<<<<
 *             # Try to wait for the handshake to finish so we can send
 *             # a disconnect request. If it doesn't finish in time
*/
  }

  /* "aioesphomeapi/connection.py":1172
 *                     )
 * 
 *         self._expected_disconnect = True             # <<<<<<<<<<<<<<
 *         if self._handshake_complete and self._frame_helper is not None:
 *             # We still want to send a disconnect request even
*/
  __pyx_cur_scope->__pyx_v_self->_expected_disconnect = 1;

  /* "aioesphomeapi/connection.py":1173
 * 
 *         self._expected_disconnect = True
 *         if self._handshake_complete and self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             # We still want to send a disconnect request even
 *             # if the hello phase isn't finished to ensure we
*/
  if (__pyx_cur_scope->__pyx_v_self->_handshake_complete) {
  } else {
    __pyx_t_1 = __pyx_cur_scope->__pyx_v_self->_handshake_complete;
    goto __pyx_L11_bool_binop_done;
  }
  __pyx_t_11 = (((PyObject *)__pyx_cur_scope->__pyx_v_self->_frame_helper) != Py_None);
  __pyx_t_1 = __pyx_t_11;
  __pyx_L11_bool_binop_done:;
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":1179
 *             # as possible.
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:             # <<<<<<<<<<<<<<
 *                 await self.send_message_await_response(
 *                     DISCONNECT_REQUEST_MESSAGE,
*/
    {
      __Pyx_ExceptionSave(&__pyx_t_12, &__pyx_t_13, &__pyx_t_14);
      __Pyx_XGOTREF(__pyx_t_12);
      __Pyx_XGOTREF(__pyx_t_13);
      __Pyx_XGOTREF(__pyx_t_14);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":1180
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:
 *                 await self.send_message_await_response(             # <<<<<<<<<<<<<<
 *                     DISCONNECT_REQUEST_MESSAGE,
 *                     DisconnectResponse,
*/
        __pyx_t_6 = ((PyObject *)__pyx_cur_scope->__pyx_v_self);
        __Pyx_INCREF(__pyx_t_6);

        /* "aioesphomeapi/connection.py":1182
 *                 await self.send_message_await_response(
 *                     DISCONNECT_REQUEST_MESSAGE,
 *                     DisconnectResponse,             # <<<<<<<<<<<<<<
 *                     timeout=DISCONNECT_RESPONSE_TIMEOUT,
 *                 )
*/
        __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_DisconnectResponse); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1182, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_5);

        /* "aioesphomeapi/connection.py":1183
 *                     DISCONNECT_REQUEST_MESSAGE,
 *                     DisconnectResponse,
 *                     timeout=DISCONNECT_RESPONSE_TIMEOUT,             # <<<<<<<<<<<<<<
 *                 )
 *             except APIConnectionError:
*/
        __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_DISCONNECT_RESPONSE_TIMEOUT); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 1183, __pyx_L13_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_7 = 0;
        {
          PyObject *__pyx_callargs[3 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_6, __pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE, __pyx_t_5};
          __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1180, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_4);
          if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_timeout, __pyx_t_8, __pyx_t_4, __pyx_callargs+3, 0) < (0)) __PYX_ERR(0, 1180, __pyx_L13_error)
          __pyx_t_2 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_send_message_await_response, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1180, __pyx_L13_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_t_9 = __Pyx_Coroutine_Yield_From(__pyx_generator, __pyx_t_2, &__pyx_r);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (likely(__pyx_t_9 == PYGEN_NEXT)) {
          __Pyx_GOTREF(__pyx_r);
          __Pyx_XGIVEREF(__pyx_t_12);
          __pyx_cur_scope->__pyx_t_0 = __pyx_t_12;
          __Pyx_XGIVEREF(__pyx_t_13);
          __pyx_cur_scope->__pyx_t_1 = __pyx_t_13;
          __Pyx_XGIVEREF(__pyx_t_14);
          __pyx_cur_scope->__pyx_t_2 = __pyx_t_14;
          __Pyx_XGIVEREF(__pyx_r);
          __Pyx_RefNannyFinishContext();
          __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
          /* return from generator, awaiting value */
          __pyx_generator->resume_label = 2;
          return __pyx_r;
          __pyx_L19_resume_from_await:;
          __pyx_t_12 = __pyx_cur_scope->__pyx_t_0;
          __pyx_cur_scope->__pyx_t_0 = 0;
          __Pyx_XGOTREF(__pyx_t_12);
          __pyx_t_13 = __pyx_cur_scope->__pyx_t_1;
          __pyx_cur_scope->__pyx_t_1 = 0;
          __Pyx_XGOTREF(__pyx_t_13);
          __pyx_t_14 = __pyx_cur_scope->__pyx_t_2;
          __pyx_cur_scope->__pyx_t_2 = 0;
          __Pyx_XGOTREF(__pyx_t_14);
          if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 1180, __pyx_L13_error)
        } else if (likely(__pyx_t_9 == PYGEN_RETURN)) {
          __Pyx_GOTREF(__pyx_r);
          __Pyx_DECREF(__pyx_r); __pyx_r = 0;
        } else {
          __Pyx_XGOTREF(__pyx_r);
          __PYX_ERR(0, 1180, __pyx_L13_error)
        }

        /* "aioesphomeapi/connection.py":1179
 *             # as possible.
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:             # <<<<<<<<<<<<<<
 *                 await self.send_message_await_response(
 *                     DISCONNECT_REQUEST_MESSAGE,
*/
      }
      __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
      __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
      __Pyx_XDECREF(__pyx_t_14); __pyx_t_14 = 0;
      goto __pyx_L18_try_end;
      __pyx_L13_error:;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

      /* "aioesphomeapi/connection.py":1185
 *                     timeout=DISCONNECT_RESPONSE_TIMEOUT,
 *                 )
 *             except APIConnectionError:             # <<<<<<<<<<<<<<
 *                 _LOGGER.exception("%s: disconnect request failed", self.log_name)
 * 
*/
      __pyx_t_15 = __Pyx_PyErr_ExceptionMatches(__pyx_v_13aioesphomeapi_10connection_APIConnectionError);
      if (__pyx_t_15) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_8) < 0) __PYX_ERR(0, 1185, __pyx_L15_except_error)
        __Pyx_XGOTREF(__pyx_t_2);
        __Pyx_XGOTREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_t_8);

        /* "aioesphomeapi/connection.py":1186
 *                 )
 *             except APIConnectionError:
 *                 _LOGGER.exception("%s: disconnect request failed", self.log_name)             # <<<<<<<<<<<<<<
 * 
 *         self._cleanup()
*/
        __pyx_t_6 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1186, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_16 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_exception); if (unlikely(!__pyx_t_16)) __PYX_ERR(0, 1186, __pyx_L15_except_error)
        __Pyx_GOTREF(__pyx_t_16);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_7 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_16))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_16);
          assert(__pyx_t_6);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_16);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_16, __pyx__function);
          __pyx_t_7 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_s_disconnect_request_failed, __pyx_cur_scope->__pyx_v_self->log_name};
          __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_16, __pyx_callargs+__pyx_t_7, (3-__pyx_t_7) | (__pyx_t_7*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_DECREF(__pyx_t_16); __pyx_t_16 = 0;
          if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1186, __pyx_L15_except_error)
          __Pyx_GOTREF(__pyx_t_5);
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L14_exception_handled;
      }
      goto __pyx_L15_except_error;

      /* "aioesphomeapi/connection.py":1179
 *             # as possible.
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:             # <<<<<<<<<<<<<<
 *                 await self.send_message_await_response(
 *                     DISCONNECT_REQUEST_MESSAGE,
*/
      __pyx_L15_except_error:;
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      goto __pyx_L1_error;
      __pyx_L14_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_12);
      __Pyx_XGIVEREF(__pyx_t_13);
      __Pyx_XGIVEREF(__pyx_t_14);
      __Pyx_ExceptionReset(__pyx_t_12, __pyx_t_13, __pyx_t_14);
      __pyx_L18_try_end:;
    }

    /* "aioesphomeapi/connection.py":1173
 * 
 *         self._expected_disconnect = True
 *         if self._handshake_complete and self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             # We still want to send a disconnect request even
 *             # if the hello phase isn't finished to ensure we
*/
  }

  /* "aioesphomeapi/connection.py":1188
 *                 _LOGGER.exception("%s: disconnect request failed", self.log_name)
 * 
 *         self._cleanup()             # <<<<<<<<<<<<<<
 * 
 *     def force_disconnect(self) -> None:
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_cur_scope->__pyx_v_self->__pyx_vtab)->_cleanup(__pyx_cur_scope->__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1188, __pyx_L1_error)
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "aioesphomeapi/connection.py":1151
 *         self.send_messages((resp,))
 * 
 *     async def disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Disconnect from the API."""
 *         if self._finish_connect_future is not None:
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_16);
  if (__Pyx_PyErr_Occurred()) {
    __Pyx_Generator_Replace_StopIteration(0);
    __Pyx_AddTraceback("disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  }
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.py":1190
 *         self._cleanup()
 * 
 *     def force_disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Forcefully disconnect from the API."""
 *         self._expected_disconnect = True
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_69force_disconnect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_13aioesphomeapi_10connection_13APIConnection_68force_disconnect, "Forcefully disconnect from the API.");
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_69force_disconnect = {"force_disconnect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_69force_disconnect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_68force_disconnect};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_69force_disconnect(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("force_disconnect (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("force_disconnect", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("force_disconnect", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_68force_disconnect(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_68force_disconnect(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  size_t __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("force_disconnect", 0);

  /* "aioesphomeapi/connection.py":1192
 *     def force_disconnect(self) -> None:
 *         """Forcefully disconnect from the API."""
 *         self._expected_disconnect = True             # <<<<<<<<<<<<<<
 *         if self._handshake_complete and self._frame_helper is not None:
 *             # Still try to tell the esp to disconnect gracefully
*/
  __pyx_v_self->_expected_disconnect = 1;

  /* "aioesphomeapi/connection.py":1193
 *         """Forcefully disconnect from the API."""
 *         self._expected_disconnect = True
 *         if self._handshake_complete and self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             # Still try to tell the esp to disconnect gracefully
 *             # but don't wait for it to finish
*/
  if (__pyx_v_self->_handshake_complete) {
  } else {
    __pyx_t_1 = __pyx_v_self->_handshake_complete;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (((PyObject *)__pyx_v_self->_frame_helper) != Py_None);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "aioesphomeapi/connection.py":1197
 *             # but don't wait for it to finish
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:             # <<<<<<<<<<<<<<
 *                 self.send_messages((DISCONNECT_REQUEST_MESSAGE,))
 *             except APIConnectionError:
*/
    {
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
      __Pyx_XGOTREF(__pyx_t_3);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_5);
      /*try:*/ {

        /* "aioesphomeapi/connection.py":1198
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:
 *                 self.send_messages((DISCONNECT_REQUEST_MESSAGE,))             # <<<<<<<<<<<<<<
 *             except APIConnectionError:
 *                 _LOGGER.exception(
*/
        __pyx_t_6 = PyTuple_New(1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1198, __pyx_L6_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE);
        __Pyx_GIVEREF(__pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE);
        if (__Pyx_PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE) != (0)) __PYX_ERR(0, 1198, __pyx_L6_error);
        ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->send_messages(__pyx_v_self, ((PyObject*)__pyx_t_6), 0); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1198, __pyx_L6_error)
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "aioesphomeapi/connection.py":1197
 *             # but don't wait for it to finish
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:             # <<<<<<<<<<<<<<
 *                 self.send_messages((DISCONNECT_REQUEST_MESSAGE,))
 *             except APIConnectionError:
*/
      }
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L11_try_end;
      __pyx_L6_error:;
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

      /* "aioesphomeapi/connection.py":1199
 *             try:
 *                 self.send_messages((DISCONNECT_REQUEST_MESSAGE,))
 *             except APIConnectionError:             # <<<<<<<<<<<<<<
 *                 _LOGGER.exception(
 *                     "%s: Failed to send (forced) disconnect request",
*/
      __pyx_t_7 = __Pyx_PyErr_ExceptionMatches(__pyx_v_13aioesphomeapi_10connection_APIConnectionError);
      if (__pyx_t_7) {
        __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.force_disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
        if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_8, &__pyx_t_9) < 0) __PYX_ERR(0, 1199, __pyx_L8_except_error)
        __Pyx_XGOTREF(__pyx_t_6);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);

        /* "aioesphomeapi/connection.py":1200
 *                 self.send_messages((DISCONNECT_REQUEST_MESSAGE,))
 *             except APIConnectionError:
 *                 _LOGGER.exception(             # <<<<<<<<<<<<<<
 *                     "%s: Failed to send (forced) disconnect request",
 *                     self.log_name,
*/
        __pyx_t_11 = NULL;
        __Pyx_GetModuleGlobalName(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_LOGGER); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 1200, __pyx_L8_except_error)
        __Pyx_GOTREF(__pyx_t_12);
        __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_12, __pyx_mstate_global->__pyx_n_u_exception); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1200, __pyx_L8_except_error)
        __Pyx_GOTREF(__pyx_t_13);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

        /* "aioesphomeapi/connection.py":1202
 *                 _LOGGER.exception(
 *                     "%s: Failed to send (forced) disconnect request",
 *                     self.log_name,             # <<<<<<<<<<<<<<
 *                 )
 * 
*/
        __pyx_t_14 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_13))) {
          __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_13);
          assert(__pyx_t_11);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_13);
          __Pyx_INCREF(__pyx_t_11);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_13, __pyx__function);
          __pyx_t_14 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[3] = {__pyx_t_11, __pyx_mstate_global->__pyx_kp_u_s_Failed_to_send_forced_disconn, __pyx_v_self->log_name};
          __pyx_t_10 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_14, (3-__pyx_t_14) | (__pyx_t_14*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 1200, __pyx_L8_except_error)
          __Pyx_GOTREF(__pyx_t_10);
        }
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_exception_handled;
      }
      goto __pyx_L8_except_error;

      /* "aioesphomeapi/connection.py":1197
 *             # but don't wait for it to finish
 *             # But skip if frame_helper is None (connection already cleaned up)
 *             try:             # <<<<<<<<<<<<<<
 *                 self.send_messages((DISCONNECT_REQUEST_MESSAGE,))
 *             except APIConnectionError:
*/
      __pyx_L8_except_error:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      goto __pyx_L1_error;
      __pyx_L7_exception_handled:;
      __Pyx_XGIVEREF(__pyx_t_3);
      __Pyx_XGIVEREF(__pyx_t_4);
      __Pyx_XGIVEREF(__pyx_t_5);
      __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
      __pyx_L11_try_end:;
    }

    /* "aioesphomeapi/connection.py":1193
 *         """Forcefully disconnect from the API."""
 *         self._expected_disconnect = True
 *         if self._handshake_complete and self._frame_helper is not None:             # <<<<<<<<<<<<<<
 *             # Still try to tell the esp to disconnect gracefully
 *             # but don't wait for it to finish
*/
  }

  /* "aioesphomeapi/connection.py":1205
 *                 )
 * 
 *         self._cleanup()             # <<<<<<<<<<<<<<
*/
  ((struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self->__pyx_vtab)->_cleanup(__pyx_v_self); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1205, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":1190
 *         self._cleanup()
 * 
 *     def force_disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Forcefully disconnect from the API."""
 *         self._expected_disconnect = True
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.force_disconnect", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":96
 * 
 *     cdef ConnectionParams _params
 *     cdef public object on_stop             # <<<<<<<<<<<<<<
 *     cdef public object _socket
 *     cdef public APIFrameHelper _frame_helper
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->on_stop);
  __pyx_r = __pyx_v_self->on_stop;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->on_stop);
  __Pyx_DECREF(__pyx_v_self->on_stop);
  __pyx_v_self->on_stop = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7on_stop_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->on_stop);
  __Pyx_DECREF(__pyx_v_self->on_stop);
  __pyx_v_self->on_stop = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":97
 *     cdef ConnectionParams _params
 *     cdef public object on_stop
 *     cdef public object _socket             # <<<<<<<<<<<<<<
 *     cdef public APIFrameHelper _frame_helper
 *     cdef public object api_version
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_socket);
  __pyx_r = __pyx_v_self->_socket;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_socket);
  __Pyx_DECREF(__pyx_v_self->_socket);
  __pyx_v_self->_socket = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_7_socket_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_socket);
  __Pyx_DECREF(__pyx_v_self->_socket);
  __pyx_v_self->_socket = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":98
 *     cdef public object on_stop
 *     cdef public object _socket
 *     cdef public APIFrameHelper _frame_helper             # <<<<<<<<<<<<<<
 *     cdef public object api_version
 *     cdef public object connection_state
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->_frame_helper);
  __pyx_r = ((PyObject *)__pyx_v_self->_frame_helper);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper))))) __PYX_ERR(2, 98, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_frame_helper);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_frame_helper);
  __pyx_v_self->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._frame_helper.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->_frame_helper);
  __Pyx_DECREF((PyObject *)__pyx_v_self->_frame_helper);
  __pyx_v_self->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":99
 *     cdef public object _socket
 *     cdef public APIFrameHelper _frame_helper
 *     cdef public object api_version             # <<<<<<<<<<<<<<
 *     cdef public object connection_state
 *     cdef public dict _message_handlers
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->api_version);
  __pyx_r = __pyx_v_self->api_version;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->api_version);
  __Pyx_DECREF(__pyx_v_self->api_version);
  __pyx_v_self->api_version = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_11api_version_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->api_version);
  __Pyx_DECREF(__pyx_v_self->api_version);
  __pyx_v_self->api_version = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":100
 *     cdef public APIFrameHelper _frame_helper
 *     cdef public object api_version
 *     cdef public object connection_state             # <<<<<<<<<<<<<<
 *     cdef public dict _message_handlers
 *     cdef public str log_name
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->connection_state);
  __pyx_r = __pyx_v_self->connection_state;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->connection_state);
  __Pyx_DECREF(__pyx_v_self->connection_state);
  __pyx_v_self->connection_state = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16connection_state_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->connection_state);
  __Pyx_DECREF(__pyx_v_self->connection_state);
  __pyx_v_self->connection_state = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":101
 *     cdef public object api_version
 *     cdef public object connection_state
 *     cdef public dict _message_handlers             # <<<<<<<<<<<<<<
 *     cdef public str log_name
 *     cdef set _read_exception_futures
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_message_handlers);
  __pyx_r = __pyx_v_self->_message_handlers;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(2, 101, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_message_handlers);
  __Pyx_DECREF(__pyx_v_self->_message_handlers);
  __pyx_v_self->_message_handlers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._message_handlers.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_message_handlers);
  __Pyx_DECREF(__pyx_v_self->_message_handlers);
  __pyx_v_self->_message_handlers = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":102
 *     cdef public object connection_state
 *     cdef public dict _message_handlers
 *     cdef public str log_name             # <<<<<<<<<<<<<<
 *     cdef set _read_exception_futures
 *     cdef object _ping_timer
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->log_name);
  __pyx_r = __pyx_v_self->log_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(2, 102, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->log_name);
  __Pyx_DECREF(__pyx_v_self->log_name);
  __pyx_v_self->log_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.log_name.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_8log_name_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->log_name);
  __Pyx_DECREF(__pyx_v_self->log_name);
  __pyx_v_self->log_name = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":111
 *     cdef object _start_connect_future
 *     cdef object _finish_connect_future
 *     cdef public Exception _fatal_exception             # <<<<<<<<<<<<<<
 *     cdef bint _expected_disconnect
 *     cdef object _loop
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_fatal_exception);
  __pyx_r = __pyx_v_self->_fatal_exception;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(__Pyx_PyException_Check(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_1))) __PYX_ERR(2, 111, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_fatal_exception);
  __Pyx_DECREF(__pyx_v_self->_fatal_exception);
  __pyx_v_self->_fatal_exception = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._fatal_exception.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_fatal_exception);
  __Pyx_DECREF(__pyx_v_self->_fatal_exception);
  __pyx_v_self->_fatal_exception = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":115
 *     cdef object _loop
 *     cdef bint _send_pending_ping
 *     cdef public bint is_connected             # <<<<<<<<<<<<<<
 *     cdef public bint _handshake_complete
 *     cdef bint _debug_enabled
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_12is_connected_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_12is_connected_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_12is_connected___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_12is_connected___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->is_connected); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.is_connected.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_12is_connected_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_12is_connected_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_12is_connected_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_12is_connected_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 115, __pyx_L1_error)
  __pyx_v_self->is_connected = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.is_connected.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":116
 *     cdef bint _send_pending_ping
 *     cdef public bint is_connected
 *     cdef public bint _handshake_complete             # <<<<<<<<<<<<<<
 *     cdef bint _debug_enabled
 *     cdef public str received_name
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->_handshake_complete); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handshake_complete.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 116, __pyx_L1_error)
  __pyx_v_self->_handshake_complete = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection._handshake_complete.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":118
 *     cdef public bint _handshake_complete
 *     cdef bint _debug_enabled
 *     cdef public str received_name             # <<<<<<<<<<<<<<
 *     cdef public str connected_address
 *     cdef str _cached_timezone
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->received_name);
  __pyx_r = __pyx_v_self->received_name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(2, 118, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->received_name);
  __Pyx_DECREF(__pyx_v_self->received_name);
  __pyx_v_self->received_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.received_name.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_13received_name_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->received_name);
  __Pyx_DECREF(__pyx_v_self->received_name);
  __pyx_v_self->received_name = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "aioesphomeapi/connection.pxd":119
 *     cdef bint _debug_enabled
 *     cdef public str received_name
 *     cdef public str connected_address             # <<<<<<<<<<<<<<
 *     cdef str _cached_timezone
 *     cdef list _addrs_info
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address___get__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address___get__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->connected_address);
  __pyx_r = __pyx_v_self->connected_address;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address_2__set__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address_2__set__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(2, 119, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->connected_address);
  __Pyx_DECREF(__pyx_v_self->connected_address);
  __pyx_v_self->connected_address = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.connected_address.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_5__del__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address_4__del__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_13aioesphomeapi_10connection_13APIConnection_17connected_address_4__del__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->connected_address);
  __Pyx_DECREF(__pyx_v_self->connected_address);
  __pyx_v_self->connected_address = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_71__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_71__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_71__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_71__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_70__reduce_cython__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_70__reduce_cython__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._addrs_info, self._cached_timezone, self._debug_enabled, self._expected_disconnect, self._fatal_exception, self._finish_connect_future, self._frame_helper, self._handshake_complete, self._keep_alive_interval, self._keep_alive_timeout, self._log_errors, self._loop, self._message_handlers, self._params, self._ping_timer, self._pong_timer, self._read_exception_futures, self._resolve_host_future, self._send_pending_ping, self._socket, self._start_connect_future, self.api_version, self.connected_address, self.connection_state, self.is_connected, self.log_name, self.on_stop, self.received_name)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->_debug_enabled); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->_expected_disconnect); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->_handshake_complete); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_interval); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->_keep_alive_timeout); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_6 = __Pyx_PyBool_FromLong(__pyx_v_self->_log_errors); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_PyBool_FromLong(__pyx_v_self->_send_pending_ping); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyBool_FromLong(__pyx_v_self->is_connected); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = PyTuple_New(28); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __Pyx_INCREF(__pyx_v_self->_addrs_info);
  __Pyx_GIVEREF(__pyx_v_self->_addrs_info);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v_self->_addrs_info) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_cached_timezone);
  __Pyx_GIVEREF(__pyx_v_self->_cached_timezone);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_v_self->_cached_timezone) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_t_1) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 3, __pyx_t_2) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_fatal_exception);
  __Pyx_GIVEREF(__pyx_v_self->_fatal_exception);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 4, __pyx_v_self->_fatal_exception) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_finish_connect_future);
  __Pyx_GIVEREF(__pyx_v_self->_finish_connect_future);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 5, __pyx_v_self->_finish_connect_future) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->_frame_helper);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->_frame_helper);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 6, ((PyObject *)__pyx_v_self->_frame_helper)) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_3);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 7, __pyx_t_3) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_4);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 8, __pyx_t_4) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_5);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 9, __pyx_t_5) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_6);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 10, __pyx_t_6) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_loop);
  __Pyx_GIVEREF(__pyx_v_self->_loop);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 11, __pyx_v_self->_loop) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_message_handlers);
  __Pyx_GIVEREF(__pyx_v_self->_message_handlers);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 12, __pyx_v_self->_message_handlers) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF((PyObject *)__pyx_v_self->_params);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->_params);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 13, ((PyObject *)__pyx_v_self->_params)) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_ping_timer);
  __Pyx_GIVEREF(__pyx_v_self->_ping_timer);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 14, __pyx_v_self->_ping_timer) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_pong_timer);
  __Pyx_GIVEREF(__pyx_v_self->_pong_timer);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 15, __pyx_v_self->_pong_timer) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_read_exception_futures);
  __Pyx_GIVEREF(__pyx_v_self->_read_exception_futures);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 16, __pyx_v_self->_read_exception_futures) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_resolve_host_future);
  __Pyx_GIVEREF(__pyx_v_self->_resolve_host_future);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 17, __pyx_v_self->_resolve_host_future) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_7);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 18, __pyx_t_7) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_socket);
  __Pyx_GIVEREF(__pyx_v_self->_socket);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 19, __pyx_v_self->_socket) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->_start_connect_future);
  __Pyx_GIVEREF(__pyx_v_self->_start_connect_future);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 20, __pyx_v_self->_start_connect_future) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->api_version);
  __Pyx_GIVEREF(__pyx_v_self->api_version);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 21, __pyx_v_self->api_version) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->connected_address);
  __Pyx_GIVEREF(__pyx_v_self->connected_address);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 22, __pyx_v_self->connected_address) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->connection_state);
  __Pyx_GIVEREF(__pyx_v_self->connection_state);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 23, __pyx_v_self->connection_state) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_8);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 24, __pyx_t_8) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->log_name);
  __Pyx_GIVEREF(__pyx_v_self->log_name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 25, __pyx_v_self->log_name) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->on_stop);
  __Pyx_GIVEREF(__pyx_v_self->on_stop);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 26, __pyx_v_self->on_stop) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->received_name);
  __Pyx_GIVEREF(__pyx_v_self->received_name);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 27, __pyx_v_self->received_name) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_t_5 = 0;
  __pyx_t_6 = 0;
  __pyx_t_7 = 0;
  __pyx_t_8 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_9);
  __pyx_t_9 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._addrs_info, self._cached_timezone, self._debug_enabled, self._expected_disconnect, self._fatal_exception, self._finish_connect_future, self._frame_helper, self._handshake_complete, self._keep_alive_interval, self._keep_alive_timeout, self._log_errors, self._loop, self._message_handlers, self._params, self._ping_timer, self._pong_timer, self._read_exception_futures, self._resolve_host_future, self._send_pending_ping, self._socket, self._start_connect_future, self.api_version, self.connected_address, self.connection_state, self.is_connected, self.log_name, self.on_stop, self.received_name)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_9 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_9);
  __pyx_v__dict = __pyx_t_9;
  __pyx_t_9 = 0;

  /* "(tree fragment)":7
 *     state = (self._addrs_info, self._cached_timezone, self._debug_enabled, self._expected_disconnect, self._fatal_exception, self._finish_connect_future, self._frame_helper, self._handshake_complete, self._keep_alive_interval, self._keep_alive_timeout, self._log_errors, self._loop, self._message_handlers, self._params, self._ping_timer, self._pong_timer, self._read_exception_futures, self._resolve_host_future, self._send_pending_ping, self._socket, self._start_connect_future, self.api_version, self.connected_address, self.connection_state, self.is_connected, self.log_name, self.on_stop, self.received_name)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_11 = (__pyx_v__dict != Py_None);
  if (__pyx_t_11) {
  } else {
    __pyx_t_10 = __pyx_t_11;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_11 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_11 < 0))) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_10 = __pyx_t_11;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_10) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_9 = PyTuple_New(1); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_8 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_8));
    __pyx_t_8 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._addrs_info is not None or self._cached_timezone is not None or self._fatal_exception is not None or self._finish_connect_future is not None or self._frame_helper is not None or self._loop is not None or self._message_handlers is not None or self._params is not None or self._ping_timer is not None or self._pong_timer is not None or self._read_exception_futures is not None or self._resolve_host_future is not None or self._socket is not None or self._start_connect_future is not None or self.api_version is not None or self.connected_address is not None or self.connection_state is not None or self.log_name is not None or self.on_stop is not None or self.received_name is not None
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._addrs_info, self._cached_timezone, self._debug_enabled, self._expected_disconnect, self._fatal_exception, self._finish_connect_future, self._frame_helper, self._handshake_complete, self._keep_alive_interval, self._keep_alive_timeout, self._log_errors, self._loop, self._message_handlers, self._params, self._ping_timer, self._pong_timer, self._read_exception_futures, self._resolve_host_future, self._send_pending_ping, self._socket, self._start_connect_future, self.api_version, self.connected_address, self.connection_state, self.is_connected, self.log_name, self.on_stop, self.received_name)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._addrs_info is not None or self._cached_timezone is not None or self._fatal_exception is not None or self._finish_connect_future is not None or self._frame_helper is not None or self._loop is not None or self._message_handlers is not None or self._params is not None or self._ping_timer is not None or self._pong_timer is not None or self._read_exception_futures is not None or self._resolve_host_future is not None or self._socket is not None or self._start_connect_future is not None or self.api_version is not None or self.connected_address is not None or self.connection_state is not None or self.log_name is not None or self.on_stop is not None or self.received_name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, None), state
*/
  /*else*/ {
    __pyx_t_11 = (__pyx_v_self->_addrs_info != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_cached_timezone != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_fatal_exception != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_finish_connect_future != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (((PyObject *)__pyx_v_self->_frame_helper) != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_loop != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_message_handlers != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (((PyObject *)__pyx_v_self->_params) != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_ping_timer != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_pong_timer != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_read_exception_futures != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_resolve_host_future != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_socket != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->_start_connect_future != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->api_version != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->connected_address != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->connection_state != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->log_name != ((PyObject*)Py_None));
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->on_stop != Py_None);
    if (!__pyx_t_11) {
    } else {
      __pyx_t_10 = __pyx_t_11;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_11 = (__pyx_v_self->received_name != ((PyObject*)Py_None));
    __pyx_t_10 = __pyx_t_11;
    __pyx_L6_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_10;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._addrs_info is not None or self._cached_timezone is not None or self._fatal_exception is not None or self._finish_connect_future is not None or self._frame_helper is not None or self._loop is not None or self._message_handlers is not None or self._params is not None or self._ping_timer is not None or self._pong_timer is not None or self._read_exception_futures is not None or self._resolve_host_future is not None or self._socket is not None or self._start_connect_future is not None or self.api_version is not None or self.connected_address is not None or self.connection_state is not None or self.log_name is not None or self.on_stop is not None or self.received_name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = self._addrs_info is not None or self._cached_timezone is not None or self._fatal_exception is not None or self._finish_connect_future is not None or self._frame_helper is not None or self._loop is not None or self._message_handlers is not None or self._params is not None or self._ping_timer is not None or self._pong_timer is not None or self._read_exception_futures is not None or self._resolve_host_future is not None or self._socket is not None or self._start_connect_future is not None or self.api_version is not None or self.connected_address is not None or self.connection_state is not None or self.log_name is not None or self.on_stop is not None or self.received_name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_APIConnection); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_180946995);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_180946995);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_mstate_global->__pyx_int_180946995) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_7 = PyTuple_New(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_8);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_8) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_9) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_8 = 0;
    __pyx_t_9 = 0;
    __pyx_r = __pyx_t_7;
    __pyx_t_7 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._addrs_info is not None or self._cached_timezone is not None or self._fatal_exception is not None or self._finish_connect_future is not None or self._frame_helper is not None or self._loop is not None or self._message_handlers is not None or self._params is not None or self._ping_timer is not None or self._pong_timer is not None or self._read_exception_futures is not None or self._resolve_host_future is not None or self._socket is not None or self._start_connect_future is not None or self.api_version is not None or self.connected_address is not None or self.connection_state is not None or self.log_name is not None or self.on_stop is not None or self.received_name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, None), state
 *     else:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_APIConnection__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_APIConnection); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = PyTuple_New(3); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_180946995);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_180946995);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 1, __pyx_mstate_global->__pyx_int_180946995) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_9, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_7);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_9);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_9) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_7 = 0;
    __pyx_t_9 = 0;
    __pyx_r = __pyx_t_8;
    __pyx_t_8 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_APIConnection__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_73__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_13APIConnection_73__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_73__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_13APIConnection_73__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13APIConnection_72__setstate_cython__(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13APIConnection_72__setstate_cython__(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_APIConnection__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(1, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(1, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_13aioesphomeapi_10connection___pyx_unpickle_APIConnection__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_APIConnection__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("aioesphomeapi.connection.APIConnection.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_ConnectionParams(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_14__pyx_unpickle_ConnectionParams(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_14__pyx_unpickle_ConnectionParams = {"__pyx_unpickle_ConnectionParams", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_14__pyx_unpickle_ConnectionParams, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_14__pyx_unpickle_ConnectionParams(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ConnectionParams (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_ConnectionParams", 0) < (0)) __PYX_ERR(1, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ConnectionParams", 1, 3, 3, i); __PYX_ERR(1, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ConnectionParams", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.__pyx_unpickle_ConnectionParams", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(1, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_13__pyx_unpickle_ConnectionParams(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_13__pyx_unpickle_ConnectionParams(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ConnectionParams", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_ConnectionParams(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')             # <<<<<<<<<<<<<<
 *     __pyx_result = ConnectionParams.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, __pyx_k_addresses_client_info_expected_m); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')
 *     __pyx_result = ConnectionParams.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')
 *     __pyx_result = ConnectionParams.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = ConnectionParams.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ConnectionParams__set_state(ConnectionParams __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(1, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_13aioesphomeapi_10connection___pyx_unpickle_ConnectionParams__set_state(((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')
 *     __pyx_result = ConnectionParams.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ConnectionParams__set_state(ConnectionParams __pyx_result, __pyx_state: tuple):
 *     __pyx_result.addresses = __pyx_state[0]; __pyx_result.client_info = __pyx_state[1]; __pyx_result.expected_mac = __pyx_state[2]; __pyx_result.expected_name = __pyx_state[3]; __pyx_result.keepalive = __pyx_state[4]; __pyx_result.noise_psk = __pyx_state[5]; __pyx_result.password = __pyx_state[6]; __pyx_result.port = __pyx_state[7]; __pyx_result.timezone = __pyx_state[8]; __pyx_result.zeroconf_manager = __pyx_state[9]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_ConnectionParams(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aioesphomeapi.connection.__pyx_unpickle_ConnectionParams", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ConnectionParams__set_state(ConnectionParams __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.addresses = __pyx_state[0]; __pyx_result.client_info = __pyx_state[1]; __pyx_result.expected_mac = __pyx_state[2]; __pyx_result.expected_name = __pyx_state[3]; __pyx_result.keepalive = __pyx_state[4]; __pyx_result.noise_psk = __pyx_state[5]; __pyx_result.password = __pyx_state[6]; __pyx_result.port = __pyx_state[7]; __pyx_result.timezone = __pyx_state[8]; __pyx_result.zeroconf_manager = __pyx_state[9]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 10)
*/

static PyObject *__pyx_f_13aioesphomeapi_10connection___pyx_unpickle_ConnectionParams__set_state(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ConnectionParams__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ConnectionParams__set_state(ConnectionParams __pyx_result, __pyx_state: tuple):
 *     __pyx_result.addresses = __pyx_state[0]; __pyx_result.client_info = __pyx_state[1]; __pyx_result.expected_mac = __pyx_state[2]; __pyx_result.expected_name = __pyx_state[3]; __pyx_result.keepalive = __pyx_state[4]; __pyx_result.noise_psk = __pyx_state[5]; __pyx_result.password = __pyx_state[6]; __pyx_result.port = __pyx_state[7]; __pyx_result.timezone = __pyx_state[8]; __pyx_result.zeroconf_manager = __pyx_state[9]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 10)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->addresses);
  __Pyx_DECREF(__pyx_v___pyx_result->addresses);
  __pyx_v___pyx_result->addresses = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->client_info);
  __Pyx_DECREF(__pyx_v___pyx_result->client_info);
  __pyx_v___pyx_result->client_info = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->expected_mac);
  __Pyx_DECREF(__pyx_v___pyx_result->expected_mac);
  __pyx_v___pyx_result->expected_mac = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->expected_name);
  __Pyx_DECREF(__pyx_v___pyx_result->expected_name);
  __pyx_v___pyx_result->expected_name = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->keepalive);
  __Pyx_DECREF(__pyx_v___pyx_result->keepalive);
  __pyx_v___pyx_result->keepalive = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->noise_psk);
  __Pyx_DECREF(__pyx_v___pyx_result->noise_psk);
  __pyx_v___pyx_result->noise_psk = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->password);
  __Pyx_DECREF(__pyx_v___pyx_result->password);
  __pyx_v___pyx_result->password = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->port);
  __Pyx_DECREF(__pyx_v___pyx_result->port);
  __pyx_v___pyx_result->port = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->timezone);
  __Pyx_DECREF(__pyx_v___pyx_result->timezone);
  __pyx_v___pyx_result->timezone = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->zeroconf_manager);
  __Pyx_DECREF(__pyx_v___pyx_result->zeroconf_manager);
  __pyx_v___pyx_result->zeroconf_manager = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ConnectionParams__set_state(ConnectionParams __pyx_result, __pyx_state: tuple):
 *     __pyx_result.addresses = __pyx_state[0]; __pyx_result.client_info = __pyx_state[1]; __pyx_result.expected_mac = __pyx_state[2]; __pyx_result.expected_name = __pyx_state[3]; __pyx_result.keepalive = __pyx_state[4]; __pyx_result.noise_psk = __pyx_state[5]; __pyx_result.password = __pyx_state[6]; __pyx_result.port = __pyx_state[7]; __pyx_result.timezone = __pyx_state[8]; __pyx_result.zeroconf_manager = __pyx_state[9]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 10)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 10); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_ConnectionParams__set_state(<ConnectionParams> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ConnectionParams__set_state(ConnectionParams __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.addresses = __pyx_state[0]; __pyx_result.client_info = __pyx_state[1]; __pyx_result.expected_mac = __pyx_state[2]; __pyx_result.expected_name = __pyx_state[3]; __pyx_result.keepalive = __pyx_state[4]; __pyx_result.noise_psk = __pyx_state[5]; __pyx_result.password = __pyx_state[6]; __pyx_result.port = __pyx_state[7]; __pyx_result.timezone = __pyx_state[8]; __pyx_result.zeroconf_manager = __pyx_state[9]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 10)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.__pyx_unpickle_ConnectionParams__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_APIConnection(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, b'_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name')
*/

/* Python wrapper */
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16__pyx_unpickle_APIConnection(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_13aioesphomeapi_10connection_16__pyx_unpickle_APIConnection = {"__pyx_unpickle_APIConnection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_16__pyx_unpickle_APIConnection, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_13aioesphomeapi_10connection_16__pyx_unpickle_APIConnection(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_APIConnection (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_APIConnection", 0) < (0)) __PYX_ERR(1, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_APIConnection", 1, 3, 3, i); __PYX_ERR(1, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_APIConnection", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("aioesphomeapi.connection.__pyx_unpickle_APIConnection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(1, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_13aioesphomeapi_10connection_15__pyx_unpickle_APIConnection(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_13aioesphomeapi_10connection_15__pyx_unpickle_APIConnection(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_APIConnection", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_APIConnection(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, b'_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name')             # <<<<<<<<<<<<<<
 *     __pyx_result = APIConnection.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, __pyx_k_addrs_info__cached_timezone__de); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, b'_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name')
 *     __pyx_result = APIConnection.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, b'_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name')
 *     __pyx_result = APIConnection.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = APIConnection.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_APIConnection__set_state(APIConnection __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(1, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_13aioesphomeapi_10connection___pyx_unpickle_APIConnection__set_state(((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, b'_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name')
 *     __pyx_result = APIConnection.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_APIConnection__set_state(APIConnection __pyx_result, __pyx_state: tuple):
 *     __pyx_result._addrs_info = __pyx_state[0]; __pyx_result._cached_timezone = __pyx_state[1]; __pyx_result._debug_enabled = __pyx_state[2]; __pyx_result._expected_disconnect = __pyx_state[3]; __pyx_result._fatal_exception = __pyx_state[4]; __pyx_result._finish_connect_future = __pyx_state[5]; __pyx_result._frame_helper = __pyx_state[6]; __pyx_result._handshake_complete = __pyx_state[7]; __pyx_result._keep_alive_interval = __pyx_state[8]; __pyx_result._keep_alive_timeout = __pyx_state[9]; __pyx_result._log_errors = __pyx_state[10]; __pyx_result._loop = __pyx_state[11]; __pyx_result._message_handlers = __pyx_state[12]; __pyx_result._params = __pyx_state[13]; __pyx_result._ping_timer = __pyx_state[14]; __pyx_result._pong_timer = __pyx_state[15]; __pyx_result._read_exception_futures = __pyx_state[16]; __pyx_result._resolve_host_future = __pyx_state[17]; __pyx_result._send_pending_ping = __pyx_state[18]; __pyx_result._socket = __pyx_state[19]; __pyx_result._start_connect_future = __pyx_state[20]; __pyx_result.api_version = __pyx_state[21]; __pyx_result.connected_address = __pyx_state[22]; __pyx_result.connection_state = __pyx_state[23]; __pyx_result.is_connected = __pyx_state[24]; __pyx_result.log_name = __pyx_state[25]; __pyx_result.on_stop = __pyx_state[26]; __pyx_result.received_name = __pyx_state[27]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_APIConnection(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xac90833, 0x161e17f, 0xb8c235d, b'_addrs_info, _cached_timezone, _debug_enabled, _expected_disconnect, _fatal_exception, _finish_connect_future, _frame_helper, _handshake_complete, _keep_alive_interval, _keep_alive_timeout, _log_errors, _loop, _message_handlers, _params, _ping_timer, _pong_timer, _read_exception_futures, _resolve_host_future, _send_pending_ping, _socket, _start_connect_future, api_version, connected_address, connection_state, is_connected, log_name, on_stop, received_name')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("aioesphomeapi.connection.__pyx_unpickle_APIConnection", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_APIConnection__set_state(APIConnection __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._addrs_info = __pyx_state[0]; __pyx_result._cached_timezone = __pyx_state[1]; __pyx_result._debug_enabled = __pyx_state[2]; __pyx_result._expected_disconnect = __pyx_state[3]; __pyx_result._fatal_exception = __pyx_state[4]; __pyx_result._finish_connect_future = __pyx_state[5]; __pyx_result._frame_helper = __pyx_state[6]; __pyx_result._handshake_complete = __pyx_state[7]; __pyx_result._keep_alive_interval = __pyx_state[8]; __pyx_result._keep_alive_timeout = __pyx_state[9]; __pyx_result._log_errors = __pyx_state[10]; __pyx_result._loop = __pyx_state[11]; __pyx_result._message_handlers = __pyx_state[12]; __pyx_result._params = __pyx_state[13]; __pyx_result._ping_timer = __pyx_state[14]; __pyx_result._pong_timer = __pyx_state[15]; __pyx_result._read_exception_futures = __pyx_state[16]; __pyx_result._resolve_host_future = __pyx_state[17]; __pyx_result._send_pending_ping = __pyx_state[18]; __pyx_result._socket = __pyx_state[19]; __pyx_result._start_connect_future = __pyx_state[20]; __pyx_result.api_version = __pyx_state[21]; __pyx_result.connected_address = __pyx_state[22]; __pyx_result.connection_state = __pyx_state[23]; __pyx_result.is_connected = __pyx_state[24]; __pyx_result.log_name = __pyx_state[25]; __pyx_result.on_stop = __pyx_state[26]; __pyx_result.received_name = __pyx_state[27]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 28)
*/

static PyObject *__pyx_f_13aioesphomeapi_10connection___pyx_unpickle_APIConnection__set_state(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  float __pyx_t_3;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_APIConnection__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_APIConnection__set_state(APIConnection __pyx_result, __pyx_state: tuple):
 *     __pyx_result._addrs_info = __pyx_state[0]; __pyx_result._cached_timezone = __pyx_state[1]; __pyx_result._debug_enabled = __pyx_state[2]; __pyx_result._expected_disconnect = __pyx_state[3]; __pyx_result._fatal_exception = __pyx_state[4]; __pyx_result._finish_connect_future = __pyx_state[5]; __pyx_result._frame_helper = __pyx_state[6]; __pyx_result._handshake_complete = __pyx_state[7]; __pyx_result._keep_alive_interval = __pyx_state[8]; __pyx_result._keep_alive_timeout = __pyx_state[9]; __pyx_result._log_errors = __pyx_state[10]; __pyx_result._loop = __pyx_state[11]; __pyx_result._message_handlers = __pyx_state[12]; __pyx_result._params = __pyx_state[13]; __pyx_result._ping_timer = __pyx_state[14]; __pyx_result._pong_timer = __pyx_state[15]; __pyx_result._read_exception_futures = __pyx_state[16]; __pyx_result._resolve_host_future = __pyx_state[17]; __pyx_result._send_pending_ping = __pyx_state[18]; __pyx_result._socket = __pyx_state[19]; __pyx_result._start_connect_future = __pyx_state[20]; __pyx_result.api_version = __pyx_state[21]; __pyx_result.connected_address = __pyx_state[22]; __pyx_result.connection_state = __pyx_state[23]; __pyx_result.is_connected = __pyx_state[24]; __pyx_result.log_name = __pyx_state[25]; __pyx_result.on_stop = __pyx_state[26]; __pyx_result.received_name = __pyx_state[27]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 28)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("list", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_addrs_info);
  __Pyx_DECREF(__pyx_v___pyx_result->_addrs_info);
  __pyx_v___pyx_result->_addrs_info = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_cached_timezone);
  __Pyx_DECREF(__pyx_v___pyx_result->_cached_timezone);
  __pyx_v___pyx_result->_cached_timezone = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_debug_enabled = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_expected_disconnect = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(__Pyx_PyException_Check(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("Exception", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_fatal_exception);
  __Pyx_DECREF(__pyx_v___pyx_result->_fatal_exception);
  __pyx_v___pyx_result->_fatal_exception = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_finish_connect_future);
  __Pyx_DECREF(__pyx_v___pyx_result->_finish_connect_future);
  __pyx_v___pyx_result->_finish_connect_future = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->_frame_helper);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->_frame_helper);
  __pyx_v___pyx_result->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_handshake_complete = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_keep_alive_interval = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyFloat_AsFloat(__pyx_t_1); if (unlikely((__pyx_t_3 == (float)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_keep_alive_timeout = __pyx_t_3;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_log_errors = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_loop);
  __Pyx_DECREF(__pyx_v___pyx_result->_loop);
  __pyx_v___pyx_result->_loop = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyDict_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_message_handlers);
  __Pyx_DECREF(__pyx_v___pyx_result->_message_handlers);
  __pyx_v___pyx_result->_message_handlers = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->_params);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->_params);
  __pyx_v___pyx_result->_params = ((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 14, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_ping_timer);
  __Pyx_DECREF(__pyx_v___pyx_result->_ping_timer);
  __pyx_v___pyx_result->_ping_timer = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 15, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_pong_timer);
  __Pyx_DECREF(__pyx_v___pyx_result->_pong_timer);
  __pyx_v___pyx_result->_pong_timer = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 16, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PySet_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("set", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_read_exception_futures);
  __Pyx_DECREF(__pyx_v___pyx_result->_read_exception_futures);
  __pyx_v___pyx_result->_read_exception_futures = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 17, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_resolve_host_future);
  __Pyx_DECREF(__pyx_v___pyx_result->_resolve_host_future);
  __pyx_v___pyx_result->_resolve_host_future = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 18, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_send_pending_ping = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 19, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_socket);
  __Pyx_DECREF(__pyx_v___pyx_result->_socket);
  __pyx_v___pyx_result->_socket = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 20, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_start_connect_future);
  __Pyx_DECREF(__pyx_v___pyx_result->_start_connect_future);
  __pyx_v___pyx_result->_start_connect_future = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 21, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->api_version);
  __Pyx_DECREF(__pyx_v___pyx_result->api_version);
  __pyx_v___pyx_result->api_version = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 22, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->connected_address);
  __Pyx_DECREF(__pyx_v___pyx_result->connected_address);
  __pyx_v___pyx_result->connected_address = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 23, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->connection_state);
  __Pyx_DECREF(__pyx_v___pyx_result->connection_state);
  __pyx_v___pyx_result->connection_state = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 24, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->is_connected = __pyx_t_2;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 25, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->log_name);
  __Pyx_DECREF(__pyx_v___pyx_result->log_name);
  __pyx_v___pyx_result->log_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 26, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->on_stop);
  __Pyx_DECREF(__pyx_v___pyx_result->on_stop);
  __pyx_v___pyx_result->on_stop = __pyx_t_1;
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 27, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyUnicode_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("str", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->received_name);
  __Pyx_DECREF(__pyx_v___pyx_result->received_name);
  __pyx_v___pyx_result->received_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_APIConnection__set_state(APIConnection __pyx_result, __pyx_state: tuple):
 *     __pyx_result._addrs_info = __pyx_state[0]; __pyx_result._cached_timezone = __pyx_state[1]; __pyx_result._debug_enabled = __pyx_state[2]; __pyx_result._expected_disconnect = __pyx_state[3]; __pyx_result._fatal_exception = __pyx_state[4]; __pyx_result._finish_connect_future = __pyx_state[5]; __pyx_result._frame_helper = __pyx_state[6]; __pyx_result._handshake_complete = __pyx_state[7]; __pyx_result._keep_alive_interval = __pyx_state[8]; __pyx_result._keep_alive_timeout = __pyx_state[9]; __pyx_result._log_errors = __pyx_state[10]; __pyx_result._loop = __pyx_state[11]; __pyx_result._message_handlers = __pyx_state[12]; __pyx_result._params = __pyx_state[13]; __pyx_result._ping_timer = __pyx_state[14]; __pyx_result._pong_timer = __pyx_state[15]; __pyx_result._read_exception_futures = __pyx_state[16]; __pyx_result._resolve_host_future = __pyx_state[17]; __pyx_result._send_pending_ping = __pyx_state[18]; __pyx_result._socket = __pyx_state[19]; __pyx_result._start_connect_future = __pyx_state[20]; __pyx_result.api_version = __pyx_state[21]; __pyx_result.connected_address = __pyx_state[22]; __pyx_result.connection_state = __pyx_state[23]; __pyx_result.is_connected = __pyx_state[24]; __pyx_result.log_name = __pyx_state[25]; __pyx_result.on_stop = __pyx_state[26]; __pyx_result.received_name = __pyx_state[27]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 28)             # <<<<<<<<<<<<<<
*/
  __pyx_t_4 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 28); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_APIConnection__set_state(<APIConnection> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_APIConnection__set_state(APIConnection __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result._addrs_info = __pyx_state[0]; __pyx_result._cached_timezone = __pyx_state[1]; __pyx_result._debug_enabled = __pyx_state[2]; __pyx_result._expected_disconnect = __pyx_state[3]; __pyx_result._fatal_exception = __pyx_state[4]; __pyx_result._finish_connect_future = __pyx_state[5]; __pyx_result._frame_helper = __pyx_state[6]; __pyx_result._handshake_complete = __pyx_state[7]; __pyx_result._keep_alive_interval = __pyx_state[8]; __pyx_result._keep_alive_timeout = __pyx_state[9]; __pyx_result._log_errors = __pyx_state[10]; __pyx_result._loop = __pyx_state[11]; __pyx_result._message_handlers = __pyx_state[12]; __pyx_result._params = __pyx_state[13]; __pyx_result._ping_timer = __pyx_state[14]; __pyx_result._pong_timer = __pyx_state[15]; __pyx_result._read_exception_futures = __pyx_state[16]; __pyx_result._resolve_host_future = __pyx_state[17]; __pyx_result._send_pending_ping = __pyx_state[18]; __pyx_result._socket = __pyx_state[19]; __pyx_result._start_connect_future = __pyx_state[20]; __pyx_result.api_version = __pyx_state[21]; __pyx_result.connected_address = __pyx_state[22]; __pyx_result.connection_state = __pyx_state[23]; __pyx_result.is_connected = __pyx_state[24]; __pyx_result.log_name = __pyx_state[25]; __pyx_result.on_stop = __pyx_state[26]; __pyx_result.received_name = __pyx_state[27]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 28)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("aioesphomeapi.connection.__pyx_unpickle_APIConnection__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection_ConnectionParams(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)o);
  p->addresses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->port = Py_None; Py_INCREF(Py_None);
  p->password = Py_None; Py_INCREF(Py_None);
  p->client_info = Py_None; Py_INCREF(Py_None);
  p->keepalive = Py_None; Py_INCREF(Py_None);
  p->zeroconf_manager = Py_None; Py_INCREF(Py_None);
  p->noise_psk = Py_None; Py_INCREF(Py_None);
  p->expected_name = Py_None; Py_INCREF(Py_None);
  p->expected_mac = Py_None; Py_INCREF(Py_None);
  p->timezone = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection_ConnectionParams(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *p = (struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection_ConnectionParams) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->addresses);
  Py_CLEAR(p->port);
  Py_CLEAR(p->password);
  Py_CLEAR(p->client_info);
  Py_CLEAR(p->keepalive);
  Py_CLEAR(p->zeroconf_manager);
  Py_CLEAR(p->noise_psk);
  Py_CLEAR(p->expected_name);
  Py_CLEAR(p->expected_mac);
  Py_CLEAR(p->timezone);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection_ConnectionParams(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *p = (struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->addresses) {
    e = (*v)(p->addresses, a); if (e) return e;
  }
  if (p->port) {
    e = (*v)(p->port, a); if (e) return e;
  }
  if (p->password) {
    e = (*v)(p->password, a); if (e) return e;
  }
  if (p->client_info) {
    e = (*v)(p->client_info, a); if (e) return e;
  }
  if (p->keepalive) {
    e = (*v)(p->keepalive, a); if (e) return e;
  }
  if (p->zeroconf_manager) {
    e = (*v)(p->zeroconf_manager, a); if (e) return e;
  }
  if (p->noise_psk) {
    e = (*v)(p->noise_psk, a); if (e) return e;
  }
  if (p->expected_name) {
    e = (*v)(p->expected_name, a); if (e) return e;
  }
  if (p->expected_mac) {
    e = (*v)(p->expected_mac, a); if (e) return e;
  }
  if (p->timezone) {
    e = (*v)(p->timezone, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13aioesphomeapi_10connection_ConnectionParams(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *p = (struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)o;
  tmp = ((PyObject*)p->addresses);
  p->addresses = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->port);
  p->port = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->password);
  p->password = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->client_info);
  p->client_info = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->keepalive);
  p->keepalive = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->zeroconf_manager);
  p->zeroconf_manager = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->noise_psk);
  p->noise_psk = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->expected_name);
  p->expected_name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->expected_mac);
  p->expected_mac = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->timezone);
  p->timezone = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_tp_richcompare_13aioesphomeapi_10connection_ConnectionParams(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_5__eq__(o1, o2);
    }
    case Py_NE: {
      PyObject *ret;
      ret = __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_5__eq__(o1, o2);
      if (likely(ret && ret != Py_NotImplemented)) {
        int b = __Pyx_PyObject_IsTrue(ret);
        Py_DECREF(ret);
        if (unlikely(b < 0)) return NULL;
        ret = (b) ? Py_False : Py_True;
        Py_INCREF(ret);
      }
      return ret;
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_addresses(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_addresses(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9addresses_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_port(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_port(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_4port_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_password(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_password(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8password_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_client_info(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_client_info(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_11client_info_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_keepalive(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_keepalive(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9keepalive_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_zeroconf_manager(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_zeroconf_manager(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_16zeroconf_manager_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_noise_psk(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_noise_psk(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9noise_psk_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_expected_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_expected_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_13expected_name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_expected_mac(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_expected_mac(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_12expected_mac_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_timezone(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_timezone(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_8timezone_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_13aioesphomeapi_10connection_ConnectionParams[] = {
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_7__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_9__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_13aioesphomeapi_10connection_ConnectionParams[] = {
  {"addresses", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_addresses, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_addresses, 0, 0},
  {"port", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_port, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_port, 0, 0},
  {"password", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_password, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_password, 0, 0},
  {"client_info", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_client_info, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_client_info, 0, 0},
  {"keepalive", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_keepalive, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_keepalive, 0, 0},
  {"zeroconf_manager", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_zeroconf_manager, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_zeroconf_manager, 0, 0},
  {"noise_psk", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_noise_psk, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_noise_psk, 0, 0},
  {"expected_name", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_expected_name, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_expected_name, 0, 0},
  {"expected_mac", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_expected_mac, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_expected_mac, 0, 0},
  {"timezone", __pyx_getprop_13aioesphomeapi_10connection_16ConnectionParams_timezone, __pyx_setprop_13aioesphomeapi_10connection_16ConnectionParams_timezone, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection_ConnectionParams_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection_ConnectionParams},
  {Py_tp_repr, (void *)__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_3__repr__},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection_ConnectionParams},
  {Py_tp_clear, (void *)__pyx_tp_clear_13aioesphomeapi_10connection_ConnectionParams},
  {Py_tp_richcompare, (void *)__pyx_tp_richcompare_13aioesphomeapi_10connection_ConnectionParams},
  {Py_tp_methods, (void *)__pyx_methods_13aioesphomeapi_10connection_ConnectionParams},
  {Py_tp_getset, (void *)__pyx_getsets_13aioesphomeapi_10connection_ConnectionParams},
  {Py_tp_init, (void *)__pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection_ConnectionParams},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection_ConnectionParams_spec = {
  "aioesphomeapi.connection.ConnectionParams",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection_ConnectionParams_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection_ConnectionParams = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""ConnectionParams", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection_ConnectionParams, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection_ConnectionParams, /*tp_traverse*/
  __pyx_tp_clear_13aioesphomeapi_10connection_ConnectionParams, /*tp_clear*/
  __pyx_tp_richcompare_13aioesphomeapi_10connection_ConnectionParams, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_13aioesphomeapi_10connection_ConnectionParams, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_13aioesphomeapi_10connection_ConnectionParams, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_13aioesphomeapi_10connection_16ConnectionParams_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection_ConnectionParams, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_13aioesphomeapi_10connection_APIConnection __pyx_vtable_13aioesphomeapi_10connection_APIConnection;

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection_APIConnection(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)o);
  p->__pyx_vtab = __pyx_vtabptr_13aioesphomeapi_10connection_APIConnection;
  p->_params = ((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)Py_None); Py_INCREF(Py_None);
  p->on_stop = Py_None; Py_INCREF(Py_None);
  p->_socket = Py_None; Py_INCREF(Py_None);
  p->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)Py_None); Py_INCREF(Py_None);
  p->api_version = Py_None; Py_INCREF(Py_None);
  p->connection_state = Py_None; Py_INCREF(Py_None);
  p->_message_handlers = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->log_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_read_exception_futures = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_ping_timer = Py_None; Py_INCREF(Py_None);
  p->_pong_timer = Py_None; Py_INCREF(Py_None);
  p->_resolve_host_future = Py_None; Py_INCREF(Py_None);
  p->_start_connect_future = Py_None; Py_INCREF(Py_None);
  p->_finish_connect_future = Py_None; Py_INCREF(Py_None);
  p->_fatal_exception = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_loop = Py_None; Py_INCREF(Py_None);
  p->received_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->connected_address = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_cached_timezone = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_addrs_info = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection_APIConnection(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *p = (struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection_APIConnection) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_params);
  Py_CLEAR(p->on_stop);
  Py_CLEAR(p->_socket);
  Py_CLEAR(p->_frame_helper);
  Py_CLEAR(p->api_version);
  Py_CLEAR(p->connection_state);
  Py_CLEAR(p->_message_handlers);
  Py_CLEAR(p->log_name);
  Py_CLEAR(p->_read_exception_futures);
  Py_CLEAR(p->_ping_timer);
  Py_CLEAR(p->_pong_timer);
  Py_CLEAR(p->_resolve_host_future);
  Py_CLEAR(p->_start_connect_future);
  Py_CLEAR(p->_finish_connect_future);
  Py_CLEAR(p->_fatal_exception);
  Py_CLEAR(p->_loop);
  Py_CLEAR(p->received_name);
  Py_CLEAR(p->connected_address);
  Py_CLEAR(p->_cached_timezone);
  Py_CLEAR(p->_addrs_info);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection_APIConnection(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *p = (struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->_params) {
    e = (*v)(((PyObject *)p->_params), a); if (e) return e;
  }
  if (p->on_stop) {
    e = (*v)(p->on_stop, a); if (e) return e;
  }
  if (p->_socket) {
    e = (*v)(p->_socket, a); if (e) return e;
  }
  if (p->_frame_helper) {
    e = (*v)(((PyObject *)p->_frame_helper), a); if (e) return e;
  }
  if (p->api_version) {
    e = (*v)(p->api_version, a); if (e) return e;
  }
  if (p->connection_state) {
    e = (*v)(p->connection_state, a); if (e) return e;
  }
  if (p->_message_handlers) {
    e = (*v)(p->_message_handlers, a); if (e) return e;
  }
  if (p->_read_exception_futures) {
    e = (*v)(p->_read_exception_futures, a); if (e) return e;
  }
  if (p->_ping_timer) {
    e = (*v)(p->_ping_timer, a); if (e) return e;
  }
  if (p->_pong_timer) {
    e = (*v)(p->_pong_timer, a); if (e) return e;
  }
  if (p->_resolve_host_future) {
    e = (*v)(p->_resolve_host_future, a); if (e) return e;
  }
  if (p->_start_connect_future) {
    e = (*v)(p->_start_connect_future, a); if (e) return e;
  }
  if (p->_finish_connect_future) {
    e = (*v)(p->_finish_connect_future, a); if (e) return e;
  }
  if (p->_fatal_exception) {
    e = (*v)(p->_fatal_exception, a); if (e) return e;
  }
  if (p->_loop) {
    e = (*v)(p->_loop, a); if (e) return e;
  }
  if (p->_addrs_info) {
    e = (*v)(p->_addrs_info, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_13aioesphomeapi_10connection_APIConnection(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *p = (struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *)o;
  tmp = ((PyObject*)p->_params);
  p->_params = ((struct __pyx_obj_13aioesphomeapi_10connection_ConnectionParams *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->on_stop);
  p->on_stop = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_socket);
  p->_socket = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_frame_helper);
  p->_frame_helper = ((struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->api_version);
  p->api_version = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->connection_state);
  p->connection_state = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_message_handlers);
  p->_message_handlers = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_read_exception_futures);
  p->_read_exception_futures = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_ping_timer);
  p->_ping_timer = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_pong_timer);
  p->_pong_timer = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_resolve_host_future);
  p->_resolve_host_future = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_start_connect_future);
  p->_start_connect_future = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_finish_connect_future);
  p->_finish_connect_future = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_fatal_exception);
  p->_fatal_exception = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_loop);
  p->_loop = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_addrs_info);
  p->_addrs_info = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_on_stop(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_on_stop(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7on_stop_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection__socket(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__socket(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_socket_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection__frame_helper(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__frame_helper(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_frame_helper_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_api_version(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_api_version(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_11api_version_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_connection_state(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_connection_state(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16connection_state_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection__message_handlers(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__message_handlers(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17_message_handlers_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_log_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_log_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_8log_name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection__fatal_exception(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__fatal_exception(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_fatal_exception_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_is_connected(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_12is_connected_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_is_connected(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_12is_connected_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection__handshake_complete(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__handshake_complete(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_19_handshake_complete_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_received_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_received_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_13received_name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_13aioesphomeapi_10connection_13APIConnection_connected_address(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_1__get__(o);
}

static int __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_connected_address(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_3__set__(o, v);
  }
  else {
    return __pyx_pw_13aioesphomeapi_10connection_13APIConnection_17connected_address_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_13aioesphomeapi_10connection_APIConnection[] = {
  {"set_debug", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_5set_debug, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_4set_debug},
  {"_connect_socket_connect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_7_connect_socket_connect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_6_connect_socket_connect},
  {"_connect_init_frame_helper", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_10_connect_init_frame_helper, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_9_connect_init_frame_helper},
  {"_connect_hello_login", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_13_connect_hello_login, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_12_connect_hello_login},
  {"_handle_login_response", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_16_handle_login_response, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_15_handle_login_response},
  {"_async_send_keep_alive", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_18_async_send_keep_alive, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_17_async_send_keep_alive},
  {"_async_pong_not_received", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_20_async_pong_not_received, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_19_async_pong_not_received},
  {"start_resolve_host", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_22start_resolve_host, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_21start_resolve_host},
  {"_set_resolve_host_future", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_25_set_resolve_host_future, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"start_connection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_27start_connection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_26start_connection},
  {"_raise_fatal_connection_exception", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_30_raise_fatal_connection_exception, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_29_raise_fatal_connection_exception},
  {"_wrap_fatal_connection_exception", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_32_wrap_fatal_connection_exception, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_31_wrap_fatal_connection_exception},
  {"_do_finish_connect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_34_do_finish_connect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_33_do_finish_connect},
  {"finish_connection", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_37finish_connection, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_36finish_connection},
  {"send_message_callback_response", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_48send_message_callback_response, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_47send_message_callback_response},
  {"send_messages_await_response_complex", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_50send_messages_await_response_complex, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_49send_messages_await_response_complex},
  {"send_message_await_response", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_53send_message_await_response, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"disconnect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_66disconnect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_65disconnect},
  {"force_disconnect", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_69force_disconnect, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_13aioesphomeapi_10connection_13APIConnection_68force_disconnect},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_71__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_73__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_13aioesphomeapi_10connection_APIConnection[] = {
  {"on_stop", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_on_stop, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_on_stop, 0, 0},
  {"_socket", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection__socket, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__socket, 0, 0},
  {"_frame_helper", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection__frame_helper, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__frame_helper, 0, 0},
  {"api_version", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_api_version, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_api_version, 0, 0},
  {"connection_state", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_connection_state, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_connection_state, 0, 0},
  {"_message_handlers", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection__message_handlers, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__message_handlers, 0, 0},
  {"log_name", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_log_name, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_log_name, 0, 0},
  {"_fatal_exception", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection__fatal_exception, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__fatal_exception, 0, 0},
  {"is_connected", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_is_connected, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_is_connected, 0, 0},
  {"_handshake_complete", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection__handshake_complete, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection__handshake_complete, 0, 0},
  {"received_name", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_received_name, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_received_name, 0, 0},
  {"connected_address", __pyx_getprop_13aioesphomeapi_10connection_13APIConnection_connected_address, __pyx_setprop_13aioesphomeapi_10connection_13APIConnection_connected_address, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection_APIConnection_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection_APIConnection},
  {Py_tp_doc, (void *)PyDoc_STR("This class represents _one_ connection to a remote native API device.\n\n    An instance of this class may only be used once, for every new connection\n    a new instance should be established.\n\n    This class should only be created from APIClient and should not be used directly.\n    ")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection_APIConnection},
  {Py_tp_clear, (void *)__pyx_tp_clear_13aioesphomeapi_10connection_APIConnection},
  {Py_tp_methods, (void *)__pyx_methods_13aioesphomeapi_10connection_APIConnection},
  {Py_tp_getset, (void *)__pyx_getsets_13aioesphomeapi_10connection_APIConnection},
  {Py_tp_init, (void *)__pyx_pw_13aioesphomeapi_10connection_13APIConnection_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection_APIConnection},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection_APIConnection_spec = {
  "aioesphomeapi.connection.APIConnection",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection_APIConnection_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection_APIConnection = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""APIConnection", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection_APIConnection, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("This class represents _one_ connection to a remote native API device.\n\n    An instance of this class may only be used once, for every new connection\n    a new instance should be established.\n\n    This class should only be created from APIClient and should not be used directly.\n    "), /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection_APIConnection, /*tp_traverse*/
  __pyx_tp_clear_13aioesphomeapi_10connection_APIConnection, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_13aioesphomeapi_10connection_APIConnection, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_13aioesphomeapi_10connection_APIConnection, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_13aioesphomeapi_10connection_13APIConnection_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection_APIConnection, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct__genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct__genexpr > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct__genexpr[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct__genexpr];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct__genexpr(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct__genexpr) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_genexpr_arg_0);
  Py_CLEAR(p->__pyx_v_msg);
  Py_CLEAR(p->__pyx_t_0);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct__genexpr < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct__genexpr[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct__genexpr++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct__genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_genexpr_arg_0) {
    e = (*v)(p->__pyx_genexpr_arg_0, a); if (e) return e;
  }
  if (p->__pyx_v_msg) {
    e = (*v)(p->__pyx_v_msg, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct__genexpr},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct__genexpr},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct__genexpr},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct__genexpr",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct__genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct__genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct__genexpr, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct__genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct__genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_8genexpr3__pyx_v_addr);
  Py_CLEAR(p->__pyx_v_addr_infos);
  Py_CLEAR(p->__pyx_v_addrs);
  Py_CLEAR(p->__pyx_v_err);
  Py_CLEAR(p->__pyx_v_genexpr);
  Py_CLEAR(p->__pyx_v_last_exception);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_sock);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  Py_CLEAR(p->__pyx_t_3);
  Py_CLEAR(p->__pyx_t_4);
  Py_CLEAR(p->__pyx_t_5);
  Py_CLEAR(p->__pyx_t_6);
  Py_CLEAR(p->__pyx_t_7);
  Py_CLEAR(p->__pyx_t_8);
  Py_CLEAR(p->__pyx_t_9);
  Py_CLEAR(p->__pyx_t_10);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_8genexpr3__pyx_v_addr) {
    e = (*v)(p->__pyx_8genexpr3__pyx_v_addr, a); if (e) return e;
  }
  if (p->__pyx_v_addr_infos) {
    e = (*v)(p->__pyx_v_addr_infos, a); if (e) return e;
  }
  if (p->__pyx_v_addrs) {
    e = (*v)(p->__pyx_v_addrs, a); if (e) return e;
  }
  if (p->__pyx_v_err) {
    e = (*v)(p->__pyx_v_err, a); if (e) return e;
  }
  if (p->__pyx_v_genexpr) {
    e = (*v)(p->__pyx_v_genexpr, a); if (e) return e;
  }
  if (p->__pyx_v_last_exception) {
    e = (*v)(p->__pyx_v_last_exception, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_sock) {
    e = (*v)(p->__pyx_v_sock, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  if (p->__pyx_t_3) {
    e = (*v)(p->__pyx_t_3, a); if (e) return e;
  }
  if (p->__pyx_t_4) {
    e = (*v)(p->__pyx_t_4, a); if (e) return e;
  }
  if (p->__pyx_t_5) {
    e = (*v)(p->__pyx_t_5, a); if (e) return e;
  }
  if (p->__pyx_t_6) {
    e = (*v)(p->__pyx_t_6, a); if (e) return e;
  }
  if (p->__pyx_t_7) {
    e = (*v)(p->__pyx_t_7, a); if (e) return e;
  }
  if (p->__pyx_t_8) {
    e = (*v)(p->__pyx_t_8, a); if (e) return e;
  }
  if (p->__pyx_t_9) {
    e = (*v)(p->__pyx_t_9, a); if (e) return e;
  }
  if (p->__pyx_t_10) {
    e = (*v)(p->__pyx_t_10, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_1__connect_socket_connect",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_1__connect_socket_connect", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_genexpr_arg_0);
  Py_CLEAR(p->__pyx_v_addr);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_genexpr_arg_0) {
    e = (*v)(p->__pyx_genexpr_arg_0, a); if (e) return e;
  }
  if (p->__pyx_v_addr) {
    e = (*v)(p->__pyx_v_addr, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_2_genexpr",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_2_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v__);
  Py_CLEAR(p->__pyx_v_err);
  Py_CLEAR(p->__pyx_v_fh);
  Py_CLEAR(p->__pyx_v_handshake_handle);
  Py_CLEAR(p->__pyx_v_noise_psk);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v__) {
    e = (*v)(p->__pyx_v__, a); if (e) return e;
  }
  if (p->__pyx_v_err) {
    e = (*v)(p->__pyx_v_err, a); if (e) return e;
  }
  if (p->__pyx_v_fh) {
    e = (*v)(p->__pyx_v_fh, a); if (e) return e;
  }
  if (p->__pyx_v_handshake_handle) {
    e = (*v)(p->__pyx_v_handshake_handle, a); if (e) return e;
  }
  if (p->__pyx_v_noise_psk) {
    e = (*v)(p->__pyx_v_noise_psk, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_3__connect_init_frame_helper",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_3__connect_init_frame_helper", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_messages);
  Py_CLEAR(p->__pyx_v_msg_types);
  Py_CLEAR(p->__pyx_v_resp);
  Py_CLEAR(p->__pyx_v_responses);
  Py_CLEAR(p->__pyx_v_self);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_messages) {
    e = (*v)(p->__pyx_v_messages, a); if (e) return e;
  }
  if (p->__pyx_v_msg_types) {
    e = (*v)(p->__pyx_v_msg_types, a); if (e) return e;
  }
  if (p->__pyx_v_resp) {
    e = (*v)(p->__pyx_v_resp, a); if (e) return e;
  }
  if (p->__pyx_v_responses) {
    e = (*v)(p->__pyx_v_responses, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_4__connect_hello_login",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_4__connect_hello_login", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_ex);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  Py_CLEAR(p->__pyx_t_3);
  Py_CLEAR(p->__pyx_t_4);
  Py_CLEAR(p->__pyx_t_5);
  Py_CLEAR(p->__pyx_t_6);
  Py_CLEAR(p->__pyx_t_7);
  Py_CLEAR(p->__pyx_t_8);
  Py_CLEAR(p->__pyx_t_9);
  Py_CLEAR(p->__pyx_t_10);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_ex) {
    e = (*v)(p->__pyx_v_ex, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  if (p->__pyx_t_3) {
    e = (*v)(p->__pyx_t_3, a); if (e) return e;
  }
  if (p->__pyx_t_4) {
    e = (*v)(p->__pyx_t_4, a); if (e) return e;
  }
  if (p->__pyx_t_5) {
    e = (*v)(p->__pyx_t_5, a); if (e) return e;
  }
  if (p->__pyx_t_6) {
    e = (*v)(p->__pyx_t_6, a); if (e) return e;
  }
  if (p->__pyx_t_7) {
    e = (*v)(p->__pyx_t_7, a); if (e) return e;
  }
  if (p->__pyx_t_8) {
    e = (*v)(p->__pyx_t_8, a); if (e) return e;
  }
  if (p->__pyx_t_9) {
    e = (*v)(p->__pyx_t_9, a); if (e) return e;
  }
  if (p->__pyx_t_10) {
    e = (*v)(p->__pyx_t_10, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_5_start_resolve_host",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_5_start_resolve_host", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_ex);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  Py_CLEAR(p->__pyx_t_3);
  Py_CLEAR(p->__pyx_t_4);
  Py_CLEAR(p->__pyx_t_5);
  Py_CLEAR(p->__pyx_t_6);
  Py_CLEAR(p->__pyx_t_7);
  Py_CLEAR(p->__pyx_t_8);
  Py_CLEAR(p->__pyx_t_9);
  Py_CLEAR(p->__pyx_t_10);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_ex) {
    e = (*v)(p->__pyx_v_ex, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  if (p->__pyx_t_3) {
    e = (*v)(p->__pyx_t_3, a); if (e) return e;
  }
  if (p->__pyx_t_4) {
    e = (*v)(p->__pyx_t_4, a); if (e) return e;
  }
  if (p->__pyx_t_5) {
    e = (*v)(p->__pyx_t_5, a); if (e) return e;
  }
  if (p->__pyx_t_6) {
    e = (*v)(p->__pyx_t_6, a); if (e) return e;
  }
  if (p->__pyx_t_7) {
    e = (*v)(p->__pyx_t_7, a); if (e) return e;
  }
  if (p->__pyx_t_8) {
    e = (*v)(p->__pyx_t_8, a); if (e) return e;
  }
  if (p->__pyx_t_9) {
    e = (*v)(p->__pyx_t_9, a); if (e) return e;
  }
  if (p->__pyx_t_10) {
    e = (*v)(p->__pyx_t_10, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_6_start_connection",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_6_start_connection", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_7__do_finish_connect",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_7__do_finish_connect", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_ex);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  Py_CLEAR(p->__pyx_t_3);
  Py_CLEAR(p->__pyx_t_4);
  Py_CLEAR(p->__pyx_t_5);
  Py_CLEAR(p->__pyx_t_6);
  Py_CLEAR(p->__pyx_t_7);
  Py_CLEAR(p->__pyx_t_8);
  Py_CLEAR(p->__pyx_t_9);
  Py_CLEAR(p->__pyx_t_10);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_ex) {
    e = (*v)(p->__pyx_v_ex, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  if (p->__pyx_t_3) {
    e = (*v)(p->__pyx_t_3, a); if (e) return e;
  }
  if (p->__pyx_t_4) {
    e = (*v)(p->__pyx_t_4, a); if (e) return e;
  }
  if (p->__pyx_t_5) {
    e = (*v)(p->__pyx_t_5, a); if (e) return e;
  }
  if (p->__pyx_t_6) {
    e = (*v)(p->__pyx_t_6, a); if (e) return e;
  }
  if (p->__pyx_t_7) {
    e = (*v)(p->__pyx_t_7, a); if (e) return e;
  }
  if (p->__pyx_t_8) {
    e = (*v)(p->__pyx_t_8, a); if (e) return e;
  }
  if (p->__pyx_t_9) {
    e = (*v)(p->__pyx_t_9, a); if (e) return e;
  }
  if (p->__pyx_t_10) {
    e = (*v)(p->__pyx_t_10, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_8_finish_connection",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_8_finish_connection", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_do_append);
  Py_CLEAR(p->__pyx_v_do_stop);
  Py_CLEAR(p->__pyx_v_err);
  Py_CLEAR(p->__pyx_v_fut);
  Py_CLEAR(p->__pyx_v_messages);
  Py_CLEAR(p->__pyx_v_msg_types);
  Py_CLEAR(p->__pyx_v_on_message);
  Py_CLEAR(p->__pyx_v_response_names);
  Py_CLEAR(p->__pyx_v_responses);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_timeout);
  Py_CLEAR(p->__pyx_v_timeout_handle);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_do_append) {
    e = (*v)(p->__pyx_v_do_append, a); if (e) return e;
  }
  if (p->__pyx_v_do_stop) {
    e = (*v)(p->__pyx_v_do_stop, a); if (e) return e;
  }
  if (p->__pyx_v_err) {
    e = (*v)(p->__pyx_v_err, a); if (e) return e;
  }
  if (p->__pyx_v_fut) {
    e = (*v)(p->__pyx_v_fut, a); if (e) return e;
  }
  if (p->__pyx_v_messages) {
    e = (*v)(p->__pyx_v_messages, a); if (e) return e;
  }
  if (p->__pyx_v_msg_types) {
    e = (*v)(p->__pyx_v_msg_types, a); if (e) return e;
  }
  if (p->__pyx_v_on_message) {
    e = (*v)(p->__pyx_v_on_message, a); if (e) return e;
  }
  if (p->__pyx_v_response_names) {
    e = (*v)(p->__pyx_v_response_names, a); if (e) return e;
  }
  if (p->__pyx_v_responses) {
    e = (*v)(p->__pyx_v_responses, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_timeout) {
    e = (*v)(p->__pyx_v_timeout, a); if (e) return e;
  }
  if (p->__pyx_v_timeout_handle) {
    e = (*v)(p->__pyx_v_timeout_handle, a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_9_send_messages_await_response_complex",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_9_send_messages_await_response_complex", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_response);
  Py_CLEAR(p->__pyx_v_response_type);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_send_msg);
  Py_CLEAR(p->__pyx_v_timeout);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_response) {
    e = (*v)(p->__pyx_v_response, a); if (e) return e;
  }
  if (p->__pyx_v_response_type) {
    e = (*v)(p->__pyx_v_response_type, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_send_msg) {
    e = (*v)(p->__pyx_v_send_msg, a); if (e) return e;
  }
  if (p->__pyx_v_timeout) {
    e = (*v)(p->__pyx_v_timeout, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_10_send_message_await_response",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_10_send_message_await_response", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect[--__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect(PyObject *o) {
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v__);
  Py_CLEAR(p->__pyx_v_pending);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_t_0);
  Py_CLEAR(p->__pyx_t_1);
  Py_CLEAR(p->__pyx_t_2);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect, sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect))))
  {
    __pyx_mstate_global->__pyx_freelist_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect[__pyx_mstate_global->__pyx_freecount_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect++] = ((struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *p = (struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v__) {
    e = (*v)(p->__pyx_v__, a); if (e) return e;
  }
  if (p->__pyx_v_pending) {
    e = (*v)(p->__pyx_v_pending, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_t_0) {
    e = (*v)(p->__pyx_t_0, a); if (e) return e;
  }
  if (p->__pyx_t_1) {
    e = (*v)(p->__pyx_t_1, a); if (e) return e;
  }
  if (p->__pyx_t_2) {
    e = (*v)(p->__pyx_t_2, a); if (e) return e;
  }
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect},
  {Py_tp_new, (void *)__pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect},
  {0, 0},
};
static PyType_Spec __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect_spec = {
  "aioesphomeapi.connection.__pyx_scope_struct_11_disconnect",
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect_slots,
};
#else

static PyTypeObject __pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect = {
  PyVarObject_HEAD_INIT(0, 0)
  "aioesphomeapi.connection.""__pyx_scope_struct_11_disconnect", /*tp_name*/
  sizeof(struct __pyx_obj_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_OPEN_STATES = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_WRITE_EXCEPTIONS = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_DISCONNECT_RESPONSE_MESSAGES = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_PING_REQUEST_MESSAGES = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_PING_RESPONSE_MESSAGES = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_asyncio_timeout = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CancelledError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_AuthenticationRequest = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_AuthenticationResponse = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_DisconnectRequest = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_PingRequest = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_GetTimeRequest = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_GetTimeResponse = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_HelloRequest = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_HelloResponse = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_APIVersion = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_partial = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_hr = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECT_AND_SETUP_TIMEOUT = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECT_REQUEST_TIMEOUT = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_APIConnectionError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_BadNameAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_HandshakeAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_PingFailedAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_TimeoutAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_SocketAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_astuple = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_make_hello_request = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection__handle_timeout = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection__handle_complex_message = Py_None; Py_INCREF(Py_None);
  __pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k = Py_None; Py_INCREF(Py_None);
  __pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  {
    __pyx_t_1 = __Pyx_ApiExport_GetApiDict(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_export_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_PyObject_float_int_APIConnection);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_export_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_export_name = __pyx_export_signature + 66;
    void *const __pyx_export_pointers[] = {(void *)&__pyx_v_13aioesphomeapi_10connection_APIConnectionError, (void *)&__pyx_v_13aioesphomeapi_10connection_APIVersion, (void *)&__pyx_v_13aioesphomeapi_10connection_AuthenticationRequest, (void *)&__pyx_v_13aioesphomeapi_10connection_AuthenticationResponse, (void *)&__pyx_v_13aioesphomeapi_10connection_BadNameAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECT_AND_SETUP_TIMEOUT, (void *)&__pyx_v_13aioesphomeapi_10connection_CONNECT_REQUEST_TIMEOUT, (void *)&__pyx_v_13aioesphomeapi_10connection_CancelledError, (void *)&__pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE, (void *)&__pyx_v_13aioesphomeapi_10connection_DISCONNECT_RESPONSE_MESSAGES, (void *)&__pyx_v_13aioesphomeapi_10connection_DisconnectRequest, (void *)&__pyx_v_13aioesphomeapi_10connection_GetTimeRequest, (void *)&__pyx_v_13aioesphomeapi_10connection_GetTimeResponse, (void *)&__pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT, (void *)&__pyx_v_13aioesphomeapi_10connection_HandshakeAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_HelloRequest, (void *)&__pyx_v_13aioesphomeapi_10connection_HelloResponse, (void *)&__pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO, (void *)&__pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO, (void *)&__pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST, (void *)&__pyx_v_13aioesphomeapi_10connection_OPEN_STATES, (void *)&__pyx_v_13aioesphomeapi_10connection_PING_REQUEST_MESSAGES, (void *)&__pyx_v_13aioesphomeapi_10connection_PING_RESPONSE_MESSAGES, (void *)&__pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE, (void *)&__pyx_v_13aioesphomeapi_10connection_PingFailedAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_PingRequest, (void *)&__pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_SocketAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError, (void *)&__pyx_v_13aioesphomeapi_10connection_WRITE_EXCEPTIONS, (void *)&__pyx_v_13aioesphomeapi_10connection__handle_complex_message, (void *)&__pyx_v_13aioesphomeapi_10connection__handle_timeout, (void *)&__pyx_v_13aioesphomeapi_10connection_astuple, (void *)&__pyx_v_13aioesphomeapi_10connection_asyncio_timeout, (void *)&__pyx_v_13aioesphomeapi_10connection_hr, (void *)&__pyx_v_13aioesphomeapi_10connection_make_hello_request, (void *)&__pyx_v_13aioesphomeapi_10connection_partial, (void *)&__pyx_v_13aioesphomeapi_10connection_KEEP_ALIVE_TIMEOUT_RATIO, (void *)&__pyx_v_13aioesphomeapi_10connection_TYPE_CHECKING, (void *)&__pyx_v_13aioesphomeapi_10connection__WIN32, (void *) NULL};
    void *const *__pyx_export_pointer = __pyx_export_pointers;
    const char *__pyx_export_current_signature = __pyx_export_signature;
    while (*__pyx_export_pointer) {
      if (__Pyx_ExportVoidPtr(__pyx_t_1, __pyx_export_name, *__pyx_export_pointer, __pyx_export_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_export_pointer;
      __pyx_export_name = strchr(__pyx_export_name, '\0') + 1;
      __pyx_export_signature = strchr(__pyx_export_signature, '\0') + 1;
      if (*__pyx_export_signature != '\0') __pyx_export_current_signature = __pyx_export_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  {
    __pyx_t_1 = __Pyx_ApiExport_GetApiDict(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_export_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_void_PyObject_PyObject_PyObject);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_export_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_export_name = __pyx_export_signature + 134;
    void (*const __pyx_export_pointers[])(void) = {(void (*)(void))&__pyx_f_13aioesphomeapi_10connection_handle_complex_message, (void (*)(void))&__pyx_f_13aioesphomeapi_10connection_handle_timeout, (void (*)(void)) NULL};
    void (*const *__pyx_export_pointer)(void) = __pyx_export_pointers;
    const char *__pyx_export_current_signature = __pyx_export_signature;
    while (*__pyx_export_pointer) {
      if (__Pyx_ExportFunction(__pyx_t_1, __pyx_export_name, *__pyx_export_pointer, __pyx_export_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_export_pointer;
      __pyx_export_name = strchr(__pyx_export_name, '\0') + 1;
      __pyx_export_signature = strchr(__pyx_export_signature, '\0') + 1;
      if (*__pyx_export_signature != '\0') __pyx_export_current_signature = __pyx_export_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection_ConnectionParams_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection_ConnectionParams_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams = &__pyx_type_13aioesphomeapi_10connection_ConnectionParams;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_ConnectionParams, (PyObject *) __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_vtabptr_13aioesphomeapi_10connection_APIConnection = &__pyx_vtable_13aioesphomeapi_10connection_APIConnection;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection.send_message = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection_send_message;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection.send_messages = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection_send_messages;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection.process_packet = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, unsigned int, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection_process_packet;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._async_cancel_pong_timer = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__async_cancel_pong_timer;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._async_schedule_keep_alive = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__async_schedule_keep_alive;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._cleanup = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__cleanup;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection.set_log_name = (PyObject *(*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection_set_log_name;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._make_auth_request = (PyObject *(*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__make_auth_request;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._process_hello_resp = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__process_hello_resp;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._set_connection_state = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__set_connection_state;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection.report_fatal_error = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection_report_fatal_error;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._add_message_callback_without_remove = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, PyObject *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__add_message_callback_without_remove;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection.add_message_callback = (PyObject *(*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection_add_message_callback;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._remove_message_callback = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection__remove_message_callback;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._handle_disconnect_request_internal = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_disconnect_request_internal;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._handle_ping_request_internal = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_ping_request_internal;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._handle_get_time_request_internal = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *, int __pyx_skip_dispatch))__pyx_f_13aioesphomeapi_10connection_13APIConnection__handle_get_time_request_internal;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._set_fatal_exception_if_unset = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *, PyObject *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__set_fatal_exception_if_unset;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._register_internal_message_handlers = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__register_internal_message_handlers;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._increase_recv_buffer_size = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__increase_recv_buffer_size;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._set_start_connect_future = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__set_start_connect_future;
  __pyx_vtable_13aioesphomeapi_10connection_APIConnection._set_finish_connect_future = (void (*)(struct __pyx_obj_13aioesphomeapi_10connection_APIConnection *))__pyx_f_13aioesphomeapi_10connection_13APIConnection__set_finish_connect_future;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection_APIConnection_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection)) __PYX_ERR(0, 240, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection_APIConnection_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection) < (0)) __PYX_ERR(0, 240, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection = &__pyx_type_13aioesphomeapi_10connection_APIConnection;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection) < (0)) __PYX_ERR(0, 240, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_vtabptr_13aioesphomeapi_10connection_APIConnection) < (0)) __PYX_ERR(0, 240, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection) < (0)) __PYX_ERR(0, 240, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_APIConnection, (PyObject *) __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection) < (0)) __PYX_ERR(0, 240, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection_APIConnection) < (0)) __PYX_ERR(0, 240, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr)) __PYX_ERR(0, 68, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct__genexpr;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr) < (0)) __PYX_ERR(0, 68, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct__genexpr->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect)) __PYX_ERR(0, 386, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect) < (0)) __PYX_ERR(0, 386, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect) < (0)) __PYX_ERR(0, 386, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_1__connect_socket_connect->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr)) __PYX_ERR(0, 392, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr) < (0)) __PYX_ERR(0, 392, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr) < (0)) __PYX_ERR(0, 392, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_2_genexpr->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper)) __PYX_ERR(0, 481, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_3__connect_init_frame_helper->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login)) __PYX_ERR(0, 529, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login) < (0)) __PYX_ERR(0, 529, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login) < (0)) __PYX_ERR(0, 529, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_4__connect_hello_login->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host)) __PYX_ERR(0, 649, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host) < (0)) __PYX_ERR(0, 649, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host) < (0)) __PYX_ERR(0, 649, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_5_start_resolve_host->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection)) __PYX_ERR(0, 687, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection) < (0)) __PYX_ERR(0, 687, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection) < (0)) __PYX_ERR(0, 687, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_6_start_connection->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect)) __PYX_ERR(0, 768, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect) < (0)) __PYX_ERR(0, 768, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect) < (0)) __PYX_ERR(0, 768, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_7__do_finish_connect->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection)) __PYX_ERR(0, 783, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection) < (0)) __PYX_ERR(0, 783, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection) < (0)) __PYX_ERR(0, 783, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_8_finish_connection->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex)) __PYX_ERR(0, 917, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_9_send_messages_await_response_complex->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response)) __PYX_ERR(0, 972, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_10_send_message_await_response->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect)) __PYX_ERR(0, 1151, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect_spec, __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect) < (0)) __PYX_ERR(0, 1151, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect = &__pyx_type_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect) < (0)) __PYX_ERR(0, 1151, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect->tp_dictoffset && __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_13aioesphomeapi_10connection___pyx_scope_struct_11_disconnect->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("aioesphomeapi._frame_helper.base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper = __Pyx_ImportType_3_2_4(__pyx_t_1, "aioesphomeapi._frame_helper.base", "APIFrameHelper",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper), __PYX_GET_STRUCT_ALIGNMENT_3_2_4(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  sizeof(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper), __PYX_GET_STRUCT_ALIGNMENT_3_2_4(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper),
  #else
  sizeof(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper), __PYX_GET_STRUCT_ALIGNMENT_3_2_4(struct __pyx_obj_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_4); if (!__pyx_mstate->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper) __PYX_ERR(3, 9, __pyx_L1_error)
  __pyx_vtabptr_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper = (struct __pyx_vtabstruct_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper*)__Pyx_GetVtable(__pyx_mstate->__pyx_ptype_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper); if (unlikely(!__pyx_vtabptr_13aioesphomeapi_13_frame_helper_4base_APIFrameHelper)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  {
    __pyx_t_1 = PyImport_ImportModule("aioesphomeapi._frame_helper.base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    const char * __pyx_import_signature = __Pyx_PyBytes_AsString(__pyx_mstate_global->__pyx_kp_b_int_TYPE_CHECKING);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (unlikely(!__pyx_import_signature)) __PYX_ERR(0, 1, __pyx_L1_error)
    #endif
    const char * __pyx_import_name = __pyx_import_signature + 4;
    void **const __pyx_import_pointers[] = {(void **)&__pyx_vp_13aioesphomeapi_13_frame_helper_4base_TYPE_CHECKING, (void **) NULL};
    void **const *__pyx_import_pointer = __pyx_import_pointers;
    const char *__pyx_import_current_signature = __pyx_import_signature;
    while (*__pyx_import_pointer) {
      if (__Pyx_ImportVoidPtr_3_2_4(__pyx_t_1, __pyx_import_name, *__pyx_import_pointer, __pyx_import_current_signature) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
      ++__pyx_import_pointer;
      __pyx_import_name = strchr(__pyx_import_name, '\0') + 1;
      __pyx_import_signature = strchr(__pyx_import_signature, '\0') + 1;
      if (*__pyx_import_signature != '\0') __pyx_import_current_signature = __pyx_import_signature;
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_connection(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_connection},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "connection",
      0, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit_connection(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_connection(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_connection(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  int __pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'connection' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "connection" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_NOT_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit_connection", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_aioesphomeapi__connection) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "aioesphomeapi.connection")) {
      if (unlikely((PyDict_SetItemString(modules, "aioesphomeapi.connection", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_variable_export_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_function_export_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_variable_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code(__pyx_mstate);
  /*--- Execution code ---*/

  /* "aioesphomeapi/connection.py":3
 * from __future__ import annotations
 * 
 * import asyncio             # <<<<<<<<<<<<<<
 * from asyncio import CancelledError
 * from collections.abc import Callable
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_asyncio, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_asyncio, __pyx_t_2) < (0)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":4
 * 
 * import asyncio
 * from asyncio import CancelledError             # <<<<<<<<<<<<<<
 * from collections.abc import Callable
 * from dataclasses import astuple, dataclass
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CancelledError};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_asyncio, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 4, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_CancelledError};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 4, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CancelledError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CancelledError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":5
 * import asyncio
 * from asyncio import CancelledError
 * from collections.abc import Callable             # <<<<<<<<<<<<<<
 * from dataclasses import astuple, dataclass
 * import enum
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Callable};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_collections_abc, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 5, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_Callable};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 5, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":6
 * from asyncio import CancelledError
 * from collections.abc import Callable
 * from dataclasses import astuple, dataclass             # <<<<<<<<<<<<<<
 * import enum
 * from functools import lru_cache, partial
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_astuple,__pyx_mstate_global->__pyx_n_u_dataclass};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_dataclasses, __pyx_imported_names, 2, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_astuple,__pyx_mstate_global->__pyx_n_u_dataclass};
    for (__pyx_t_3=0; __pyx_t_3 < 2; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 6, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_astuple);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_astuple, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 6, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":7
 * from collections.abc import Callable
 * from dataclasses import astuple, dataclass
 * import enum             # <<<<<<<<<<<<<<
 * from functools import lru_cache, partial
 * import logging
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_enum, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_enum, __pyx_t_2) < (0)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":8
 * from dataclasses import astuple, dataclass
 * import enum
 * from functools import lru_cache, partial             # <<<<<<<<<<<<<<
 * import logging
 * import socket
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_lru_cache,__pyx_mstate_global->__pyx_n_u_partial};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_functools, __pyx_imported_names, 2, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 8, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_lru_cache,__pyx_mstate_global->__pyx_n_u_partial};
    for (__pyx_t_3=0; __pyx_t_3 < 2; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 8, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 1:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_partial);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_partial, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 8, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":9
 * import enum
 * from functools import lru_cache, partial
 * import logging             # <<<<<<<<<<<<<<
 * import socket
 * import sys
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_logging, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 9, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_logging, __pyx_t_2) < (0)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":10
 * from functools import lru_cache, partial
 * import logging
 * import socket             # <<<<<<<<<<<<<<
 * import sys
 * import time
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_socket, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 10, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_socket, __pyx_t_2) < (0)) __PYX_ERR(0, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":11
 * import logging
 * import socket
 * import sys             # <<<<<<<<<<<<<<
 * import time
 * from typing import TYPE_CHECKING, Any, NoReturn
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_sys, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 11, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_sys, __pyx_t_2) < (0)) __PYX_ERR(0, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":12
 * import socket
 * import sys
 * import time             # <<<<<<<<<<<<<<
 * from typing import TYPE_CHECKING, Any, NoReturn
 * 
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_time, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 12, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_time, __pyx_t_2) < (0)) __PYX_ERR(0, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":13
 * import sys
 * import time
 * from typing import TYPE_CHECKING, Any, NoReturn             # <<<<<<<<<<<<<<
 * 
 * import aiohappyeyeballs
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_TYPE_CHECKING,__pyx_mstate_global->__pyx_n_u_Any,__pyx_mstate_global->__pyx_n_u_NoReturn};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_typing, __pyx_imported_names, 3, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 13, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_TYPE_CHECKING,__pyx_mstate_global->__pyx_n_u_Any,__pyx_mstate_global->__pyx_n_u_NoReturn};
    for (__pyx_t_3=0; __pyx_t_3 < 3; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 13, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 13, __pyx_L1_error)
        __pyx_v_13aioesphomeapi_10connection_TYPE_CHECKING = __pyx_t_5;
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 13, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":15
 * from typing import TYPE_CHECKING, Any, NoReturn
 * 
 * import aiohappyeyeballs             # <<<<<<<<<<<<<<
 * from async_interrupt import interrupt
 * from google.protobuf import message
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_aiohappyeyeballs, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 15, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_aiohappyeyeballs, __pyx_t_2) < (0)) __PYX_ERR(0, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":16
 * 
 * import aiohappyeyeballs
 * from async_interrupt import interrupt             # <<<<<<<<<<<<<<
 * from google.protobuf import message
 * from google.protobuf.json_format import MessageToDict
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_interrupt};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_async_interrupt, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 16, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_interrupt};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 16, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 16, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":17
 * import aiohappyeyeballs
 * from async_interrupt import interrupt
 * from google.protobuf import message             # <<<<<<<<<<<<<<
 * from google.protobuf.json_format import MessageToDict
 * 
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_message};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_google_protobuf, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_message};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 17, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 17, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":18
 * from async_interrupt import interrupt
 * from google.protobuf import message
 * from google.protobuf.json_format import MessageToDict             # <<<<<<<<<<<<<<
 * 
 * import aioesphomeapi.host_resolver as hr
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_MessageToDict};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_google_protobuf_json_format, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_MessageToDict};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 18, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 18, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":20
 * from google.protobuf.json_format import MessageToDict
 * 
 * import aioesphomeapi.host_resolver as hr             # <<<<<<<<<<<<<<
 * 
 * from ._frame_helper.noise import APINoiseFrameHelper
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_aioesphomeapi_host_resolver, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_t_6 = __Pyx_ImportFrom(__pyx_t_1, __pyx_mstate_global->__pyx_n_u_host_resolver);
  Py_DECREF(__pyx_t_1);
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_t_1 = __pyx_t_6;
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_hr);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_hr, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":22
 * import aioesphomeapi.host_resolver as hr
 * 
 * from ._frame_helper.noise import APINoiseFrameHelper             # <<<<<<<<<<<<<<
 * from ._frame_helper.plain_text import APIPlaintextFrameHelper
 * from .api_pb2 import (  # type: ignore  # type: ignore
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_APINoiseFrameHelper};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_frame_helper_noise, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi__frame_helper_nois, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_APINoiseFrameHelper};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":23
 * 
 * from ._frame_helper.noise import APINoiseFrameHelper
 * from ._frame_helper.plain_text import APIPlaintextFrameHelper             # <<<<<<<<<<<<<<
 * from .api_pb2 import (  # type: ignore  # type: ignore
 *     DST_RULE_TYPE_DAY_OF_YEAR as DST_RULE_TYPE_DAY_OF_YEAR_PB,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_APIPlaintextFrameHelper};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_frame_helper_plain_text, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi__frame_helper_plai, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_APIPlaintextFrameHelper};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 23, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 23, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":24
 * from ._frame_helper.noise import APINoiseFrameHelper
 * from ._frame_helper.plain_text import APIPlaintextFrameHelper
 * from .api_pb2 import (  # type: ignore  # type: ignore             # <<<<<<<<<<<<<<
 *     DST_RULE_TYPE_DAY_OF_YEAR as DST_RULE_TYPE_DAY_OF_YEAR_PB,
 *     DST_RULE_TYPE_JULIAN_NO_LEAP as DST_RULE_TYPE_JULIAN_NO_LEAP_PB,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_DAY_OF_YEAR,__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_JULIAN_NO_LEAP,__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_MONTH_WEEK_DAY,__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_NONE,__pyx_mstate_global->__pyx_n_u_AuthenticationRequest,__pyx_mstate_global->__pyx_n_u_AuthenticationResponse,__pyx_mstate_global->__pyx_n_u_DisconnectRequest,__pyx_mstate_global->__pyx_n_u_DisconnectResponse,__pyx_mstate_global->__pyx_n_u_DSTRule,__pyx_mstate_global->__pyx_n_u_GetTimeRequest,__pyx_mstate_global->__pyx_n_u_GetTimeResponse,__pyx_mstate_global->__pyx_n_u_HelloRequest,__pyx_mstate_global->__pyx_n_u_HelloResponse,__pyx_mstate_global->__pyx_n_u_ParsedTimezone,__pyx_mstate_global->__pyx_n_u_PingRequest,__pyx_mstate_global->__pyx_n_u_PingResponse};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_api_pb2, __pyx_imported_names, 16, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_api_pb2, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_DAY_OF_YEAR,__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_JULIAN_NO_LEAP,__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_MONTH_WEEK_DAY,__pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_NONE,__pyx_mstate_global->__pyx_n_u_AuthenticationRequest,__pyx_mstate_global->__pyx_n_u_AuthenticationResponse,__pyx_mstate_global->__pyx_n_u_DisconnectRequest,__pyx_mstate_global->__pyx_n_u_DisconnectResponse,__pyx_mstate_global->__pyx_n_u_DSTRule,__pyx_mstate_global->__pyx_n_u_GetTimeRequest,__pyx_mstate_global->__pyx_n_u_GetTimeResponse,__pyx_mstate_global->__pyx_n_u_HelloRequest,__pyx_mstate_global->__pyx_n_u_HelloResponse,__pyx_mstate_global->__pyx_n_u_ParsedTimezone,__pyx_mstate_global->__pyx_n_u_PingRequest,__pyx_mstate_global->__pyx_n_u_PingResponse};
    for (__pyx_t_3=0; __pyx_t_3 < 16; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 24, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_DAY_OF_YEAR_PB, __pyx_t_4) < (0)) __PYX_ERR(0, 25, __pyx_L1_error)
        break;
        case 1:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_JULIAN_NO_LEAP_PB, __pyx_t_4) < (0)) __PYX_ERR(0, 26, __pyx_L1_error)
        break;
        case 2:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_MONTH_WEEK_DAY_PB, __pyx_t_4) < (0)) __PYX_ERR(0, 27, __pyx_L1_error)
        break;
        case 3:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_NONE_PB, __pyx_t_4) < (0)) __PYX_ERR(0, 28, __pyx_L1_error)
        break;
        case 4:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_AuthenticationRequest);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_AuthenticationRequest, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 5:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_AuthenticationResponse);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_AuthenticationResponse, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 6:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_DisconnectRequest);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_DisconnectRequest, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 8:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DSTRuleProto, __pyx_t_4) < (0)) __PYX_ERR(0, 33, __pyx_L1_error)
        break;
        case 9:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_GetTimeRequest);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_GetTimeRequest, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 10:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_GetTimeResponse);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_GetTimeResponse, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 11:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_HelloRequest);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_HelloRequest, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 12:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_HelloResponse);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_HelloResponse, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 13:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ParsedTimezoneProto, __pyx_t_4) < (0)) __PYX_ERR(0, 38, __pyx_L1_error)
        break;
        case 14:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_PingRequest);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_PingRequest, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 24, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":42
 *     PingResponse,
 * )
 * from .core import (             # <<<<<<<<<<<<<<
 *     MESSAGE_TYPE_TO_PROTO,
 *     APIConnectionCancelledError,
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_MESSAGE_TYPE_TO_PROTO,__pyx_mstate_global->__pyx_n_u_APIConnectionCancelledError,__pyx_mstate_global->__pyx_n_u_APIConnectionError,__pyx_mstate_global->__pyx_n_u_BadNameAPIError,__pyx_mstate_global->__pyx_n_u_ConnectionNotEstablishedAPIError,__pyx_mstate_global->__pyx_n_u_HandshakeAPIError,__pyx_mstate_global->__pyx_n_u_InvalidAuthAPIError,__pyx_mstate_global->__pyx_n_u_PingFailedAPIError,__pyx_mstate_global->__pyx_n_u_ProtocolAPIError,__pyx_mstate_global->__pyx_n_u_ReadFailedAPIError,__pyx_mstate_global->__pyx_n_u_SocketAPIError,__pyx_mstate_global->__pyx_n_u_SocketClosedAPIError,__pyx_mstate_global->__pyx_n_u_TimeoutAPIError,__pyx_mstate_global->__pyx_n_u_UnhandledAPIConnectionError};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_core, __pyx_imported_names, 14, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_core, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_MESSAGE_TYPE_TO_PROTO,__pyx_mstate_global->__pyx_n_u_APIConnectionCancelledError,__pyx_mstate_global->__pyx_n_u_APIConnectionError,__pyx_mstate_global->__pyx_n_u_BadNameAPIError,__pyx_mstate_global->__pyx_n_u_ConnectionNotEstablishedAPIError,__pyx_mstate_global->__pyx_n_u_HandshakeAPIError,__pyx_mstate_global->__pyx_n_u_InvalidAuthAPIError,__pyx_mstate_global->__pyx_n_u_PingFailedAPIError,__pyx_mstate_global->__pyx_n_u_ProtocolAPIError,__pyx_mstate_global->__pyx_n_u_ReadFailedAPIError,__pyx_mstate_global->__pyx_n_u_SocketAPIError,__pyx_mstate_global->__pyx_n_u_SocketClosedAPIError,__pyx_mstate_global->__pyx_n_u_TimeoutAPIError,__pyx_mstate_global->__pyx_n_u_UnhandledAPIConnectionError};
    for (__pyx_t_3=0; __pyx_t_3 < 14; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 42, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 2:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_APIConnectionError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_APIConnectionError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 3:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_BadNameAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_BadNameAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 5:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_HandshakeAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_HandshakeAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 6:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_InvalidAuthAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 7:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_PingFailedAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_PingFailedAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 9:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_ReadFailedAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 10:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_SocketAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_SocketAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 11:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_SocketClosedAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 12:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_TimeoutAPIError, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        case 0:
        __pyx_t_7 = __pyx_t_4;
        __Pyx_INCREF(__pyx_t_7);
        if (!(likely(PyDict_CheckExact(__pyx_t_7))||((__pyx_t_7) == Py_None) || __Pyx_RaiseUnexpectedTypeError("dict", __pyx_t_7))) __PYX_ERR(0, 42, __pyx_L1_error)
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO, ((PyObject*)__pyx_t_7));
        __Pyx_GIVEREF(__pyx_t_7);
        __pyx_t_7 = 0;
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 42, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":58
 *     UnhandledAPIConnectionError,
 * )
 * from .model import APIVersion, message_types_to_names             # <<<<<<<<<<<<<<
 * from .posix_tz import DSTRule as DSTRuleParsed, DSTRuleType, parse_posix_tz
 * from .timezone import get_timezone
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_APIVersion,__pyx_mstate_global->__pyx_n_u_message_types_to_names};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_model, __pyx_imported_names, 2, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_model, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_APIVersion,__pyx_mstate_global->__pyx_n_u_message_types_to_names};
    for (__pyx_t_3=0; __pyx_t_3 < 2; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 58, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_APIVersion);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_APIVersion, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 58, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":59
 * )
 * from .model import APIVersion, message_types_to_names
 * from .posix_tz import DSTRule as DSTRuleParsed, DSTRuleType, parse_posix_tz             # <<<<<<<<<<<<<<
 * from .timezone import get_timezone
 * from .util import asyncio_timeout
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_DSTRule,__pyx_mstate_global->__pyx_n_u_DSTRuleType,__pyx_mstate_global->__pyx_n_u_parse_posix_tz};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_posix_tz, __pyx_imported_names, 3, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_posix_tz, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 59, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_DSTRule,__pyx_mstate_global->__pyx_n_u_DSTRuleType,__pyx_mstate_global->__pyx_n_u_parse_posix_tz};
    for (__pyx_t_3=0; __pyx_t_3 < 3; __pyx_t_3++) {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 59, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DSTRuleParsed, __pyx_t_4) < (0)) __PYX_ERR(0, 59, __pyx_L1_error)
        break;
        default:;
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 59, __pyx_L1_error)
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":60
 * from .model import APIVersion, message_types_to_names
 * from .posix_tz import DSTRule as DSTRuleParsed, DSTRuleType, parse_posix_tz
 * from .timezone import get_timezone             # <<<<<<<<<<<<<<
 * from .util import asyncio_timeout
 * from .zeroconf import ZeroconfManager
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_get_timezone};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_timezone, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_timezone, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 60, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_get_timezone};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 60, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 60, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":61
 * from .posix_tz import DSTRule as DSTRuleParsed, DSTRuleType, parse_posix_tz
 * from .timezone import get_timezone
 * from .util import asyncio_timeout             # <<<<<<<<<<<<<<
 * from .zeroconf import ZeroconfManager
 * 
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_asyncio_timeout};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_util, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_util, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 61, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_asyncio_timeout};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 61, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_asyncio_timeout);
        __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_asyncio_timeout, __pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_4);
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":62
 * from .timezone import get_timezone
 * from .util import asyncio_timeout
 * from .zeroconf import ZeroconfManager             # <<<<<<<<<<<<<<
 * 
 * _LOGGER = logging.getLogger(__name__)
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_ZeroconfManager};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_zeroconf, __pyx_imported_names, 1, __pyx_mstate_global->__pyx_kp_u_aioesphomeapi_zeroconf, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 62, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_ZeroconfManager};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 62, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_imported_names[__pyx_t_3], __pyx_t_4) < (0)) __PYX_ERR(0, 62, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":64
 * from .zeroconf import ZeroconfManager
 * 
 * _LOGGER = logging.getLogger(__name__)             # <<<<<<<<<<<<<<
 * 
 * MESSAGE_NUMBER_TO_PROTO: tuple[
*/
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_logging); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_getLogger); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_7};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 64, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_LOGGER, __pyx_t_2) < (0)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":68
 * MESSAGE_NUMBER_TO_PROTO: tuple[
 *     tuple[Callable[[], message.Message], Callable[[message.Message, bytes], None]], ...
 * ] = tuple((msg, msg.MergeFromString) for msg in MESSAGE_TYPE_TO_PROTO.values())             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_2 = __pyx_pf_13aioesphomeapi_10connection_10genexpr(NULL, __pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PySequence_Tuple(__pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_MESSAGE_NUMBER_TO_PROTO, ((PyObject*)__pyx_t_8));
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":71
 * 
 * 
 * PREFERRED_BUFFER_SIZE = 2097152  # Set buffer limit to 2MB             # <<<<<<<<<<<<<<
 * MIN_BUFFER_SIZE = 131072  # Minimum buffer size to use
 * 
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_PREFERRED_BUFFER_SIZE, __pyx_mstate_global->__pyx_int_2097152) < (0)) __PYX_ERR(0, 71, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":72
 * 
 * PREFERRED_BUFFER_SIZE = 2097152  # Set buffer limit to 2MB
 * MIN_BUFFER_SIZE = 131072  # Minimum buffer size to use             # <<<<<<<<<<<<<<
 * 
 * DISCONNECT_REQUEST_MESSAGE = DisconnectRequest()
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_MIN_BUFFER_SIZE, __pyx_mstate_global->__pyx_int_131072) < (0)) __PYX_ERR(0, 72, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":74
 * MIN_BUFFER_SIZE = 131072  # Minimum buffer size to use
 * 
 * DISCONNECT_REQUEST_MESSAGE = DisconnectRequest()             # <<<<<<<<<<<<<<
 * DISCONNECT_RESPONSE_MESSAGES = (DisconnectResponse(),)
 * PING_REQUEST_MESSAGES = (PingRequest(),)
*/
  __pyx_t_2 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_DisconnectRequest);
  __pyx_t_7 = __pyx_v_13aioesphomeapi_10connection_DisconnectRequest; 
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 74, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_DISCONNECT_REQUEST_MESSAGE, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":75
 * 
 * DISCONNECT_REQUEST_MESSAGE = DisconnectRequest()
 * DISCONNECT_RESPONSE_MESSAGES = (DisconnectResponse(),)             # <<<<<<<<<<<<<<
 * PING_REQUEST_MESSAGES = (PingRequest(),)
 * PING_RESPONSE_MESSAGES = (PingResponse(),)
*/
  __pyx_t_7 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_DisconnectResponse); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 75, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __pyx_t_2 = PyTuple_Pack(1, __pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_DISCONNECT_RESPONSE_MESSAGES);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_DISCONNECT_RESPONSE_MESSAGES, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":76
 * DISCONNECT_REQUEST_MESSAGE = DisconnectRequest()
 * DISCONNECT_RESPONSE_MESSAGES = (DisconnectResponse(),)
 * PING_REQUEST_MESSAGES = (PingRequest(),)             # <<<<<<<<<<<<<<
 * PING_RESPONSE_MESSAGES = (PingResponse(),)
 * NO_PASSWORD_AUTH_REQUEST = AuthenticationRequest()
*/
  __pyx_t_8 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_PingRequest);
  __pyx_t_7 = __pyx_v_13aioesphomeapi_10connection_PingRequest; 
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_7 = PyTuple_Pack(1, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_PING_REQUEST_MESSAGES);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_PING_REQUEST_MESSAGES, ((PyObject*)__pyx_t_7));
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":77
 * DISCONNECT_RESPONSE_MESSAGES = (DisconnectResponse(),)
 * PING_REQUEST_MESSAGES = (PingRequest(),)
 * PING_RESPONSE_MESSAGES = (PingResponse(),)             # <<<<<<<<<<<<<<
 * NO_PASSWORD_AUTH_REQUEST = AuthenticationRequest()
 * 
*/
  __pyx_t_2 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_PingResponse); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __pyx_t_8 = PyTuple_Pack(1, __pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_PING_RESPONSE_MESSAGES);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_PING_RESPONSE_MESSAGES, ((PyObject*)__pyx_t_8));
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":78
 * PING_REQUEST_MESSAGES = (PingRequest(),)
 * PING_RESPONSE_MESSAGES = (PingResponse(),)
 * NO_PASSWORD_AUTH_REQUEST = AuthenticationRequest()             # <<<<<<<<<<<<<<
 * 
 * PROTO_TO_MESSAGE_TYPE: dict[
*/
  __pyx_t_7 = NULL;
  __Pyx_INCREF(__pyx_v_13aioesphomeapi_10connection_AuthenticationRequest);
  __pyx_t_2 = __pyx_v_13aioesphomeapi_10connection_AuthenticationRequest; 
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_7, NULL};
    __pyx_t_8 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_2, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 78, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
  }
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_NO_PASSWORD_AUTH_REQUEST, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":82
 * PROTO_TO_MESSAGE_TYPE: dict[
 *     type[message.Message], tuple[int, Callable[[message.Message], bytes]]
 * ] = {v: (k, v.SerializeToString) for k, v in MESSAGE_TYPE_TO_PROTO.items()}             # <<<<<<<<<<<<<<
 * 
 * KEEP_ALIVE_TIMEOUT_RATIO = 4.5
*/
  { /* enter inner scope */
    __pyx_t_8 = PyDict_New(); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 82, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = 0;
    if (unlikely(__pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "items");
      __PYX_ERR(0, 82, __pyx_L4_error)
    }
    __pyx_t_7 = __Pyx_dict_iterator(__pyx_v_13aioesphomeapi_10connection_MESSAGE_TYPE_TO_PROTO, 1, __pyx_mstate_global->__pyx_n_u_items, (&__pyx_t_10), (&__pyx_t_11)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 82, __pyx_L4_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_XDECREF(__pyx_t_2);
    __pyx_t_2 = __pyx_t_7;
    __pyx_t_7 = 0;
    while (1) {
      __pyx_t_12 = __Pyx_dict_iter_next(__pyx_t_2, __pyx_t_10, &__pyx_t_3, &__pyx_t_7, &__pyx_t_4, NULL, __pyx_t_11);
      if (unlikely(__pyx_t_12 == 0)) break;
      if (unlikely(__pyx_t_12 == -1)) __PYX_ERR(0, 82, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k);
      __Pyx_DECREF_SET(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k, __pyx_t_7);
      __Pyx_GIVEREF(__pyx_t_7);
      __pyx_t_7 = 0;
      __Pyx_XGOTREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v);
      __Pyx_DECREF_SET(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v, __pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_4);
      __pyx_t_4 = 0;
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v, __pyx_mstate_global->__pyx_n_u_SerializeToString); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 82, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 82, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_INCREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k);
      __Pyx_GIVEREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k) != (0)) __PYX_ERR(0, 82, __pyx_L4_error);
      __Pyx_GIVEREF(__pyx_t_4);
      if (__Pyx_PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_4) != (0)) __PYX_ERR(0, 82, __pyx_L4_error);
      __pyx_t_4 = 0;
      if (unlikely(PyDict_SetItem(__pyx_t_8, (PyObject*)__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v, (PyObject*)__pyx_t_7))) __PYX_ERR(0, 82, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_GOTREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k);
    __Pyx_DECREF_SET(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k, Py_None);
    __Pyx_GOTREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v);
    __Pyx_DECREF_SET(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v, Py_None);
    goto __pyx_L7_exit_scope;
    __pyx_L4_error:;
    __Pyx_GOTREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k);
    __Pyx_DECREF_SET(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_k, Py_None);
    __Pyx_GOTREF(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v);
    __Pyx_DECREF_SET(__pyx_8genexpr1__pyx_v_13aioesphomeapi_10connection_v, Py_None);
    goto __pyx_L1_error;
    __pyx_L7_exit_scope:;
  } /* exit inner scope */
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_PROTO_TO_MESSAGE_TYPE, ((PyObject*)__pyx_t_8));
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":84
 * ] = {v: (k, v.SerializeToString) for k, v in MESSAGE_TYPE_TO_PROTO.items()}
 * 
 * KEEP_ALIVE_TIMEOUT_RATIO = 4.5             # <<<<<<<<<<<<<<
 * #
 * # We use 4.5x the keep-alive time as the timeout for the pong
*/
  __pyx_v_13aioesphomeapi_10connection_KEEP_ALIVE_TIMEOUT_RATIO = 4.5;

  /* "aioesphomeapi/connection.py":94
 * #
 * 
 * DISCONNECT_CONNECT_TIMEOUT = 5.0             # <<<<<<<<<<<<<<
 * # How long to wait for an existing connection to finish being
 * # setup when requesting a disconnect so we can try to disconnect
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DISCONNECT_CONNECT_TIMEOUT, __pyx_mstate_global->__pyx_float_5_0) < (0)) __PYX_ERR(0, 94, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":100
 * # the esp device
 * 
 * DISCONNECT_RESPONSE_TIMEOUT = 10.0             # <<<<<<<<<<<<<<
 * # Timeout for the noise handshake to complete. 60s is intentionally high
 * # because on ESP8266 with power_save_mode: LIGHT and weak WiFi (-70 dBm+),
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DISCONNECT_RESPONSE_TIMEOUT, __pyx_mstate_global->__pyx_float_10_0) < (0)) __PYX_ERR(0, 100, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":107
 * # handshake times of 28-30s have been observed due to retransmissions.
 * # See https://github.com/esphome/esphome/issues/14999
 * HANDSHAKE_TIMEOUT = 60.0             # <<<<<<<<<<<<<<
 * CONNECT_REQUEST_TIMEOUT = 30.0
 * 
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_float_60_0);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_HANDSHAKE_TIMEOUT, __pyx_mstate_global->__pyx_float_60_0);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_float_60_0);

  /* "aioesphomeapi/connection.py":108
 * # See https://github.com/esphome/esphome/issues/14999
 * HANDSHAKE_TIMEOUT = 60.0
 * CONNECT_REQUEST_TIMEOUT = 30.0             # <<<<<<<<<<<<<<
 * 
 * # The connect timeout should be the maximum time we expect the esp to take
*/
  __Pyx_INCREF(__pyx_mstate_global->__pyx_float_30_0);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECT_REQUEST_TIMEOUT);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECT_REQUEST_TIMEOUT, __pyx_mstate_global->__pyx_float_30_0);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_float_30_0);

  /* "aioesphomeapi/connection.py":112
 * # The connect timeout should be the maximum time we expect the esp to take
 * # to reboot and connect to the network/WiFi.
 * TCP_CONNECT_TIMEOUT = 60.0             # <<<<<<<<<<<<<<
 * 
 * WRITE_EXCEPTIONS = (RuntimeError, ConnectionResetError, OSError)
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_TCP_CONNECT_TIMEOUT, __pyx_mstate_global->__pyx_float_60_0) < (0)) __PYX_ERR(0, 112, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":114
 * TCP_CONNECT_TIMEOUT = 60.0
 * 
 * WRITE_EXCEPTIONS = (RuntimeError, ConnectionResetError, OSError)             # <<<<<<<<<<<<<<
 * 
 * _WIN32 = sys.platform == "win32"
*/
  __pyx_t_8 = PyTuple_Pack(3, ((PyObject *)(((PyTypeObject*)PyExc_RuntimeError))), ((PyObject *)(((PyTypeObject*)PyExc_ConnectionResetError))), ((PyObject *)(((PyTypeObject*)PyExc_OSError)))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_WRITE_EXCEPTIONS);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_WRITE_EXCEPTIONS, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":116
 * WRITE_EXCEPTIONS = (RuntimeError, ConnectionResetError, OSError)
 * 
 * _WIN32 = sys.platform == "win32"             # <<<<<<<<<<<<<<
 * 
 * _int = int
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_sys); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_platform); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = PyObject_RichCompare(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_win32, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_v_13aioesphomeapi_10connection__WIN32 = __pyx_t_5;

  /* "aioesphomeapi/connection.py":118
 * _WIN32 = sys.platform == "win32"
 * 
 * _int = int             # <<<<<<<<<<<<<<
 * _bytes = bytes
 * _float = float
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_int, ((PyObject *)(&PyLong_Type))) < (0)) __PYX_ERR(0, 118, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":119
 * 
 * _int = int
 * _bytes = bytes             # <<<<<<<<<<<<<<
 * _float = float
 * 
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_bytes, ((PyObject *)(&PyBytes_Type))) < (0)) __PYX_ERR(0, 119, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":120
 * _int = int
 * _bytes = bytes
 * _float = float             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_float, ((PyObject *)(&PyFloat_Type))) < (0)) __PYX_ERR(0, 120, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":123
 * 
 * 
 * class ConnectionInterruptedError(Exception):             # <<<<<<<<<<<<<<
 *     """An error that is raised when a connection is interrupted."""
 * 
*/
  __pyx_t_8 = PyTuple_Pack(1, ((PyObject *)(((PyTypeObject*)PyExc_Exception)))); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_7, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_kp_u_An_error_that_is_raised_when_a_c); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_2 != __pyx_t_8) {
    if (unlikely((PyDict_SetItemString(__pyx_t_4, "__orig_bases__", __pyx_t_8) < 0))) __PYX_ERR(0, 123, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_Py3ClassCreate(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError, __pyx_t_2, __pyx_t_4, NULL, 0, 0); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_8);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ConnectionInterruptedError, __pyx_t_8) < (0)) __PYX_ERR(0, 123, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":128
 * 
 * @dataclass
 * class ConnectionParams:             # <<<<<<<<<<<<<<
 *     addresses: list[str]
 *     port: int
*/
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_DataclassParams); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_eq, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_order, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_unsafe_hash, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_frozen, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_kw_only, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_match_args, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_kw_only, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_slots, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_weakref_slot, Py_False) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_4 = __Pyx_DataclassesCallHelper(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_params, __pyx_t_4) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_8 = __Pyx_DataclassesCallHelper(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_addresses, __pyx_t_8) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_7 = __Pyx_DataclassesCallHelper(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_port, __pyx_t_7) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_2 = __Pyx_DataclassesCallHelper(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_password, __pyx_t_2) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_8 = __Pyx_DataclassesCallHelper(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_client_info, __pyx_t_8) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_7 = __Pyx_DataclassesCallHelper(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_keepalive, __pyx_t_7) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_2 = __Pyx_DataclassesCallHelper(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_zeroconf_manager, __pyx_t_2) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_8 = __Pyx_DataclassesCallHelper(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_noise_psk, __pyx_t_8) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_7 = __Pyx_DataclassesCallHelper(__pyx_t_2, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_expected_name, __pyx_t_7) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __pyx_t_2 = __Pyx_DataclassesCallHelper(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_expected_mac, __pyx_t_2) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_repr, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_hash, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_init, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_compare, Py_True) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_metadata, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":138
 *     expected_name: str | None
 *     expected_mac: str | None
 *     timezone: str | None = None             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_default, Py_None) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":128
 * 
 * @dataclass
 * class ConnectionParams:             # <<<<<<<<<<<<<<
 *     addresses: list[str]
 *     port: int
*/
  __pyx_t_8 = __Pyx_DataclassesCallHelper(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_timezone, __pyx_t_8) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields, __pyx_t_4) < (0)) __PYX_ERR(0, 128, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * __dataclass_fields__['addresses'].name = 'addresses'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['addresses'].type = PLACEHOLDER_addresses
 * __dataclass_fields__['addresses']._field_type = PLACEHOLDER_FIELD_TYPE_addresses
*/
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_addresses); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_addresses) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":2
 * __dataclass_fields__['addresses'].name = 'addresses'
 * __dataclass_fields__['addresses'].type = PLACEHOLDER_addresses             # <<<<<<<<<<<<<<
 * __dataclass_fields__['addresses']._field_type = PLACEHOLDER_FIELD_TYPE_addresses
 * __dataclass_fields__['port'].name = 'port'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_addresses); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_kp_u_list_str) < (0)) __PYX_ERR(1, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":3
 * __dataclass_fields__['addresses'].name = 'addresses'
 * __dataclass_fields__['addresses'].type = PLACEHOLDER_addresses
 * __dataclass_fields__['addresses']._field_type = PLACEHOLDER_FIELD_TYPE_addresses             # <<<<<<<<<<<<<<
 * __dataclass_fields__['port'].name = 'port'
 * __dataclass_fields__['port'].type = PLACEHOLDER_port
*/
  __pyx_t_4 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_addresses); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":4
 * __dataclass_fields__['addresses'].type = PLACEHOLDER_addresses
 * __dataclass_fields__['addresses']._field_type = PLACEHOLDER_FIELD_TYPE_addresses
 * __dataclass_fields__['port'].name = 'port'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['port'].type = PLACEHOLDER_port
 * __dataclass_fields__['port']._field_type = PLACEHOLDER_FIELD_TYPE_port
*/
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_port); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_port) < (0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":5
 * __dataclass_fields__['addresses']._field_type = PLACEHOLDER_FIELD_TYPE_addresses
 * __dataclass_fields__['port'].name = 'port'
 * __dataclass_fields__['port'].type = PLACEHOLDER_port             # <<<<<<<<<<<<<<
 * __dataclass_fields__['port']._field_type = PLACEHOLDER_FIELD_TYPE_port
 * __dataclass_fields__['password'].name = 'password'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_port); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_n_u_int_2) < (0)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":6
 * __dataclass_fields__['port'].name = 'port'
 * __dataclass_fields__['port'].type = PLACEHOLDER_port
 * __dataclass_fields__['port']._field_type = PLACEHOLDER_FIELD_TYPE_port             # <<<<<<<<<<<<<<
 * __dataclass_fields__['password'].name = 'password'
 * __dataclass_fields__['password'].type = PLACEHOLDER_password
*/
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_port); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":7
 * __dataclass_fields__['port'].type = PLACEHOLDER_port
 * __dataclass_fields__['port']._field_type = PLACEHOLDER_FIELD_TYPE_port
 * __dataclass_fields__['password'].name = 'password'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['password'].type = PLACEHOLDER_password
 * __dataclass_fields__['password']._field_type = PLACEHOLDER_FIELD_TYPE_password
*/
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_password); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_password) < (0)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":8
 * __dataclass_fields__['port']._field_type = PLACEHOLDER_FIELD_TYPE_port
 * __dataclass_fields__['password'].name = 'password'
 * __dataclass_fields__['password'].type = PLACEHOLDER_password             # <<<<<<<<<<<<<<
 * __dataclass_fields__['password']._field_type = PLACEHOLDER_FIELD_TYPE_password
 * __dataclass_fields__['client_info'].name = 'client_info'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_password); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_kp_u_str_None) < (0)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":9
 * __dataclass_fields__['password'].name = 'password'
 * __dataclass_fields__['password'].type = PLACEHOLDER_password
 * __dataclass_fields__['password']._field_type = PLACEHOLDER_FIELD_TYPE_password             # <<<<<<<<<<<<<<
 * __dataclass_fields__['client_info'].name = 'client_info'
 * __dataclass_fields__['client_info'].type = PLACEHOLDER_client_info
*/
  __pyx_t_4 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_password); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":10
 * __dataclass_fields__['password'].type = PLACEHOLDER_password
 * __dataclass_fields__['password']._field_type = PLACEHOLDER_FIELD_TYPE_password
 * __dataclass_fields__['client_info'].name = 'client_info'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['client_info'].type = PLACEHOLDER_client_info
 * __dataclass_fields__['client_info']._field_type = PLACEHOLDER_FIELD_TYPE_client_info
*/
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_client_info); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_client_info) < (0)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":11
 * __dataclass_fields__['password']._field_type = PLACEHOLDER_FIELD_TYPE_password
 * __dataclass_fields__['client_info'].name = 'client_info'
 * __dataclass_fields__['client_info'].type = PLACEHOLDER_client_info             # <<<<<<<<<<<<<<
 * __dataclass_fields__['client_info']._field_type = PLACEHOLDER_FIELD_TYPE_client_info
 * __dataclass_fields__['keepalive'].name = 'keepalive'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_client_info); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_n_u_str) < (0)) __PYX_ERR(1, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":12
 * __dataclass_fields__['client_info'].name = 'client_info'
 * __dataclass_fields__['client_info'].type = PLACEHOLDER_client_info
 * __dataclass_fields__['client_info']._field_type = PLACEHOLDER_FIELD_TYPE_client_info             # <<<<<<<<<<<<<<
 * __dataclass_fields__['keepalive'].name = 'keepalive'
 * __dataclass_fields__['keepalive'].type = PLACEHOLDER_keepalive
*/
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_client_info); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":13
 * __dataclass_fields__['client_info'].type = PLACEHOLDER_client_info
 * __dataclass_fields__['client_info']._field_type = PLACEHOLDER_FIELD_TYPE_client_info
 * __dataclass_fields__['keepalive'].name = 'keepalive'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['keepalive'].type = PLACEHOLDER_keepalive
 * __dataclass_fields__['keepalive']._field_type = PLACEHOLDER_FIELD_TYPE_keepalive
*/
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_keepalive); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_keepalive) < (0)) __PYX_ERR(1, 13, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":14
 * __dataclass_fields__['client_info']._field_type = PLACEHOLDER_FIELD_TYPE_client_info
 * __dataclass_fields__['keepalive'].name = 'keepalive'
 * __dataclass_fields__['keepalive'].type = PLACEHOLDER_keepalive             # <<<<<<<<<<<<<<
 * __dataclass_fields__['keepalive']._field_type = PLACEHOLDER_FIELD_TYPE_keepalive
 * __dataclass_fields__['zeroconf_manager'].name = 'zeroconf_manager'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_keepalive); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_n_u_float_2) < (0)) __PYX_ERR(1, 14, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":15
 * __dataclass_fields__['keepalive'].name = 'keepalive'
 * __dataclass_fields__['keepalive'].type = PLACEHOLDER_keepalive
 * __dataclass_fields__['keepalive']._field_type = PLACEHOLDER_FIELD_TYPE_keepalive             # <<<<<<<<<<<<<<
 * __dataclass_fields__['zeroconf_manager'].name = 'zeroconf_manager'
 * __dataclass_fields__['zeroconf_manager'].type = PLACEHOLDER_zeroconf_manager
*/
  __pyx_t_4 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_keepalive); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 * __dataclass_fields__['keepalive'].type = PLACEHOLDER_keepalive
 * __dataclass_fields__['keepalive']._field_type = PLACEHOLDER_FIELD_TYPE_keepalive
 * __dataclass_fields__['zeroconf_manager'].name = 'zeroconf_manager'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['zeroconf_manager'].type = PLACEHOLDER_zeroconf_manager
 * __dataclass_fields__['zeroconf_manager']._field_type = PLACEHOLDER_FIELD_TYPE_zeroconf_manager
*/
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_zeroconf_manager); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_zeroconf_manager) < (0)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":17
 * __dataclass_fields__['keepalive']._field_type = PLACEHOLDER_FIELD_TYPE_keepalive
 * __dataclass_fields__['zeroconf_manager'].name = 'zeroconf_manager'
 * __dataclass_fields__['zeroconf_manager'].type = PLACEHOLDER_zeroconf_manager             # <<<<<<<<<<<<<<
 * __dataclass_fields__['zeroconf_manager']._field_type = PLACEHOLDER_FIELD_TYPE_zeroconf_manager
 * __dataclass_fields__['noise_psk'].name = 'noise_psk'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_zeroconf_manager); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_n_u_ZeroconfManager) < (0)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":18
 * __dataclass_fields__['zeroconf_manager'].name = 'zeroconf_manager'
 * __dataclass_fields__['zeroconf_manager'].type = PLACEHOLDER_zeroconf_manager
 * __dataclass_fields__['zeroconf_manager']._field_type = PLACEHOLDER_FIELD_TYPE_zeroconf_manager             # <<<<<<<<<<<<<<
 * __dataclass_fields__['noise_psk'].name = 'noise_psk'
 * __dataclass_fields__['noise_psk'].type = PLACEHOLDER_noise_psk
*/
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_zeroconf_manager); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":19
 * __dataclass_fields__['zeroconf_manager'].type = PLACEHOLDER_zeroconf_manager
 * __dataclass_fields__['zeroconf_manager']._field_type = PLACEHOLDER_FIELD_TYPE_zeroconf_manager
 * __dataclass_fields__['noise_psk'].name = 'noise_psk'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['noise_psk'].type = PLACEHOLDER_noise_psk
 * __dataclass_fields__['noise_psk']._field_type = PLACEHOLDER_FIELD_TYPE_noise_psk
*/
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_noise_psk); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_noise_psk) < (0)) __PYX_ERR(1, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":20
 * __dataclass_fields__['zeroconf_manager']._field_type = PLACEHOLDER_FIELD_TYPE_zeroconf_manager
 * __dataclass_fields__['noise_psk'].name = 'noise_psk'
 * __dataclass_fields__['noise_psk'].type = PLACEHOLDER_noise_psk             # <<<<<<<<<<<<<<
 * __dataclass_fields__['noise_psk']._field_type = PLACEHOLDER_FIELD_TYPE_noise_psk
 * __dataclass_fields__['expected_name'].name = 'expected_name'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_noise_psk); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_kp_u_str_None) < (0)) __PYX_ERR(1, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":21
 * __dataclass_fields__['noise_psk'].name = 'noise_psk'
 * __dataclass_fields__['noise_psk'].type = PLACEHOLDER_noise_psk
 * __dataclass_fields__['noise_psk']._field_type = PLACEHOLDER_FIELD_TYPE_noise_psk             # <<<<<<<<<<<<<<
 * __dataclass_fields__['expected_name'].name = 'expected_name'
 * __dataclass_fields__['expected_name'].type = PLACEHOLDER_expected_name
*/
  __pyx_t_4 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_noise_psk); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 21, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":22
 * __dataclass_fields__['noise_psk'].type = PLACEHOLDER_noise_psk
 * __dataclass_fields__['noise_psk']._field_type = PLACEHOLDER_FIELD_TYPE_noise_psk
 * __dataclass_fields__['expected_name'].name = 'expected_name'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['expected_name'].type = PLACEHOLDER_expected_name
 * __dataclass_fields__['expected_name']._field_type = PLACEHOLDER_FIELD_TYPE_expected_name
*/
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_expected_name); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_expected_name) < (0)) __PYX_ERR(1, 22, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":23
 * __dataclass_fields__['noise_psk']._field_type = PLACEHOLDER_FIELD_TYPE_noise_psk
 * __dataclass_fields__['expected_name'].name = 'expected_name'
 * __dataclass_fields__['expected_name'].type = PLACEHOLDER_expected_name             # <<<<<<<<<<<<<<
 * __dataclass_fields__['expected_name']._field_type = PLACEHOLDER_FIELD_TYPE_expected_name
 * __dataclass_fields__['expected_mac'].name = 'expected_mac'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_expected_name); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_kp_u_str_None) < (0)) __PYX_ERR(1, 23, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":24
 * __dataclass_fields__['expected_name'].name = 'expected_name'
 * __dataclass_fields__['expected_name'].type = PLACEHOLDER_expected_name
 * __dataclass_fields__['expected_name']._field_type = PLACEHOLDER_FIELD_TYPE_expected_name             # <<<<<<<<<<<<<<
 * __dataclass_fields__['expected_mac'].name = 'expected_mac'
 * __dataclass_fields__['expected_mac'].type = PLACEHOLDER_expected_mac
*/
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_expected_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":25
 * __dataclass_fields__['expected_name'].type = PLACEHOLDER_expected_name
 * __dataclass_fields__['expected_name']._field_type = PLACEHOLDER_FIELD_TYPE_expected_name
 * __dataclass_fields__['expected_mac'].name = 'expected_mac'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['expected_mac'].type = PLACEHOLDER_expected_mac
 * __dataclass_fields__['expected_mac']._field_type = PLACEHOLDER_FIELD_TYPE_expected_mac
*/
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_expected_mac); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_expected_mac) < (0)) __PYX_ERR(1, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":26
 * __dataclass_fields__['expected_name']._field_type = PLACEHOLDER_FIELD_TYPE_expected_name
 * __dataclass_fields__['expected_mac'].name = 'expected_mac'
 * __dataclass_fields__['expected_mac'].type = PLACEHOLDER_expected_mac             # <<<<<<<<<<<<<<
 * __dataclass_fields__['expected_mac']._field_type = PLACEHOLDER_FIELD_TYPE_expected_mac
 * __dataclass_fields__['timezone'].name = 'timezone'
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_expected_mac); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_kp_u_str_None) < (0)) __PYX_ERR(1, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":27
 * __dataclass_fields__['expected_mac'].name = 'expected_mac'
 * __dataclass_fields__['expected_mac'].type = PLACEHOLDER_expected_mac
 * __dataclass_fields__['expected_mac']._field_type = PLACEHOLDER_FIELD_TYPE_expected_mac             # <<<<<<<<<<<<<<
 * __dataclass_fields__['timezone'].name = 'timezone'
 * __dataclass_fields__['timezone'].type = PLACEHOLDER_timezone
*/
  __pyx_t_4 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_GetNameInClass(__pyx_t_4, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_expected_mac); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":28
 * __dataclass_fields__['expected_mac'].type = PLACEHOLDER_expected_mac
 * __dataclass_fields__['expected_mac']._field_type = PLACEHOLDER_FIELD_TYPE_expected_mac
 * __dataclass_fields__['timezone'].name = 'timezone'             # <<<<<<<<<<<<<<
 * __dataclass_fields__['timezone'].type = PLACEHOLDER_timezone
 * __dataclass_fields__['timezone']._field_type = PLACEHOLDER_FIELD_TYPE_timezone
*/
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_timezone); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_timezone) < (0)) __PYX_ERR(1, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "(tree fragment)":29
 * __dataclass_fields__['expected_mac']._field_type = PLACEHOLDER_FIELD_TYPE_expected_mac
 * __dataclass_fields__['timezone'].name = 'timezone'
 * __dataclass_fields__['timezone'].type = PLACEHOLDER_timezone             # <<<<<<<<<<<<<<
 * __dataclass_fields__['timezone']._field_type = PLACEHOLDER_FIELD_TYPE_timezone
*/
  __Pyx_GetNameInClass(__pyx_t_8, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_Dict_GetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_timezone); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_type, __pyx_mstate_global->__pyx_n_u_object) < (0)) __PYX_ERR(1, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":30
 * __dataclass_fields__['timezone'].name = 'timezone'
 * __dataclass_fields__['timezone'].type = PLACEHOLDER_timezone
 * __dataclass_fields__['timezone']._field_type = PLACEHOLDER_FIELD_TYPE_timezone             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_Load_dataclasses_Module(); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_FIELD); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_GetNameInClass(__pyx_t_2, (PyObject*)__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_dataclass_fields); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Dict_GetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_timezone); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_PyObject_SetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_field_type, __pyx_t_8) < (0)) __PYX_ERR(1, 30, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_16ConnectionParams_7__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ConnectionParams___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ConnectionParams, (type(self), 0xb91db3b, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ConnectionParams__set_state(self, __pyx_state)
*/
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_16ConnectionParams_9__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_ConnectionParams___setstate_cyth, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_ConnectionParams, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_4) < (0)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "aioesphomeapi/connection.py":141
 * 
 * 
 * class ConnectionState(enum.Enum):             # <<<<<<<<<<<<<<
 *     # The connection is initialized, but connect() wasn't called yet
 *     INITIALIZED = 0
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_enum); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_Enum); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = PyTuple_Pack(1, __pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __pyx_t_8 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_8); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_8, __pyx_mstate_global->__pyx_n_u_ConnectionState, __pyx_mstate_global->__pyx_n_u_ConnectionState, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, (PyObject *) NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_8 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_7, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 141, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "aioesphomeapi/connection.py":143
 * class ConnectionState(enum.Enum):
 *     # The connection is initialized, but connect() wasn't called yet
 *     INITIALIZED = 0             # <<<<<<<<<<<<<<
 *     # The host has been resolved, but the socket hasn't been opened yet
 *     HOST_RESOLVED = 1
*/
  if (__Pyx_SetNameInClass(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_INITIALIZED, __pyx_mstate_global->__pyx_int_0) < (0)) __PYX_ERR(0, 143, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":145
 *     INITIALIZED = 0
 *     # The host has been resolved, but the socket hasn't been opened yet
 *     HOST_RESOLVED = 1             # <<<<<<<<<<<<<<
 *     # The socket has been opened, but the handshake and login haven't been completed
 *     SOCKET_OPENED = 2
*/
  if (__Pyx_SetNameInClass(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HOST_RESOLVED, __pyx_mstate_global->__pyx_int_1) < (0)) __PYX_ERR(0, 145, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":147
 *     HOST_RESOLVED = 1
 *     # The socket has been opened, but the handshake and login haven't been completed
 *     SOCKET_OPENED = 2             # <<<<<<<<<<<<<<
 *     # The handshake has been completed, messages can be exchanged
 *     HANDSHAKE_COMPLETE = 3
*/
  if (__Pyx_SetNameInClass(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_SOCKET_OPENED, __pyx_mstate_global->__pyx_int_2) < (0)) __PYX_ERR(0, 147, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":149
 *     SOCKET_OPENED = 2
 *     # The handshake has been completed, messages can be exchanged
 *     HANDSHAKE_COMPLETE = 3             # <<<<<<<<<<<<<<
 *     # The connection has been established, authenticated data can be exchanged
 *     CONNECTED = 4
*/
  if (__Pyx_SetNameInClass(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_HANDSHAKE_COMPLETE, __pyx_mstate_global->__pyx_int_3) < (0)) __PYX_ERR(0, 149, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":151
 *     HANDSHAKE_COMPLETE = 3
 *     # The connection has been established, authenticated data can be exchanged
 *     CONNECTED = 4             # <<<<<<<<<<<<<<
 *     CLOSED = 5
 * 
*/
  if (__Pyx_SetNameInClass(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_CONNECTED, __pyx_mstate_global->__pyx_int_4) < (0)) __PYX_ERR(0, 151, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":152
 *     # The connection has been established, authenticated data can be exchanged
 *     CONNECTED = 4
 *     CLOSED = 5             # <<<<<<<<<<<<<<
 * 
 * 
*/
  if (__Pyx_SetNameInClass(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_CLOSED, __pyx_mstate_global->__pyx_int_5) < (0)) __PYX_ERR(0, 152, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":141
 * 
 * 
 * class ConnectionState(enum.Enum):             # <<<<<<<<<<<<<<
 *     # The connection is initialized, but connect() wasn't called yet
 *     INITIALIZED = 0
*/
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ConnectionState, __pyx_t_8, __pyx_t_7, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_ConnectionState, __pyx_t_4) < (0)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":155
 * 
 * 
 * CONNECTION_STATE_INITIALIZED = ConnectionState.INITIALIZED             # <<<<<<<<<<<<<<
 * CONNECTION_STATE_HOST_RESOLVED = ConnectionState.HOST_RESOLVED
 * CONNECTION_STATE_SOCKET_OPENED = ConnectionState.SOCKET_OPENED
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ConnectionState); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_INITIALIZED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_INITIALIZED, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":156
 * 
 * CONNECTION_STATE_INITIALIZED = ConnectionState.INITIALIZED
 * CONNECTION_STATE_HOST_RESOLVED = ConnectionState.HOST_RESOLVED             # <<<<<<<<<<<<<<
 * CONNECTION_STATE_SOCKET_OPENED = ConnectionState.SOCKET_OPENED
 * CONNECTION_STATE_HANDSHAKE_COMPLETE = ConnectionState.HANDSHAKE_COMPLETE
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ConnectionState); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HOST_RESOLVED); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 156, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HOST_RESOLVED, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":157
 * CONNECTION_STATE_INITIALIZED = ConnectionState.INITIALIZED
 * CONNECTION_STATE_HOST_RESOLVED = ConnectionState.HOST_RESOLVED
 * CONNECTION_STATE_SOCKET_OPENED = ConnectionState.SOCKET_OPENED             # <<<<<<<<<<<<<<
 * CONNECTION_STATE_HANDSHAKE_COMPLETE = ConnectionState.HANDSHAKE_COMPLETE
 * CONNECTION_STATE_CONNECTED = ConnectionState.CONNECTED
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ConnectionState); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_SOCKET_OPENED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 157, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_SOCKET_OPENED, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":158
 * CONNECTION_STATE_HOST_RESOLVED = ConnectionState.HOST_RESOLVED
 * CONNECTION_STATE_SOCKET_OPENED = ConnectionState.SOCKET_OPENED
 * CONNECTION_STATE_HANDSHAKE_COMPLETE = ConnectionState.HANDSHAKE_COMPLETE             # <<<<<<<<<<<<<<
 * CONNECTION_STATE_CONNECTED = ConnectionState.CONNECTED
 * CONNECTION_STATE_CLOSED = ConnectionState.CLOSED
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ConnectionState); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_HANDSHAKE_COMPLETE); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 158, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_HANDSHAKE_COMPLETE, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":159
 * CONNECTION_STATE_SOCKET_OPENED = ConnectionState.SOCKET_OPENED
 * CONNECTION_STATE_HANDSHAKE_COMPLETE = ConnectionState.HANDSHAKE_COMPLETE
 * CONNECTION_STATE_CONNECTED = ConnectionState.CONNECTED             # <<<<<<<<<<<<<<
 * CONNECTION_STATE_CLOSED = ConnectionState.CLOSED
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_ConnectionState); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_CONNECTED); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CONNECTED, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":160
 * CONNECTION_STATE_HANDSHAKE_COMPLETE = ConnectionState.HANDSHAKE_COMPLETE
 * CONNECTION_STATE_CONNECTED = ConnectionState.CONNECTED
 * CONNECTION_STATE_CLOSED = ConnectionState.CLOSED             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_ConnectionState); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_CLOSED); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_CONNECTION_STATE_CLOSED, __pyx_t_8);
  __Pyx_GIVEREF(__pyx_t_8);
  __pyx_t_8 = 0;

  /* "aioesphomeapi/connection.py":163
 * 
 * 
 * def _make_hello_request(client_info: str) -> HelloRequest:             # <<<<<<<<<<<<<<
 *     """Make a HelloRequest."""
 *     return HelloRequest(
*/
  __pyx_t_8 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_client_info, __pyx_mstate_global->__pyx_n_u_str) < (0)) __PYX_ERR(0, 163, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_8, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_HelloRequest) < (0)) __PYX_ERR(0, 163, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_1_make_hello_request, 0, __pyx_mstate_global->__pyx_n_u_make_hello_request, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_2, __pyx_t_8);
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_make_hello_request, __pyx_t_2) < (0)) __PYX_ERR(0, 163, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":170
 * 
 * 
 * _cached_make_hello_request = lru_cache(maxsize=16)(_make_hello_request)             # <<<<<<<<<<<<<<
 * make_hello_request = _cached_make_hello_request
 * 
*/
  __pyx_t_8 = NULL;
  __pyx_t_4 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_lru_cache); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_4, NULL};
    __pyx_t_14 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_14);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_maxsize, __pyx_mstate_global->__pyx_int_16, __pyx_t_14, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 170, __pyx_L1_error)
    __pyx_t_7 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_14);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_make_hello_request); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_8, __pyx_t_13};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 170, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_cached_make_hello_request, __pyx_t_2) < (0)) __PYX_ERR(0, 170, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":171
 * 
 * _cached_make_hello_request = lru_cache(maxsize=16)(_make_hello_request)
 * make_hello_request = _cached_make_hello_request             # <<<<<<<<<<<<<<
 * 
 * _DST_RULE_TYPE_MAP: dict[DSTRuleType, int] = {
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_cached_make_hello_request); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection_make_hello_request);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection_make_hello_request, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":174
 * 
 * _DST_RULE_TYPE_MAP: dict[DSTRuleType, int] = {
 *     DSTRuleType.NONE: DST_RULE_TYPE_NONE_PB,             # <<<<<<<<<<<<<<
 *     DSTRuleType.MONTH_WEEK_DAY: DST_RULE_TYPE_MONTH_WEEK_DAY_PB,
 *     DSTRuleType.JULIAN_NO_LEAP: DST_RULE_TYPE_JULIAN_NO_LEAP_PB,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DSTRuleType); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_NONE); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_NONE_PB); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_2, __pyx_t_13, __pyx_t_7) < (0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":175
 * _DST_RULE_TYPE_MAP: dict[DSTRuleType, int] = {
 *     DSTRuleType.NONE: DST_RULE_TYPE_NONE_PB,
 *     DSTRuleType.MONTH_WEEK_DAY: DST_RULE_TYPE_MONTH_WEEK_DAY_PB,             # <<<<<<<<<<<<<<
 *     DSTRuleType.JULIAN_NO_LEAP: DST_RULE_TYPE_JULIAN_NO_LEAP_PB,
 *     DSTRuleType.DAY_OF_YEAR: DST_RULE_TYPE_DAY_OF_YEAR_PB,
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DSTRuleType); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_MONTH_WEEK_DAY); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_MONTH_WEEK_DAY_PB); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_2, __pyx_t_13, __pyx_t_7) < (0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":176
 *     DSTRuleType.NONE: DST_RULE_TYPE_NONE_PB,
 *     DSTRuleType.MONTH_WEEK_DAY: DST_RULE_TYPE_MONTH_WEEK_DAY_PB,
 *     DSTRuleType.JULIAN_NO_LEAP: DST_RULE_TYPE_JULIAN_NO_LEAP_PB,             # <<<<<<<<<<<<<<
 *     DSTRuleType.DAY_OF_YEAR: DST_RULE_TYPE_DAY_OF_YEAR_PB,
 * }
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DSTRuleType); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_JULIAN_NO_LEAP); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_JULIAN_NO_LEAP_PB); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 176, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_2, __pyx_t_13, __pyx_t_7) < (0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":177
 *     DSTRuleType.MONTH_WEEK_DAY: DST_RULE_TYPE_MONTH_WEEK_DAY_PB,
 *     DSTRuleType.JULIAN_NO_LEAP: DST_RULE_TYPE_JULIAN_NO_LEAP_PB,
 *     DSTRuleType.DAY_OF_YEAR: DST_RULE_TYPE_DAY_OF_YEAR_PB,             # <<<<<<<<<<<<<<
 * }
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DSTRuleType); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_13 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DAY_OF_YEAR); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_DAY_OF_YEAR_PB); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 177, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_2, __pyx_t_13, __pyx_t_7) < (0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_DST_RULE_TYPE_MAP, __pyx_t_2) < (0)) __PYX_ERR(0, 173, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "aioesphomeapi/connection.py":181
 * 
 * 
 * def _build_dst_rule_proto(rule: DSTRuleParsed) -> DSTRuleProto:             # <<<<<<<<<<<<<<
 *     """Build a protobuf DSTRule from a parsed DSTRule."""
 *     return DSTRuleProto(
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_rule, __pyx_mstate_global->__pyx_n_u_DSTRuleParsed) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_DSTRuleProto) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_3_build_dst_rule_proto, 0, __pyx_mstate_global->__pyx_n_u_build_dst_rule_proto, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_build_dst_rule_proto, __pyx_t_7) < (0)) __PYX_ERR(0, 181, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":193
 * 
 * 
 * @lru_cache(maxsize=8)             # <<<<<<<<<<<<<<
 * def _build_parsed_tz_proto(tz_string: str) -> ParsedTimezoneProto | None:
 *     """Build a protobuf ParsedTimezone from a POSIX TZ string.
*/
  __pyx_t_2 = NULL;
  __pyx_t_8 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_lru_cache); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_8, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_maxsize, __pyx_mstate_global->__pyx_int_8, __pyx_t_4, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 193, __pyx_L1_error)
    __pyx_t_13 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_t_14, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
    if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_13);
  }
  __pyx_t_14 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_14)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_14);
  if (PyDict_SetItem(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_tz_string, __pyx_mstate_global->__pyx_n_u_str) < (0)) __PYX_ERR(0, 193, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_14, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_kp_u_ParsedTimezoneProto_None) < (0)) __PYX_ERR(0, 193, __pyx_L1_error)
  __pyx_t_4 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_5_build_parsed_tz_proto, 0, __pyx_mstate_global->__pyx_n_u_build_parsed_tz_proto, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_4, __pyx_t_14);
  __Pyx_DECREF(__pyx_t_14); __pyx_t_14 = 0;
  __pyx_t_9 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_4};
    __pyx_t_7 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_13, __pyx_callargs+__pyx_t_9, (2-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 193, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
  }
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_build_parsed_tz_proto, __pyx_t_7) < (0)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":213
 * 
 * 
 * def handle_timeout(fut: asyncio.Future[None]) -> None:             # <<<<<<<<<<<<<<
 *     """Handle a timeout."""
 *     if not fut.done():
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fut, __pyx_mstate_global->__pyx_kp_u_asyncio_Future_None) < (0)) __PYX_ERR(0, 213, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_7handle_timeout, 0, __pyx_mstate_global->__pyx_n_u_handle_timeout, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_handle_timeout, __pyx_t_13) < (0)) __PYX_ERR(0, 213, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":219
 * 
 * 
 * _handle_timeout = handle_timeout             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_handle_timeout); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 219, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection__handle_timeout);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection__handle_timeout, __pyx_t_13);
  __Pyx_GIVEREF(__pyx_t_13);
  __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":222
 * 
 * 
 * def handle_complex_message(             # <<<<<<<<<<<<<<
 *     fut: asyncio.Future[None],
 *     responses: list[message.Message],
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_fut, __pyx_mstate_global->__pyx_kp_u_asyncio_Future_None) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_responses, __pyx_mstate_global->__pyx_kp_u_list_message_Message) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_do_append, __pyx_mstate_global->__pyx_kp_u_Callable_message_Message_bool_No) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_do_stop, __pyx_mstate_global->__pyx_kp_u_Callable_message_Message_bool_No) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_resp, __pyx_mstate_global->__pyx_kp_u_message_Message) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_9handle_complex_message, 0, __pyx_mstate_global->__pyx_n_u_handle_complex_message, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_handle_complex_message, __pyx_t_7) < (0)) __PYX_ERR(0, 222, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":237
 * 
 * 
 * _handle_complex_message = handle_complex_message             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_handle_complex_message); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_XGOTREF(__pyx_v_13aioesphomeapi_10connection__handle_complex_message);
  __Pyx_DECREF_SET(__pyx_v_13aioesphomeapi_10connection__handle_complex_message, __pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_7);
  __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":250
 * 
 *     __slots__ = (
 *         "_addrs_info",             # <<<<<<<<<<<<<<
 *         "_cached_timezone",
 *         "_debug_enabled",
*/
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_slots_2, __pyx_mstate_global->__pyx_tuple[1]) < (0)) __PYX_ERR(0, 249, __pyx_L1_error)

  /* "aioesphomeapi/connection.py":326
 *         self._log_errors = log_errors
 * 
 *     def set_log_name(self, name: str) -> None:             # <<<<<<<<<<<<<<
 *         """Set the friendly log name for this connection."""
 *         self.log_name = name
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_str) < (0)) __PYX_ERR(0, 326, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_3set_log_name, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_set_log_name, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_set_log_name, __pyx_t_13) < (0)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":382
 *             on_stop(self._expected_disconnect)
 * 
 *     def set_debug(self, enable: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Enable or disable debug logging."""
 *         self._debug_enabled = enable
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_enable, __pyx_mstate_global->__pyx_n_u_bool) < (0)) __PYX_ERR(0, 382, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 382, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_5set_debug, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_set_debug, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_set_debug, __pyx_t_7) < (0)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":386
 *         self._debug_enabled = enable
 * 
 *     async def _connect_socket_connect(self, addrs: list[hr.AddrInfo]) -> None:             # <<<<<<<<<<<<<<
 *         """Step 2 in connect process: connect the socket."""
 *         if self._debug_enabled:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_addrs, __pyx_mstate_global->__pyx_kp_u_list_hr_AddrInfo) < (0)) __PYX_ERR(0, 386, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 386, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_7_connect_socket_connect, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection__connect_socket_co, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_connect_socket_connect, __pyx_t_13) < (0)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":481
 *                 return
 * 
 *     async def _connect_init_frame_helper(self) -> None:             # <<<<<<<<<<<<<<
 *         """Step 3 in connect process: initialize the frame helper and init read loop."""
 *         fh: APIPlaintextFrameHelper | APINoiseFrameHelper
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_10_connect_init_frame_helper, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection__connect_init_fram, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_connect_init_frame_helper, __pyx_t_7) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":529
 *         self._set_connection_state(CONNECTION_STATE_HANDSHAKE_COMPLETE)
 * 
 *     async def _connect_hello_login(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Step 4 in connect process: send hello and login and get api version."""
 *         messages = [make_hello_request(self._params.client_info)]
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_login, __pyx_mstate_global->__pyx_n_u_bool) < (0)) __PYX_ERR(0, 529, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 529, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_13_connect_hello_login, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection__connect_hello_log, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_connect_hello_login, __pyx_t_13) < (0)) __PYX_ERR(0, 529, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":553
 *         self._process_hello_resp(resp)
 * 
 *     def _handle_login_response(self, login_response: AuthenticationResponse) -> None:             # <<<<<<<<<<<<<<
 *         """Handle a AuthenticationResponse."""
 *         if login_response.invalid_password:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_login_response, __pyx_mstate_global->__pyx_n_u_AuthenticationResponse) < (0)) __PYX_ERR(0, 553, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 553, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_16_handle_login_response, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__handle_login_resp, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_handle_login_response, __pyx_t_7) < (0)) __PYX_ERR(0, 553, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":597
 *         )
 * 
 *     def _async_send_keep_alive(self) -> None:             # <<<<<<<<<<<<<<
 *         """Send a keep alive message."""
 *         now = self._loop.time()
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 597, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_18_async_send_keep_alive, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__async_send_keep_a, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_async_send_keep_alive, __pyx_t_13) < (0)) __PYX_ERR(0, 597, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":635
 *             self._pong_timer = None
 * 
 *     def _async_pong_not_received(self) -> None:             # <<<<<<<<<<<<<<
 *         """Ping not received."""
 *         if self._debug_enabled:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 635, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_20_async_pong_not_received, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__async_pong_not_re, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_async_pong_not_received, __pyx_t_7) < (0)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":649
 *         )
 * 
 *     async def start_resolve_host(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the host resolution process.
 * 
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 649, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_22start_resolve_host, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection_start_resolve_host, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_start_resolve_host, __pyx_t_13) < (0)) __PYX_ERR(0, 649, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":679
 *         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)
 * 
 *     def _set_resolve_host_future(self) -> None:             # <<<<<<<<<<<<<<
 *         if (
 *             self._resolve_host_future is not None
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 679, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_25_set_resolve_host_future, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__set_resolve_host, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_set_resolve_host_future, __pyx_t_7) < (0)) __PYX_ERR(0, 679, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":687
 *             self._resolve_host_future = None
 * 
 *     async def start_connection(self) -> None:             # <<<<<<<<<<<<<<
 *         """Start the connection process.
 * 
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 687, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_27start_connection, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection_start_connection, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_start_connection, __pyx_t_13) < (0)) __PYX_ERR(0, 687, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":721
 *             self._start_connect_future = None
 * 
 *     def _raise_fatal_connection_exception(             # <<<<<<<<<<<<<<
 *         self, action: str, ex: BaseException
 *     ) -> NoReturn:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_action, __pyx_mstate_global->__pyx_n_u_str) < (0)) __PYX_ERR(0, 721, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_ex, __pyx_mstate_global->__pyx_n_u_BaseException) < (0)) __PYX_ERR(0, 721, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_NoReturn) < (0)) __PYX_ERR(0, 721, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_30_raise_fatal_connection_exception, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__raise_fatal_conne, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_raise_fatal_connection_exceptio, __pyx_t_7) < (0)) __PYX_ERR(0, 721, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":741
 *         raise fatal_exc
 * 
 *     def _wrap_fatal_connection_exception(             # <<<<<<<<<<<<<<
 *         self, action: str, ex: BaseException
 *     ) -> APIConnectionError:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(3); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_action, __pyx_mstate_global->__pyx_n_u_str) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_ex, __pyx_mstate_global->__pyx_n_u_BaseException) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_APIConnectionError) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_32_wrap_fatal_connection_exception, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__wrap_fatal_connec, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 741, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_wrap_fatal_connection_exception, __pyx_t_13) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":768
 *         return new_exc
 * 
 *     async def _do_finish_connect(self, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process."""
 *         # Cache timezone before registering handlers to ensure it's available
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 768, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_login, __pyx_mstate_global->__pyx_n_u_bool) < (0)) __PYX_ERR(0, 768, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 768, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_34_do_finish_connect, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection__do_finish_connect, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 768, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_do_finish_connect, __pyx_t_7) < (0)) __PYX_ERR(0, 768, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":783
 *         self._async_schedule_keep_alive(self._loop.time())
 * 
 *     async def finish_connection(self, *, login: bool) -> None:             # <<<<<<<<<<<<<<
 *         """Finish the connection process.
 * 
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_login, __pyx_mstate_global->__pyx_n_u_bool) < (0)) __PYX_ERR(0, 783, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 783, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_37finish_connection, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection_finish_connection, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_finish_connection, __pyx_t_13) < (0)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":831
 *         return NO_PASSWORD_AUTH_REQUEST
 * 
 *     def send_message(self, msg: message.Message) -> None:             # <<<<<<<<<<<<<<
 *         """Send a message to the remote."""
 *         self.send_messages((msg,))
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_msg, __pyx_mstate_global->__pyx_kp_u_message_Message) < (0)) __PYX_ERR(0, 831, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_40send_message, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_send_message, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_send_message, __pyx_t_7) < (0)) __PYX_ERR(0, 831, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":835
 *         self.send_messages((msg,))
 * 
 *     def send_messages(self, msgs: tuple[message.Message, ...]) -> None:             # <<<<<<<<<<<<<<
 *         """Send a protobuf message to the remote."""
 *         if not self._handshake_complete:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_msgs, __pyx_mstate_global->__pyx_kp_u_tuple_message_Message) < (0)) __PYX_ERR(0, 835, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_42send_messages, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_send_messages, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_send_messages, __pyx_t_13) < (0)) __PYX_ERR(0, 835, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":888
 *                 handlers.add(on_message)
 * 
 *     def add_message_callback(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> Callable[[], None]:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 888, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_on_message, __pyx_mstate_global->__pyx_kp_u_Callable_Any_None) < (0)) __PYX_ERR(0, 888, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_msg_types, __pyx_mstate_global->__pyx_kp_u_tuple_type_Any) < (0)) __PYX_ERR(0, 888, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_44add_message_callback, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_add_message_callba, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 888, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_add_message_callback, __pyx_t_7) < (0)) __PYX_ERR(0, 888, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":895
 *         return partial(self._remove_message_callback, on_message, msg_types)
 * 
 *     def _remove_message_callback(             # <<<<<<<<<<<<<<
 *         self, on_message: Callable[[Any], None], msg_types: tuple[type[Any], ...]
 *     ) -> None:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_on_message, __pyx_mstate_global->__pyx_kp_u_Callable_Any_None) < (0)) __PYX_ERR(0, 895, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_msg_types, __pyx_mstate_global->__pyx_kp_u_tuple_type_Any) < (0)) __PYX_ERR(0, 895, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_46_remove_message_callback, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__remove_message_ca, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_remove_message_callback, __pyx_t_13) < (0)) __PYX_ERR(0, 895, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":903
 *             handlers.discard(on_message)
 * 
 *     def send_message_callback_response(             # <<<<<<<<<<<<<<
 *         self,
 *         send_msg: message.Message,
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_send_msg, __pyx_mstate_global->__pyx_kp_u_message_Message) < (0)) __PYX_ERR(0, 903, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_on_message, __pyx_mstate_global->__pyx_kp_u_Callable_Any_None) < (0)) __PYX_ERR(0, 903, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_msg_types, __pyx_mstate_global->__pyx_kp_u_tuple_type_Any) < (0)) __PYX_ERR(0, 903, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_kp_u_Callable_None) < (0)) __PYX_ERR(0, 903, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_48send_message_callback_response, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_send_message_callb, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_send_message_callback_response, __pyx_t_7) < (0)) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":917
 *         return self.add_message_callback(on_message, msg_types)
 * 
 *     async def send_messages_await_response_complex(  # pylint: disable=too-many-locals             # <<<<<<<<<<<<<<
 *         self,
 *         messages: tuple[message.Message, ...],
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_messages, __pyx_mstate_global->__pyx_kp_u_tuple_message_Message) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_do_append, __pyx_mstate_global->__pyx_kp_u_Callable_message_Message_bool_No) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_do_stop, __pyx_mstate_global->__pyx_kp_u_Callable_message_Message_bool_No) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_msg_types, __pyx_mstate_global->__pyx_kp_u_tuple_type_Any) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_timeout, __pyx_mstate_global->__pyx_n_u_float) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_kp_u_list_message_Message) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_50send_messages_await_response_complex, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection_send_messages_awai, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_send_messages_await_response_com, __pyx_t_13) < (0)) __PYX_ERR(0, 917, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":972
 *         return responses
 * 
 *     async def send_message_await_response(             # <<<<<<<<<<<<<<
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(4); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_send_msg, __pyx_mstate_global->__pyx_kp_u_message_Message) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_response_type, __pyx_mstate_global->__pyx_n_u_Any) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_timeout, __pyx_mstate_global->__pyx_n_u_float) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_Any) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_53send_message_await_response, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection_send_message_await, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_send_message_await_response, __pyx_t_7) < (0)) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":984
 *         return response
 * 
 *     def report_fatal_error(self, err: Exception) -> None:             # <<<<<<<<<<<<<<
 *         """Report a fatal error that occurred during an operation.
 * 
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_err, __pyx_mstate_global->__pyx_n_u_Exception) < (0)) __PYX_ERR(0, 984, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_56report_fatal_error, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_report_fatal_error, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[35])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_report_fatal_error, __pyx_t_13) < (0)) __PYX_ERR(0, 984, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":1024
 *             self._fatal_exception = err
 * 
 *     def process_packet(self, msg_type_proto: _int, data: _bytes) -> None:             # <<<<<<<<<<<<<<
 *         """Process an incoming packet."""
 *         # This method is HOT and extremely performance critical
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_msg_type_proto, __pyx_mstate_global->__pyx_n_u_int) < (0)) __PYX_ERR(0, 1024, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_data_2, __pyx_mstate_global->__pyx_n_u_bytes) < (0)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_58process_packet, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_process_packet, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[36])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_process_packet, __pyx_t_7) < (0)) __PYX_ERR(0, 1024, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":1122
 *         )
 * 
 *     def _handle_disconnect_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: DisconnectRequest
 *     ) -> None:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_msg_2, __pyx_mstate_global->__pyx_n_u_DisconnectRequest) < (0)) __PYX_ERR(0, 1122, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_60_handle_disconnect_request_internal, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__handle_disconnect, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[37])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_handle_disconnect_request_inter, __pyx_t_13) < (0)) __PYX_ERR(0, 1122, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":1133
 *         self._cleanup()
 * 
 *     def _handle_ping_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: PingRequest
 *     ) -> None:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_msg_2, __pyx_mstate_global->__pyx_n_u_PingRequest) < (0)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_62_handle_ping_request_internal, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__handle_ping_reque, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[38])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_handle_ping_request_internal, __pyx_t_7) < (0)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":1139
 *         self.send_messages(PING_RESPONSE_MESSAGES)
 * 
 *     def _handle_get_time_request_internal(  # pylint: disable=unused-argument             # <<<<<<<<<<<<<<
 *         self, _msg: GetTimeRequest
 *     ) -> None:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_msg_2, __pyx_mstate_global->__pyx_n_u_GetTimeRequest) < (0)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_64_handle_get_time_request_internal, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection__handle_get_time_r, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[39])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_handle_get_time_request_interna, __pyx_t_13) < (0)) __PYX_ERR(0, 1139, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":1151
 *         self.send_messages((resp,))
 * 
 *     async def disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Disconnect from the API."""
 *         if self._finish_connect_future is not None:
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_t_13, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_66disconnect, __Pyx_CYFUNCTION_CCLASS | __Pyx_CYFUNCTION_COROUTINE, __pyx_mstate_global->__pyx_n_u_APIConnection_disconnect, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_7, __pyx_t_13);
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_disconnect, __pyx_t_7) < (0)) __PYX_ERR(0, 1151, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "aioesphomeapi/connection.py":1190
 *         self._cleanup()
 * 
 *     def force_disconnect(self) -> None:             # <<<<<<<<<<<<<<
 *         """Forcefully disconnect from the API."""
 *         self._expected_disconnect = True
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_return, __pyx_mstate_global->__pyx_n_u_None) < (0)) __PYX_ERR(0, 1190, __pyx_L1_error)
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_69force_disconnect, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection_force_disconnect, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[40])); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1190, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  __Pyx_CyFunction_SetAnnotationsDict(__pyx_t_13, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_force_disconnect, __pyx_t_13) < (0)) __PYX_ERR(0, 1190, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_71__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[41])); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_13) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_APIConnection, (type(self), 0xac90833, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_APIConnection__set_state(self, __pyx_state)
*/
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_13APIConnection_73__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_APIConnection___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[42])); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_13aioesphomeapi_10connection_APIConnection, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_13) < (0)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_ConnectionParams(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xb91db3b, 0xf8ad194, 0x2f722c3, b'addresses, client_info, expected_mac, expected_name, keepalive, noise_psk, password, port, timezone, zeroconf_manager')
*/
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_14__pyx_unpickle_ConnectionParams, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ConnectionParams, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[43])); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_ConnectionParams, __pyx_t_13) < (0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "(tree fragment)":1
 * cdef extern from *:             # <<<<<<<<<<<<<<
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
*/
  __pyx_t_13 = __Pyx_CyFunction_New(&__pyx_mdef_13aioesphomeapi_10connection_16__pyx_unpickle_APIConnection, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_APIConnection, NULL, __pyx_mstate_global->__pyx_n_u_aioesphomeapi_connection, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[44])); if (unlikely(!__pyx_t_13)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_13);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_APIConnection, __pyx_t_13) < (0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /* "aioesphomeapi/connection.py":1
 * from __future__ import annotations             # <<<<<<<<<<<<<<
 * 
 * import asyncio
*/
  __pyx_t_13 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_13);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_13) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_XDECREF(__pyx_t_14);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init aioesphomeapi.connection", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init aioesphomeapi.connection");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin___import__ = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_import); if (!__pyx_builtin___import__) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_id); if (!__pyx_builtin_id) __PYX_ERR(1, 16, __pyx_L1_error)
  __pyx_builtin_NotImplemented = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_NotImplemented); if (!__pyx_builtin_NotImplemented) __PYX_ERR(1, 26, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_get.method_name = &__pyx_mstate->__pyx_n_u_get;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  __pyx_mstate->__pyx_umethod_PySet_Type__copy.type = (PyObject*)(&PySet_Type);
  __pyx_mstate->__pyx_umethod_PySet_Type__copy.method_name = &__pyx_mstate->__pyx_n_u_copy;
  __pyx_mstate->__pyx_umethod_PyUnicode_Type__title.type = (PyObject*)(&PyUnicode_Type);
  __pyx_mstate->__pyx_umethod_PyUnicode_Type__title.method_name = &__pyx_mstate->__pyx_n_u_title;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "aioesphomeapi/connection.py":410
 *         while addr_infos:
 *             try:
 *                 async with asyncio_timeout(TCP_CONNECT_TIMEOUT):             # <<<<<<<<<<<<<<
 *                     # Devices are likely on the local network so we
 *                     # only use a 100ms happy eyeballs delay
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 410, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "aioesphomeapi/connection.py":250
 * 
 *     __slots__ = (
 *         "_addrs_info",             # <<<<<<<<<<<<<<
 *         "_cached_timezone",
 *         "_debug_enabled",
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(28, __pyx_mstate_global->__pyx_n_u_addrs_info, __pyx_mstate_global->__pyx_n_u_cached_timezone, __pyx_mstate_global->__pyx_n_u_debug_enabled_2, __pyx_mstate_global->__pyx_n_u_expected_disconnect, __pyx_mstate_global->__pyx_n_u_fatal_exception, __pyx_mstate_global->__pyx_n_u_finish_connect_future, __pyx_mstate_global->__pyx_n_u_frame_helper, __pyx_mstate_global->__pyx_n_u_handshake_complete, __pyx_mstate_global->__pyx_n_u_keep_alive_interval, __pyx_mstate_global->__pyx_n_u_keep_alive_timeout, __pyx_mstate_global->__pyx_n_u_log_errors_2, __pyx_mstate_global->__pyx_n_u_loop_2, __pyx_mstate_global->__pyx_n_u_message_handlers, __pyx_mstate_global->__pyx_n_u_params_2, __pyx_mstate_global->__pyx_n_u_ping_timer, __pyx_mstate_global->__pyx_n_u_pong_timer, __pyx_mstate_global->__pyx_n_u_read_exception_futures, __pyx_mstate_global->__pyx_n_u_resolve_host_future, __pyx_mstate_global->__pyx_n_u_send_pending_ping, __pyx_mstate_global->__pyx_n_u_socket_2, __pyx_mstate_global->__pyx_n_u_start_connect_future, __pyx_mstate_global->__pyx_n_u_api_version, __pyx_mstate_global->__pyx_n_u_connected_address, __pyx_mstate_global->__pyx_n_u_connection_state, __pyx_mstate_global->__pyx_n_u_is_connected, __pyx_mstate_global->__pyx_n_u_log_name, __pyx_mstate_global->__pyx_n_u_on_stop, __pyx_mstate_global->__pyx_n_u_received_name); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(0, 250, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);

  /* "aioesphomeapi/connection.py":972
 *         return responses
 * 
 *     async def send_message_await_response(             # <<<<<<<<<<<<<<
 *         self, send_msg: message.Message, response_type: Any, timeout: _float = 10.0
 *     ) -> Any:
*/
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(1, __pyx_mstate_global->__pyx_float_10_0); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(0, 972, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<3; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 11; } index[] = {{3},{57},{21},{18},{40},{28},{40},{65},{17},{22},{34},{61},{62},{20},{12},{25},{10},{18},{26},{26},{17},{31},{4},{179},{26},{33},{11},{56},{20},{28},{1},{1},{1},{0},{2},{2},{2},{1},{1},{8},{11},{7},{33},{38},{21},{27},{18},{19},{22},{22},{18},{22},{20},{37},{14},{13},{21},{6},{7},{25},{6},{15},{16},{2},{14},{9},{12},{17},{21},{9},{15},{12},{11},{7},{48},{20},{33},{29},{46},{39},{47},{45},{26},{104},{47},{18},{36},{43},{30},{169},{29},{8},{10},{11},{15},{27},{21},{19},{13},{27},{18},{31},{33},{38},{36},{34},{40},{37},{32},{49},{47},{36},{43},{47},{38},{38},{46},{34},{24},{31},{30},{28},{32},{26},{41},{44},{27},{50},{23},{26},{30},{32},{19},{23},{10},{3},{21},{22},{15},{13},{6},{9},{8},{14},{26},{32},{16},{34},{36},{15},{8},{11},{5},{26},{27},{7},{13},{12},{11},{25},{28},{28},{31},{18},{28},{31},{18},{21},{16},{17},{18},{4},{9},{6},{14},{15},{18},{13},{17},{12},{13},{11},{11},{19},{14},{7},{21},{15},{14},{15},{13},{4},{8},{4},{14},{21},{14},{19},{18},{11},{12},{16},{20},{18},{13},{10},{9},{17},{14},{20},{19},{11},{12},{13},{15},{27},{7},{15},{1},{6},{3},{20},{4},{10},{9},{5},{11},{10},{9},{24},{27},{16},{7},{11},{17},{17},{7},{15},{24},{18},{22},{7},{18},{15},{9},{4},{21},{22},{6},{26},{16},{7},{6},{10},{9},{5},{9},{17},{11},{18},{5},{15},{7},{20},{38},{26},{44},{23},{40},{17},{10},{16},{4},{4},{17},{13},{12},{4},{9},{20},{20},{11},{3},{11},{5},{13},{14},{7},{8},{5},{10},{9},{18},{7},{7},{4},{7},{18},{9},{6},{4},{13},{2},{3},{7},{5},{2},{8},{9},{20},{12},{13},{6},{9},{16},{2},{5},{11},{22},{17},{9},{6},{5},{16},{13},{19},{24},{6},{8},{9},{3},{7},{3},{9},{16},{12},{11},{12},{15},{27},{22},{35},{33},{22},{29},{14},{19},{16},{20},{4},{13},{2},{10},{4},{4},{4},{3},{10},{9},{16},{12},{13},{5},{20},{19},{9},{5},{7},{8},{14},{3},{10},{11},{8},{7},{5},{14},{4},{5},{9},{8},{5},{19},{10},{7},{7},{17},{22},{8},{13},{8},{5},{10},{5},{15},{3},{4},{14},{9},{4},{8},{4},{7},{7},{4},{9},{3},{6},{10},{7},{5},{6},{7},{14},{6},{15},{7},{8},{7},{11},{8},{11},{3},{25},{4},{8},{11},{14},{5},{12},{14},{26},{12},{11},{10},{28},{31},{14},{12},{33},{23},{13},{10},{17},{13},{6},{24},{18},{4},{20},{9},{4},{8},{14},{13},{9},{6},{4},{7},{1},{4},{4},{12},{27},{30},{13},{36},{8},{18},{11},{9},{13},{12},{12},{24},{10},{11},{10},{10},{12},{19},{5},{9},{4},{8},{6},{7},{21},{16},{18},{8},{5},{18},{3},{3},{8},{5},{4},{12},{7},{15},{14},{8},{5},{4},{6},{9},{11},{6},{12},{4},{5},{6},{4},{7},{12},{4},{5},{32},{8},{16},{996},{17},{7},{2},{11},{340},{51},{5},{62},{8},{54},{261},{31},{14},{72},{14},{37},{37},{31},{16},{39},{81},{88},{196},{34},{14},{2},{436},{1062},{29},{5},{5},{20},{5},{15},{130},{87},{11},{113},{52},{7},{56},{58},{28},{25},{82},{171}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (5916 bytes) */
const char* const cstring = "(\265/\375`\1773\225\270\000\252\312\014+H\360\256l\003\314\300\314\216F\007Y&\034$\356N\221T\300\n\211\237\022\022\252@\000&2#d\263\354\265n\222\223%w*\005Y\203\3233\232\035t\210\333\310\001\205\203\250\021i\017\007\202\342\250\002\312\313\013\303\370P\366n2i\002i\002\324\0029\373\335s6\3636\333\234O\251\223v5\301\272>\304\257{\207#\367n=\310\t)\242[\362\347\313y\316\375vU\024R\223'\337\006qU\336#\275&\267\231\366\366I\356\035g\305h%\232\372\376\323\273\235\253\355\230\335\365nm\347f\335\256\222B\251\244Z6Y\216\245j\247\217\356\361\3613Mw\205\341\253nuY6\325y\320T\327\344\320\317\335IOn\263\207\340@f\274\257\233\205u\256\246HS\310\351q\262n\325\363\224\276]^\310\271\315U\333\272\246\311y\3678u]{\322\343\304\267\\\327\331l\302D\013ujs.\276\247\352\211\252\331\275u\244\310\331\323\216k\265\252\272\250\3378\023wrb\352]\336\244\310\272\331m\3726\251n\367\324'\332\353\223T\267I\232\234ptMs\252~\354\364\332n\024\325\244_\333\335o\344\207\313n]\357\256N<5\327_\327(owL\352]\037\343\263/\355\234\345\273\226\233?\366D\301\302\215\315\336\267S\255\262\273\247|=\265\233Uw.\357\235n\321|JY\275w<\307S}\262[\220\235\317S\237>N!\"\"!\037\310\276[\216j\230\332\251\367\354\334\273\257\234\355\273\215\305\324\316\335[\035;\355\256\352\325\356\265\351I~d\355\371\272\352\017\322\357\356\271\337+\351\362\037\211\266\310SWS\367F\242\234\354[5on'\373\313nI\262DC\311\204X\031#W3m}o\367-U\367\231\363\277\324\366\374u\262\331\237'MH\324\275q\317[\323\331r\230\336\317T\325\275\265\005\322\333\335#[1\324-\365\232\335qsm\221\355\366\250&\275\273K=irj\224\334:\235\353\347\252Y\037\254\257\373\25535\233\266&\344V\323rM\321W\031\203\345Z\251\222\202\321|\253*E\270`\264r\226k4\030\313ww\347\367/\327S\3377\351?V\0202\305\325\355\265\357\244O=_E\266\351\222\373F\376\324\225\223\2536S!& d\367 \271g]\022\267\246\253\336jJ\005iB\345\374\266r\3328\335\373\336\231\330+\177\221\"&~+\312\305e\2537\255\252:u:\267g]\277\246+O\275\271\327\225\233\244&\327\235\203!\317\335S\335\222=%\275\316y\262\345m\273\311""\025u:\235\002\250\254\232\301Xh4\324K\234\225(+\266\227(\212\"M,m\262\230\255V\322r\316e\rU\243\345\027\254%\212\242(\212~ME1\002\t\373\305,\220\254)\014\2527L\253a\372vO\326\315\033\325}u\267\352S\030\336\346~\275\365\375\364\275\373\354Y\336\275zC\325\335{\356\247V\265\313\353mUg\246\354\352\366\263\325..\267}\347v\235\271s\331\3149\263\334\267\254\252S\337\356\225\337\270\207\331\035\327\323\326\257;\3636\347\327\234\271\317\275\365\351\335.g\267Us\356>\303\350t=\3674\247\273\247\267\272Ks>\323\\\327\271_\316\333\347\274\336\365\227\323i\2538\316\327vo\226\345\\]\327\215\303qW\367\346\232\327\035\303[\324k\256\2429\207\376\324t\247\321m\372\266\257m7o\347L\316\356\215\242|\257ff\271\357\231\237^u5\272\317\234u{\303d\224\347\237>\373fOu\247\350nM3\267\254o\257\033\347A\237\267\235\366\320\271\332\341\264[\363\246O]\335\246n}\336k\316\273\355S\277q9\357\356f\257\363\326\365\355\1771>\263\356\231\317\234\263\317}\257i_\323\226\277\350]\356\267\375\333?\226\325\271\326\247>\235s\356]\347\025\022\257\031\212\345\\m)\257\226J\252d\245ZP\256RE\213I\333M\023\221\232e\030-\243\276\375\312\276_,g\225\233\252rV\316b\347n\255/o\246*^k=m\343\332\275B\313\263r\265\262\241\344Wk%\005E\243\225+\2632%\005\346\312\260\032l\026\213\265T%\211\226\013\325\252\265^-VY+]\261\225R\005\305\242\251^(\226\337)*W\331Z\345\235\273\226*\355u\371R)K\226\314\252\275`+9\206*]\031\366\312/T\313e\013\201\022\313\271\214\265\270l\354\257\266UwV\243\225\264\027,V\263\301\312\225V-\323Z\252\024\033J\251\232\241\312\226\274\202\345\026\224j\307\333\325\217\245u\035+=\177\265j\251\362\365\202\245T%\r\346J\225\273k\224\327\251Qf\332L\275\213\022\021c\255^\031\347\336\366\231\275o\253\271\353\274w\233\363yjV\2670\235\347T\356w\277\251\2526s\336=\276\3339\347\214S\365\232s\336\352\271\352\\\346\347\232\342\270\232\271\232s\034\024(86\033G&\343\204\010\301YqV\233\024\242\370\"\246\367{/\377\005\375(\211Ob\365;f>\004}\314@\001\000\317f\374\250\035\0166\215\237\367\000^\215\0109|\374\000o\215\306Pe\226:j""\014\010}P'\330\0100\032\312\340\336K\3405\201\004F\303\004\177\233\336\032=\252A\305\016\342\016\253\274#\240\220%X\n\256\300,\0104\" \370\033x\006DH\022n0\360U\300\001!Q\000\001R\034\202\311\227\005h\302\213K\342\322G\275\243)\266m`\332\273\240EP\001\307\3672\033\250\327A\r\207\200\032\323\373\032\275@\271\220\275\250\227\005,\305\262xB\001\227\214iF\rbH\360=P\003a\264Pd\364Q\302\016\010\206(\0272\031\231FL\211\303\001\374\002`\232\311 \204\305c\254\202;\002\374\016Qh\310\243Ig\000\0105\030\030\232d6\031:>\351\365 \213\375\330\205\177a\374+\200\005F5#B\223\320\244\300/\001\232\200\002\342\023\202\217\200F'\354\020\232\034\270\000,=\010\360M\3023\330\022\243\342\365\005 \222\305\245x\023\203\320\250H\370\235\030\204\300\357 \301\346\333B\007Eb\020\230\004;\030 {K\257\201\322p\300\333\343\005u\260C\301\347xC<\340G\3032\330\002l\202A\336\023\250!B\022\326\202\250`\311\306P\300\007z\330\301\3707BUp$H\2621\032}S\034a\374$\232\026\311'\021\375\315\253\002\226\2005\031\023\360\224\001\000\032\2424\276\006h\270\001\252\006t\220\311\030k\310O\212\026D\237\004\327\336\005\245!*f\301#L\276\rJCY<\002.\301\2468\007w\300\016JC\3752(\r\r|\020PEF#\264yy\300\002o\006p#.a\326\273@C\004\360b\344\025\240eH4FC\323L\027~\204\274`I\320\024<\343\223\202?\2764`Kf\004\252\3210\004R\034\262\361I#p\312\273b\0242\005U8\367\312^\006$\340H\rHz1\360dJd2@5\000XD|\021\266\337\242\031\241Q\341\360wP\026\274P\032n`rc8\002?\330\370\033\243\032\340\200\215!\nX\3320!m\324\240^\005\006\206\007\014\014I0\2067A\251\004\310\2611\224\275\017^\322\033#\263\241!\202^\240\336\0252\001@$ i4\344\001=\200\002^\r@\216\321pdF\300\014\205 )8\202\340'\320\0201^\210B\242\350\005IX\"\2360~Sp\025\217\304\022\306\341\306\305\210\014I\030\nV\020}\013$\257\026K\026E\224\210\034+K\025Q\024\333\234(\212Y\244\305\312\324\210\2625c\021EQ\214\211\254\232-b\010\313\246:\322\257))\002B\236)\357\245\300\275$\357\006\222\0005\260\000\007\214h\264\014\321F\200\341\306\205\351\335\276F\264\325\376:\263\"\321\000?\340\370#CM\003`\006D""\205\317\213)\274\031\200*H \216q8\223}\335\014\007aF\260\002\371\307\232\215\013^0\215O\242\037\334\004]F\266x\302\030\343B\233\003\003\014\220d5%\014~\307\313\221\005G\304=\270\004\206\001\007\214\300\035\370\204\367j\331\353y!\324\214_\026\354\241b\324c\343o\304\0301\017\376\000e\374\214Q\006\t4\364\214\303\021\031\357\002\344\2010\334\324\360b\035\237\000,0\000\227\274\035\177`\310\203=\340\002\303\221\016\2168$\000\007l\032\215\177\374\"\001\244\030D\364iX\030>I\220\304\364W<\304/\253A\321\260\314H\020f\310(32T\242\025+D\353*zy\021\341p\242\333M\344\342\"\232\315D\257\227HEE$Z\205Z\264\010\271n\210\307\013\255k(MC\267[(\026\013\271\\*\025h\305\n\320\313\013\010EA'@ \026P\210\020\240\325\n\224J}X\260\370\300\300|P\240\370\224(\361ii\371\324j\237\225\225\217J\365\001\001\002c\305\n\036\017\303\305\005\203\205\005\243V\303\200\301<\036\356\361\350<\236\234\307cz<%<0\230GE\305\303byR\251Q\205\212\221\363Q\2473M\032mt\215c\3128\202\350\354v\2354\355\240hG\353\016\t\022\035\032\255\003\203u\\\256NJJ\007\004\010\016\217\307\331\3558//\234\2759'N\264`\361\373\277\366\267(>\212\022\277D\355\263\320\254p\370>\230\n\272\361'>\t\215\010\261\260\204h\264P\253\025\n\225\240\230\030P\257\007\302\341@\246\t\212\305@.\220\353\343\363}\274\377\240\350\347v\373\204\010\361\371\2540T\250\300\360\3710L\023#g\014\026\013C\205\241\362\274\274xP\3243b\204\307S\363xPF\237o\364\276D\211\261V\033c\261\021\005\245\303\202E'\227\353\234g\253\305\351=g\216\213\013\007\205\203\262Y\001\373%4*&\377\014bx\360\375\313`C\363\241q\021\342J\301\021\236g\345)=\345\310\202\305\250\252c.7\356=\242@1\266\264\2140\330\250\2422\262XcjLuT\250\350\364z\235\266\355p\336yy\351\230f\247\245\245c\263uf\263\316\312J\247\263\342\244H\301\201\201\341x\317\341\234\223\313qL\230\340\264\264pD\210\350\277\257\177\305}\\\313o\261}\333\354\333hp3\013\036\004m1\016\346\360\372\032P\td\221\351\271\370\214\"\357*Ef4\0169\233s\236\001\205\361\250\023w\222L\r2\245hdF\222$\205\016\223a D4\225\211\205\263i\361\003\363sDi""\246\210H\006)\305\014qDdDDD\222\244\240\220\016C!\367\003\275{\240G\254\026\rFj\021\2656\252\322\036\375M\212\360\236\014\177\\\312\303\326\3726!nz\000\336\354\206\330U\264\360\251\002\225\270\252\350\237SD\023\303\264I\023\241\367>\353\356~%\331\206\262\251\354\001}'\343}p\376#\020y\366\335q`\017\030x\202|/S\032h\024\311\242\0359\313\227\376\\\341ro\363\032\030u}+\243\313r\026E\234\337\324\306\2765\325W\226\271\201X!\313\241\227\002\217\361\261\254\237\032&\021\371p\337_\350\200~\251\t/\223#\205\250\036~S\270\345{\331\325m\370\325V`\244}\225b'ql\305\020oRoJ\006\325\021\024\204@'o>\330\003J\017\273\370\000\243\366i\001w]\340tV&<\231\301Or\306,\352\253\227\002\317\211+(\034t\301~,!2\273\310\352^\005\341\200?\022Z\344\330\367b\270\025L\223\341\370\203\362%N\001\276\325\377\010\367I\372P\374 \0018\352!Q|\237B\236\217I\026J\312\257$\364?\222\202u\200\336\212\266\366\203X\366t\257\242%<\021VOa\266\030X\001+\213\212\304\317<\2405\265ohT\337\344\375\033\253\025hG\344{ui\210\227\224E\211\372-\007\370\024\353\027\363P\310\027\346\354\371:To\302\357I\262#g\224J\026\334\227b4\017\256\256\362\233H\337P\233\217\261\027\376\353e04\030\r\360\201\210~\2670\345\315\016\373\t\265\3223U\021\240\014*\237\375 \326\373\225\222\344\026%1O[~j\2422B\252\353\216;'Cb\003\2537t\016{d\243\34394S\350\017\207\221z\235m\351-\203\025\001\343\212KPP\266\256\310\262\007\271\315\005\334\027\343NO\327\027\205\3207]\211?\235\356\001\311\214\344{\315\027B\343\021\373\345\t\335N\224\036\001\216\342\320\022\316o\022u\311\013Z\354'\371\342h\271\023\321|\221\236d\270RiiaC\203\266{,|6\330\344@\005\210\263$0\361[?\232\007\301Uqv\t;QA\301\217\316!]\005\334\241\004\254j\355\007(|\"\034\313\232\205#B\355\254\347\010P\000\325\217r\344\001\211n\256\236\336\033\234\261f\301\242\3570\326\t\000=\001\351\304\037\004J\330\264\"\337\236\235\260\262\274v\300\365\207\217\345\024\301\254\370$c\t\362\037)C\371 QhQ\226\256\376\0176\213\210\227\362&\220q\231o@@QH\026\033\257\2037\330\306\001:\021\000>\267\004\026\311;+\\\370""\023\237\372\034|1Jr% \226/\351b\252\244\375,\024\375`#\325\272w\244\210H\210\375\2632\2421\016\2121\037\024p\237\034\\\035E\272\237+\362,\277(\374P\227\203\232\243\036\212\227`\037\004A\271\251O\t\311\000\223\010\252\256\251\304\347p\177\357\315\223\322\277Lw\306\253\314\275\r\317\203e\300\005w\035\030\246\263\312\337 \3430\324=\001\326\017N\372>\t\317__\353\275\\}\324\010f\017\367\310\224\"\335\206\222\036\033\354t\200\021r\031TWVw\252<0\343o-\247\375\303\217.:\213;\347d\261\211\205:t\214\244\357W\214%\334>@\353\217\361ZDS\232\256\315\325#\232\260\356\330\266\037\324}\016j\203:'z\300V$\325\005\335r\0322\212\2212,\212\203\321\351\266\230\256\276\246Q\355\246t\243\031\237] @\030Z\226P~\303\010\\\030\325\321\237\307'\330v\3177\333N.\243\n\210J\020X|E\304a)<\2214J!_\275\300\252=4l\203\032\336\320\313d\231\315\302\203/i\224a\213'\350j\247`DN\001\200y\232\024\007s\364\373sb\033_L\033\207\0029*\024@ 7sGf\275\035\317\033H60\276\025(<\306\016\002B\005I\366\374\234L\345r/DX\332\016\303d8\367<\335\302\262\250j=\275\264\213\206#\nEP\377\"\250ge\020H\270\2578P\351\374\006\r}\356B\251\307.\316\202\014\013\255\306\347\215\321#\013\233)\346\311\0309\331\030\354\372\300\334\304\341'!\244q\350\003\365N,\271C\343\330\226~\345\314\370\241E\226\010\212\3556]!\204H\310\337'\r\355\251u\033\326Rh6wR\222\275Sd\267\222\020\356:\t\342\014\271\022\223BW!Z\326\256\265\217\306\3337s\353\304\361\034\353\022\205\211\336=\341\000\313\036\231\207m\350\243\3630\001\340qX\344\203a\314\254v\373\322F\301\3040\230T\201\205L\202\242\223\367\250'\274M\215\274D\326\004`F\312z\230\246Fc\371\257$Z1\213\321\313Z\266\302\322P\315\025]\000R\222\310,\0249\231x\242p\024\331\345\200\205\025q\213\312\313\003\307\362\301bl\232\214\237F\327\354u\324\363!\246_\233fx:\3416\001eq\263k1\304\000\"\302G,n\n\321\321 oL\342M\232@5\216\247mY\263s\224\372\264\\E,\360/\240C\035\037\371\312\021_)\266\230\246\207c\376\0174@\346b\251\246\275\026\373\350\361\330\335\221/MD\004\271\274\201\354\377FY\337\216\202\250\244\001J\001""\016\237\377G\212'\317\323\274\351\302\017\206|iH>\010\324\376\032\253H\026Z\306\234v\314l\315\375\036iK\001\\dle>.W=M\366J|\"A*\362N\022b\251\226v\235\317\027\030^\256\377\243\035\014{\247\354a\"\034(\220\200\275\036\340l\021\214\330\336rO\304\365g'\377\227\264J\262jh\315\311\225\244\270\371\021c\266sB\375\271E\203N`<\315(U0]\000\024[\251\357\026\203\263f\200\361\220\210\000\337\351u\220\367\317\010\270\017\340\361\"f9\341\251\226M9\224P\233\341\274T=e\342g\026b\252\2430Q\241\026\272\2537,\000N=/\014\260.\251\022\361\341\030\215L\016\002)( \350i\324\0014\322\037\370\355x_v\355\312\006:j\000`\250\nL\022}2+\266\375\251\362g\2023J\001!\322\260\t\216\032y\0338\317X\"A\216\342'\303\245.\177\320\\\004\3476\332\317\352\031?\013\202\270\331\013rXV\221mM\234\364\373,\353;\236.\016\213\003\221:a\037\274\232\340\3616\352\372\267\200L\326\227\244(\351fQ\252\357RA\304S\325\000\214\204D\244\352\273\261Ly|\263\\\223,\361;\341y\214)\240\371\332\216\315\035\241P\331\241d\026\nl\231\254vkW\221\245\022\023\026\026}\036\317\342\270p\365\327\373\3150\263\364\202\031\373\032c\035\234\365Q\253\370G\377\210\332\036gc\335K\203G\352N1\252\000\266\002\300#%\207\221\207\227\236\024\333A\355L\321#\006wr-\354\\\355\316\341\337y1^A\305\304\306\002\237\247\244\222\230\242\001jJ\214<T9\274\320\016\365m&!\314pKb`m\037\312\223\026\372\333\010Iv\213^.\210\332d\214\310D\277\256D\226\263\021\375u:\263\"\305\354*\351@\366RV\341\034T\312\200\206\030<\302JrY\244\314JK\354\021@\024\265;_\223\356Z\314\0308\225\355\316\311Z4O\213LH\25572m\346\271\022[\206\305\"\353\025I\037\225\2275Q]\025\332]Z\202sQG\237\034@\232\262L?\365T<\342\263/ze\240\365\330RK\300M&\344\315\232\022\305Z\374\344;\350\350\276\345\035\330\225t\031\035.\364\375\224\356u\000\031\275\326;O\t n\262\2577\322\210\257/\237\007`6[V\374\262&x@\n\217\311\224\006\217\321[\345\307&\314\244\363\367y<Z\325j\3704Kp\277\217 ?,\311\177\335!G\270&\004{\351S\3128\2330{\005\t\231\377\234\324\023\376\205\222\260\336\257\253\230\263<?\226:\304\330i\354sC\255""\010\230\314\314\353*er\346\366\227\334A\000'c\371\302H\016cO\355=\343K\353\014.\324L\021\0252M\316'(I\024m\313!\224~\366\r\270\243\253\275P{|\321S\001\216\023\251\014\220/t\204\304\215\343\260\326\253\353\002\010@\353\247\222M\235\341a\2710\256\240\323\230Wr\276R\210\035\234Y\204HI-\366\253\373>\223B\243w\277P\242\317O7\014\024\2356\220qy\252\206\311\257\2373\306y\004\200\352QA\315Y}\304\013\035x\227\3132 \016\003'\327<9\264l{\355\216\233DI\023\246\004E\335\243x'\324\237\214\013\314g\215\373\253D\342\355F\027\260\333k\206C>(\256\242\023\000U\374\313\322a\022\210\252\215\213\020&\300\267\307\233Z\342\014\335@-m\357iA\236\210j\020\031Hr\323\020F\343\014\013\026\007\370x\252L\261\017\320P\326\007\025P\365W\357\242}\000}P9p\361\032\241\247\263\3416?\014\215-l\346\033O\327\257\237\344*\212a\004\205?(\227r+\356\364P\022\300\343Bq\334\342\305\372\263\214\353\360\320\271\241\243\376w\202\353\347\024\236\264\316\304\303\250\264\367\273\350\222C8\225\231(\235\222\247X1\315\032\352\301+\251W\214\346g\r\326\230k\260O\032\371\027\352/AQ\216\254\rN#\275\220\037\206D\346L\344&\320\204`\2175\020\222&\316\031pW\222eJm)\361XU\254J\367+\367\244'4\237\232\220\313\344g5\236d\224\210\243\233\240q\343\\\031br\245\342(\253\276\247\010\242p\002\253'`4\036\200\002\302cVq#\001]H3]\273\265\232*\270\030\010]\242\010\242\\\270\224|\016\315\262\211\241\212\275\303SGd\305\201(n4\273\230\225\316\320g\371\212\343(\222/\310\350D>g\243q\230.\200\326+&\204\351\314\277\016\331\2737t\031\320\362\276\215\245E\266\317!\252L\177/i\272\361\332\211\021\\\252\010|\236\265\014M\212&m\237}\215DsVl\305\002\376\265z\267\026\245\317\213\014\325$#\263\231\227\021\2144J\017W<\nVX\367\351\353\021\325\221\311\222\t,.\017!\265w\022\02410e\210j\001\321\273,\225Z\322j}^\3727\3224\266!\224\014\276\017\200\000\271\211i$\273\306\"\304n\214$b\260@0\003\217\226\037\240\360\246\316\335\207C\236\025N\013\321\024A\315\003\201(KFr0\017\325\030\365\337s\373Gx3gX\177p\321?\324\305\220r+\220\330\260h\337\312\201\013\332\341/\0054a""\017W\234\250\033c\004\023\265\211s\225\033\350\232'\351\270\t\027\000c/\031@y\034\341C<\370U\372\252\230( \226I\207\014\311\240A\247\025\361<C\244\342RWJ\357j\037_\263\323,\215\2426\273(\250\000|\202k\272\204\264'\317R\r\260\003\314\\\347\306l\\\327\034\024\271\177\245\323+\232\271#\237\340\314]\262\221\035\210\217x\202\231\336\000\317\rR\363\305w\374r{&\3345\204#\360 ,\240\353\033\0277KN\213\257f)\377\240\212\000A\021N\027\213=l\r\372\373\235\270.\032\344\267$\213z\341)\310\345\212\033\325\370fC\035x8J\\\224\247\246&Z{.!~\010i\220DvS\372`\346\210\234\007\376\033\201)A\257\252\300\320F\235%\311\315\207\204\376\376\2426^V\312\356\212\243\253\304j\223E\364\213\326zp9\344\340\340:]\211\275\024\210v\275\225\237\037\210\242<G\376\221\251\351Z\315*\234\310\346\305;D\351\031g\245\275[\233\t\234\236\340&L\265\303\032\177\2443\360\366\224\006\337\217\321C8)\366\371G\304\2213\013\220\371f\037\256\257\t#%\004\202\370\0355\272\001";
    PyObject *data = __Pyx_DecompressString(cstring, 5916, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (6003 bytes) */
const char* const cstring = "BZh91AY&SY\210KJ~\000\004\031\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\377\300@@@@@@@@@@@@\000@\000`\030=\311\356=v|\337r(\365\036\335\273\275oW\231[\007\275\272y\267eL\240=\354\320{\306\226\202\333@\000\320\016\216#\247mvb\035\336\343\244e\001\367\300\003MD@\t\264L\332\231\032C&\000\214Jm6\232M\030\247\246\221\223\023F\322\032\033S!\243&\236\215\023\030\223\320\233I\241\345\004\241\024\314I\262&\204&\206\243\324~\246\232\230\002=G\223Hh\3204\332\200\320\321\240\000\000\000\000\0002\003M\001\023D#@T\375\010\215=OI\346\224\331\251\265OSbM\031\r\036H\323@i\220\006\203 =# h\000\000\022jH\2051\222d&\236\223#@\332M\006\032M4\364\203F\215\250\000\0004h\323A\241\2401\000x\240=\032A\025M\032dbi\264\237\352\232M\250\323C&M0FC\021\221\246F\231\001\243\000\023M0\0202i\265\000\302\0324\032\004\211\004\004\004\302h\006\211\251\344\324\3010\324hS\324~\2116\247\352\236\221\215M\006\200\000\000\000\r4\000=F\234\004\377\247M\002\020\307\225\2304\326\342\263\3239/noT\275\032\263gf]\007\265\354\341\200\366\n?|x\nz\201\300w\365\031\020P T\313\2729D\360\224G\364\324\224\022R\222\320\300X\242\202\004\2630BB\001\351M\331\007\336\226d\001\221a8\366\272\302\220\302\311W\324TT\227\244\206\276\276\251\263\326\352\354\035Z\235k\252\366\363`pu\255V\202U%\\\340\270\270\326\346ek\363y\307\314}\030>\253\376\316\217K\361,\274\212\351},\332\014\253x\270\004\002\000\020\020 DHB@\216\026 \210\2022I%\031\244\337]J\222vv\326\267\347\243s~\224.\032\034\\^\307\236\277\307J\305\273\214u\214d\300\306:\307X\311\354d\3261\372\024(<\377\"r\220\204\024\004b@\024I\3706\335\036\037U*?\315\372^\263\216\n_\320\273J\355\021\357j\343\372\272W\000\270\227\024\311\254u\216\261\320\307X\306\014\220\30602{\254}\367\365\000\251\253B\204V2\364\346N\367N\261:w\004\217\211J\\\372Rt\235)J_u\375\020\016\276~\205'?\341\333{\264\334\337\233\022\254\224\245\031JQ\204y\024\253\261\271\\\3115Y\253\357z\265]:\210\227C\332\332\363%\314\222\343\2169\220\343\21692m \005\374\010\214R*\243\021TAdU\204F\033\235""\235\376\177\013m\275\344\360\367\245\377\336\326\317L\332\327';g\216s\2319\333:]y\313TUQ\023\301\3549\\\373\227\267\272\376\016\357G\213\326\337j\247Gk\245u\324.\235\327]u\327]\177b\231{\224L\372\333\177\347\177\265=mju\004\214c\030\3061\214\244c\217\017\250\342\000c9\271IrS\351\325rA\341\032m!\26467x\330\260X\\4\0304}\332\364\244\301\226X\024\373\220\242\013\327\272I\\\275\377\301\372\013\003*L\023i\025\223\225\233|RE\205\340,L\314.\030L,\346\027\2311\021\210\234\360\236\374w\207\002Sw\216\244\211j,\324\337Y\352\021\221\311\016J\330mM\005\350\373m\025V9\031\313\350\227\205i[\325UC=\245\255.\241\004\021 \271\"E\024\024\261U\253\t$\257L^\220e3\322\3115Vzh\026\223q7:\270\025\214\245J)d\tBl\351\221\002b>\327\254\024\006\031]\200\254\201{8\256-\220\326\244\341\255\037\220\305\306\326\276\370\373\201\244\177^\377\243\351\016\2338\016GQ\221~{\221\347J\217M\325V!\023\364\215\250^\307\332\372\177\232\311h\226\000\234\037\327\251G\315L\005\202\024\007f\003\3732\334\257\023RL\337\325}s\365\245G\373\3474\345\341a+LY\232\021\205_M\220}\246\225',e\313_VZ\366\375\370E\334\031}}ac\034LF\267\001\232\244h\24092\362\356w\201\013\347EH?\027}[\026\340Q\201\232\334\021\266\331\200\304q\320p\365\226+\020\316\261\334\023\235\202\212\242\272\331\316\231b]\230($`\032;\202#\"\034w\354\233o\233\020\271a\275l\213\225wj\005\256\374\332\2620h\333Rn\304\307\036\356\016j`++%#F\r\221\030,\264\033\345*\206\026\235fS\267\346f1|\265\320\335\356\221\257\273ZUy\"b\006\315\311\331\327\362\001(\207\246\251\313}M\023\226\362\023\\\346\220*\322T4\222\223\240315\211\252&%K\036\250\363\037Pp?Ia&\224\023\250eJ\200\313\321\235J\360\2416w$\246\260\264\375)2\241\242U\227/\312Xu\034\024]\272\241\272\001\250\320y\241\222\265\202\335G\306\255\233\035\1774\311n=\3331\336`\315NA[\013e\251\330g X\205\030\222/\244`C\335%Y+\345\352w\203\235\303H\205\277>\237\362\376\337A\323g\227o\027U6\234\334\343\262\252\240u\374T\323\261\263X\317\251\362\254.1\205S\317\233S\315<\030t_\357\350\202\3336\366\262jy\031\213\222\230\367""\336\005\230u\277h B\024\302_\204k+\315kr\365\234\342\304\342\376j\343\364i\033\357\017\032\347\336m\361{\276F\373{yf\r\2524\212\314\2158\013\017g\313B\206\257&\203\312}f\257\026DL\371R\264\3660\257s\345\311\236\3242'\t\222\021\244\224'\277\316\341>\243\301\307\324\033\200\r\344\362O\nET\365|:\r\350\033\373\374o\306>\200r\374l\252\352D:p\371\276\006\237<\357\3263\221\3500\301mM\250\262\343pav\241\317u>\3319\331\236\271\362!\242m~#4\307\3359\274\215\366\210\316#{\342\025\310\345X,\314\240\206\361\3127\270\222\010\202\037\270\352\357AX\344?x=S\211\2035\321\302\373B\026T\025\227\030\306<^\010\351\313hz\000\222\213j\002\231QQ.\202B\305T+\000\354\352\354\274\242\021\303\212\272\346T\346\271\264uJ\360\004\r27\262\216\2044^\205\276\332\260\364\317I\261\030p\024P\236\365\305\216\010\201\356\2424\251s\365\025\033\202\244\223\276\270\255\230\320\242g\310\241K>\331\311\337\014d\302\370\354.\247\271\244\231\223F\342e\000\330\307\304\373\3363\336\317y\251\333&\353U\252\222\321\r!w\225(\376\313)pk\376e\321\030\262k\265\253\300\377\020\326g\340k\2472\276C\201\203S\030_\236t\246\252\225\265\003:\275\335]m(\316\234\244\306\004>\232vW\240fe\014:\312&\340:\306\251\356S\200>\037;WS#\241<\225M\311\272\257\212p\322\3066\352M\3239\316b\312(\223!\201\n\341\226\346\001*\023q\315\022\r\032\311BswQq\350\331\003\244\270R\3174\271\030Y\342\323\326}Vu\263\300Xe;\227!$ \353b\t\245\357x\3169\221\263?\222v\001\251D\014\350\202C@\207\203\263\310v\366%\355\316\213\226\005n\203B\206r\261\234\367e\025\3265\216:\310\245\r\224P\330Jh\367\177\237v\034u\306\267.\304\212&s`\022\210\352\034`\340^\222-\027\035\227w\245\0016\372n\226G\007\354|~\203\275\236]\276l8\262j\016\320\014\253\"\230\2632\0003\032\230]$\242v\253&.\362\222\247Y\331\355\217\235\222\362\234H\321\344-\313\306\316q\365?W\2146h^I\005\373\312&\001\207J\363(\025\252\377=\347\310\271\021\302\226\013\026\326Fz\322\370xu\37495\277#\356\307B:\016\315\341\204\tf\232\004\255\030p\355&\266\347\337\202\335vB\353\333g\207\241{\036\372\366\306\207w\312""\374\323\354\321P\315N\372\246\345\365ZzM\271\214/7\232O*\250\217\0047\267Nku\310\334\264^2\327\\[B\250\277/\273>FC\220x\275L\234 \352\353\227@\256\200lvM\271sq\364\017\037'3\273\273\27333\273\315\370C\375K\303\335\277S\356~\036\300:Ci\252\325\361(\016\016\254T\362\300-n\251L\030\2022\003J\034\345\267\"\331\233\237\226\256\001\3018ids/-yk\333\342D\302y\261\362,0\343\017j\334\302{\224\213\322\236\373\005\304\246\014\310\027\\\367\323?\310\341\"V\226\025VH\222c\020\314\273\303\313\311+\375\223]K\275r\034\201\334Y\343}~\340|\300\316\344\016\024\243\003\310(\221@M\247z\360\325\217\027\"[\332I\231 \021\005\270u\216\211\275.\250\025\032\200@ \203\004\355\210v\344Wx<\333\016a\270neMCI\021\013\344\003\025oC\212\370\025\341\270\334\222\371\303\261\315P^#3\r\207D\250\312.\222\346\267\016C\347\322\263t\233n)\314^\315\320\202\350\337\200\322z\375\034\224.\357\362>\204\251Z\325\372J\017\316G{G\274\254\213[rd\231z|\273\022R{\rR\351\3633qO\360\327\306(\t\0106d;9h\211k\323\222KJ\204K\246\034\244\306\005\301\246\300/8\3663\230\227*\232\254\233n7+\037\037\217\247\025\254\335\251\237Y\331;\307p\307\343\3674\355\2278\263\021df\355\006\263:\247\225T\216\302\371>\261\301\216\221\202\255\323\213\002\255C\331\211\220\232P$(F\027$\351\223\350\277\206\025f\345\363\002\231\235\302Dt\325\356\036\017\231\026x\271J\351!\304\032\322\245j\206\301[f\253\025\\o\333LI\305\353f\241,\232\201R\220TY\304\361T\035d\225b(\022\205\000\022V\216oddr\021\341s\017uV\031\244\014\364\221\220\321q\346\215\241\222\304\007\016\222EHa\026\267|\211I\273\3532D\363\3264d\0108Gia\234\333+\307\302\324\010\324\362\342f\222\235!\346)\027\304D\245{\320\302\347\371\214\201\305\360\007\001\275\013\345\202\003\320Q\266\200R\033,\316zD\270*-$\221\220\326\347r\313\345J\216s\233\r\t\343\223\352\333\334\317x~o7\372K\340\350\322\315\220fX\360KA\314\021\000\316;\036\345\373\302\347\007Aqn^A\310\307#\302\270sG\2671\022ze\241\r\tX\230\254\335\340l\036\236\005fl\330t\341:\364Drb\035 `t\211\252\364cz`3\357\266,\246\354\300\353\305\365""\301\213\214\367'%\256\343J\301x\315$$\300eY\2634\300\323M'\321\006d\364\320A\341\006j>\345\304\002\2577\036Zn\341\270\362\206\346\316\026O_\024\250\rjI\216Yw\334L\245\211\014\031\315\215\227\233\331\303\335]\025\226\006\313W\274\321j\007v'\026T\257i\274\216\257\003-l6\005/ \270\312\206\003hb\324559\006\241^)\313k\223}X\242\226\257\265nx/-\361\366\3468g(\355\323\023\021\344\026\261\"<\250HH\010\363lm\024b\240XX2o\235\003\256z\004k\010\367\366\372\306+\326,SX\231\036\231\346\006i\363\331u\034\303\250x\306\tLX\202\260\351\257\303\225NX\263\262\324]$Q\261\264\r<G\016QV\264<\243\317:\361\254d\266,\231f\022ZK\241\2032]\030*\0021k6\255ad\352\262\030\302\317%$\255\227\277(\244\333@gi\215$\332Cb\020e`I\342\312\213\334<\007O\007\304\371lX\027\302\301\251)BZ\031\377\214!\240\230\305@h\211B\247\025\034\2444\327s\033\024(\335;)BOr\254kV\350\310\367K\247f\331\005\276\330A\222\336Gs\307?\031,\272\\\014\2659}\250\\t\013\034\231Q\006\251\310l\313m\342*\2654\211\312\231\3471]\214x\2163\302\273\210\035\253N+\245\252.\215C\221\303\204\007m\214\317\312;R\346\340\032\361&\321\323|9\270\370X\244\261\223\261=\362z\361\0202a\251\206\270-a:\345C\215\t[[ \267t\243@\310\202\204\253\000\372D\335PT\020U1{\013\202\272\241\205\236\354,\000\305\251\rU\311$\220\340\270\261w\024\324\275\205K\2004\t\231\240D\323\002n\241\304\242\2724S\330\261\223\267\323ah)\305\260A\223\250\346E\264K\034\370\001\236\177}*\211*\260I\000\204D\003\"\356\010\2018\022\271\205j\337h\312\264u\272\330\244\312\205B\220\n\242\251\001\213\225p\316\266\220\220WP\215\300\006\233'Maw\205\364\276\222\312\225sM\332\247\320\263\312H\365\206\201d\244\nC11\331\212\224\311\317\242\315\345c\"\210\216\336p\341t\306\226:\014\332-\010\337\014H#\001V\016c\0052\002Y\231\022M\014\014\310\247\024fF\r8pN\347\262\266]_\216\3356\336\006_\273\2548r\230\377\236\323\331\256m^\037c\021\265\304KHiz\016&Ns\341\276(\310\313HA\332\013\204!\r\014\024\351\270\200\3324\232i\004\355d\367\241`\322\3239J\007\277\236\366+}\241\nH$\310!^H\321\"Q\031\355\212\252""\220\303-\366g\304\267m\346\312\354i\371\364A\251\245\207DO\007\252W\227S\345d\247\257R\017t\356;\266y+\264\273\213\265\206\221\254\333\316\247\tE~dg5\342^\2573\360\310J#G\264\201$T\345e\037 }X\342p\225M\322\260Y\224\225\0007&\241\255\003{\310\004\242Wo\227\273\303\302{\322\200jb\031\254\202\370P(c=\371\344B\352i-Y\240;X\242y\316\315\016vu\264\010([\033u3W\211\3535\220\250\3657\014\346\271\343og\333]\nm\213\302\232\333\260i\302\033\024e\244B\010\346\315 \332l\360J\300\373\016\362\361\251\211z\346\215\264\220\370\301M*#\223\2563}\232Y\273u5\277f\002\0138\336\266/T\317\254y\312\354,D9p(%\333;\272\016\003\244\010\227\233p@61\261\210i\243e\252<\033\235\254\031M\265\311a01\244\246\317\2560\323\3149\031\3309\323=m,X\352\217U-\225\322\340A\227*\325\177\006\337\236\246\026\001\035\021\252p\371\321$-jHBlBcCh3\255:.DZ\305\2711f\270`\357\320\006\261\333RJ^\252\3000\307,j\034\235g(\014\314\220\234\003\177\211\331\301\356>w\326\374O\305\362\001\206C\263\304s\026w;\363\033Rw\\\203i\304\017U@*|\226H\3338\265\250\237\342\353)\216H\0162\2741$\210\016S\301OO\340T\310#~\237\310\272\332\211\332\333%\033\233GX\373\201\251\227\021.\244\206\327\320S\250\274\270\261\325\204\223\222\225;\342\331G\024#\035-\213:\255\367\036\237\323*%F-v\244\240\223\260G:\326\256\340\346\\\227B9f\345#)\254n\253\n\010\317\013\021\345\312jd\276\2079\264\030\326\330\237\275\337\315\332\373n\305\335\034zQV\033\245\243\233:\262\350K(4\223P\t\220k\302HT\234\014\274\260\364\3612\t$Y|\r\212\301\026C\354\202\213\004\356\020\222\362\022\n\214\341l\367\034\361\263\234\365\230b\216\030\017=\224'9\021\n\034\341\325)\000\r\223\210\022\324\300%(\033Hi\215?\246H\213\023\240\262\3469\034\230\023\331\360\n\371Z)#Q)\272_V\2201\003\222n\004\322Sg(\002S\214\034\333w\0237\310S\271\245\014\001\260Lhm\006K\213\214f%\203\252@\224\024\333\217\024VH\310\\\373\013K\357\206\2017\345\2643\000\304ejr\010\037\332sD\266*qx\3718\316\201\364?\033\363l\000\300\356\217[\345\017d\246O\235\233 \252\223Qb\205\276a9.:\236\036\225\347\006\2122\373j""\217\201`\nw\034\312\324}\266\007\010\353\360\226h\336\035v\r\323*\014\207\267\343\005T\031\360\002L\020\r\r\264U\222k\241\206vU,}\254Tbn\031`\270w\001X6\327N\032I\026+\021Q\214\020\330\206\230\333b\030\211\320\004\315o\230\363\2316&WR\210\354\276\375B6\345\204\221\370\347\014\327\031s \260\254\020B\230\244b\206\341m\002\317\235\206\301\246\363Z\n\305\343\3412\266\300\230\315j\302JGqi\001\260\361\022\323\"b\032\265h\352\352\277W^\250#\351Y\001\305\264@\1778M\\\326\327\317\220\204\020\272 \227\210\030\021\305\000[8\346\023\n\"Y~\334R\320\022\310U\222\0201.K#L\321\265\305d\352\265\240\350\251\214p\314\244b\020\004k*f\305\225P\027ew\227\231\006hG\177\327\017(|\004\217\236G\262\007O\325\031\366h\251&\306VR+\005\223\225hW0\216\254=\030\204\333M0\327\024a&\333\030\321\275\220\3217\001\3012%.IR\235r\326p\375\262`r\001\332`X\326\014\233G\031\005\322\355W\203t\213\370\224\332\331Stl\353M\3414\266\236%C\231\004\001Mgg\000Y\202\204\\\342\346K\307^\004\346W-K3\221{a\240\202\205\260\225#\246\302h\324eB\017\225M&\3556F\313\002\342\221\353HcM\266$\300\316K\020$\225-Wd\276\341\210\212\016\377;~\341\310\014\323\376\2644}#w\222\252c\305V2v7ZI!\024\360\215R\310\tDH\215hZE\010*\352x\332\207[d&\006V\017\273h\222\212\326\221\215\022\002\311\024\022\224aAd\254\"+\013\200d&_\004\244\267\344\033\300\220\260\352\222\211\212z\241\321\031\222D\346\335X\236#\027]\333 dk\347\211\237\004\224L\225+\3232\312\374\202P\t\210\246EZR\360\025\032\367\352-\203Y\\\226X\316_\315yUG\235\325\007\2335\245\323\001\030\244\306\301\0048\030\333\3140\201k\034<\355\025N\035'%!J\006j\227\036\\vM\322Yh\230R\2075\233\027\033\236b\321lf\034H\335^\203H\020\2106\370\377\037X\204\007h\226\343\350\236\351\3641\323D\253\245\356\037?\234\307r\231j\312\224-\306\314\342\241\032\251\232\344\t\014L\005\006\272*\025\241\235Q\200dQy=+\036\224\222)\036\210P\300\323\026*Q\261\314j\361\251\nuQy2\311\216e\002P\324\323\"\260\260R\365\262\t0x\300\211\001\257l/\211\240\306\312\204\242\3659\215\244A\216S\031%Zd+\275\024\250\227N6)\316D""\226[\263\274`\256j\261H.J\334\n\025\n\023-\262\003\227\226\014\354\355\247D\225\232\3564\255\t\274\271\016\t$\220\323\212\0234\032`\306\220\245U\304V\0348\363\361\3113\311\215&\023\026\001q\253\030\271\214[\230b\272\210\024\031\352%\255\275\320g\000\362\225\3145\346\014\243d\221P\214)y3\222\376\005\205\214+\251\241%\235y\023\003\264\3024\201\003\251\215\002u@j \374\007E\303\345\213bf\333\r\206I/G\200.\250@\216u\201\305g\230\340Z\030\214\2370\272\354\230O\275\213\227w\276\376\214\372\305v\316\2211<\023A\241\272\2234\033U\222\377\334)OA\324t\235>\215=\374\227\372\037\2371\352\322\214\305\234\361\277D\244\253\260\250P\231.\334=,\344\016<f\222\214I\272\214\250x\002A`mXen\216#s\274\323\342\261A\023c\260\317*\317JD\365\007\314\322%\331\354\356\0267\244o\352\327\340\2745\256\014\276\347n\255H,\343<\007\300b\267\316;_;\347\310\273\tI\350\177\215j\303\256e\356\367\323\323\007\236\325\300\3579\276\326\225\325\357\263}\203\005\346Y\2340\302j\363\215\005\022t\205Be\3245UU\007`\335\217=\023\251:$O\275\034\372\226\262{u1gQ2p\205\352x\214,\327\351=3D\255\271\370V\207N\215a/l\030\322)\333kt\013\213\245\275\331\364\321C\226V\265,\253\255\304)\201D\371S\351\033\2035^\005Sl&Y\363\024\313Q\361w-\226\274\304\"\226\025\330\341+\357\356l\330^\360SZ\035A\204C\271\360\357\016AyH\327\240\353\265\332\027\223;K;\323\241\207Uiq\327:\032\\U5_\362\364\331\344\367\346\263\307\234mo\034\372\356\304S3/\345m6\030\353\351\326\235\347^\026\212\220h\224^;\367\323\254\237\302\224\024VjP\2760\013\330\024[h\265\236\245\370\241\246u\265^\024\000\3316b\357/f\220\256\252\3731\333\265\010Z\357y\354lH\264\335\306\303c\214jX\004\020}r\274;\302\036\325\270:\344X)\321U\235a\233\255_y\242\320\343OMp\241\337\221\273\255\355x\331\3747,\n\342\302\255Y\343\254}\302\262\227b\302\373\303\273iU\312\017ni\324\307\331W\237\215&\246a\036\336\252hb)[\306k\264F\310\313N\234\315\371\n\351}p\nT\340\323kKb\327\261k\313C\373m\3120\017&\226 o\350\362\355q\236\305@i\310\032\360S\350\366\206WGph\n/\336\345\277o\235\252\347\303\354""\347\215\276\314\361\177G\236\177-\376\232:\307t\243\333v\234\027o\252\316/\035\350K\207\nL%4;\r\252\257m\007\200\014\036\022@\360\204\206A\024\323j\340\346\332\211\363\363\t\225|ww\"\223\034\333\307\216\224\022rM\2241\315\024\357d\302\003\370'\207\2774\036q+Y}p6\202\346\305\270!\237X\016\3706\337\002\312\353\372\374\303\227\311]\263\354x?\220\345\0361\267\301U\277R\347\006\240\301\022Z\243\2656\211P\332P\213\210S!\246\314\037\233%\346>S^W({\261a\223\253\366\330>\263\0077-r\025\303\212\264\315\345y\355V\247\312\204\013\2252\025\016\r\257\232\201Y0\374\363\253\262+\021\211\014N\247B\245\361\001\245\240\214\007\201\022\276\025\316\234H\263\250\370P\372\242\216\267\214E\241oN\262\216\354\230\303H/\330\355w8\202\360\023\345z\261\305\320\205\241f\212\330\367\021p\326\026\026\265U\251\236\230L,\t\262\325XM\030sUj\222\326\251x\252V\253@\231\201c&\003\321\265\231\355V\026\203\013\026\201S\022%l\352\3232\306\"a\203<5\246\030E\235\355\017\022\025\375q\0232\"\377\202\"\337\373,\367\365\354\331\304\252\273\337\003\203\200l\212\0347\274\234\237&\2568\231\273\177\361w$S\205\t\010\204\264\247\340";
    PyObject *data = __Pyx_DecompressString(cstring, 6003, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (5971 bytes) */
const char* const cstring = "x\332\225YK[\033G\326\026H\200\270\330\006\033'\316u\232\330\216\361\014\321\014\261s\371\354\361\214\005\022 \003\222@\302\204\330N\271\221J\027\220\272Ew\213\213c;Y\262\324RK-\265\324\222%K\226\275\324\222\237\220\237\360\275\247\272[RK\340d\3628z\253\353^\247N\235\363\236C \020\010*\022\3274U\223\214\234lHy]\322\344\274\316\323\322A\216+\222,\245TE\341)#\257*\324\226W\014t.\227\014\236\016\314\313\205\202\274]\340/^\004\225\243W3RTU\370\253veoM\221\353\272\234\345\201U\013\321a[U\013\257\244\267\242\343|\201\313J^\311J\345R\347\242\206*\335\325?\324&M\213\355?B\361\376|\273-%+\222\252\024\216\244m.\225\351<\252\222\3423]\307QT\234\030%%oH\272!\033\274s\202\202\212a\363\256\376V\2354\355\252U\356\031\022\307\350\355B^\317\241\365\210\033\256\036\305\262n\320.\260\320R,\221d\353\341Dl\345y8d\255H\207@A3:\266v\361\340Dl~9\234d\261x8\33298\203\335\353\271\216\321aq\233\3167\204\206NV\335A._\340\322#\347K\313\213\346\222\234\332\345\206\216\352\303\022F\340\004\367\226d%\255\347\344].ed\014I?\222\3325F\276H\322,\033\222\234\2012H\021%\245\026K\262\221\307\035K\301xD\332\347\232N[\237\216(\373r!\237\306\002\272~\240j\351\251V\205\246\032\352v9#\331\032\361H2\216J\374\tiAT\245S\221&\316\037\0319K\352i^\310os\r\347\305u\352\206\226O\321\272\350\244H\361p\374\233\207?>\224\260;I\343;\330\276.\351\345\355T\001Kr]R3\322v9_\3001\305\nz@\212d\244#\265,)\034g\200\\hk\235\003\014Rz\035\027\210\202tOV\240 2\311\224a8duOJ\3475\222\352\276\020\375\202\\\320y .kP\212$\304\362\006\007\210\323\331l\225\216\223x5\256\227TE\347B\3310\230cp\332\226\335:\227\323\266\210\223-\261\036\310y\347\332\334w\013=\310\250\032\307\036\364\366\355\322\270\316Q\350!\265\352\304}\2735\341\277\367gf\240\002\367\003\367\002r:\315\260)>\215\002v\211\363?\261\367%\347U\354:\247\026\271\\\312\007XF\223\213\234\345x\241\304\265\200\242\302<|\250C\251 \347!0~h\270{\341\177V\332\376\326U\371\317\266\332\006JG\356\376)\234\325]ST\241\n\356\252\222\252\347\017\231\361\306]k\330\227\341\256-\033\371\256\321o""\270\246b\007\031Y?RRy5\260P6\312\032\177!\354\326=\250\016\254\002\327\240\317\000\005\372\016\311g2\\\243\262\202\003?\222\356\301\240\024\362\370fy%\243>\351x\204\217\244NS\003c\224\342\005\334\262\224\226\r\371\t.\220,\242\224.[/\030u\005\211\037\246x\311\032\312\025j\236A\225\365 YQN=\351\370\244\305\237dSYnd\3120\256ZVG\003nE\014K\317H\273\234\227\360\320\366\371\023\230$\343EN\013\004q\303\021l\361\225\250\3506\304\242\022\017\353UW\303\214$.\233\225\364],\357<c*\252\232\361\344.L\206m\251$C\326w!\2354\331B[i{\224\265\243\277\255\213w\365\316:\325qDj*U\3264\262:V\207K\314\225\325\270 ^\217\260\242\034F`\032\213\246x\372~\307\272xt{e\030h\352\275\2107h\037\222\214\003\031\005\341B\n\\yr7}\337\231\324e\323\034{vW\237\222\346\341\000h\023\355\273\265\272\177\320\256a\001q\355\326\334\261\022W\260a]\245cXRxd\265\220\271Xw\254\3054 {\037\317Zw\033\016,\237\342\022\314\225!\256Y\022\367l?[,\244sl-\255?&\263\223\2027*\027h\273\355\236\316:\0374K\355y\250{\002r\245\021w[2O\354\346K%\341\216\225]E=PZ\"\265\345)\372\224S)\324\222\202\0369\342\"\327y\357\256~\217\334\004\244\021\200\307\246\256\311\3718[\333\210\314/\007\347\227\305v\364r\2114\214\247\251uCh5\t\n'\207E\320\2710\316\266\374\354\235\343\255\322\313\204t\336pK\246\217\245d\216w\276\301\242|\204\375\n?\rw\232\021\223\204\023\361%\230\002x\230\375<\244J\032d\335\225\004\007\264\257\026\312h\233\346\207\001<\375\271B\231\033\252j\344\350\226\017\217$r\261x\206\232\334R\232^\215\263M\273#L\2740\3331\314H\216\205z2mh\034nV\223\263E\330\221\373F\271\004\252\324\363\016\003\201\300+\253\211Dl\023.\252\234\221\034\033\006#\241\240\257\366\004\302m?)\327\307\274c\206\304\213r5\365\326\004\030\303\033,\2478K\tO\314Xw3\374\244`!\227u\020V\225\221\036\223\233a\216\222]\330\213D\317HM\231P\323\256>\266\\\311\303\024TVP\263y\345\222\036D\345\\\356\350\222~\226\3728\237]\235\322*\263\214\330%\355`\036\351\002g\355\013g\366\2053\301\217\025\271p\361\000\030lF\027\377\027\273\213s2\347\241^\334\207\236\341\237M'\010=\023^""\206\265\037\004k9\234\356\356\274\250\356sf\353 K\201\277o\303\346v\365\302\345\323\326\324\002\272\346T,\236\021\276\263\253\327\201&\227\376\362\312\304G>\274j[\342\356z\367m\365L,|\002\273l0\3363Y*f9\026w\233\306\311\020\331'\340\275oD\250\255\275\347\313[\230L\354\354\222\233tut\216\375\027\372\352\037h\352Z\220\2217+\360\303\356\021\006K\363\355r\266\267\032\232'HFW\013\205)\227\n\331j\355T\t\264G\211?,\320s\\r^c\234\350!\261Cw\365s\313\313\302\262\005\313\304\302\215|J0\357uK\267\273+\255\203\315\311\351\250\265Ma\277\346\340\034\302\216n\315\257\304\022\341\320|,\032\r\317'Q\260\303P\267\rl\357?\322\216l\273Z\020\222\204\333\341\235\263V\273\035\374_.\352\335\337\275\346\363\202\036=\026\264\335'a\205\243\245\243\005M-\206\202[,\266\300\266\302\301\365Pxnc1\024I\330\007c\016&#\253\341\330F\262\243\005qf<\026M\204[M\211\344z\271\300m\260\242\026\347\203\210\213]N\302\301\204(L\335X\301\320\255x\230u\256~Y\003\213\317\271\333\236m\254D\202Q\026\215\261\225p0\376\2416\014e\356\366\325\356\001\253\261hr\211m\206\303\313\264\346\207\332z\366\021\215!Z\356\251\021k\342Q\213\270\317\272\216P\313<\330*\327Ya\251[X)\027[\372\305\026\"\341\225\320\"7(\332\262\207\264\276\254\376K\301h(\261\024\\\016\343\226V\343+\341d\330\225\000h\305\324\216N-\221{\263\247\262\313\326D\221h$\031\t\256D~\016\207\"\361\370z,\031c M6\355\244\247\341\314\320%\331\225\330\342bx}5\234H\004\027\355\323c\250\230`5\022es\033\013\013\341u\226\300\274n)\256r-\313I\361\022\210\266\225\254MC\222j\010\2417\211/\212\235\301\332+v\304\036!\353B\364\205\247\343\353aL\271\036\016uN\336\025!\367\306\313\304H-\"\357\034\304\342\302B\022\235\264X\364N\251\005\247\033c\361\243C\374O\033cQ\030\225u\236\241\240\332=\231+s\002\3313\253\"\021c\353\363\317\261\317\004\327\362\220\344\033\234\320:pB\265]\2015\\|\315\213\334\217SG\234\265\353\351QU4\026\n\257\004\267:\031\255\220\372\374Rx~9\022]\264#sg\226\r\305r\343\351^.\266\031\\\217b\300\3176\273[\265\310\035\223E\017x\312\213\234%\305\361\364\277\210F\365V""TO\005\270\004\361K-\214\311tW\020\036\225\016\341(Xw\344\355l\305]/\374\274m\341A*\324\234\\*\035\361#\276\215\365u'\274\007\330\021SG\021\344tG\325\\\025y\005\025\272 \265\026\005l\345\026/#\216V\265\213t\\H\036\355h\336\t\352S\252\006\221\347\025\010\302\252\021\034\014U\314v\223\214\322\220\214\022Ei\226\246\023\226\211X\221\246\331\225%\241\261\314x\343\324\036\031\360\261)\231\002,\234l\227\333\304\324\341av\213\303\361\351z\230lXI\000;\025@\264\r\375\312p\316- [Dwb\025@\026\363\006/\322g+\303\200\242\302\031eW49\305\351\312EF\022\017\242`\335\227\036\220\267S\"v\325\370E\254\371\242\272\300\277\013*\266\250\377\007\005\271\270\235\226\377s9\233\276\274\345\003\263\270\271\366%\325\355\361Y\304\307\207%\255\02532[\221\333Z\331A%\205\373L\301MR\266\210\302C\203w0\024\273\302\242\246\"\251\000AR\256\202b\274\264c\377\031k\0273y^H\213[h\327\225\204\177\350\250\341zZ>\302?\246f\330\001\347\273\202E\211\037f\347`\230\353+\3153r\271\000EK\223\231\262\240\315F\021m\340\035\221\n\367\304\035\250\320\r\265\204\021j\212~\024N\372\211\256\004j&C\\\315\016.\251FP0kM\016W\005\356\232\3129\355|\017\217\013\377\320I\023D\226\037\202\206\013\255j\321q\326\3120\265w\327\231\203r%\2402r1_8\262\2311fj\225\254\27129!JK\240\224>\345]g\263o\245\207\272[\025\364>2\005U6\254\237.\002\357\326\311\336\364\344e\t\311\214\246\276\341\240Y\2312\254\206\370EH_\320\261\023[\351\360\350\360oE\315f\311\003\222)P\350\357\016x(j\311\211\336\350I\243\\\342\024ka\025z\251\026\213cYU\315\026x\300\311\003u}\006vt\310\030g)\312\206\035\276\331\304\274e\311\377<\262\374\323X\362\342\350\361\203\361\242\335\346\230\305\234\303K\354\355\031\274]cu\025\206\2379\226\237QV\366('\3539\227\203\310\247\031\313\027E\374$\324L\230\014,i\375\343\032\354\376>o\031\375\274Ef\230\223e\314\353\254m\001\304\207m\305\311(\352\035\326\336:\005\006w\326\331Gi%Bw\351\331\356\0360\372\213\220c\237(\207\326\326W\210\213\242\036\3614t\326.:\241\020\020\362\314\n\251\272EK\272!\024\244\240\225-\323\317\360X\320\301rz\0278""\010\334?\036&\345m\213\362!\245\254\234\353w\253\201\246;\337\342\217\027\014\236\207v\242\267\242<\364o\2310*\222\201\0229r|\252i\362b@\305\310\0014\025f\003\334F\214\322\263\364?\201x\227B?\235/\035\005\364\2614\333\376\345\007\326\017\204\205Wb\264\222\302\212z\240n\323\037]\310\237[\233\022&Y\305\245\247\271f\333M\013\204\017eN\272\336\362\250\216_\265\037\025>\r\3600G\003JV\336\321RY\352\243\341!\223)(Z\324\300\252RK\370\307\332\254\207\265u\213T\317Y\220\341\224\\xE\346\216\367-\217\316J\240\221)\342'\314.\347xjW/\027\255/\341<tR,L\242\261lY\326\322N\213.,;\225m#@E!WQ*+\245|j\027W\341\242x]m\335\241\241\325\274/r\225\264\243\275\262\\\260\356\342O\2239\330\221\234\356\370\264\254\254\356p(\373J\355\320\224\365\004\251\255\n\030%'\023tIB\25077\"\204sAR\310\252\3025\322\223q\236M\3132\t\235n}\221\344\234\017\324R\224A|\3146\304\360\275\205\214H\323v\246;.O\265|8\271\342\232\344\257$P\254>x)\242\320\251\240\326\337\211,\025t\362*Th]DgZE\004\376v\351\262D\232]\r\345Bi\033\334hW\254b\330\\\2022\007\250S\301\226\333Y\004vAF\001\006\314\320\333\2001\364\277\010\006,\376\345@gr\307\331CW\302\2477\305#jhc\264\246n\244\273H\t\270\206~\204e\251\r\2779M=\240W+\374\226\335\305\266\324\216\357\201\023\316C\003;]Q\201;\006\002\204\030\037P\020\353\257\261x\325\272\010\327\312\212.g\310j\352\271r\tF\220\023\237v$A\177\365\203w(s\361\243\323\275\036\310\032)\323\001\227\241\304\031!\033\242q\007y\345\301\267\177\226\262t\322\355\335i\367\370QL\230B\351\357\236\377\345?Ao<\260Y\036Oo\014\350i\347\305<\027f\305<\027\247\305<]y1\217\035\251FbQ\226H\006\223\210GEr\354\202z']\326\333\324\233\314\270\240Ogz\243\267\271#\215\321\333\350\n\322\235f\206EY\"\234\334\210;Q\266\247\235\336Z\333\010'\222\355zWb\317\343J\204Y=\355<\210\347\242\034\231\335\226\360\364\244\201<\356\024\217\247+\307\343i\313\305\331IO^\307\323\231\330\361\2702;\236\013\2228\036'a\023\335X\235\013\257\267R6\236\013\0239\236(\n\301Db3\266\036b\301\215\344\222s^\017\211\322\022n\302\023\217D\027\273\005\321\252""\355\226\201\235a\212\261\316\005=\275I\032OG\226\306\323\233v\361\270\323(\236\213\362(\236\256\214\210gs=\002e\010\3774\037\216\223r$<\354b\252\334\252\267M\205\307\316%x\272\202|ON\363\36420\217\3155<\313\341p\234A%\237\267\256\217\255\007\261\260\307\225\266\361\260\315H\364\301\267\364L]\365\346\310\277\032\303'\023\307\263\277\3675}\377\250\317\326\027\032S\246\357\233\372\313\223\211?F<\303#\177\014z\256|f~:\333\230h\374\363\364\372\351ls\354F\345\307*\340z\345ne\25796Q\231\020\005\377\265\343w\325\340\037~\317\225\361\3437\325\211\352\355\352Zs\374F\345!\232&?\253~_\233h\336\372\244y\353\363j\020x>>A\323<\242.\023\315\361\311J\2609~\263B\037\347c\343\346\370\355\332ls\\\252\3655'\277k\2545\270\371x\371l\366,bn<7\237o\322v\256\236\373G\217\037\036\357\321\034\337W'\332s\340\247\222\244\212\033\225\331\n\355e\342\313\352Z5W+4\2460nx\014\203\336U\027k\2634\311\2049~\277>a\317dW|]\223\317\375c\307\221J\242\3523?\017\324\323\215\251\306\303\306\336\311\320\211~:\325\034\273B\275\037\034\3138\337\365\352\354\037W=W\276\250.\325\302u\034f\2742\n\231\214\177\\\331\253\366\241#\332\206\257\034/W\304\026\261\025TU\374\346-\210\2679\371eUnN~U\273S\313\325Q\230\252M\326~\252\2575'\377V\205\240\244\232\257\266T\0176'\277\250&k\023\346\310\267\270\013\217dJ\300s\337\350\361\277I\3167\305\021\261\315P\245\257\351\277b^\301H\014\0136\375W\217\323\225)T\035'+\3500Y\tU\373\314\221\007\346\203\310\331\204\351\373{}\352\003b\303\261\3151\332\302\030mo\374Ac\266\021:\3513}_\325\354QF\205\256\334\274\216\2354\307\0376\202\215$\364\0032\235\301\221\256\340\356*\351\352tm\252\366mm\255&\2238\206\350\354\343\225\341\352\r\363s\322\235;\r\271\261\327\332\200\325\016q\355C\005&o6'oAl\364S\235\254\256\341\336n~U\233\250M\327g\0322>\304\275\375VMA\205\240tw\314\217\246\353^\363\037\320\211\346\370t\275\257>Y_\243\215\214\320\254W\253J=X\337j\244O \207k\346\265\333\326]OV\276\203(>ytr\347d\367\014\263|j~\032\200\360\307\257W\246*\017\315\217\037\235L\235\314\236\217}\\y\007)\332\nH%\034""\027%\323\367\235\371]\334\214\257\321\356\377S\235pz\230w~\200j\364\035O5o\375xr\3654\177\266\327\034\036\261%\006a\336\241\033\301q\315kwj\241\272\257\036;\371\341t\212\236\307U\022x\264\366\025\332oL\222\014\336\340`\266\014\350 W\240\177k\307\2666\21374\373{\260\371\351\277O\202\242\325\034#q\316\234\310\364\342\016*r\3050?\373\276Q8\235\352\354t\274Bj\3701\265}S_\303Q\307&+K\325`u\r\235&o\3761&\346\021;\034;\216V\247\240\272B\355\355f1\203\250\376=x~\375\006m\016;\035\0369\027\275\277\302+\207a\030?6\314\033$\377\317\033\355a\343\270'l\014\307\377\242\232\250\365A\2677\352S4\367J\325\207\227\357\2374'\241\207\365\207\365=\222\315\243\312f\325\022\270T\033\256OX=\305\002\230\360#\322F\230\237\353\236\341\233\225\220y\353\377N&N\246O\247\232\266\242\302:\210\031\240x\346\247\230\263y\345S\230\234\315\332jc\262\261yB\263V\306\252\3135\275\376U=x\016\265\231\242\347!\267\347=wFW\247\304\033m\372\361\370\235)\271\371\031\356\2669\366Q%U\375\250\272_[k\216]\023/`\3743\022\266\3340NP\276Eg:\027\235&\252w\240\237\223P\376\024Y\003zU\355\335\341\006\250*diw\363\n\031\t\222\206\350\346\372:\267F\n\313Z\275f\336{|2\213\243\010\313\272P\273\215m\240\270g\372\246j\023\347B\246B\006\3646\262\264ER\377\"\014x\320\364\301x\266.\013\227\336\007\275\033\270\n\263\3609\314\312\227u\243\361\337\323;\346\002\314\351\317\346\317\0313\263c\356\354\233\373o\376\360x~\355[\350\007,\364?#x\326\277F\260\326\277A\260\321\377\232\340u\177\232 \335\317\373\233\276\261\343E\230\267\205\352\014d\347\033\372}\377x\263\022\026\207\364\013\211\343l\347\3701\252\217k\331\372F\3436\214\306\027\247\331\263\rs}\303\334\3302\267\262fV1\025\335\324[\313\273\327}N\360\274\177\213`\253\377\025\301\253\376\034A\256_%P\373\r\002\243\377\200\340\240\377W\202_\373\027\274t\010\357*\301\252w\235`\335\273A\260\341\335\"\330\362\246\010R\336\035\202\035\257J\240z5\002\315\273O\260\357}K\360\326;\357\003\314\373\226\010\226|\313\004\313\276\030A\314\267E\260\345{M\360\332\307\t\270/G\220\363\025\010\n\276\367""\004\357}\341\001@x`a\200\344\264'\214\323,Tu\255\221=yq6t\0069\271j\372\332w\366e\3250o\177\337H\233\217\226\317\036\2321\353\276\336\321\244}st\334\271\376\027\004/\372\031\001s\004\243\023\350\375\021:D\304\021E\232 \355\315\021\344\274\357\t\336{\347h\207s\2768A\334\227$H\372^\022\274tNf\020\030\276C\202C\337\034\235e\016g!)\017$\t\222\003?\021\3744\220!\310\014\354\020\354\014\204\006\001\241\301%\202\245AF\300\006S\004\251\301C\202\303\301\267\004o\007\027\206h\262\241g\004\317\206\266\t\266\2072\004\231!\203\300\030:$8\034Z\361\003V\374q\202\270?E\220\362g\t\262\376}\202}\377\033\2027\376\3600\311|8B\020\031N\020$\2067\t6\207\177\032\376\353\332\373\244>T/7\346\033\206\371x\365l\323LBs_\232/_\233\2575S;2\217\336c\302\337\234\273\260\337\317/\004\277\340\215\220F\364\357\020\354\364\027\t\212\375{\004{\375OI\372O\275\213\004\213\336\025\202\025o\214 \346M\020$\274?\023\374\354\225\tdo\206 \343\315\023\344\275E\202\242\367)]\311S\337\"\301\242o\205`\305\321\316\004A\302\271\312\024A\312\321\316]\202]\237J\240\372\336\022\274\365\315\323\255\315\017,\021,\r,\023,\017\304\010b\003/\t^\016\244\010R\0039\202\334\300.\301\356\200J\240\016\330\327\034#\210\r&\t\222\203\233\004\233\203/\010^\014\356\021\3549w\377\216\340\335\340S\272\330\247C!\202\320\320*\301\352P\222 9\264E\2605\364\222\340\345\320k\202\327C\007\004\007C\357\t\336\017\315\323m\317\373\027\010\026\374\317\010\236\3717\t6\375\277\020\374\342hI\206 \343\337!\330\361\277#x\347\017\221B\204\034-Y!X\031\216\023\304\207\267\t\266\207\363\004\371a\205@\031\336#\330\033.\023\224\207\177#\370mxa\204V\037Y&X\036\211\022DG\326\t\326G^\020\274\030\331&\330\036\311\022dGv\010vF\024\002e\344\035\301\273\221\320(\355e4<\332\266\022\026\013\375\341\344\037\247\233g\301sw\315\232\351\271K\354\261\305\r'+\253\346\324\203\306\003\363\373\2453p?\370Es\004>\372x\212\3348|\370\304u\373\027Le\004L\205<\3050\261`8U\020?\323\007'H\036w\214x\313T\355\006\261\240k\344\350\357\327o 0\270C\304\212\310;\030\375l\363c\370\241\332""\335\332^\363\023r7\343\364Rf\005\221i\3639o\365\241p\325\202\317b6\270\251I\213\360\334\372\002D\357\252`\335\240b\365\275s\233\200\3645\277\3067\010\306\200\377w\343xVx\274\246o\240\351\237\200!\244\251F\307\354\272Qs\224\270\252\337\n.\034\000g\273W\333\253\017\202\363\370A,\356\302%g\352A\212mh\231\375\306\232\351\203@\316\005W\376\272:P]\353\340\244W\25321`r\365Oj^PG\004\0207+\261\332\303\332~\375\247\306\032\035\250\277z\307\374\362!q\343s4\255AV\023\267l\022\013\376s\363\023\233\306\332L\215v\322'\250w\323?,\210\316\327p\204\302U\373\210\323|\003\2478%B\236\273\265\027\215\276\206\305-\261?s$P\3177\366~\357;\367\201\256\230\322\277\032\243'\217Ow\315\265\365\246\357\0066\347\007\357\232 \035y[\371\001\004\306\377\317\372\236\371\257\247\247W\317`\310F)\224\273h\234y\343v\355G\342\007\035\003)\300x\260p\252\230\311\r1\222\242\024Lvq8\342\301\311\3161\326@\020\304\2112\t>\\\235\202tA\277\257\215\323?\320\333\221Q\034_\\_\253\333#DT\003`\204\026\313\033\273I\304\217\354\354\350\361\217\225\007\025^}@\014\206\002\207k\307\277\022\357\334W\363ii\272\235\217\232\221\376\3272\342\\\311Ni\357\346K\364\327\250\022\375\305\342\276\247w\352K\273^\022\265\273\203\366\377\007\2355@\322";
    PyObject *data = __Pyx_DecompressString(cstring, 5971, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (13439 bytes) */
const char* const bytes = "...An error that is raised when a connection is interrupted.Callable[[Any], None]Callable[[], None]Callable[[message.Message], bool] | NoneCleaning up connection to %sCleaning up connection to %s (error: %s)Connection can only be used once, connection is not in init stateConnection closedConnection is closed (Connection isn't established yet (Connection must be in HOST_RESOLVED state to start connectionConnection must be in SOCKET_OPENED state to finish connectionError connecting to Error while : Error writing packets: Expected 'Handshake failed: Handshake timed out after Incompatible API version (Invalid password!Invalid protobuf message: type=NoneNote that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.ParsedTimezoneProto | NonePing response not received after Read failedTimed out waiting to finish connect before disconnectingTimeout waiting for Timeout while connecting to ?),, : ).'.add_note(addresses= after aioesphomeapi._frame_helper.noiseaioesphomeapi._frame_helper.plain_textaioesphomeapi.api_pb2aioesphomeapi/connection.pyaioesphomeapi.coreaioesphomeapi.modelaioesphomeapi.posix_tzaioesphomeapi.timezoneaioesphomeapi.utilaioesphomeapi.zeroconfasyncio.Future[None]' but server sent a different name: ', client_info= connection:  connection cancelled data=disable due to fatal exception: enable, expected_mac=, expected_name=gcgetfullargspecisenabled, keepalive=list[hr.AddrInfo]list[message.Message]list[str]message.Message, noise_psk=, password=, port=%s: Connect task didn't finish before disconnect%s: Connecting to %s%s: Connection error occurred: %s%s: Error writing packets: %s%s: Failed to send (forced) disconnect request%s: Got message of type %s (len=%d): %s%s: Incompatible version %s! Closing connection%s: Invalid protobuf message: type=%s data=%s%s: Opened socket to %s:%s%s: PingResponse (pong) was not received since last keep alive after %s"" seconds; rescheduling keep alive%s: Ping response not received after %s seconds%s: Sending %s: %s%s: Skipping unknown message type %s%s: Successfully connected ('%s' API=%s.%s)%s: TCP_QUICKACK not supported%s: Unable to increase the socket receive buffer size to %s; The connection may unstable if the ESPHome device sends data at volume (ex. a Bluetooth proxy or camera): %s%s: disconnect request failed secondsstr | None, timezone=(tree fragment)tuple[message.Message, ...]tuple[type[Any], ...], zeroconf_manager=APIConnectionAPIConnectionCancelledErrorAPIConnectionErrorAPIConnection.__reduce_cython__APIConnection.__setstate_cython__APIConnection._async_pong_not_receivedAPIConnection._async_send_keep_aliveAPIConnection._connect_hello_loginAPIConnection._connect_init_frame_helperAPIConnection._connect_socket_connectAPIConnection._do_finish_connectAPIConnection._handle_disconnect_request_internalAPIConnection._handle_get_time_request_internalAPIConnection._handle_login_responseAPIConnection._handle_ping_request_internalAPIConnection._raise_fatal_connection_exceptionAPIConnection._remove_message_callbackAPIConnection._set_resolve_host_futureAPIConnection._wrap_fatal_connection_exceptionAPIConnection.add_message_callbackAPIConnection.disconnectAPIConnection.finish_connectionAPIConnection.force_disconnectAPIConnection.process_packetAPIConnection.report_fatal_errorAPIConnection.send_messageAPIConnection.send_message_await_responseAPIConnection.send_message_callback_responseAPIConnection.send_messagesAPIConnection.send_messages_await_response_complexAPIConnection.set_debugAPIConnection.set_log_nameAPIConnection.start_connectionAPIConnection.start_resolve_hostAPINoiseFrameHelperAPIPlaintextFrameHelperAPIVersionAnyAuthenticationRequestAuthenticationResponseBadNameAPIErrorBaseExceptionCLOSEDCONNECTEDCallableCancelledErrorConnectionInterruptedErrorConnectionNotEstablishedAPIErrorConnectionParamsConnectionParams.__reduce_cython__ConnectionParams.__setstate_cython__ConnectionStateC""opyFromDAY_OF_YEARDEBUGDISCONNECT_CONNECT_TIMEOUTDISCONNECT_RESPONSE_TIMEOUTDSTRuleDSTRuleParsedDSTRuleProtoDSTRuleTypeDST_RULE_TYPE_DAY_OF_YEARDST_RULE_TYPE_DAY_OF_YEAR_PBDST_RULE_TYPE_JULIAN_NO_LEAPDST_RULE_TYPE_JULIAN_NO_LEAP_PB_DST_RULE_TYPE_MAPDST_RULE_TYPE_MONTH_WEEK_DAYDST_RULE_TYPE_MONTH_WEEK_DAY_PBDST_RULE_TYPE_NONEDST_RULE_TYPE_NONE_PB_DataclassParamsDisconnectRequestDisconnectResponseEnumException_FIELDGetTimeRequestGetTimeResponseHANDSHAKE_COMPLETEHOST_RESOLVEDHandshakeAPIErrorHelloRequestHelloResponseINITIALIZEDIPPROTO_TCPInvalidAuthAPIErrorJULIAN_NO_LEAP_LOGGERMESSAGE_TYPE_TO_PROTOMIN_BUFFER_SIZEMONTH_WEEK_DAYMergeFromStringMessageToDictNONENoReturnNoneNotImplementedPREFERRED_BUFFER_SIZEParsedTimezoneParsedTimezoneProtoPingFailedAPIErrorPingRequestPingResponseProtocolAPIError__Pyx_PyDict_NextRefReadFailedAPIErrorSOCKET_OPENEDSOL_SOCKETSO_RCVBUFSerializeToStringSocketAPIErrorSocketClosedAPIErrorTCP_CONNECT_TIMEOUTTCP_NODELAYTCP_QUICKACKTYPE_CHECKINGTimeoutAPIErrorUnhandledAPIConnectionErrorWARNINGZeroconfManager_actionaddadd_message_callbackaddraddr_infosaddressesaddrs_addrs_info__aenter____aexit__aioesphomeapi.connectionaioesphomeapi.host_resolveraiohappyeyeballsapi_pb2api_versionapi_version_majorapi_version_minorastupleasync_interrupt_async_pong_not_receivedasync_resolve_host_async_send_keep_aliveasyncioasyncio.coroutinesasyncio_timeout__await__bool_build_dst_rule_proto_build_parsed_tz_proto_bytes_cached_make_hello_request_cached_timezonecall_atcancelcancelling__cause__cause__class____class_getitem__client_infocline_in_tracebackclosecollections.abccompare_connect_hello_login_connect_hello_login.<locals>.<lambda>_connect_init_frame_helper_connect_init_frame_helper.<locals>.<lambda>_connect_socket_connect_connect_socket_connect.<locals>.genexprconnected_addressconnectionconnection_statecopycorecreate_connectioncreate_futurecurrent_taskdatadataclass__dataclass_fields____dataclass_params__dataclassesdayday_of_weekdebugdebug_enabled_debug_enableddefault__d""ict___dictdisconnectdo_append_do_finish_connectdo_stop__doc__donedst_enddst_offset_secondsdst_startenableenumepoch_secondseqerrerr_strerrorexexc_infoexception_expected_disconnectexpected_macexpected_namefamilyfatal_exc_fatal_exceptionfhfield_field_type_finish_connect_futurefinish_connectionfinishing_floatfloatforce_disconnect_frame_helper_frame_helper.noise_frame_helper.plain_textfrozen__func__functoolsfutgenexprgetgetLoggerget_running_loopget_timezonegetpeername__getstate__google.protobufgoogle.protobuf.json_formathandle_complex_message_handle_disconnect_request_internal_handle_get_time_request_internal_handle_login_response_handle_ping_request_internalhandle_timeout_handshake_completehandshake_handlehappy_eyeballs_delayhashhost_resolverid__import__infoinit_intintinterleaveinterruptinvalid_passwordis_connected_is_coroutineitems_keep_alive_interval_keep_alive_timeoutkeepaliveklasskw_only<lambda>last_exceptionloglog_errors_log_errorslog_nameloggingloginlogin_responseloop_looplru_cache__main__major_make_hello_requestmatch_argsmaxsizemessage_message_handlersmessage_types_to_namesmessages__metaclass__metadatamodel__module__month__mro_entries__msg_msgmsg_type_protomsg_typesmsgs__name__name__new__new_excnextnoise_psknowobjecton_messageon_stoporderparams_paramsparse_posix_tzparsedparsed_timezonepartialpasswordpending_ping_timerplatform_pong_timerpoppop_addr_infos_interleaveportposix_tz__prepare__process_packetproto__pyx_capi____pyx_checksum__pyx_recursive_repr_guard__pyx_result__pyx_state__pyx_type__pyx_unpickle_APIConnection__pyx_unpickle_ConnectionParams__pyx_vtable____qualname___raise_fatal_connection_exception_read_exception_futuresreceived_name__reduce____reduce_cython____reduce_ex__remove_remove_message_callbackreport_fatal_errorrepr_resolve_host_futureresolvingrespresponseresponse_namesresponse_typeresponsesreturnrulerunningsselfsendsend_messagesend_message_await_responsesend_message_callback_responsesend_messagessend_messages_await_response_complexsend_msg_send_pen""ding_pingserver_infoset_debugset_exceptionset_log_name__set_name___set_resolve_host_futureset_resultsetblockingsetdefaultsetsockopt__setstate____setstate_cython__slots__slots__socksockaddrsocket_socket_start_connect_futurestart_connectionstart_resolve_hoststartingstatestd_offset_secondsstrsys__test__throwtimetime_secondstimeouttimeout_expiredtimeout_handletimezonetitletypetypingtz_stringunsafe_hashupdateuse_setstateutilvaluevalueswaitwarningweakref_slotweekwin32_wrap_fatal_connection_exceptionzeroconfzeroconf_managerPyObject *\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000float\000int\000\000APIConnectionError\000APIVersion\000AuthenticationRequest\000AuthenticationResponse\000BadNameAPIError\000CONNECTION_STATE_CLOSED\000CONNECTION_STATE_CONNECTED\000CONNECTION_STATE_HANDSHAKE_COMPLETE\000CONNECTION_STATE_HOST_RESOLVED\000CONNECTION_STATE_INITIALIZED\000CONNECTION_STATE_SOCKET_OPENED\000CONNECT_AND_SETUP_TIMEOUT\000CONNECT_REQUEST_TIMEOUT\000CancelledError\000DISCONNECT_REQUEST_MESSAGE\000DISCONNECT_RESPONSE_MESSAGES\000DisconnectRequest\000GetTimeRequest\000GetTimeResponse\000HANDSHAKE_TIMEOUT\000HandshakeAPIError\000HelloRequest\000HelloResponse\000InvalidAuthAPIError\000MESSAGE_NUMBER_TO_PROTO\000MESSAGE_TYPE_TO_PROTO\000NO_PASSWORD_AUTH_REQUEST\000OPEN_STATES\000PING_REQUEST_MESSAGES\000PING_RESPONSE_MESSAGES\000PROTO_TO_MESSAGE_TYPE\000PingFailedAPIError\000PingRequest\000ReadFailedAPIError\000SocketAPIError\000SocketClosedAPIError\000TimeoutAPIError\000WRITE_EXCEPTIONS\000_handle_complex_message\000_handle_timeout\000astuple\000asyncio_timeout\000hr\000make_hello_request\000partial\000KEEP_ALIVE_TIMEOUT_RATIO\000TYPE_CHECKING\000_WIN32int\000TYPE_CHECKING\320\n0\260\t\270\021\2101\200\001\330\004+\2501\250F\260!\320\004-\250\\\270\021\360\n\000\t\n\360\006\000\r\033\320\0321\260\021\260/\300\022\3001\330\014\023\2208\2301\330""\014\022\220%\220q\330\014\021\220\021\220%\220q\330\010\017\210}\230A\360\010\000\r\020\210z\230\021\230#\230Q\330\020\023\2204\220q\330\024\033\2306\240\021\330\030\031\330\030\034\230A\330\030\031\340\020\021\330\014\023\220:\230Q\330\020\021\330\020\024\220A\330\020\025\220Q\330\020\021\340\014\020\320\020#\2401\330\020 \240\001\330\0245\260Q\260e\320;K\3101\310I\320UV\320VW\360\006\000\r\016\340\010\013\2104\210q\330\014\023\2206\230\021\330\020\021\330\020\024\220A\330\020\024\220A\220T\230\021\330\020\023\2201\220A\360\010\000\021\036\230Q\230h\240l\260!\360\006\000\t\014\2104\210}\230G\2401\360\006\000\r\021\320\020)\250\021\340\010\013\2104\210q\360\006\000\r\021\320\020&\240a\340\010\014\210I\220S\230\004\320\034.\250d\260!\2604\260q\270\007\270s\300!\330\014\r\340\010\013\2103\210a\210z\230\022\2301\360\016\000\r\035\230H\240E\250\021\330\014\020\220\013\2301\330\020\027\220q\230\001\330\014\r\360\016\000\t\r\210K\220q\330\014\023\2201\220A\330\014\r\220\010\320\030+\2501\330\024\036\230a\330\024\"\240$\240h\250a\330\024!\240\024\240X\250Q\330\024\037\230q\330\024 \240\004\240H\250A\330\024\035\230T\240\021\320\n2\260!\320\000 \320 2\260!\340\004\013\210<\220q\330\010\025\220T\230\021\330\010\014\210D\220\001\330\010\r\320\r\037\230q\240\004\240A\330\010\016\210d\220!\330\010\r\210T\220\021\330\010\024\220D\230\001\320\n3\3203I\310\021\320\004*\250!\340\010\013\2104\210q\330\014\023\2206\230\021\330\020\021\330\020\024\220A\330\020\024\220A\340\010\014\320\014\037\230q\330\014\036\230a\330\0203\2601\260D\270\001\320\004\"\240!\340\010\013\2104\210t\2201\330\014\022\320\0222\260!\330\0204\260A\260T\270\021\360\006\000\t,\2501\330\r\025\220Q\220d\230(\240!\2402\240Q\240a\330\014\020\220\007\220q\330\014\020\220\t\230\023\320\0341\260\021\260$\260a\260q\340\010\013\2104\210q\330\014\020\220\007\220q\330\020\027\220v\230Q\330\024\025\330\024\030\230\001\330\024\030\230\001\230\024\230Q\360\010\000\025\"\240\021\240(\250,\260a\360\010\000\t\014\2104\210\177""\230c\240\021\330\014\022\220$\320\026(\250\003\320+K\3101\330\020(\250\001\250\024\250Q\360\006\000\t\n\330\014\020\220\016\230n\250A\250Y\260d\270!\330\010\017\320\017#\2401\360\006\000\r\024\2205\230\001\320\031:\270$\270k\310\021\330\014\032\320\032.\250a\330\020\022\220!\2204\320\027:\270!\2701\340\014\027\220}\240A\330\014\020\320\020#\2401\240A\330\014\022\320\022#\2401\320\0045\3205P\320PQ\340\010\013\210>\230\021\330\014\020\320\020#\2401\320$7\260q\270\001\210!\330\0308\270\016\300i\310q\330\t\n\320\004\"\240!\340\010\014\320\014$\240A\330\010\013\2104\320\017$\240D\250\004\250O\2707\300!\360\010\000\r\016\330\020\024\220N\240\"\240A\330\023\024\330\020\027\220z\240\021\330\024\025\330\024\030\230\001\360\006\000\t\r\210I\220Q\210A\330\020\024\220A\220Q\330\020\023\2201\200A\330\032<\270A\360\006\000\t\r\320\0141\260\021\260,\270a\330\010\017\210w\220a\220t\320\0336\260l\300!\200A\330\032<\270A\360\006\000\t\r\210L\230\001\330\014\027\220t\320\033-\250Q\250a\330\014\024\220H\230A\230Q\200A\330\024\025\360\014\000\t\r\320\014$\240A\330\010\014\210N\230!\2301\330\010\014\210I\220Q\200A\330\024\025\360\006\000\t\r\210N\230!\2301\200A\340\022\023\330\024\025\330\023\024\330\t\n\340\010\014\210N\230\"\230A\360\n\000\t\020\210t\320\023(\250\001\250\034\260Q\200A\330\024\025\360\006\000\t\020\210\177\230a\330\010\014\320\014\035\230S\240\001\240\024\240U\250!\330\010\014\210L\230\004\230A\330\010\024\320\024*\250!\2504\250q\330\010\013\210:\220W\230A\330\014\020\320\020 \240\t\250\021\250!\330\010\014\210N\230\"\230A\200A\330\026\037\230q\330\t\n\360\022\000\t\025\220D\320\0309\270\021\270(\300!\330\010\014\320\014\037\230q\240\001\330\010\013\210:\220Q\220d\320\032*\250!\330\r\032\230#\230W\240M\260\024\260W\270A\330\014\020\220\014\230K\240s\250\"\250A\340\014\022\220!\330\010\016\210a\200A\330\026\037\230q\330\t\n\340\010\013\210:\220Q\220d\230!\330\014\023\2201\330\010&\240a\330\010\013\210:\220Q\220e\320\0337\260q\330\014\026\220c\230\026\230v\240Q\330\014""\017\210t\2201\330\020\033\320\0336\260a\260t\2701\330\020\030\230\004\230A\340\014\026\220c\230\021\230$\230c\240\024\240Q\240c\250\021\330\014\024\220A\330\010\013\210:\220Q\220d\320\032-\250Q\330\014\024\220D\230\001\230\024\230Q\330\r\027\220q\230\004\230A\330\014\024\220A\330\r\027\220q\230\004\230A\330\014\024\220A\340\014\024\220A\330\010\022\220%\220q\230\017\320';\2701\270A\330\010\017\210}\230F\240#\240Q\330\010\017\210q\320\004!\240\021\340\010\014\210L\230\001\330\010\013\2104\210\177\230g\240Q\330\014\020\220\016\230m\2501\250A\320\004 \240\001\340\010\014\210N\230\"\230A\230Q\200\001\360\010\000\005\016\210T\220\034\230T\240\036\250t\260?\300$\320FV\320VZ\320Zf\320fj\320jv\320vz\360\000\000{\001F\002\360\000\000F\002J\002\360\000\000J\002Q\002\360\000\000Q\002U\002\360\000\000U\002`\002\360\000\000`\002d\002\360\000\000d\002e\002\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\230;\240g\250U\260#\260T\270\035\300g\310U\320RU\320UY\320Yg\320gn\320ns\320sv\320vz\360\000\000{\001J\002\360\000\000J\002Q\002\360\000\000Q\002V\002\360\000\000V\002Y\002\360\000\000Y\002]\002\360\000\000]\002h\002\360\000\000h\002o\002\360\000\000o\002t\002\360\000\000t\002w\002\360\000\000w\002{\002\360\000\000{\002F\003\360\000\000F\003M\003\360\000\000M\003R\003\360\000\000R\003U\003\360\000\000U\003Y\003\360\000\000Y\003c\003\360\000\000c\003j\003\360\000\000j\003o\003\360\000\000o\003r\003\360\000\000r\003v\003\360\000\000v\003|\003\360\000\000|\003C\004\360\000\000C\004H\004\360\000\000H\004K\004\360\000\000K\004O\004\360\000\000O\004Y\004\360\000\000Y\004`\004\360\000\000`\004e\004\360\000\000e\004h\004\360\000\000h\004l\004\360\000\000l\004~\004\360\000\000~\004E\005\360\000\000E\005F\005\330\004\007\200q\330\010\017\320\0171\260\024\260Q\260g\270[\310\007\310q\340\010\017\320\0171\260\024\260Q\260g\270[\310\001\200\001\360\010\000\005\016\210T\220\036\230t\320#6\260d\320:K\3104\320Of\320fj""\320j}\360\000\000~\001B\002\360\000\000B\002[\002\360\000\000[\002_\002\360\000\000_\002o\002\360\000\000o\002s\002\360\000\000s\002I\003\360\000\000I\003M\003\360\000\000M\003d\003\360\000\000d\003h\003\360\000\000h\003~\003\360\000\000~\003B\004\360\000\000B\004P\004\360\000\000P\004T\004\360\000\000T\004\\\004\360\000\000\\\004`\004\360\000\000`\004t\004\360\000\000t\004x\004\360\000\000x\004B\005\360\000\000B\005F\005\360\000\000F\005T\005\360\000\000T\005X\005\360\000\000X\005f\005\360\000\000f\005j\005\360\000\000j\005D\006\360\000\000D\006H\006\360\000\000H\006_\006\360\000\000_\006c\006\360\000\000c\006x\006\360\000\000x\006|\006\360\000\000|\006F\007\360\000\000F\007J\007\360\000\000J\007b\007\360\000\000b\007f\007\360\000\000f\007t\007\360\000\000t\007x\007\360\000\000x\007L\010\360\000\000L\010P\010\360\000\000P\010c\010\360\000\000c\010g\010\360\000\000g\010v\010\360\000\000v\010z\010\360\000\000z\010E\t\360\000\000E\tI\t\360\000\000I\tS\t\360\000\000S\tW\t\360\000\000W\tX\t\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\027\220t\230=\250\007\250u\260C\260t\320;M\310W\320TY\320Y\\\320\\`\320`r\320ry\320y~\360\000\000\177\001B\002\360\000\000B\002F\002\360\000\000F\002^\002\360\000\000^\002e\002\360\000\000e\002j\002\360\000\000j\002m\002\360\000\000m\002q\002\360\000\000q\002@\003\360\000\000@\003G\003\360\000\000G\003L\003\360\000\000L\003O\003\360\000\000O\003S\003\360\000\000S\003Z\003\360\000\000Z\003a\003\360\000\000a\003f\003\360\000\000f\003i\003\360\000\000i\003m\003\360\000\000m\003@\004\360\000\000@\004G\004\360\000\000G\004L\004\360\000\000L\004O\004\360\000\000O\004S\004\360\000\000S\004\\\004\360\000\000\\\004c\004\360\000\000c\004h\004\360\000\000h\004k\004\360\000\000k\004o\004\360\000\000o\004|\004\360\000\000|\004C\005\360\000\000C\005H\005\360\000\000H\005K\005\360\000\000K\005O\005\360\000\000O\005\\\005\360\000\000\\\005c\005\360\000\000c\005h\005\360\000\000h""\005k\005\360\000\000k\005o\005\360\000\000o\005H\006\360\000\000H\006O\006\360\000\000O\006T\006\360\000\000T\006W\006\360\000\000W\006[\006\360\000\000[\006q\006\360\000\000q\006x\006\360\000\000x\006}\006\360\000\000}\006@\007\360\000\000@\007D\007\360\000\000D\007M\007\360\000\000M\007T\007\360\000\000T\007Y\007\360\000\000Y\007\\\007\360\000\000\\\007`\007\360\000\000`\007w\007\360\000\000w\007~\007\360\000\000~\007C\010\360\000\000C\010F\010\360\000\000F\010J\010\360\000\000J\010W\010\360\000\000W\010^\010\360\000\000^\010c\010\360\000\000c\010f\010\360\000\000f\010j\010\360\000\000j\010}\010\360\000\000}\010D\t\360\000\000D\tI\t\360\000\000I\tL\t\360\000\000L\tP\t\360\000\000P\tb\t\360\000\000b\ti\t\360\000\000i\tn\t\360\000\000n\tq\t\360\000\000q\tu\t\360\000\000u\t\177\t\360\000\000\177\tF\n\360\000\000F\nK\n\360\000\000K\nN\n\360\000\000N\nR\n\360\000\000R\n[\n\360\000\000[\nb\n\360\000\000b\ng\n\360\000\000g\nj\n\360\000\000j\nn\n\360\000\000n\n}\n\360\000\000}\nD\013\360\000\000D\013E\013\330\004\007\200q\330\010\017\320\017.\250d\260!\2607\270+\300W\310A\340\010\017\320\017.\250d\260!\2607\270+\300Q\320\000%\240X\250Q\340\004\013\210<\220q\330\010\024\220M\320!3\2603\3206H\310\001\320\n*\250!\320\n(\250\001\210!\340\022\023\330\023\024\330\021\022\330\023\024\330\021\022\330\t\n\320\n\"\240!\320\004 \240\t\250\021\340\010\014\320\014\036\230a\320\004&\240a\360\022\000\t\014\2104\320\017!\240\023\240A\330\014\017\210t\320\023)\250\023\250F\260$\260d\270!\340\020\021\330\024\033\2301\330\027!\240\021\240%\240q\330\031 \240\001\340\020\027\220t\2301\330\024\025\330\024\025\330\024\030\230\001\330\024\030\230\003\2304\230q\240\001\330\024\035\230T\240\023\240A\240Q\330\024\027\220z\240\021\330\030\035\230Q\360\016\000\r\021\320\020.\250a\250q\340\010\014\210I\220Q\200\001\330&.\250a\360\n\000\005\010\200t\2101\330\010\017\210q\330\004\005\330\010\021\220\036\230q\240\001\330\013\014\330\010\017\210q\330\004\013\320\013\036\230a\330\010\033\2306\240\021\330""\010\033\2306\240\021\330\010\022\320\022'\240q\250\006\250a\330\010\020\320\020%\240Q\240f\250A\200\001\330\004.\250a\250v\260Q\320\004(\250\001\340\010\016\210d\220&\230\005\230Q\340\010\013\2104\210q\330\014\020\220\016\230a\230q\330\014\017\210t\220=\240\003\2401\360\010\000\021\025\220O\2404\240v\250X\260Q\330\024\030\230\002\230$\320\0364\260D\270\001\340\021\025\220Q\360\022\000\021\030\220v\230Q\330\024\025\360\006\000\025\031\230\001\330\024\030\230\001\360\006\000\t\r\320\014'\240q\250\001\320\004*\250!\330\010\t\330\014\020\320\020&\240g\250Q\330\014\020\220\004\220D\320\030-\250U\260!\340\014\020\320\020%\240[\260\001\260\021\330\014\020\320\020(\250\001\320\n.\250i\260q\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\220=\240\010\250\001\250\021\330\004\007\200|\2207\230!\330\010/\250q\3200@\300\016\310a\330\004\013\2101\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\320\023#\2408\2501\250A\330\004\007\200|\2207\230!\330\0102\260!\3203F\300n\320TU\330\004\013\2101\220\010\320\030/\250q\330\024\037\230q\330\024 \240\004\240H\250A\330\024\035\230T\240\021\320\000\030\230\001\340\004\007\200t\2103\210e\2201\330\010\013\210>\230\021\230!\200\001\330\t\n\330\017\020\330\017\020\330\r\016\330\n\013\360\006\000\005\010\200t\2103\210e\2201\330\010\013\210:\220S\230\005\230S\240\t\250\021\250!\330\014\025\220W\230A\230Q\330\010\013\2108\2203\220e\2303\230g\240Q\240a\330\014\017\210{\230!\2301void (PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int __pyx_skip_dispatch)\000void (PyObject *, int __pyx_skip_dispatch)\000handle_complex_message\000handle_timeout";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 521; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 98) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 521; i < 568; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 568; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 521;
      for (Py_ssize_t i=0; i<47; ++i) {
        #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        #if PY_VERSION_HEX < 0x030E0000
        if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
        #else
        if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
        #endif
        {
          Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
        }
        #else
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab;
    double const c_constants[] = {0.1,5.0,10.0,30.0,60.0};
    for (int i = 0; i < 5; i++) {
      numbertab[i] = PyFloat_FromDouble(c_constants[i]);
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 5;
    int8_t const cint_constants_1[] = {0,1,2,3,4,5,8,14,16};
    int32_t const cint_constants_4[] = {131072L,2097152L,180946995L,194108219L};
    for (int i = 0; i < 13; i++) {
      numbertab[i] = PyLong_FromLong((i < 9 ? cint_constants_1[i - 0] : cint_constants_4[i - 9]));
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<18; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      #if PY_VERSION_HEX < 0x030E0000
      if (_Py_IsOwnedByCurrentThread(table[i]) && Py_REFCNT(table[i]) == 1)
      #else
      if (PyUnstable_Object_IsUniquelyReferenced(table[i]))
      #endif
      {
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      }
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 3;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 1;
    unsigned int nlocals : 4;
    unsigned int flags : 10;
    unsigned int first_line : 11;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 68};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_msg};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_genexpr, __pyx_mstate->__pyx_kp_b_iso88591_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_GENERATOR), 392};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_addr};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_genexpr, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 10, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 386};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_addrs, __pyx_mstate->__pyx_n_u_addr_infos, __pyx_mstate->__pyx_n_u_last_exception, __pyx_mstate->__pyx_n_u_sock, __pyx_mstate->__pyx_n_u_interleave, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_genexpr, __pyx_mstate->__pyx_n_u_genexpr, __pyx_mstate->__pyx_n_u_addr};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_connect_socket_connect, __pyx_mstate->__pyx_kp_b_iso88591_33I, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 481};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_fh, __pyx_mstate->__pyx_n_u__11, __pyx_mstate->__pyx_n_u_handshake_handle, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_noise_psk};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_connect_init_frame_helper, __pyx_mstate->__pyx_kp_b_iso88591_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 0, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 489};
    PyObject* const varnames[] = {0};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_lambda, __pyx_mstate->__pyx_kp_b_iso88591_q_q_HA_T, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {0, 0, 0, 0, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 498};
    PyObject* const varnames[] = {0};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_lambda, __pyx_mstate->__pyx_kp_b_iso88591_1_a_ha_XQ_q_HA_T, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 529};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_login, __pyx_mstate->__pyx_n_u_messages, __pyx_mstate->__pyx_n_u_msg_types, __pyx_mstate->__pyx_n_u_responses, __pyx_mstate->__pyx_n_u_resp};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_connect_hello_login, __pyx_mstate->__pyx_kp_b_iso88591_0, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 542};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_resp};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_lambda, __pyx_mstate->__pyx_kp_b_iso88591_A_AQ_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 649};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ex};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_start_resolve_host, __pyx_mstate->__pyx_kp_b_iso88591__12, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 687};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ex};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_start_connection, __pyx_mstate->__pyx_kp_b_iso88591__13, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 768};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_login};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_do_finish_connect, __pyx_mstate->__pyx_kp_b_iso88591_iq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 1, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 783};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_login, __pyx_mstate->__pyx_n_u_ex};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_finish_connection, __pyx_mstate->__pyx_kp_b_iso88591_0, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {6, 0, 0, 13, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 917};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_messages, __pyx_mstate->__pyx_n_u_do_append, __pyx_mstate->__pyx_n_u_do_stop, __pyx_mstate->__pyx_n_u_msg_types, __pyx_mstate->__pyx_n_u_timeout, __pyx_mstate->__pyx_n_u_fut, __pyx_mstate->__pyx_n_u_responses, __pyx_mstate->__pyx_n_u_on_message, __pyx_mstate->__pyx_n_u_timeout_handle, __pyx_mstate->__pyx_n_u_timeout_expired, __pyx_mstate->__pyx_n_u_err, __pyx_mstate->__pyx_n_u_response_names};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_send_messages_await_response_com, __pyx_mstate->__pyx_kp_b_iso88591__14, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 972};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_send_msg, __pyx_mstate->__pyx_n_u_response_type, __pyx_mstate->__pyx_n_u_timeout, __pyx_mstate->__pyx_n_u_response};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_send_message_await_response, __pyx_mstate->__pyx_kp_b_iso88591_8_iq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_COROUTINE), 1151};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u__11, __pyx_mstate->__pyx_n_u_pending};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_disconnect, __pyx_mstate->__pyx_kp_b_iso88591__15, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_T_t_FVVZZffjjvvz_F_F_J_J_Q_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_avQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 163};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_client_info};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_make_hello_request, __pyx_mstate->__pyx_kp_b_iso88591_XQ_q_M_336H, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 181};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_rule};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_build_dst_rule_proto, __pyx_mstate->__pyx_kp_b_iso88591_2_q_T_D_q_A_d_T_D, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 193};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_tz_string, __pyx_mstate->__pyx_n_u_parsed};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_build_parsed_tz_proto, __pyx_mstate->__pyx_kp_b_iso88591_a_t1_q_q_q_a_6_6_q_a_QfA, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 213};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_fut};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_handle_timeout, __pyx_mstate->__pyx_kp_b_iso88591_t3e1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {5, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 222};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_fut, __pyx_mstate->__pyx_n_u_responses, __pyx_mstate->__pyx_n_u_do_append, __pyx_mstate->__pyx_n_u_do_stop, __pyx_mstate->__pyx_n_u_resp};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_handle_complex_message, __pyx_mstate->__pyx_kp_b_iso88591_t3e1_S_S_WAQ_83e3gQa_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 326};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name_2};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_set_log_name, __pyx_mstate->__pyx_kp_b_iso88591_L_4_gQ_m1A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 382};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_enable};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_set_debug, __pyx_mstate->__pyx_kp_b_iso88591_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 553};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_login_response};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_handle_login_response, __pyx_mstate->__pyx_kp_b_iso88591_55PPQ_1_7q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 597};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_now};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_async_send_keep_alive, __pyx_mstate->__pyx_kp_b_iso88591_d_Q_4q_aq_t_1_O4vXQ_4D_Q_vQ_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 635};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_async_pong_not_received, __pyx_mstate->__pyx_kp_b_iso88591_4q_6_A_A_q_a_31D, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 679};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_set_resolve_host_future, __pyx_mstate->__pyx_kp_b_iso88591_gQ_D_U, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 721};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_action, __pyx_mstate->__pyx_n_u_ex, __pyx_mstate->__pyx_n_u_fatal_exc, __pyx_mstate->__pyx_n_u_current_task};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_raise_fatal_connection_exceptio, __pyx_mstate->__pyx_kp_b_iso88591_A_q_D_9_q_Qd_WM_WA_Ks_A_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 7, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 741};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_action, __pyx_mstate->__pyx_n_u_ex, __pyx_mstate->__pyx_n_u_cause_2, __pyx_mstate->__pyx_n_u_err_str, __pyx_mstate->__pyx_n_u_klass, __pyx_mstate->__pyx_n_u_new_exc};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_wrap_fatal_connection_exception, __pyx_mstate->__pyx_kp_b_iso88591_A_q_Qd_1_a_Qe_7q_c_vQ_t1_6at1_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 831};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_msg};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_send_message, __pyx_mstate->__pyx_kp_b_iso88591_N_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 835};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_msgs};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_send_messages, __pyx_mstate->__pyx_kp_b_iso88591_4t1_2_4AT_1_Qd_2Qa_q_1_aq_4q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 888};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_on_message, __pyx_mstate->__pyx_n_u_msg_types};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_add_message_callback, __pyx_mstate->__pyx_kp_b_iso88591_A_A_1_a_wat_6l, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 895};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_on_message, __pyx_mstate->__pyx_n_u_msg_types};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_remove_message_callback, __pyx_mstate->__pyx_kp_b_iso88591_A_A_L_t_Qa_HAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 903};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_send_msg, __pyx_mstate->__pyx_n_u_on_message, __pyx_mstate->__pyx_n_u_msg_types};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_send_message_callback_response, __pyx_mstate->__pyx_kp_b_iso88591_A_N_A_t_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 984};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_err};
    __pyx_mstate_global->__pyx_codeobj_tab[35] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_report_fatal_error, __pyx_mstate->__pyx_kp_b_iso88591_a_4_A_t_F_d_1_q_t1_4q_T_AQ_z_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[35])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1024};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_msg_type_proto, __pyx_mstate->__pyx_n_u_data_2};
    __pyx_mstate_global->__pyx_codeobj_tab[36] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_process_packet, __pyx_mstate->__pyx_kp_b_iso88591_1_1_81_q_q_A_z_Q_4q_6_A_Q_A_Q_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[36])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1122};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_msg_2};
    __pyx_mstate_global->__pyx_codeobj_tab[37] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_handle_disconnect_request_inter, __pyx_mstate->__pyx_kp_b_iso88591_A_A_N_1_IQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[37])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1133};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_msg_2};
    __pyx_mstate_global->__pyx_codeobj_tab[38] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_handle_ping_request_internal, __pyx_mstate->__pyx_kp_b_iso88591_A_N_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[38])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1139};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_msg_2};
    __pyx_mstate_global->__pyx_codeobj_tab[39] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_handle_get_time_request_interna, __pyx_mstate->__pyx_kp_b_iso88591_A_a_S_U_L_A_4q_WA_N_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[39])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1190};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[40] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_aioesphomeapi_connection_py, __pyx_mstate->__pyx_n_u_force_disconnect, __pyx_mstate->__pyx_kp_b_iso88591_A_4_D_O7_N_A_z_IQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[40])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[41] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_t_6d_K4Offjj_B_B_____o_o_s_s, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[41])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[42] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_1F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[42])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[43] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle_ConnectionParams, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_81A_7_2_3FnTU_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[43])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[44] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_tree_fragment, __pyx_mstate->__pyx_n_u_pyx_unpickle_APIConnection, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_7_q0_a_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[44])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* Coroutine.init */
  if (likely(__pyx_Coroutine_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* Coro_CheckExact.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedCoroType = PyObject_GetAttrString(typesModule, "CoroutineType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* Generator.init */
  if (likely(__pyx_Generator_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* IterFinish (used by dict_iter) */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall (used by PyObjectCallNoArg) */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* PyObjectCallNoArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallOneArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod0 (used by dict_iter) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* RaiseNeedMoreValuesToUnpack (used by UnpackTuple2) */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseTooManyValuesToUnpack (used by UnpackItemEndCheck) */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* UnpackItemEndCheck (used by UnpackTuple2) */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseNoneIterError (used by UnpackTupleError) */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* UnpackTupleError (used by UnpackTuple2) */
static void __Pyx_UnpackTupleError(PyObject *t, Py_ssize_t index) {
    if (t == Py_None) {
      __Pyx_RaiseNoneNotIterableError();
    } else {
      Py_ssize_t size = __Pyx_PyTuple_GET_SIZE(t);
 #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(size < 0)) return;
 #endif
      if (size < index) {
        __Pyx_RaiseNeedMoreValuesError(size);
      } else {
        __Pyx_RaiseTooManyValuesError(index);
      }
    }
}

/* UnpackTuple2 (used by dict_iter) */
static CYTHON_INLINE int __Pyx_unpack_tuple2(
        PyObject* tuple, PyObject** value1, PyObject** value2, int is_tuple, int has_known_size, int decref_tuple) {
    if (likely(is_tuple || PyTuple_Check(tuple))) {
        Py_ssize_t size;
        if (has_known_size) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        size = __Pyx_PyTuple_GET_SIZE(tuple);
        if (likely(size == 2)) {
            return __Pyx_unpack_tuple2_exact(tuple, value1, value2, decref_tuple);
        }
        if (size >= 0) {
            __Pyx_UnpackTupleError(tuple, 2);
        }
        return -1;
    } else {
        return __Pyx_unpack_tuple2_generic(tuple, value1, value2, has_known_size, decref_tuple);
    }
}
static CYTHON_INLINE int __Pyx_unpack_tuple2_exact(
        PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2, int decref_tuple) {
    PyObject *value1 = NULL, *value2 = NULL;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
    value1 = __Pyx_PySequence_ITEM(tuple, 0);  if (unlikely(!value1)) goto bad;
    value2 = __Pyx_PySequence_ITEM(tuple, 1);  if (unlikely(!value2)) goto bad;
#else
    value1 = PyTuple_GET_ITEM(tuple, 0);  Py_INCREF(value1);
    value2 = PyTuple_GET_ITEM(tuple, 1);  Py_INCREF(value2);
#endif
    if (decref_tuple) {
        Py_DECREF(tuple);
    }
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
bad:
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
#endif
}
static int __Pyx_unpack_tuple2_generic(PyObject* tuple, PyObject** pvalue1, PyObject** pvalue2,
                                       int has_known_size, int decref_tuple) {
    Py_ssize_t index;
    PyObject *value1 = NULL, *value2 = NULL, *iter = NULL;
    iternextfunc iternext;
    iter = PyObject_GetIter(tuple);
    if (unlikely(!iter)) goto bad;
    if (decref_tuple) { Py_DECREF(tuple); tuple = NULL; }
    iternext = __Pyx_PyObject_GetIterNextFunc(iter);
    value1 = iternext(iter); if (unlikely(!value1)) { index = 0; goto unpacking_failed; }
    value2 = iternext(iter); if (unlikely(!value2)) { index = 1; goto unpacking_failed; }
    if (!has_known_size && unlikely(__Pyx_IternextUnpackEndCheck(iternext(iter), 2))) goto bad;
    Py_DECREF(iter);
    *pvalue1 = value1;
    *pvalue2 = value2;
    return 0;
unpacking_failed:
    if (!has_known_size && __Pyx_IterFinish() == 0)
        __Pyx_RaiseNeedMoreValuesError(index);
bad:
    Py_XDECREF(iter);
    Py_XDECREF(value1);
    Py_XDECREF(value2);
    if (decref_tuple) { Py_XDECREF(tuple); }
    return -1;
}

/* dict_iter */
#if CYTHON_COMPILING_IN_PYPY
#include <string.h>
#endif
static CYTHON_INLINE PyObject* __Pyx_dict_iterator(PyObject* iterable, int is_dict, PyObject* method_name,
                                                   Py_ssize_t* p_orig_length, int* p_source_is_dict) {
    is_dict = is_dict || likely(PyDict_CheckExact(iterable));
    *p_source_is_dict = is_dict;
    if (is_dict) {
#if !CYTHON_COMPILING_IN_PYPY
        *p_orig_length = PyDict_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
#else
        static PyObject *py_items = NULL, *py_keys = NULL, *py_values = NULL;
        PyObject **pp = NULL;
        if (method_name) {
            const char *name = PyUnicode_AsUTF8(method_name);
            if (strcmp(name, "iteritems") == 0) pp = &py_items;
            else if (strcmp(name, "iterkeys") == 0) pp = &py_keys;
            else if (strcmp(name, "itervalues") == 0) pp = &py_values;
            if (pp) {
                if (!*pp) {
                    *pp = PyUnicode_FromString(name + 4);
                    if (!*pp)
                        return NULL;
                }
                method_name = *pp;
            }
        }
#endif
    }
    *p_orig_length = 0;
    if (method_name) {
        PyObject* iter;
        iterable = __Pyx_PyObject_CallMethod0(iterable, method_name);
        if (!iterable)
            return NULL;
#if !CYTHON_COMPILING_IN_PYPY
        if (PyTuple_CheckExact(iterable) || PyList_CheckExact(iterable))
            return iterable;
#endif
        iter = PyObject_GetIter(iterable);
        Py_DECREF(iterable);
        return iter;
    }
    return PyObject_GetIter(iterable);
}
#if !CYTHON_AVOID_BORROWED_REFS
static CYTHON_INLINE int __Pyx_dict_iter_next_source_is_dict(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem) {
    PyObject *key, *value;
    if (unlikely(orig_length != PyDict_Size(iter_obj))) {
        PyErr_SetString(PyExc_RuntimeError, "dictionary changed size during iteration");
        return -1;
    }
    if (unlikely(!PyDict_Next(iter_obj, ppos, &key, &value))) {
        return 0;
    }
    if (pitem) {
        PyObject* tuple = PyTuple_New(2);
        if (unlikely(!tuple)) {
            return -1;
        }
        Py_INCREF(key);
        Py_INCREF(value);
        #if CYTHON_ASSUME_SAFE_MACROS
        PyTuple_SET_ITEM(tuple, 0, key);
        PyTuple_SET_ITEM(tuple, 1, value);
        #else
        if (unlikely(PyTuple_SetItem(tuple, 0, key) < 0)) {
            Py_DECREF(value);
            Py_DECREF(tuple);
            return -1;
        }
        if (unlikely(PyTuple_SetItem(tuple, 1, value) < 0)) {
            Py_DECREF(tuple);
            return -1;
        }
        #endif
        *pitem = tuple;
    } else {
        if (pkey) {
            Py_INCREF(key);
            *pkey = key;
        }
        if (pvalue) {
            Py_INCREF(value);
            *pvalue = value;
        }
    }
    return 1;
}
#endif
static CYTHON_INLINE int __Pyx_dict_iter_next(
        PyObject* iter_obj, CYTHON_NCP_UNUSED Py_ssize_t orig_length, CYTHON_NCP_UNUSED Py_ssize_t* ppos,
        PyObject** pkey, PyObject** pvalue, PyObject** pitem, int source_is_dict) {
    PyObject* next_item;
#if !CYTHON_AVOID_BORROWED_REFS
    if (source_is_dict) {
        int result;
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_BEGIN_CRITICAL_SECTION(iter_obj);
#endif
        result = __Pyx_dict_iter_next_source_is_dict(iter_obj, orig_length, ppos, pkey, pvalue, pitem);
#if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_END_CRITICAL_SECTION();
#endif
        return result;
    } else if (PyTuple_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(tuple_size < 0)) return -1;
        #endif
        if (unlikely(pos >= tuple_size)) return 0;
        *ppos = pos + 1;
        #if CYTHON_ASSUME_SAFE_MACROS
        next_item = PyTuple_GET_ITEM(iter_obj, pos);
        #else
        next_item = PyTuple_GetItem(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
        #endif
        Py_INCREF(next_item);
    } else if (PyList_CheckExact(iter_obj)) {
        Py_ssize_t pos = *ppos;
        Py_ssize_t list_size = __Pyx_PyList_GET_SIZE(iter_obj);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(list_size < 0)) return -1;
        #endif
        if (unlikely(pos >= list_size)) return 0;
        *ppos = pos + 1;
        next_item = __Pyx_PyList_GetItemRef(iter_obj, pos);
        if (unlikely(!next_item)) return -1;
    } else
#endif
    {
        next_item = PyIter_Next(iter_obj);
        if (unlikely(!next_item)) {
            return __Pyx_IterFinish();
        }
    }
    if (pitem) {
        *pitem = next_item;
    } else if (pkey && pvalue) {
        if (__Pyx_unpack_tuple2(next_item, pkey, pvalue, source_is_dict, source_is_dict, 1))
            return -1;
    } else if (pkey) {
        *pkey = next_item;
    } else {
        *pvalue = next_item;
    }
    return 1;
}

/* RaiseUnboundLocalError */
static void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* GetException (used by pep479) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* pep479 */
static void __Pyx_Generator_Replace_StopIteration(int in_async_gen) {
    PyObject *exc, *val, *tb, *cur_exc, *new_exc;
    __Pyx_PyThreadState_declare
    int is_async_stopiteration = 0;
    CYTHON_MAYBE_UNUSED_VAR(in_async_gen);
    __Pyx_PyThreadState_assign
    cur_exc = __Pyx_PyErr_CurrentExceptionType();
    if (likely(!__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopIteration))) {
        if (in_async_gen && unlikely(__Pyx_PyErr_GivenExceptionMatches(cur_exc, PyExc_StopAsyncIteration))) {
            is_async_stopiteration = 1;
        } else {
            return;
        }
    }
    __Pyx_GetException(&exc, &val, &tb);
    Py_XDECREF(exc);
    Py_XDECREF(tb);
    new_exc = PyObject_CallFunction(PyExc_RuntimeError, "s",
        is_async_stopiteration ? "async generator raised StopAsyncIteration" :
        in_async_gen ? "async generator raised StopIteration" :
        "generator raised StopIteration");
    if (!new_exc) {
        Py_XDECREF(val);
        return;
    }
    PyException_SetCause(new_exc, val); // steals ref to val
    PyErr_SetObject(PyExc_RuntimeError, new_exc);
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
        Py_INCREF(src[i]);
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals (used by fastcall) */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
        Py_INCREF(values[i]);
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    Py_INCREF(key);
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS)) {
        return __Pyx_PyList_GetItemRefFast(o, wrapped_i, unsafe_shared);
    } else
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyList_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)) {
            return __Pyx_PyList_GetItemRefFast(o, n, unsafe_shared);
        } else if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_NewRef(PyList_GET_ITEM(o, n));
        }
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject *index) {
    PyObject *runerr = NULL;
    Py_ssize_t key_value;
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        __Pyx_TypeName index_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(index));
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError,
            "cannot fit '" __Pyx_FMT_TYPENAME "' into an index-sized integer", index_type_name);
        __Pyx_DECREF_TypeName(index_type_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem_Slow(PyObject *obj, PyObject *key) {
    __Pyx_TypeName obj_type_name;
    if (likely(PyType_Check(obj))) {
        PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(obj, __pyx_mstate_global->__pyx_n_u_class_getitem);
        if (!meth) {
            PyErr_Clear();
        } else {
            PyObject *result = __Pyx_PyObject_CallOneArg(meth, key);
            Py_DECREF(meth);
            return result;
        }
    }
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "'" __Pyx_FMT_TYPENAME "' object is not subscriptable", obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject *key) {
    PyTypeObject *tp = Py_TYPE(obj);
    PyMappingMethods *mm = tp->tp_as_mapping;
    PySequenceMethods *sm = tp->tp_as_sequence;
    if (likely(mm && mm->mp_subscript)) {
        return mm->mp_subscript(obj, key);
    }
    if (likely(sm && sm->sq_item)) {
        return __Pyx_PyObject_GetIndex(obj, key);
    }
    return __Pyx_PyObject_GetItem_Slow(obj, key);
}
#endif

/* set_iter */
static CYTHON_INLINE PyObject* __Pyx_set_iterator(PyObject* iterable, int is_set,
                                                  Py_ssize_t* p_orig_length, int* p_source_is_set) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
    is_set = is_set || likely(PySet_CheckExact(iterable) || PyFrozenSet_CheckExact(iterable));
    *p_source_is_set = is_set;
    if (likely(is_set)) {
        *p_orig_length = PySet_Size(iterable);
        Py_INCREF(iterable);
        return iterable;
    }
#else
    CYTHON_UNUSED_VAR(is_set);
    *p_source_is_set = 0;
#endif
    *p_orig_length = 0;
    return PyObject_GetIter(iterable);
}
static CYTHON_INLINE int __Pyx_set_iter_next(
        PyObject* iter_obj, Py_ssize_t orig_length,
        Py_ssize_t* ppos, PyObject **value,
        int source_is_set) {
    if (!CYTHON_COMPILING_IN_CPYTHON || PY_VERSION_HEX >= 0x030d0000 || unlikely(!source_is_set)) {
        *value = PyIter_Next(iter_obj);
        if (unlikely(!*value)) {
            return __Pyx_IterFinish();
        }
        CYTHON_UNUSED_VAR(orig_length);
        CYTHON_UNUSED_VAR(ppos);
        return 1;
    }
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
    if (unlikely(PySet_GET_SIZE(iter_obj) != orig_length)) {
        PyErr_SetString(
            PyExc_RuntimeError,
            "set changed size during iteration");
        return -1;
    }
    {
        Py_hash_t hash;
        int ret = _PySet_NextEntry(iter_obj, ppos, value, &hash);
        assert (ret != -1);
        if (likely(ret)) {
            Py_INCREF(*value);
            return 1;
        }
    }
#endif
    return 0;
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if __PYX_LIMITED_VERSION_HEX >= 0x030F0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4)
    PyDict_SetDefaultRef(d, key, default_value, &value);
#elif CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType (used by FetchCommonType) */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CoroutineBase) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* RaiseException (used by CoroutineBase) */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* CallTypeTraverse (used by CoroutineBase) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* IterNextPlain (used by CoroutineBase) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *__Pyx_GetBuiltinNext_LimitedAPI(void) {
    if (unlikely(!__pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache))
        __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache = __Pyx_GetBuiltinName(__pyx_mstate_global->__pyx_n_u_next);
    return __pyx_mstate_global->__Pyx_GetBuiltinNext_LimitedAPI_cache;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next_Plain(PyObject *iterator) {
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    PyObject *result;
    PyObject *next = __Pyx_GetBuiltinNext_LimitedAPI();
    if (unlikely(!next)) return NULL;
    result = PyObject_CallFunctionObjArgs(next, iterator, NULL);
    return result;
#else
    (void)__Pyx_GetBuiltinName; // only for early limited API
    iternextfunc iternext = __Pyx_PyObject_GetIterNextFunc(iterator);
    assert(iternext);
    return iternext(iterator);
#endif
}

/* PyObjectCall2Args (used by PyObjectCallMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectCallMethod1 (used by CoroutineBase) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* ReturnWithStopIteration (used by CoroutineBase) */
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async);
static CYTHON_INLINE void __Pyx_ReturnWithStopIteration(PyObject* value, int async, int iternext) {
    if (value == Py_None) {
        if (async || !iternext)
            PyErr_SetNone(async ? PyExc_StopAsyncIteration : PyExc_StopIteration);
        return;
    }
    __Pyx__ReturnWithStopIteration(value, async);
}
static void __Pyx__ReturnWithStopIteration(PyObject* value, int async) {
#if CYTHON_COMPILING_IN_CPYTHON
    __Pyx_PyThreadState_declare
#endif
    PyObject *exc;
    PyObject *exc_type = async ? PyExc_StopAsyncIteration : PyExc_StopIteration;
#if CYTHON_COMPILING_IN_CPYTHON
    if ((PY_VERSION_HEX >= (0x030C00A6)) || unlikely(PyTuple_Check(value) || PyExceptionInstance_Check(value))) {
        if (PY_VERSION_HEX >= (0x030e00A1)) {
            exc = __Pyx_PyObject_CallOneArg(exc_type, value);
        } else {
            PyObject *args_tuple = PyTuple_New(1);
            if (unlikely(!args_tuple)) return;
            Py_INCREF(value);
            PyTuple_SET_ITEM(args_tuple, 0, value);
            exc = PyObject_Call(exc_type, args_tuple, NULL);
            Py_DECREF(args_tuple);
        }
        if (unlikely(!exc)) return;
    } else {
        Py_INCREF(value);
        exc = value;
    }
    #if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    #if CYTHON_USE_EXC_INFO_STACK
    if (!__pyx_tstate->exc_info->exc_value)
    #else
    if (!__pyx_tstate->exc_type)
    #endif
    {
        Py_INCREF(exc_type);
        __Pyx_ErrRestore(exc_type, exc, NULL);
        return;
    }
    #endif
#else
    exc = __Pyx_PyObject_CallOneArg(exc_type, value);
    if (unlikely(!exc)) return;
#endif
    PyErr_SetObject(exc_type, exc);
    Py_DECREF(exc);
}

/* CoroutineBase (used by Coroutine) */
#if !CYTHON_COMPILING_IN_LIMITED_API
#include <frameobject.h>
#if PY_VERSION_HEX >= 0x030b00a6 && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#endif // CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE void
__Pyx_Coroutine_Undelegate(__pyx_CoroutineObject *gen) {
#if CYTHON_USE_AM_SEND
    gen->yieldfrom_am_send = NULL;
#endif
    Py_CLEAR(gen->yieldfrom);
}
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    CYTHON_UNUSED_VAR(__pyx_tstate);
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
        else if (likely(__Pyx_IS_TYPE(ev, (PyTypeObject*)PyExc_StopIteration))) {
            #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
            value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
            if (unlikely(!value)) goto limited_api_failure;
            #else
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            #endif
            Py_DECREF(ev);
        }
        else if (unlikely(PyTuple_Check(ev))) {
            Py_ssize_t tuple_size = __Pyx_PyTuple_GET_SIZE(ev);
            #if !CYTHON_ASSUME_SAFE_SIZE
            if (unlikely(tuple_size < 0)) {
                Py_XDECREF(tb);
                Py_DECREF(ev);
                Py_DECREF(et);
                return -1;
            }
            #endif
            if (tuple_size >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#elif CYTHON_ASSUME_SAFE_MACROS
                value = PySequence_ITEM(ev, 0);
#else
                value = PySequence_GetItem(ev, 0);
                if (!value) goto limited_api_failure;
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_GetAttr(ev, __pyx_mstate_global->__pyx_n_u_value);
#else
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
#endif
    Py_DECREF(ev);
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!value)) return -1;
#endif
    *pvalue = value;
    return 0;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL || !CYTHON_ASSUME_SAFE_MACROS
  limited_api_failure:
    Py_XDECREF(et);
    Py_XDECREF(tb);
    Py_XDECREF(ev);
    return -1;
#endif
}
static CYTHON_INLINE
__Pyx_PySendResult __Pyx_Coroutine_status_from_result(PyObject **retval) {
    if (*retval) {
        return PYGEN_NEXT;
    } else if (likely(__Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, retval) == 0)) {
        return PYGEN_RETURN;
    } else {
        return PYGEN_ERROR;
    }
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_CLEAR(exc_state->exc_value);
#else
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
#endif
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(__pyx_CoroutineObject *gen) {
    const char *msg;
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
static void __Pyx_Coroutine_AlreadyTerminatedError(PyObject *gen, PyObject *value, int closing) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    CYTHON_MAYBE_UNUSED_VAR(closing);
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
__Pyx_PySendResult __Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, PyObject **result, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(__Pyx_Coroutine_get_is_running(self));  // Callers should ensure is_running
    if (unlikely(self->resume_label == -1)) {
        __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
        return PYGEN_ERROR;
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_value) {
        #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        #else
        PyObject *exc_tb;
        #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
        exc_tb = PyException_GetTraceback(exc_state->exc_value);
        #elif PY_VERSION_HEX >= 0x030B00a4
        exc_tb = ((PyBaseExceptionObject*) exc_state->exc_value)->traceback;
        #else
        exc_tb = exc_state->exc_traceback;
        #endif
        if (exc_tb) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
            PyFrameObject *f = tb->tb_frame;
            assert(f->f_back == NULL);
            #if PY_VERSION_HEX >= 0x030B00A1
            f->f_back = PyThreadState_GetFrame(tstate);
            #else
            Py_XINCREF(tstate->frame);
            f->f_back = tstate->frame;
            #endif
            #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_CPYTHON
            Py_DECREF(exc_tb);
            #endif
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    retval = self->body(self, tstate, value);
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    *result = retval;
    if (self->resume_label == -1) {
        return likely(retval) ? PYGEN_RETURN : PYGEN_ERROR;
    }
    return PYGEN_NEXT;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED_VAR(exc_state);
#else
    PyObject *exc_tb;
    #if PY_VERSION_HEX >= 0x030B00a4
    if (!exc_state->exc_value) return;
    exc_tb = PyException_GetTraceback(exc_state->exc_value);
    #else
    exc_tb = exc_state->exc_traceback;
    #endif
    if (likely(exc_tb)) {
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
        #if PY_VERSION_HEX >= 0x030B00a4
        Py_DECREF(exc_tb);
        #endif
    }
#endif
}
#define __Pyx_Coroutine_MethodReturnFromResult(gen, result, retval, iternext)\
    ((result) == PYGEN_NEXT ? (retval) : __Pyx__Coroutine_MethodReturnFromResult(gen, result, retval, iternext))
static PyObject *
__Pyx__Coroutine_MethodReturnFromResult(PyObject* gen, __Pyx_PySendResult result, PyObject *retval, int iternext) {
    CYTHON_MAYBE_UNUSED_VAR(gen);
    if (likely(result == PYGEN_RETURN)) {
        int is_async = 0;
        #ifdef __Pyx_AsyncGen_USED
        is_async = __Pyx_AsyncGen_CheckExact(gen);
        #endif
        __Pyx_ReturnWithStopIteration(retval, is_async, iternext);
        Py_XDECREF(retval);
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
#if PY_VERSION_HEX < 0x030d00A1
            _PyGen_SetStopIterationValue(result);
#else
            if (!PyTuple_Check(result) && !PyExceptionInstance_Check(result)) {
                PyErr_SetObject(PyExc_StopIteration, result);
            } else {
                PyObject *exc = __Pyx_PyObject_CallOneArg(PyExc_StopIteration, result);
                if (likely(exc != NULL)) {
                    PyErr_SetObject(PyExc_StopIteration, exc);
                    Py_DECREF(exc);
                }
            }
#endif
        }
        Py_DECREF(result);
        result = NULL;
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE __Pyx_PySendResult
__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen, PyObject** retval) {
    __Pyx_PySendResult result;
    PyObject *val = NULL;
    assert(__Pyx_Coroutine_get_is_running(gen));
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    result = __Pyx_Coroutine_SendEx(gen, val, retval, 0);
    Py_XDECREF(val);
    return result;
}
#if CYTHON_USE_AM_SEND
static __Pyx_PySendResult
__Pyx_Coroutine_SendToDelegate(__pyx_CoroutineObject *gen, __Pyx_pyiter_sendfunc gen_am_send, PyObject *value, PyObject **retval) {
    PyObject *ret = NULL;
    __Pyx_PySendResult delegate_result, result;
    assert(__Pyx_Coroutine_get_is_running(gen));
    delegate_result = gen_am_send(gen->yieldfrom, value, &ret);
    if (delegate_result == PYGEN_NEXT) {
        assert (ret != NULL);
        *retval = ret;
        return PYGEN_NEXT;
    }
    assert (delegate_result != PYGEN_ERROR || ret == NULL);
    __Pyx_Coroutine_Undelegate(gen);
    result = __Pyx_Coroutine_SendEx(gen, ret, retval, 0);
    Py_XDECREF(ret);
    return result;
}
#endif
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result = __Pyx_Coroutine_AmSend(self, value, &retval);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
}
static __Pyx_PySendResult
__Pyx_Coroutine_AmSend(PyObject *self, PyObject *value, PyObject **retval) {
    __Pyx_PySendResult result;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, value, retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
      #if !CYTHON_USE_AM_SEND
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
      #endif
        {
            #if !CYTHON_COMPILING_IN_LIMITED_API || __PYX_LIMITED_VERSION_HEX >= 0x03080000
            if (value == Py_None && PyIter_Check(yf))
                ret = __Pyx_PyIter_Next_Plain(yf);
            else
            #endif
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_mstate_global->__pyx_n_u_send, value);
        }
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            *retval = ret;
            return PYGEN_NEXT;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, value, retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return result;
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    CYTHON_UNUSED_VAR(gen);
    assert(__Pyx_Coroutine_get_is_running(gen));
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        result = __Pyx_Coroutine_Close(yf, &retval);
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        result = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf);
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
        result = PYGEN_RETURN;
    } else
    #endif
    {
        PyObject *meth;
        result = PYGEN_RETURN;
        meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_close);
        if (unlikely(!meth)) {
            if (unlikely(PyErr_Occurred())) {
                PyErr_WriteUnraisable(yf);
            }
        } else {
            retval = __Pyx_PyObject_CallNoArg(meth);
            Py_DECREF(meth);
            if (unlikely(!retval)) {
                result = PYGEN_ERROR;
            }
        }
    }
    Py_XDECREF(retval);
    return result == PYGEN_ERROR ? -1 : 0;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __Pyx_PySendResult result;
    PyObject *retval = NULL;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    }
    #if CYTHON_USE_AM_SEND
    if (gen->yieldfrom_am_send) {
        result = __Pyx_Coroutine_SendToDelegate(gen, gen->yieldfrom_am_send, Py_None, &retval);
    } else
    #endif
    if (gen->yieldfrom) {
        PyObject *yf = gen->yieldfrom;
        PyObject *ret;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && (PY_VERSION_HEX < 0x030A00A3 || !CYTHON_USE_AM_SEND)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
            ret = __Pyx_PyIter_Next_Plain(yf);
        if (likely(ret)) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &retval);
    } else {
        result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
    }
    __Pyx_Coroutine_unset_is_running(gen);
    return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 1);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, PyObject *arg) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result;
    CYTHON_UNUSED_VAR(arg);
    result = __Pyx_Coroutine_Close(self, &retval);
    if (unlikely(result == PYGEN_ERROR))
        return NULL;
    Py_XDECREF(retval);
    Py_RETURN_NONE;
}
static __Pyx_PySendResult
__Pyx_Coroutine_Close(PyObject *self, PyObject **retval) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PySendResult result;
    PyObject *yf;
    int err = 0;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
        *retval = __Pyx_Coroutine_AlreadyRunningError(gen);
        return PYGEN_ERROR;
    }
    yf = gen->yieldfrom;
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    result = __Pyx_Coroutine_SendEx(gen, NULL, retval, 1);
    if (result == PYGEN_ERROR) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_Coroutine_unset_is_running(gen);
        if (!__Pyx_PyErr_Occurred()) {
            return PYGEN_RETURN;
        } else if (likely(__Pyx_PyErr_ExceptionMatches2(PyExc_GeneratorExit, PyExc_StopIteration))) {
            __Pyx_PyErr_Clear();
            return PYGEN_RETURN;
        }
        return PYGEN_ERROR;
    } else if (likely(result == PYGEN_RETURN && *retval == Py_None)) {
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_RETURN;
    } else {
        const char *msg;
        Py_DECREF(*retval);
        *retval = NULL;
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
            msg = "async generator ignored GeneratorExit";
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        __Pyx_Coroutine_unset_is_running(gen);
        return PYGEN_ERROR;
    }
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf;
    if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen)))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    yf = gen->yieldfrom;
    if (yf) {
        __Pyx_PySendResult result;
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                goto propagate_exception;
            goto throw_here;
        }
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStrNoError(yf, __pyx_mstate_global->__pyx_n_u_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (unlikely(PyErr_Occurred())) {
                    __Pyx_Coroutine_unset_is_running(gen);
                    return NULL;
                }
                __Pyx_Coroutine_Undelegate(gen);
                goto throw_here;
            }
            if (likely(args)) {
                ret = __Pyx_PyObject_Call(meth, args, NULL);
            } else {
                PyObject *cargs[4] = {NULL, typ, val, tb};
                ret = __Pyx_PyObject_FastCall(meth, cargs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
            }
            Py_DECREF(meth);
        }
        Py_DECREF(yf);
        if (ret) {
            __Pyx_Coroutine_unset_is_running(gen);
            return ret;
        }
        result = __Pyx_Coroutine_FinishDelegation(gen, &ret);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, ret, 0);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
propagate_exception:
    {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, NULL, &retval, 0);
        __Pyx_Coroutine_unset_is_running(gen);
        return __Pyx_Coroutine_MethodReturnFromResult(self, result, retval, 0);
    }
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (unlikely(!PyArg_UnpackTuple(args, "throw", 1, 3, &typ, &val, &tb)))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
#if PY_VERSION_HEX >= 0x030B00a4
    Py_VISIT(exc_state->exc_value);
#else
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
#endif
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    {
        int e = __Pyx_call_type_traverse((PyObject*)gen, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    if (gen->gi_weakreflist != NULL)
    #endif
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if CYTHON_USE_TP_FINALIZE
        if (unlikely(PyObject_CallFinalizerFromDealloc(self)))
#else
        {
            destructor del = __Pyx_PyObject_GetSlot(gen, tp_del, destructor);
            if (del) del(self);
        }
        if (unlikely(Py_REFCNT(self) > 0))
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    __Pyx_PyHeapTypeObject_GC_Del(gen);
}
#if CYTHON_USE_TP_FINALIZE
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *retval = NULL;
        __Pyx_PySendResult result = __Pyx_Coroutine_Close(self, &retval);
        if (result == PYGEN_ERROR) {
            PyErr_WriteUnraisable(self);
        } else {
            Py_XDECREF(retval);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
}
#endif
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_name;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_name, value);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, void *context)
{
    PyObject *name = self->gi_qualname;
    CYTHON_UNUSED_VAR(context);
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_Py_XDECREF_SET(self->gi_qualname, value);
    return 0;
}
static PyObject *
__Pyx__Coroutine_get_frame(__pyx_CoroutineObject *self)
{
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyObject *frame;
    #if PY_VERSION_HEX >= 0x030d0000
    Py_BEGIN_CRITICAL_SECTION(self);
    #endif
    frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        PyObject *globals = PyDict_New();
        if (unlikely(!globals)) return NULL;
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            globals,                        /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        Py_DECREF(globals);
        if (unlikely(!frame))
            return NULL;
        if (unlikely(self->gi_frame)) {
            Py_DECREF(frame);
            frame = self->gi_frame;
        } else {
            self->gi_frame = frame;
        }
    }
    Py_INCREF(frame);
    #if PY_VERSION_HEX >= 0x030d0000
    Py_END_CRITICAL_SECTION();
    #endif
    return frame;
#else
    CYTHON_UNUSED_VAR(self);
    Py_RETURN_NONE;
#endif
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, void *context) {
    CYTHON_UNUSED_VAR(context);
    PyObject *frame = self->gi_frame;
    if (frame)
        return __Pyx_NewRef(frame);
    return __Pyx__Coroutine_get_frame(self);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->yieldfrom_am_send = NULL;
    #if PY_VERSION_HEX >= 0x030B00a4 && !CYTHON_COMPILING_IN_LIMITED_API
    gen->gi_exc_state.exc_value = NULL;
    #else
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
    #endif
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    gen->gi_weakreflist = NULL;
#endif
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}
static char __Pyx_Coroutine_test_and_set_is_running(__pyx_CoroutineObject *gen) {
    char result;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    result = gen->is_running;
    gen->is_running = 1;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
    return result;
}
static void __Pyx_Coroutine_unset_is_running(__pyx_CoroutineObject *gen) {
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    assert(gen->is_running);
    gen->is_running = 0;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
}
static char __Pyx_Coroutine_get_is_running(__pyx_CoroutineObject *gen) {
    char result;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_BEGIN_CRITICAL_SECTION(gen);
    #endif
    result = gen->is_running;
    #if PY_VERSION_HEX >= 0x030d0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_END_CRITICAL_SECTION();
    #endif
    return result;
}
static PyObject *__Pyx_Coroutine_get_is_running_getter(PyObject *gen, void *closure) {
    CYTHON_UNUSED_VAR(closure);
    char result = __Pyx_Coroutine_get_is_running((__pyx_CoroutineObject*)gen);
    if (result) Py_RETURN_TRUE;
    else Py_RETURN_FALSE;
}
#if __PYX_HAS_PY_AM_SEND == 2
static void __Pyx_SetBackportTypeAmSend(PyTypeObject *type, __Pyx_PyAsyncMethodsStruct *static_amsend_methods, __Pyx_pyiter_sendfunc am_send) {
    Py_ssize_t ptr_offset = (char*)(type->tp_as_async) - (char*)type;
    if (ptr_offset < 0 || ptr_offset > type->tp_basicsize) {
        return;
    }
    memcpy((void*)static_amsend_methods, (void*)(type->tp_as_async), sizeof(*type->tp_as_async));
    static_amsend_methods->am_send = am_send;
    type->tp_as_async = __Pyx_SlotTpAsAsync(static_amsend_methods);
}
#endif
static PyObject *__Pyx_Coroutine_fail_reduce_ex(PyObject *self, PyObject *arg) {
    CYTHON_UNUSED_VAR(arg);
    __Pyx_TypeName self_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE((PyObject*)self));
    PyErr_Format(PyExc_TypeError, "cannot pickle '" __Pyx_FMT_TYPENAME "' object",
                         self_type_name);
    __Pyx_DECREF_TypeName(self_type_name);
    return NULL;
}

/* Coroutine (used by CoroutineYieldFrom) */
static void __Pyx_CoroutineAwait_dealloc(PyObject *self) {
    PyObject_GC_UnTrack(self);
    Py_CLEAR(((__pyx_CoroutineAwaitObject*)self)->coroutine);
    __Pyx_PyHeapTypeObject_GC_Del(self);
}
static int __Pyx_CoroutineAwait_traverse(__pyx_CoroutineAwaitObject *self, visitproc visit, void *arg) {
    {
        int e = __Pyx_call_type_traverse((PyObject*)self, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(self->coroutine);
    return 0;
}
static int __Pyx_CoroutineAwait_clear(__pyx_CoroutineAwaitObject *self) {
    Py_CLEAR(self->coroutine);
    return 0;
}
static PyObject *__Pyx_CoroutineAwait_Next(__pyx_CoroutineAwaitObject *self) {
    return __Pyx_Generator_Next(self->coroutine);
}
static PyObject *__Pyx_CoroutineAwait_Send(__pyx_CoroutineAwaitObject *self, PyObject *value) {
    return __Pyx_Coroutine_Send(self->coroutine, value);
}
#if __PYX_HAS_PY_AM_SEND
static __Pyx_PySendResult __Pyx_CoroutineAwait_AmSend(PyObject *self, PyObject *value, PyObject **retval) {
    return __Pyx_Coroutine_AmSend(((__pyx_CoroutineAwaitObject*)self)->coroutine, value, retval);
}
#endif
static PyObject *__Pyx_CoroutineAwait_Throw(__pyx_CoroutineAwaitObject *self, PyObject *args) {
    return __Pyx_Coroutine_Throw(self->coroutine, args);
}
static __Pyx_PySendResult __Pyx_CoroutineAwait_Close(__pyx_CoroutineAwaitObject *self) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result = __Pyx_Coroutine_Close(self->coroutine, &retval);
    Py_XDECREF(retval);
    return result;
}
static PyObject *__Pyx_CoroutineAwait_Close_Method(__pyx_CoroutineAwaitObject *self, PyObject *arg) {
    PyObject *retval = NULL;
    __Pyx_PySendResult result;
    CYTHON_UNUSED_VAR(arg);
    result = __Pyx_Coroutine_Close(self->coroutine, &retval);
    if (unlikely(result == PYGEN_ERROR))
        return NULL;
    Py_XDECREF(retval);
    Py_RETURN_NONE;
}
static PyObject *__Pyx_CoroutineAwait_self(PyObject *self) {
    Py_INCREF(self);
    return self;
}
#if !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_CoroutineAwait_no_new(PyTypeObject *type, PyObject *args, PyObject *kwargs) {
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(args);
    CYTHON_UNUSED_VAR(kwargs);
    PyErr_SetString(PyExc_TypeError, "cannot instantiate type, use 'await coroutine' instead");
    return NULL;
}
#endif
static PyMethodDef __pyx_CoroutineAwait_methods[] = {
    {"send", (PyCFunction) __Pyx_CoroutineAwait_Send, METH_O,
     PyDoc_STR("send(arg) -> send 'arg' into coroutine,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_CoroutineAwait_Throw, METH_VARARGS,
     PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in coroutine,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_CoroutineAwait_Close_Method, METH_NOARGS, PyDoc_STR("close() -> raise GeneratorExit inside coroutine.")},
    {"__reduce_ex__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_O, 0},
    {"__reduce__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_NOARGS, 0},
    {0, 0, 0, 0}
};
static PyType_Slot __pyx_CoroutineAwaitType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CoroutineAwait_dealloc},
    {Py_tp_traverse, (void *)__Pyx_CoroutineAwait_traverse},
    {Py_tp_clear, (void *)__Pyx_CoroutineAwait_clear},
#if !CYTHON_COMPILING_IN_PYPY
    {Py_tp_new, (void *)__Pyx_CoroutineAwait_no_new},
#endif
    {Py_tp_methods, (void *)__pyx_CoroutineAwait_methods},
    {Py_tp_iter, (void *)__Pyx_CoroutineAwait_self},
    {Py_tp_iternext, (void *)__Pyx_CoroutineAwait_Next},
#if __PYX_HAS_PY_AM_SEND == 1
    {Py_am_send, (void *)__Pyx_CoroutineAwait_AmSend},
#endif
    {0, 0},
};
static PyType_Spec __pyx_CoroutineAwaitType_spec = {
    __PYX_TYPE_MODULE_PREFIX "coroutine_wrapper",
    sizeof(__pyx_CoroutineAwaitObject),
    0,
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | __Pyx_TPFLAGS_HAVE_AM_SEND,
    __pyx_CoroutineAwaitType_slots
};
#if __PYX_HAS_PY_AM_SEND == 2 // backport
static __Pyx_PyAsyncMethodsStruct __pyx_CoroutineAwait_as_async;
#endif
static CYTHON_INLINE PyObject *__Pyx__Coroutine_await(PyObject *coroutine) {
    __pyx_CoroutineAwaitObject *await = PyObject_GC_New(__pyx_CoroutineAwaitObject, __pyx_mstate_global->__pyx_CoroutineAwaitType);
    if (unlikely(!await)) return NULL;
    Py_INCREF(coroutine);
    await->coroutine = coroutine;
    PyObject_GC_Track(await);
    return (PyObject*)await;
}
static PyObject *__Pyx_Coroutine_await(PyObject *coroutine) {
    if (unlikely(!coroutine || !__Pyx_Coroutine_Check(coroutine))) {
        PyErr_SetString(PyExc_TypeError, "invalid input, expected coroutine");
        return NULL;
    }
    return __Pyx__Coroutine_await(coroutine);
}
static PyMethodDef __pyx_Coroutine_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     PyDoc_STR("send(arg) -> send 'arg' into coroutine,\nreturn next iterated value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in coroutine,\nreturn next iterated value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS, PyDoc_STR("close() -> raise GeneratorExit inside coroutine.")},
    {"__reduce_ex__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_O, 0},
    {"__reduce__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_NOARGS, 0},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Coroutine_memberlist[] = {
    {"cr_await", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     PyDoc_STR("object being awaited, or None")},
    {"cr_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {"__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Coroutine_getsets[] = {
    {"__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     PyDoc_STR("name of the coroutine"), 0},
    {"__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     PyDoc_STR("qualified name of the coroutine"), 0},
    {"cr_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     PyDoc_STR("Frame of the coroutine"), 0},
    {"cr_running", __Pyx_Coroutine_get_is_running_getter, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CoroutineType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
    {Py_am_await, (void *)&__Pyx_Coroutine_await},
    {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
    {Py_tp_methods, (void *)__pyx_Coroutine_methods},
    {Py_tp_members, (void *)__pyx_Coroutine_memberlist},
    {Py_tp_getset, (void *)__pyx_Coroutine_getsets},
    {Py_tp_getattro, (void *) PyObject_GenericGetAttr},
#if CYTHON_USE_TP_FINALIZE
    {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    {Py_am_send, (void *)__Pyx_Coroutine_AmSend},
#endif
    {0, 0},
};
static PyType_Spec __pyx_CoroutineType_spec = {
    __PYX_TYPE_MODULE_PREFIX "coroutine",
    sizeof(__pyx_CoroutineObject),
    0,
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_WEAKREF |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | __Pyx_TPFLAGS_HAVE_AM_SEND,
    __pyx_CoroutineType_slots
};
#if __PYX_HAS_PY_AM_SEND == 2
static __Pyx_PyAsyncMethodsStruct __pyx_Coroutine_as_async;
#endif
static int __pyx_Coroutine_init(PyObject *module) {
    __pyx_mstatetype *mstate;
    CYTHON_MAYBE_UNUSED_VAR(module);
    mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CoroutineType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CoroutineType_spec, NULL);
    if (unlikely(!mstate->__pyx_CoroutineType))
        return -1;
#if __PYX_HAS_PY_AM_SEND == 2
    __Pyx_SetBackportTypeAmSend(mstate->__pyx_CoroutineType, &__pyx_Coroutine_as_async, &__Pyx_Coroutine_AmSend);
#endif
#ifdef __Pyx_IterableCoroutine_USED
    if (unlikely(__pyx_IterableCoroutine_init(module) == -1))
        return -1;
#endif
    mstate->__pyx_CoroutineAwaitType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CoroutineAwaitType_spec, NULL);
    if (unlikely(!mstate->__pyx_CoroutineAwaitType))
        return -1;
#if __PYX_HAS_PY_AM_SEND == 2
    __Pyx_SetBackportTypeAmSend(mstate->__pyx_CoroutineAwaitType, &__pyx_CoroutineAwait_as_async, &__Pyx_CoroutineAwait_AmSend);
#endif
    return 0;
}

/* CoroutineSetYieldFrom (used by CoroutineYieldFrom) */
static void
__Pyx_Coroutine_Set_Owned_Yield_From(__pyx_CoroutineObject *gen, PyObject *yf) {
    assert (!gen->yieldfrom);
#if CYTHON_USE_AM_SEND
    assert (!gen->yieldfrom_am_send);
    #if PY_VERSION_HEX < 0x030A00F0
    if (__Pyx_PyType_HasFeature(Py_TYPE(yf), __Pyx_TPFLAGS_HAVE_AM_SEND))
    #endif
    {
        __Pyx_pyiter_sendfunc am_send;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
        am_send = __Pyx_PyObject_TryGetSubSlot(yf, tp_as_async, am_send, __Pyx_pyiter_sendfunc);
        #else
        __Pyx_PyAsyncMethodsStruct* tp_as_async = (__Pyx_PyAsyncMethodsStruct*) Py_TYPE(yf)->tp_as_async;
        am_send = tp_as_async ? tp_as_async->am_send : NULL;
        #endif
        if (likely(am_send)) {
            gen->yieldfrom_am_send = am_send;
        }
    }
#endif
    gen->yieldfrom = yf;
}

/* Coro_CheckExact (used by GetAwaitIter) */
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_PyCoro_CheckExact(PyObject *o) {
    return (PyObject*)Py_TYPE(o) == __pyx_mstate_global->__Pyx_CachedCoroType;
}
#endif

/* GetAwaitIter (used by CoroutineYieldFrom) */
static CYTHON_INLINE PyObject *__Pyx_Coroutine_GetAwaitableIter(PyObject *o) {
#ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(o)) {
        return __Pyx_NewRef(o);
    }
#endif
    return __Pyx__Coroutine_GetAwaitableIter(o);
}
static void __Pyx_Coroutine_AwaitableIterError(PyObject *source) {
#if (PY_VERSION_HEX < 0x030d0000 || defined(_PyErr_FormatFromCause)) && !CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_TypeName source_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(source));
    _PyErr_FormatFromCause(PyExc_TypeError,
        "'async for' received an invalid object from __anext__: " __Pyx_FMT_TYPENAME, source_type_name);
    __Pyx_DECREF_TypeName(source_type_name);
#else
    PyObject *exc, *val, *val2, *tb;
    __Pyx_TypeName source_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(source));
    assert(PyErr_Occurred());
    PyErr_Fetch(&exc, &val, &tb);
    PyErr_NormalizeException(&exc, &val, &tb);
    if (tb != NULL) {
        PyException_SetTraceback(val, tb);
        Py_DECREF(tb);
    }
    Py_DECREF(exc);
    assert(!PyErr_Occurred());
    PyErr_Format(PyExc_TypeError,
        "'async for' received an invalid object from __anext__: " __Pyx_FMT_TYPENAME, source_type_name);
    __Pyx_DECREF_TypeName(source_type_name);
    PyErr_Fetch(&exc, &val2, &tb);
    PyErr_NormalizeException(&exc, &val2, &tb);
    Py_INCREF(val);
    PyException_SetCause(val2, val);
    PyException_SetContext(val2, val);
    PyErr_Restore(exc, val2, tb);
#endif
}
static PyObject *__Pyx__Coroutine_GetAwaitableIter(PyObject *obj) {
    PyObject *res;
    unaryfunc am_await;
    am_await = __Pyx_PyObject_TryGetSubSlot(obj, tp_as_async, am_await, unaryfunc);
    if (likely(am_await)) {
        res = (*am_await)(obj);
    } else
#if CYTHON_COMPILING_IN_CPYTHON && defined(CO_ITERABLE_COROUTINE)
#if PY_VERSION_HEX >= 0x030C00A6
    if (PyGen_CheckExact(obj) && (PyGen_GetCode((PyGenObject*)obj)->co_flags & CO_ITERABLE_COROUTINE)) {
#else
    if (PyGen_CheckExact(obj) && ((PyGenObject*)obj)->gi_code && ((PyCodeObject *)((PyGenObject*)obj)->gi_code)->co_flags & CO_ITERABLE_COROUTINE) {
#endif
        return __Pyx_NewRef(obj);
    } else
#endif
    {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
        PyObject *args[] = { obj };
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_await, args, 1 |  PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
        if (unlikely(!res)) {
            if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyObject *type, *value, *traceback;
                PyErr_Fetch(&type, &value, &traceback);
                int has_attr = PyObject_HasAttr(obj, __pyx_mstate_global->__pyx_n_u_await);
                PyErr_Restore(type, value, traceback);
                if (!has_attr) goto slot_error;
            }
        }
#else
        PyObject *method = NULL;
        int is_method = __Pyx_PyObject_GetMethod(obj, __pyx_mstate_global->__pyx_n_u_await, &method);
        if (likely(is_method)) {
            res = __Pyx_PyObject_CallOneArg(method, obj);
        } else if (likely(method)) {
            res = __Pyx_PyObject_CallNoArg(method);
        } else
            goto slot_error;
        Py_DECREF(method);
#endif
    }
    if (unlikely(!res)) {
        __Pyx_Coroutine_AwaitableIterError(obj);
        goto bad;
    }
    if (unlikely(!PyIter_Check(res))) {
        __Pyx_TypeName res_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(res));
        PyErr_Format(PyExc_TypeError,
            "__await__() returned non-iterator of type '" __Pyx_FMT_TYPENAME "'", res_type_name);
        __Pyx_DECREF_TypeName(res_type_name);
        Py_CLEAR(res);
    } else {
        int is_coroutine = 0;
        #ifdef __Pyx_Coroutine_USED
        is_coroutine |= __Pyx_Coroutine_Check(res);
        #endif
        is_coroutine |= __Pyx_PyCoro_CheckExact(res);
        if (unlikely(is_coroutine)) {
            /* __await__ must return an *iterator*, not
               a coroutine or another awaitable (see PEP 492) */
            PyErr_SetString(PyExc_TypeError,
                            "__await__() returned a coroutine");
            Py_CLEAR(res);
        }
    }
    return res;
slot_error:
    {
        __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
        PyErr_Format(PyExc_TypeError,
            "object " __Pyx_FMT_TYPENAME " can't be used in 'await' expression", obj_type_name);
        __Pyx_DECREF_TypeName(obj_type_name);
    }
bad:
    return NULL;
}

/* CoroutineYieldFrom */
  static __Pyx_PySendResult __Pyx_Coroutine_Yield_From_Coroutine(__pyx_CoroutineObject *gen, PyObject *source, PyObject **retval) {
      __Pyx_PySendResult result;
      if (unlikely(((__pyx_CoroutineObject*)source)->yieldfrom)) {
          PyErr_SetString(
              PyExc_RuntimeError,
              "coroutine is being awaited already");
          return PYGEN_ERROR;
      }
      result = __Pyx_Coroutine_AmSend(source, Py_None, retval);
      if (result == PYGEN_NEXT) {
          Py_INCREF(source);
          __Pyx_Coroutine_Set_Owned_Yield_From(gen, source);
      }
      return result;
  }
  static __Pyx_PySendResult __Pyx_Coroutine_Yield_From_Generic(__pyx_CoroutineObject *gen, PyObject *source, PyObject **retval) {
      __Pyx_PySendResult result;
      PyObject *source_gen = NULL;
      source_gen = __Pyx_Coroutine_GetAwaitableIter(source);
      if (unlikely(!source_gen)) return PYGEN_ERROR;
      if (__Pyx_Coroutine_Check(source_gen)) {
          result = __Pyx_Coroutine_Yield_From_Coroutine(gen, source_gen, retval);
          Py_DECREF(source_gen);
          return result;
      }
      *retval = __Pyx_PyIter_Next_Plain(source_gen);
      if (*retval) {
          __Pyx_Coroutine_Set_Owned_Yield_From(gen, source_gen);
          return PYGEN_NEXT;
      }
      result = __Pyx_Coroutine_status_from_result(retval);
      Py_XDECREF(source_gen);
      return result;
  }
  static CYTHON_INLINE __Pyx_PySendResult __Pyx_Coroutine_Yield_From(__pyx_CoroutineObject *gen, PyObject *source, PyObject **retval) {
      if (__Pyx_Coroutine_Check(source)) {
          return __Pyx_Coroutine_Yield_From_Coroutine(gen, source, retval);
      }
  #ifdef __Pyx_AsyncGen_USED
      if (__pyx_PyAsyncGenASend_CheckExact(source)) {
          *retval = __Pyx_async_gen_asend_iternext(source);
          if (*retval) {
              Py_INCREF(source);
              __Pyx_Coroutine_Set_Owned_Yield_From(gen, source);
              return PYGEN_NEXT;
          }
          return __Pyx_Coroutine_status_from_result(retval);
      }
  #endif
      return __Pyx_Coroutine_Yield_From_Generic(gen, source, retval);
  }
  
/* FastTypeChecks */
  #if CYTHON_COMPILING_IN_CPYTHON
  static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
      while (a) {
          a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
          if (a == b)
              return 1;
      }
      return b == &PyBaseObject_Type;
  }
  static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
      PyObject *mro;
      if (a == b) return 1;
      mro = a->tp_mro;
      if (likely(mro)) {
          Py_ssize_t i, n;
          n = PyTuple_GET_SIZE(mro);
          for (i = 0; i < n; i++) {
              if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                  return 1;
          }
          return 0;
      }
      return __Pyx_InBases(a, b);
  }
  static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
      PyObject *mro;
      if (cls == a || cls == b) return 1;
      mro = cls->tp_mro;
      if (likely(mro)) {
          Py_ssize_t i, n;
          n = PyTuple_GET_SIZE(mro);
          for (i = 0; i < n; i++) {
              PyObject *base = PyTuple_GET_ITEM(mro, i);
              if (base == (PyObject *)a || base == (PyObject *)b)
                  return 1;
          }
          return 0;
      }
      return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
  }
  static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
      if (exc_type1) {
          return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
      } else {
          return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
      }
  }
  static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
      Py_ssize_t i, n;
      assert(PyExceptionClass_Check(exc_type));
      n = PyTuple_GET_SIZE(tuple);
      for (i=0; i<n; i++) {
          if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
      }
      for (i=0; i<n; i++) {
          PyObject *t = PyTuple_GET_ITEM(tuple, i);
          if (likely(PyExceptionClass_Check(t))) {
              if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
          } else {
          }
      }
      return 0;
  }
  static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
      if (likely(err == exc_type)) return 1;
      if (likely(PyExceptionClass_Check(err))) {
          if (likely(PyExceptionClass_Check(exc_type))) {
              return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
          } else if (likely(PyTuple_Check(exc_type))) {
              return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
          } else {
          }
      }
      return PyErr_GivenExceptionMatches(err, exc_type);
  }
  static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
      assert(PyExceptionClass_Check(exc_type1));
      assert(PyExceptionClass_Check(exc_type2));
      if (likely(err == exc_type1 || err == exc_type2)) return 1;
      if (likely(PyExceptionClass_Check(err))) {
          return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
      }
      return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
  }
  #endif
  
/* PyLongBinop */
  #if !CYTHON_COMPILING_IN_PYPY
  static PyObject* __Pyx_Fallback___Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, int inplace) {
      return (inplace ? PyNumber_InPlaceFloorDivide : PyNumber_FloorDivide)(op1, op2);
  }
  #if CYTHON_USE_PYLONG_INTERNALS
  static PyObject* __Pyx_Unpacked___Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
      CYTHON_MAYBE_UNUSED_VAR(inplace);
      CYTHON_UNUSED_VAR(zerodivision_check);
      const long b = intval;
      long a;
      const PY_LONG_LONG llb = intval;
      PY_LONG_LONG lla;
      if (unlikely(__Pyx_PyLong_IsZero(op1))) {
          return __Pyx_NewRef(op1);
      }
      const int is_positive = __Pyx_PyLong_IsPos(op1);
      const digit* digits = __Pyx_PyLong_Digits(op1);
      const Py_ssize_t size = __Pyx_PyLong_DigitCount(op1);
      if (likely(size == 1)) {
          a = (long) digits[0];
          if (!is_positive) a *= -1;
      } else {
          switch (size) {
              case 2:
                  if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                      a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                      if (!is_positive) a *= -1;
                      goto calculate_long;
                  } else if (8 * sizeof(PY_LONG_LONG) - 1 > 2 * PyLong_SHIFT) {
                      lla = (PY_LONG_LONG) (((((unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                      if (!is_positive) lla *= -1;
                      goto calculate_long_long;
                  }
                  break;
              case 3:
                  if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                      a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                      if (!is_positive) a *= -1;
                      goto calculate_long;
                  } else if (8 * sizeof(PY_LONG_LONG) - 1 > 3 * PyLong_SHIFT) {
                      lla = (PY_LONG_LONG) (((((((unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                      if (!is_positive) lla *= -1;
                      goto calculate_long_long;
                  }
                  break;
              case 4:
                  if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                      a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                      if (!is_positive) a *= -1;
                      goto calculate_long;
                  } else if (8 * sizeof(PY_LONG_LONG) - 1 > 4 * PyLong_SHIFT) {
                      lla = (PY_LONG_LONG) (((((((((unsigned PY_LONG_LONG)digits[3]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[2]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[1]) << PyLong_SHIFT) | (unsigned PY_LONG_LONG)digits[0]));
                      if (!is_positive) lla *= -1;
                      goto calculate_long_long;
                  }
                  break;
          }
          return PyLong_Type.tp_as_number->nb_floor_divide(op1, op2);
      }
      calculate_long:
          {
              long q, r;
              q = a / b;
              r = a - q*b;
              q -= ((r != 0) & ((r ^ b) < 0));
              return PyLong_FromLong(q);
          }
      calculate_long_long:
          {
              PY_LONG_LONG q, r;
              q = lla / llb;
              r = lla - q*llb;
              q -= ((r != 0) & ((r ^ llb) < 0));
              return PyLong_FromLongLong(q);
          }
      
  }
  #endif
  static CYTHON_INLINE PyObject* __Pyx_PyLong_FloorDivideObjC(PyObject *op1, PyObject *op2, long intval, int inplace, int zerodivision_check) {
      CYTHON_MAYBE_UNUSED_VAR(intval);
      CYTHON_UNUSED_VAR(zerodivision_check);
      #if CYTHON_USE_PYLONG_INTERNALS
      if (likely(PyLong_CheckExact(op1))) {
          return __Pyx_Unpacked___Pyx_PyLong_FloorDivideObjC(op1, op2, intval, inplace, zerodivision_check);
      }
      #endif
      return __Pyx_Fallback___Pyx_PyLong_FloorDivideObjC(op1, op2, inplace);
  }
  #endif
  
/* RaiseClosureNameError */
  static void __Pyx_RaiseClosureNameError(const char *varname) {
      PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
  }
  
/* PyMethodNew (used by CythonFunctionShared) */
  #if CYTHON_COMPILING_IN_LIMITED_API
  static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
      PyObject *result;
      CYTHON_UNUSED_VAR(typ);
      if (!self)
          return __Pyx_NewRef(func);
      #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
      {
          PyObject *args[] = {func, self};
          result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
      }
      #else
      result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
      #endif
      return result;
  }
  #else
  static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
      CYTHON_UNUSED_VAR(typ);
      if (!self)
          return __Pyx_NewRef(func);
      return PyMethod_New(func, self);
  }
  #endif
  
/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
  #if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
  static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
  {
      PyObject *res = NULL;
      PyObject *kwnames;
      PyObject **newargs;
      PyObject **kwvalues;
      Py_ssize_t i;
      #if CYTHON_AVOID_BORROWED_REFS
      PyObject *pos;
      #else
      Py_ssize_t pos;
      #endif
      size_t j;
      PyObject *key, *value;
      unsigned long keys_are_strings;
      #if !CYTHON_ASSUME_SAFE_SIZE
      Py_ssize_t nkw = PyDict_Size(kw);
      if (unlikely(nkw == -1)) return NULL;
      #else
      Py_ssize_t nkw = PyDict_GET_SIZE(kw);
      #endif
      newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
      if (unlikely(newargs == NULL)) {
          PyErr_NoMemory();
          return NULL;
      }
      for (j = 0; j < nargs; j++) newargs[j] = args[j];
      kwnames = PyTuple_New(nkw);
      if (unlikely(kwnames == NULL)) {
          PyMem_Free(newargs);
          return NULL;
      }
      kwvalues = newargs + nargs;
      pos = 0;
      i = 0;
      keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
      while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
          keys_are_strings &=
          #if CYTHON_COMPILING_IN_LIMITED_API
              PyType_GetFlags(Py_TYPE(key));
          #else
              Py_TYPE(key)->tp_flags;
          #endif
          #if !CYTHON_ASSUME_SAFE_MACROS
          if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
          #else
          PyTuple_SET_ITEM(kwnames, i, key);
          #endif
          kwvalues[i] = value;
          i++;
      }
      if (unlikely(!keys_are_strings)) {
          PyErr_SetString(PyExc_TypeError, "keywords must be strings");
          goto cleanup;
      }
      res = vc(func, newargs, nargs, kwnames);
  cleanup:
      #if CYTHON_AVOID_BORROWED_REFS
      Py_DECREF(pos);
      #endif
      Py_DECREF(kwnames);
      for (i = 0; i < nkw; i++)
          Py_DECREF(kwvalues[i]);
      PyMem_Free(newargs);
      return res;
  }
  static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
  {
      Py_ssize_t kw_size =
          likely(kw == NULL) ?
          0 :
  #if !CYTHON_ASSUME_SAFE_SIZE
          PyDict_Size(kw);
  #else
          PyDict_GET_SIZE(kw);
  #endif
      if (kw_size == 0) {
          return vc(func, args, nargs, NULL);
      }
  #if !CYTHON_ASSUME_SAFE_SIZE
      else if (unlikely(kw_size == -1)) {
          return NULL;
      }
  #endif
      return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
  }
  #endif
  
/* CythonFunctionShared (used by CythonFunction) */
  #if CYTHON_COMPILING_IN_LIMITED_API
  static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
      if (__Pyx_CyFunction_Check(func)) {
          return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
      } else if (PyCFunction_Check(func)) {
          return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
      }
      return 0;
  }
  static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
      if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
          int result;
          PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
          if (unlikely(!newFunc)) {
              PyErr_Clear(); // It's only an optimization, so don't throw an error
              return 0;
          }
          result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
          Py_DECREF(newFunc);
          return result;
      }
      return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
  }
  #else
  static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
      if (PyMethod_Check(func)) {
          func = PyMethod_GET_FUNCTION(func);
      }
      return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
  }
  #endif
  static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
  #if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
      __Pyx_Py_XDECREF_SET(
          __Pyx_CyFunction_GetClassObj(f),
              ((classobj) ? __Pyx_NewRef(classobj) : NULL));
  #else
      __Pyx_Py_XDECREF_SET(
          ((PyCMethodObject *) (f))->mm_class,
          (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
  #endif
  }
  static PyObject *
  __Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
  {
      if (unlikely(op->func_doc == NULL)) {
  #if CYTHON_COMPILING_IN_LIMITED_API
          op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
          if (unlikely(!op->func_doc)) return NULL;
  #else
          if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
              op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
              if (unlikely(op->func_doc == NULL))
                  return NULL;
          } else {
              Py_INCREF(Py_None);
              return Py_None;
          }
  #endif
      }
      Py_INCREF(op->func_doc);
      return op->func_doc;
  }
  static PyObject *
  __Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
      PyObject *result;
      CYTHON_UNUSED_VAR(closure);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      result = __Pyx_CyFunction_get_doc_locked(op);
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static int
  __Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      if (value == NULL) {
          value = Py_None;
      }
      Py_INCREF(value);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      __Pyx_Py_XDECREF_SET(op->func_doc, value);
      __Pyx_END_CRITICAL_SECTION();
      return 0;
  }
  static PyObject *
  __Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
  {
      if (unlikely(op->func_name == NULL)) {
  #if CYTHON_COMPILING_IN_LIMITED_API
          op->func_name = PyObject_GetAttrString(op->func, "__name__");
  #else
          op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
  #endif
          if (unlikely(op->func_name == NULL))
              return NULL;
      }
      Py_INCREF(op->func_name);
      return op->func_name;
  }
  static PyObject *
  __Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
  {
      PyObject *result = NULL;
      CYTHON_UNUSED_VAR(context);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      result = __Pyx_CyFunction_get_name_locked(op);
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static int
  __Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      if (unlikely(value == NULL || !PyUnicode_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__name__ must be set to a string object");
          return -1;
      }
      Py_INCREF(value);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      __Pyx_Py_XDECREF_SET(op->func_name, value);
      __Pyx_END_CRITICAL_SECTION();
      return 0;
  }
  static PyObject *
  __Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      PyObject *result;
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      Py_INCREF(op->func_qualname);
      result = op->func_qualname;
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static int
  __Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      if (unlikely(value == NULL || !PyUnicode_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__qualname__ must be set to a string object");
          return -1;
      }
      Py_INCREF(value);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      __Pyx_Py_XDECREF_SET(op->func_qualname, value);
      __Pyx_END_CRITICAL_SECTION();
      return 0;
  }
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
  static PyObject *
  __Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      if (unlikely(op->func_dict == NULL)) {
          op->func_dict = PyDict_New();
          if (unlikely(op->func_dict == NULL))
              return NULL;
      }
      Py_INCREF(op->func_dict);
      return op->func_dict;
  }
  #endif
  static PyObject *
  __Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
  {
      CYTHON_UNUSED_VAR(context);
      Py_INCREF(op->func_globals);
      return op->func_globals;
  }
  static PyObject *
  __Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
  {
      CYTHON_UNUSED_VAR(op);
      CYTHON_UNUSED_VAR(context);
      Py_INCREF(Py_None);
      return Py_None;
  }
  static PyObject *
  __Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
  {
      PyObject* result = (op->func_code) ? op->func_code : Py_None;
      CYTHON_UNUSED_VAR(context);
      Py_INCREF(result);
      return result;
  }
  static int
  __Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
      int result = 0;
      PyObject *res = op->defaults_getter((PyObject *) op);
      if (unlikely(!res))
          return -1;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
      Py_INCREF(op->defaults_tuple);
      op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
      Py_INCREF(op->defaults_kwdict);
      #else
      op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
      if (unlikely(!op->defaults_tuple)) result = -1;
      else {
          op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
          if (unlikely(!op->defaults_kwdict)) result = -1;
      }
      #endif
      Py_DECREF(res);
      return result;
  }
  static int
  __Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
      CYTHON_UNUSED_VAR(context);
      if (!value) {
          value = Py_None;
      } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__defaults__ must be set to a tuple object");
          return -1;
      }
      PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                   "currently affect the values used in function calls", 1);
      Py_INCREF(value);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
      __Pyx_END_CRITICAL_SECTION();
      return 0;
  }
  static PyObject *
  __Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
      PyObject* result = op->defaults_tuple;
      if (unlikely(!result)) {
          if (op->defaults_getter) {
              if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
              result = op->defaults_tuple;
          } else {
              result = Py_None;
          }
      }
      Py_INCREF(result);
      return result;
  }
  static PyObject *
  __Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
      PyObject* result = NULL;
      CYTHON_UNUSED_VAR(context);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      result = __Pyx_CyFunction_get_defaults_locked(op);
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static int
  __Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
      CYTHON_UNUSED_VAR(context);
      if (!value) {
          value = Py_None;
      } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__kwdefaults__ must be set to a dict object");
          return -1;
      }
      PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                   "currently affect the values used in function calls", 1);
      Py_INCREF(value);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
      __Pyx_END_CRITICAL_SECTION();
      return 0;
  }
  static PyObject *
  __Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
      PyObject* result = op->defaults_kwdict;
      if (unlikely(!result)) {
          if (op->defaults_getter) {
              if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
              result = op->defaults_kwdict;
          } else {
              result = Py_None;
          }
      }
      Py_INCREF(result);
      return result;
  }
  static PyObject *
  __Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
      PyObject* result;
      CYTHON_UNUSED_VAR(context);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      result = __Pyx_CyFunction_get_kwdefaults_locked(op);
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static int
  __Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
      CYTHON_UNUSED_VAR(context);
      if (!value || value == Py_None) {
          value = NULL;
      } else if (unlikely(!PyDict_Check(value))) {
          PyErr_SetString(PyExc_TypeError,
                          "__annotations__ must be set to a dict object");
          return -1;
      }
      Py_XINCREF(value);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      __Pyx_Py_XDECREF_SET(op->func_annotations, value);
      __Pyx_END_CRITICAL_SECTION();
      return 0;
  }
  static PyObject *
  __Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
      PyObject* result = op->func_annotations;
      if (unlikely(!result)) {
          result = PyDict_New();
          if (unlikely(!result)) return NULL;
          op->func_annotations = result;
      }
      Py_INCREF(result);
      return result;
  }
  static PyObject *
  __Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
      PyObject *result;
      CYTHON_UNUSED_VAR(context);
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      result = __Pyx_CyFunction_get_annotations_locked(op);
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static PyObject *
  __Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
      int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
      if (is_coroutine) {
          PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
          fromlist = PyList_New(1);
          if (unlikely(!fromlist)) return NULL;
          Py_INCREF(marker);
  #if CYTHON_ASSUME_SAFE_MACROS
          PyList_SET_ITEM(fromlist, 0, marker);
  #else
          if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
              Py_DECREF(marker);
              Py_DECREF(fromlist);
              return NULL;
          }
  #endif
          module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
          Py_DECREF(fromlist);
          if (unlikely(!module)) goto ignore;
          is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
          Py_DECREF(module);
          if (likely(is_coroutine_value)) {
              return is_coroutine_value;
          }
  ignore:
          PyErr_Clear();
      }
      return __Pyx_PyBool_FromLong(is_coroutine);
  }
  static PyObject *
  __Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
      PyObject *result;
      CYTHON_UNUSED_VAR(context);
      if (op->func_is_coroutine) {
          return __Pyx_NewRef(op->func_is_coroutine);
      }
      result = __Pyx_CyFunction_get_is_coroutine_value(op);
      if (unlikely(!result))
          return NULL;
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      if (op->func_is_coroutine) {
          Py_DECREF(result);
          result = __Pyx_NewRef(op->func_is_coroutine);
      } else {
          op->func_is_coroutine = __Pyx_NewRef(result);
      }
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
      if (!py_name) return;
      PyErr_Format(PyExc_TypeError,
          "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
          py_name, message, size);
      Py_DECREF(py_name);
  #else
      const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
      PyErr_Format(PyExc_TypeError,
          "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
          name, message, size);
  #endif
  }
  static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
      if (!py_name) return;
      PyErr_Format(PyExc_TypeError,
          "%.200S() %s",
          py_name, message);
      Py_DECREF(py_name);
  #else
      const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
      PyErr_Format(PyExc_TypeError,
          "%.200s() %s",
          name, message);
  #endif
  }
  #if CYTHON_COMPILING_IN_LIMITED_API
  static PyObject *
  __Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
      CYTHON_UNUSED_VAR(context);
      return PyObject_GetAttrString(op->func, "__module__");
  }
  static int
  __Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
      CYTHON_UNUSED_VAR(context);
      return PyObject_SetAttrString(op->func, "__module__", value);
  }
  #endif
  static PyGetSetDef __pyx_CyFunction_getsets[] = {
      {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
      {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
      {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
      {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
      {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
      {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
      {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
  #else
      {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
      {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
  #endif
      {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
      {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
      {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
      {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
      {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
      {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
      {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
      {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
      {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
      {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
      {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
  #if CYTHON_COMPILING_IN_LIMITED_API
      {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
  #endif
      {0, 0, 0, 0, 0}
  };
  static PyMemberDef __pyx_CyFunction_members[] = {
  #if !CYTHON_COMPILING_IN_LIMITED_API
      {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
  #endif
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
  #endif
  #if CYTHON_METH_FASTCALL
  #if CYTHON_COMPILING_IN_LIMITED_API
      {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
  #else
      {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
  #endif
  #if CYTHON_COMPILING_IN_LIMITED_API
      {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
  #else
      {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
  #endif
  #endif
      {0, 0, 0,  0, 0}
  };
  static PyObject *
  __Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
  {
      PyObject *result = NULL;
      CYTHON_UNUSED_VAR(args);
      __Pyx_BEGIN_CRITICAL_SECTION(m);
      Py_INCREF(m->func_qualname);
      result = m->func_qualname;
      __Pyx_END_CRITICAL_SECTION();
      return result;
  }
  static PyMethodDef __pyx_CyFunction_methods[] = {
      {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
      {0, 0, 0, 0}
  };
  #if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
  #else
  #define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
  #endif
  static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                         PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
  #if !CYTHON_COMPILING_IN_LIMITED_API
      PyCFunctionObject *cf = (PyCFunctionObject*) op;
  #endif
      if (unlikely(op == NULL))
          return NULL;
  #if CYTHON_COMPILING_IN_LIMITED_API
      op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
      if (unlikely(!op->func)) return NULL;
  #endif
      op->flags = flags;
      __Pyx_CyFunction_weakreflist(op) = NULL;
  #if !CYTHON_COMPILING_IN_LIMITED_API
      cf->m_ml = ml;
      cf->m_self = (PyObject *) op;
  #endif
      Py_XINCREF(closure);
      op->func_closure = closure;
  #if !CYTHON_COMPILING_IN_LIMITED_API
      Py_XINCREF(module);
      cf->m_module = module;
  #endif
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      op->func_dict = NULL;
  #endif
      op->func_name = NULL;
      Py_INCREF(qualname);
      op->func_qualname = qualname;
      op->func_doc = NULL;
  #if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
      op->func_classobj = NULL;
  #else
      ((PyCMethodObject*)op)->mm_class = NULL;
  #endif
      op->func_globals = globals;
      Py_INCREF(op->func_globals);
      Py_XINCREF(code);
      op->func_code = code;
      op->defaults = NULL;
      op->defaults_tuple = NULL;
      op->defaults_kwdict = NULL;
      op->defaults_getter = NULL;
      op->func_annotations = NULL;
      op->func_is_coroutine = NULL;
  #if CYTHON_METH_FASTCALL
      switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
      case METH_NOARGS:
          __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
          break;
      case METH_O:
          __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
          break;
      case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
          __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
          break;
      case METH_FASTCALL | METH_KEYWORDS:
          __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
          break;
      case METH_VARARGS | METH_KEYWORDS:
          __Pyx_CyFunction_func_vectorcall(op) = NULL;
          break;
      default:
          PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
          Py_DECREF(op);
          return NULL;
      }
  #endif
      return (PyObject *) op;
  }
  static int
  __Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
  {
      Py_CLEAR(m->func_closure);
  #if CYTHON_COMPILING_IN_LIMITED_API
      Py_CLEAR(m->func);
  #else
      Py_CLEAR(((PyCFunctionObject*)m)->m_module);
  #endif
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      Py_CLEAR(m->func_dict);
  #elif PY_VERSION_HEX < 0x030d0000
      _PyObject_ClearManagedDict((PyObject*)m);
  #else
      PyObject_ClearManagedDict((PyObject*)m);
  #endif
      Py_CLEAR(m->func_name);
      Py_CLEAR(m->func_qualname);
      Py_CLEAR(m->func_doc);
      Py_CLEAR(m->func_globals);
      Py_CLEAR(m->func_code);
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x030900B1
      Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
  #else
      {
          PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
          ((PyCMethodObject *) (m))->mm_class = NULL;
          Py_XDECREF(cls);
      }
  #endif
  #endif
      Py_CLEAR(m->defaults_tuple);
      Py_CLEAR(m->defaults_kwdict);
      Py_CLEAR(m->func_annotations);
      Py_CLEAR(m->func_is_coroutine);
      Py_CLEAR(m->defaults);
      return 0;
  }
  static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
  {
      if (__Pyx_CyFunction_weakreflist(m) != NULL)
          PyObject_ClearWeakRefs((PyObject *) m);
      __Pyx_CyFunction_clear(m);
      __Pyx_PyHeapTypeObject_GC_Del(m);
  }
  static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
  {
      PyObject_GC_UnTrack(m);
      __Pyx__CyFunction_dealloc(m);
  }
  static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
  {
      {
          int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
          if (e) return e;
      }
      Py_VISIT(m->func_closure);
  #if CYTHON_COMPILING_IN_LIMITED_API
      Py_VISIT(m->func);
  #else
      Py_VISIT(((PyCFunctionObject*)m)->m_module);
  #endif
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      Py_VISIT(m->func_dict);
  #else
      {
          int e =
  #if PY_VERSION_HEX < 0x030d0000
              _PyObject_VisitManagedDict
  #else
              PyObject_VisitManagedDict
  #endif
                  ((PyObject*)m, visit, arg);
          if (e != 0) return e;
      }
  #endif
      __Pyx_VISIT_CONST(m->func_name);
      __Pyx_VISIT_CONST(m->func_qualname);
      Py_VISIT(m->func_doc);
      Py_VISIT(m->func_globals);
      __Pyx_VISIT_CONST(m->func_code);
  #if !CYTHON_COMPILING_IN_LIMITED_API
      Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
  #endif
      Py_VISIT(m->defaults_tuple);
      Py_VISIT(m->defaults_kwdict);
      Py_VISIT(m->func_is_coroutine);
      Py_VISIT(m->defaults);
      return 0;
  }
  static PyObject*
  __Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
  {
      PyObject *repr;
      __Pyx_BEGIN_CRITICAL_SECTION(op);
      repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                  op->func_qualname, (void *)op);
      __Pyx_END_CRITICAL_SECTION();
      return repr;
  }
  static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
      PyCFunction meth;
      int flags;
      meth = PyCFunction_GetFunction(f);
      if (unlikely(!meth)) return NULL;
      flags = PyCFunction_GetFlags(f);
      if (unlikely(flags < 0)) return NULL;
  #else
      PyCFunctionObject* f = (PyCFunctionObject*)func;
      PyCFunction meth = f->m_ml->ml_meth;
      int flags = f->m_ml->ml_flags;
  #endif
      Py_ssize_t size;
      switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
      case METH_VARARGS:
          if (likely(kw == NULL || PyDict_Size(kw) == 0))
              return (*meth)(self, arg);
          break;
      case METH_VARARGS | METH_KEYWORDS:
          return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
      case METH_NOARGS:
          if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
  #if CYTHON_ASSUME_SAFE_SIZE
              size = PyTuple_GET_SIZE(arg);
  #else
              size = PyTuple_Size(arg);
              if (unlikely(size < 0)) return NULL;
  #endif
              if (likely(size == 0))
                  return (*meth)(self, NULL);
              __Pyx_CyFunction_raise_argument_count_error(
                  (__pyx_CyFunctionObject*)func,
                  "takes no arguments", size);
              return NULL;
          }
          break;
      case METH_O:
          if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
  #if CYTHON_ASSUME_SAFE_SIZE
              size = PyTuple_GET_SIZE(arg);
  #else
              size = PyTuple_Size(arg);
              if (unlikely(size < 0)) return NULL;
  #endif
              if (likely(size == 1)) {
                  PyObject *result, *arg0;
                  #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                  arg0 = PyTuple_GET_ITEM(arg, 0);
                  #else
                  arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                  #endif
                  result = (*meth)(self, arg0);
                  #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                  Py_DECREF(arg0);
                  #endif
                  return result;
              }
              __Pyx_CyFunction_raise_argument_count_error(
                  (__pyx_CyFunctionObject*)func,
                  "takes exactly one argument", size);
              return NULL;
          }
          break;
      default:
          PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
          return NULL;
      }
      __Pyx_CyFunction_raise_type_error(
          (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
      return NULL;
  }
  static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
      PyObject *self, *result;
  #if CYTHON_COMPILING_IN_LIMITED_API
      self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
      if (unlikely(!self) && PyErr_Occurred()) return NULL;
  #else
      self = ((PyCFunctionObject*)func)->m_self;
  #endif
      result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
      return result;
  }
  static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
      PyObject *result;
      __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
  #if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
       __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
      if (vc) {
  #if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
          return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
  #else
          (void) &__Pyx_PyVectorcall_FastCallDict;
          return PyVectorcall_Call(func, args, kw);
  #endif
      }
  #endif
      if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
          Py_ssize_t argc;
          PyObject *new_args;
          PyObject *self;
  #if CYTHON_ASSUME_SAFE_SIZE
          argc = PyTuple_GET_SIZE(args);
  #else
          argc = PyTuple_Size(args);
          if (unlikely(argc < 0)) return NULL;
  #endif
          new_args = PyTuple_GetSlice(args, 1, argc);
          if (unlikely(!new_args))
              return NULL;
          self = PyTuple_GetItem(args, 0);
          if (unlikely(!self)) {
              Py_DECREF(new_args);
              PyErr_Format(PyExc_TypeError,
                           "unbound method %.200S() needs an argument",
                           cyfunc->func_qualname);
              return NULL;
          }
          result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
          Py_DECREF(new_args);
      } else {
          result = __Pyx_CyFunction_Call(func, args, kw);
      }
      return result;
  }
  #if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
  static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
  {
      int ret = 0;
      if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
          if (unlikely(nargs < 1)) {
              __Pyx_CyFunction_raise_type_error(
                  cyfunc, "needs an argument");
              return -1;
          }
          ret = 1;
      }
      if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
          __Pyx_CyFunction_raise_type_error(
              cyfunc, "takes no keyword arguments");
          return -1;
      }
      return ret;
  }
  static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
  {
      __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
      Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
      PyObject *self;
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
      if (unlikely(!meth)) return NULL;
  #else
      PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
  #endif
      switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
      case 1:
          self = args[0];
          args += 1;
          nargs -= 1;
          break;
      case 0:
  #if CYTHON_COMPILING_IN_LIMITED_API
          self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
          if (unlikely(!self) && PyErr_Occurred()) return NULL;
  #else
          self = ((PyCFunctionObject*)cyfunc)->m_self;
  #endif
          break;
      default:
          return NULL;
      }
      if (unlikely(nargs != 0)) {
          __Pyx_CyFunction_raise_argument_count_error(
              cyfunc, "takes no arguments", nargs);
          return NULL;
      }
      return meth(self, NULL);
  }
  static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
  {
      __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
      Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
      PyObject *self;
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
      if (unlikely(!meth)) return NULL;
  #else
      PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
  #endif
      switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
      case 1:
          self = args[0];
          args += 1;
          nargs -= 1;
          break;
      case 0:
  #if CYTHON_COMPILING_IN_LIMITED_API
          self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
          if (unlikely(!self) && PyErr_Occurred()) return NULL;
  #else
          self = ((PyCFunctionObject*)cyfunc)->m_self;
  #endif
          break;
      default:
          return NULL;
      }
      if (unlikely(nargs != 1)) {
          __Pyx_CyFunction_raise_argument_count_error(
              cyfunc, "takes exactly one argument", nargs);
          return NULL;
      }
      return meth(self, args[0]);
  }
  static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
  {
      __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
      Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
      PyObject *self;
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
      if (unlikely(!meth)) return NULL;
  #else
      PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
  #endif
      switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
      case 1:
          self = args[0];
          args += 1;
          nargs -= 1;
          break;
      case 0:
  #if CYTHON_COMPILING_IN_LIMITED_API
          self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
          if (unlikely(!self) && PyErr_Occurred()) return NULL;
  #else
          self = ((PyCFunctionObject*)cyfunc)->m_self;
  #endif
          break;
      default:
          return NULL;
      }
      return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
  }
  static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
  {
      __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
      PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
      Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
      PyObject *self;
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
      if (unlikely(!meth)) return NULL;
  #else
      PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
  #endif
      switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
      case 1:
          self = args[0];
          args += 1;
          nargs -= 1;
          break;
      case 0:
  #if CYTHON_COMPILING_IN_LIMITED_API
          self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
          if (unlikely(!self) && PyErr_Occurred()) return NULL;
  #else
          self = ((PyCFunctionObject*)cyfunc)->m_self;
  #endif
          break;
      default:
          return NULL;
      }
      #if PY_VERSION_HEX < 0x030e00A6
      size_t nargs_value = (size_t) nargs;
      #else
      Py_ssize_t nargs_value = nargs;
      #endif
      return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
  }
  #endif
  static PyType_Slot __pyx_CyFunctionType_slots[] = {
      {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
      {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
      {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
      {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
      {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
      {Py_tp_methods, (void *)__pyx_CyFunction_methods},
      {Py_tp_members, (void *)__pyx_CyFunction_members},
      {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
      {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
      {0, 0},
  };
  static PyType_Spec __pyx_CyFunctionType_spec = {
      __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
      sizeof(__pyx_CyFunctionObject),
      0,
  #ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
      Py_TPFLAGS_METHOD_DESCRIPTOR |
  #endif
  #if CYTHON_METH_FASTCALL
  #if defined(Py_TPFLAGS_HAVE_VECTORCALL)
      Py_TPFLAGS_HAVE_VECTORCALL |
  #elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
      _Py_TPFLAGS_HAVE_VECTORCALL |
  #endif
  #endif // CYTHON_METH_FASTCALL
  #if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
      Py_TPFLAGS_MANAGED_DICT |
  #endif
      Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
      Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
      __pyx_CyFunctionType_slots
  };
  static int __pyx_CyFunction_init(PyObject *module) {
      __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
      mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
          mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
      if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
          return -1;
      }
      return 0;
  }
  static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
      __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
      m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
      if (unlikely(!m->defaults))
          return NULL;
      return m->defaults;
  }
  static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
      __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
      m->defaults_tuple = tuple;
      Py_INCREF(tuple);
  }
  static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
      __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
      m->defaults_kwdict = dict;
      Py_INCREF(dict);
  }
  static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
      __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
      m->func_annotations = dict;
      Py_INCREF(dict);
  }
  
/* CythonFunction */
  static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                        PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
      PyObject *op = __Pyx_CyFunction_Init(
          PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
          ml, flags, qualname, closure, module, globals, code
      );
      if (likely(op)) {
          PyObject_GC_Track(op);
      }
      return op;
  }
  
/* PyObjectVectorCallMethodKwBuilder */
  #if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
  static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
      PyObject *result;
      PyObject *obj = PyObject_GetAttr(args[0], name);
      if (unlikely(!obj))
          return NULL;
      result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
      Py_DECREF(obj);
      return result;
  }
  #endif
  
/* ExtTypeTest */
  static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
      __Pyx_TypeName obj_type_name;
      __Pyx_TypeName type_name;
      if (unlikely(!type)) {
          PyErr_SetString(PyExc_SystemError, "Missing type object");
          return 0;
      }
      if (likely(__Pyx_TypeCheck(obj, type)))
          return 1;
      obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
      type_name = __Pyx_PyType_GetFullyQualifiedName(type);
      PyErr_Format(PyExc_TypeError,
                   "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                   obj_type_name, type_name);
      __Pyx_DECREF_TypeName(obj_type_name);
      __Pyx_DECREF_TypeName(type_name);
      return 0;
  }
  
/* pop_index */
  static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix) {
      PyObject *r;
      if (unlikely(!py_ix)) return NULL;
      r = __Pyx__PyObject_PopIndex(L, py_ix);
      Py_DECREF(py_ix);
      return r;
  }
  static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix) {
      return __Pyx_PyObject_CallMethod1(L, __pyx_mstate_global->__pyx_n_u_pop, py_ix);
  }
  #if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
  static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix) {
      Py_ssize_t size = PyList_GET_SIZE(L);
      if (likely(size > (((PyListObject*)L)->allocated >> 1))) {
          Py_ssize_t cix = ix;
          if (cix < 0) {
              cix += size;
          }
          if (likely(__Pyx_is_valid_index(cix, size))) {
              PyObject* v = PyList_GET_ITEM(L, cix);
              __Pyx_SET_SIZE(L, Py_SIZE(L) - 1);
              size -= 1;
              memmove(&PyList_GET_ITEM(L, cix), &PyList_GET_ITEM(L, cix+1), (size_t)(size-cix)*sizeof(PyObject*));
              return v;
          }
      }
      if (py_ix == Py_None) {
          return __Pyx__PyObject_PopNewIndex(L, PyLong_FromSsize_t(ix));
      } else {
          return __Pyx__PyObject_PopIndex(L, py_ix);
      }
  }
  #endif
  
/* UnicodeConcatInPlace */
  # if CYTHON_COMPILING_IN_CPYTHON
  static int
  __Pyx_unicode_modifiable(PyObject *unicode, int unsafe_shared)
  {
      if (!__Pyx_IS_UNIQUELY_REFERENCED(unicode, unsafe_shared))
          return 0;
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX > 0x030F0000
      if (PyUnstable_Unicode_GET_CACHED_HASH(unicode) != -1)
          return 0;
  #endif
      if (!PyUnicode_CheckExact(unicode))
          return 0;
      if (PyUnicode_CHECK_INTERNED(unicode))
          return 0;
      return 1;
  }
  static CYTHON_INLINE PyObject *__Pyx_PyUnicode_ConcatInPlaceImpl(PyObject **p_left, PyObject *right, int unsafe_shared
          #if CYTHON_REFNANNY
          , void* __pyx_refnanny
          #endif
      ) {
      PyObject *left = *p_left;
      Py_ssize_t left_len, right_len, new_len;
      if (unlikely(__Pyx_PyUnicode_READY(left) == -1))
          return NULL;
      if (unlikely(__Pyx_PyUnicode_READY(right) == -1))
          return NULL;
      left_len = PyUnicode_GET_LENGTH(left);
      if (left_len == 0) {
          Py_INCREF(right);
          return right;
      }
      right_len = PyUnicode_GET_LENGTH(right);
      if (right_len == 0) {
          Py_INCREF(left);
          return left;
      }
      if (unlikely(left_len > PY_SSIZE_T_MAX - right_len)) {
          PyErr_SetString(PyExc_OverflowError,
                          "strings are too large to concat");
          return NULL;
      }
      new_len = left_len + right_len;
      if (__Pyx_unicode_modifiable(left, unsafe_shared)
              && PyUnicode_CheckExact(right)
              && PyUnicode_KIND(right) <= PyUnicode_KIND(left)
              && !(PyUnicode_IS_ASCII(left) && !PyUnicode_IS_ASCII(right))) {
          int ret;
          __Pyx_GIVEREF(*p_left);
          ret = PyUnicode_Resize(p_left, new_len);
          __Pyx_GOTREF(*p_left);
          if (unlikely(ret != 0))
              return NULL;
          #if PY_VERSION_HEX >= 0x030d0000
          if (unlikely(PyUnicode_CopyCharacters(*p_left, left_len, right, 0, right_len) < 0)) return NULL;
          #else
          _PyUnicode_FastCopyCharacters(*p_left, left_len, right, 0, right_len);
          #endif
          __Pyx_INCREF(*p_left);
          __Pyx_GIVEREF(*p_left);
          return *p_left;
      } else {
          return __Pyx_PyUnicode_Concat(left, right);
      }
    }
  #endif
  
/* RaiseKeywordRequired */
  static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name) {
      PyErr_Format(PyExc_TypeError,
          "%s() needs keyword-only argument %U", func_name, kw_name);
  }
  
/* DictGetItem */
  #if !CYTHON_COMPILING_IN_PYPY
  static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
      PyObject *value;
      if (unlikely(__Pyx_PyDict_GetItemRef(d, key, &value) == 0)) { // no value, no error
          if (unlikely(PyTuple_Check(key))) {
              PyObject* args = PyTuple_Pack(1, key);
              if (likely(args)) {
                  PyErr_SetObject(PyExc_KeyError, args);
                  Py_DECREF(args);
              }
          } else {
              PyErr_SetObject(PyExc_KeyError, key);
          }
      }
      return value;
  }
  #endif
  
/* PyUnicode_Unicode */
  static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
      if (unlikely(obj == Py_None))
          obj = __pyx_mstate_global->__pyx_kp_u_None;
      return __Pyx_NewRef(obj);
  }
  
/* CallUnboundCMethod1 */
  #if CYTHON_COMPILING_IN_CPYTHON
  static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
      int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
      if (likely(was_initialized == 2 && cfunc->func)) {
          int flag = cfunc->flag;
          if (flag == METH_O) {
              return __Pyx_CallCFunction(cfunc, self, arg);
          } else if (flag == METH_FASTCALL) {
              return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
          } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
              return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
          }
      }
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      else if (unlikely(was_initialized == 1)) {
          __Pyx_CachedCFunction tmp_cfunc = {
  #ifndef __cplusplus
              0
  #endif
          };
          tmp_cfunc.type = cfunc->type;
          tmp_cfunc.method_name = cfunc->method_name;
          return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
      }
  #endif
      PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
      __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
      return result;
  }
  #endif
  static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
      PyObject *result = NULL;
      if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
  #if CYTHON_COMPILING_IN_CPYTHON
      if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
          PyObject *args = PyTuple_New(1);
          if (unlikely(!args)) return NULL;
          Py_INCREF(arg);
          PyTuple_SET_ITEM(args, 0, arg);
          if (cfunc->flag & METH_KEYWORDS)
              result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
          else
              result = __Pyx_CallCFunction(cfunc, self, args);
          Py_DECREF(args);
      } else
  #endif
      {
          result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
      }
      return result;
  }
  
/* dict_getitem_default */
  static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
      PyObject* value;
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
      value = PyDict_GetItemWithError(d, key);
      if (unlikely(!value)) {
          if (unlikely(PyErr_Occurred()))
              return NULL;
          value = default_value;
      }
      Py_INCREF(value);
      if ((1));
  #else
      if (PyBytes_CheckExact(key) || PyUnicode_CheckExact(key) || PyLong_CheckExact(key)) {
          value = PyDict_GetItem(d, key);
          if (unlikely(!value)) {
              value = default_value;
          }
          Py_INCREF(value);
      }
  #endif
      else {
          if (default_value == Py_None)
              value = __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key);
          else
              value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_get, d, key, default_value);
      }
      return value;
  }
  
/* pyfrozenset_new (used by py_set_discard_unhashable) */
  static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
      if (it) {
          PyObject* result;
  #if CYTHON_COMPILING_IN_PYPY
          PyObject* args;
          args = PyTuple_Pack(1, it);
          if (unlikely(!args))
              return NULL;
          result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
          Py_DECREF(args);
          return result;
  #else
          if (PyFrozenSet_CheckExact(it)) {
              Py_INCREF(it);
              return it;
          }
          result = PyFrozenSet_New(it);
          if (unlikely(!result))
              return NULL;
          if ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #if CYTHON_COMPILING_IN_LIMITED_API
              || __Pyx_get_runtime_version() >= 0x030A0000
  #endif
              )
              return result;
          {
              Py_ssize_t size = __Pyx_PySet_GET_SIZE(result);
              if (likely(size > 0))
                  return result;
  #if !CYTHON_ASSUME_SAFE_SIZE
              if (unlikely(size < 0)) {
                  Py_DECREF(result);
                  return NULL;
              }
  #endif
          }
          Py_DECREF(result);
  #endif
      }
      return __Pyx_PyObject_CallNoArg((PyObject*) &PyFrozenSet_Type);
  }
  
/* py_set_discard_unhashable (used by py_set_discard) */
  static int __Pyx_PySet_DiscardUnhashable(PyObject *set, PyObject *key) {
      PyObject *tmpkey;
      int rv;
      if (likely(!PySet_Check(key) || !PyErr_ExceptionMatches(PyExc_TypeError)))
          return -1;
      PyErr_Clear();
      tmpkey = __Pyx_PyFrozenSet_New(key);
      if (tmpkey == NULL)
          return -1;
      rv = PySet_Discard(set, tmpkey);
      Py_DECREF(tmpkey);
      return rv;
  }
  
/* py_set_discard */
  static CYTHON_INLINE int __Pyx_PySet_Discard(PyObject *set, PyObject *key) {
      int found = PySet_Discard(set, key);
      if (unlikely(found < 0)) {
          found = __Pyx_PySet_DiscardUnhashable(set, key);
      }
      return found;
  }
  
/* AllocateExtensionType */
  static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
      if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
          allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
          return alloc_func(t, 0);
      } else {
          newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
      #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
          if (!tp_new) {
              PyObject *new_str = PyUnicode_FromString("__new__");
              if (likely(new_str)) {
                  PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                  Py_DECREF(new_str);
                  return o;
              } else
                  return NULL;
          } else
      #endif
          return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
      }
  }
  
/* VoidPtrExport */
  static int __Pyx_ExportVoidPtr(PyObject *api_dict, const char *name, void *p, const char *sig) {
      PyObject *cobj;
      cobj = PyCapsule_New(p, sig, 0);
      if (!cobj)
          goto bad;
      if (PyDict_SetItemString(api_dict, name, cobj) < 0)
          goto bad;
      Py_DECREF(cobj);
      return 0;
  bad:
      Py_XDECREF(cobj);
      return -1;
  }
  
/* GetApiDict */
  static PyObject *__Pyx_ApiExport_GetApiDict(void) {
      PyObject *d;
      if (__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_capi, &d) == -1)
          return NULL;
      if (!d) {
          d = PyDict_New();
          if (!d)
              goto bad;
          if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_pyx_capi, d) < 0)
              goto bad;
      }
      return d;
  bad:
      Py_XDECREF(d);
      return NULL;
  }
  
/* FunctionExport */
  static int __Pyx_ExportFunction(PyObject *api_dict, const char *name, void (*f)(void), const char *sig) {
      PyObject *cobj;
      union {
          void (*fp)(void);
          void *p;
      } tmp;
      tmp.fp = f;
      cobj = PyCapsule_New(tmp.p, sig, 0);
      if (!cobj)
          goto bad;
      if (PyDict_SetItemString(api_dict, name, cobj) < 0)
          goto bad;
      Py_DECREF(cobj);
      return 0;
  bad:
      Py_XDECREF(cobj);
      return -1;
  }
  
/* ValidateBasesTuple (used by PyType_Ready) */
  #if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
  static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
      Py_ssize_t i, n;
  #if CYTHON_ASSUME_SAFE_SIZE
      n = PyTuple_GET_SIZE(bases);
  #else
      n = PyTuple_Size(bases);
      if (unlikely(n < 0)) return -1;
  #endif
      for (i = 1; i < n; i++)
      {
          PyTypeObject *b;
  #if CYTHON_AVOID_BORROWED_REFS
          PyObject *b0 = PySequence_GetItem(bases, i);
          if (!b0) return -1;
  #elif CYTHON_ASSUME_SAFE_MACROS
          PyObject *b0 = PyTuple_GET_ITEM(bases, i);
  #else
          PyObject *b0 = PyTuple_GetItem(bases, i);
          if (!b0) return -1;
  #endif
          b = (PyTypeObject*) b0;
          if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
          {
              __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
              PyErr_Format(PyExc_TypeError,
                  "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
              __Pyx_DECREF_TypeName(b_name);
  #if CYTHON_AVOID_BORROWED_REFS
              Py_DECREF(b0);
  #endif
              return -1;
          }
          if (dictoffset == 0)
          {
              Py_ssize_t b_dictoffset = 0;
  #if CYTHON_USE_TYPE_SLOTS
              b_dictoffset = b->tp_dictoffset;
  #else
              PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
              if (!py_b_dictoffset) goto dictoffset_return;
              b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
              Py_DECREF(py_b_dictoffset);
              if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
  #endif
              if (b_dictoffset) {
                  {
                      __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                      PyErr_Format(PyExc_TypeError,
                          "extension type '%.200s' has no __dict__ slot, "
                          "but base type '" __Pyx_FMT_TYPENAME "' has: "
                          "either add 'cdef dict __dict__' to the extension type "
                          "or add '__slots__ = [...]' to the base type",
                          type_name, b_name);
                      __Pyx_DECREF_TypeName(b_name);
                  }
  #if !CYTHON_USE_TYPE_SLOTS
                dictoffset_return:
  #endif
  #if CYTHON_AVOID_BORROWED_REFS
                  Py_DECREF(b0);
  #endif
                  return -1;
              }
          }
  #if CYTHON_AVOID_BORROWED_REFS
          Py_DECREF(b0);
  #endif
      }
      return 0;
  }
  #endif
  
/* PyType_Ready */
  CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
      while (t) {
          PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
          if (bases) {
              return 1;
          }
          t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
      }
      return 0;
  }
  static int __Pyx_PyType_Ready(PyTypeObject *t) {
  #if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
      (void)__Pyx_PyObject_CallMethod0;
  #if CYTHON_USE_TYPE_SPECS
      (void)__Pyx_validate_bases_tuple;
  #endif
      return PyType_Ready(t);
  #else
      int r;
      if (!__Pyx_PyType_HasMultipleInheritance(t)) {
          return PyType_Ready(t);
      }
      PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
      if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
          return -1;
  #if !defined(PYSTON_MAJOR_VERSION)
      {
          int gc_was_enabled;
      #if PY_VERSION_HEX >= 0x030A00b1
          gc_was_enabled = PyGC_Disable();
          (void)__Pyx_PyObject_CallMethod0;
      #else
          PyObject *ret, *py_status;
          PyObject *gc = NULL;
          #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                  !CYTHON_COMPILING_IN_GRAAL
          gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
          #endif
          if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
          if (unlikely(!gc)) return -1;
          py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
          if (unlikely(!py_status)) {
              Py_DECREF(gc);
              return -1;
          }
          gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
          Py_DECREF(py_status);
          if (gc_was_enabled > 0) {
              ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
              if (unlikely(!ret)) {
                  Py_DECREF(gc);
                  return -1;
              }
              Py_DECREF(ret);
          } else if (unlikely(gc_was_enabled == -1)) {
              Py_DECREF(gc);
              return -1;
          }
      #endif
          t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
  #if PY_VERSION_HEX >= 0x030A0000
          t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
  #endif
  #else
          (void)__Pyx_PyObject_CallMethod0;
  #endif
      r = PyType_Ready(t);
  #if !defined(PYSTON_MAJOR_VERSION)
          t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
      #if PY_VERSION_HEX >= 0x030A00b1
          if (gc_was_enabled)
              PyGC_Enable();
      #else
          if (gc_was_enabled) {
              PyObject *tp, *v, *tb;
              PyErr_Fetch(&tp, &v, &tb);
              ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
              if (likely(ret || r == -1)) {
                  Py_XDECREF(ret);
                  PyErr_Restore(tp, v, tb);
              } else {
                  Py_XDECREF(tp);
                  Py_XDECREF(v);
                  Py_XDECREF(tb);
                  r = -1;
              }
          }
          Py_DECREF(gc);
      #endif
      }
  #endif
      return r;
  #endif
  }
  
/* DelItemOnTypeDict (used by SetupReduce) */
  static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
      int result;
      PyObject *tp_dict;
  #if CYTHON_COMPILING_IN_LIMITED_API
      tp_dict = __Pyx_GetTypeDict(tp);
      if (unlikely(!tp_dict)) return -1;
  #else
      tp_dict = tp->tp_dict;
  #endif
      result = PyDict_DelItem(tp_dict, k);
      if (likely(!result)) PyType_Modified(tp);
      return result;
  }
  
/* SetupReduce */
  static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
    int ret;
    PyObject *name_attr;
    name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name);
    if (likely(name_attr)) {
        ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
    } else {
        ret = -1;
    }
    if (unlikely(ret < 0)) {
        PyErr_Clear();
        ret = 0;
    }
    Py_XDECREF(name_attr);
    return ret;
  }
  static int __Pyx_setup_reduce(PyObject* type_obj) {
      int ret = 0;
      PyObject *object_reduce = NULL;
      PyObject *object_getstate = NULL;
      PyObject *object_reduce_ex = NULL;
      PyObject *reduce = NULL;
      PyObject *reduce_ex = NULL;
      PyObject *reduce_cython = NULL;
      PyObject *setstate = NULL;
      PyObject *setstate_cython = NULL;
      PyObject *getstate = NULL;
  #if CYTHON_USE_PYTYPE_LOOKUP
      getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
  #else
      getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
      if (!getstate && PyErr_Occurred()) {
          goto __PYX_BAD;
      }
  #endif
      if (getstate) {
  #if CYTHON_USE_PYTYPE_LOOKUP
          object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
  #else
          object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
          if (!object_getstate && PyErr_Occurred()) {
              goto __PYX_BAD;
          }
  #endif
          if (object_getstate != getstate) {
              goto __PYX_GOOD;
          }
      }
  #if CYTHON_USE_PYTYPE_LOOKUP
      object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
  #else
      object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
  #endif
      reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
      if (reduce_ex == object_reduce_ex) {
  #if CYTHON_USE_PYTYPE_LOOKUP
          object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
  #else
          object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
  #endif
          reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
          if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
              reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
              if (likely(reduce_cython)) {
                  ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                  ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
              } else if (reduce == object_reduce || PyErr_Occurred()) {
                  goto __PYX_BAD;
              }
              setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
              if (!setstate) PyErr_Clear();
              if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                  setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                  if (likely(setstate_cython)) {
                      ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                      ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                  } else if (!setstate || PyErr_Occurred()) {
                      goto __PYX_BAD;
                  }
              }
              PyType_Modified((PyTypeObject*)type_obj);
          }
      }
      goto __PYX_GOOD;
  __PYX_BAD:
      if (!PyErr_Occurred()) {
          __Pyx_TypeName type_obj_name =
              __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
          PyErr_Format(PyExc_RuntimeError,
              "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
          __Pyx_DECREF_TypeName(type_obj_name);
      }
      ret = -1;
  __PYX_GOOD:
  #if !CYTHON_USE_PYTYPE_LOOKUP
      Py_XDECREF(object_reduce);
      Py_XDECREF(object_reduce_ex);
      Py_XDECREF(object_getstate);
      Py_XDECREF(getstate);
  #endif
      Py_XDECREF(reduce);
      Py_XDECREF(reduce_ex);
      Py_XDECREF(reduce_cython);
      Py_XDECREF(setstate);
      Py_XDECREF(setstate_cython);
      return ret;
  }
  
/* SetVTable */
  static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
      PyObject *ob = PyCapsule_New(vtable, 0, 0);
      if (unlikely(!ob))
          goto bad;
  #if CYTHON_COMPILING_IN_LIMITED_API
      if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
  #else
      if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
  #endif
          goto bad;
      Py_DECREF(ob);
      return 0;
  bad:
      Py_XDECREF(ob);
      return -1;
  }
  
/* GetVTable (used by MergeVTables) */
  static void* __Pyx_GetVtable(PyTypeObject *type) {
      void* ptr;
  #if CYTHON_COMPILING_IN_LIMITED_API
      PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
  #else
      PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
  #endif
      if (!ob)
          goto bad;
      ptr = PyCapsule_GetPointer(ob, 0);
      if (!ptr && !PyErr_Occurred())
          PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
      Py_DECREF(ob);
      return ptr;
  bad:
      Py_XDECREF(ob);
      return NULL;
  }
  
/* MergeVTables */
  static int __Pyx_MergeVtables(PyTypeObject *type) {
      int i=0;
      Py_ssize_t size;
      void** base_vtables;
      __Pyx_TypeName tp_base_name = NULL;
      __Pyx_TypeName base_name = NULL;
      void* unknown = (void*)-1;
      PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
      int base_depth = 0;
      {
          PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
          while (base) {
              base_depth += 1;
              base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
          }
      }
      base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
      base_vtables[0] = unknown;
  #if CYTHON_COMPILING_IN_LIMITED_API
      size = PyTuple_Size(bases);
      if (size < 0) goto other_failure;
  #else
      size = PyTuple_GET_SIZE(bases);
  #endif
      for (i = 1; i < size; i++) {
          PyObject *basei;
          void* base_vtable;
  #if CYTHON_AVOID_BORROWED_REFS
          basei = PySequence_GetItem(bases, i);
          if (unlikely(!basei)) goto other_failure;
  #elif !CYTHON_ASSUME_SAFE_MACROS
          basei = PyTuple_GetItem(bases, i);
          if (unlikely(!basei)) goto other_failure;
  #else
          basei = PyTuple_GET_ITEM(bases, i);
  #endif
          base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
  #if CYTHON_AVOID_BORROWED_REFS
          Py_DECREF(basei);
  #endif
          if (base_vtable != NULL) {
              int j;
              PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
              for (j = 0; j < base_depth; j++) {
                  if (base_vtables[j] == unknown) {
                      base_vtables[j] = __Pyx_GetVtable(base);
                      base_vtables[j + 1] = unknown;
                  }
                  if (base_vtables[j] == base_vtable) {
                      break;
                  } else if (base_vtables[j] == NULL) {
                      goto bad;
                  }
                  base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
              }
          }
      }
      PyErr_Clear();
      PyMem_Free(base_vtables);
      return 0;
  bad:
      {
          PyTypeObject* basei = NULL;
          PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
          tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
  #if CYTHON_AVOID_BORROWED_REFS
          basei = (PyTypeObject*)PySequence_GetItem(bases, i);
          if (unlikely(!basei)) goto really_bad;
  #elif !CYTHON_ASSUME_SAFE_MACROS
          basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
          if (unlikely(!basei)) goto really_bad;
  #else
          basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
  #endif
          base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
  #if CYTHON_AVOID_BORROWED_REFS
          Py_DECREF(basei);
  #endif
      }
      PyErr_Format(PyExc_TypeError,
          "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
  #if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
  really_bad: // bad has failed!
  #endif
      __Pyx_DECREF_TypeName(tp_base_name);
      __Pyx_DECREF_TypeName(base_name);
  #if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
  other_failure:
  #endif
      PyMem_Free(base_vtables);
      return -1;
  }
  
/* TypeImport */
  #ifndef __PYX_HAVE_RT_ImportType_3_2_4
  #define __PYX_HAVE_RT_ImportType_3_2_4
  static PyTypeObject *__Pyx_ImportType_3_2_4(PyObject *module, const char *module_name, const char *class_name,
      size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_4 check_size)
  {
      PyObject *result = 0;
      Py_ssize_t basicsize;
      Py_ssize_t itemsize;
  #if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
      PyObject *py_basicsize;
      PyObject *py_itemsize;
  #endif
      result = PyObject_GetAttrString(module, class_name);
      if (!result)
          goto bad;
      if (!PyType_Check(result)) {
          PyErr_Format(PyExc_TypeError,
              "%.200s.%.200s is not a type object",
              module_name, class_name);
          goto bad;
      }
  #if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
      basicsize = ((PyTypeObject *)result)->tp_basicsize;
      itemsize = ((PyTypeObject *)result)->tp_itemsize;
  #else
      if (size == 0) {
          return (PyTypeObject *)result;
      }
      py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
      if (!py_basicsize)
          goto bad;
      basicsize = PyLong_AsSsize_t(py_basicsize);
      Py_DECREF(py_basicsize);
      py_basicsize = 0;
      if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
          goto bad;
      py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
      if (!py_itemsize)
          goto bad;
      itemsize = PyLong_AsSsize_t(py_itemsize);
      Py_DECREF(py_itemsize);
      py_itemsize = 0;
      if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
          goto bad;
  #endif
      if (itemsize) {
          if (size % alignment) {
              alignment = size % alignment;
          }
          if (itemsize < (Py_ssize_t)alignment)
              itemsize = (Py_ssize_t)alignment;
      }
      if ((size_t)(basicsize + itemsize) < size) {
          PyErr_Format(PyExc_ValueError,
              "%.200s.%.200s size changed, may indicate binary incompatibility. "
              "Expected %zd from C header, got %zd from PyObject",
              module_name, class_name, size, basicsize+itemsize);
          goto bad;
      }
      if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_4 &&
              ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
          PyErr_Format(PyExc_ValueError,
              "%.200s.%.200s size changed, may indicate binary incompatibility. "
              "Expected %zd from C header, got %zd-%zd from PyObject",
              module_name, class_name, size, basicsize, basicsize+itemsize);
          goto bad;
      }
      else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_4 && (size_t)basicsize > size) {
          if (PyErr_WarnFormat(NULL, 0,
                  "%.200s.%.200s size changed, may indicate binary incompatibility. "
                  "Expected %zd from C header, got %zd from PyObject",
                  module_name, class_name, size, basicsize) < 0) {
              goto bad;
          }
      }
      return (PyTypeObject *)result;
  bad:
      Py_XDECREF(result);
      return NULL;
  }
  #endif
  
/* PxdImportShared (used by VoidPtrImport) */
  #ifndef __PYX_HAVE_RT_ImportFromPxd_3_2_4
  #define __PYX_HAVE_RT_ImportFromPxd_3_2_4
  static int __Pyx_ImportFromPxd_3_2_4(PyObject *module, const char *name, void **p, const char *sig, const char *what) {
      PyObject *d = 0;
      PyObject *cobj = 0;
      d = PyObject_GetAttrString(module, "__pyx_capi__");
      if (!d)
          goto bad;
  #if (defined(Py_LIMITED_API) && Py_LIMITED_API >= 0x030d0000) || (!defined(Py_LIMITED_API) && PY_VERSION_HEX >= 0x030d0000)
      PyDict_GetItemStringRef(d, name, &cobj);
  #else
      cobj = PyDict_GetItemString(d, name);
      Py_XINCREF(cobj);
  #endif
      if (!cobj) {
          PyErr_Format(PyExc_ImportError,
              "%.200s does not export expected C %.8s %.200s",
                  PyModule_GetName(module), what, name);
          goto bad;
      }
      if (!PyCapsule_IsValid(cobj, sig)) {
          PyErr_Format(PyExc_TypeError,
              "C %.8s %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
               what, PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
          goto bad;
      }
      *p = PyCapsule_GetPointer(cobj, sig);
      if (!(*p))
          goto bad;
      Py_DECREF(d);
      Py_DECREF(cobj);
      return 0;
  bad:
      Py_XDECREF(d);
      Py_XDECREF(cobj);
      return -1;
  }
  #endif
  
/* VoidPtrImport */
  #ifndef __PYX_HAVE_RT_ImportVoidPtr_3_2_4
  #define __PYX_HAVE_RT_ImportVoidPtr_3_2_4
  static int __Pyx_ImportVoidPtr_3_2_4(PyObject *module, const char *name, void **p, const char *sig) {
      return __Pyx_ImportFromPxd_3_2_4(module, name, p, sig, "variable");
  }
  #endif
  
/* HasAttr (used by ImportImpl) */
  #if __PYX_LIMITED_VERSION_HEX < 0x030d0000
  static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
      PyObject *r;
      if (unlikely(!PyUnicode_Check(n))) {
          PyErr_SetString(PyExc_TypeError,
                          "hasattr(): attribute name must be string");
          return -1;
      }
      r = __Pyx_PyObject_GetAttrStrNoError(o, n);
      if (!r) {
          return (unlikely(PyErr_Occurred())) ? -1 : 0;
      } else {
          Py_DECREF(r);
          return 1;
      }
  }
  #endif
  
/* ImportImpl (used by Import) */
  static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
      PyObject *imported_module = PyImport_GetModule(qualname);
      if (unlikely(!imported_module)) {
          *module = NULL;
          if (PyErr_Occurred()) {
              return -1;
          }
          return 0;
      }
      *module = imported_module;
      return 1;
  }
  static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
      PyObject *imported_module;
      PyObject *top_level_package_name;
      Py_ssize_t i;
      int status, module_found;
      Py_ssize_t dot_index;
      module_found = __Pyx__Import_GetModule(qualname, &imported_module);
      if (unlikely(!module_found || module_found == -1)) {
          *module = NULL;
          return module_found;
      }
      if (imported_names) {
          for (i = 0; i < len_imported_names; i++) {
              PyObject *imported_name = imported_names[i];
  #if __PYX_LIMITED_VERSION_HEX < 0x030d0000
              int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
  #else
              int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
              if (unlikely(has_imported_attribute == -1)) goto error;
  #endif
              if (!has_imported_attribute) {
                  goto not_found;
              }
          }
          *module = imported_module;
          return 1;
      }
      dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
      if (dot_index == -1) {
          *module = imported_module;
          return 1;
      }
      if (unlikely(dot_index == -2)) goto error;
      top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
      if (unlikely(!top_level_package_name)) goto error;
      Py_DECREF(imported_module);
      status = __Pyx__Import_GetModule(top_level_package_name, module);
      Py_DECREF(top_level_package_name);
      return status;
  error:
      Py_DECREF(imported_module);
      *module = NULL;
      return -1;
  not_found:
      Py_DECREF(imported_module);
      *module = NULL;
      return 0;
  }
  static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
      PyObject *module = 0;
      PyObject *empty_dict = 0;
      PyObject *from_list = 0;
      int module_found;
      if (!qualname) {
          qualname = name;
      }
      module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
      if (likely(module_found == 1)) {
          return module;
      } else if (unlikely(module_found == -1)) {
          return NULL;
      }
      empty_dict = PyDict_New();
      if (unlikely(!empty_dict))
          goto bad;
      if (imported_names) {
  #if CYTHON_COMPILING_IN_CPYTHON
          from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
          if (unlikely(!from_list))
              goto bad;
  #else
          from_list = PyList_New(len_imported_names);
          if (unlikely(!from_list)) goto bad;
          for (Py_ssize_t i=0; i<len_imported_names; ++i) {
              if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
          }
  #endif
      }
      if (level == -1) {
          const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
          if (package_sep != (0)) {
              module = PyImport_ImportModuleLevelObject(
                  name, moddict, empty_dict, from_list, 1);
              if (unlikely(!module)) {
                  if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                      goto bad;
                  PyErr_Clear();
              }
          }
          level = 0;
      }
      if (!module) {
          module = PyImport_ImportModuleLevelObject(
              name, moddict, empty_dict, from_list, level);
      }
  bad:
      Py_XDECREF(from_list);
      Py_XDECREF(empty_dict);
      return module;
  }
  
/* Import */
  static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
      return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
  }
  
/* ImportFrom */
  static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
      PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
      if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
          const char* module_name_str = 0;
          PyObject* module_name = 0;
          PyObject* module_dot = 0;
          PyObject* full_name = 0;
          PyErr_Clear();
          module_name_str = PyModule_GetName(module);
          if (unlikely(!module_name_str)) { goto modbad; }
          module_name = PyUnicode_FromString(module_name_str);
          if (unlikely(!module_name)) { goto modbad; }
          module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__9);
          if (unlikely(!module_dot)) { goto modbad; }
          full_name = PyUnicode_Concat(module_dot, name);
          if (unlikely(!full_name)) { goto modbad; }
          #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                  CYTHON_COMPILING_IN_GRAAL
          {
              PyObject *modules = PyImport_GetModuleDict();
              if (unlikely(!modules))
                  goto modbad;
              value = PyObject_GetItem(modules, full_name);
          }
          #else
          value = PyImport_GetModule(full_name);
          #endif
        modbad:
          Py_XDECREF(full_name);
          Py_XDECREF(module_dot);
          Py_XDECREF(module_name);
      }
      if (unlikely(!value)) {
          PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
      }
      return value;
  }
  
/* Py3UpdateBases */
  static PyObject*
  __Pyx_PEP560_update_bases(PyObject *bases)
  {
      Py_ssize_t i, j, size_bases;
      PyObject *base = NULL, *meth, *new_base, *result, *new_bases = NULL;
  #if CYTHON_ASSUME_SAFE_SIZE
      size_bases = PyTuple_GET_SIZE(bases);
  #else
      size_bases = PyTuple_Size(bases);
      if (size_bases < 0) return NULL;
  #endif
      for (i = 0; i < size_bases; i++) {
  #if CYTHON_AVOID_BORROWED_REFS
          Py_CLEAR(base);
  #endif
  #if CYTHON_ASSUME_SAFE_MACROS
          base = PyTuple_GET_ITEM(bases, i);
  #else
          base = PyTuple_GetItem(bases, i);
          if (!base) goto error;
  #endif
  #if CYTHON_AVOID_BORROWED_REFS
          Py_INCREF(base);
  #endif
          if (PyType_Check(base)) {
              if (new_bases) {
                  if (PyList_Append(new_bases, base) < 0) {
                      goto error;
                  }
              }
              continue;
          }
          meth = __Pyx_PyObject_GetAttrStrNoError(base, __pyx_mstate_global->__pyx_n_u_mro_entries);
          if (!meth && PyErr_Occurred()) {
              goto error;
          }
          if (!meth) {
              if (new_bases) {
                  if (PyList_Append(new_bases, base) < 0) {
                      goto error;
                  }
              }
              continue;
          }
          new_base = __Pyx_PyObject_CallOneArg(meth, bases);
          Py_DECREF(meth);
          if (!new_base) {
              goto error;
          }
          if (!PyTuple_Check(new_base)) {
              PyErr_SetString(PyExc_TypeError,
                              "__mro_entries__ must return a tuple");
              Py_DECREF(new_base);
              goto error;
          }
          if (!new_bases) {
              if (!(new_bases = PyList_New(i))) {
                  goto error;
              }
              for (j = 0; j < i; j++) {
                  PyObject *base_from_list;
  #if CYTHON_ASSUME_SAFE_MACROS
                  base_from_list = PyTuple_GET_ITEM(bases, j);
                  PyList_SET_ITEM(new_bases, j, base_from_list);
                  Py_INCREF(base_from_list);
  #else
                  base_from_list = PyTuple_GetItem(bases, j);
                  if (!base_from_list) goto error;
                  Py_INCREF(base_from_list);
                  if (PyList_SetItem(new_bases, j, base_from_list) < 0) goto error;
  #endif
              }
          }
  #if CYTHON_ASSUME_SAFE_SIZE
          j = PyList_GET_SIZE(new_bases);
  #else
          j = PyList_Size(new_bases);
          if (j < 0) goto error;
  #endif
          if (PyList_SetSlice(new_bases, j, j, new_base) < 0) {
              goto error;
          }
          Py_DECREF(new_base);
      }
      if (!new_bases) {
          Py_INCREF(bases);
          return bases;
      }
      result = PyList_AsTuple(new_bases);
      Py_DECREF(new_bases);
  #if CYTHON_AVOID_BORROWED_REFS
      Py_XDECREF(base);
  #endif
      return result;
  error:
      Py_XDECREF(new_bases);
  #if CYTHON_AVOID_BORROWED_REFS
      Py_XDECREF(base);
  #endif
      return NULL;
  }
  
/* CalculateMetaclass */
  static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
      Py_ssize_t i, nbases;
  #if CYTHON_ASSUME_SAFE_SIZE
      nbases = PyTuple_GET_SIZE(bases);
  #else
      nbases = PyTuple_Size(bases);
      if (nbases < 0) return NULL;
  #endif
      for (i=0; i < nbases; i++) {
          PyTypeObject *tmptype;
  #if CYTHON_ASSUME_SAFE_MACROS
          PyObject *tmp = PyTuple_GET_ITEM(bases, i);
  #else
          PyObject *tmp = PyTuple_GetItem(bases, i);
          if (!tmp) return NULL;
  #endif
          tmptype = Py_TYPE(tmp);
          if (!metaclass) {
              metaclass = tmptype;
              continue;
          }
          if (PyType_IsSubtype(metaclass, tmptype))
              continue;
          if (PyType_IsSubtype(tmptype, metaclass)) {
              metaclass = tmptype;
              continue;
          }
          PyErr_SetString(PyExc_TypeError,
                          "metaclass conflict: "
                          "the metaclass of a derived class "
                          "must be a (non-strict) subclass "
                          "of the metaclasses of all its bases");
          return NULL;
      }
      if (!metaclass) {
          metaclass = &PyType_Type;
      }
      Py_INCREF((PyObject*) metaclass);
      return (PyObject*) metaclass;
  }
  
/* Py3ClassCreate */
  static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                             PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
      PyObject *ns;
      if (metaclass) {
          PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_mstate_global->__pyx_n_u_prepare);
          if (prep) {
              PyObject *pargs[3] = {NULL, name, bases};
              ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
              Py_DECREF(prep);
          } else {
              if (unlikely(PyErr_Occurred()))
                  return NULL;
              ns = PyDict_New();
          }
      } else {
          ns = PyDict_New();
      }
      if (unlikely(!ns))
          return NULL;
      if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_module, modname) < 0)) goto bad;
      if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_qualname, qualname) < 0)) goto bad;
      if (unlikely(doc && PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_doc, doc) < 0)) goto bad;
      return ns;
  bad:
      Py_DECREF(ns);
      return NULL;
  }
  static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                        PyObject *dict, PyObject *mkw,
                                        int calculate_metaclass, int allow_py2_metaclass) {
      PyObject *result;
      PyObject *owned_metaclass = NULL;
      PyObject *margs[4] = {NULL, name, bases, dict};
      if (allow_py2_metaclass) {
          owned_metaclass = PyObject_GetItem(dict, __pyx_mstate_global->__pyx_n_u_metaclass);
          if (owned_metaclass) {
              metaclass = owned_metaclass;
          } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
              PyErr_Clear();
          } else {
              return NULL;
          }
      }
      if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
          metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
          Py_XDECREF(owned_metaclass);
          if (unlikely(!metaclass))
              return NULL;
          owned_metaclass = metaclass;
      }
      result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
      Py_XDECREF(owned_metaclass);
      return result;
  }
  
/* DataclassesCallHelper */
  static int __Pyx_DataclassesCallHelper_FilterToDict(PyObject *callable, PyObject *kwds, PyObject *new_kwds, PyObject *args_list, int is_kwonly) {
      Py_ssize_t size, i;
      size = PySequence_Size(args_list);
      if (unlikely(size < 0)) return -1;
      for (i=0; i<size; ++i) {
          PyObject *key, *value;
          int setitem_result;
          key = PySequence_GetItem(args_list, i);
          if (!key) return -1;
          if (PyUnicode_Check(key) && (
                  PyUnicode_CompareWithASCIIString(key, "self") == 0 ||
                  PyUnicode_CompareWithASCIIString(key, "_cls") == 0)) {
              Py_DECREF(key);
              continue;
          }
          value = PyDict_GetItem(kwds, key);
          if (!value) {
              if (is_kwonly) {
                  Py_DECREF(key);
                  continue;
              } else {
                  value = Py_None;
                  if (PyErr_WarnFormat(
                          PyExc_RuntimeWarning, 1,
                          "Argument %S not passed to %R. This is likely a bug in Cython so please report it.",
                          key, callable) == -1) {
                      Py_DECREF(key);
                      return -1;
                  }
              }
          }
          Py_INCREF(value);
          setitem_result = PyDict_SetItem(new_kwds, key, value);
          Py_DECREF(key);
          Py_DECREF(value);
          if (setitem_result == -1) return -1;
      }
      return 0;
  }
  static PyObject* __Pyx_DataclassesCallHelper(PyObject *callable, PyObject *kwds) {
      PyObject *new_kwds=NULL, *result=NULL;
      PyObject *inspect;
      PyObject *args_list=NULL, *kwonly_args_list=NULL, *getfullargspec_result=NULL;
      inspect = PyImport_ImportModule("inspect");
      if (!inspect) goto bad;
      getfullargspec_result = PyObject_CallMethodObjArgs(inspect, __pyx_mstate_global->__pyx_kp_u_getfullargspec, callable, NULL);
      Py_DECREF(inspect);
      if (!getfullargspec_result) goto bad;
      args_list = PyObject_GetAttrString(getfullargspec_result, "args");
      if (!args_list) goto bad;
      kwonly_args_list = PyObject_GetAttrString(getfullargspec_result, "kwonlyargs");
      if (!kwonly_args_list) goto bad;
      new_kwds = PyDict_New();
      if (!new_kwds) goto bad;
      if (__Pyx_DataclassesCallHelper_FilterToDict(callable, kwds, new_kwds, args_list, 0) == -1) goto bad;
      if (__Pyx_DataclassesCallHelper_FilterToDict(callable, kwds, new_kwds, kwonly_args_list, 1) == -1) goto bad;
      result = PyObject_Call(callable, __pyx_mstate_global->__pyx_empty_tuple, new_kwds);
  bad:
      Py_XDECREF(getfullargspec_result);
      Py_XDECREF(args_list);
      Py_XDECREF(kwonly_args_list);
      Py_XDECREF(new_kwds);
      return result;
  }
  
/* LoadDataclassesModule */
  static PyObject *__Pyx_Load_dataclasses_Module(void) {
      return PyImport_Import(__pyx_mstate_global->__pyx_n_u_dataclasses);
  }
  
/* GetNameInClass */
  static PyObject *__Pyx__GetNameInClass(PyObject *nmspace, PyObject *name) {
      PyObject *result;
      PyObject *dict;
      assert(PyType_Check(nmspace));
  #if CYTHON_USE_TYPE_SLOTS
      dict = ((PyTypeObject*)nmspace)->tp_dict;
      Py_XINCREF(dict);
  #else
      dict = PyObject_GetAttr(nmspace, __pyx_mstate_global->__pyx_n_u_dict);
  #endif
      if (likely(dict)) {
          result = PyObject_GetItem(dict, name);
          Py_DECREF(dict);
          if (result) {
              return result;
          }
      }
      PyErr_Clear();
      __Pyx_GetModuleGlobalNameUncached(result, name);
      return result;
  }
  
/* CLineInTraceback (used by AddTraceback) */
  #if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
  #define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
  #elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  #define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
  #else
  PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
      PyObject **dict_ptr = _PyObject_GetDictPtr(o);
      return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
  }
  #endif
  static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
      PyObject *use_cline = NULL;
      PyObject *ptype, *pvalue, *ptraceback;
      PyObject *cython_runtime_dict;
      CYTHON_MAYBE_UNUSED_VAR(tstate);
      if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
          return c_line;
      }
      __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
      cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
      if (likely(cython_runtime_dict)) {
          __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
              use_cline, cython_runtime_dict,
              __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
      }
      if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
          c_line = 0;
      }
      Py_XDECREF(use_cline);
      Py_XDECREF(cython_runtime_dict);
      __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
      return c_line;
  }
  #endif
  
/* CodeObjectCache (used by AddTraceback) */
  static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
      int start = 0, mid = 0, end = count - 1;
      if (end >= 0 && code_line > entries[end].code_line) {
          return count;
      }
      while (start < end) {
          mid = start + (end - start) / 2;
          if (code_line < entries[mid].code_line) {
              end = mid;
          } else if (code_line > entries[mid].code_line) {
               start = mid + 1;
          } else {
              return mid;
          }
      }
      if (code_line <= entries[mid].code_line) {
          return mid;
      } else {
          return mid + 1;
      }
  }
  static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
      __Pyx_CachedCodeObjectType* code_object;
      int pos;
      if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
          return NULL;
      }
      pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
      if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
          return NULL;
      }
      code_object = code_cache->entries[pos].code_object;
      Py_INCREF(code_object);
      return code_object;
  }
  static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
      (void)__pyx__find_code_object;
      return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
  #else
      struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
      if (old_count < 0) {
          __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
          return NULL;
      }
  #endif
      __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
  #endif
      return result;
  #endif
  }
  static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
  {
      int pos, i;
      __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
      if (unlikely(!code_line)) {
          return;
      }
      if (unlikely(!entries)) {
          entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
          if (likely(entries)) {
              code_cache->entries = entries;
              code_cache->max_count = 64;
              code_cache->count = 1;
              entries[0].code_line = code_line;
              entries[0].code_object = code_object;
              Py_INCREF(code_object);
          }
          return;
      }
      pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
      if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
          __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
          entries[pos].code_object = code_object;
          Py_INCREF(code_object);
          Py_DECREF(tmp);
          return;
      }
      if (code_cache->count == code_cache->max_count) {
          int new_max = code_cache->max_count + 64;
          entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
              code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
          if (unlikely(!entries)) {
              return;
          }
          code_cache->entries = entries;
          code_cache->max_count = new_max;
      }
      for (i=code_cache->count; i>pos; i--) {
          entries[i] = entries[i-1];
      }
      entries[pos].code_line = code_line;
      entries[pos].code_object = code_object;
      code_cache->count++;
      Py_INCREF(code_object);
  }
  static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
      (void)__pyx__insert_code_object;
      return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
  #else
      struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      __pyx_nonatomic_int_type expected = 0;
      if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
          return;
      }
  #endif
      __pyx__insert_code_object(code_cache, code_line, code_object);
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
  #endif
  #endif
  }
  
/* AddTraceback */
  #include "compile.h"
  #include "frameobject.h"
  #include "traceback.h"
  #if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
    #ifndef Py_BUILD_CORE
      #define Py_BUILD_CORE 1
    #endif
    #include "internal/pycore_frame.h"
  #endif
  #if CYTHON_COMPILING_IN_LIMITED_API
  static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                         PyObject *firstlineno, PyObject *name) {
      PyObject *replace = NULL;
      if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
      if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
      replace = PyObject_GetAttrString(code, "replace");
      if (likely(replace)) {
          PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
          Py_DECREF(replace);
          return result;
      }
      PyErr_Clear();
      return NULL;
  }
  static void __Pyx_AddTraceback(const char *funcname, int c_line,
                                 int py_line, const char *filename) {
      PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
      PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
      PyObject *exc_type, *exc_value, *exc_traceback;
      int success = 0;
      if (c_line) {
          c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
      }
      PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
      code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
      if (!code_object) {
          code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
          if (unlikely(!code_object)) goto bad;
          py_py_line = PyLong_FromLong(py_line);
          if (unlikely(!py_py_line)) goto bad;
          if (c_line) {
              py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
          } else {
              py_funcname = PyUnicode_FromString(funcname);
          }
          if (unlikely(!py_funcname)) goto bad;
          dict = PyDict_New();
          if (unlikely(!dict)) goto bad;
          {
              PyObject *old_code_object = code_object;
              code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
              Py_DECREF(old_code_object);
          }
          if (unlikely(!code_object)) goto bad;
          __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
      } else {
          dict = PyDict_New();
      }
      getframe = PySys_GetObject("_getframe");
      if (unlikely(!getframe)) goto bad;
      if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
      frame = PyEval_EvalCode(code_object, dict, dict);
      if (unlikely(!frame) || frame == Py_None) goto bad;
      success = 1;
    bad:
      PyErr_Restore(exc_type, exc_value, exc_traceback);
      Py_XDECREF(code_object);
      Py_XDECREF(py_py_line);
      Py_XDECREF(py_funcname);
      Py_XDECREF(dict);
      Py_XDECREF(replace);
      if (success) {
          PyTraceBack_Here(
              (struct _frame*)frame);
      }
      Py_XDECREF(frame);
  }
  #else
  static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
              const char *funcname, int c_line,
              int py_line, const char *filename) {
      PyCodeObject *py_code = NULL;
      PyObject *py_funcname = NULL;
      if (c_line) {
          py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
          if (!py_funcname) goto bad;
          funcname = PyUnicode_AsUTF8(py_funcname);
          if (!funcname) goto bad;
      }
      py_code = PyCode_NewEmpty(filename, funcname, py_line);
      Py_XDECREF(py_funcname);
      return py_code;
  bad:
      Py_XDECREF(py_funcname);
      return NULL;
  }
  static void __Pyx_AddTraceback(const char *funcname, int c_line,
                                 int py_line, const char *filename) {
      PyCodeObject *py_code = 0;
      PyFrameObject *py_frame = 0;
      PyThreadState *tstate = __Pyx_PyThreadState_Current;
      PyObject *ptype, *pvalue, *ptraceback;
      if (c_line) {
          c_line = __Pyx_CLineForTraceback(tstate, c_line);
      }
      py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
      if (!py_code) {
          __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
          py_code = __Pyx_CreateCodeObjectForTraceback(
              funcname, c_line, py_line, filename);
          if (!py_code) {
              /* If the code object creation fails, then we should clear the
                 fetched exception references and propagate the new exception */
              Py_XDECREF(ptype);
              Py_XDECREF(pvalue);
              Py_XDECREF(ptraceback);
              goto bad;
          }
          __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
          __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
      }
      py_frame = PyFrame_New(
          tstate,            /*PyThreadState *tstate,*/
          py_code,           /*PyCodeObject *code,*/
          __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
          0                  /*PyObject *locals*/
      );
      if (!py_frame) goto bad;
      __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
      PyTraceBack_Here(py_frame);
  bad:
      Py_XDECREF(py_code);
      Py_XDECREF(py_frame);
  }
  #endif
  
/* CheckUnpickleChecksum */
  static void __Pyx_RaiseUnpickleChecksumError(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
      PyObject *pickle_module = PyImport_ImportModule("pickle");
      if (unlikely(!pickle_module)) return;
      PyObject *pickle_error = PyObject_GetAttrString(pickle_module, "PickleError");
      Py_DECREF(pickle_module);
      if (unlikely(!pickle_error)) return;
      if (checksum2 == checksum1) {
          PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x) = (%s))",
              checksum, checksum1, members);
      } else if (checksum3 == checksum2) {
          PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x) = (%s))",
              checksum, checksum1, checksum2, members);
      } else {
          PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x, 0x%x) = (%s))",
              checksum, checksum1, checksum2, checksum3, members);
      }
      Py_DECREF(pickle_error);
  }
  static int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
      int found = 0;
      found |= checksum1 == checksum;
      found |= checksum2 == checksum;
      found |= checksum3 == checksum;
      if (likely(found))
          return 0;
      __Pyx_RaiseUnpickleChecksumError(checksum, checksum1, checksum2, checksum3, members);
      return -1;
  }
  
/* CIntFromPyVerify */
  #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
      __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
  #define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
      __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
  #define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
      {\
          func_type value = func_value;\
          if (sizeof(target_type) < sizeof(func_type)) {\
              if (unlikely(value != (func_type) (target_type) value)) {\
                  func_type zero = 0;\
                  if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                      return (target_type) -1;\
                  if (is_unsigned && unlikely(value < zero))\
                      goto raise_neg_overflow;\
                  else\
                      goto raise_overflow;\
              }\
          }\
          return (target_type) value;\
      }
  
/* CIntFromPy */
  static CYTHON_INLINE unsigned int __Pyx_PyLong_As_unsigned_int(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          unsigned int val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (unsigned int) -1;
          val = __Pyx_PyLong_As_unsigned_int(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(unsigned int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(unsigned int) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT)) {
                              return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(unsigned int) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT)) {
                              return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(unsigned int) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT)) {
                              return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (unsigned int) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(unsigned int) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(unsigned int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT)) {
                              return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(unsigned int) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT)) {
                              return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT)) {
                              return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(unsigned int) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT)) {
                              return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT)) {
                              return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(unsigned int) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT)) {
                              return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(unsigned int) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
          } else if ((sizeof(unsigned int) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          unsigned int val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (unsigned int) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (unsigned int) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (unsigned int) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (unsigned int) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(unsigned int) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((unsigned int) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(unsigned int) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((unsigned int) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((unsigned int) 1) << (sizeof(unsigned int) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (unsigned int) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to unsigned int");
      return (unsigned int) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to unsigned int");
      return (unsigned int) -1;
  }
  
/* CIntFromPy */
  static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const long neg_one = (long) -1, const_zero = (long) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          long val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (long) -1;
          val = __Pyx_PyLong_As_long(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                              return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                              return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                              return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (long) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(long) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                              return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                              return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                              return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                              return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                              return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                              return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(long) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
          } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          long val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (long) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (long) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (long) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (long) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((long) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((long) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (long) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to long");
      return (long) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to long");
      return (long) -1;
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const long neg_one = (long) -1, const_zero = (long) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(long) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(long) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(long) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(long),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* CIntToPy */
  static CYTHON_INLINE PyObject* __Pyx_PyLong_From_unsigned_int(unsigned int value) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (is_unsigned) {
          if (sizeof(unsigned int) < sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
              return PyLong_FromUnsignedLong((unsigned long) value);
  #if !CYTHON_COMPILING_IN_PYPY
          } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
              return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
  #endif
          }
      } else {
          if (sizeof(unsigned int) <= sizeof(long)) {
              return PyLong_FromLong((long) value);
          } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
              return PyLong_FromLongLong((PY_LONG_LONG) value);
          }
      }
      {
          unsigned char *bytes = (unsigned char *)&value;
  #if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
          if (is_unsigned) {
              return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
          } else {
              return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
          }
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
          int one = 1; int little = (int)*(unsigned char *)&one;
          return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                       little, !is_unsigned);
  #else
          int one = 1; int little = (int)*(unsigned char *)&one;
          PyObject *from_bytes, *result = NULL, *kwds = NULL;
          PyObject *py_bytes = NULL, *order_str = NULL;
          from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
          if (!from_bytes) return NULL;
          py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(unsigned int));
          if (!py_bytes) goto limited_bad;
          order_str = PyUnicode_FromString(little ? "little" : "big");
          if (!order_str) goto limited_bad;
          {
              PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
              if (!is_unsigned) {
                  kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                  if (!kwds) goto limited_bad;
                  if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
              }
              result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
          }
          limited_bad:
          Py_XDECREF(kwds);
          Py_XDECREF(order_str);
          Py_XDECREF(py_bytes);
          Py_XDECREF(from_bytes);
          return result;
  #endif
      }
  }
  
/* UpdateUnpickledDict */
  static int __Pyx__UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
      PyObject *state_dict = __Pyx_PySequence_ITEM(state, index);
      if (unlikely(!state_dict)) {
          return -1;
      }
      int non_empty = PyObject_IsTrue(state_dict);
      if (non_empty == 0) {
          Py_DECREF(state_dict);
          return 0;
      } else if (unlikely(non_empty == -1)) {
          return -1;
      }
      PyObject *dict;
      #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
      dict = PyObject_GetAttrString(obj, "__dict__");
      #else
      dict = PyObject_GenericGetDict(obj, NULL);
      #endif
      if (unlikely(!dict)) {
          Py_DECREF(state_dict);
          return -1;
      }
      int result;
      if (likely(PyDict_CheckExact(dict))) {
          result = PyDict_Update(dict, state_dict);
      } else {
          PyObject *obj_result = __Pyx_PyObject_CallMethod1(dict, __pyx_mstate_global->__pyx_n_u_update, state_dict);
          if (likely(obj_result)) {
              Py_DECREF(obj_result);
              result = 0;
          } else {
              result = -1;
          }
      }
      Py_DECREF(state_dict);
      Py_DECREF(dict);
      return result;
  }
  static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
      Py_ssize_t state_size = __Pyx_PyTuple_GET_SIZE(state);
      #if !CYTHON_ASSUME_SAFE_SIZE
      if (unlikely(state_size == -1)) return -1;
      #endif
      if (state_size <= index) {
          return 0;
      }
      return __Pyx__UpdateUnpickledDict(obj, state, index);
  }
  
/* FormatTypeName */
  #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
  static __Pyx_TypeName
  __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
  {
      PyObject *module = NULL, *name = NULL, *result = NULL;
      #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
      name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                                 __pyx_mstate_global->__pyx_n_u_qualname);
      #else
      name = PyType_GetQualName(tp);
      #endif
      if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
      module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                                 __pyx_mstate_global->__pyx_n_u_module);
      if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
      if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
          result = name;
          name = NULL;
          goto done;
      }
      result = PyUnicode_FromFormat("%U.%U", module, name);
      if (unlikely(result == NULL)) goto bad;
    done:
      Py_XDECREF(name);
      Py_XDECREF(module);
      return result;
    bad:
      PyErr_Clear();
      if (name) {
          result = name;
          name = NULL;
      } else {
          result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__10);
      }
      goto done;
  }
  #endif
  
/* CIntFromPy */
  static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic push
  #pragma GCC diagnostic ignored "-Wconversion"
  #endif
      const int neg_one = (int) -1, const_zero = (int) 0;
  #ifdef __Pyx_HAS_GCC_DIAGNOSTIC
  #pragma GCC diagnostic pop
  #endif
      const int is_unsigned = neg_one > const_zero;
      if (unlikely(!PyLong_Check(x))) {
          int val;
          PyObject *tmp = __Pyx_PyNumber_Long(x);
          if (!tmp) return (int) -1;
          val = __Pyx_PyLong_As_int(tmp);
          Py_DECREF(tmp);
          return val;
      }
      if (is_unsigned) {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (unlikely(__Pyx_PyLong_IsNeg(x))) {
              goto raise_neg_overflow;
          } else if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_DigitCount(x)) {
                  case 2:
                      if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                              return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                              return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                              return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                          }
                      }
                      break;
              }
          }
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
          if (unlikely(Py_SIZE(x) < 0)) {
              goto raise_neg_overflow;
          }
  #else
          {
              int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
              if (unlikely(result < 0))
                  return (int) -1;
              if (unlikely(result == 1))
                  goto raise_neg_overflow;
          }
  #endif
          if ((sizeof(int) <= sizeof(unsigned long))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
          } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
          }
      } else {
  #if CYTHON_USE_PYLONG_INTERNALS
          if (__Pyx_PyLong_IsCompact(x)) {
              __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
          } else {
              const digit* digits = __Pyx_PyLong_Digits(x);
              assert(__Pyx_PyLong_DigitCount(x) > 1);
              switch (__Pyx_PyLong_SignedDigitCount(x)) {
                  case -2:
                      if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                              return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case 2:
                      if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                              return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case -3:
                      if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                              return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case 3:
                      if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                              return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case -4:
                      if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                              return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
                  case 4:
                      if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                          if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                              __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                          } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                              return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                          }
                      }
                      break;
              }
          }
  #endif
          if ((sizeof(int) <= sizeof(long))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
          } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
              __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
          }
      }
      {
          int val;
          int ret = -1;
  #if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
          Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
              x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
          if (unlikely(bytes_copied == -1)) {
          } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
              goto raise_overflow;
          } else {
              ret = 0;
          }
  #elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
          int one = 1; int is_little = (int)*(unsigned char *)&one;
          unsigned char *bytes = (unsigned char *)&val;
          ret = _PyLong_AsByteArray((PyLongObject *)x,
                                      bytes, sizeof(val),
                                      is_little, !is_unsigned);
  #else
          PyObject *v;
          PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
          int bits, remaining_bits, is_negative = 0;
          int chunk_size = (sizeof(long) < 8) ? 30 : 62;
          if (likely(PyLong_CheckExact(x))) {
              v = __Pyx_NewRef(x);
          } else {
              v = PyNumber_Long(x);
              if (unlikely(!v)) return (int) -1;
              assert(PyLong_CheckExact(v));
          }
          {
              int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
              if (unlikely(result < 0)) {
                  Py_DECREF(v);
                  return (int) -1;
              }
              is_negative = result == 1;
          }
          if (is_unsigned && unlikely(is_negative)) {
              Py_DECREF(v);
              goto raise_neg_overflow;
          } else if (is_negative) {
              stepval = PyNumber_Invert(v);
              Py_DECREF(v);
              if (unlikely(!stepval))
                  return (int) -1;
          } else {
              stepval = v;
          }
          v = NULL;
          val = (int) 0;
          mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
          shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
          for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
              PyObject *tmp, *digit;
              long idigit;
              digit = PyNumber_And(stepval, mask);
              if (unlikely(!digit)) goto done;
              idigit = PyLong_AsLong(digit);
              Py_DECREF(digit);
              if (unlikely(idigit < 0)) goto done;
              val |= ((int) idigit) << bits;
              tmp = PyNumber_Rshift(stepval, shift);
              if (unlikely(!tmp)) goto done;
              Py_DECREF(stepval); stepval = tmp;
          }
          Py_DECREF(shift); shift = NULL;
          Py_DECREF(mask); mask = NULL;
          {
              long idigit = PyLong_AsLong(stepval);
              if (unlikely(idigit < 0)) goto done;
              remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
              if (unlikely(idigit >= (1L << remaining_bits)))
                  goto raise_overflow;
              val |= ((int) idigit) << bits;
          }
          if (!is_unsigned) {
              if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                  goto raise_overflow;
              if (is_negative)
                  val = ~val;
          }
          ret = 0;
      done:
          Py_XDECREF(shift);
          Py_XDECREF(mask);
          Py_XDECREF(stepval);
  #endif
          if (unlikely(ret))
              return (int) -1;
          return val;
      }
  raise_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "value too large to convert to int");
      return (int) -1;
  raise_neg_overflow:
      PyErr_SetString(PyExc_OverflowError,
          "can't convert negative value to int");
      return (int) -1;
  }
  
/* GetRuntimeVersion */
  #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
  void __Pyx_init_runtime_version(void) {
      if (__Pyx_cached_runtime_version == 0) {
          const char* rt_version = Py_GetVersion();
          unsigned long version = 0;
          unsigned long factor = 0x01000000UL;
          unsigned int digit = 0;
          int i = 0;
          while (factor) {
              while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                  digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                  ++i;
              }
              version += factor * digit;
              if (rt_version[i] != '.')
                  break;
              digit = 0;
              factor >>= 8;
              ++i;
          }
          __Pyx_cached_runtime_version = version;
      }
  }
  #endif
  static unsigned long __Pyx_get_runtime_version(void) {
  #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
      return Py_Version & ~0xFFUL;
  #else
      return __Pyx_cached_runtime_version;
  #endif
  }
  
/* Generator */
  static PyMethodDef __pyx_Generator_methods[] = {
      {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
       PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
      {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
       PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
      {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
       PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
      {"__reduce_ex__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_O, 0},
      {"__reduce__", (PyCFunction) __Pyx_Coroutine_fail_reduce_ex, METH_NOARGS, 0},
      {0, 0, 0, 0}
  };
  static PyMemberDef __pyx_Generator_memberlist[] = {
      {"gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
       PyDoc_STR("object being iterated by 'yield from', or None")},
      {"gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
      {"__module__", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_modulename), 0, 0},
  #if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
      {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CoroutineObject, gi_weakreflist), READONLY, 0},
  #endif
      {0, 0, 0, 0, 0}
  };
  static PyGetSetDef __pyx_Generator_getsets[] = {
      {"__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
       PyDoc_STR("name of the generator"), 0},
      {"__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
       PyDoc_STR("qualified name of the generator"), 0},
      {"gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
       PyDoc_STR("Frame of the generator"), 0},
      {"gi_running", __Pyx_Coroutine_get_is_running_getter, NULL, NULL, NULL},
      {0, 0, 0, 0, 0}
  };
  static PyType_Slot __pyx_GeneratorType_slots[] = {
      {Py_tp_dealloc, (void *)__Pyx_Coroutine_dealloc},
      {Py_tp_traverse, (void *)__Pyx_Coroutine_traverse},
      {Py_tp_iter, (void *)PyObject_SelfIter},
      {Py_tp_iternext, (void *)__Pyx_Generator_Next},
      {Py_tp_methods, (void *)__pyx_Generator_methods},
      {Py_tp_members, (void *)__pyx_Generator_memberlist},
      {Py_tp_getset, (void *)__pyx_Generator_getsets},
      {Py_tp_getattro, (void *) PyObject_GenericGetAttr},
  #if CYTHON_USE_TP_FINALIZE
      {Py_tp_finalize, (void *)__Pyx_Coroutine_del},
  #endif
  #if __PYX_HAS_PY_AM_SEND == 1
      {Py_am_send, (void *)__Pyx_Coroutine_AmSend},
  #endif
      {0, 0},
  };
  static PyType_Spec __pyx_GeneratorType_spec = {
      __PYX_TYPE_MODULE_PREFIX "generator",
      sizeof(__pyx_CoroutineObject),
      0,
  #if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
      Py_TPFLAGS_MANAGED_WEAKREF |
  #endif
      Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
      Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | __Pyx_TPFLAGS_HAVE_AM_SEND,
      __pyx_GeneratorType_slots
  };
  #if __PYX_HAS_PY_AM_SEND == 2
  static __Pyx_PyAsyncMethodsStruct __pyx_Generator_as_async;
  #endif
  static int __pyx_Generator_init(PyObject *module) {
      __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
      mstate->__pyx_GeneratorType = __Pyx_FetchCommonTypeFromSpec(
          mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_GeneratorType_spec, NULL);
      if (unlikely(!mstate->__pyx_GeneratorType)) {
          return -1;
      }
  #if __PYX_HAS_PY_AM_SEND == 2
      __Pyx_SetBackportTypeAmSend(mstate->__pyx_GeneratorType, &__pyx_Generator_as_async, &__Pyx_Coroutine_AmSend);
  #endif
      return 0;
  }
  static PyObject *__Pyx_Generator_GetInlinedResult(PyObject *self) {
      __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
      PyObject *retval = NULL;
      if (unlikely(__Pyx_Coroutine_test_and_set_is_running(gen))) {
          return __Pyx_Coroutine_AlreadyRunningError(gen);
      }
      __Pyx_PySendResult result = __Pyx_Coroutine_SendEx(gen, Py_None, &retval, 0);
      __Pyx_Coroutine_unset_is_running(gen);
      (void) result;
      assert (result == PYGEN_RETURN || result == PYGEN_ERROR);
      assert ((result == PYGEN_RETURN && retval != NULL) || (result == PYGEN_ERROR && retval == NULL));
      return retval;
  }
  
/* CheckBinaryVersion */
  static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
      const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
      if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
          return 0;
      if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
          return 1;
      {
          char message[200];
          PyOS_snprintf(message, sizeof(message),
                        "compile time Python version %d.%d "
                        "of module '%.100s' "
                        "%s "
                        "runtime version %d.%d",
                         (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                         __Pyx_MODULE_NAME,
                         (allow_newer) ? "was newer than" : "does not match",
                         (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
         );
          return PyErr_WarnEx(NULL, message, 1);
      }
  }
  
/* NewCodeObj */
  #if CYTHON_COMPILING_IN_LIMITED_API
      static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
          PyObject *exception_table = NULL;
          PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
          #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
          PyObject *version_info;
          PyObject *py_minor_version = NULL;
          #endif
          long minor_version = 0;
          PyObject *type, *value, *traceback;
          PyErr_Fetch(&type, &value, &traceback);
          #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
          minor_version = 11;
          #else
          if (!(version_info = PySys_GetObject("version_info"))) goto end;
          if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
          minor_version = PyLong_AsLong(py_minor_version);
          Py_DECREF(py_minor_version);
          if (minor_version == -1 && PyErr_Occurred()) goto end;
          #endif
          if (!(types_module = PyImport_ImportModule("types"))) goto end;
          if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
          if (minor_version <= 7) {
              (void)p;
              result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                            c, n, v, fn, name, fline, lnos, fv, cell);
          } else if (minor_version <= 10) {
              result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                            c, n, v, fn, name, fline, lnos, fv, cell);
          } else {
              if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
              result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                            c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
          }
      end:
          Py_XDECREF(code_type);
          Py_XDECREF(exception_table);
          Py_XDECREF(types_module);
          if (type) {
              PyErr_Restore(type, value, traceback);
          }
          return result;
      }
  #elif PY_VERSION_HEX >= 0x030B0000
    static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                           PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                           PyObject *fv, PyObject *cell, PyObject* fn,
                                           PyObject *name, int fline, PyObject *lnos) {
      PyCodeObject *result;
      result =
        #if PY_VERSION_HEX >= 0x030C0000
          PyUnstable_Code_NewWithPosOnlyArgs
        #else
          PyCode_NewWithPosOnlyArgs
        #endif
          (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
      #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
      if (likely(result))
          result->_co_firsttraceable = 0;
      #endif
      return result;
    }
  #elif !CYTHON_COMPILING_IN_PYPY
    #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
            PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #else
    #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
            PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #endif
  static PyObject* __Pyx_PyCode_New(
          const __Pyx_PyCode_New_function_description descr,
          PyObject * const *varnames,
          PyObject *filename,
          PyObject *funcname,
          PyObject *line_table,
          PyObject *tuple_dedup_map
  ) {
      PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
      Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
      PyObject *varnames_tuple = PyTuple_New(var_count);
      if (unlikely(!varnames_tuple)) return NULL;
      for (Py_ssize_t i=0; i < var_count; i++) {
          Py_INCREF(varnames[i]);
          if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
      }
      #if CYTHON_COMPILING_IN_LIMITED_API
      varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
      if (!varnames_tuple_dedup) {
          if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
          varnames_tuple_dedup = varnames_tuple;
      }
      #else
      varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
      if (unlikely(!varnames_tuple_dedup)) goto done;
      #endif
      #if CYTHON_AVOID_BORROWED_REFS
      Py_INCREF(varnames_tuple_dedup);
      #endif
      if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
          Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
          #if !CYTHON_ASSUME_SAFE_SIZE
          if (unlikely(line_table_length == -1)) goto done;
          #endif
          Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
          code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
          if (unlikely(!code_bytes)) goto done;
          char* c_code_bytes = PyBytes_AsString(code_bytes);
          if (unlikely(!c_code_bytes)) goto done;
          memset(c_code_bytes, 0, (size_t) code_len);
      }
      code_obj = (PyObject*) __Pyx__PyCode_New(
          (int) descr.argcount,
          (int) descr.num_posonly_args,
          (int) descr.num_kwonly_args,
          (int) descr.nlocals,
          0,
          (int) descr.flags,
          code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
          __pyx_mstate_global->__pyx_empty_tuple,
          __pyx_mstate_global->__pyx_empty_tuple,
          varnames_tuple_dedup,
          __pyx_mstate_global->__pyx_empty_tuple,
          __pyx_mstate_global->__pyx_empty_tuple,
          filename,
          funcname,
          (int) descr.first_line,
          (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
      );
  done:
      Py_XDECREF(code_bytes);
      #if CYTHON_AVOID_BORROWED_REFS
      Py_XDECREF(varnames_tuple_dedup);
      #endif
      Py_DECREF(varnames_tuple);
      return code_obj;
  }
  
/* DecompressString */
  static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
      PyObject *module = NULL, *decompress, *compressed_bytes, *decompressed;
      const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
      PyObject *methodname = PyUnicode_FromString("decompress");
      if (unlikely(!methodname)) return NULL;
      #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
      if (algo == 3) {
          PyObject *fromlist = Py_BuildValue("[O]", methodname);
          if (unlikely(!fromlist)) goto bad;
          module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
          Py_DECREF(fromlist);
      } else
      #endif
          module = PyImport_ImportModule(module_name);
      if (unlikely(!module)) goto import_failed;
      decompress = PyObject_GetAttr(module, methodname);
      if (unlikely(!decompress)) goto import_failed;
      {
          #ifdef __cplusplus
              char *memview_bytes = const_cast<char*>(s);
          #else
              #if defined(__clang__)
                #pragma clang diagnostic push
                #pragma clang diagnostic ignored "-Wcast-qual"
              #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
                #pragma GCC diagnostic push
                #pragma GCC diagnostic ignored "-Wcast-qual"
              #endif
              char *memview_bytes = (char*) s;
              #if defined(__clang__)
                #pragma clang diagnostic pop
              #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
                #pragma GCC diagnostic pop
              #endif
          #endif
          #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
          int memview_flags = 0x100;
          #else
          int memview_flags = PyBUF_READ;
          #endif
          compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
      }
      if (unlikely(!compressed_bytes)) {
          Py_DECREF(decompress);
          goto bad;
      }
      decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
      Py_DECREF(compressed_bytes);
      Py_DECREF(decompress);
      Py_DECREF(module);
      Py_DECREF(methodname);
      return decompressed;
  import_failed:
      PyErr_Format(PyExc_ImportError,
          "Failed to import '%.20s.decompress' - cannot initialise module strings. "
          "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
          module_name, algo);
  bad:
      Py_XDECREF(module);
      Py_DECREF(methodname);
      return NULL;
  }
  
#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


  /* MultiPhaseInitModuleState */
  #if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
  #ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  #if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
    #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
  #else
    #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
  #endif
  #endif
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
  #error "Module state with PEP489 requires atomics. Currently that's one of\
   C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
  #endif
  #if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  #define __Pyx_ModuleStateLookup_Lock()
  #define __Pyx_ModuleStateLookup_Unlock()
  #elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
  static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
  #define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
  #elif defined(__cplusplus) && __cplusplus >= 201103L
  #include <mutex>
  static std::mutex __Pyx_ModuleStateLookup_mutex;
  #define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
  #define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
  #elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
  #include <threads.h>
  static mtx_t __Pyx_ModuleStateLookup_mutex;
  static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
  static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
      mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
  }
  #define __Pyx_ModuleStateLookup_Lock()\
    call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
    mtx_lock(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
  #elif defined(HAVE_PTHREAD_H)
  #include <pthread.h>
  static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
  #define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
  #elif defined(_WIN32)
  #include <Windows.h>  // synchapi.h on its own doesn't work
  static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
  #define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
  #define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
  #else
  #error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
   Requires C standard >= C11, or C++ standard >= C++11,\
   or pthreads, or the Windows 32 API, or Python >= 3.13."
  #endif
  typedef struct {
      int64_t id;
      PyObject *module;
  } __Pyx_InterpreterIdAndModule;
  typedef struct {
      char interpreter_id_as_index;
      Py_ssize_t count;
      Py_ssize_t allocated;
      __Pyx_InterpreterIdAndModule table[1];
  } __Pyx_ModuleStateLookupData;
  #define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
  #endif
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
  #else
  static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
  #endif
  static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
          __Pyx_InterpreterIdAndModule* table,
          Py_ssize_t count,
          int64_t interpreterId) {
      __Pyx_InterpreterIdAndModule* begin = table;
      __Pyx_InterpreterIdAndModule* end = begin + count;
      if (begin->id == interpreterId) {
          return begin;
      }
      while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
          __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
          if (halfway->id == interpreterId) {
              return halfway;
          }
          if (halfway->id < interpreterId) {
              begin = halfway;
          } else {
              end = halfway;
          }
      }
      for (; begin < end; ++begin) {
          if (begin->id >= interpreterId) return begin;
      }
      return begin;
  }
  static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
      int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
      if (interpreter_id == -1) return NULL;
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
      {
          __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
          if (likely(data)) {
              __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
              if (likely(data == new_data)) {
                  goto read_finished;
              }
          }
          __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
          __Pyx_ModuleStateLookup_Lock();
          __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
          data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
          __Pyx_ModuleStateLookup_Unlock();
      }
    read_finished:;
  #else
      __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
  #endif
      __Pyx_InterpreterIdAndModule* found = NULL;
      if (unlikely(!data)) goto end;
      if (data->interpreter_id_as_index) {
          if (interpreter_id < data->count) {
              found = data->table+interpreter_id;
          }
      } else {
          found = __Pyx_State_FindModuleStateLookupTableLowerBound(
              data->table, data->count, interpreter_id);
      }
    end:
      {
          PyObject *result=NULL;
          if (found && found->id == interpreter_id) {
              result = found->module;
          }
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
          __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
  #endif
          return result;
      }
  }
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
  static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
      while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
  }
  #else
  #define __Pyx_ModuleStateLookup_wait_until_no_readers()
  #endif
  static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
      Py_ssize_t to_allocate = (*old_data)->allocated;
      while (to_allocate <= interpreter_id) {
          if (to_allocate == 0) to_allocate = 1;
          else to_allocate *= 2;
      }
      __Pyx_ModuleStateLookupData *new_data = *old_data;
      if (to_allocate != (*old_data)->allocated) {
           new_data = (__Pyx_ModuleStateLookupData *)realloc(
              *old_data,
              sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
          if (!new_data) {
              PyErr_NoMemory();
              return -1;
          }
          for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
              new_data->table[i].id = i;
              new_data->table[i].module = NULL;
          }
          new_data->allocated = to_allocate;
      }
      new_data->table[interpreter_id].module = module;
      if (new_data->count < interpreter_id+1) {
          new_data->count = interpreter_id+1;
      }
      *old_data = new_data;
      return 0;
  }
  static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
      __Pyx_InterpreterIdAndModule *read = data->table;
      __Pyx_InterpreterIdAndModule *write = data->table;
      __Pyx_InterpreterIdAndModule *end = read + data->count;
      for (; read<end; ++read) {
          if (read->module) {
              write->id = read->id;
              write->module = read->module;
              ++write;
          }
      }
      data->count = write - data->table;
      for (; write<end; ++write) {
          write->id = 0;
          write->module = NULL;
      }
      data->interpreter_id_as_index = 0;
  }
  static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
      int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
      if (interpreter_id == -1) return -1;
      int result = 0;
      __Pyx_ModuleStateLookup_Lock();
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
              __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
  #else
      __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
  #endif
      __Pyx_ModuleStateLookupData *new_data = old_data;
      if (!new_data) {
          new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
          if (!new_data) {
              result = -1;
              PyErr_NoMemory();
              goto end;
          }
          new_data->allocated = 1;
          new_data->interpreter_id_as_index = 1;
      }
      __Pyx_ModuleStateLookup_wait_until_no_readers();
      if (new_data->interpreter_id_as_index) {
          if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
              result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
              goto end;
          }
          __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
      }
      {
          Py_ssize_t insert_at = 0;
          {
              __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                  new_data->table, new_data->count, interpreter_id);
              assert(lower_bound);
              insert_at = lower_bound - new_data->table;
              if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                  lower_bound->module = module;
                  goto end;  // already in table, nothing more to do
              }
          }
          if (new_data->count+1 >= new_data->allocated) {
              Py_ssize_t to_allocate = (new_data->count+1)*2;
              new_data =
                  (__Pyx_ModuleStateLookupData*)realloc(
                      new_data,
                      sizeof(__Pyx_ModuleStateLookupData) +
                      (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
              if (!new_data) {
                  result = -1;
                  new_data = old_data;
                  PyErr_NoMemory();
                  goto end;
              }
              new_data->allocated = to_allocate;
          }
          ++new_data->count;
          int64_t last_id = interpreter_id;
          PyObject *last_module = module;
          for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
              int64_t current_id = new_data->table[i].id;
              new_data->table[i].id = last_id;
              last_id = current_id;
              PyObject *current_module = new_data->table[i].module;
              new_data->table[i].module = last_module;
              last_module = current_module;
          }
      }
    end:
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
  #else
      __Pyx_ModuleStateLookup_data = new_data;
  #endif
      __Pyx_ModuleStateLookup_Unlock();
      return result;
  }
  static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
      int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
      if (interpreter_id == -1) return -1;
      __Pyx_ModuleStateLookup_Lock();
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
              __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
  #else
      __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
  #endif
      if (data->interpreter_id_as_index) {
          if (interpreter_id < data->count) {
              data->table[interpreter_id].module = NULL;
          }
          goto done;
      }
      {
          __Pyx_ModuleStateLookup_wait_until_no_readers();
          __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
              data->table, data->count, interpreter_id);
          if (!lower_bound) goto done;
          if (lower_bound->id != interpreter_id) goto done;
          __Pyx_InterpreterIdAndModule *end = data->table+data->count;
          for (;lower_bound<end-1; ++lower_bound) {
              lower_bound->id = (lower_bound+1)->id;
              lower_bound->module = (lower_bound+1)->module;
          }
      }
      --data->count;
      if (data->count == 0) {
          free(data);
          data = NULL;
      }
    done:
  #if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
      __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
  #else
      __Pyx_ModuleStateLookup_data = data;
  #endif
      __Pyx_ModuleStateLookup_Unlock();
      return 0;
  }
  #endif
  
/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
